# DBObject.pm - The Object Class that provides a DBObject Object
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: DBObject.pm,v 1.1 2001/09/06 10:21:19 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package DBObject;
use strict;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();
	
$VERSION = '0.02';

# new 
# instantiates an instance of a DBObject
# takes: dbHost, dbType, dbName, dbPort, dbUser, dbPasswd
sub new
{
  my $that;
  $that = shift;
  my $class;
  $class = ref($that) || $that;
  my $self;
  $self = bless {}, $class;
  my %args;
  %args = ( dbHost => "", dbType => "Pg", dbName => "", dbPort => "5432", dbUser => "", dbPasswd => "", @_ );
  my $errStr;
  $errStr = "DBObject->new() - Error!<br>\n";
  
  $self->{dbHost} = $args{dbHost};
  $self->{dbType} = $args{dbType};
  $self->{dbName} = $args{dbName};
  $self->{dbPort} = $args{dbPort};
  $self->{dbUser} = $args{dbUser};
  $self->{dbPasswd} = $args{dbPasswd};

  # do validation
  $self->{error} = !$self->isValid;  
  if ($self->{error})
  {
    $self->{errorString} = $errStr . $self->{errorString};
    $self->{errorString} .= "<p>Port in DBObject:<br>dbPort:$self->{dbPort}</p>\n" ;
  }

  return $self;
}

# isValid - Returns 0 or 1 to indicate if the object is valid.
sub isValid
{
  my $self;
  $self = shift;
  my $error;
  $error = 0;
  my $errorString;
  $errorString = "";
  my $errStr;
  $errStr = "DBObject->isValid() - Error!<br>\n";

  # do validation code here.
  if (length $self->{dbHost} == 0)
  {
    $error = 1;
    $errorString .= "dbHost = '$self->{dbHost}' is invalid!<br>\n";
  }
  if ($self->{dbType} !~ /^(Pg|MySQL)$/)
  {
    $error = 1;
    $errorString .= "dbType = '$self->{dbType}' is invalid!<br>\n";
  }
  if ($self->{dbPort} !~ /^\d+$/)
  {
    $error = 1;
    $errorString .= "dbPort = '$self->{dbPort}' is invalid!<br>\n";
  }
  if (length $self->{dbName} == 0)
  {
    $error = 1;
    $errorString .= "dbName = '$self->{dbName}' is invalid!<br>\n";
  }
  if (length $self->{dbUser} == 0)
  {
    $error = 1;
    $errorString .= "dbUser = '$self->{dbUser}' is invalid!<br>\n";
  }

  $self->{error} = $error;
  $self->{errorString} = $errStr if $error;
  $self->{errorString} .= $errorString;

  return !$error;
}

sub DESTROY
{
  my $self = shift;
}

sub AUTOLOAD
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  if (@_)
  {
    return $self->{$name} = shift;
  }
  else
  {
    return $self->{$name};
  }
}

# setError
# requires: errorString
# returns: nothing
sub setError
{
  my $self = shift;
  my %args = ( errorString => "", @_ );
  my $errorString = $args{errorString};
  
  $self->{errorString} = $errorString;
  $self->{error} = 1;
}

# didErrorOccur
# returns the value of error.
sub didErrorOccur
{
  my $self = shift;

  return $self->{error};
}

# errorMessage
# returns the value of errorString
sub errorMessage
{
  my $self = shift;
  
  return $self->{errorString};
}

1;
__END__

=head1 NAME

DBObject - Object used to build a DBObject Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::DBObject;
  my $obj = Portal::Accounting::DBObject->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

DBObject is a DBObject class.

=head1 Exported FUNCTIONS

  scalar new(dbHost, dbPort, dbName, dbType, dbUser, dbPasswd)
    Creates a new instance of the Portal::Accounting::DBObject
    object.

  bool isValid(void)
    Returns 0 or 1 to indicate if the object is valid.  The error will 
    be available via errorMessage().

  scalar didErrorOccur(void)
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.


  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

PC & Web Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3)

=cut
