# PortalWrapper.pm - The Object Class that provides a PortalWrapper Object
#$Id: PortalWrapper.pm,v 1.12 2004/06/24 22:26:33 pattieja Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package PortalWrapper;
use strict;
use Portal::Session;
use Portal::SessionHandler;
use DBIWrapper;
use DBObject;
use HTMLObject::ReadCookie;
use HTMLObject::Base;
use Portal::Data::Config;
use Portal::Accounting::Data::Config;
use Portal::Objects::CompanyObject;
use Portal::Objects::User;
use Portal::Methods;
use Portal::Language;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.01';

# new
# instantiates an instance of a PortalWrapper
sub new
{
  my $that = shift;
  my $class = ref($that) || $that;
  my $self = bless {}, $class;
  my %args = (  @_ );
  my $errStr = "PortalWrapper->new() - Error!<br>\n";

  # instantiate the langObj (en for now)
  $self->{langObj} = Portal::Language->new(lang => "en");
  if ($self->{langObj}->didErrorOccur)
  {
    $self->setError(errorString => $errStr . $self->{langObj}->errorMessage);
    return $self;
  }

  # get the Portal Config object
  eval { $self->{configObj} = Portal::Data::Config->new(langObj => $self->{langObj}); };
  if ($@)
  {
    $self->setError(errorString => $errStr . "Error instantiating Portal::Data::Config Object!  $@");
    return $self;
  }

  $self->{methods} = Portal::Methods->new(langObj => $self->{langObj});
  if ($self->{methods}->didErrorOccur)
  {
    $self->setError(errorString => $errStr . $self->{methods}->errorMessage);
    return $self;
  }

  $self->{portalDB} = $self->{methods}->portalDBSetup(type => "portal", configObj => $self->{configObj});
  if ($self->{methods}->didErrorOccur)
  {
    $self->setError(errorString => $errStr . $self->{methods}->errorMessage);
    return $self;
  }
  if (! defined $self->{portalDB})
  {
    $self->setError(errorString => $errStr . "portalDB is not defined!<br>\n");
    return $self;
  }

  # instantiate our config file for our children to use
  eval "\$self->{appConfigObj} = Portal::Accounting::Data::Config->new(langObj => \$self->{langObj});";
  if ($@)
  {
    $self->setError($@);
    return;
  }
  if ($self->{appConfigObj}->didErrorOccur)
  {
    $self->setError($self->{appConfigObj}->errorMessage);
    return;
  }

  # get the pcx_session_id cookie
  $self->{cookieObj} = HTMLObject::ReadCookie->new;
  my %cookies = $self->{cookieObj}->getCookies;
  if (not exists $cookies{pcx_session_id})
  {
    $self->setError(errorString => $errStr . "Portal Session cookie does not exist!");
    return $self;
  }

  # instantiate the PortalSession Object
  $self->{portalSession} = $self->{methods}->portalSession(portalDB => $self->{portalDB}, cookieObj => $self->{cookieObj}, appName => "pcx_session_id", configObj => $self->{configObj});
  if ($self->{methods}->didErrorOccur)
  {
    $self->setError(errorString => $errStr . $self->{methods}->errorMessage);
    return $self;
  }
  if ($self->{portalSession}->didErrorOccur)
  {
    $self->setError(errorString => $errStr . $self->{portalSession}->errorMessage);
    return $self;
  }

  my $companyObj = $self->{portalSession}->{store}->{companyObj};
  my $userObj = $self->{portalSession}->{store}->{userObj};

  # make sure the session is still valid.
  eval {
    if (!$companyObj->isValid)
    {
      $self->{portalSession}->delete;
      $self->setError(errorString => $errStr . "Session = '" . $self->{portalSession}->session_id . "' is invalid!  " . $companyObj->errorMessage);
      return $self;
    }
    if (!$userObj->isValid)
    {
      $self->{portalSession}->delete;
      $self->setError(errorString => $errStr . "Session = '" . $self->{portalSession}->session_id . "' is invalid!  " . $userObj->errorMessage);
      return $self;
    }
  };
  if ($@)
  {
    $self->{portalSession}->delete;
    $self->setError(errorString => $errStr . "Eval Failed! Session = '" . $self->{portalSession}->session_id . "' is invalid! $@");
    return $self;
  }

  # instantiate the Accounting Session Object
  $self->{appSessions} = $self->{methods}->portalSessionHandler(portalDB => $self->{portalDB}, configObj => $self->{configObj}, sessionObj => $self->{portalSession});
  if ($self->{methods}->didErrorOccur)
  {
    $self->setError(errorString => $errStr . $self->{methods}->errorMessage);
    return $self;
  }
  if ($self->{appSessions}->didErrorOccur)
  {
    $self->setError(errorString => $errStr . $self->{appSessions}->errorMessage);
    return $self;
  }

  # do validation
  $self->{error} = !$self->isValid;
  if ($self->{error})
  {
    $self->{errorString} = $errStr . $self->{errorString};
  }

  return $self;
}

# isValid - Returns 0 or 1 to indicate if the object is valid.
sub isValid
{
  my $self = shift;
  my $error = 0;
  my $errorString = "";
  my $errStr = "PortalWrapper->isValid() - Error!<br>\n";

  # do validation code here.

  $self->{error} = $error;
  $self->{errorString} = $errStr if $error;
  $self->{errorString} .= $errorString;

  return !$error;
}

sub DESTROY
{
  my $self = shift;

  # make sure that the Session info is properly flushed.
  my $doc = HTMLObject::Base->new;
  $self->{appSessions}->writeCookie(doc => $doc);
}

sub AUTOLOAD
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  if (@_)
  {
    return $self->{$name} = shift;
  }
  else
  {
    return $self->{$name};
  }
}

# setError
# requires: errorString
# returns: nothing
sub setError
{
  my $self = shift;
  my %args = ( errorString => "", @_ );
  my $errorString = $args{errorString};

  $self->{errorString} = $errorString;
  $self->{error} = 1;
}

# didErrorOccur
# returns the value of error.
sub didErrorOccur
{
  my $self = shift;

  return $self->{error};
}

# errorMessage
# returns the value of errorString
sub errorMessage
{
  my $self = shift;

  return $self->{errorString};
}

# getDBInfo
# takes: nothing
# returns: DBObject or undef if error - defining the Database connection information
sub getDBInfo
{
  my $self = shift;
  my $errStr = "PortalWrapper->getDBInfo()  - Error!<br>\n";

  my $dbObj = DBObject->new(dbHost => $self->{appSessions}->{apps}->{Accounting}->{store}->{dbHost},
	dbName => '"' . $self->{appSessions}->{apps}->{Accounting}->{store}->{dbName} . '"',
        dbPort => $self->{appSessions}->{apps}->{Accounting}->{store}->{dbPort},
	dbType => $self->{appSessions}->{apps}->{Accounting}->{store}->{dbType},
        dbUser => $self->{appSessions}->{apps}->{Accounting}->{store}->{dbUser},
	dbPasswd => $self->{appSessions}->{apps}->{Accounting}->{store}->{dbPasswd});
  if ($dbObj->didErrorOccur)
  {
    $self->setError(errorString => $errStr . $dbObj->errorMessage . "<p>Data from session in getDBInfo: <br>\n"
    	. "appSessions: $self->{appSessions}<br>\n"
    	. "keys:" . (keys %{$self->{appSessions}} ). "&nbsp;");

    	foreach my $tempkey (keys %{$self->{appSessions}} )
    	  {
            $self->{errorString} .= "'". $tempkey . "'&nbsp;";
    	  }
        $self->{errorString} .= "<br>\n"
    	. "{appSessions}->{apps}:$self->{appSessions}->{apps}<br>\n"
    	. "keys:" . (keys %{$self->{appSessions}->{apps}} ) ;
        foreach my $tempkey (keys %{$self->{appSessions}->{apps}} )
        {
          $self->{errorString} .= "'". $tempkey . "'&nbsp;";
        }
        $self->{errorString} .= "<br>\n"
    	. "{appSessions}->{apps}->{Accounting}:$self->{appSessions}->{apps}->{Accounting}<br>\n"
    	. "keys" . (keys %{$self->{appSessions}->{apps}->{Accounting}});
        foreach my $tempkey (keys %{$self->{appSessions}->{apps}->{Accounting}})
        {
          $self->{errorString} .= "'". $tempkey . "'&nbsp;";
        }
        $self->{errorString} .= "<br>\n"
    	. "{appSessions}->{apps}->{Accounting}->{store}:$self->{appSessions}->{apps}->{Accounting}->{store}<br>\n"
    	. "keys" . (keys %{$self->{appSessions}->{apps}->{Accounting}->{store}});
        foreach my $tempkey (keys %{$self->{appSessions}->{apps}->{Accounting}->{store}})
        {
          $self->{errorString} .= "'". $tempkey . "'&nbsp;";
        }
        $self->{errorString} .= "<br>\n"
    	. "{appSessions}->{apps}->{Accounting}->{store}->{dbPort}:$self->{appSessions}->{apps}->{Accounting}->{store}->{dbPort}<br>\n"
    	. "</p>\n";
    return undef;
  }
  return $dbObj;
}

1;
__END__

=head1 NAME

PortalWrapper - Object used to build a PortalWrapper Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::PortalWrapper;
  my $obj = Portal::Accounting::PortalWrapper->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

PortalWrapper is a PortalWrapper class.

=head1 Exported FUNCTIONS

  scalar new(void)
    Creates a new instance of the Portal::Accounting::PortalWrapper
    object.

  bool isValid(void)
    Returns 0 or 1 to indicate if the object is valid.  The error will
    be available via errorMessage().

  scalar didErrorOccur(void)
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.

  DBObject getDBInfo(void)
    Returns the DBObject which defines the database connect info
    or undef if an error occurred.

  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

PC & Web Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), DBIWrapper(3), Portal::Session(3), Portal::SessionHandler(3)

=cut
