#!/usr/bin/perl
#$Id: assign_accounts2.cgi,v 1.2 2001/10/25 20:46:28 pcxuser Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

# assign_accounts2.cgi  -  Created by James Pattie, 05/22/1999
# This script takes either the ledger or account the user selected
# and lets them select the opposite type to work with.

# for instance:  If the user select an account, then they would be
# presented with a list of all ledgers in the system and the status
# of the assignment of the selected account to each ledger.
# The same would be the case for ledgers to accounts.

use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features
use reports;

use vars '$date';
my $ledger_code;
my $report_obj;
my $really_in_ledger;

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

$really_in_ledger = $input{'really_in_ledger'};

# validate we have correct data
my ($mode, $acct_num, $acct_name, $ledger_db, $ledger_name, $result2, $result3);
# result2, result3 will be used to have multiple select query results stored in.
# conn2 will be used to connect to each of the ledger databases as needed.
# modes are 1=accounts, 2=ledgers
if (not (exists $input{'select_acct'} || exists $input{'select_ledger'}))
{
  PrintErrorHeader();
  print "<h1>Error:  Access denied!</h1>\n";
  print "Error: 1008<br>$sys_admin";
  CleanUp();
}
if (exists $input{'select_acct'})
{
  $mode = 1;
  $acct_num = $input{'acct_num'};
  if (length $acct_num == 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  You must select an account to work with!</h1>\n";
    print "$back_button";
    CleanUp();
  }
}
elsif (exists $input{'select_ledger'})
{
  $mode = 2;
  $ledger_db = $input{'ledger'};
  if (length $ledger_db == 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  You must select a ledger to work with!</h1>\n";
    print "$back_button";
    CleanUp();
  }
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# validate we got a valid account or ledger based upon the mode we are in.
if ($mode == 1)
{
  $result = $conn->exec("select name, active from account_tb where number = '$acct_num'");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    PrintErrorHeader();
    print "<h1>Error:  Accessing database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to get list of accounts.<br>$sys_admin";
    CleanUp();
  }
  if ($result->ntuples == 0)
  {
    PrintErrorHeader();
    print "<h1>Internal Error:  The account <font color=\"blue\">$acct_num</font> does not exist!</h1>\n";
    print "Tried to validate account <font color=\"blue\">$acct_num</font>.<br>$sys_admin";
    CleanUp();
  }
  else
  {
    $acct_name = $result->getvalue(0, 0);
    my $active;
    $active = $result->getvalue(0, 1);
    if ($active eq "f")
    {
      PrintErrorHeader();
      print "<h1>Error:  The account <font color=\"blue\">$acct_name</font> is not active!</h1>\n";
      print "Tried to validate account <font color=\"blue\">$acct_name</font>.<br>$sys_admin";
      CleanUp();
    }
  }
}
elsif ($mode == 2)
{
  $result = $conn->exec("select name from entity_tb where database_name = '$ledger_db'");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    PrintErrorHeader();
    print "<h1>Error:  Accessing database failed!</h1><br>\n";
    print $conn->errorMessage . "<br>Tried to validate ledger <font color=\"blue\">$ledger_db</font>.<br>$sys_admin";
    CleanUp();
  }
  if ($result->ntuples == 0)
  {
    PrintErrorHeader();
    print "<h1>Internal Error:  The ledger <font color=\"blue\">$ledger_db</font> does not exist!</h1>\n";
    print "Tried to validate ledger <font color=\"blue\">$ledger_db</font>.<br>$sys_admin";
    CleanUp();
  }
  else
  {
    $ledger_name = $result->getvalue(0, 0);
  }
}

# now we get a list of all accounts or all ledgers in the system based upon the mode we are in.
if ($mode == 1)
{
  $result = $conn->exec("select name, database_name from entity_tb order by name");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    PrintErrorHeader();
    print "<h1>Error:  Accessing database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to get list of all ledgers available.<br>$sys_admin";
    CleanUp();
  }
  if ($result->ntuples == 0)
  {
    PrintErrorHeader();
    print "<h1>Internal Error:  No ledgers defined!</h1>\n";
    print "Tried to get list of all ledgers available.<br>$sys_admin";
    CleanUp();
  }
}
elsif ($mode == 2)
{
  $result = $conn->exec("select name, number, taxable from account_tb where active = true order by name");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    PrintErrorHeader();
    print "<h1>Error:  Accessing database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to get list of all accounts available.<br>$sys_admin";
    CleanUp();
  }
  if ($result->ntuples == 0)
  {
    PrintErrorHeader();
    print "<h1>Internal Error:  No accounts defined!</h1>\n";
    print "Tried to get list of all accounts available.<br>$sys_admin";
    CleanUp();
  }
}

print "<head><title>$prog_name $prog_version Ledger Configuration (Accounts)</title>\n";
print "<script language=\"JavaScript1.2\">\n";
if ($mode == 1)
{
  print <<"END_OF_CODE";
  function finish_assignment()
  {
    return true;
  }
END_OF_CODE
}
elsif ($mode == 2)
{
  print <<"END_OF_CODE";
  var AssignedAccounts = new Array();
  var UnAssignedAccounts = new Array();

  function set_account(text, value)
  {
    this.text = text;
    this.value = value;
  }

  function finish_assignment()
  {
    // massage all entries in the assigned and unassigned select boxes and stick them in the hidden variables.
    var d = document.assign_form;

    if (d.unassigned.length == 0)
    {
      d.unassigned_accounts.value = "";
    }
    else
    {
      var unassigned_string = "";

      for (var i=0; i < d.unassigned.length; i++)
      {
        if (unassigned_string.length > 0)
        {
          unassigned_string += "|";
        }
        unassigned_string += d.unassigned.options[i].value;
      }
      d.unassigned_accounts.value = unassigned_string;
    }

    if (d.assigned.length == 0)
    {
      d.assigned_accounts.value = "";
    }
    else
    {
      var assigned_string = "";

      for (var i=0; i < d.assigned.length; i++)
      {
        if (assigned_string.length > 0)
        {
          assigned_string += "|";
        }
        assigned_string += d.assigned.options[i].value;
      }
      d.assigned_accounts.value = assigned_string;
    }

    return true;
  }

  function assign_one_account()
  {
    var d = document.assign_form;

    if (d.unassigned.selectedIndex == -1)
    {
      alert("You must select at least one Account to Assign!");
      return;
    }

    var count = 0;
    for (var i=0; i < d.unassigned.length; i++)  // first pass copies stuff over.
    {
      if (d.unassigned.options[i].selected)
      {
        d.assigned.options[d.assigned.options.length] = new Option(d.unassigned.options[i].text, d.unassigned.options[i].value);
        count++;
      }
    }

    while (count > 0)
    {
      var found = false;
      for (var i=0; i < d.unassigned.length && !found; i++)
      {
        if (d.unassigned.options[i].selected)
        {
          d.unassigned.options[i] = null;  // Let JavaScript remove it for me. :)
          count--;
          found = true;
        }
      }
    }
  }

  function assign_all_accounts()
  {
    var d = document.assign_form;

    if (d.unassigned.length == 0)
    {
      alert("There must be accounts to Assign!");
      return;
    }

    for (var i=0; i < d.unassigned.length; i++)  // first pass copies stuff over.
    {
      d.assigned.options[d.assigned.options.length] = new Option(d.unassigned.options[i].text, d.unassigned.options[i].value);
    }

    d.unassigned.options.length = 0;  // empty the Un-Assigned box.
  }

  function unassign_one_account()
  {
    var d = document.assign_form;

    if (d.assigned.selectedIndex == -1)
    {
      alert("You must select at least one Account to Un-Assign!");
      return;
    }

    var count = 0;
    for (var i=0; i < d.assigned.length; i++)  // first pass copies stuff over.
    {
      if (d.assigned.options[i].selected)
      {
        d.unassigned.options[d.unassigned.options.length] = new Option(d.assigned.options[i].text, d.assigned.options[i].value);
        count++;
      }
    }

    while (count > 0)
    {
      var found = false;
      for (var i=0; i < d.assigned.length && !found; i++)
      {
        if (d.assigned.options[i].selected)
        {
          d.assigned.options[i] = null;  // Let JavaScript remove it for me. :)
          count--;
          found = true;
        }
      }
    }
  }

  function unassign_all_accounts()
  {
    var d = document.assign_form;

    if (d.assigned.length == 0)
    {
      alert("There must be accounts to Un-Assign!");
      return;
    }

    for (var i=0; i < d.assigned.length; i++)  // first pass copies stuff over.
    {
      d.unassigned.options[d.unassigned.options.length] = new Option(d.assigned.options[i].text, d.assigned.options[i].value);
    }

    d.assigned.options.length = 0;  // empty the Assigned box.
  }

  function reset_accounts()
  {
    var d = document.assign_form;

    d.unassigned.options.length = 0;

    for (var i=0; i < UnAssignedAccounts.length; i++)
    {
      d.unassigned.options[d.unassigned.options.length] = new Option(UnAssignedAccounts[i].text, UnAssignedAccounts[i].value);
    }

    d.assigned.options.length = 0;

    for (var i=0; i < AssignedAccounts.length; i++)
    {
      d.assigned.options[d.assigned.options.length] = new Option(AssignedAccounts[i].text, AssignedAccounts[i].value);
    }
  }

END_OF_CODE
}
print "</script>\n";
print "</head>\n";
print "<body bgcolor=$body_bgcolor>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\" width=50%>\n";
print "<tr>\n";
print "<td width=47%><br></td>\n";
print "<td align=left><a href=\"$site_dir/help.html#ledger_account_assignment\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardMainMenu($ledger_db);
print "<h2><font color=$prog_fgcolor>$prog_name $prog_version</font> Ledger Account Assignment</h2>\n";

# now we need to create the table that will display the selected account or ledger at the top and then list down the
# left column the ledgers or accounts that are available to assign this account to or to this ledger.
# the next column should have a set of radio buttons that say yes/no.

print <<"END_OF_FORM";
<form method=post name=\"assign_form\" action=\"$site_dir/cgi-bin/assign_accounts3.cgi\" onSubmit=\"return finish_assignment();\">
END_OF_FORM

if ($mode == 1)
{
  print "<h3>Account = <font color=\"blue\">$acct_name</font></h3>\n";
  print "<input type=hidden name=\"acct_name\" value=\"$acct_name\">\n";
  print "<input type=hidden name=\"acct_num\" value=\"$acct_num\">\n";
  print "<input type=hidden name=\"ledger_num\" value=\"". $result->ntuples. "\">\n";
}
elsif ($mode == 2)
{
  print "<h3>Ledger = <font color=\"blue\">$ledger_name</font></h3>\n";
  print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
  print "<input type=hidden name=\"ledger_name\" value=\"$ledger_name\">\n";
  if ($really_in_ledger eq "yes")
  {
    print "<input type=hidden name=\"really_in_ledger\" value=\"yes\">\n";
  }
  print "<input type=hidden name=\"account_num\" value=\"" . $result->ntuples . "\">\n";
}

print <<"END_OF_FORM";
<table border=0 cellspacing=0 cellpadding=2 bgcolor=\"$menu_bgcolor\">
<tr>
END_OF_FORM

if ($mode == 1)
{
  print "<th align=center>Ledger</th><th align=center colspan=2>Assigned</th>\n";
  print "</tr>\n";

  # now we loop through all the ledgers and connect to each one and see if
  # this account has been assigned to the ledger and display it in the
  # table.  We set the assigned flag based upon the previous result.

  my ($i, $name, $ldb_name, $assigned);
  for ($i=0; $i < $result->ntuples; $i++)
  {
    $name = $result->getvalue($i, 0);
    $ldb_name = $result->getvalue($i, 1);

    # create the reports object
    $report_obj = reports->new($conn, $ldb_name);
    if ($report_obj->didErrorOccur())
    {
      PrintErrorHeader();
      print $report_obj->getErrorMessage();
      CleanUp();
    }
    $ledger_code = $report_obj->getLedgerCode();
    if ($report_obj->didErrorOccur())
    {
      PrintErrorHeader();
      print $report_obj->getErrorMessage();
      CleanUp();
    }

    # now we check to see if this account has been assigned to this ledger.
    $result2 = $conn->exec("select acct_num from local_accounts_tb where acct_num = '$acct_num' and sub_entity_code is null AND ledger_code = '$ledger_code'");
    if ($result2->resultStatus != PGRES_TUPLES_OK)
    {
      PrintErrorHeader();
      print "<h1>Error:  Accessing database <font color=\"blue\">$ldb_name</font> failed!</h1>\n";
      print $conn->errorMessage . "<br>Tried to lookup account <font color=\"blue\">$acct_name</font>.<br>$sys_admin";
      CleanUp();
    }
    if ($result2->ntuples == 0)
    { # they haven't assigned the account to this ledger
      $assigned = 0;
    }
    else
    { # they have assigned the account to this ledger
      $assigned = 1;
    }

    # now display the info for this ledger
    print "<tr><td>";
    print "<input type=hidden name=\"ledger_$i\" value=\"$name\">\n";
    print "<font color=\"$menu_fgcolor\">$name</font></td>\n";
    if ($assigned)
    {
      print "<td><input type=radio name=\"radio_$i\" value=\"YES\" checked> Yes</td>\n";
      print "<td><input type=radio name=\"radio_$i\" value=\"NO\"> No</td>\n";
    }
    else
    {
      print "<td><input type=radio name=\"radio_$i\" value=\"YES\"> Yes</td>\n";
      print "<td><input type=radio name=\"radio_$i\" value=\"NO\" checked> No</td>\n";
    }
    print "</tr>\n";
  }
}
elsif ($mode == 2)
{
  print "<th align=center>Un-Assigned</th><th>&nbsp;</th><th align=center>Assigned</th>\n";
  print "</tr>\n";

  # now we connect to the ledger and loop through all accounts and see
  # if any are assigned.  We display each account and set the assigned
  # flag based upon the previous result.

  my ($assigned_accounts, $unassigned_accounts);

  # create the reports object
  $report_obj = reports->new($conn, $ledger_name);
  if ($report_obj->didErrorOccur())
  {
    PrintErrorHeader();
    print $report_obj->getErrorMessage();
    CleanUp();
  }
  $ledger_code = $report_obj->getLedgerCode();
  if ($report_obj->didErrorOccur())
  {
    PrintErrorHeader();
    print $report_obj->getErrorMessage();
    CleanUp();
  }

  my ($i, $name, $num, $assigned, $taxable);
  for ($i=0; $i < $result->ntuples; $i++)
  {
    $name = $result->getvalue($i, 0);
    $num = $result->getvalue($i, 1);
    $taxable = ($result->getvalue($i, 2) eq "t" ? "~ " : "");
    # now we check to see if this account has been assigned to this ledger.
    $result2 = $conn->exec("select acct_num from local_accounts_tb where acct_num = '$num' and sub_entity_code is null AND ledger_code = '$ledger_code'");
    if ($result2->resultStatus != PGRES_TUPLES_OK)
    {
      PrintErrorHeader();
      print "<h1>Error:  Accessing database <font color=\"blue\">$ledger_name</font> failed!</h1>\n";
      print $conn->errorMessage . "<br>Tried to lookup account <font color=\"blue\">$name</font>.<br>$sys_admin";
      CleanUp();
    }
    if ($result2->ntuples == 0)
    { # they haven't assigned the account to this ledger
      $assigned = 0;
    }
    else
    { # they have assigned the account to this ledger
      $assigned = 1;
    }

    # now display the info for this ledger
    #print "<tr><td>";
    #print "<input type=hidden name=\"account_$i\" value=\"$name\">\n";
    #print "<input type=hidden name=\"account_num_$i\" value=\"$num\">\n";
    #print "<font color=\"$menu_fgcolor\">$name - $num</font></td>\n";
    if ($assigned)
    {
      #print "<td><input type=radio name=\"radio_$i\" value=\"YES\" checked> Yes</td>\n";
      #print "<td><input type=radio name=\"radio_$i\" value=\"NO\"> No</td>\n";
      $assigned_accounts .= "|" if (length $assigned_accounts > 0);
      $assigned_accounts .= "$taxable$name!!$num";
    }
    else
    {
      #print "<td><input type=radio name=\"radio_$i\" value=\"YES\"> Yes</td>\n";
      #print "<td><input type=radio name=\"radio_$i\" value=\"NO\" checked> No</td>\n";
      $unassigned_accounts .= "|" if (length $unassigned_accounts > 0);
      $unassigned_accounts .= "$taxable$name!!$num";
    }
    #print "</tr>\n";
  }
  print <<"END_OF_CODE";
  <script language="JavaScript1.2">
    var count=0;
END_OF_CODE
  foreach my $value (split(/\|/, $unassigned_accounts))
  {
    my ($name, $num) = split /!!/, $value;
    print "  UnAssignedAccounts[count++] = new set_account(\"$name - $num\", \"$value\");\n";
  }
  print <<"END_OF_CODE";
    count = 0;
END_OF_CODE
  foreach my $value (split(/\|/, $assigned_accounts))
  {
    my ($name, $num) = split /!!/, $value;
    print "  AssignedAccounts[count++] = new set_account(\"$name - $num\", \"$value\");\n";
  }
  print <<"END_OF_CODE";
  </script>
END_OF_CODE
  print "<tr>\n";
  print "  <td align=center><select name=\"unassigned\" multiple size=\"10\" width=\"300\">\n";
  foreach my $value (split(/\|/, $unassigned_accounts))
  {
    my ($name, $num) = split /!!/, $value;
    print "      <option value=\"$value\">$name - $num\n";
  }
  print "    </select>\n";
  print "  </td>\n";
  print "  <td align=center><input type=button name=\"assign_all\" value=\">>\" onClick=\"assign_all_accounts();\"><br>\n";
  print "    <input type=button name=\"assign_one\" value=\">\" onClick=\"assign_one_account();\"><br><br>\n";
  print "    <input type=button name=\"unassign_one\" value=\"<\" onClick=\"unassign_one_account();\"><br>\n";
  print "    <input type=button name=\"unassign_all\" value=\"<<\" onClick=\"unassign_all_accounts();\">\n";
  print "  </td>\n";
  print "  <td align=center><select name=\"assigned\" multiple size=\"10\" width=\"300\">\n";
  foreach my $value (split(/\|/, $assigned_accounts))
  {
    my ($name, $num) = split /!!/, $value;
    print "      <option value=\"$value\">$name - $num\n";
  }
  print "    </select>\n";
  print "  </td>\n";
  print "</tr>\n";
  print "<input type=hidden name=\"assigned_accounts\" value=\"$assigned_accounts\"><input type=hidden name=\"unassigned_accounts\" value=\"$unassigned_accounts\">\n";
}

print "</table>\n";

if ($mode == 1)
{
  print "<input type=submit name=\"submit_account\" value=\"Update Ledgers\">&nbsp;<input type=reset value=\"Reset\">";
}
elsif ($mode == 2)
{
  print "<input type=submit name=\"submit_ledger\" value=\"Update Ledger\">&nbsp;<input type=reset value=\"Reset\" onClick=\"reset_accounts();  return false;\">";
  print "<br><i>~ indicates a taxable account</i>\n";
}

print <<"END_OF_FORM";
</form>
END_OF_FORM

# print a horizontal rule to break the menu from the behind the scenes links
print "<br><hr>\n";

if ($really_in_ledger ne "yes")
{
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/assign_accounts.cgi\">Ledger Account Assignment</a></div>\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/account_config.cgi\">Account Configuration</a></div>\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/prg_config.cgi\">Program Configuration</a></div>\n";
}
else
{
  print "<form name=\"form1\" method=post action=\"\">\n";
  print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/account_config.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/account_config.cgi'); return false;\">Account Configuration</a></div>\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/prg_config.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/prg_config.cgi'); return false;\">Program Configuration</a></div>\n";
  print "<input type=hidden name=\"open\" value=\"Open\">\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/ledger.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/ledger.cgi'); return false;\">Return to Ledger Summary</a></div>\n";
  print "</form>\n";
}

print "</body>\n";
print "</html>\n";
exit 0;
