#!/usr/bin/perl
#$Id: calculate_quarters.cgi,v 1.1 2001/09/06 10:21:19 moreejt Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

# calculate_quarters.cgi  -  Created by James Pattie, 11/23/1999
# This script calculates the quarters start and end dates based
# upon the info the user specified in the create ledger page.
# It then pushes all information the new_delete_ledger script to
# actually create the ledger.
use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features

use vars '$date';

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

my $ledger_name;  # name of ledger to work with
my ($ledger_code, $ledger_db, $ledger_color, $cash_accrual, $business_type, $fiscal_month, $fiscal_day, $fiscal_current, $fiscal_year, $start_year, $yes_no, $old_ledger_name, $mode);
my $error_string;  # used to store the result of IsValidString();

# validate we got valid input from the user.
if (exists $input{'new2'})
{
  $ledger_name = $input{'ledger_name'};
  $error_string = IsValidString($ledger_name);
  if (length $error_string > 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Ledger Name!</h1>\n";
    print "<b>$error_string</b> is invalid.<br>$back_button";
    CleanUp();
  }
  $ledger_db   = $input{'db_name'};
  if (length $ledger_db == 0)
  {
    PrintErrorHeader();
    print "<h1>Error: You must specify the Ledger Database name!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  $ledger_code = $input{'ledger_code'};
  if (length $ledger_code < 1)
  {
    PrintErrorHeader();
    print "<h1>Error: You have to enter at least one character for your code!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  $error_string = IsValidString($ledger_code);
  if (length $error_string > 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Ledger Code!</h1>\n";
    print "<b>$error_string</b> is invalid.<br>$back_button";
    CleanUp();
  }
  if (not exists $input{'color'})
  {
    PrintErrorHeader();
    print "<h1>Error: You must select a Color!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  $ledger_color = $input{'color'};
  if (not exists $input{'cash_accrual'})
  {
    PrintErrorHeader();
    print "<h1>Error: You must select either Cash or Accrual!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  $cash_accrual = $input{'cash_accrual'};
  if (not ($cash_accrual eq "cash" or $cash_accrual eq "accrual"))
  {
    PrintErrorHeader();
    print "<h1>Error: Invalid value for Cash or Accrual!</h1>\n";
    print "$sys_admin";
    CleanUp();
  }
  $business_type = $input{'business_type'};
  if (length $business_type == 0)
  {
    PrintErrorHeader();
    print "<h1>Error: You must select a Business Type!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  $fiscal_month = $input{'fiscal_month'};
  if (length $fiscal_month == 0)
  {
    PrintErrorHeader();
    print "<h1>Error: You must enter a month between 1 and 12!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  $error_string = IsValidInteger($fiscal_month);
  if (length $error_string > 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Fiscal Month Value!</h1>\n";
    print "<b>$error_string</b> is invalid.<br>$back_button";
    CleanUp();
  }
  if ($fiscal_month < 1 or $fiscal_month > 12)
  {
    PrintErrorHeader();
    print "<h1>Error: You must enter a month between 1 and 12!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  $fiscal_day = $input{'fiscal_day'};
  if (length $fiscal_day == 0)
  {
    PrintErrorHeader();
    print "<h1>Error: You must enter a valid day for the month specified!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  $error_string = IsValidInteger($fiscal_day);
  if (length $error_string > 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Fiscal Day Value!</h1>\n";
    print "<b>$error_string</b> is invalid.<br>$back_button";
    CleanUp();
  }
  if ($fiscal_day < 1 or $fiscal_day > 31)
  {
    PrintErrorHeader();
    print "<h1>Error: The day you entered is obviously wrong!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  if ($fiscal_month == 2 && $fiscal_day > 28)
  {
    PrintErrorHeader();
    print "<h1>Error:  February's official last day is the 28<sup>th</sup>!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  if ($fiscal_month == 4 && $fiscal_day == 31)
  {
    PrintErrorHeader();
    print "<h1>Error:  April's last day is the 30<sup>th</sup>!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  if ($fiscal_month == 6 && $fiscal_day == 31)
  {
    PrintErrorHeader();
    print "<h1>Error:  June's last day is the 30<sup>th</sup>!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  if ($fiscal_month == 9 && $fiscal_day == 31)
  {
    PrintErrorHeader();
    print "<h1>Error:  September's last day is the 30<sup>th</sup>!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  if ($fiscal_month == 11 && $fiscal_day == 31)
  {
    PrintErrorHeader();
    print "<h1>Error:  November's last day is the 30<sup>th</sup>!</h1>\n";
    print "$back_button";
    CleanUp();
  }
#  $fiscal_current = $input{'fiscal_year'};
#  if ($fiscal_current eq "Current")
#  {
#    $fiscal_current = "true";
#  }
#  elsif ($fiscal_current eq "Last")
#  {
#    $fiscal_current = "false";
#  }
#  else
#  {
#    PrintErrorHeader();
#    print "<h1>Error:  Invalid value for Fiscal Year Current or Previous!</h1>\n";
#    print "$sys_admin";
#    CleanUp();
#  }
  $fiscal_year = $input{'fiscal_year'};
  if ($fiscal_year <= 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid value for Fiscal Year!</h1>\n";
    print "You have to enter a year > 0.<br>$back_button";
    CleanUp();
  }
  $error_string = IsValidInteger($fiscal_year);
  if (length $error_string > 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Fiscal Year Value!</h1>\n";
    print "<b>$error_string</b> is invalid.<br>$back_button";
    CleanUp();
  }
  $start_year = $input{'start_year'};
  if ($start_year <= 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid value for Starting Book Year!</h1>\n";
    print "You have to enter a year > 0.<br>$back_button";
    CleanUp();
  }
  $error_string = IsValidInteger($start_year);
  if (length $error_string > 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Start Year Value!</h1>\n";
    print "<b>$error_string</b> is invalid.<br>$back_button";
    CleanUp();
  }
  # if fiscal_year is equal to start_year, fiscal_current is true
  if ($fiscal_year == $start_year)
  {
    $fiscal_current = "true";
  }
  else
  {
    $fiscal_current = "false";
  }
  $mode = 3;
}
else
{
  PrintErrorHeader();
  print "<h1>Access Denied!</h1>\n";
  print "Error: Invalid options given!\n";
  CleanUp();
}

if (length $ledger_name == 0)
{
  PrintErrorHeader();
  print "Error: You must specify a ledger to work with!<br>\n";
  print "$back_button";
  CleanUp();
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    &PrintErrorHeader;
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print "$sys_admin";
    &CleanUp;
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# now validate that the ledger they specified is valid, if in mode 2
# or that it doesn't already exist if in mode 3.
$result = $conn->exec("select database_name from entity_tb where name = '$ledger_name'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print "Trying to check entity table.<br>$sys_admin";
  CleanUp();
}
if ($mode == 3)
{ # we want to create a new one so make sure it doesn't exist
  if ($result->ntuples > 0)
  {
    PrintErrorHeader();
    print "<h1>Error: Ledger already exists!</h1>\n";
    print "$back_button";
    CleanUp();
  }
}

# if the mode is create or edit ledger, then validate that the business type is valid.
if ($mode == 3)
{
  $result = $conn->exec("select name from business_type_tb where num = $business_type");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    PrintErrorHeader();
    print "<h1>Error: Accessing database failed!</h1>\n";
    print "Trying to check business type validity.<br>$sys_admin";
    CleanUp();
  }
  if ($result->ntuples == 0)
  {
    PrintErrorHeader();
    print "<h1>Internal Error: No Business Types defined!</h1>\n";
    print "$sys_admin";
    CleanUp();
  }
  if ($result->ntuples != 1)
  {
    PrintErrorHeader();
    print "<h1>Error: Invalid option for Business Type!</h1>\n";
    print "$sys_admin";
    CleanUp();
  }
}

# present them with the main menu 

print "<head><title>$prog_name $prog_version Ledger Maintenance</title>\n";
  print <<"END_OF_SCRIPT";
  <script language="JavaScript1.2">
  function ValidateLedgerForm()
  {
    var msg = "";
    var doc = document.ledger_form;
    if (doc.q1_start.value.length == 0)
    {
      msg += "You must specify the Start Date for the First Quarter!\\n";
    }
    else
    {
      var tmp = IsValidDateSyntax(doc.q1_start.value);
      if (tmp.length > 0)
      {
        msg += tmp + "\\n";
      }
    }
    if (doc.q1_end.value.length == 0)
    {
      msg += "You must specify the End Date for the First Quarter!\\n";
    }
    else
    {
      var tmp = IsValidDateSyntax(doc.q1_end.value);
      if (tmp.length > 0)
      {
        msg += tmp + "\\n";
      }
    }
    if (doc.q2_start.value.length == 0)
    {
      msg += "You must specify the Start Date for the Second Quarter!\\n";
    }
    else
    {
      var tmp = IsValidDateSyntax(doc.q2_start.value);
      if (tmp.length > 0)
      {
        msg += tmp + "\\n";
      }
    }
    if (doc.q2_end.value.length == 0)
    {
      msg += "You must specify the End Date for the Second Quarter!\\n";
    }
    else
    {
      var tmp = IsValidDateSyntax(doc.q2_end.value);
      if (tmp.length > 0)
      {
        msg += tmp + "\\n";
      }
    }
    if (doc.q3_start.value.length == 0)
    {
      msg += "You must specify the Start Date for the Third Quarter!\\n";
    }
    else
    {
      var tmp = IsValidDateSyntax(doc.q3_start.value);
      if (tmp.length > 0)
      {
        msg += tmp + "\\n";
      }
    }
    if (doc.q3_end.value.length == 0)
    {
      msg += "You must specify the End Date for the Third Quarter!\\n";
    }
    else
    {
      var tmp = IsValidDateSyntax(doc.q3_end.value);
      if (tmp.length > 0)
      {
        msg += tmp + "\\n";
      }
    }
    if (doc.q4_start.value.length == 0)
    {
      msg += "You must specify the Start Date for the Fourth Quarter!\\n";
    }
    else
    {
      var tmp = IsValidDateSyntax(doc.q4_start.value);
      if (tmp.length > 0)
      {
        msg += tmp + "\\n";
      }
    }
    if (doc.q4_end.value.length == 0)
    {
      msg += "You must specify the End Date for the Fourth Quarter!\\n";
    }
    else
    {
      var tmp = IsValidDateSyntax(doc.q4_end.value);
      if (tmp.length > 0)
      {
        msg += tmp + "\\n";
      }
    }

    if (msg.length > 0)
    {
      alert(msg);
      return false;
    }
    alert("Creation of the Ledger may take up to a minute!\\n\\nPlease be patient.");
    return true;
  }
  </script>
END_OF_SCRIPT
print "</head>\n";
print "<body bgcolor=$body_bgcolor>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td align=left width=47%><font color=\"red\">";
if ($mode == 3)
{
  print "Creating Ledger <font color=\"blue\">$ledger_name</font> - Calculating Quarter Info";
}
print "</font></td>\n";
if ($mode == 3)
{
  print "<td align=left><a href=\"$site_dir/help.html#ledger_creation\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
}
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardMainMenu($ledger_db);

# for now I'm assuming that we really delete the ledger or create it and
# don't support the feature of just removing it from the list of entities, etc.

if ($mode == 3) # Calculate the Quarter Info
{
  # I need to calculate the beginning and ending dates for the current fiscal year and insert them into the known_years_tb table.
  my ($fiscal_start_date, $fiscal_end_date);
  my $prevFiscalEndDate;
  $prevFiscalEndDate = $fiscal_month . "/" . $fiscal_day . "/" . $fiscal_year;
  
  $fiscal_start_date = `/bin/date -d \"$prevFiscalEndDate 1 year ago 1 day\" +%m/%d/%Y`;
  chomp $fiscal_start_date;

  $fiscal_end_date = `/bin/date -d \"$fiscal_start_date 1 day ago 1 year\" +%m/%d/%Y`;
  chomp $fiscal_end_date;

  # now we calculate the quarter start and end dates and display them
  # in edit boxes along with all the other fields as hidden values to
  # be passed into the new_delete_ledger.cgi script.
  print << "END_OF_FORM";
  <form method=post action=\"$site_dir/cgi-bin/new_delete_ledger.cgi\" name=\"ledger_form\" onSubmit=\"return ValidateLedgerForm();\">
  <input type=hidden name=\"ledger_name\" value=\"$ledger_name\">
  <input type=hidden name=\"db_name\" value=\"$ledger_name\">
  <input type=hidden name=\"ledger_code\" value=\"$ledger_code\">
  <input type=hidden name=\"color\" value=\"$ledger_color\">
  <input type=hidden name=\"cash_accrual\" value=\"$cash_accrual\">
  <input type=hidden name=\"business_type\" value=\"$business_type\">
  <input type=hidden name=\"start_year\" value=\"$start_year\">
  <input type=hidden name=\"fiscal_month\" value=\"$fiscal_month\">
  <input type=hidden name=\"fiscal_day\" value=\"$fiscal_day\">
  <input type=hidden name=\"fiscal_year\" value=\"$fiscal_year\">
  <table border=0 width=75% cellspacing=0 cellpadding=2 bgcolor=\"$menu_bgcolor\">
END_OF_FORM
  if ($fiscal_month == 12 && $fiscal_day == 31) # calendar year
  {
    print "<tr><td align=right><font color=\"$menu_fgcolor\">Quarter 1 - Start:</font></td><td><input type=text name=\"q1_start\" value=\"01-01-$start_year\"></td></tr>\n";
    print "<tr><td align=right><font color=\"$menu_fgcolor\">End:</font></td><td><input type=text name=\"q1_end\" value=\"03-31-$start_year\"></td></tr>\n";

    print "<tr><td align=right><font color=\"$menu_fgcolor\">Quarter 2 - Start:</font></td><td><input type=text name=\"q2_start\" value=\"04-01-$start_year\"></td></tr>\n";
    print "<tr><td align=right><font color=\"$menu_fgcolor\">End:</font></td><td><input type=text name=\"q2_end\" value=\"06-30-$start_year\"></td></tr>\n";

    print "<tr><td align=right><font color=\"$menu_fgcolor\">Quarter 3 - Start:</font></td><td><input type=text name=\"q3_start\" value=\"07-01-$start_year\"></td></tr>\n";
    print "<tr><td align=right><font color=\"$menu_fgcolor\">End:</font></td><td><input type=text name=\"q3_end\" value=\"09-30-$start_year\"></td></tr>\n";

    print "<tr><td align=right><font color=\"$menu_fgcolor\">Quarter 4 - Start:</font></td><td><input type=text name=\"q4_start\" value=\"10-01-$start_year\"></td></tr>\n";
    print "<tr><td align=right><font color=\"$menu_fgcolor\">End:</font></td><td><input type=text name=\"q4_end\" value=\"12-31-$start_year\"></td></tr>\n";
  }
  else
  {
    my ($k, $start_date, $quarter_start, $quarter_end);
    $start_date = $fiscal_start_date;
    for ($k=1; $k <= 4; $k++)
    {
      $quarter_start = $start_date;
      if ($k != 4)
      {
        #$quarter_end = `/bin/date --date \"$start_date 90 days\" +%m/%d/%Y`;
        $quarter_end = `/bin/date --date \"$start_date 1 day ago 3 months\" +%m/%d/%Y`;
        chomp $quarter_end;

        $start_date = `/bin/date --date \"$quarter_end 1 day\" +%m/%d/%Y`;
        chomp $start_date;
      }
      else
      {
        $quarter_end = `/bin/date --date \"$start_date 3 months\" +%m/%d/%Y`;
        chomp $quarter_end;

        if (not ($quarter_end eq $fiscal_end_date))  # must be a leap year.
        {
          $quarter_end = $fiscal_end_date;
        }
      }
      $quarter_start =~ s/\//-/g;
      $quarter_end =~ s/\//-/g;
      print "<tr><td align=right><font color=\"$menu_fgcolor\">Quarter $k - Start:</font></td><td><input type=text name=\"q" . $k . "_start\" value=\"$quarter_start\"></td></tr>\n";
      print "<tr><td align=right><font color=\"$menu_fgcolor\">End:</font></td><td><input type=text name=\"q" . $k . "_end\" value=\"$quarter_end\"></td></tr>\n";
    }
  }

  print "</table>\n<input type=submit name=\"new2\" value=\"Create Ledger\">&nbsp;<input type=reset name=reset value=\"Reset\">\n";
  print "</form>\n";
}

# print a horizontal rule to break the menu from the behind the scenes links
#print "<br><hr>\n";

print "</body>\n";
print "</html>\n";
exit 0;
