#!/usr/bin/perl
#$Id: create_account.cgi,v 1.1 2001/09/06 10:21:19 moreejt Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

# create_account.cgi  -  Created by James Pattie, 05/22/1999
# This script allows the user to create new accounts.

use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features
my $ledger_db;

use vars '$date';

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

$ledger_db = $input{'ledger'};

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print "$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# do a database lookup here and fill in the select box with all the account types in the database.
$result = $conn->exec("select name, num, first_digit from acct_type_tb WHERE num <> '8' AND  num <> '18' order by first_digit, name");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get account types.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  print "<h1>Internal Error: No Account Types exist!</h1>\n";
  print "Tried to get account types.<br>$sys_admin";
  CleanUp();
}

print "<head><title>$prog_name $prog_version Create Accounts</title></head>\n";
print "<body bgcolor=$body_bgcolor>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
<script language="Javascript1.2">
  var AccountTypeData = new Array();

  function AccountTypeObject(name, num, first_digit)
  {
    this.name = name;
    this.num = num;
    this.first_digit = first_digit;
  }

END_OF_STYLE

for (my $i=0; $i < $result->ntuples; $i++)
{
  print "  AccountTypeData[$i] = new AccountTypeObject(\"".$result->getvalue($i, 0)."\",\"".$result->getvalue($i,1)."\",\"".$result->getvalue($i,2)."\");\n";
}

print <<"END_OF_STYLE";

  function validate_tax_status()
  {
    var d = document.acct_form;
    var index = d.acct_type.selectedIndex;

    if (index == -1)
    {
      alert("You must select an Account Type!");
      return false;
    }

    if (AccountTypeData[index].first_digit == "4" || AccountTypeData[index].first_digit == "5")
    {
      // don't need to do anything.
    }
    else
    {
      if (d.acct_taxable.checked)
      {
        d.acct_taxable.checked = false;
      }
    }

    return true;
  }

  function default_tax_status()
  {
    var d = document.acct_form;
    var index = d.acct_type.selectedIndex;

    if (index == -1)
    {
      alert("You must select an Account Type!");
      return false;
    }

    if (AccountTypeData[index].first_digit == "4" || AccountTypeData[index].first_digit == "5")
    {
      d.acct_taxable.checked = true;  // default to taxable.
    }
    else
    {
      d.acct_taxable.checked = false;
    }

    return true;
  }
</script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td width=47%><br></td>\n";
print "<td align=left><a href=\"$site_dir/help.html#create_accounts\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardMainMenu($ledger_db);
print "<h2><font color=$prog_fgcolor>$prog_name $prog_version</font> Create Account</h2>\n";

# do a database lookup and get the size that an account must be in length for the number.
my $result2;
$result2 = $conn->exec("select acc_digits from version_tb");
if ($result2->resultStatus != PGRES_TUPLES_OK)
{
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get account number size info.<br>$sys_admin";
  CleanUp();
}
if ($result2->ntuples == 0)
{
  print "<h1>Internal Error: Application not configured!</h1>\n";
  print "Tried to get account number size info.<br>$sys_admin";
  CleanUp();
}

my $acct_num_size;
$acct_num_size = $result2->getvalue(0, 0);

# create the input form to gather the account information
print <<"END_OF_FORM";
<form method=post action=\"$site_dir/cgi-bin/modify_account.cgi\" onSubmit="return validate_tax_status()" name="acct_form">
<table border=0 cellspacing=0 cellpadding=2 bgcolor=\"$menu_bgcolor\">
<tr>
  <td><font color=\"$menu_fgcolor\">Account Type</font></td>
  <td><select name=\"acct_type\" onChange="default_tax_status()">
END_OF_FORM

# get the Account Type info from the previous database call.
my $i;
for ($i=0; $i < $result->ntuples; $i++)
{
  my $name = $result->getvalue($i, 0);
  my $num = $result->getvalue($i, 1);
  my $first = $result->getvalue($i, 2);
  print "<option value=\"$num\">$name ($first)\n";
}

print <<"END_OF_FORM";
  </select></td>
</tr>
<tr>
  <td><font color=\"$menu_fgcolor\">Account Name</font></td>
  <td><input type=text name=\"acct_name\"></td>
</tr>
<tr>
  <td><font color=\"$menu_fgcolor\">Account Number</font></td>
  <td><input type=text name=\"acct_num\" size=\"$acct_num_size\" maxlength=\"$acct_num_size\"></td>
</tr>
<tr>
  <td><font color="$menu_fgcolor">Account is Taxable</font></td>
  <td><input type=checkbox name="acct_taxable" value="YES" onClick="validate_tax_status()"></td>
</tr>
<tr>
  <td><font color=\"$menu_fgcolor\">Account is Active</font></td>
  <td><input type=checkbox name=\"acct_active\" value=\"YES\" checked></td>
</tr>
</table>
END_OF_FORM
if (length $ledger_db > 0)
{
  print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
}
print <<"END_OF_FORM";
<input type=submit name=\"create_account\" value=\"Create Account\">&nbsp;
<input type=reset value=\"Reset\">
</form>
END_OF_FORM

# print a horizontal rule to break the menu from the behind the scenes links
print "<br><hr>\n";

if (length $ledger_db == 0)
{
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/account_config.cgi\">Account Configuration</a></div>\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/prg_config.cgi\">Program Configuration</a></div>\n";
}
else
{
  print "<form name=\"form1\" method=post action=\"\">\n";
  print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/account_config.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/account_config.cgi'); return false;\">Account Configuration</a></div>\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/prg_config.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/prg_config.cgi'); return false;\">Program Configuration</a></div>\n";
  print "<input type=hidden name=\"open\" value=\"Open\">\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/ledger.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/ledger.cgi'); return false;\">Return to Ledger Summary</a></div>\n";
  print "</form>\n";
}

print "</body>\n";
print "</html>\n";
exit 0;
