#!/usr/bin/perl
#$Id: edit_account.cgi,v 1.3 2001/10/25 17:27:36 pcxuser Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

# edit_account.cgi  -  Created by James Pattie, 05/22/1999
# This script allows the user to edit accounts.

use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features

use vars '$date';
my $ledger_db;

my %restrictedAccounts = ( 4200 => "Short Term Capital Gain", 4201 => "Long Term Capital Gain", 5200 => "Short Term Capital Loss", 5201 => "Long Term Capital Loss", );

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

$ledger_db = $input{'ledger'};

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

print "<head><title>$prog_name $prog_version Edit Accounts</title>\n";

print <<"END_OF_SCRIPT";
  <script language="Javascript1.2">
    function doLoad()
    {
      document.edit_form.acct_name.selectedIndex = 0;
    }

    function validate_form()
    {
      if (document.edit_form.acct_name.selectedIndex == -1)
      {
        alert("You must select an Account to Edit!");
        return false;
      }
      return true;
    }
  </script>
END_OF_SCRIPT

print "</head>\n";
print "<body bgcolor=$body_bgcolor onLoad=\"doLoad();\">\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td width=47%><br></td>\n";
print "<td align=left><a href=\"$site_dir/help.html#edit_accounts\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardMainMenu($ledger_db);
print "<h2><font color=$prog_fgcolor>$prog_name $prog_version</font> Edit Accounts</h2>\n";

# create the form that lets them select from the accounts to work with.
print "<form method=post action=\"$site_dir/cgi-bin/edit_account2.cgi\" name=\"edit_form\" onSubmit=\"return validate_form()\">\n";
if (length $ledger_db > 0)
{
  print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
}
print <<"END_OF_FORM";
<table border=0 cellspacing=0 cellpadding=2 bgcolor=\"$menu_bgcolor\">
<tr>
  <td><font color=\"$menu_fgcolor\">Select Account to Edit</font></td>
  <td><select name=\"acct_name\" onChange=\"document.edit_form.submit();\">
END_OF_FORM

# now do a database lookup and printout all accounts with number - name format
$result = $conn->exec("select name, number from account_tb WHERE number NOT IN (" . join(",", keys(%restrictedAccounts)) . ") order by name");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get all accounts.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  print "<h1>Internal Error:  No accounts defined!</h1>\n";
  print "Tried to get all accounts.<br>$sys_admin";
  CleanUp();
}
my $i;
for ($i=0; $i < $result->ntuples; $i++)
{
  my $name = $result->getvalue($i, 0);
  my $num  = $result->getvalue($i, 1);
  print "<option value=\"$num - $name\">$name - $num\n";
}

print <<"END_OF_FORM";
  </select></td>
</tr>
</table>
<input type=submit name="submit_for_edit" value="Edit">
</form>
END_OF_FORM

# print a horizontal rule to break the menu from the behind the scenes links
print "<br><hr>\n";

if (length $ledger_db == 0)
{
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/account_config.cgi\">Account Configuration</a></div>\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/prg_config.cgi\">Program Configuration</a></div>\n";
}
else
{
  print "<form name=\"form1\" method=post action=\"\">\n";
  print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/account_config.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/account_config.cgi'); return false;\">Account Configuration</a></div>\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/prg_config.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/prg_config.cgi'); return false;\">Program Configuration</a></div>\n";
  print "<input type=hidden name=\"open\" value=\"Open\">\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/ledger.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/ledger.cgi'); return false;\">Return to Ledger Summary</a></div>\n";
  print "</form>\n";
}

print "</body>\n";
print "</html>\n";
exit 0;
