#!/usr/bin/perl
#$Id: edit_account2.cgi,v 1.1 2001/09/06 10:21:19 moreejt Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

# edit_account2.cgi  -  Created by James Pattie, 05/22/1999
# This script allows the user to edit the selected account.

use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features

use vars '$date';
my $error_string;  # used to hold the returned value from IsValidString();
my $ledger_db;

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

$ledger_db = $input{'ledger'};

# validate that I have a valid account to work with.
my ($acct_name, $acct_num);
$acct_name = $input{'acct_name'};
if (length $acct_name == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  You must select an account to work with!</h1>\n";
  print "$back_button";
  CleanUp();
}

# now we split the name and number apart.
($acct_num, $acct_name) = split / - /, $acct_name;
if (length $acct_num == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  Invalid Account Number specified!</h1>\n";
  print "$sys_admin";
  CleanUp();
}
$error_string = IsValidInteger($acct_num);
if (length $error_string != 0)
{
  PrintErrorHeader();
  print "<h1>Error:  Invalid Account Number!</h1>\n";
  print "<b>$error_string</b> is invalid.<br>$back_button";
  CleanUp();
}
if (length $acct_name == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  Invalid Account Name specified!</h1>\n";
  print "$sys_admin";
  CleanUp();
}
$error_string = IsValidString($acct_name);
if (length $error_string != 0)
{
  PrintErrorHeader();
  print "<h1>Error:  Invalid Account Name!</h1>\n";
  print "<b>$error_string</b> is invalid.<br>$back_button";
  CleanUp();
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# validate the version of the database.
$result = $conn->exec("select version from version_tb");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to check database version.<br>$sys_admin";
  CleanUp();
}
my $db_version;
$db_version = $result->getvalue(0, 0);
if ($db_version lt "1.2")
{
  PrintErrorHeader();
  print "<h1>Error: &nbsp;You need to upgrade your database to at least version <font color=\"red\">1.2</font>!</h1><br>$sys_admin";
  CleanUp();
}

# now we validate that the account name and number given is valid.
my ($acct_type, $acct_active, $acct_taxable);
$result = $conn->exec("select type, active, taxable from account_tb where name = '$acct_name' and number = $acct_num"); # removed '' around number
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  Account <font color=\"blue\">$acct_name</font> with account number <font color=\"blue\">$acct_num</font> does not exist!</h1>\n";
  print "$sys_admin";
  CleanUp();
}
else
{
  $acct_type    = $result->getvalue(0, 0);
  $acct_active  = $result->getvalue(0, 1);
  $acct_taxable = $result->getvalue(0, 2);
}

# do a database lookup and get the size that an account must be in length for the number.
$result = $conn->exec("select acc_digits from version_tb");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get account number size info.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Internal Error: Application not configured!</h1>\n";
  print "Tried to get account number size info.<br>$sys_admin";
  CleanUp();
}

my $acct_num_size;
$acct_num_size = $result->getvalue(0, 0);

print "<head><title>$prog_name $prog_version Edit Account</title></head>\n";
print "<body bgcolor=$body_bgcolor>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td width=47%><br></td>\n";
print "<td align=left><a href=\"$site_dir/help.html#edit_account\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardMainMenu($ledger_db);
print "<h2><font color=$prog_fgcolor>$prog_name $prog_version</font> Edit Account</h2>\n";

# create the form that lets them edit the selected accounts information.
print "<form method=post action=\"$site_dir/cgi-bin/modify_account.cgi\">\n";
if (length $ledger_db > 0)
{
  print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
}
print <<"END_OF_FORM";
<table border=0 cellspacing=0 cellpadding=2 bgcolor=\"$menu_bgcolor\">
<tr>
  <td><font color=\"$menu_fgcolor\">Account Name</font></td>
  <td><input type=text name=\"acct_name\" value=\"$acct_name\"><input type=hidden name=\"old_acct_name\" value=\"$acct_name\"></td>
</tr>
<tr>
  <td><font color=\"$menu_fgcolor\">Account Number</font></td>
  <td>$acct_num <input type=hidden name=\"acct_num\" value=\"$acct_num\"></td>
</tr>
<tr>
  <td><font color=\"$menu_fgcolor\">Account Type</font></td>
END_OF_FORM

# now do a database lookup and printout the account type name
$result = $conn->exec("select name, num, first_digit from acct_type_tb where num = '$acct_type'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get all account types.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  print "<h1>Internal Error:  No account types defined!</h1>\n";
  print "Tried to get all account types.<br>$sys_admin";
  CleanUp();
}
my $name;
$name = $result->getvalue(0, 0);
my $num;
$num  = $result->getvalue(0, 1);
my $first;
$first = $result->getvalue(0, 2);
print "<td>$name <input type=hidden name=\"acct_type\" value=\"$acct_type\"></td>\n";

print <<"END_OF_FORM";
  </select></td>
</tr>
END_OF_FORM

if ($first eq "4" || $first eq "5")
{
  print <<"END_OF_FORM";
  <tr>
    <td><font color="$menu_fgcolor">Account is Taxable</font></td>
END_OF_FORM

  if ($acct_taxable eq "t")
  {
    print "<td><input type=checkbox name=\"acct_taxable\" value=\"YES\" checked></td>\n";
  }
  else
  {
    print "<td><input type=checkbox name=\"acct_taxable\" value=\"YES\"></td>\n";
  }

  print <<"END_OF_FORM";
  </tr>
END_OF_FORM
}

print <<"END_OF_FORM";
<tr>
  <td><font color=\"$menu_fgcolor\">Account is Active</font></td>
END_OF_FORM

if ($acct_active eq "t")
{
  print "<td><input type=checkbox name=\"acct_active\" value=\"YES\" checked></td>\n";
}
else
{
  print "<td><input type=checkbox name=\"acct_active\" value=\"YES\"></td>\n";
}

print <<"END_OF_FORM";
</tr>
</table>
<input type=submit name=\"update_account\" value=\"Modify Account\">&nbsp;
<input type=reset value=\"Reset\">
</form>
END_OF_FORM

# print a horizontal rule to break the menu from the behind the scenes links
print "<br><hr>\n";

if (length $ledger_db == 0)
{
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/edit_account.cgi\">Edit Account</a></div>\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/account_config.cgi\">Account Configuration</a></div>\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/prg_config.cgi\">Program Configuration</a></div>\n";
}
else
{
  print "<form name=\"form1\" method=post action=\"\">\n";
  print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/edit_account.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/edit_account.cgi'); return false;\">Edit Account</a></div>\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/account_config.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/account_config.cgi'); return false;\">Account Configuration</a></div>\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/prg_config.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/prg_config.cgi'); return false;\">Program Configuration</a></div>\n";
  print "<input type=hidden name=\"open\" value=\"Open\">\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/ledger.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/ledger.cgi'); return false;\">Return to Ledger Summary</a></div>\n";
  print "</form>\n";
}

print "</body>\n";
print "</html>\n";
exit 0;
