#!/usr/bin/perl
#$Id: federal_deposit_entry.cgi,v 1.2 2001/10/24 19:29:46 moreejt Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

# federal_deposit_entry.cgi  -  Created by Jason A. Pattie, 04/03/2000
# This script allows the user to specify the first part of the federal deposit transaction.
# When the user submits this portion, the information is submitted to the journal_activity_rows_sheet.cgi
# script as a refresh, passing the parameters as records in the format supported by the transaction
# row script.

use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features
use reports;

use vars '$date';
use vars '$date_ago';
use vars '$count';
use vars '$count2';
use vars '$journal_code';
use vars '@journal_code_array';
use vars '$trans_date';
use vars '$trans_month';
use vars '$trans_day';
use vars '$trans_day1';
use vars '$trans_day2';
use vars '$trans_year';
use vars '$payee_payer';
use vars '@payee_payer_array';
use vars '@payee_payer_ref_num_array';
use vars '@payee_payer_journal_code_array';
use vars '$memo';
use vars '$remember';
use vars '$sub_entity';
use vars '$current_fiscal_year';
use vars '$start_fiscal';
use vars '$stop_fiscal';
use vars '$start_month_fiscal';
use vars '$stop_month_fiscal';
use vars '$start_day_fiscal';
use vars '$stop_day_fiscal';
use vars '$start_year_fiscal';
use vars '$stop_year_fiscal';
use vars '$start_day1';
use vars '$start_day2';
use vars '$stop_day1';
use vars '$stop_day2';
use vars '$temp_payee_payer';
use vars '$start_month_date';
use vars '$end_month_date';

# account numbers and sub-accounts for employee wage information
use vars '$salary_expense_accounts';
use vars '@salary_expense_array';
use vars '$fitw_pay_accounts';
use vars '%fitw_pay_array';
use vars '$fica_pay_accounts';
use vars '%fica_pay_array';
use vars '$medicare_pay_accounts';
use vars '%medicare_pay_array';
use vars '%account_types';
use vars '$account_num';

use vars '$salary_sum';
use vars '$temp_salary';
use vars '$temp_debit';
use vars '$temp_credit';
use vars '$fitw_sum';
use vars '$temp_fitw';
use vars '$fica_sum';
use vars '$temp_fica';
use vars '$medicare_sum';
use vars '$temp_medicare';
use vars '$total_deposit_amount';

use vars '%employee_info';
use vars '$employee_id';
use vars '$employer_id';
use vars '$employer_name';
use vars '$employer_fica_sum';
use vars '$employer_medicare_sum';

use vars '$quarter_code';
use vars '$quarter_name';
use vars '$month_name';

# records to pass through to the journal_activity_rows_sheet script for automatic entry
use vars '$debit_values';
use vars '$credit_values';

use vars '$debit_amount';
use vars '$credit_amount';

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

my $ledger_name;  # name of ledger to work with
my $ledger_db;
my $ledger_color;
my $journal_name;
my $result2;
my $ledger_code;
my $report_obj;

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

$ledger_db = $input{'ledger'};
if (length $ledger_db == 0)
{
  PrintErrorHeader();
  print "<h1>Error: You must specify a ledger to work with!</h1>\n";
  print "$back_button";
  CleanUp();
}

$journal_code = "PAY";  # payroll journal if it exists

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# now validate that the ledger they specified is valid

$result = $conn->exec("select name from entity_tb where database_name = '$ledger_db'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Trying to check entity table.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: Ledger <font color=\"blue\">$ledger_db</font> does not exist!</h1>\n";
  print "$sys_admin";
  CleanUp();
}
else
{
  $ledger_name = $result->getvalue(0, 0);
}

# create the reports object
$report_obj = reports->new($conn, $ledger_db);
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_code = $report_obj->getLedgerCode();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_color = $report_obj->getLedgerColor();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}

# now make sure that the journal exists that we are supposed to work with
$result = $conn->exec("select name from journal_tb where code = '$journal_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database <font color=\"blue\">$ledger_db</font> failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to validate Journal with code = <font color=\"blue\">$journal_code</font>.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  # payroll journal "PAY" not found; try payroll journal "P"
  $journal_code = "P";
  $result = $conn->exec("select name from journal_tb where code = '$journal_code'");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    PrintErrorHeader();
    print "<h1>Error:  Accessing database <font color=\"blue\">$ledger_db</font> failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to validate Journal with code = <font color=\"blue\">$journal_code</font>.<br>$sys_admin";
    CleanUp();
  }
  if ($result->ntuples == 0)
  {
    # payroll journal "P" not found; try general journal
    $journal_code = "G";
    $result = $conn->exec("select name from journal_tb where code = '$journal_code'");
    if ($result->resultStatus != PGRES_TUPLES_OK)
    {
      PrintErrorHeader();
      print "<h1>Error:  Accessing database <font color=\"blue\">$ledger_db</font> failed!</h1>\n";
      print $conn->errorMessage . "<br>Tried to validate Journal with code = <font color=\"blue\">$journal_code</font>.<br>$sys_admin";
      CleanUp();
    }
    if ($result->ntuples == 0)
    {
      PrintErrorHeader();
      print "<h1>Error:  Journal with code = <font color=\"blue\">$journal_code</font> does not exist!</h1>\n";
      print "$sys_admin";
      CleanUp();
    }
    $journal_name = $result->getvalue(0, 0);
  }
  else
  {
    $journal_name = $result->getvalue(0, 0);
  }
}
else
{
  $journal_name = $result->getvalue(0, 0);
}

# get current fiscal year and verify that it is not closed
$result = $conn->exec("select working_year from variables_tb WHERE ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  print "<h1>Error: Accessing database <font color=\"blue\"$ledger_db</font> failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get current fiscal year from database.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples != 1)
{
  print "<h1>Error: Wrong number of records returned!</h1>\n";
  print "There should only be one record in the variables_tb.  " . $result->ntuples . " were returned.<br>$sys_admin";
  CleanUp();
}
$current_fiscal_year = $result->getvalue(0, 0);
# get the date ranges for the current fiscal year if it is not closed
$result = $conn->exec("select closed, start_date, end_date from known_years_tb where year = $current_fiscal_year AND ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  print "<h1>Error: Accessing database <font color=\"blue\"$ledger_db</font> failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get current fiscal year info. from database.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples != 1)
{
  print "<h1>Error: Wrong number of records returned!</h1>\n";
  print "There should only be one record for each individual fiscal year.<br>$sys_admin";
  CleanUp();
}
if ($result->getvalue(0, 0) eq "t")
{
  print "<h1>Fiscal Year ($current_fiscal_year) is Closed!</h1>\n";
  print "You cannot work with the records in a closed fiscal year.<br>$sys_admin";
  CleanUp();
}
$start_fiscal = $result->getvalue(0, 1);
$stop_fiscal = $result->getvalue(0, 2);
($start_month_fiscal, $start_day_fiscal, $start_year_fiscal) = split "-", $start_fiscal;
($stop_month_fiscal, $stop_day_fiscal, $stop_year_fiscal) = split "-", $stop_fiscal;

# get the sub-entity selected from the environment
if (not exists $input{'sub_entity_select'})
{
  $sub_entity = "";
}
else
{
  $sub_entity = "$input{'sub_entity_select'}";
}

# determine the quarter information to use to enter the transaction
if ($input{'month_select'} < 4)
{
  $quarter_code = 1;
}
elsif ($input{'month_select'} < 7)
{
  $quarter_code = 2;
}
elsif ($input{'month_select'} < 10)
{
  $quarter_code = 3;
}
elsif ($input{'month_select'} < 13)
{
  $quarter_code = 4;
}
# get the sub-account name associated with the quarter code
$result = $conn->exec("select sub_acc_name from sub_acc_tb where sub_acc_number = $quarter_code");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing the database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get the name for quarter: $quarter_code.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples <= 0)
{
  PrintErrorHeader();
  print "<h1>Error: No records returned!</h1>\n";
  print "Tried to get the name for quarter: $quarter_code.<br>$sys_admin";
  CleanUp();
}
$quarter_name = $result->getvalue(0, 0);

# get the month name
$month_name = `date -d "$input{'month_select'}/01/2000" +%B`;
chomp $month_name;

# get the salary expense, FICA, Medicare, and FITW payable account numbers, looking them up by account type
# Salary Expenses account type = 55
$result = $conn->exec("select number from account_tb where number in (select acct_num from local_accounts_tb where ledger_code = '$ledger_code') and type = 55");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing the database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get the list of salary expense accounts assigned to this ledger: $ledger_name.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: No records returned!</h1>\n";
  print "Apparently there are no salary expense accounts available for this ledger: $ledger_name.<br>$sys_admin";
  CleanUp();
}
$salary_expense_accounts = "";
for ($count=0; $count < $result->ntuples; $count++)
{
  $salary_expense_accounts .= $result->getvalue($count, 0);
  $salary_expense_array[$count] = $result->getvalue($count, 0);
  $account_types{$result->getvalue($count, 0)} = "";
  if ($count < $result->ntuples - 1)
  {
    $salary_expense_accounts .= ", ";
  }
}
# FITW Liability account type = 24
$result = $conn->exec("select number from account_tb where number in (select acct_num from local_accounts_tb where ledger_code = '$ledger_code') and type = 24");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing the database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get the list of FITW payable accounts assigned to this ledger: $ledger_name.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: No records returned!</h1>\n";
  print "Apparently there are no FITW Liability accounts available for this ledger: $ledger_name.<br>$sys_admin";
  CleanUp();
}
$fitw_pay_accounts = "";
for ($count=0; $count < $result->ntuples; $count++)
{
  $fitw_pay_accounts .= $result->getvalue($count, 0);
  $fitw_pay_array{$result->getvalue($count, 0)} = 0;
  $account_types{$result->getvalue($count, 0)} = "";
  if ($count < $result->ntuples - 1)
  {
    $fitw_pay_accounts .= ", ";
  }
}
# FICA Liability account type = 26
$result = $conn->exec("select number from account_tb where number in (select acct_num from local_accounts_tb where ledger_code = '$ledger_code') and type = 26");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing the database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get the list of FICA payable accounts assigned to this ledger: $ledger_name.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: No records returned!</h1>\n";
  print "Apparently there are no FICA Liability accounts available for this ledger: $ledger_name.<br>$sys_admin";
  CleanUp();
}
$fica_pay_accounts = "";
for ($count=0; $count < $result->ntuples; $count++)
{
  $fica_pay_accounts .= $result->getvalue($count, 0);
  $fica_pay_array{$result->getvalue($count, 0)} = 0;
  $account_types{$result->getvalue($count, 0)} = "";
  if ($count < $result->ntuples - 1)
  {
    $fica_pay_accounts .= ", ";
  }
}
# Medicare Liability account type = 27
$result = $conn->exec("select number from account_tb where number in (select acct_num from local_accounts_tb where ledger_code = '$ledger_code') and type = 27");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing the database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get the list of Medicare payable accounts assigned to this ledger: $ledger_name.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: No records returned!</h1>\n";
  print "Apparently there are no Medicare Liability accounts available for this ledger: $ledger_name.<br>$sys_admin";
  CleanUp();
}
$medicare_pay_accounts = "";
for ($count=0; $count < $result->ntuples; $count++)
{
  $medicare_pay_accounts .= $result->getvalue($count, 0);
  $medicare_pay_array{$result->getvalue($count, 0)} = 0;
  $account_types{$result->getvalue($count, 0)} = "";
  if ($count < $result->ntuples - 1)
  {
    $medicare_pay_accounts .= ", ";
  }
}

# get the debit_increases flag for each of the accounts
foreach $account_num (keys %account_types)
{
  $result = $conn->exec("select debit_increases from acct_type_tb where num = (select type from account_tb where number = $account_num)");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    PrintErrorHeader();
    print "<h1>Error: Accessing the database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to get the debit increases flag for account: $account_num.<br>$sys_admin";
    CleanUp();
  }
  if ($result->ntuples == 0)
  {
    PrintErrorHeader();
    print "<h1>Error: No records returned!</h1>\n";
    print "Tried to get the debit increases flag for account: $account_num.<br>$sys_admin";
    CleanUp();
  }
  $account_types{$account_num} = $result->getvalue(0, 0);
}

# get info for active employees for this month that have wages entered in the salary expense account
$start_month_date = $input{'month_select'} . "-01-" . $input{'year_select'};
$end_month_date = $input{'month_select'} . "-";
if ($input{'month_select'} == 4 || $input{'month_select'} == 6 || $input{'month_select'} == 9 || $input{'month_select'} == 11)
{
  $end_month_date .= "30";
}
elsif ($input{'month_select'} != 2)
{
  $end_month_date .= "31";
}
else
{
  # check for leap year status
  if ($input{'year_select'} % 4 == 0 && $input{'year_select'} % 100 != 0 || $input{'year_select'} % 400 == 0)
  {
    $end_month_date .= "29";
  }
  else
  {
    $end_month_date .= "28";
  }
}
$end_month_date .= "-" . $input{'year_select'};
# get the sub-account numbers for the employees that have activity in the given month
$result = $conn->exec("select distinct sub_account_num from debitcredit_tb where ledger_code = '$ledger_code' and account_num in ($salary_expense_accounts) and sub_account_num not in (-1, 1, 2, 3, 4) and parent_num in (select ref_num from journal_entry_tb where ledger_code = '$ledger_code' and (trans_date between '$start_month_date' and '$end_month_date'))");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing the database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get the sub-account numbers to use for the employees that have wage activity for the month.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: No records returned!</h1>\n";
  print "No employees had wage information for the selected month.<br>$back_button";
  CleanUp();
}
for ($count=0; $count < $result->ntuples; $count++)
{
  $employee_id = $result->getvalue($count, 0);
  $employee_info{$employee_id} = [];
}
# run through the employee_info hash and obtain data for each employee
foreach $employee_id (keys %employee_info)
{
  # get the sub-account name for the sub-account number
  $result = $conn->exec("select sub_acc_name from sub_acc_tb where sub_acc_number = $employee_id");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    PrintErrorHeader();
    print "<h1>Error: Accessing the database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to get the sub-account name for: $employee_id.<br>$sys_admin";
    CleanUp();
  }
  $employee_info{$employee_id}[0] = $result->getvalue(0, 0);
  # get the wage amount for this employee
  $temp_salary = 0.0;
  for ($count=0; $count < @salary_expense_array; $count++)
  {
    # debits
    $result = $conn->exec("select sum(amount) from debitcredit_tb where ledger_code = '$ledger_code' and account_num = $salary_expense_array[$count] and sub_account_num = $employee_id and dorc = true and parent_num in (select ref_num from journal_entry_tb where ledger_code = '$ledger_code' and (trans_date between '$start_month_date' and '$end_month_date'))");
    if ($result->resultStatus != PGRES_TUPLES_OK)
    {
      PrintErrorHeader();
      print "<h1>Error: Accessing the database failed!</h1>\n";
      print $conn->errorMessage . "<br>Tried to get the debits from the salaray expense account: $salary_expense_array[$count] for: $employee_info{$employee_id}[0] - $employee_id.<br>$sys_admin";
      CleanUp();
    }
    $temp_debit = $result->getvalue(0, 0);
    # credits
    $result = $conn->exec("select sum(amount) from debitcredit_tb where ledger_code = '$ledger_code' and account_num = $salary_expense_array[$count] and sub_account_num = $employee_id and dorc = false and parent_num in (select ref_num from journal_entry_tb where ledger_code = '$ledger_code' and (trans_date between '$start_month_date' and '$end_month_date'))");
    if ($result->resultStatus != PGRES_TUPLES_OK)
    {
      PrintErrorHeader();
      print "<h1>Error: Accessing the database failed!</h1>\n";
      print $conn->errorMessage . "<br>Tried to get the credits from the salaray expense account: $salary_expense_array[$count] for: $employee_info{$employee_id}[0] - $employee_id.<br>$sys_admin";
      CleanUp();
    }
    $temp_credit = $result->getvalue(0, 0);
    # add to the total salary for this employee
    if ($account_types{$salary_expense_array[$count]} eq "t")
    {
      $temp_salary += $temp_debit - $temp_credit;
    }
    else
    {
      $temp_salary += $temp_credit - $temp_debit;
    }
  }
  $employee_info{$employee_id}[1] = $temp_salary;
  # get the FITW payable amounts for this employee
  $temp_fitw = 0.0;
  foreach $count (keys %fitw_pay_array)
  {
    # debits
    $result = $conn->exec("select sum(amount) from debitcredit_tb where ledger_code = '$ledger_code' and account_num = $count and sub_account_num = $employee_id and dorc = true and parent_num in (select ref_num from journal_entry_tb where ledger_code = '$ledger_code' and (trans_date between '$start_month_date' and '$end_month_date'))");
    if ($result->resultStatus != PGRES_TUPLES_OK)
    {
      PrintErrorHeader();
      print "<h1>Error: Accessing the database failed!</h1>\n";
      print $conn->errorMessage . "<br>Tried to get the debits from the fitw pay account: $count for: $employee_info{$employee_id}[0] - $employee_id.<br>$sys_admin";
      CleanUp();
    }
    $temp_debit = $result->getvalue(0, 0);
    # credits
    $result = $conn->exec("select sum(amount) from debitcredit_tb where ledger_code = '$ledger_code' and account_num = $count and sub_account_num = $employee_id and dorc = false and parent_num in (select ref_num from journal_entry_tb where ledger_code = '$ledger_code' and (trans_date between '$start_month_date' and '$end_month_date'))");
    if ($result->resultStatus != PGRES_TUPLES_OK)
    {
      PrintErrorHeader();
      print "<h1>Error: Accessing the database failed!</h1>\n";
      print $conn->errorMessage . "<br>Tried to get the credits from the fitw pay account: $count for: $employee_info{$employee_id}[0] - $employee_id.<br>$sys_admin";
      CleanUp();
    }
    $temp_credit = $result->getvalue(0, 0);
    # add to the total fitw pay for this employee
    if ($account_types{$count} eq "t")
    {
      $temp_fitw += $temp_debit - $temp_credit;
    }
    else
    {
      $temp_fitw += $temp_credit - $temp_debit;
    }
    # assign the total fitw to the account it came from
    $fitw_pay_array{$count} += $temp_fitw;
  }
  $employee_info{$employee_id}[2] = $temp_fitw;

  # get the FICA payable amounts for this employee
  $temp_fica = 0.0;
  foreach $count (keys %fica_pay_array)
  {
    # debits
    $result = $conn->exec("select sum(amount) from debitcredit_tb where ledger_code = '$ledger_code' and account_num = $count and sub_account_num = $employee_id and dorc = true and parent_num in (select ref_num from journal_entry_tb where ledger_code = '$ledger_code' and (trans_date between '$start_month_date' and '$end_month_date'))");
    if ($result->resultStatus != PGRES_TUPLES_OK)
    {
      PrintErrorHeader();
      print "<h1>Error: Accessing the database failed!</h1>\n";
      print $conn->errorMessage . "<br>Tried to get the debits from the fica pay account: $count for: $employee_info{$employee_id}[0] - $employee_id.<br>$sys_admin";
      CleanUp();
    }
    $temp_debit = $result->getvalue(0, 0);
    # credits
    $result = $conn->exec("select sum(amount) from debitcredit_tb where ledger_code = '$ledger_code' and account_num = $count and sub_account_num = $employee_id and dorc = false and parent_num in (select ref_num from journal_entry_tb where ledger_code = '$ledger_code' and (trans_date between '$start_month_date' and '$end_month_date'))");
    if ($result->resultStatus != PGRES_TUPLES_OK)
    {
      PrintErrorHeader();
      print "<h1>Error: Accessing the database failed!</h1>\n";
      print $conn->errorMessage . "<br>Tried to get the credits from the fica pay account: $count for: $employee_info{$employee_id}[0] - $employee_id.<br>$sys_admin";
      CleanUp();
    }
    $temp_credit = $result->getvalue(0, 0);
    # add to the total fica pay for this employee
    if ($account_types{$count} eq "t")
    {
      $temp_fica += $temp_debit - $temp_credit;
    }
    else
    {
      $temp_fica += $temp_credit - $temp_debit;
    }
    # assign the total fica to the account it came from
    $fica_pay_array{$count} += $temp_fica;
  }
  $employee_info{$employee_id}[3] = $temp_fica;

  # get the Medicare payable amounts for this employee
  $temp_medicare = 0.0;
  foreach $count (keys %medicare_pay_array)
  {
    # debits
    $result = $conn->exec("select sum(amount) from debitcredit_tb where ledger_code = '$ledger_code' and account_num = $count and sub_account_num = $employee_id and dorc = true and parent_num in (select ref_num from journal_entry_tb where ledger_code = '$ledger_code' and (trans_date between '$start_month_date' and '$end_month_date'))");
    if ($result->resultStatus != PGRES_TUPLES_OK)
    {
      PrintErrorHeader();
      print "<h1>Error: Accessing the database failed!</h1>\n";
      print $conn->errorMessage . "<br>Tried to get the debits from the medicare pay account: $count for: $employee_info{$employee_id}[0] - $employee_id.<br>$sys_admin";
      CleanUp();
    }
    $temp_debit = $result->getvalue(0, 0);
    # credits
    $result = $conn->exec("select sum(amount) from debitcredit_tb where ledger_code = '$ledger_code' and account_num = $count and sub_account_num = $employee_id and dorc = false and parent_num in (select ref_num from journal_entry_tb where ledger_code = '$ledger_code' and (trans_date between '$start_month_date' and '$end_month_date'))");
    if ($result->resultStatus != PGRES_TUPLES_OK)
    {
      PrintErrorHeader();
      print "<h1>Error: Accessing the database failed!</h1>\n";
      print $conn->errorMessage . "<br>Tried to get the credits from the medicare pay account: $count for: $employee_info{$employee_id}[0] - $employee_id.<br>$sys_admin";
      CleanUp();
    }
    $temp_credit = $result->getvalue(0, 0);
    # add to the total medicare pay for this employee
    if ($account_types{$count} eq "t")
    {
      $temp_medicare += $temp_debit - $temp_credit;
    }
    else
    {
      $temp_medicare += $temp_credit - $temp_debit;
    }
    # assign the total medicare to the account it came from
    $medicare_pay_array{$count} += $temp_medicare;
  }
  $employee_info{$employee_id}[4] = $temp_medicare;

  # keep running totals for gross pay, FITW, FICA, and Medicare
  $salary_sum += $temp_salary;
  $fitw_sum += $temp_fitw;
  $fica_sum += $temp_fica;
  $medicare_sum += $temp_medicare;

  # keep running total for deposit amount
  $total_deposit_amount += $temp_fitw + $temp_fica + $temp_medicare;
}

# get the matching values the employer entered for FICA and Medicare taxes
$employer_id = 10;  # employer sub-account number; hardcoded value
# lookup name information for the employer sub-account
$result = $conn->exec("select sub_acc_name from sub_acc_tb where sub_acc_number = $employer_id");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  $employer_name = "Default Employer: " . $conn->errorMessage;
}
else
{
  if ($result->ntuples > 0)
  {
    $employer_name = $result->getvalue(0, 0);
  }
  else
  {
    $employer_name = "Default Employer";
  }
}
$temp_fica = 0.0;
foreach $count (keys %fica_pay_array)
{
  # debits
  $result = $conn->exec("select sum(amount) from debitcredit_tb where ledger_code = '$ledger_code' and account_num = $count and sub_account_num = $employer_id and dorc = true and parent_num in (select ref_num from journal_entry_tb where ledger_code = '$ledger_code' and (trans_date between '$start_month_date' and '$end_month_date'))");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    PrintErrorHeader();
    print "<h1>Error: Accessing the database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to get the debits from the fica pay account: $count for Employer.<br>$sys_admin";
    CleanUp();
  }
  $temp_debit = $result->getvalue(0, 0);
  # credits
  $result = $conn->exec("select sum(amount) from debitcredit_tb where ledger_code = '$ledger_code' and account_num = $count and sub_account_num = $employer_id and dorc = false and parent_num in (select ref_num from journal_entry_tb where ledger_code = '$ledger_code' and (trans_date between '$start_month_date' and '$end_month_date'))");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    PrintErrorHeader();
    print "<h1>Error: Accessing the database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to get the credits from the fica pay account: $count for Employer.<br>$sys_admin";
    CleanUp();
  }
  $temp_credit = $result->getvalue(0, 0);
  # add to the total fica pay for this employee
  if ($account_types{$count} eq "t")
  {
    $temp_fica += $temp_debit - $temp_credit;
  }
  else
  {
    $temp_fica += $temp_credit - $temp_debit;
  }
  # add the total fica to the account it came from
  $fica_pay_array{$count} += $temp_fica;
}
$employer_fica_sum = $temp_fica;
$temp_medicare = 0.0;
foreach $count (keys %medicare_pay_array)
{
  # debits
  $result = $conn->exec("select sum(amount) from debitcredit_tb where ledger_code = '$ledger_code' and account_num = $count and sub_account_num = $employer_id and dorc = true and parent_num in (select ref_num from journal_entry_tb where ledger_code = '$ledger_code' and (trans_date between '$start_month_date' and '$end_month_date'))");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    PrintErrorHeader();
    print "<h1>Error: Accessing the database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to get the debits from the medicare pay account: $count for Employer.<br>$sys_admin";
    CleanUp();
  }
  $temp_debit = $result->getvalue(0, 0);
  # credits
  $result = $conn->exec("select sum(amount) from debitcredit_tb where ledger_code = '$ledger_code' and account_num = $count and sub_account_num = $employer_id and dorc = false and parent_num in (select ref_num from journal_entry_tb where ledger_code = '$ledger_code' and (trans_date between '$start_month_date' and '$end_month_date'))");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    PrintErrorHeader();
    print "<h1>Error: Accessing the database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to get the credits from the medicare pay account: $count for Employer.<br>$sys_admin";
    CleanUp();
  }
  $temp_credit = $result->getvalue(0, 0);
  # add to the total medicare pay for this employee
  if ($account_types{$count} eq "t")
  {
    $temp_medicare += $temp_debit - $temp_credit;
  }
  else
  {
    $temp_medicare += $temp_credit - $temp_debit;
  }
  # add the total medicare to the account it came from
  $medicare_pay_array{$count} += $temp_medicare;
}
$employer_medicare_sum = $temp_medicare;
$total_deposit_amount += $employer_fica_sum + $employer_medicare_sum;

# generate the strings for the hidden values that store the debit and credit information for the transaction rows script
foreach $count (keys %fitw_pay_array)
{
  if ($fitw_pay_array{$count} < 0)
  {
    $credit_values .= "se=$sub_entity|ac=$count|sa=$quarter_code|cd=$quarter_name - $quarter_code|am=" . sprintf("%.2f", abs($fitw_pay_array{$count})) . "|cl= ;";
  }
  else
  {
    $debit_values .= "se=$sub_entity|ac=$count|sa=$quarter_code|cd=$quarter_name - $quarter_code|am=" . sprintf("%.2f", abs($fitw_pay_array{$count})) . "|cl= ;";
  }
}
foreach $count (keys %fica_pay_array)
{
  if ($fica_pay_array{$count} < 0)
  {
    $credit_values .= "se=$sub_entity|ac=$count|sa=$quarter_code|cd=$quarter_name - $quarter_code|am=" . sprintf("%.2f", abs($fica_pay_array{$count})) . "|cl= ;";
  }
  else
  {
    $debit_values .= "se=$sub_entity|ac=$count|sa=$quarter_code|cd=$quarter_name - $quarter_code|am=" . sprintf("%.2f", abs($fica_pay_array{$count})) . "|cl= ;";
  }
}
foreach $count (keys %medicare_pay_array)
{
  if ($medicare_pay_array{$count} < 0)
  {
    $credit_values .= "se=$sub_entity|ac=$count|sa=$quarter_code|cd=$quarter_name - $quarter_code|am=" . sprintf("%.2f", abs($medicare_pay_array{$count})) . "|cl= ;";
  }
  else
  {
    $debit_values .= "se=$sub_entity|ac=$count|sa=$quarter_code|cd=$quarter_name - $quarter_code|am=" . sprintf("%.2f", abs($medicare_pay_array{$count})) . "|cl= ;";
  }
}

# calculate the difference to display in either the debit amount field or credit amount field
if ($total_deposit_amount < 0)
{
  $debit_amount = sprintf("%.2f", abs($total_deposit_amount));
}
else
{
  $credit_amount = sprintf("%.2f", abs($total_deposit_amount));
}

print "<head><title>$prog_name $prog_version Federal Deposit</title></head>\n";
print "<body bgcolor=$body_bgcolor onLoad=\"do_load();\">\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
<script language=\"Javascript1.2\">
  <!--
END_OF_STYLE
$start_day1 = int ($start_day_fiscal / 10);
$start_day2 = ($start_day_fiscal % 10);
$stop_day1 = int ($stop_day_fiscal / 10);
$stop_day2 = ($stop_day_fiscal % 10);
print <<"END_OF_STYLE";
  var trans_month;
  var trans_day1;
  var trans_day2;
  var trans_year;
  var start_month_fiscal = $start_month_fiscal;
  var start_day_fiscal = $start_day_fiscal;
  var start_day1_fiscal = $start_day1;
  var start_day2_fiscal = $start_day2;
  var start_year_fiscal = $start_year_fiscal;
  var stop_month_fiscal = $stop_month_fiscal;
  var stop_day_fiscal = $stop_day_fiscal;
  var stop_day1_fiscal = $stop_day1;
  var stop_day2_fiscal = $stop_day2;
  var stop_year_fiscal = $stop_year_fiscal;
  var start_day;
  var stop_day;
  var month_name;
  var month;
  var year;
  var old_trans_month;
  var old_trans_day1;
  var old_trans_day2;
  var old_trans_year;
  var date_valid;
  var journal_code_array = new Array();

  function do_load()
  {
    // fill in values for month, day1, day2, and year select form values

    // assign values to trans_month, trans_day, trans_year variables
    trans_month = document.edit_trans_form.trans_month.value;
    trans_day1 = document.edit_trans_form.trans_day1.value;
    trans_day2 = document.edit_trans_form.trans_day2.value;
    trans_year = document.edit_trans_form.trans_year.value;

    old_trans_month = trans_month;
    old_trans_day1 = trans_day1;
    old_trans_day2 = trans_day2;
    old_trans_year = trans_year;
    date_valid = true;

    // set the focus to the journal drop-down select
    document.edit_trans_form.journal_code_select.focus();
  }

  function payee_payer_text_change()
  {
    var count;
    var regexp;
    var payee_payer;
    var reg_exp;
    var non_space_found = false;

    regexp = eval("/^" + document.edit_trans_form.payee_payer.value + "+/i");

    // run through the payee_payer_select and see if the typed value matches any options in the select
    for (count=0; count < document.edit_trans_form.payee_payer_select.options.length; count++)
    {
      if (document.edit_trans_form.payee_payer_select.options[count].text.search(regexp) != -1)
      {
        document.edit_trans_form.payee_payer_select.selectedIndex = count;
        document.edit_trans_form.payee_payer.value = document.edit_trans_form.payee_payer_select.options[count].text;
        // select the last used journal code for the payee-payer from the journal code select
        document.edit_trans_form.journal_code_select.selectedIndex = journal_code_array[document.edit_trans_form.payee_payer_select.options[count].value];
      }
    }
    // if what is currently typed in the text field does not match what is selected in the select combo and what is typed starts with
    // "R> ", then throw an alert and clear the text field
    if (document.edit_trans_form.payee_payer_select.options[document.edit_trans_form.payee_payer_select.selectedIndex].text != document.edit_trans_form.payee_payer.value)
    {
      if ((document.edit_trans_form.payee_payer.value[0] == "R" || document.edit_trans_form.payee_payer.value[0] == "r") && document.edit_trans_form.payee_payer.value[1] == ">" && document.edit_trans_form.payee_payer.value[2] == " ")
      {
        alert("No match for remembered payee-payer found!");
        document.edit_trans_form.payee_payer.value = "";
      }
    }
    // if blank is typed in, uncheck the remembered checkbox
    // make sure the payee-payer is not empty
    payee_payer = "";
    payee_payer = document.edit_trans_form.payee_payer.value;
    for (var count=0; count < payee_payer.length && non_space_found == false; count++)
    {
      if (payee_payer[count] != " ")
      {
        non_space_found = true;
      }
    }
    if (!non_space_found)
    {
      // clear the payee-payer text field
      document.edit_trans_form.payee_payer.value = "";
      document.edit_trans_form.remember.checked = false;
      // select the first (blank) payee-payer
      document.edit_trans_form.payee_payer_select.selectedIndex = 0;
    }
  }

  function payee_payer_select_change()
  {
    // set the payee_payer text field to the string value of the option selected
    if (document.edit_trans_form.payee_payer_select.selectedIndex != 0)
    {
      document.edit_trans_form.payee_payer.value = document.edit_trans_form.payee_payer_select.options[document.edit_trans_form.payee_payer_select.selectedIndex].text;
      // select the last used journal code for the payee-payer from the journal code select
      document.edit_trans_form.journal_code_select.selectedIndex = journal_code_array[document.edit_trans_form.payee_payer_select.options[document.edit_trans_form.payee_payer_select.selectedIndex].value];
    }
    else
    {
      document.edit_trans_form.payee_payer.value = "";
    }
    // if blank is selected, uncheck the remembered checkbox
    if (document.edit_trans_form.payee_payer_select.options[document.edit_trans_form.payee_payer_select.selectedIndex].text == "" || document.edit_trans_form.payee_payer_select.options[document.edit_trans_form.payee_payer_select.selectedIndex].value == -1)
    {
      document.edit_trans_form.remember.checked = false;
    }
  }

  function trans_month_text_change()
  {
    var date_array = new Array();
    var value;

    // split the date on "-"'s
    date_array = document.edit_trans_form.trans_date.value.split("-", 3);

    // make sure the value entered is a number
    value = document.edit_trans_form.trans_month_text.value;
    if (isNaN(parseInt(value, 10)))
    {
      if (value != "")
      {
        alert("Month value entered is not a number!\\nValue: " + value);
        document.edit_trans_form.trans_month_text.value = "";
        document.edit_trans_form.trans_month_text.focus();
      }
    }
    else
    {
      value = parseInt(value, 10);
      // make sure the month is between 1 and 12
      if (value < 1 || value > 12)
      {
        alert("Month value entered is out of range (1 - 12)!");
        document.edit_trans_form.trans_month_text.value = "";
        document.edit_trans_form.trans_month_text.focus();
      }
      else
      {
        // change the date_array element to contain the entered value
        date_array[0] = value;
        // clear the input field
        document.edit_trans_form.trans_month_text.value = "";
        // output the modified date
        document.edit_trans_form.trans_date.value = date_array[0] + "-" + date_array[1] + "-" + date_array[2];
        // validate the entered value
        trans_date_change();
      }
    }
  }

  function trans_day_text_change()
  {
    var date_array = new Array();
    var value;

    // split the date on "-"'s
    date_array = document.edit_trans_form.trans_date.value.split("-", 3);

    // make sure the value entered is a number
    value = document.edit_trans_form.trans_day_text.value;
    if (isNaN(parseInt(value, 10)))
    {
      if (value != "")
      {
        alert("Day value entered is not a number!\\nValue: " + value);
        document.edit_trans_form.trans_day_text.value = "";
        document.edit_trans_form.trans_day_text.focus();
      }
    }
    else
    {
      value = parseInt(value, 10);
      // make sure the day is between 1 and 31
      if (value < 1 || value > 31)
      {
        alert("Day value entered is out of range (1 - 31)!");
        document.edit_trans_form.trans_day_text.value = "";
        document.edit_trans_form.trans_day_text.focus();
      }
      else
      {
        // change the date_array element to contain the entered value
        date_array[1] = value;
        // clear the input field
        document.edit_trans_form.trans_day_text.value = "";
        // output the modified date
        document.edit_trans_form.trans_date.value = date_array[0] + "-" + date_array[1] + "-" + date_array[2];
        // validate the entered value
        trans_date_change();
      }
    }
  }

  function trans_year_text_change()
  {
    var date_array = new Array();
    var value;

    // split the date on "-"'s
    date_array = document.edit_trans_form.trans_date.value.split("-", 3);

    // make sure the value entered is a number
    value = document.edit_trans_form.trans_year_text.value;
    if (isNaN(parseInt(value, 10)))
    {
      if (value != "")
      {
        alert("Year value entered is not a number!\\nValue: " + value);
        document.edit_trans_form.trans_year_text.value = "";
        document.edit_trans_form.trans_year_text.focus();
      }
    }
    else
    {
      value = parseInt(value, 10);
      // make sure the year is greater than or equal to the beginning fiscal year value
      if (value < start_year_fiscal)
      {
        alert("Year value entered is less than the beginning fiscal year value (" + start_year_fiscal + ")!");
        document.edit_trans_form.trans_year_text.value = "";
        document.edit_trans_form.trans_year_text.focus();
      }
      else
      {
        // change the date_array element to contain the entered value
        date_array[2] = value;
        // clear the input field
        document.edit_trans_form.trans_year_text.value = "";
        // output the modified date
        document.edit_trans_form.trans_date.value = date_array[0] + "-" + date_array[1] + "-" + date_array[2];
        // validate the entered value
        trans_date_change();
      }
    }
  }

  function trans_date_change()
  {
    // parse the inputted date for validity to see if it falls between the start and stop dates of the current fiscal range
    var date_to_parse = new Date();
    var month;
    var day;
    var day1;
    var day2;
    var year;
    var date_string;
    var regular_expression;
    var date_array = new Array();

    regular_expression = new RegExp("-", "g");
    // before parsing date text field, substitute all "-"'s with "/"'s
    date_string = document.edit_trans_form.trans_date.value;
    date_string = date_string.replace(regular_expression, "/");
    // determine if the date string has a month, day, and year by 'split'ting it on '/'s and seeing how many array elements are returned
    date_array = date_string.split("/");
    if (date_array.length == 2)
    {
      // year was not entered; attempt to determine which fiscal year to enter based on the month and day values
      month = date_array[0];
      day = date_array[1];
      if (month >= start_month_fiscal)
      {
        year = start_year_fiscal;
      }
      else if (month <= stop_month_fiscal)
      {
        year = stop_year_fiscal;
      }
      day1 = Math.floor(day/10);
      day2 = day % 10;
    }
    else
    {
      // make sure they entered a correct year; if not, mangle the year into one of the (possibly) two year values for the fiscal range
      if (parseInt(date_array[2], 10) < 100)  // user entered a two or less digit year
      {
        var year_count = start_year_fiscal;
        var year_check;
        var year_stop = parseInt(date_array[2], 10);
        // run from the start_fiscal_year up to the value entered by adding to the start year until the last two digits match the counter
        year_check = parseInt(year_count % 100, 10);
        while (year_check != year_stop)
        {
          year_count++;
          year_check = parseInt(year_count % 100, 10);
        }
        year = year_count;
        month = date_array[0];
        day = date_array[1];
        day1 = Math.floor(day/10);
        day2 = day % 10;
      }
      else
      {
        date_to_parse.setTime(Date.parse(date_string));
        month = parseInt(date_to_parse.getMonth()) + 1;
        day = parseInt(date_to_parse.getDate());
        day1 = Math.floor(day/10);
        day2 = day % 10;
        year = parseInt(date_to_parse.getFullYear());
      }
    }

    // make sure that year is in the fiscal range of years
    if (year != start_year_fiscal && year != stop_year_fiscal)
    {
      if (year > stop_year_fiscal)
      {
        if (!confirm("You are entering a date that is in the future\\\nbased on the current fiscal range.\\\nDo you want to enter this date?"))
        {
          month = old_trans_month;
          day1 = old_trans_day1;
          day2 = old_trans_day2;
          year = old_trans_year;
        }
        else
        {
          date_valid = true;
        }
      }
      else
      {
        date_valid = false;
        alert("Year entered is not in fiscal range!");
        return;
      }
    }
    // make sure that month entered is in fiscal range for the year input
    if (year == start_year_fiscal && month < start_month_fiscal)
    {
      date_valid = false;
      alert("Month entered is not in fiscal range!");
      return;
    }
    if (year == stop_year_fiscal && month > stop_month_fiscal)
    {
      if (!confirm("You are entering a date that is in the future\\\nbased on the current fiscal range.\\\nDo you want to enter this date?"))
      {
        month = old_trans_month;
        day1 = old_trans_day1;
        day2 = old_trans_day2;
        year = old_trans_year;
      }
      else
      {
        date_valid = true;
      }
    }
    // make sure that day entered is in fiscal range for the year and month input
    if (year == start_year_fiscal && month == start_month_fiscal && day < start_day_fiscal)
    {
      date_valid = false;
      alert("Day entered is not in fiscal range!");
      return;
    }
    if (year == stop_year_fiscal && month == stop_month_fiscal && day > stop_day_fiscal)
    {
      if (!confirm("You are entering a date that is in the future\\\nbased on the current fiscal range.\\\nDo you want to enter this date?"))
      {
        month = old_trans_month;
        day1 = old_trans_day1;
        day2 = old_trans_day2;
        year = old_trans_year;
      }
      else
      {
        date_valid = true;
      }
    }

    // redisplay the date from the parsed input into the trans_date text field
    document.edit_trans_form.trans_date.value = month + "-" + day1 + day2 + "-" + year;
    old_trans_month = month;
    old_trans_day1 = day1;
    old_trans_day2 = day2;
    old_trans_year = year;
  }

  function validate_form()
  {
    var date_to_parse = new Date();
    var month;
    var day;
    var day1;
    var day2;
    var year;
    var date_string;
    var regular_expression;
    var payee_payer;
    var non_space_found = false;

    // make sure the date is within the fiscal year range

    regular_expression = new RegExp("-", "g");
    // before parsing date text field, substitute all "-"'s with "/"'s
    date_string = document.edit_trans_form.trans_date.value;
    date_string = date_string.replace(regular_expression, "/");
    date_to_parse.setTime(Date.parse(date_string));
    month = parseInt(date_to_parse.getMonth()) + 1;
    day = parseInt(date_to_parse.getDate());
    day1 = Math.floor(day/10);
    day2 = day % 10;
    year = parseInt(date_to_parse.getFullYear());

    // make sure that year is in the fiscal range of years
    if (year != start_year_fiscal && year != stop_year_fiscal)
    {
      if (!date_valid)
      {
        alert("Year entered is not in fiscal range!");
        return false;
      }
    }
    // make sure that month entered is in fiscal range for the year input
    if (year == start_year_fiscal && month < start_month_fiscal)
    {
      alert("Month entered is not in fiscal range!");
      return false;
    }
    if (year == stop_year_fiscal && month > stop_month_fiscal)
    {
      if (!date_valid)
      {
        alert("Month entered is not in fiscal range!");
        return false;
      }
    }
    // make sure that day entered is in fiscal range for the year and month input
    if (year == start_year_fiscal && month == start_month_fiscal && day < start_day_fiscal)
    {
      alert("Day entered is not in fiscal range!");
      return false;
    }
    if (year == stop_year_fiscal && month == stop_month_fiscal && day > stop_day_fiscal)
    {
      if (!date_valid)
      {
        alert("Day entered is not in fiscal range!");
        return false;
      }
    }

    // make sure the payee-payer is not empty
    payee_payer = document.edit_trans_form.payee_payer.value;
    for (var count=0; count < payee_payer.length && non_space_found == false; count++)
    {
      if (payee_payer[count] != " ")
      {
        non_space_found = true;
      }
    }
    if (!non_space_found)
    {
      // clear the payee-payer text field
      document.edit_trans_form.payee_payer.value = "";
      document.edit_trans_form.remember.checked = false;
    }
    return true;
  }

  //-->
</script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;
$date_ago = `/bin/date -d '1 year 6 months ago' +%m-%d-%Y`;
chomp $date_ago;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td align=left width=47%><font color=\"red\">";
print "Working with Federal Deposit in Journal <font color=\"skyblue\">$journal_name</font> in Ledger <font color=\"$ledger_color\">$ledger_name</font></font></td>\n";
print "<td align=left><a href=\"$site_dir/help.html#journal\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardLedgerMainMenu($ledger_db);

# create the new journal entry
# use today's date as the transaction date
$trans_date = `/bin/date +%m-%d-%Y`;
chomp $trans_date;
($trans_month, $trans_day, $trans_year) = split "-", $trans_date;
$trans_day1 = int ($trans_day / 10);
$trans_day2 = int ($trans_day % 10);
$payee_payer = "";
$memo = "Federal Deposit";
$remember = "f";

if ($trans_year < $start_year_fiscal || $trans_year > $stop_year_fiscal)
{
  $trans_month = $start_month_fiscal;
  $trans_day = $start_day_fiscal;
  $trans_day1 = int ($start_day_fiscal / 10);
  $trans_day2 = int ($start_day_fiscal % 10);
  $trans_year = $start_year_fiscal;
  $trans_date = $trans_month . "-" . $trans_day . "-" . $trans_year;
}
elsif (($trans_month < $start_month_fiscal && $trans_year == $start_year_fiscal) || ($trans_month > $stop_month_fiscal && $trans_year == $stop_year_fiscal))
{
  $trans_month = $start_month_fiscal;
  $trans_day = $start_day_fiscal;
  $trans_day1 = int ($start_day_fiscal / 10);
  $trans_day2 = int ($start_day_fiscal % 10);
  $trans_year = $start_year_fiscal;
  $trans_date = $trans_month . "-" . $trans_day . "-" . $trans_year;
}
elsif (($trans_day < $start_day_fiscal && $trans_month == $start_month_fiscal && $trans_year == $start_year_fiscal) || ($trans_day > $stop_day_fiscal && $trans_month == $stop_month_fiscal && $trans_year == $stop_year_fiscal))
{
  $trans_month = $start_month_fiscal;
  $trans_day = $start_day_fiscal;
  $trans_day1 = int ($start_day_fiscal / 10);
  $trans_day2 = int ($start_day_fiscal % 10);
  $trans_year = $start_year_fiscal;
  $trans_date = $trans_month . "-" . $trans_day . "-" . $trans_year;
}

# get the journal codes from the ledger to fill in the journal_code_array
$result = $conn->exec("select code from journal_tb");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  print "<h1>Error: Accessing database <font color=\"blue\">$ledger_db</font> failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to retrieve Journal Codes from ledger.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  print "<h1>Error: No journals defined for the ledger $ledger_db!</h1>\n";
  print "$sys_admin";
  CleanUp();
}
for ($count=0; $count < $result->ntuples; $count++)
{
  $journal_code_array[$count] = $result->getvalue($count, 0);
}

# get the payee/payers in the current fiscal year for remembered transactions from the ledger to fill in the payee_payer_array
$result = $conn->exec("select payee_payer, memo, ref_num from journal_entry_tb where remember = true and trans_date between '$start_fiscal' and '$stop_fiscal' AND ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  print "<h1>Error: Accessing database <font color=\"blue\"$ledger_db</font> failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get remembered payee/payers for fiscal year.<br>$sys_admin";
  CleanUp();
}
for ($count=0; $count < $result->ntuples; $count++)
{
  # add payee/payers to payee_payer_array with R> in front and " - " and up to 8 characters of the memo field at the end
  $payee_payer_array[$count] = "R> " . $result->getvalue($count, 0) . " - " . substr($result->getvalue($count, 1), 0, 8);
  $payee_payer_ref_num_array[$count] = $result->getvalue($count, 2);
}
# get the distinct payee/payers from the current date back a year and a half for all most recently entered journal entries (recall data)
$result = $conn->exec("select distinct payee_payer from journal_entry_tb where trans_date between '$date_ago' and '$date' AND ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  print "<h1>Error: Accessing database <font color=\"blue\"$ledger_db</font> failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get unique payee/payers for one year and a half ago to now.<br>$sys_admin";
  CleanUp();
}
for ($count2=0; $count2 < $result->ntuples; $count++, $count2++)
{
  $payee_payer_array[$count] = $result->getvalue($count2, 0);
  $temp_payee_payer = $payee_payer_array[$count];
  $temp_payee_payer =~ s/\\/\\\\/g;
  $temp_payee_payer =~ s/\'/\\\'/g;
  $temp_payee_payer =~ s/\$/\\\$/g;
  # get the maximum ref_num for the payee_payer entry
  $result2 = $conn->exec("select max(ref_num) from journal_entry_tb where payee_payer = '" . $temp_payee_payer . "' AND ledger_code = '$ledger_code'");
  if ($result2->resultStatus != PGRES_TUPLES_OK)
  {
    print "<h1>Error: Accessing database <font color=\"blue\"$ledger_db</font> failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to get max ref_num for distinct payee/payer " . $payee_payer_array[$count] . " for fiscal year.<br>$sys_admin";
    CleanUp();
  }
  if ($result2->ntuples != 1)
  {
    print "<h1>Error: No value returned!</h1>\n";
    print "Tried to get max ref_num for distinct payee/payer.  No record returned!<br>$sys_admin";
    CleanUp();
  }
  $payee_payer_ref_num_array[$count] = $result2->getvalue(0, 0);
  # find the most recently used journal code for the unique set of payee-payers in the database
  $result2 = $conn->exec("select journal_code from journal_entry_tb where ledger_code = '$ledger_code' and payee_payer = '$temp_payee_payer' and ref_num = $payee_payer_ref_num_array[$count]");
  if ($result2->resultStatus != PGRES_TUPLES_OK)
  {
    print "<h1>Error: Accessing the database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to get the most recently used journal code for payee-payer: $temp_payee_payer.<br>$sys_admin";
    CleanUp();
  }
  if ($result2->ntuples <= 0)
  {
    print "<h1>Error: No records returned!</h1>\n";
    print "Tried to get the most recently used journal code for payee-payer: $temp_payee_payer.<br>$sys_admin";
    CleanUp();
  }
  $payee_payer_journal_code_array[$count] = $result2->getvalue(0, 0);
}

# display the table for the month's liability
print "<font size=+2><b>Liability for <font color=\"blue\">$month_name $input{'year_select'}</font></b></font><br><br>\n";
print "<table border=0 width=\"85%\" bgcolor=\"$menu_bgcolor\" cellspacing=0 cellpadding=2>\n";
print "<tr>\n";
print "<th align=left>&nbsp;&nbsp;Employee Name</th><th align=right>Gross Wages&nbsp;&nbsp;&nbsp;&nbsp;</th><th align=right>FITW&nbsp;&nbsp;&nbsp;&nbsp;</th><th align=right>FICA&nbsp;&nbsp;&nbsp;&nbsp;</th><th align=right>Medicare&nbsp;&nbsp;&nbsp;&nbsp;</th>\n";
print "</tr>\n";
foreach $employee_id (sort {$employee_info{$a}[0] cmp $employee_info{$b}[0]} keys %employee_info)
{
  print "<tr>\n";
  print "<td align=left><font size=-1>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;$employee_info{$employee_id}[0] - $employee_id</font></td>\n";
  print "<td align=right><font size=-1>";
  if ($employee_info{$employee_id}[1] < 0)
  {
    printf "<font color=red>(%s)</font>", commify(sprintf("%.2f", abs($employee_info{$employee_id}[1])));
  }
  else
  {
    printf "%s&nbsp;&nbsp;", commify(sprintf("%.2f", abs($employee_info{$employee_id}[1])));
  }
  print "</font></td>\n";
  print "<td align=right><font size=-1>";
  if ($employee_info{$employee_id}[2] < 0)
  {
    printf "<font color=red>(%s)</font>", commify(sprintf("%.2f", abs($employee_info{$employee_id}[2])));
  }
  else
  {
    printf "%s&nbsp;&nbsp;", commify(sprintf("%.2f", abs($employee_info{$employee_id}[2])));
  }
  print "</font></td>\n";
  print "<td align=right><font size=-1>";
  if ($employee_info{$employee_id}[3] < 0)
  {
    printf "<font color=red>(%s)</font>", commify(sprintf("%.2f", abs($employee_info{$employee_id}[3])));
  }
  else
  {
    printf "%s&nbsp;&nbsp;", commify(sprintf("%.2f", abs($employee_info{$employee_id}[3])));
  }
  print "</font></td>\n";
  print "<td align=right><font size=-1>";
  if ($employee_info{$employee_id}[4] < 0)
  {
    printf "<font color=red>(%s)</font>", commify(sprintf("%.2f", abs($employee_info{$employee_id}[4])));
  }
  else
  {
    printf "%s&nbsp;&nbsp;", commify(sprintf("%.2f", abs($employee_info{$employee_id}[4])));
  }
  print "</font></td>\n";
  print "</tr>\n";
}
print "<tr><td colspan=5><br></td></tr>\n";
print "<tr><th align=left colspan=5>&nbsp;&nbsp;Employer Name</th></tr>\n";
print "<tr>\n";
print "<td><font size=-1>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;$employer_name</font></td>\n";
print "<td colspan=2>";
if (sprintf("%.2f", $employer_fica_sum) ne sprintf("%.2f", $fica_sum) && sprintf("%.2f", $employer_medicare_sum) ne sprintf("%.2f", $medicare_sum))
{
  print "<font size=-1><b><i>Both FICA and Medicare withholdings do not match!</i></b>";
  print "<br>Employer FICA = " . commify(sprintf("%.2f", $employer_fica_sum)) . " : Withheld FICA = " . commify(sprintf("%.2f", $fica_sum));
  print "<br>Employer Medicare = " . commify(sprintf("%.2f", $employer_medicare_sum)) . " : Withheld Medicare = " . commify(sprintf("%.2f", $medicare_sum));
  print "</font>";
}
elsif (sprintf("%.2f", $employer_fica_sum) ne sprintf("%.2f", $fica_sum))
{
  print "<font size=-1><b><i>FICA withholdings do not match!</i></b>";
  print "<br>Employer FICA = " . commify(sprintf("%.2f", $employer_fica_sum)) . " : Withheld FICA = " . commify(sprintf("%.2f", $fica_sum));
  print "</font>";
}
elsif (sprintf("%.2f", $employer_medicare_sum) ne sprintf("%.2f", $medicare_sum))
{
  print "<font size=-1><b><i>Medicare withholdings do not match!</i></b></font>";
  print "<br>Employer Medicare = " . commify(sprintf("%.2f", $employer_medicare_sum)) . " : Withheld Medicare = " . commify(sprintf("%.2f", $medicare_sum));
  print "</font>";
}
else
{
  print "<br>";
}
print "</td>\n";
print "<td align=right><font size=-1>";
if ($employer_fica_sum < 0)
{
  printf "<font color=red>(%s)</font>", commify(sprintf("%.2f", abs($employer_fica_sum)));
}
else
{
  printf "%s&nbsp;&nbsp;", commify(sprintf("%.2f", abs($employer_fica_sum)));
}
print "</font></td>\n";
print "<td align=right><font size=-1>";
if ($employer_medicare_sum < 0)
{
  printf "<font color=red>(%s)</font>", commify(sprintf("%.2f", abs($employer_medicare_sum)));
}
else
{
  printf "%s&nbsp;&nbsp;", commify(sprintf("%.2f", abs($employer_medicare_sum)));
}
print "</font></td>\n";
print "</tr>\n";
print "<tr><td colspan=5><hr></td></tr>\n";
print "<tr>\n";
print "<td align=left><font size=-1><b>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Total</b></font></td>\n";
print "<td align=right><font size=-1><b>";
if ($salary_sum < 0)
{
  printf "<font color=red>(%s)</font>", commify(sprintf("%.2f", abs($salary_sum)));
}
else
{
  printf "%s&nbsp;&nbsp;", commify(sprintf("%.2f", abs($salary_sum)));
}
print "</b></font></td>\n";
print "<td align=right><font size=-1><b>";
if ($fitw_sum < 0)
{
  printf "<font color=red>(%s)</font>", commify(sprintf("%.2f", abs($fitw_sum)));
}
else
{
  printf "%s&nbsp;&nbsp;", commify(sprintf("%.2f", abs($fitw_sum)));
}
print "</b></font></td>\n";
print "<td align=right><font size=-1><b>";
if ($fica_sum + $employer_fica_sum < 0)
{
  printf "<font color=red>(%s)</font>", commify(sprintf("%.2f", abs($fica_sum + $employer_fica_sum)));
}
else
{
  printf "%s&nbsp;&nbsp;", commify(sprintf("%.2f", abs($fica_sum + $employer_fica_sum)));
}
print "</b></font></td>\n";
print "<td align=right><font size=-1><b>";
if ($medicare_sum + $employer_medicare_sum < 0)
{
  printf "<font color=red>(%s)</font>", commify(sprintf("%.2f", abs($medicare_sum + $employer_medicare_sum)));
}
else
{
  printf "%s&nbsp;&nbsp;", commify(sprintf("%.2f", abs($medicare_sum + $employer_medicare_sum)));
}
print "</b></font></td>\n";
print "</tr>\n";
print "<tr><td colspan=5><br></td></tr>\n";
print "<tr>\n";
print "<th>Federal Deposit Amount:</th>\n";
print "<td colspan=4 align=left>&nbsp;&nbsp;";
if ($total_deposit_amount < 0)
{
  printf "<font color=red>(%s)</font>", commify(sprintf("%.2f", abs($total_deposit_amount)));
}
else
{
  printf "%s", commify(sprintf("%.2f", abs($total_deposit_amount)));
}
print "</td>\n";
print "</tr>\n";
print "</table>\n";

print "<hr>\n";
# start form
print "<form method=post name=\"edit_trans_form\" action=\"$site_dir/cgi-bin/journal_activity_rows_sheet.cgi\">\n";
# hidden ledger_name value
print "<input type=hidden name=\"ledger_db\" value=\"$ledger_db\">\n";
# hidden journal code value
print "<input type=hidden name=\"journal_code\" value=\"$journal_code\">\n";
# hidden transaction date value
print "<input type=hidden name=\"trans_month\" value=\"$trans_month\">\n";
print "<input type=hidden name=\"trans_day1\" value=\"$trans_day1\">\n";
print "<input type=hidden name=\"trans_day2\" value=\"$trans_day2\">\n";
print "<input type=hidden name=\"trans_year\" value=\"$trans_year\">\n";
# hidden memo value
print "<input type=hidden name=\"memo\" value=\"$memo\">\n";
# hidden mode for row script (edit or new)
print "<input type=hidden name=\"mode\" value=\"new\">\n";
# hidden values for the debit rows and credit rows
print "<input type=hidden name=\"debit_values\" value=\"$debit_values\">\n";
print "<input type=hidden name=\"credit_values\" value=\"$credit_values\">\n";
# hidden value for the amount to put into debit or credit amount to draw from the bank account
if ($debit_amount > 0)
{
  print "<input type=hidden name=\"debit_amount\" value=\"$debit_amount\">\n";
}
elsif ($credit_amount > 0)
{
  print "<input type=hidden name=\"credit_amount\" value=\"$credit_amount\">\n";
}

# build table out of the data
print "<table border=1 cellspacing=1 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<th valign=top>Journal</th><th valign=top align=left>Date</th><th valign=top align=left>Payee-Payer</th>";
print "</tr>\n";
print "<tr nowrap>\n";
# journal code select
print "<td><select name=\"journal_code_select\">\n";
for ($count=0; $count < @journal_code_array; $count++)
{
  print "<option";
  if ($journal_code eq $journal_code_array[$count])
  {
    print " selected";
  }
  print ">" . $journal_code_array[$count] . "\n";
}
print "</select></td>\n";
# date input field
print "<td>\n";
print "M:<input type=text size=2 maxlength=2 name=\"trans_month_text\" value=\"\" onChange=\"trans_month_text_change();\">";
print "D:<input type=text size=2 maxlength=2 name=\"trans_day_text\" value=\"\" onChange=\"trans_day_text_change();\">";
print "Y:<input type=text size=4 maxlength=4 name=\"trans_year_text\" value=\"\" onChange=\"trans_year_text_change();\">";
print "&nbsp;<input type=text size=10 name=\"trans_date\" value=\"$trans_date\" onChange=\"trans_date_change()\">\n";
print "</td>\n";
# payee/payer select and input text field
print "<td><select name=\"payee_payer_select\" onChange=\"payee_payer_select_change();\">\n";
print "<option value=\"-1\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;\n";
for ($count=0; $count < @payee_payer_array; $count++)
{
  print "<option value=\"$payee_payer_ref_num_array[$count]\"";
  if ($payee_payer eq $payee_payer_array[$count])
  {
    print " selected";
  }
  print ">" . $payee_payer_array[$count] . "\n";
}
print "</select> <input type=text name=\"payee_payer\" value=\"$payee_payer\" size=25 onChange=\"payee_payer_text_change();\"></td>\n";
print "</tr>\n";
print "<tr>\n";
print "<th align=left>Remembered</th><th colspan=2 align=left>Memo</th>\n";
print "</tr>\n";
print "<tr>\n";
# remembered check box
print "<td><input type=checkbox name=\"remember\" value=\"true\"";
if ($remember eq "t")
{
  print " checked";
}
print "></td>\n";
# memo text field
print "<td colspan=2><input type=text name=\"memo_text\" value=\"$memo\" size=75></td>\n";
print "</tr>\n";
print "</table>\n";
# changed the name from create_submit to refresh1 to cause the journal entry script to go into refresh mode
print "<center><input type=submit name=\"refresh1\" value=\"New Transaction\" onClick=\"return validate_form();\"><input type=reset></center>\n";
print "</form>\n";

# adding the Javascript code for the most recently used journal code for payee-payers
print "<script language=Javascript>\n";
print "  <!--\n";
print "  // Inserting values for journal_code_array...\n";
my $match_found;
$match_found = 0;
for ($count=0; $count < scalar @payee_payer_ref_num_array; $count++)
{
  print "  journal_code_array[\"$payee_payer_ref_num_array[$count]\"] = ";
  # search for the journal code to assign the ref_num to
  for ($count2=0; $count2 < scalar @journal_code_array; $count2++)
  {
    if ($journal_code_array[$count2] eq $payee_payer_journal_code_array[$count])
    {
      print "$count2";
      $match_found = 1;
    }
  }
  if (not $match_found)
  {
    print "0";
  }
  print ";\n";
}
print "  //-->\n";
print "</script>\n";

print "</center>\n";

print "</body>\n";
print "</html>\n";
exit 0;
