#!/usr/bin/perl
#$Id: journal_activity.cgi,v 1.9 2004/02/18 00:03:59 james Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

# journal_activity.cgi  -  Created by James Pattie, 06/15/1999
# This script allows the user to either edit a journal entry, delete
# a specified journal entry or create a new journal entry.
use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features
use reports;

use vars '$date';
use vars '$date_ago';
use vars '$count';
use vars '$count2';
use vars '$journal_code';
use vars '@journal_code_array';
use vars '$trans_date';
use vars '$trans_month';
use vars '$trans_day';
use vars '$trans_day1';
use vars '$trans_day2';
use vars '$trans_year';
use vars '$payee_payer';
use vars '@payee_payer_array';
use vars '@payee_payer_ref_num_array';
use vars '@payee_payer_journal_code_array';
use vars '$memo';
use vars '$remember';
use vars '$current_fiscal_year';
use vars '$start_fiscal';
use vars '$stop_fiscal';
use vars '$start_month_fiscal';
use vars '$stop_month_fiscal';
use vars '$start_day_fiscal';
use vars '$stop_day_fiscal';
use vars '$start_year_fiscal';
use vars '$stop_year_fiscal';
use vars '$start_day1';
use vars '$start_day2';
use vars '$stop_day1';
use vars '$stop_day2';
use vars '$temp_payee_payer';
use vars '$journal_command';
use vars '$journal_view_command';
use vars '$journal_display_string';

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

my $mode;  # modes are 1=open, 2=delete, 3=add
my $ledger_name;  # name of ledger to work with
my $ledger_db;
my $ledger_color;
my $journal_name;
my $result2;
my $date_from;
my $date_to;
my $sort_journal;
my $sort_by;
my $journal_bgcolor;
my $num_journal_entries;
my $num_extra_rows;
$num_extra_rows = 5;
#my $use_java;
my $ledger_code;
my $report_obj;
my %journal_commands;
%journal_commands = ( "edit" => "edit", "delete" => "delete", "create" => "create" );

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

# set variables to default values
$ledger_db = "";

if (not exists $input{'journal_command'})
{
  PrintErrorHeader();
  print "<h1>Error:  Access Denied!</h1>\n";
  print "Command not received.<br>$sys_admin";
  CleanUp();
}
if (not exists $journal_commands{$input{'journal_command'}})
{
  PrintErrorHeader();
  print "<h1>Error:  Access Denied!</h1>\n";
  print "Invalid options.  Option = <b>$input{'journal_command'}</b>.<br>$sys_admin";
  CleanUp();
}

$ledger_db = $input{'ledger'};
if (length $ledger_db == 0)
{
  PrintErrorHeader();
  print "<h1>Error: You must specify a ledger to work with!</h1>\n";
  print "$back_button";
  CleanUp();
}

$journal_code = $input{'journal_code'};
if (length $journal_code == 0)
{
  PrintErrorHeader();
  print "<h1>Error: You must specify a journal to work with!</h1>\n";
  print "$back_button";
  CleanUp();
}

$journal_bgcolor = $input{'journal_bgcolor'};
if (length $journal_bgcolor == 0)
{
  $journal_bgcolor = "deepskyblue";
}

# validate we got valid input from the user.
if ($input{'journal_command'} eq "edit")
{
  $mode = 1;
}
elsif ($input{'journal_command'} eq "delete")
{
  $mode = 2;
}
elsif ($input{'journal_command'} eq "create")
{
  $mode = 3;
  if ($journal_code eq "all_journals")
  {
    $journal_code = "G";
  }
}
else
{
  PrintErrorHeader();
  print "<h1>Access Denied!</h1>\n";
  print "Error: Invalid options given!\n<br>$sys_admin";
  CleanUp();
}

if ($mode == 1 or $mode == 2)
{
  # check and make sure that we have either an entry to edit in the range or that
  # all the entries to delete are in the range and that the total number does not
  # exceed the number of entries displayed.
  if (not exists $input{'journal_entry'})
  {
    PrintErrorHeader();
    print "<h1>Error:  You must select an entry to Edit!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  if (length $input{'journal_entry'} == 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  The Transaction Reference Number must be specified!</h1>\n";
    print "$back_button";
    CleanUp();
  }
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# now validate that the ledger they specified is valid

$result = $conn->exec("select name from entity_tb where database_name = '$ledger_db'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Trying to check entity table.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: Ledger <font color=\"blue\">$ledger_db</font> does not exist!</h1>\n";
  print "$sys_admin";
  CleanUp();
}
else
{
  $ledger_name = $result->getvalue(0, 0);
}

# create the reports object
$report_obj = reports->new($conn, $ledger_db);
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_code = $report_obj->getLedgerCode();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_color = $report_obj->getLedgerColor();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}

# now make sure that the journal exists that we are supposed to work with
if ($journal_code eq "all_journals")
{ # this will be a special case
  $journal_name = "All Journals";
}
else
{
  $result = $conn->exec("select name from journal_tb where code = '$journal_code'");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    PrintErrorHeader();
    print "<h1>Error:  Accessing database <font color=\"blue\">$ledger_db</font> failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to validate Journal with code = <font color=\"blue\">$journal_code</font>.<br>$sys_admin";
    CleanUp();
  }
  if ($result->ntuples == 0)
  {
    if ($journal_code eq "G" && $mode == 3) # Creating
    {
      # Pick the first available Journal and go with that.
      $result = $conn->exec("SELECT name, code FROM journal_tb");
      if ($result->resultStatus != PGRES_TUPLES_OK)
      {
        PrintErrorHeader();
        print "<h1>Error:  Accessing database <font color=\"blue\">$ledger_db</font> failed!</h1>\n";
        print $conn->errorMessage . "<br>Tried to get new Default Journal to work with as Journal <b>General</b> does not exist.<br>$sys_admin";
        CleanUp();
      }
      if ($result->ntuples == 0)
      {
        PrintErrorHeader();
        print "<h1>Error:  No Journals exist! &nbsp;This should be impossible!</h1>\n";
        print "$sys_admin";
        CleanUp();
      }
      $journal_code = $result->getvalue(0, 1);  # get the new Journal Code to work with.
    }
    else
    {
      PrintErrorHeader();
      print "<h1>Error:  Journal with code = <font color=\"blue\">$journal_code</font> does not exist!</h1>\n";
      print "$sys_admin";
      CleanUp();
    }
  }
  $journal_name = $result->getvalue(0, 0);
}

# get current fiscal year and verify that it is not closed
$result = $conn->exec("select working_year from variables_tb WHERE ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  print "<h1>Error: Accessing database <font color=\"blue\"$ledger_db</font> failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get current fiscal year from database.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples != 1)
{
  print "<h1>Error: Wrong number of records returned!</h1>\n";
  print "There should only be one record in the variables_tb.  " . $result->ntuples . " were returned.<br>$sys_admin";
  CleanUp();
}
$current_fiscal_year = $result->getvalue(0, 0);
# get the date ranges for the current fiscal year if it is not closed
$result = $conn->exec("select closed, start_date, end_date from known_years_tb where year = $current_fiscal_year AND ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  print "<h1>Error: Accessing database <font color=\"blue\"$ledger_db</font> failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get current fiscal year info. from database.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples != 1)
{
  print "<h1>Error: Wrong number of records returned!</h1>\n";
  print "There should only be one record for each individual fiscal year.<br>$sys_admin";
  CleanUp();
}
if ($result->getvalue(0, 0) eq "t")
{
  print "<h1>Fiscal Year ($current_fiscal_year) is Closed!</h1>\n";
  print "You cannot work with the records in a closed fiscal year.<br>$sys_admin";
  CleanUp();
}
$start_fiscal = $result->getvalue(0, 1);
$stop_fiscal = $result->getvalue(0, 2);
($start_month_fiscal, $start_day_fiscal, $start_year_fiscal) = split "-", $start_fiscal;
($stop_month_fiscal, $stop_day_fiscal, $stop_year_fiscal) = split "-", $stop_fiscal;

# find out if there is a close out date in the current fiscal year and use it for the start date.
$result = $conn->exec("SELECT DISTINCT(acct_bal_tb.bal_date) from acct_bal_tb, known_years_tb, variables_tb WHERE acct_bal_tb.ledger_code = '$ledger_code' AND acct_bal_tb.ledger_code = known_years_tb.ledger_code AND known_years_tb.ledger_code = variables_tb.ledger_code AND known_years_tb.year = variables_tb.working_year AND bal_date >= known_years_tb.start_date AND acct_bal_tb.bal_date < known_years_tb.end_date ORDER BY acct_bal_tb.bal_date DESC");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  print "</td></tr></table>\n";
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to see if they have any closed out periods in the current fiscal year.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples > 0)
{
  my $temp_date = $result->getvalue(0, 0);
  $temp_date =~ s/-/\//g;
  $temp_date = `/bin/date --date="$temp_date 1 day" +%m-%d-%Y`;
  chomp $temp_date;
  ($start_month_fiscal, $start_day_fiscal, $start_year_fiscal) = split "-", $temp_date;
}

print "<head><title>$prog_name $prog_version Journals Activity</title></head>\n";
print "<body bgcolor=$body_bgcolor onLoad=\"do_load();\">\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
<script language=\"Javascript1.2\">
  <!--
END_OF_STYLE
if ($mode != 2)
{
  $start_day1 = int ($start_day_fiscal / 10);
  $start_day2 = ($start_day_fiscal % 10);
  $stop_day1 = int ($stop_day_fiscal / 10);
  $stop_day2 = ($stop_day_fiscal % 10);
  print <<"END_OF_STYLE";
  var trans_month;
  var trans_day1;
  var trans_day2;
  var trans_year;
  var start_month_fiscal = $start_month_fiscal;
  var start_day_fiscal = $start_day_fiscal;
  var start_day1_fiscal = $start_day1;
  var start_day2_fiscal = $start_day2;
  var start_year_fiscal = $start_year_fiscal;
  var stop_month_fiscal = $stop_month_fiscal;
  var stop_day_fiscal = $stop_day_fiscal;
  var stop_day1_fiscal = $stop_day1;
  var stop_day2_fiscal = $stop_day2;
  var stop_year_fiscal = $stop_year_fiscal;
  var start_day;
  var stop_day;
  var month_name;
  var month;
  var year;
  var old_trans_month;
  var old_trans_day1;
  var old_trans_day2;
  var old_trans_year;
  var date_valid;
  var journal_code_array = new Array();

END_OF_STYLE
}
if ($mode != 2)
{
  print <<"END_OF_STYLE";

  function do_load()
  {
    // fill in values for month, day1, day2, and year select form values

    // assign values to trans_month, trans_day, trans_year variables
    trans_month = document.edit_trans_form.trans_month.value;
    trans_day1 = document.edit_trans_form.trans_day1.value;
    trans_day2 = document.edit_trans_form.trans_day2.value;
    trans_year = document.edit_trans_form.trans_year.value;

    old_trans_month = trans_month;
    old_trans_day1 = trans_day1;
    old_trans_day2 = trans_day2;
    old_trans_year = trans_year;
    date_valid = true;

    // set the focus to the journal drop-down select
    document.edit_trans_form.journal_code_select.focus();
  }

  function payee_payer_text_change()
  {
    var count;
    var regexp;
    var payee_payer;
    var reg_exp;
    var non_space_found = false;

    regexp = eval("/^" + document.edit_trans_form.payee_payer_text.value + "+/i");

    // run through the payee_payer_select and see if the typed value matches any options in the select
    for (count=0; count < document.edit_trans_form.payee_payer_select.options.length; count++)
    {
      if (document.edit_trans_form.payee_payer_select.options[count].text.search(regexp) != -1)
      {
        document.edit_trans_form.payee_payer_select.selectedIndex = count;
        document.edit_trans_form.payee_payer_text.value = document.edit_trans_form.payee_payer_select.options[count].text;
        // select the last used journal code for the payee-payer from the journal code select
        document.edit_trans_form.journal_code_select.selectedIndex = journal_code_array[document.edit_trans_form.payee_payer_select.options[count].value];
      }
    }
    // if what is currently typed in the text field does not match what is selected in the select combo and what is typed starts with
    // "R> ", then throw an alert and clear the text field
    if (document.edit_trans_form.payee_payer_select.options[document.edit_trans_form.payee_payer_select.selectedIndex].text != document.edit_trans_form.payee_payer_text.value)
    {
      if ((document.edit_trans_form.payee_payer_text.value[0] == "R" || document.edit_trans_form.payee_payer_text.value[0] == "r") && document.edit_trans_form.payee_payer_text.value[1] == ">" && document.edit_trans_form.payee_payer_text.value[2] == " ")
      {
        alert("No match for remembered payee-payer found!");
        document.edit_trans_form.payee_payer_text.value = "";
      }
    }
    // if blank is typed in, uncheck the remembered checkbox
    // make sure the payee-payer is not empty
    payee_payer = "";
    payee_payer = document.edit_trans_form.payee_payer_text.value;
    for (var count=0; count < payee_payer.length && non_space_found == false; count++)
    {
      if (payee_payer[count] != " ")
      {
        non_space_found = true;
      }
    }
    if (!non_space_found)
    {
      // clear the payee-payer text field
      document.edit_trans_form.payee_payer_text.value = "";
      document.edit_trans_form.remembered_check.checked = false;
      // select the first (blank) payee-payer
      document.edit_trans_form.payee_payer_select.selectedIndex = 0;
    }
  }

  function payee_payer_select_change()
  {
    // set the payee_payer_text field to the string value of the option selected
    if (document.edit_trans_form.payee_payer_select.selectedIndex != 0)
    {
      document.edit_trans_form.payee_payer_text.value = document.edit_trans_form.payee_payer_select.options[document.edit_trans_form.payee_payer_select.selectedIndex].text;
      // select the last used journal code for the payee-payer from the journal code select
      document.edit_trans_form.journal_code_select.selectedIndex = journal_code_array[document.edit_trans_form.payee_payer_select.options[document.edit_trans_form.payee_payer_select.selectedIndex].value];
    }
    else
    {
      document.edit_trans_form.payee_payer_text.value = "";
    }
    // if blank is selected, uncheck the remembered checkbox
    if (document.edit_trans_form.payee_payer_select.options[document.edit_trans_form.payee_payer_select.selectedIndex].text == "" || document.edit_trans_form.payee_payer_select.options[document.edit_trans_form.payee_payer_select.selectedIndex].value == -1)
    {
      document.edit_trans_form.remembered_check.checked = false;
    }
  }

  function trans_month_text_change()
  {
    var date_array = new Array();
    var value;

    // split the date on "-"'s
    date_array = document.edit_trans_form.trans_date.value.split("-", 3);

    // make sure the value entered is a number
    value = document.edit_trans_form.trans_month_text.value;
    if (isNaN(parseInt(value, 10)))
    {
      if (value != "")
      {
        alert("Month value entered is not a number!\\nValue: " + value);
        document.edit_trans_form.trans_month_text.value = "";
        document.edit_trans_form.trans_month_text.focus();
      }
    }
    else
    {
      value = parseInt(value, 10);
      // make sure the month is between 1 and 12
      if (value < 1 || value > 12)
      {
        alert("Month value entered is out of range (1 - 12)!");
        document.edit_trans_form.trans_month_text.value = "";
        document.edit_trans_form.trans_month_text.focus();
      }
      else
      {
        // change the date_array element to contain the entered value
        date_array[0] = value;
        // clear the input field
        document.edit_trans_form.trans_month_text.value = "";
        // output the modified date
        document.edit_trans_form.trans_date.value = date_array[0] + "-" + date_array[1] + "-" + date_array[2];
        // validate the entered value
        trans_date_change();
      }
    }
  }

  function trans_day_text_change()
  {
    var date_array = new Array();
    var value;

    // split the date on "-"'s
    date_array = document.edit_trans_form.trans_date.value.split("-", 3);

    // make sure the value entered is a number
    value = document.edit_trans_form.trans_day_text.value;
    if (isNaN(parseInt(value, 10)))
    {
      if (value != "")
      {
        alert("Day value entered is not a number!\\nValue: " + value);
        document.edit_trans_form.trans_day_text.value = "";
        document.edit_trans_form.trans_day_text.focus();
      }
    }
    else
    {
      value = parseInt(value, 10);
      // make sure the day is between 1 and 31
      if (value < 1 || value > 31)
      {
        alert("Day value entered is out of range (1 - 31)!");
        document.edit_trans_form.trans_day_text.value = "";
        document.edit_trans_form.trans_day_text.focus();
      }
      else
      {
        // change the date_array element to contain the entered value
        date_array[1] = value;
        // clear the input field
        document.edit_trans_form.trans_day_text.value = "";
        // output the modified date
        document.edit_trans_form.trans_date.value = date_array[0] + "-" + date_array[1] + "-" + date_array[2];
        // validate the entered value
        trans_date_change();
      }
    }
  }

  function trans_year_text_change()
  {
    var date_array = new Array();
    var value;

    // split the date on "-"'s
    date_array = document.edit_trans_form.trans_date.value.split("-", 3);

    // make sure the value entered is a number
    value = document.edit_trans_form.trans_year_text.value;
    if (isNaN(parseInt(value, 10)))
    {
      if (value != "")
      {
        alert("Year value entered is not a number!\\nValue: " + value);
        document.edit_trans_form.trans_year_text.value = "";
        document.edit_trans_form.trans_year_text.focus();
      }
    }
    else
    {
      value = parseInt(value, 10);
      // make sure the year is greater than or equal to the beginning fiscal year value
      if (value < start_year_fiscal)
      {
        alert("Year value entered is less than the beginning fiscal year value (" + start_year_fiscal + ")!");
        document.edit_trans_form.trans_year_text.value = "";
        document.edit_trans_form.trans_year_text.focus();
      }
      else
      {
        // change the date_array element to contain the entered value
        date_array[2] = value;
        // clear the input field
        document.edit_trans_form.trans_year_text.value = "";
        // output the modified date
        document.edit_trans_form.trans_date.value = date_array[0] + "-" + date_array[1] + "-" + date_array[2];
        // validate the entered value
        trans_date_change();
      }
    }
  }

  function trans_date_change()
  {
    // parse the inputted date for validity to see if it falls between the start and stop dates of the current fiscal range
    var date_to_parse = new Date();
    var month;
    var day;
    var day1;
    var day2;
    var year;
    var date_string;
    var regular_expression;
    var date_array = new Array();

    regular_expression = new RegExp("-", "g");
    // before parsing date text field, substitute all "-"'s with "/"'s
    date_string = document.edit_trans_form.trans_date.value;
    date_string = date_string.replace(regular_expression, "/");
    // determine if the date string has a month, day, and year by 'split'ting it on '/'s and seeing how many array elements are returned
    date_array = date_string.split("/");
    if (date_array.length == 2)
    {
      // year was not entered; attempt to determine which fiscal year to enter based on the month and day values
      month = date_array[0];
      day = date_array[1];
      if (month >= start_month_fiscal)
      {
        year = start_year_fiscal;
      }
      else if (month <= stop_month_fiscal)
      {
        year = stop_year_fiscal;
      }
      day1 = Math.floor(day/10);
      day2 = day % 10;
    }
    else
    {
      // make sure they entered a correct year; if not, mangle the year into one of the (possibly) two year values for the fiscal range
      if (parseInt(date_array[2], 10) < 100)  // user entered a two or less digit year
      {
        var year_count = start_year_fiscal;
        var year_check;
        var year_stop = parseInt(date_array[2], 10);
        // run from the start_fiscal_year up to the value entered by adding to the start year until the last two digits match the counter
        year_check = parseInt(year_count % 100, 10);
        while (year_check != year_stop)
        {
          year_count++;
          year_check = parseInt(year_count % 100, 10);
        }
        year = year_count;
        month = date_array[0];
        day = date_array[1];
        day1 = Math.floor(day/10);
        day2 = day % 10;
      }
      else
      {
        date_to_parse.setTime(Date.parse(date_string));
        month = parseInt(date_to_parse.getMonth()) + 1;
        day = parseInt(date_to_parse.getDate());
        day1 = Math.floor(day/10);
        day2 = day % 10;
        year = parseInt(date_to_parse.getFullYear());
      }
    }

    // make sure that year is in the fiscal range of years
    if (year != start_year_fiscal && year != stop_year_fiscal)
    {
      if (year > stop_year_fiscal)
      {
        if (!confirm("You are entering a date that is in the future\\\nbased on the current fiscal range.\\\nDo you want to enter this date?"))
        {
          month = old_trans_month;
          day1 = old_trans_day1;
          day2 = old_trans_day2;
          year = old_trans_year;
        }
        else
        {
          date_valid = true;
        }
      }
      else
      {
        date_valid = false;
        alert("Year entered is not in fiscal range!");
        return;
      }
    }
    // make sure that month entered is in fiscal range for the year input
    if (year == start_year_fiscal && month < start_month_fiscal)
    {
      date_valid = false;
      alert("Month entered is not in fiscal range!");
      return;
    }
    if (year == stop_year_fiscal && month > stop_month_fiscal)
    {
      if (!confirm("You are entering a date that is in the future\\\nbased on the current fiscal range.\\\nDo you want to enter this date?"))
      {
        month = old_trans_month;
        day1 = old_trans_day1;
        day2 = old_trans_day2;
        year = old_trans_year;
      }
      else
      {
        date_valid = true;
      }
    }
    // make sure that day entered is in fiscal range for the year and month input
    if (year == start_year_fiscal && month == start_month_fiscal && day < start_day_fiscal)
    {
      date_valid = false;
      alert("Day entered is not in fiscal range!");
      return;
    }
    if (year == stop_year_fiscal && month == stop_month_fiscal && day > stop_day_fiscal)
    {
      if (!confirm("You are entering a date that is in the future\\\nbased on the current fiscal range.\\\nDo you want to enter this date?"))
      {
        month = old_trans_month;
        day1 = old_trans_day1;
        day2 = old_trans_day2;
        year = old_trans_year;
      }
      else
      {
        date_valid = true;
      }
    }

    // redisplay the date from the parsed input into the trans_date text field
    document.edit_trans_form.trans_date.value = month + "-" + day1 + day2 + "-" + year;
    old_trans_month = month;
    old_trans_day1 = day1;
    old_trans_day2 = day2;
    old_trans_year = year;
  }

  function validate_form()
  {
    var date_to_parse = new Date();
    var month;
    var day;
    var day1;
    var day2;
    var year;
    var date_string;
    var regular_expression;
    var payee_payer;
    var non_space_found = false;

    // make sure the date is within the fiscal year range

    regular_expression = new RegExp("-", "g");
    // before parsing date text field, substitute all "-"'s with "/"'s
    date_string = document.edit_trans_form.trans_date.value;
    date_string = date_string.replace(regular_expression, "/");
    date_to_parse.setTime(Date.parse(date_string));
    month = parseInt(date_to_parse.getMonth()) + 1;
    day = parseInt(date_to_parse.getDate());
    day1 = Math.floor(day/10);
    day2 = day % 10;
    year = parseInt(date_to_parse.getFullYear());

    // make sure that year is in the fiscal range of years
    if (year != start_year_fiscal && year != stop_year_fiscal)
    {
      if (!date_valid)
      {
        alert("Year entered is not in fiscal range!");
        return false;
      }
    }
    // make sure that month entered is in fiscal range for the year input
    if (year == start_year_fiscal && month < start_month_fiscal)
    {
      alert("Month entered is not in fiscal range!");
      return false;
    }
    if (year == stop_year_fiscal && month > stop_month_fiscal)
    {
      if (!date_valid)
      {
        alert("Month entered is not in fiscal range!");
        return false;
      }
    }
    // make sure that day entered is in fiscal range for the year and month input
    if (year == start_year_fiscal && month == start_month_fiscal && day < start_day_fiscal)
    {
      alert("Day entered is not in fiscal range!");
      return false;
    }
    if (year == stop_year_fiscal && month == stop_month_fiscal && day > stop_day_fiscal)
    {
      if (!date_valid)
      {
        alert("Day entered is not in fiscal range!");
        return false;
      }
    }

    // make sure the payee-payer is not empty
    payee_payer = document.edit_trans_form.payee_payer_text.value;
    for (var count=0; count < payee_payer.length && non_space_found == false; count++)
    {
      if (payee_payer[count] != " ")
      {
        non_space_found = true;
      }
    }
    if (!non_space_found)
    {
      // clear the payee-payer text field
      document.edit_trans_form.payee_payer_text.value = "";
      document.edit_trans_form.remembered_check.checked = false;
    }
    return true;
  }

END_OF_STYLE
}
else  # we just deleted an entry
{
  print <<"END_OF_STYLE";
  var error_occurred = false;
  function do_load()
  {
    if (!error_occurred)
    {
      document.journal_view_form.submit();
    }
  }
END_OF_STYLE
}
print <<"END_OF_STYLE";

  //-->
</script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;
$date_ago = `/bin/date -d '1 year 6 months ago' +%m-%d-%Y`;
chomp $date_ago;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
if ($mode == 1)
{
  print "<td align=left width=47%><font color=\"red\">";
  if ($journal_code eq "all_journals")
  {
    print "Editing Journal Entry in Journal <font color=\"blue\">$journal_name</font>";
  }
  else
  {
    print "Working with Journal <font color=\"blue\">$journal_name</font>";
  }
}
elsif ($mode == 2)
{
  print "<td align=left width=47%><font color=\"red\">";
  print "Deleting Journal Entries in Journal <font color=\"blue\">$journal_name</font>";
}
elsif ($mode == 3)
{
  print "<td align=left width=47%><font color=\"red\">";
  print "Creating Journal Entry in Journal <font color=\"blue\">$journal_name</font>";
}
print " in Ledger <font color=\"$ledger_color\">$ledger_name</font></font></td>\n";
print "<td align=left><a href=\"$site_dir/help.html#journal\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardLedgerMainMenu($ledger_db);

if ($mode == 1) # find the Journal Entry to edit and display our own edit code
{
  my ($ref_num, $temp_num, $temp);

  $ref_num = $input{'journal_entry'};

  # do a quick validation that this Reference number exists.
  $result = $conn->exec("select journal_code from journal_entry_tb where ref_num = $ref_num AND ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    print "<h1>Error:  Accessing database <font color=\"blue\">$ledger_db</font> failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to validate that the Journal Entry exists for ref_num = <font color=\"blue\">$ref_num</font>.<br>$sys_admin";
    CleanUp();
  }

  if ($result->ntuples == 0)
  {
    print "<h1>Error:  The Journal Entry with ref_num = <font color=\"blue\">$ref_num</font> does not exist!</h1>\n";
    print "It may have already been deleted.<br>$back_button";
    CleanUp();
  }

   # Create the HTML only Edit Journal Transaction stuff here.

   # get the first half of the transaction information
   $result = $conn->exec("select ref_num, journal_code, trans_date, payee_payer, memo, remember from journal_entry_tb where ref_num = $ref_num AND ledger_code = '$ledger_code'");
   if ($result->resultStatus != PGRES_TUPLES_OK)
   {
     print "<h1>Error: Accessing database <font color=\"blue\">$ledger_db</font> failed!</h1>\n";
     print $conn->errorMessage . "<br>Tried to retrieve Journal Entry for ref_num = <font color=\"blue\">$ref_num</font>.<br>$sys_admin";
     CleanUp();
   }
   if ($result->ntuples != 1)
   {
     print "<h1>Error: Wrong number of records returned!</h1>\n";
     print "The number of records that was returned = " . $result->ntuples . ". There should only be 1.\n";
     CleanUp();
   }

   $journal_code = $result->getvalue(0, 1);
   $trans_date = $result->getvalue(0, 2);
   ($trans_month, $trans_day, $trans_year) = split "-", $trans_date;
   $trans_day1 = int ($trans_day / 10);
   $trans_day2 = int ($trans_day % 10);
   $payee_payer = $result->getvalue(0, 3);
   $memo = $result->getvalue(0, 4);
   $remember = $result->getvalue(0, 5);

   if ($trans_year < $start_year_fiscal) # || $trans_year > $stop_year_fiscal)
   {
     # cannot edit selected record
     print "<h1>Error: Cannot edit selected record!</h1>\n";
     print "The date of the selected record falls outside the currently open working year.<br>$back_button";
     CleanUp();
   }
   elsif (($trans_month < $start_month_fiscal && $trans_year == $start_year_fiscal)) # || ($trans_month > $stop_month_fiscal && $trans_year == $stop_year_fiscal))
   {
     # cannot edit selected record
     print "<h1>Error: Cannot edit selected record!</h1>\n";
     print "The date of the selected record falls outside the currently open working year.<br>$back_button";
     CleanUp();
   }
   elsif (($trans_day < $start_day_fiscal && $trans_month == $start_month_fiscal && $trans_year == $start_year_fiscal)) # || ($trans_day > $stop_day_fiscal && $trans_month == $stop_month_fiscal && $trans_year == $stop_year_fiscal))
   {
     # cannot edit selected record
     print "<h1>Error: Cannot edit selected record!</h1>\n";
     print "The date of the selected record falls outside the currently open working year.<br>$back_button";
     CleanUp();
   }

   # get the journal codes from the ledger to fill in the journal_code_array
   $result = $conn->exec("select distinct code from journal_tb"); # WHERE code <> 'IS'"); # remove investment journal code from list
   if ($result->resultStatus != PGRES_TUPLES_OK)
   {
     print "<h1>Error: Accessing database <font color=\"blue\">$ledger_db</font> failed!</h1>\n";
     print $conn->errorMessage . "<br>Tried to retrieve Journal Codes from ledger.<br>$sys_admin";
     CleanUp();
   }
   if ($result->ntuples == 0)
   {
     print "<h1>Error: No journals defined for the ledger $ledger_db!</h1>\n";
     print "$sys_admin";
     CleanUp();
   }
   for ($count=0; $count < $result->ntuples; $count++)
   {
     $journal_code_array[$count] = $result->getvalue($count, 0);
   }

   # get the payee/payers in the current fiscal year for remembered transactions from the ledger to fill in the payee_payer_array
   $result = $conn->exec("select payee_payer, memo, ref_num from journal_entry_tb where remember = true and trans_date between '$start_fiscal' and '$stop_fiscal' AND ledger_code = '$ledger_code'");
   if ($result->resultStatus != PGRES_TUPLES_OK)
   {
     print "<h1>Error: Accessing database <font color=\"blue\"$ledger_db</font> failed!</h1>\n";
     print $conn->errorMessage . "<br>Tried to get remembered payee/payers for fiscal year.<br>$sys_admin";
     CleanUp();
   }
   for ($count=0; $count < $result->ntuples; $count++)
   {
     # add payee/payers to payee_payer_array with R> in front and " - " and up to 8 characters of the memo field at the end
     $payee_payer_array[$count] = "R> " . $result->getvalue($count, 0) . " - " . substr($result->getvalue($count, 1), 0, 8);
     $payee_payer_ref_num_array[$count] = $result->getvalue($count, 2);
   }
   # get the distinct payee/payers from the current date back a year and a half for all most recently entered journal entries (recall data)
   $result = $conn->exec("select distinct payee_payer from journal_entry_tb where trans_date between '$date_ago' and '$date' AND ledger_code = '$ledger_code'");
   if ($result->resultStatus != PGRES_TUPLES_OK)
   {
     print "<h1>Error: Accessing database <font color=\"blue\"$ledger_db</font> failed!</h1>\n";
     print $conn->errorMessage . "<br>Tried to get unique payee/payers for one year and a half ago to now.<br>$sys_admin";
     CleanUp();
   }
   for ($count2=0; $count2 < $result->ntuples; $count++, $count2++)
   {
     $payee_payer_array[$count] = $result->getvalue($count2, 0);
     $temp_payee_payer = $payee_payer_array[$count];
     $temp_payee_payer =~ s/\\/\\\\/g;
     $temp_payee_payer =~ s/\'/\\\'/g;
     $temp_payee_payer =~ s/\$/\\\$/g;
     # get the maximum ref_num for the payee_payer entry
     $result2 = $conn->exec("select max(ref_num) from journal_entry_tb where payee_payer = '" . $temp_payee_payer . "' AND ledger_code = '$ledger_code'");
     if ($result2->resultStatus != PGRES_TUPLES_OK)
     {
       print "<h1>Error: Accessing database <font color=\"blue\"$ledger_db</font> failed!</h1>\n";
       print $conn->errorMessage . "<br>Tried to get max ref_num for distinct payee/payer " . $payee_payer_array[$count] . " for fiscal year.<br>$sys_admin";
       CleanUp();
     }
     if ($result2->ntuples != 1)
     {
       print "<h1>Error: No value returned!</h1>\n";
       print "Tried to get max ref_num for distinct payee/payer.  No record returned!<br>$sys_admin";
       CleanUp();
     }
     $payee_payer_ref_num_array[$count] = $result2->getvalue(0, 0);
     # find the most recently used journal code for the unique set of payee-payers in the database
     $result2 = $conn->exec("select journal_code from journal_entry_tb where ledger_code = '$ledger_code' and payee_payer = '$temp_payee_payer' and ref_num = $payee_payer_ref_num_array[$count]");
     if ($result2->resultStatus != PGRES_TUPLES_OK)
     {
       print "<h1>Error: Accessing the database failed!</h1>\n";
       print $conn->errorMessage . "<br>Tried to get the most recently used journal code for payee-payer: $temp_payee_payer.<br>$sys_admin";
       CleanUp();
     }
     if ($result2->ntuples <= 0)
     {
       print "<h1>Error: No records returned!</h1>\n";
       print "Tried to get the most recently used journal code for payee-payer: $temp_payee_payer.<br>$sys_admin";
       CleanUp();
     }
     $payee_payer_journal_code_array[$count] = $result2->getvalue(0, 0);
   }

   # start form
   print "<form method=post name=\"edit_trans_form\" action=\"$site_dir/cgi-bin/journal_activity_rows_sheet.cgi\">\n";
   # hidden ref_num value
   print "<input type=hidden name=\"ref_num\" value=\"$ref_num\">\n";
   # hidden ledger_name value
   print "<input type=hidden name=\"ledger_db\" value=\"$ledger_db\">\n";
   # hidden journal code value
   print "<input type=hidden name=\"journal_code\" value=\"$journal_code\">\n";
   # hidden transaction date value
   print "<input type=hidden name=\"trans_month\" value=\"$trans_month\">\n";
   print "<input type=hidden name=\"trans_day1\" value=\"$trans_day1\">\n";
   print "<input type=hidden name=\"trans_day2\" value=\"$trans_day2\">\n";
   print "<input type=hidden name=\"trans_year\" value=\"$trans_year\">\n";
   # hidden payee-payer value
   print "<input type=hidden name=\"payee_payer\" value=\"$payee_payer\">\n";
   # hidden memo value
   print "<input type=hidden name=\"memo\" value=\"$memo\">\n";
   # hidden mode for row script (edit or new)
   print "<input type=hidden name=\"mode\" value=\"edit\">\n";

   print "</center>\n";
   print "Reference Number: $ref_num<br><br>\n";
   print "<center>\n";
   # build table out of the data
   print "<table border=1 cellspacing=1 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
   print "<tr>\n";
   print "<th valign=top>Journal</th><th valign=top align=left>Date</th><th valign=top align=left>Payee-Payer</th>";
   print "</tr>\n";
   print "<tr nowrap>\n";
   # journal code select
   print "<td><select name=\"journal_code_select\">\n";
   for ($count=0; $count < @journal_code_array; $count++)
   {
     print "<option";
     if ($journal_code eq $journal_code_array[$count])
     {
       print " selected";
     }
     print ">" . $journal_code_array[$count] . "\n";
   }
   print "</select></td>\n";
   # date input field
   print "<td>\n";
   print "M:<input type=text size=2 maxlength=2 name=\"trans_month_text\" value=\"\" onChange=\"trans_month_text_change();\">";
   print "D:<input type=text size=2 maxlength=2 name=\"trans_day_text\" value=\"\" onChange=\"trans_day_text_change();\">";
   print "Y:<input type=text size=4 maxlength=4 name=\"trans_year_text\" value=\"\" onChange=\"trans_year_text_change();\">";
   print "&nbsp;<input type=text size=10 name=\"trans_date\" value=\"$trans_date\" onChange=\"trans_date_change()\">\n";
   print "</td>\n";
   # payee/payer select and input text field
   print "<td><input type=text name=\"payee_payer_text\" value=\"$payee_payer\" size=25 onChange=\"payee_payer_text_change();\"> <select name=\"payee_payer_select\" onChange=\"payee_payer_select_change();\">\n";
   print "<option value=\"-1\">---------------\n";
   for ($count=0; $count < @payee_payer_array; $count++)
   {
     print "<option value=\"$payee_payer_ref_num_array[$count]\"";
     if ($payee_payer eq $payee_payer_array[$count])
     {
       print " selected";
     }
     print ">" . $payee_payer_array[$count] . "\n";
   }
   print "</select></td>\n";
   print "</tr>\n";
   print "<tr>\n";
   print "<th align=left>R</th><th colspan=2 align=left>Memo</th>\n";
   print "</tr>\n";
   print "<tr>\n";
   # remembered check box
   print "<td><input type=checkbox name=\"remembered_check\" value=\"ON\"";
   if ($remember eq "t")
   {
     print " checked";
   }
   print "></td>\n";
   # memo text field
   print "<td colspan=2><input type=text name=\"memo_text\" value=\"$memo\" size=75></td>\n";
   print "</tr>\n";
   print "</table>\n";
   print "<center><input type=submit name=\"edit_submit\" value=\"Continue\" onClick=\"return validate_form();\"><input type=reset></center>\n";
   my $temp_str = GetJournalViewInput();
   if (length $temp_str > 0)
   {
     print "$temp_str";
   }
   print "</form>\n";
   my $temp_str = GetJournalViewInput();
   if (length $temp_str > 0)
   {
     print "<form name=\"journal_view_form\" method=post action=\"$site_dir/cgi-bin/journal_mod1.cgi\">\n";
     print "$temp_str";
     print "<br><br><br><a href=\"$site_dir/cgi-bin/journal_mod1.cgi\" onClick=\"document.journal_view_form.submit(); return false;\">To return to the Journal View, click <u>here</u>.</a>\n";
     print "</form>\n";
   }
#  }
}
elsif ($mode == 2) # delete the specified entry.
{
  # we need to loop through all $num_journal_entries and see if any are set to be deleted
  my ($count, $i, $ref_num, $temp2, $temp3, $temp4);
  $ref_num = $input{'journal_entry'};
  if (length $ref_num == 0)
  {
    print "<h1>Error:  Incomplete Entry record!</h1>\n";
    print "The selected Transaction does not have a valid Reference Number!<br>$sys_admin";
    print "<script language=\"JavaScript1.2\">error_occurred = true;</script>\n";
    CleanUp();
  }

  # lookup the reference number in the database and make sure it exists.
   $result = $conn->exec("select journal_code, trans_date from journal_entry_tb where ref_num = $ref_num AND ledger_code = '$ledger_code'");
   if ($result->resultStatus != PGRES_TUPLES_OK)
   {
     print "<h1>Error: Accessing database <font color=\"blue\">$ledger_db</font> failed!</h1>\n";
     print $conn->errorMessage . "<br>Tried to retrieve Journal Entry for ref_num = <font color=\"blue\">$ref_num</font>.<br>$sys_admin";
     print "<script language=\"JavaScript1.2\">error_occurred = true;</script>\n";
     CleanUp();
   }
   if ($result->ntuples != 1)
   {
     print "<h1>Error: Wrong number of records returned!</h1>\n";
     print "The number of records that was returned = " . $result->ntuples . ". There should only be 1.\n";
     print "<script language=\"JavaScript1.2\">error_occurred = true;</script>\n";
     CleanUp();
   }

   my $j_code = $result->getvalue(0, 0);
   $trans_date = $result->getvalue(0, 1);
   ($trans_month, $trans_day, $trans_year) = split "-", $trans_date;
   $trans_day1 = int ($trans_day / 10);
   $trans_day2 = int ($trans_day % 10);

   if ($trans_year < $start_year_fiscal) # || $trans_year > $stop_year_fiscal)
   {
     # cannot edit selected record
     print "<h1>Error: Cannot delete selected record!</h1>\n";
     print "The date of the selected record falls outside the currently open working year.<br>$back_button";
     print "<script language=\"JavaScript1.2\">error_occurred = true;</script>\n";
     CleanUp();
   }
   elsif (($trans_month < $start_month_fiscal && $trans_year == $start_year_fiscal)) # || ($trans_month > $stop_month_fiscal && $trans_year == $stop_year_fiscal))
   {
     # cannot edit selected record
     print "<h1>Error: Cannot delete selected record!</h1>\n";
     print "The date of the selected record falls outside the currently open working year.<br>$back_button";
     print "<script language=\"JavaScript1.2\">error_occurred = true;</script>\n";
     CleanUp();
   }
   elsif (($trans_day < $start_day_fiscal && $trans_month == $start_month_fiscal && $trans_year == $start_year_fiscal)) # || ($trans_day > $stop_day_fiscal && $trans_month == $stop_month_fiscal && $trans_year == $stop_year_fiscal))
   {
     # cannot edit selected record
     print "<h1>Error: Cannot delete selected record!</h1>\n";
     print "The date of the selected record falls outside the currently open working year.<br>$back_button";
     print "<script language=\"JavaScript1.2\">error_occurred = true;</script>\n";
     CleanUp();
   }

  $result = $conn->exec("select name from journal_tb where code = '$j_code'");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    print "<h1>Error:  Accessing database <font color=\"blue\">$ledger_db</font> failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to get Journal name for code = <font color=\"blue\">$j_code</font>.<br>$sys_admin";
    print "<script language=\"JavaScript1.2\">error_occurred = true;</script>\n";
    CleanUp();
  }
  if ($result->ntuples == 0)
  {
    print "<h1>Error: The Journal with Code = <font color=\"blue\">$j_code</font> does not exist!</h1>\n";
    print "It may have already been deleted.<br>$sys_admin";
    print "<script language=\"JavaScript1.2\">error_occurred = true;</script>\n";
    CleanUp();
  }
  my $j_name = $result->getvalue(0, 0);

  # now we start a transaction
  $result = $conn->exec("begin work");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Starting Transaction failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to start transaction before deleting entry <font color=\"blue\">$ref_num</font>.<br>$sys_admin";
    print "<script language=\"JavaScript1.2\">error_occurred = true;</script>\n";
    CleanUp();
  }

  # delete the entry
  $result = $conn->exec("delete from journal_entry_tb where ref_num = $ref_num AND ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Deleting Journal Entry <font color=\"blue\">$ref_num</font> failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to delete entry <font color=\"blue\">$ref_num</font> from journal_entry_tb.<br>$sys_admin";
    print "<script language=\"JavaScript1.2\">error_occurred = true;</script>\n";
    CleanUp();
  }
  $result = $conn->exec("delete from debitcredit_tb where parent_num = $ref_num AND ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Deleting Journal Entry <font color=\"blue\">$ref_num</font> failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to delete entry <font color=\"blue\">$ref_num</font> from debitcredit_tb.<br>$sys_admin";
    print "<script language=\"JavaScript1.2\">error_occurred = true;</script>\n";
    CleanUp();
  }

  # commit the transaction
  $result = $conn->exec("commit work");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Committing Transaction failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to commit transaction after deleting entry <font color=\"blue\">$ref_num</font>.<br>$sys_admin";
    print "<script language=\"JavaScript1.2\">error_occurred = true;</script>\n";
    CleanUp();
  }

  print "Successfully deleted Journal Entry <font color=\"blue\">$ref_num</font> in Journal <font color=\"red\">$j_name</font><br>\n";
  print "<form name=\"journal_view_form\" method=post action=\"$site_dir/cgi-bin/journal_mod1.cgi\">\n";
  # hidden ledger_name value
  print "<input type=hidden name=\"ledger_db\" value=\"$ledger_db\">\n";
  my $temp_str = GetJournalViewInput();
  if (length $temp_str > 0)
  {
    print "$temp_str";
    print "<a href=\"$site_dir/cgi-bin/journal_mod1.cgi\" onClick=\"document.journal_view_form.submit(); return false;\">To return to the Journal View, click <u>here</u>.</a>\n";
  }
  print "</form>\n";
}
elsif ($mode == 3) # create the new journal entry
{
  # get the date of the largest reference number
  $result = $conn->exec("select max(ref_num) from journal_entry_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    print "<h1>Error: Accessing database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to get the maximum ref_num in order to get the most recently entered date.<br>$sys_admin";
    CleanUp();
  }
  if ($result->ntuples == 0)
  {
    $trans_date = `/bin/date +%m-%d-%Y`;
    chomp $trans_date;
  }
  elsif ($result->ntuples == 1)
  {
    if ($result->getvalue(0, 0) eq "")
    {
      $trans_date = `/bin/date +%m-%d-%Y`;
      chomp $trans_date;
    }
    else
    {
      $result2 = $conn->exec("select trans_date from journal_entry_tb where ledger_code = '$ledger_code' AND ref_num = " . $result->getvalue(0, 0));
      if ($result2->resultStatus != PGRES_TUPLES_OK)
      {
        print "<h1>Error: Accessing database failed!</h1>\n";
        print $conn->errorMessage . "<br>Tried to get trans_date from record " . $result->getvalue(0, 0) . ".<br>$sys_admin";
        CleanUp();
      }
      if ($result2->ntuples == 1)
      {
        $trans_date = $result2->getvalue(0, 0);
      }
      else
      {
        print "<h1>Error: Wrong number of records returned!</h1>\n";
        print $result2->ntuples . " records were returned. There should be only one.<br>$sys_admin";
        CleanUp();
      }
    }
  }
  else
  {
    print "<h1>Error: Too many records returned!</h1>\n";
    print "There should only be one date returned for the maximum reference number.<br>$sys_admin";
    CleanUp();
  }
  ($trans_month, $trans_day, $trans_year) = split "-", $trans_date;
  $trans_day1 = int ($trans_day / 10);
  $trans_day2 = int ($trans_day % 10);
  $payee_payer = "";
  $memo = "";
  $remember = "f";

  if ($trans_year < $start_year_fiscal || $trans_year > $stop_year_fiscal)
  {
    $trans_month = $start_month_fiscal;
    $trans_day = $start_day_fiscal;
    $trans_day1 = int ($start_day_fiscal / 10);
    $trans_day2 = int ($start_day_fiscal % 10);
    $trans_year = $start_year_fiscal;
    $trans_date = $trans_month . "-" . $trans_day . "-" . $trans_year;
  }
  elsif (($trans_month < $start_month_fiscal && $trans_year == $start_year_fiscal) || ($trans_month > $stop_month_fiscal && $trans_year == $stop_year_fiscal))
  {
    $trans_month = $start_month_fiscal;
    $trans_day = $start_day_fiscal;
    $trans_day1 = int ($start_day_fiscal / 10);
    $trans_day2 = int ($start_day_fiscal % 10);
    $trans_year = $start_year_fiscal;
    $trans_date = $trans_month . "-" . $trans_day . "-" . $trans_year;
  }
  elsif (($trans_day < $start_day_fiscal && $trans_month == $start_month_fiscal && $trans_year == $start_year_fiscal) || ($trans_day > $stop_day_fiscal && $trans_month == $stop_month_fiscal && $trans_year == $stop_year_fiscal))
  {
    $trans_month = $start_month_fiscal;
    $trans_day = $start_day_fiscal;
    $trans_day1 = int ($start_day_fiscal / 10);
    $trans_day2 = int ($start_day_fiscal % 10);
    $trans_year = $start_year_fiscal;
    $trans_date = $trans_month . "-" . $trans_day . "-" . $trans_year;
  }

  # get the journal codes from the ledger to fill in the journal_code_array
  $result = $conn->exec("select distinct code from journal_tb WHERE code <> 'IS'"); # remove investment journal code from list
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    print "<h1>Error: Accessing database <font color=\"blue\">$ledger_db</font> failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to retrieve Journal Codes from ledger.<br>$sys_admin";
    CleanUp();
  }
  if ($result->ntuples == 0)
  {
    print "<h1>Error: No journals defined for the ledger $ledger_db!</h1>\n";
    print "$sys_admin";
    CleanUp();
  }
  for ($count=0; $count < $result->ntuples; $count++)
  {
    $journal_code_array[$count] = $result->getvalue($count, 0);
  }

  # get the payee/payers in the current fiscal year for remembered transactions from the ledger to fill in the payee_payer_array
  $result = $conn->exec("select payee_payer, memo, ref_num from journal_entry_tb where remember = true and trans_date between '$start_fiscal' and '$stop_fiscal' AND ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    print "<h1>Error: Accessing database <font color=\"blue\"$ledger_db</font> failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to get remembered payee/payers for fiscal year.<br>$sys_admin";
    CleanUp();
  }
  for ($count=0; $count < $result->ntuples; $count++)
  {
    # add payee/payers to payee_payer_array with R> in front and " - " and up to 8 characters of the memo field at the end
    $payee_payer_array[$count] = "R> " . $result->getvalue($count, 0) . " - " . substr($result->getvalue($count, 1), 0, 8);
    $payee_payer_ref_num_array[$count] = $result->getvalue($count, 2);
  }
  # get the distinct payee/payers from the current date back a year and a half for all most recently entered journal entries (recall data)
  $result = $conn->exec("select distinct payee_payer from journal_entry_tb where trans_date between '$date_ago' and '$date' AND ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    print "<h1>Error: Accessing database <font color=\"blue\"$ledger_db</font> failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to get unique payee/payers for one year and a half ago to now.<br>$sys_admin";
    CleanUp();
  }
  for ($count2=0; $count2 < $result->ntuples; $count++, $count2++)
  {
    $payee_payer_array[$count] = $result->getvalue($count2, 0);
    $temp_payee_payer = $payee_payer_array[$count];
    $temp_payee_payer =~ s/\\/\\\\/g;
    $temp_payee_payer =~ s/\'/\\\'/g;
    $temp_payee_payer =~ s/\$/\\\$/g;
    # get the maximum ref_num for the payee_payer entry
    $result2 = $conn->exec("select max(ref_num) from journal_entry_tb where payee_payer = '" . $temp_payee_payer . "' AND ledger_code = '$ledger_code'");
    if ($result2->resultStatus != PGRES_TUPLES_OK)
    {
      print "<h1>Error: Accessing database <font color=\"blue\"$ledger_db</font> failed!</h1>\n";
      print $conn->errorMessage . "<br>Tried to get max ref_num for distinct payee/payer " . $payee_payer_array[$count] . " for fiscal year.<br>$sys_admin";
      CleanUp();
    }
    if ($result2->ntuples != 1)
    {
      print "<h1>Error: No value returned!</h1>\n";
      print "Tried to get max ref_num for distinct payee/payer.  No record returned!<br>$sys_admin";
      CleanUp();
    }
    $payee_payer_ref_num_array[$count] = $result2->getvalue(0, 0);
    # find the most recently used journal code for the unique set of payee-payers in the database
    $result2 = $conn->exec("select journal_code from journal_entry_tb where ledger_code = '$ledger_code' and payee_payer = '$temp_payee_payer' and ref_num = $payee_payer_ref_num_array[$count]");
    if ($result2->resultStatus != PGRES_TUPLES_OK)
    {
      print "<h1>Error: Accessing the database failed!</h1>\n";
      print $conn->errorMessage . "<br>Tried to get the most recently used journal code for payee-payer: $temp_payee_payer.<br>$sys_admin";
      CleanUp();
    }
    if ($result2->ntuples <= 0)
    {
      print "<h1>Error: No records returned!</h1>\n";
      print "Tried to get the most recently used journal code for payee-payer: $temp_payee_payer.<br>$sys_admin";
      CleanUp();
    }
    $payee_payer_journal_code_array[$count] = $result2->getvalue(0, 0);
  }

  # start form
  print "<form method=post name=\"edit_trans_form\" action=\"$site_dir/cgi-bin/journal_activity_rows_sheet.cgi\">\n";
  # hidden ledger_name value
  print "<input type=hidden name=\"ledger_db\" value=\"$ledger_db\">\n";
  # hidden journal code value
  print "<input type=hidden name=\"journal_code\" value=\"$journal_code\">\n";
  # hidden transaction date value
  print "<input type=hidden name=\"trans_month\" value=\"$trans_month\">\n";
  print "<input type=hidden name=\"trans_day1\" value=\"$trans_day1\">\n";
  print "<input type=hidden name=\"trans_day2\" value=\"$trans_day2\">\n";
  print "<input type=hidden name=\"trans_year\" value=\"$trans_year\">\n";
  # hidden payee-payer value
  print "<input type=hidden name=\"payee_payer\" value=\"$payee_payer\">\n";
  # hidden memo value
  print "<input type=hidden name=\"memo\" value=\"$memo\">\n";
  # hidden mode for row script (edit or new)
  print "<input type=hidden name=\"mode\" value=\"new\">\n";

  # build table out of the data
  print "<table border=1 cellspacing=1 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
  print "<tr>\n";
  print "<th valign=top>Journal</th><th valign=top align=left>Date</th><th valign=top align=left>Payee-Payer</th>";
  print "</tr>\n";
  print "<tr nowrap>\n";
  # journal code select
  print "<td><select name=\"journal_code_select\">\n";
  for ($count=0; $count < @journal_code_array; $count++)
  {
    print "<option";
    if ($journal_code eq $journal_code_array[$count])
    {
      print " selected";
    }
    print ">" . $journal_code_array[$count] . "\n";
  }
  print "</select></td>\n";
  # date input field
  print "<td>\n";
  print "M:<input type=text size=2 maxlength=2 name=\"trans_month_text\" value=\"\" onChange=\"trans_month_text_change();\">";
  print "D:<input type=text size=2 maxlength=2 name=\"trans_day_text\" value=\"\" onChange=\"trans_day_text_change();\">";
  print "Y:<input type=text size=4 maxlength=4 name=\"trans_year_text\" value=\"\" onChange=\"trans_year_text_change();\">";
  print "&nbsp;<input type=text size=10 name=\"trans_date\" value=\"$trans_date\" onChange=\"trans_date_change()\">\n";
  print "</td>\n";
  # payee/payer select and input text field
  print "<td><input type=text name=\"payee_payer_text\" value=\"$payee_payer\" size=25 onChange=\"payee_payer_text_change();\"> <select name=\"payee_payer_select\" onChange=\"payee_payer_select_change();\">\n";
  print "<option value=\"-1\">---------------\n";
  for ($count=0; $count < @payee_payer_array; $count++)
  {
    print "<option value=\"$payee_payer_ref_num_array[$count]\"";
    if ($payee_payer eq $payee_payer_array[$count])
    {
      print " selected";
    }
    print ">" . $payee_payer_array[$count] . "\n";
  }
  print "</select></td>\n";
  print "</tr>\n";
  print "<tr>\n";
  print "<th align=left>R</th><th colspan=2 align=left>Memo</th>\n";
  print "</tr>\n";
  print "<tr>\n";
  # remembered check box
  print "<td><input type=checkbox name=\"remembered_check\" value=\"ON\"";
  if ($remember eq "t")
  {
    print " checked";
  }
  print "></td>\n";
  # memo text field
  print "<td colspan=2><input type=text name=\"memo_text\" value=\"$memo\" size=75></td>\n";
  print "</tr>\n";
  print "</table>\n";
  print "<center><input type=submit name=\"create_submit\" value=\"Continue\" onClick=\"return validate_form();\"><input type=reset></center>\n";
  my $temp_str = GetJournalViewInput();
  if (length $temp_str > 0)
  {
    print "$temp_str";
  }
  print "</form>\n";
  my $temp_str = GetJournalViewInput();
  if (length $temp_str > 0)
  {
    print "<form name=\"journal_view_form\" method=post action=\"$site_dir/cgi-bin/journal_mod1.cgi\">\n";
    print "$temp_str";
    print "<br><br><br><a href=\"$site_dir/cgi-bin/journal_mod1.cgi\" onClick=\"document.journal_view_form.submit(); return false;\">To return to the Journal View, click <u>here</u>.</a>\n";
    print "</form>\n";
  }
}

# print a horizontal rule to break the menu from the behind the scenes links
#print "<br><hr>\n";
print "</center>\n";

# adding the Javascript code for the most recently used journal code for payee-payers
if ($mode != 2)
{
  print "<script language=Javascript>\n";
  print "  <!--\n";
  print "  // Inserting values for journal_code_array...\n";
  my $match_found = 0;
  for ($count=0; $count < scalar @payee_payer_ref_num_array; $count++)
  {
    print "  journal_code_array[\"$payee_payer_ref_num_array[$count]\"] = ";
    # search for the journal code to assign the ref_num to
    for ($count2=0; $count2 < scalar @journal_code_array; $count2++)
    {
      if ($journal_code_array[$count2] eq $payee_payer_journal_code_array[$count])
      {
        print "$count2";
        $match_found = 1;
      }
    }
    if (not $match_found)
    {
      print "0";
    }
    else
    {
      $match_found = 0;
    }
    print ";\n";
  }
  print "  //-->\n";
  print "</script>\n";
}

print "</body>\n";
print "</html>\n";
exit 0;

sub GetJournalViewInput
{
  my ($mode, $date_type, $date_from, $date_to, $redisplay_string, $journal_name, $journal_code, $ref_num, $error_string, $payee_payer, $memo, $account_name, $account_num, $sub_acct_name, $sub_acct_num, $sub_acct_parent);
  my ($money_type, $money_search, $money1, $money2, $sort_by, $sort_journal, $num_entries_to_display, $start_index, $ledger_db, $check_num, $deposit_num, $sort_operator);
  my %valid_entries_to_display = ( 0 => "all", 5 => "5", 10 => "10", 15 => "15", 20 => "20", 25 => "25", 30 => "30", 35 => "35", 40 => "40", 45 => "45", 50 => "50" );
  my %valid_options = ( "open" => "open", "delete" => "delete", "add" => "add", "search" => "search", "search2" => "search2" );

  if (not exists $input{'journal_view_command'})
  {
    return "";  # nothing to do.
  }
  if (not exists $valid_options{$input{'journal_view_command'}})
  {
    PrintErrorHeader();
    print "<h1>(Journal View Re-display Code) Error:  Access Denied!</h1>\n";
    print "Invalid option given.  Option = '$input{'journal_view_command'}'.<br>$sys_admin";
    CleanUp();
  }

  $ledger_db = $input{'ledger'};
  if (length $ledger_db == 0)
  {
    PrintErrorHeader();
    print "<h1>(Journal View Re-display Code) Error: You must specify a ledger to work with!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  $error_string = IsValidString($ledger_db);
  if (length $error_string > 0)
  {
    PrintErrorHeader();
    print "<h1>(Journal View Re-display Code) Error:  Invalid Database Name!</h1>\n";
    print "<b>$error_string</b> is invalid!<br>$back_button";
    CleanUp();
  }
  $redisplay_string .= "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";

  # validate we got valid input from the user.
  if ($input{'journal_view_command'} eq "open")
  {
    $mode = 1;
    $redisplay_string .= "<input type=hidden name=\"journal_view_command\" value=\"open\">\n";
    if (not exists $input{'journal'})
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: You must specify a Journal to work with!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $redisplay_string .= "<input type=hidden name=\"journal\" value=\"" . $input{'journal'} . "\">\n";
    $date_from = $input{'date_from'};
    if (length $date_from == 0)
    {
      PrintErrorHeader();
      print "<h1>Error:  You must specify a From date!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $date_to = $input{'date_to'};
    if (length $date_to == 0)
    {
      PrintErrorHeader();
      print "<h1>Error:  You must specify a To date!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $sort_by = $input{'sort_by'};
    if (length $sort_by == 0)
    {
      PrintErrorHeader();
      print "<h1>Error:  You must specify a SortBy Option!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    if (not ($sort_by eq "date" || $sort_by eq "ref_num" || $sort_by eq "payee_payer"))
    {
      PrintErrorHeader();
      print "<h1>Error:  Invalid SortBy Option!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    if ($sort_by eq "date")
    {
      $sort_by = "trans_date";
    }
    if (exists $input{'sort_journal'})
    {
      $sort_journal = 1;
      $redisplay_string .= "<input type=hidden name=\"sort_journal\" value=\"1\">\n";
    }
    else
    {
      $sort_journal = 0;
    }
    $redisplay_string .= "<input type=hidden name=\"date_from\" value=\"$date_from\">\n<input type=hidden name=\"date_to\" value=\"$date_to\">\n<input type=hidden name=\"sort_by\" value=\"" . $input{'sort_by'} . "\">\n";
  }
  elsif ($input{'journal_view_command'} eq "search")
  {
    $mode = 4;
    # first make sure all the correct variables are being passed in.
    if (not exists $input{'date_type'})
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: Date Qualifier must be given!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $date_type = $input{'date_type'};
    if (not exists $input{'date_from_search'})
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: Date to search from must be given!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    if (not ($date_type eq "equal" || $date_type eq "betweeni" || $date_type eq "betweene" || $date_type eq "afteri" || $date_type eq "aftere" || $date_type eq "beforei" || $date_type eq "beforee"))
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  Invalid Date Qualifier Option!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $date_from = $input{'date_from_search'};
    if (not exists $input{'date_to_search'})
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: Date to search to must be given!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    if (length $date_from > 0 && (not IsValidDateSyntax($date_from)))
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  Invalid Date given!</h1>\n";
      print "The From date is invalid.<br>$back_button";
      CleanUp();
    }
    $date_to = $input{'date_to_search'};
    if (length $date_to > 0 && (not IsValidDateSyntax($date_to)))
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  Invalid Date given!</h1>\n";
      print "The To date is invalid.<br>$back_button";
      CleanUp();
    }
    $redisplay_string .= "<input type=hidden name=\"journal_view_command\" value=\"search\">\n";
    $redisplay_string .= "<input type=hidden name=\"date_type\" value=\"$date_type\">\n<input type=hidden name=\"date_from_search\" value=\"$date_from\">\n<input type=hidden name=\"date_to_search\" value=\"$date_to\">\n";
    if (not exists $input{'journal2'})
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: Journal selection must be given!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    ($journal_name, $journal_code) = split /\|/, $input{'journal2'};
    if (not exists $input{'ref_num2'})
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: Reference Number field must be given!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $ref_num = $input{'ref_num2'};
    $error_string = IsValidUInteger($ref_num);
    if (length $ref_num > 0 && length $error_string > 0)
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  Invalid Ref Number!</h1>\n";
      print "<b>$error_string</b> is invalid!<br>$back_button";
      CleanUp();
    }
    if (not exists $input{'check_num'})
    {
      PrintErrorHeader();
      print "<h1>Error: Check Number field must be given!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $check_num = $input{'check_num'};
    $error_string = IsValidUInteger($check_num);
    if (length $check_num > 0 && length $error_string > 0)
    {
      PrintErrorHeader();
      print "<h1>Error:  Invalid Check Number!</h1>\n";
      print "<b>$error_string</b> is invalid!<br>$back_button";
      CleanUp();
    }
    if (not exists $input{'deposit_num'})
    {
      PrintErrorHeader();
      print "<h1>Error: Deposit Number field must be given!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $deposit_num = $input{'deposit_num'};
    $error_string = IsValidUInteger($deposit_num);
    if (length $deposit_num > 0 && length $error_string > 0)
    {
      PrintErrorHeader();
      print "<h1>Error:  Invalid Deposit Number!</h1>\n";
      print "<b>$error_string</b> is invalid!<br>$back_button";
      CleanUp();
    }
    if (not exists $input{'payee_payer2'})
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: Payee-Payer selection must be given!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $payee_payer = $input{'payee_payer2'};
    $error_string = IsValidString($payee_payer);
    if (length $payee_payer > 0 && length $error_string > 0 && (not $error_string eq "_"))
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  Invalid Payee-Payer!</h1>\n";
      print "<b>$error_string</b> is invalid!<br>$back_button";
      CleanUp();
    }
    $redisplay_string .= "<input type=hidden name=\"journal2\" value=\"" . $input{'journal2'} . "\">\n<input type=hidden name=\"ref_num2\" value=\"$ref_num\">\n<input type=hidden name=\"check_num\" value=\"$check_num\">\n<input type=hidden name=\"deposit_num\" value=\"$deposit_num\">\n<input type=hidden name=\"payee_payer2\" value=\"$payee_payer\">\n";
    if (not exists $input{'memo2'})
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: Memo field must be given!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $memo = $input{'memo2'};
    $error_string = IsValidString($memo);
    if (length $memo > 0 && length $error_string > 0 && (not ($error_string eq "*" || $error_string eq "?")))
    {
      PrintErrorHeader();
      print "<h1(Journal View Re-display Code) Error:  Invalid Memo!</h1>\n";
      print "<b>$error_string</b> is invalid!<br>$back_button";
      CleanUp();
    }
    if (not exists $input{'sub_entity'})
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code)Error: Sub Entity selection must be given!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    if (not exists $input{'account'})
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: Account selection must be given!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    ($account_name, $account_num) = split /\|/, $input{'account'};
    if (not exists $input{'sub_account'})
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: Sub Account selection must be given!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    ($sub_acct_name, $sub_acct_num, $sub_acct_parent) = split /\|/, $input{'sub_account'};
    if (not exists $input{'dorc'})
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: Debit or Credit Selection must be given!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $money_type = $input{'dorc'};
    if (not ($money_type eq "select_dorc" || $money_type eq "debit" || $money_type eq "credit" || $money_type eq "both"))
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  Invalid Debit or Credit Selection type!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $redisplay_string .= "<input type=hidden name=\"memo2\" value=\"$memo\">\n<input type=hidden name=\"account\" value=\"" . $input{'account'} . "\">\n<input type=hidden name=\"sub_account\" value=\"" . $input{'sub_account'} . "\">\n";
    $redisplay_string .= "<input type=hidden name=\"dorc\" value=\"$money_type\">\n<input type=hidden name=\"sub_entity\" value=\"$input{sub_entity}\">\n";
    if (not exists $input{'dorc_search'})
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: Debit or Credit Search Type Selection must be given!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $money_search = $input{'dorc_search'};
    if (not ($money_search eq "all" || $money_search eq "equal" || $money_search eq "less_than" || $money_search eq "less_than_equal" || $money_search eq "greater_than" || $money_search eq "greater_than_equal" || $money_search eq "betweene" || $money_search eq "betweeni"))
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  Invalid Debit or Credit Search type!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $money1 = $input{'money1'};
    $error_string = IsValidInteger($money1);
    my $error_string2 = IsValidFloat($money1);
    if (length $money1 > 0)
    {
      if (length $error_string > 0 && (not $error_string eq "."))
      {
        PrintErrorHeader();
        print "<h1>(Journal View Re-display Code) Error:  Invalid First Money Field!</h1>\n";
        print "<b>$error_string</b> is invalid!<br>$back_button";
        CleanUp();
      }
      elsif (length $error_string > 0 && $error_string eq "." && length $error_string2 > 0)
      {
        PrintErrorHeader();
        print "<h1>(Journal View Re-display Code) Error:  Invalid First Money Field!</h1>\n";
        print "<b>$error_string</b> is invalid!<br>$back_button";
        CleanUp();
      }
    }
    $redisplay_string .= "<input type=hidden name=\"dorc_search\" value=\"$money_search\">\n<input type=hidden name=\"money1\" value=\"$money1\">\n<input type=hidden name=\"money2\" value=\"" . $input{'money2'} . "\">\n";
    $money2 = $input{'money2'};
    $error_string = IsValidInteger($money2);
    $error_string2 = IsValidFloat($money2);
    if (length $money2 > 0)
    {
      if (length $error_string > 0 && (not $error_string eq "."))
      {
        PrintErrorHeader();
        print "<h1>(Journal View Re-display Code) Error:  Invalid Second Money Field!</h1>\n";
        print "<b>$error_string</b> is invalid!<br>$back_button";
        CleanUp();
      }
      elsif (length $error_string > 0 && $error_string eq "." && length $error_string2 > 0)
      {
        PrintErrorHeader();
        print "<h1>(Journal View Re-display Code) Error:  Invalid Second Money Field!</h1>\n";
        print "<b>$error_string</b> is invalid!<br>$back_button";
        CleanUp();
      }
    }
    $sort_by = $input{'sort_by2'};
    if (length $sort_by == 0)
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  You must specify a SortBy Option!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    #print "\"$sort_by\"<br>";
    if (not ($sort_by eq "date" || $sort_by eq "ref_num" || $sort_by eq "payee_payer"))
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  Invalid SortBy Option!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    if ($sort_by eq "date")
    {
      $sort_by = "trans_date";
    }
    if (exists $input{'sort_journal'})
    {
      $sort_journal = 1;
      $redisplay_string .= "<input type=hidden name=\"sort_journal\" value=\"1\">\n";
    }
    else
    {
      $sort_journal = 0;
    }
    $sort_operator = $input{'sort_operator'};
    if (length $sort_operator == 0)
    {
      PrintErrorHeader();
      print "<h1>Error:  You must specify a Search Operator!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    if (not ($sort_operator eq "AND" || $sort_operator eq "OR"))
    {
      PrintErrorHeader();
      print "<h1>Error:  Invalid Search Operator!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $redisplay_string .= "<input type=hidden name=\"sort_by2\" value=\"" . $input{'sort_by2'} . "\">\n<input type=hidden name=\"sort_operator\" value=\"$sort_operator\">\n";

    # make sure they entered something in at least one field, but not something in both the debit and credit fields.
    if ((length $input{'date_from_search'} == 0) && (length $input{'date_to_search'} == 0) && ($input{'journal2'} eq "select_journal") && (length $input{'ref_num2'} == 0) && (length $input{'check_num'} == 0) && (length $input{'deposit_num'} == 0) && ($input{'payee_payer2'} eq "select_payee_payer") && (length $input{'memo2'} == 0) && ($input{'account'} eq "select_account") && ($input{'sub_account'} eq "select_sub_acct") && ($input{'dorc'} eq "select_dorc") && ($input{'dorc_search'} eq "all") && (length $input{'money1'} == 0) && (length $input{'money2'} == 0))
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: You must specify something for your search!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    # make sure that the date_to_search field is given only if the date_type is "Between".
    if ((length $input{'date_from_search'} > 0) && (length $input{'date_to_search'} > 0) && ((not $input{'date_type'} eq "betweeni") && (not $input{'date_type'} eq "betweene")))
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  You must specify <b>Between</b> to use the second Date field!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    # make sure that both money fields are not both filled in if they are not doing a between search or no Dorc value selected.
    if ((length $input{'money1'} > 0) && (length $input{'money2'} > 0) && ((not $input{'dorc'} eq "select_dorc") && (not $input{'dorc_search'} eq "betweene") && (not $input{'dorc_search'} eq "betweeni")))
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  You can not specify both money fields when not searching with between!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    if (($input{'dorc_search'} eq "betweene" || $input{'dorc_search'} eq "betweeni") && (length $input{'money1'} == 0 || length $input{'money2'} == 0))
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  You must specify both money fields when searching with between!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    if (((not $input{'dorc'} eq "select_dorc") && (not $input{'dorc_search'} eq "all")) && ((length $input{'money1'} == 0) && (length $input{'money2'} == 0)))
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  You must specify an amount to search for!</h1>\n";
      print "Error: 1007<br>$back_button";
      CleanUp();
    }
    # make sure that if they enter a Ref-Num that they don't select anything else.
    if ((length $input{'ref_num2'} > 0) && ((length $input{'date_from_search'} > 0) || (length $input{'date_to_search'} > 0) || (length $input{'check_num'} > 0) || (length $input{'deposit_num'} > 0) || (not $input{'payee_payer2'} eq "select_payee_payer") || (not $input{'journal2'} eq "select_journal") || (not $input{'account'} eq "select_account") || (not $input{'sub_account'} eq "select_sub_acct") || (not $input{'dorc'} eq "select_dorc") || (not $input{'dorc_search'} eq "all") || (length $input{'money1'} > 0) || (length $input{'money2'} > 0) || (length $input{'memo2'} > 0)))
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  You can not specify anything else when specifying the Reference Number!</h1>\n";
      print "$back_button";
      CleanUp();
    }
  }
  elsif ($input{'journal_view_command'} eq "search2")
  {
    $mode = 5;
    # first make sure all the correct variables are being passed in.
    if (not exists $input{'ref_num2'})
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: Reference Number field must be given!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $ref_num = $input{'ref_num2'};
    $error_string = IsValidUInteger($ref_num);
    if (length $ref_num > 0 && length $error_string > 0)
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  Invalid Ref Number!</h1>\n";
      print "<b>$error_string</b> is invalid!<br>$back_button";
      CleanUp();
    }
    $redisplay_string .= "<input type=hidden name=\"journal_view_command\" value=\"search2\">\n<input type=hidden name=\"ref_num2\" value=\"$ref_num\">\n";
  }

  if ($mode == 1 || $mode == 4 || $mode == 5)
  {
    $num_entries_to_display = $input{'num_entries_to_display'};
    if (length $num_entries_to_display == 0)
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: You must specify the number of Journal entries to display!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $error_string = IsValidInteger($num_entries_to_display);
    if (length $error_string > 0)
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  Invalid Value for Number of Entries to Display!</h1>\n";
      print "<b>$error_string</b> is invalid!<br>$back_button";
      CleanUp();
    }

    # verify it is a valid value
    if (not exists $valid_entries_to_display{$num_entries_to_display})
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  Invalid Value for Number of Entries to Display!</h1>\n";
      print "<b>$num_entries_to_display</b> is not valid!<br>$back_button";
      CleanUp();
    }

    $start_index = $input{'start_index'};
    if (length $start_index == 0)
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: You must specify the Starting Index to work from!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $error_string = IsValidInteger($start_index);
    if (length $error_string > 0)
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  Invalid Value for Start Index to display from!</h1>\n";
      print "<b>$error_string</b> is invalid!<br>$back_button";
      CleanUp();
    }
    if ($start_index < 1)
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  The Start Index can not be less than 1!</h1>\n";
      print "$sys_admin";
      CleanUp();
    }
    $redisplay_string .= "<input type=hidden name=\"num_entries_to_display\" value=\"$num_entries_to_display\">\n<input type=hidden name=\"start_index\" value=\"$start_index\">\n";
  }

  return $redisplay_string;  # return the string that will contain everything necessary to re-display the journal like how the person just left it.
}
