#!/usr/bin/perl
#$Id: modify_account.cgi,v 1.5 2003/02/20 23:28:05 pcxuser Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
# modify_account.cgi  -  Created by James Pattie, 05/22/1999
# This script allows the user to modify accounts.  It will either
# create an account if the create_account button exists or edit
# an account if the update_account button exists.

use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features
use reports;

use vars '$date';
my $error_string;  # holds the returned value from IsValidString();
my $report_obj;
my $ledger_code;
my $ledger_db;

my %restrictedAccounts = ( 4200 => "Short Term Capital Gain", 4201 => "Long Term Capital Gain", 5200 => "Short Term Capital Loss", 5201 => "Long Term Capital Loss", );

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";

    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";

    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

# verify we have valid data and determine the mode the program
# needs to be running in.  Create or Edit an account.
my $mode;  # 1=Create, 2=Edit
$ledger_db = $input{'ledger'};

if (not (exists $input{'create_account'} or exists $input{'update_account'}))
{
  PrintErrorHeader();
  print "<h1>Error:  Access denied!</h1>\n";
  print "Invalid options!<br>$sys_admin";
  CleanUp();
}

my ($acct_name, $acct_type, $acct_num, $acct_active, $old_acct_name, $acct_taxable);

if (exists $input{'create_account'})
{
  $mode = 1;
}
elsif (exists $input{'update_account'})
{
  $mode = 2;
  $old_acct_name = $input{'old_acct_name'};
  if (length $old_acct_name == 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Access denied!</h1>\n";
    print "Invalid options!<br>$sys_admin";
    CleanUp();
  }
  $error_string = IsValidString($old_acct_name);
  if (length $error_string != 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Old Account Name!</h1>\n";
    print "<b>$error_string</b> is invalid.<br>$back_button";
    CleanUp();
  }
}

# check and make sure we have an account name, valid account type, and that
# the account number starts with the account type digit.
$acct_name = $input{'acct_name'};
if (length $acct_name == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  You must specify an account name!</h1>\n";
  print "$back_button";
  CleanUp();
}
$error_string = IsValidString($acct_name);
if (length $error_string != 0)
{
  PrintErrorHeader();
  print "<h1>Error:  Invalid Account Name!</h1>\n";
  print "<b>$error_string</b> is invalid.<br>$back_button";
  CleanUp();
}
$acct_type = $input{'acct_type'};
if (length $acct_type == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  You must specify an account type!</h1>\n";
  print "$back_button";
  CleanUp();
}
$acct_num = $input{'acct_num'};
if (length $acct_num == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  You must enter an account number!</h1>\n";
  print "$back_button";
  CleanUp();
}
# make sure we are not working with any restricted accounts
if (exists $restrictedAccounts{$acct_num})
{
  PrintErrorHeader();
  print "<h1>Error:  You can not " . (exists $input{create_account} ? "create" : "edit") . " account = '$acct_num'!</h1>\n";
  print "<b>" . $restrictedAccounts{$acct_num} . "</b> is a restricted System Account.<br>\n";
  print $back_button;
  CleanUp();
}
$error_string = IsValidInteger($acct_num);
if (length $error_string != 0)
{
  PrintErrorHeader();
  print "<h1>Error:  Invalid Account Number!</h1>\n";
  print "<b>$error_string</b> is invalid.<br>$back_button";
  CleanUp();
}
if (exists $input{'acct_active'})
{
  $acct_active = $input{'acct_active'};
  if (not $acct_active eq "YES")
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Option for Account Active!</h1>\n";
    print "$sys_admin";
    CleanUp();
  }
  else
  {
    $acct_active = "true";
  }
}
else
{
  $acct_active = "false";
}

if (exists $input{'acct_taxable'})
{
  if (not $input{'acct_taxable'} eq "YES")
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Option for Account Taxable Status!</h1>\n";
    print "$sys_admin";
    CleanUp();
  }
  else
  {
    $acct_taxable = "true";
  }
}
else
{
  $acct_taxable = "false";
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# verify we are at database release 1.2.
$result = $conn->exec("select version from version_tb");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to check database version.<br>$sys_admin";
  CleanUp();
}
my $db_version;
$db_version = $result->getvalue(0, 0);
if ($db_version lt "1.2")
{
  PrintErrorHeader();
  print "<h1>Error: &nbsp;You need to upgrade your database to at least version <font color=\"red\">1.2</font>!</h1><br>$sys_admin";
  CleanUp();
}

if ($mode == 1)
{
  # first we check to see if the account doesn't exist for Creation or does exist for updating.
  $result = $conn->exec("select * from account_tb where name = '$acct_name'");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    PrintErrorHeader();
    print "<h1>Error:  Accessing database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to check for <font color=\"blue\">$acct_name</font>'s existence.<br>$sys_admin";
    CleanUp();
  }

  if ($result->ntuples != 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Account <font color=\"blue\">$acct_name</font> already exists!</h1>\n";
    print "$back_button";
    CleanUp();
  }

  # make sure the account number isn't already in use.
  $result = $conn->exec("select * from account_tb where number = '$acct_num'");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    PrintErrorHeader();
    print "<h1>Error:  Accessing database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to check for <font color=\"blue\">$acct_num</font>'s existence.<br>$sys_admin";
    CleanUp();
  }

  if ($result->ntuples != 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Account number <font color=\"blue\">$acct_num</font> is already in use!</h1>\n";
    print "$back_button";
    CleanUp();
  }
}
elsif ($mode == 2)
{
  # check to see if the account exists for updating.
  if ($acct_name eq $old_acct_name)
  { # they aren't changing the name
    $result = $conn->exec("select * from account_tb where name = '$acct_name' and number = '$acct_num'");
    if ($result->resultStatus != PGRES_TUPLES_OK)
    {
      PrintErrorHeader();
      print "<h1>Error:  Accessing database failed!</h1>\n";
      print $conn->errorMessage . "<br>Tried to check for <font color=\"blue\">$acct_name</font>'s existence.<br>$sys_admin";
      CleanUp();
    }
    if ($result->ntuples != 1)
    {
      PrintErrorHeader();
      print "<h1>Error:  Account <font color=\"blue\">$acct_name</font> with account number <font color=\"blue\">$acct_num</font> does not exist!</h1>\n";
      print "$sys_admin";
      CleanUp();
    }
  }
  else
  { # they are changing the name
    # first we validate that the old name and number is still valid.
    $result = $conn->exec("select * from account_tb where name = '$old_acct_name' and number = '$acct_num'");
    if ($result->resultStatus != PGRES_TUPLES_OK)
    {
      PrintErrorHeader();
      print "<h1>Error:  Accessing database failed!</h1>\n";
      print $conn->errorMessage . "<br>Tried to check for <font color=\"blue\">$old_acct_name</font>'s existence.<br>$sys_admin";
      CleanUp();
    }
    if ($result->ntuples != 1)
    {
      PrintErrorHeader();
      print "<h1>Error:  Account <font color=\"blue\">$old_acct_name</font> with account number <font color=\"blue\">$acct_num</font> does not exist!</h1>\n";
      print "$sys_admin";
      CleanUp();
    }

    # then we check to make sure that the new name isn't already in use.
    $result = $conn->exec("select * from account_tb where name = '$acct_name'");
    if ($result->resultStatus != PGRES_TUPLES_OK)
    {
      PrintErrorHeader();
      print "<h1>Error: Accessing database failed!</h1>\n";
      print $conn->errorMessage . "<br>Tried to check for <font color=\"blue\">$acct_name</font>'s existence.<br>$sys_admin";
      CleanUp();
    }
    if ($result->ntuples != 0)
    {
      PrintErrorHeader();
      print "<h1>Error:  Account name <font color=\"blue\">$acct_name</font> already exists!</h1>\n";
      print "$back_button";
      CleanUp();
    }
  }
}

# do a database lookup and get the size that an account must be in length for the number.
$result = $conn->exec("select acc_digits from version_tb");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get account number size info.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Internal Error: Application not configured!</h1>\n";
  print "Tried to get account number size info.<br>$sys_admin";
  CleanUp();
}

my $acct_num_size;
$acct_num_size = $result->getvalue(0, 0);

# check to make sure that the account number we have is the correct length.
if (length $acct_num == 0 || length $acct_num > $acct_num_size)
{
  PrintErrorHeader();
  print "<h1>Error:  The account number can not be more than <font color=\"blue\">$acct_num_size</font> digits long!</h1>\n";
  print "$back_button";
  CleanUp();
}

# do a lookup and verify that the account type is valid and that the starting digit is the same.
$result = $conn->exec("select name, first_digit from acct_type_tb where num = $acct_type");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get account types.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: Account Type <font color=\"blue\">$acct_type</font> is invalid!</h1>\n";
  print "Tried to get account types.<br>$sys_admin";
  CleanUp();
}
my $name;
$name = $result->getvalue(0, 0);
my $first;
$first = $result->getvalue(0, 1);

if (not $first eq substr($acct_num, 0, 1))
{
  PrintErrorHeader();
  print "<h1>Error:  The Account Number is not correct for the Account Type!</h1>\n";
  print "The account number should start with <font color=\"blue\">$first</font>.<br>$back_button";
  CleanUp();
}

print "<head><title>$prog_name $prog_version ";
if ($mode == 1)
{
  print "Creating Account</title>";
}
elsif ($mode == 2)
{
  print "Modifying Account</title>";
}
print <<"END_OF_CODE";
  <script language="Javascript1.2">
  <!--
    function SubmitForm()
    {
      document.assign_account_form.submit();
    }
  //-->
  </script>
END_OF_CODE
print "</head>\n";

print "<body bgcolor=$body_bgcolor>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td width=47%><br></td>\n";
if ($mode == 1)
{
  print "<td align=left><a href=\"$site_dir/help.html#create_account\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
}
elsif ($mode == 2)
{
  print "<td align=left><a href=\"$site_dir/help.html#modify_account\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
}
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardMainMenu($ledger_db);

# now we either create or update the account in the database.
if ($mode == 1)
{ # create account in database
  # start a transaction
  $result = $conn->exec("begin work");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Starting transaction failed!</h1>\n";
    print $conn->errorMessage . "<br>Before adding account to database.<br>$sys_admin";
    CleanUp();
  }
  else
  {
    $transaction = 1;
  }

  # now we create the account
  $result = $conn->exec("insert into account_tb (name, number, type, taxable, active) values ('$acct_name', '$acct_num', $acct_type, $acct_taxable, $acct_active)");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Creating Account failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to insert data into database.<br>$sys_admin";
    CleanUp();
  }

  # now we commit the work
  $result = $conn->exec("commit work");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Committing transaction failed!</h1>\n";
    print $conn->errorMessage . "<br>After inserting data.<br>$sys_admin";
    CleanUp();
  }

  print "<h1>Account <font color=\"blue\">$acct_name</font> Created Successfully!</h1><br><br>\n";

  # check and see if we have any ledgers created before we display this prompt.
  $result = $conn->exec("select * from entity_tb");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    print "<h1>Error:  Accessing database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to get count of ledgers.<br>$sys_admin";
    CleanUp();
  }
  if ($result->ntuples > 0)
  {
    print "<form name=\"assign_account_form\" method=post action=\"$site_dir/cgi-bin/assign_accounts2.cgi\" target=\"assign_account_window\">\n";
    print "<input type=hidden name=\"select_acct\" value=\"\">\n";
    print "<input type=hidden name=\"acct_num\" value=\"$acct_num\">\n";
    if (length $ledger_db == 0)
    {
      print "<a href=\"$site_dir/cgi-bin/assign_accounts.cgi\" onClick=\"SubmitForm(); return false;\">You need to assign this account to any Ledgers you would like to use it with. Go <u>here</u>.</a><br>\n";
    }
    else
    {
      print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
      print "<input type=hidden name=\"select_ledger\" value=\"\">\n";
      print "<a href=\"$site_dir/cgi-bin/assign_accounts2.cgi\" onClick=\"go_here(document.assign_account_form, '$site_dir/cgi-bin/assign_accounts2.cgi'); return false;\">To assign this account to Ledger ($ledger_db), go <u>here</u>.</a><br>\n";
    }
    print "</form>\n";
  }
  else
  {
    print "<a href=\"$site_dir/cgi-bin/main_menu.cgi\">You need to create a <u>Ledger</u> to be able to assign this account to</a><br>\n";
  }
  if (length $ledger_db == 0)
  {
    print "<a href=\"$site_dir/cgi-bin/create_account.cgi\">To create another Account go <u>here</u>.</a><br>\n";
  }
  else
  {
    print "<form name=\"assign_form2\" method=post action=\"$site_dir/cgi-bin/create_account.cgi\">\n";
    print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
    print "<a href=\"$site_dir/cgi-bin/create_account.cgi\" onClick=\"document.assign_form2.submit(); return false;\">To create another Account go <u>here</u>.</a><br>\n";
    print "</form>\n";
  }
}
elsif ($mode == 2)
{ # update account in database
  # start a transaction
  $result = $conn->exec("begin work");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Starting transaction failed!</h1>\n";
    print $conn->errorMessage . "<br>Before updating account in database.<br>$sys_admin";
    CleanUp();
  }
  else
  {
    $transaction = 1;
  }

  my $in_use = 0;  # flag to indicate if the account is currently in use.
  if ($acct_active eq "false")
  {
    # only let them make an account in-active, if it is not in use
    # to check this, we need to connect to each ledger and query the debitcredit_tb for this account number
    $result = $conn->exec("select database_name from entity_tb");
    if ($result->resultStatus != PGRES_TUPLES_OK)
    {
      print "<h1>Error:  Accessing database failed!</h1>\n";
      print $conn->errorMessage . "<br>Tried to get list of ledgers to work with.<br>$sys_admin";
      CleanUp();
    }
    if ($result->ntuples > 0)
    {
      my ($result2, $ledger_name, $i, $conn2);
      for ($i=0; $i < $result->ntuples; $i++)
      {
        $ledger_name = $result->getvalue($i, 0);
        # create the reports object
        $report_obj = reports->new($conn, $ledger_name);
        if ($report_obj->didErrorOccur())
        {
          PrintErrorHeader();
          print $report_obj->getErrorMessage();
          CleanUp();
        }
        $ledger_code = $report_obj->getLedgerCode();
        if ($report_obj->didErrorOccur())
        {
          PrintErrorHeader();
          print $report_obj->getErrorMessage();
          CleanUp();
        }

        # now query the debitcredit_tb for activity.
        $result2 = $conn->exec("SELECT COUNT(account_num) FROM debitcredit_tb WHERE account_num = $acct_num AND ledger_code = '$ledger_code'");
        if ($result2->resultStatus != PGRES_TUPLES_OK)
        {
          print "<h1>Error:  Accessing database failed!</h1>\n";
          print $conn->errorMessage . "<br>Tried to get count of transactions using Account ($acct_name) = $acct_num.<br>$sys_admin";
          CleanUp();
        }
        if ($result2->getvalue(0, 0) > 0)
        {
          $in_use = 1;
        }
      }
    }

    # now check and make sure that it does not have an entry in the initial_balances table.
    $result = $conn->exec("SELECT bal_date FROM acct_bal_tb WHERE acct_num = $acct_num");
    if ($result->resultStatus != PGRES_TUPLES_OK)
    {
      print "<h1>Error:  Accessing database failed!</h1>\n";
      print $conn->errorMessage . "<br>Tried to see if Account = '$acct_num' had any activity in the Initial Balances Table.<br>$sys_admin";
      CleanUp();
    }
    if ($result->ntuples > 0)
    {
      $in_use = 1;
    }

    if ($in_use)
    {
      # generate a warning message saying that the account is currently in use, but go ahead and allow them to change the name.
      print "Warning:  The Account is in use!  Disabling anyway.<br>\n";
      #$result = $conn->exec("update account_tb set name = '$acct_name' where number = '$acct_num'");
      $result = $conn->exec("update account_tb set name = '$acct_name', taxable = $acct_taxable, active = $acct_active where number = '$acct_num'");
      if ($result->resultStatus != PGRES_COMMAND_OK)
      {
        print "<h1>Error:  Updating Account failed!</h1>\n";
        print $conn->errorMessage . "<br>Tried to update data in database.<br>$sys_admin";
        CleanUp();
      }
    }
    else
    {
      $result = $conn->exec("update account_tb set name = '$acct_name', taxable = $acct_taxable, active = $acct_active where number = '$acct_num'");
      if ($result->resultStatus != PGRES_COMMAND_OK)
      {
        print "<h1>Error:  Updating Account failed!</h1>\n";
        print $conn->errorMessage . "<br>Tried to update data in database.<br>$sys_admin";
        CleanUp();
      }
    }
  }
  else
  {
    # now we update the account
    $result = $conn->exec("update account_tb set name = '$acct_name', taxable = $acct_taxable, active = $acct_active where number = '$acct_num'");
    if ($result->resultStatus != PGRES_COMMAND_OK)
    {
      print "<h1>Error:  Updating Account failed!</h1>\n";
      print $conn->errorMessage . "<br>Tried to update data in database.<br>$sys_admin";
      CleanUp();
    }

    # now we have to apply the taxable status to all local_account_tb entries for this account.
    $result = $conn->exec("update local_accounts_tb set taxable = $acct_taxable where acct_num = $acct_num");
    if ($result->resultStatus != PGRES_COMMAND_OK)
    {
      print "<h1>Error:  Updating Account Tax Status in Local Accounts Table failed!</h1>\n";
      print $conn->errorMessage . "<br>Tried to update data in database.<br>$sys_admin";
      CleanUp();
    }
  }
  # now we commit the work
  $result = $conn->exec("commit work");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Committing transaction failed!</h1>\n";
    print $conn->errorMessage . "<br>After updating data.<br>$sys_admin";
    CleanUp();
  }

  # if the account is made inactive, we need to remove all entries from all ledgers that use that account. (only if not in use)
  if ($acct_active eq "false" && (not $in_use))
  {
    # get the list of all ledgers from the main database
    $result = $conn->exec("select name, database_name from entity_tb");
    if ($result->resultStatus != PGRES_TUPLES_OK)
    {
      print "<h1>Error:  Accessing database failed!</h1>\n";
      print $conn->errorMessage . "<br>Tried to get list of all ledgers.<br>$sys_admin";
      CleanUp();
    }
    if ($result->ntuples > 0)
    {
      my $i;
      for ($i=0; $i < $result->ntuples; $i++)
      {
        my $ledger_name = $result->getvalue($i, 0);
        my $ledger_db = $result->getvalue($i, 1);
        # now we connect to the ledger and see if we have this account being used.
        # create the reports object
        $report_obj = reports->new($conn, $ledger_name);
        if ($report_obj->didErrorOccur())
        {
          PrintErrorHeader();
          print $report_obj->getErrorMessage();
          CleanUp();
        }
        $ledger_code = $report_obj->getLedgerCode();
        if ($report_obj->didErrorOccur())
        {
          PrintErrorHeader();
          print $report_obj->getErrorMessage();
          CleanUp();
        }

        my $result2 = $conn->exec("select * from local_accounts_tb where acct_num = $acct_num AND ledger_code = '$ledger_code'");
        if ($result2->resultStatus != PGRES_TUPLES_OK)
        {
          print "<h1>Error:  Accessing database <font color=\"blue\">$ledger_db</font> failed!</h1>\n";
          print $conn->errorMessage . "<br>Tried to check for account <font color=\"blue\">$acct_name</font>.<br>$sys_admin";
          CleanUp();
        }
        if ($result2->ntuples > 0)
        { # we have to remove the entries.
          $result2 = $conn->exec("begin work");  # start a transaction
          if ($result2->resultStatus != PGRES_COMMAND_OK)
          {
            print "<h1>Error:  Starting transaction with database <font color=\"blue\">$ledger_db</font> failed!</h1>\n";
            print $conn->errorMessage . "<br>$sys_admin";
            CleanUp();
          }
          $result2 = $conn->exec("delete from local_accounts_tb where acct_num = $acct_num");
          if ($result2->resultStatus != PGRES_COMMAND_OK)
          {
            print "<h1>Error:  Deleting account <font color=\"blue\">$acct_name</font> from database <font color=\"blue\">$ledger_db</font> failed!</h1>\n";
            print $conn->errorMessage . "<br>Tried to remove the assigned accounts.<br>$sys_admin";
            $result2 = $conn->exec("abort transaction");
            CleanUp();
          }
          $result2 = $conn->exec("commit work");
          if ($result2->resultStatus != PGRES_COMMAND_OK)
          {
            print "<h1>Error:  Committing work to database <font color=\"blue\">$ledger_db</font> failed!</h1>\n";
            print $conn->errorMessage . "<br>Tried to commit changes.<br>$sys_admin";
            $result2 = $conn->exec("abort transaction");
            CleanUp();
          }
        }
      }
    }
  }

  print "<h1>Account <font color=\"blue\">$acct_name</font> Updated Successfully!</h1><br><br>\n";

  print "<form name=\"assign_account_form\" method=post action=\"$site_dir/cgi-bin/edit_account2.cgi\">\n";
  print "<input type=hidden name=\"select_acct\" value=\"\">\n";
  print "<input type=hidden name=\"acct_name\" value=\"$acct_num - $acct_name\">\n";
  if (length $ledger_db > 0)
  {
    print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
  }
  print "<a href=\"$site_dir/cgi-bin/edit_account2.cgi\" onClick=\"SubmitForm(); return false;\">To re-edit this account, go <u>here</u>.</a><br>\n";
  print "</form>\n";
}

# print a horizontal rule to break the menu from the behind the scenes links
print "<br><hr>\n";

if (length $ledger_db == 0)
{
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/account_config.cgi\">Account Configuration</a></div>\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/prg_config.cgi\">Program Configuration</a></div>\n";
}
else
{
  print "<form name=\"form1\" method=post action=\"\">\n";
  print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/account_config.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/account_config.cgi'); return false;\">Account Configuration</a></div>\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/prg_config.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/prg_config.cgi'); return false;\">Program Configuration</a></div>\n";
  print "<input type=hidden name=\"open\" value=\"Open\">\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/ledger.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/ledger.cgi'); return false;\">Return to Ledger Summary</a></div>\n";
  print "</form>\n";
}

print "</body>\n";
print "</html>\n";
exit 0;
