#!/usr/bin/perl

# register.cgi  -  Created by Jason A. Pattie, 10/18/1999
# This script asks the user for the account and the range of dates they want
# to view for the account.

use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features
use reports;

use vars '$date';
use vars '$current_month';
use vars '$current_year';
use vars '$last_month_start_date';
use vars '$last_month_end_date';
use vars '$count';
use vars '$account_numbers_string';
use vars '@account_name_array';
use vars '@bank_account_array';

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

my $ledger_name;  # name of ledger to work with
my $ledger_db;
my $ledger_color;
my $result2;
my $error_string;
my $ledger_code;
my $report_obj;

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

$ledger_db = $input{'ledger'};
if (length $ledger_db == 0)
{
  PrintErrorHeader();
  print "<h1>Error: You must specify a ledger to work with!</h1>\n";
  print "$back_button";
  CleanUp();
}
$error_string = IsValidString($ledger_db);
if (length $error_string > 0)
{
  PrintErrorHeader();
  print "<h1>Error:  Invalid Database Name!</h1>\n";
  print "<b>$error_string</b> is invalid!<br>$back_button";
  CleanUp();
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# now validate that the ledger they specified is valid
$result = $conn->exec("select name from entity_tb where database_name = '$ledger_db'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Trying to check entity table.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: Ledger <font color=\"blue\">$ledger_db</font> does not exist!</h1>\n";
  print "$sys_admin";
  CleanUp();
}
else
{
  $ledger_name = $result->getvalue(0, 0);
}

# create the reports object
$report_obj = reports->new($conn, $ledger_db);
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_code = $report_obj->getLedgerCode();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_color = $report_obj->getLedgerColor();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}

# get the list of accounts from the database
# fill the account_name_array
$result2 = $conn->exec("select name, number, type from account_tb where number in (select acct_num from local_accounts_tb WHERE ledger_code = '$ledger_code') order by name");
if ($result2->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get the name for account " . $result->getvalue($count, 0) . ".<br>$sys_admin";
  CleanUp();
}
my $count2;
$count2 = 0;
for ($count=0; $count < $result2->ntuples; $count++)
{
  $account_name_array[$count] = $result2->getvalue($count, 0) . " - " . $result2->getvalue($count, 1);
  if ($result2->getvalue($count, 2) eq "0")
  {
    $bank_account_array[$count2] = $count;
    $count2++;
  }
}

# figure out the beginning and ending date of the current working year
my ($working_year, $from_date, $to_date);

$result = $conn->exec("select year, start_date, end_date from known_years_tb where year in (select working_year from variables_tb WHERE ledger_code = '$ledger_code') AND ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get working year and fiscal year dates from ledger <font color=\"blue\">$ledger_db</font>.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: Program not configured!</h1>\n";
  print "Working Year is not set!<br>$sys_admin";
  CleanUp();
}
$working_year = $result->getvalue(0, 0);
$from_date    = $result->getvalue(0, 1);
$to_date      = $result->getvalue(0, 2);

print "<head><title>$prog_name $prog_version Account Registers</title>\n";
print <<"END_OF_SCRIPT";
  <script language="JavaScript1.2">

  function bank_account_select_change()
  {
    // set the account selected index to 0 to "unselect" any selected account
    document.register_selection_form.account_select.selectedIndex = 0;
  }

  function account_select_change()
  {
    // set the bank selected index to -1 to unselect any selected account
    document.register_selection_form.bank_account_select.selectedIndex = -1;
  }

  function ValidateRegisterForm()
  {
    var error_occurred = false;
    var msg = "";
    if (document.register_selection_form.from_search_date.value.length == 0)
    {
      error_occurred = true;
      msg += "You must specify the From Date!\\n";
    }
    var tmp = IsValidDateSyntax(document.register_selection_form.from_search_date.value);
    if (tmp.length > 0)
    {
      error_occurred = true;
      msg += "The From Date is Invalid!\\n" + tmp + "\\n\\n";
    }
    if (document.register_selection_form.to_search_date.value.length == 0)
    {
      error_occurred = true;
      msg += "You must specify the To Date!\\n";
    }
    tmp = IsValidDateSyntax(document.register_selection_form.to_search_date.value);
    if (tmp.length > 0)
    {
      error_occurred = true;
      msg += "The To Date is Invalid!\\n" + tmp + "\\n\\n";
    }
    if (document.register_selection_form.bank_account_select.selectedIndex == -1 && document.register_selection_form.account_select.selectedIndex == 0)
    {
      error_occurred = true;
      msg += "You must select an account!\\n";
    }
    if (error_occurred)
    {
      alert(msg);
      return false;
    }
    // set the account_value hidden form element to the selected account
    if (document.register_selection_form.bank_account_select.selectedIndex != -1)
    {
      // bank account was selected
      document.register_selection_form.account_value.value = document.register_selection_form.bank_account_select.options[document.register_selection_form.bank_account_select.selectedIndex].value;
    }
    else
    {
      // normal account dropdown with all accounts was selected
      document.register_selection_form.account_value.value = document.register_selection_form.account_select.options[document.register_selection_form.account_select.selectedIndex].value;
    }
    return true;
  }
  </script>
END_OF_SCRIPT
print "</head>\n";
print "<body bgcolor=$body_bgcolor>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

# get the current month and year
$current_month = `/bin/date +%m`;
chomp $current_month;
$current_year = `/bin/date +%Y`;
chomp $current_year;

# build the previous month's start and end dates
$last_month_start_date = `/bin/date -d "$current_month/01/$current_year 1 month ago" +%m-%d-%Y`;
chomp $last_month_start_date;
$last_month_end_date = `/bin/date -d "$current_month/01/$current_year 1 day ago" +%m-%d-%Y`;
chomp $last_month_end_date;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td align=left width=47%><font color=\"red\">";
print "Working in Ledger <font color=\"$ledger_color\">$ledger_name</font>. &nbsp;FY is <font color=\"black\">$working_year</font>.";
print "</td>\n";
print "<td align=left><a href=\"$site_dir/help.html#register\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print "<h2>Register Selection</h2>\n";
print "<form method=post name=\"register_selection_form\" action=\"$site_dir/cgi-bin/register_display.cgi\" onSubmit=\"return ValidateRegisterForm();\">\n";
print "<input type=hidden name=\"ledger_db\" value=\"$ledger_db\">\n";
print "<table border=0 cellspacing=0 cellpadding=$table_cellpadding bgcolor=\"$menu_bgcolor\">\n";
print "<tr>\n";
print "<th>Account Name</th><th colspan=2>Date Range</th>\n";
print "</tr>\n";
print "<tr>\n";
print "<td><br></td><th>From</th><th>To</th>\n";
print "</tr>\n";
print "<tr valign=top>\n";
print "<td valign=top nowrap>";
print "All Accounts<br><select name=\"account_select\" onChange=\"account_select_change();\">\n";
print "<option value=\"\">---------------\n";
for ($count=0; $count < @account_name_array; $count++)
{
  my $temp = $account_name_array[$count];
  $temp =~ s/ - /\|/g;
  print "<option value=\"$temp\">$account_name_array[$count]\n";
}
print "</select><input type=hidden name=\"account_value\" value=\"\">\n";
if ((scalar @bank_account_array) > 0)
{
  print "<br>Cash Accounts<br><select name=\"bank_account_select\" size=5 onChange=\"bank_account_select_change();\">\n";
  my $count2 = 0;
  for ($count=0; $count < @bank_account_array; $count++)
  {
    my $temp = $account_name_array[$bank_account_array[$count]];
    $temp =~ s/ - /\|/g;
    print "<option value=\"$temp\"";
    if ($count2 == 0)
    {
      print " selected";
    }
    print ">$account_name_array[$bank_account_array[$count]]\n";
    $count2++;
  }
  print "</select>&nbsp;\n";
}
else
{
  print "<input type=hidden name=\"bank_account_select\" value=\"\">\n";
  print "<script language=Javascript1.2>\n";
  print "  document.register_selection_form.bank_account_select.selectedIndex = -1;\n";
  print "</script>\n";
}
print "</td>\n";
print "<td valign=top><input type=text name=\"from_search_date\" value=\"$last_month_start_date\" size=10 maxlength=10></td>\n";
print "<td valign=top><input type=text name=\"to_search_date\" value=\"$last_month_end_date\" size=10 maxlength=10></td>\n";
print "</tr>\n";
print "<tr>\n";
print "<td colspan=3 align=left><input type=checkbox name=\"display_outstanding\" checked> Reconcile</td>\n";
print "</tr>\n";
print "<tr>\n";
print "<td align=right colspan=3><input type=submit name=\"register_select_submit\" value=\"Display Account Register\"><input type=reset></td>\n";
print "</tr>\n";
print "</table>\n";
print "</form>\n";
print "<form method=post action=\"$site_dir/cgi-bin/journal_activity.cgi\" target=\"journal_window\" name=\"registers_form\">\n";
print "<br><br>\n";
print "<b>NOTE:</b> Make sure to enter service charges and any extra debits or credits not yet entered (i.e., interest, fees, etc.) in the Journal. <input type=submit name=\"new_trans_input\" value=\"New Transaction\">\n";
print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
print "<input type=hidden name=\"journal_command\" value=\"create\">\n";
print "<input type=hidden name=\"journal_code\" value=\"all_journals\">\n";
print "</form>\n";
# print a horizontal rule
#print "<br><hr>\n";
print "</center>\n";

print "</body>\n";
print "</html>\n";
exit 0;
