#!/usr/bin/perl
#$Id: report_detailed_view.cgi,v 1.1 2001/09/06 10:21:20 moreejt Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
# report_detailed_view.cgi  -  Created by James Pattie, 12/15/1999
# This script displays the data based upon the criteria given by the user.
use strict;
no strict 'refs';

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features
use reports;  # the Report Module

use vars '$date';
use vars '$ledger_db';
use vars '$ledger_name';
use vars '$report';
use vars '$report_name';
use vars '$report_obj';  # The report Object
use vars '$print_report';  # Boolean to specify if the report is to be printable or not.
use vars '$show_zero_balances'; # boolean to specify if we should show activity / balances that = 0.00
use vars '$show_accounts_as';  # boolean: 1 = Name - Number, 0 = Number - Name
use vars '$show_by_sub_entities'; # boolean: 1 = show by Sub-Entities, 0 = Normal report.
use vars '$show_non_tax_section'; # boolean: 1 = show Non-Tax sectionm, 0 = Don't show Non-Tax section.

use vars '$data_precision';  # indicates what level of precision to display the data in.

use vars '$custom_title';
use vars '$error_message';  # global error message string
use vars '$ledger_color';
use vars '$date_range_string';
use vars '$precision_table';
$precision_table = { '10' => 'tens', '100' => 'hundreds', '1000' => 'thousands', '10000' => 'ten-thousands', '100000' => 'hundred-thousands', '1000000' => 'millions' };
my ($i, $acct_num, $result2, $result3);  # result2 will hold the account types, result3 will hold the other data.
my ($text_width, $data_width, $max_columns, $font_size);
$text_width = 30;  # width's in % of the text and data columns of printable / detailed view reports.
$data_width = 70;
$font_size = 2;  # the number of points to decrease the font by.
$max_columns = 4;  # the maximum number of data columns to be displayed at any time.
my $line_length = 12; # length of the line that should be generated under any text that needs it.
my $font_str = "Helvetica";
my @beginning_balances;
my $ledger_code;

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

$ledger_db = $input{'ledger'};
if (length $ledger_db == 0)
{
  PrintErrorHeader();
  print "<h1>Error: You must specify a ledger to work with!</h1>\n";
  print "$back_button";
  CleanUp();
}

# Global Values set by every report
$report = $input{'report'};
if (length $report == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  You must specify the name of the Report we are working with!</h1>\n";
  print "$back_button";
  CleanUp();
}
$report_name = $input{'report_name'};
if (length $report_name == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  You must specify the Report Name to display!</h1>\n";
  print "$back_button";
  CleanUp();
}

$custom_title = $input{'custom_report_name'};

# see what they specified for Show Activities/Balances = $0.00
if (not exists $input{'show_zero_balances'})
{
  PrintErrorHeader();
  print "<h1>Error:  Show Zero Activities/Balances must be set!</h1>\n";
  print "$sys_admin";
  CleanUp();
}
$show_zero_balances = $input{'show_zero_balances'};
if (not ($show_zero_balances eq "YES" || $show_zero_balances eq "NO"))
{
  PrintErrorHeader();
  print "<h1>Error:  Invalid value set for Show Zero Activities/Balances!</h1>\n";
  print "The value specified was '$show_zero_balances'.<br>$sys_admin";
  CleanUp();
}
$show_zero_balances = ($show_zero_balances eq "YES" ? 1:0);

# see what they specified for Show Account Names: Name - Number or Number - Name
if (not exists $input{'show_accounts_as'})
{
  PrintErrorHeader();
  print "<h1>Error:  Show Accounts as Name - Number or Number - Name must be set!</h1>\n";
  print "$sys_admin";
  CleanUp();
}
$show_accounts_as = $input{'show_accounts_as'};
if (not ($show_accounts_as eq "name-number" || $show_accounts_as eq "number-name"))
{
  PrintErrorHeader();
  print "<h1>Error:  Invalid value set for Show Accounts as!</h1>\n";
  print "The value specified was '$show_accounts_as'.<br>$sys_admin";
  CleanUp();
}
$show_accounts_as = ($show_accounts_as eq "name-number" ? 1:0);

# see what value they specified for the data precision.
if (length $input{'data_precision'} == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  You must specify the precision to use for the data!</h1>\n";
  print "$back_button";
  CleanUp();
}
$data_precision = $input{'data_precision'};
if (not ($data_precision == 1 || $data_precision == 10 || $data_precision == 100 || $data_precision == 1000 || $data_precision == 10000 || $data_precision == 100000 || $data_precision == 1000000))
{
  PrintErrorHeader();
  print "<h1>Error: <b>$data_precision</b> is an invalid value for the data precision!</h1>\n";
  print "$back_button";
  CleanUp();
}

my (@Acct_Type, @Account_Info, @Sub_Acct_Info, @SubEntity, $start_date, $end_date, $header_string);

# now gather the information that will be used to actually do the data lookup with.
if (length $input{'acct_type_num'} == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  You must specify the Account Type to work with!</h1>\n$back_button";
  CleanUp();
}
$Acct_Type[0] = $input{'acct_type_num'};

if (length $input{'acct_num'} == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  You must specify the Account Number to work with!</h1>\n$back_button";
  CleanUp();
}
$Account_Info[0] = $input{'acct_num'};

if (length $input{'acct_name'} == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  You must specify the Account Name to work with!</h1>\n$back_button";
  CleanUp();
}
$Account_Info[2] = $input{'acct_name'};

if (length $input{'debit_increases'} == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  You must specify the Debit Increases value to work with!</h1>\n$back_button";
  CleanUp();
}
$Account_Info[1] = $input{'debit_increases'};

if (length $input{'sub_acct_name'} == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  You must specify the Sub Account Name to work with!</h1>\n$back_button";
  CleanUp();
}
$Sub_Acct_Info[1] = $input{'sub_acct_name'};

if (length $input{'sub_acct_num'} == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  You must specify the Sub Account Number to work with!</h1>\n$back_button";
  CleanUp();
}
$Sub_Acct_Info[0] = $input{'sub_acct_num'};

if (length $input{'start_date'} == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  You must specify the Start Date to work with!</h1>\n$back_button";
  CleanUp();
}
$start_date = $input{'start_date'};

if (length $input{'end_date'} == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  You must specify the End Date to work with!</h1>\n$back_button";
  CleanUp();
}
$end_date = $input{'end_date'};

if (length $input{'header_string'} == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  You must specify the Header String to work with!</h1>\n$back_button";
  CleanUp();
}
$header_string = $input{'header_string'};

if (length $input{'sub_entity_code'} > 0 && length $input{'sub_entity_name'} > 0)
{
  $SubEntity[0] = $input{'sub_entity_code'};
  $SubEntity[1] = $input{'sub_entity_name'};
}
else
{
  @SubEntity = ();
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

$result = $conn->exec("select name from entity_tb where database_name = '$ledger_db'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print "Trying to check entity table.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: Ledger <font color=\"blue\">$ledger_db</font> does not exist!</h1>\n";
  print "$sys_admin";
  CleanUp();
}
$ledger_name = $result->getvalue(0, 0);

# create the reports object
$report_obj = reports->new($conn, $ledger_db);
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_code = $report_obj->getLedgerCode();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_color = $report_obj->getLedgerColor();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}

if ($report eq "balance_sheet")
{
  # get the Beginning Balance upto the start_date
  my @Accounts = ();
  $Accounts[0] = [ @Account_Info ];
  @beginning_balances = $report_obj->GetAccountBalances(\@Accounts, $start_date);
  if ($report_obj->didErrorOccur)
  {
    PrintErrorHeader();
    print "<h1>Error:  Getting Initial Balance for Account failed!</h1>\n";
    print $report_obj->getErrorMessage;
    CleanUp();
  }
}

print "<head><title>$prog_name $prog_version - Detailed Data for '$report_name' Report</title>\n";

print <<"END_OF_SCRIPT";
<script language=JavaScript1.2>
function SubmitViewForm(ref_num)
{
  document.Detailed_View.ref_num2.value = ref_num;
  document.Detailed_View.submit();
  return false;
}
</script>
END_OF_SCRIPT

print "</head>\n";
print "<body bgcolor=$body_bgcolor>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: blue }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language=\"Javascript1.2\" src=\"../js/common.js\"></script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

# display the header string for this ledger, etc.
print "$header_string\n";

# display the selected reports results
# now we open a table and start displaying the results in it.
print "<table border=0 cellspacing=0 cellpadding=0 width=90%>\n";
if ((not $SubEntity[0] eq "-2") && length $SubEntity[1] > 0)
{
  print "<tr><td align=left><h3>Sub Entity = <font color=red>$SubEntity[1]</font> ($SubEntity[0])</h3></td></tr>\n";
  if ($SubEntity[0] eq " " && $SubEntity[1] eq "<i>None</i>")
  {
    $SubEntity[0] = "";
    $SubEntity[1] = "";
  }
}
if ($Sub_Acct_Info[0] != -2)
{
  print "<tr><td align=left><h3>Sub Account = <font color=red>$Sub_Acct_Info[1]</font> ($Sub_Acct_Info[0])</h3></td></tr>\n";
}
print "<tr><td align=left><h4>Account = <font color=red>$Account_Info[2]</font> ($Account_Info[0])</h4></td></tr>\n";
print "<tr><td align=left><h4>From <font color=red>$start_date</font> To <font color=red>$end_date</font></h4></td></tr>\n";
if ($report eq "balance_sheet")
{
  print "<tr><td align=left><h4>Beginning Balance = \$" . commify($beginning_balances[0][0]) . "</h4></td></tr>\n";
}
print "</table>\n";

if ($Sub_Acct_Info[0] == -2)
{
  @Sub_Acct_Info = ();  # we are not doing a Sub Account search.
}

my @entries;
@entries = $report_obj->GetActivityData(\@Account_Info, \@Sub_Acct_Info, \@SubEntity, $start_date, $end_date);
if ($report_obj->didErrorOccur)
{
  print $report_obj->getErrorMessage;
  CleanUp();
}

if (scalar @entries == 0)
{
  if (scalar @Sub_Acct_Info > 0)
  {
    print "<h1>Error:  Oops!  There doesn't appear to be any data in the database that matches the criterion given!</h1>\n$sys_admin";
  }
  else
  {
    print "<h1>There are no records to display!</h1>\n";
  }
  CleanUp();
}

print "<form method=post action=\"$site_dir/cgi-bin/journal_mod1.cgi\" name=\"Detailed_View\">\n";
print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
print "<input type=hidden name=\"journal_view_command\" value=\"search2\">\n";
print "<input type=hidden name=\"ref_num2\" value=\"refnum\">\n";
print "<input type=hidden name=\"num_entries_to_display\" value=\"10\">\n";
print "<input type=hidden name=\"start_index\" value=\"1\">\n";
print "<table border=1 cellspacing=1 cellpadding=2 width=90%>\n";
print "<tr><th>Ref Num</th><th>Jnl</th><th>Date</th><th>Payee-Payer</th><th>Memo</th><th>SE</th><th>Debit</th><th>Credit</th></tr>\n";

my ($i);
#print "<tr><td colspan=8>There are " . int (scalar @entries) . " entries returned.</td></tr>\n";
for ($i=0; $i < scalar @entries; $i++)
{
  print "<tr><td align=center><a href=\"\" onClick=\"SubmitViewForm('$entries[$i][0]'); return false;\">$entries[$i][0]</a></td>\n";
  print "    <td align=center>$entries[$i][12]</td>\n";
  print "    <td align=right>$entries[$i][9]</td>\n";
  print "    <td align=center>";
  if (length $entries[$i][10] == 0)
  {
    print "&nbsp;";
  }
  else
  {
    print "$entries[$i][10]";
  }
  print "</td>\n";
  print "    <td align=center>";
  if (length $entries[$i][11] == 0)
  {
    print "&nbsp;";
  }
  else
  {
    print "$entries[$i][11]";
  }
  print "</td>\n";
  print "    <td align=center>";
  if (length $entries[$i][1] == 0)
  {
    print "&nbsp;";
  }
  else
  {
    print "$entries[$i][1]";
  }
  print "</td>\n";
  if ($entries[$i][6] eq "t")  # Debit
  {
    print "    <td align=right>$entries[$i][5]</td>\n    <td align=right>&nbsp;</td>\n";
  }
  else  # Credit
  {
    print "    <td align=right>&nbsp;</td>\n    <td align=right>$entries[$i][5]</td>\n";
  }
  print "</tr>\n";
}

print "</table>\n";
print "</form>\n";

print "</body>\n";
print "</html>\n";
exit 0;