#!/usr/bin/perl
#$Id: sub_accounts_config3.cgi,v 1.5 2004/06/18 19:43:09 pattieja Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
# sub_accounts_config3.cgi  -  Created by James Pattie, 06/22/1999
# This script either displays the create new sub account screen,
# the edit sub account screen or actually deletes
# the specified sub account based upon the users actions.
# It may also present the user the Assign SubAccount to Accounts screen.

use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features

use vars '$date';
my $error_string;
my $onLoad;
$onLoad = "";
my $ledger_db;
my %commands;
%commands = ( "create" => "create", "createStock" => "createStock", "edit" => "edit", "delete" => "delete" );
my ($account_name, $account_num);
my $count;

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";

    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";

    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

$ledger_db = $input{'ledger'};

# check and make sure that we have either the edit, create, or delete
# buttons being passed in.
if (not exists $input{'submitted_by'})
{
  PrintErrorHeader();
  print "<h1>Error:  Access Denied!</h1>\n";
  print "Invalid command given.<br>$sys_admin";
  CleanUp();
}
if (not exists $commands{$input{'submitted_by'}})
{
  PrintErrorHeader();
  print "<h1>Error:  Access Denied!</h1>\n";
  print "Invalid option given.<br>$sys_admin";
  CleanUp();
}

my $mode;

if ($input{'submitted_by'} eq "edit")
{
  $mode = 1;
}
elsif ($input{'submitted_by'} eq "create")
{
  $mode = 2;
}
elsif ($input{'submitted_by'} eq "delete")
{
  $mode = 3;
}
elsif ($input{'submitted_by'} eq "createStock")
{
  $mode = 4;
}

my $subAcct = $input{sub_acct};

my $edit_acct;

if ($mode == 1 or $mode == 3)
{ # check and make sure we have some sub-accounts to work with
  if (length $subAcct == 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  You must specify a sub account to edit or delete!</h1>\n";
    print "$sys_admin";
    CleanUp();
  }
  if ($mode == 1)
  {
    if (not exists $input{'sub_acct'})
    {
      PrintErrorHeader();
      print "<h1>Error:  You must select a sub account to edit!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    my ($tmp_num, $tmp_e) = split /\|/, $subAcct;
    if (length $tmp_num == 0 || length $tmp_e == 0)
    {
      PrintErrorHeader();
      print "<h1>Error:  Invalid sub account to edit!</h1>\n";
      print "$sys_admin";
      CleanUp();
    }
    $error_string = IsValidInteger($tmp_num);
    if (length $error_string != 0)
    {
      PrintErrorHeader();
      print "<h1>Error:  Invalid Sub Account Number!</h1>\n";
      print "<b>$error_string</b> is invalid.<br>$back_button";
      CleanUp();
    }
    $error_string = IsValidString($tmp_e);
    if (length $error_string != 0)
    {
      PrintErrorHeader();
      print "<h1>Error:  Invalid Sub Account Name!</h1>\n";
      print "<b>$error_string</b> is invalid.<br>$back_button";
      CleanUp();
    }
  }
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# need to get the max length for a sub account number
$result = $conn->exec("select sub_acc_digits from version_tb");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get length of a sub account from database.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  This software does not appear to be properly configured!</h1>\n";
  print "Tried to get length of a sub account from database.<br>$sys_admin";
  CleanUp();
}
my $sub_acct_length;
$sub_acct_length = $result->getvalue(0, 0);

# gather the market values to use.
my @markets;
@markets = getStockMarkets();

print "<head><title>$prog_name $prog_version Work with Sub Accounts</title>\n";

if ($mode == 3) # delete
{
  print <<"END_OF_CODE";
  <script language="Javascript1.2">
  <!--
    function SubmitForm2()
    {
      document.sub_acct_form.submit();
    }
    function doTimer()
    {
      setTimeout("SubmitForm2();", 3000);  // 3 seconds
    }
  //-->
  </script>
END_OF_CODE
  $onLoad = " onLoad=doTimer();";
}

print "</head>\n";
print "<body bgcolor=$body_bgcolor$onLoad>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
<script language=\"Javascript1.2\">
  <!--
  function SubmitForm()
  {
    document.sub_acct_admin.submit();
  }

END_OF_STYLE
if ($mode == 1 || $mode == 4)
{
  print <<"END_OF_MATCH_CODE";
  function matchMarket(market)
  {
    var f = document.sub_acc_form.market;
    var count = 0;
    var found = 0;
    while (!found && count < f.options.length)
    {
      var searchPattern = new RegExp('^' + f.options[count].value + '\$', 'i');
      if (market.search(searchPattern) != -1)
      {
        found = 1;
        f.selectedIndex = count;
      }
      count++;
    }
    if (!found)
    {
      // create new market entry in select field
      var newMarketOption = new Option(market, market, false, true);
      f.options[f.options.length] = newMarketOption;
    }
  }

END_OF_MATCH_CODE
}
print <<"END_OF_STYLE";
  //-->
</script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td width=47%>&nbsp;</td>\n";
print "<td align=left><a href=\"$site_dir/help.html#sub_accounts_work\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardMainMenu($ledger_db);

# create the form that lets them update, delete or create new sub accounts from.
if ($mode != 3)
{
  print "<form method=post action=\"$site_dir/cgi-bin/sub_accounts_config4.cgi\" name=\"sub_acc_form\">\n";
  if (length $ledger_db > 0)
  {
    print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
  }
}

if ($mode == 1)  # Edit
{
  print "<h2>Editing</h2>\n";
  my ($sub_acct_num, $sub_acct_name) = split /\|/, $subAcct;
  # I already validated that they are not empty above so just look them up in the database.
  $result = $conn->exec("select type from sub_acc_tb where sub_acc_name = '$sub_acct_name' and sub_acc_number = $sub_acct_num");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    print "<h1>Error:  Accessing database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to validate sub account <font color=\"blue\">$sub_acct_name</font>.<br>$sys_admin";
    CleanUp();
  }
  if ($result->ntuples == 0)
  {
    print "<h1>Error:  Sub Account <font color=\"blue\">$sub_acct_name</font> does not exist!</h1>\n";
    print "Tried to validate sub account <font color=\"blue\">$sub_acct_name</font>.<br>$sys_admin";
    CleanUp();
  }
  my $sub_acct_type = $result->getvalue(0, 0);
  # lookup the sub-account type name for the sub-account type number
  $result = $conn->exec("select name from sub_acct_type_tb where num = $sub_acct_type");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    print "<h1>Error: Accessing the database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to map the sub-account type number: $sub_acct_type to its name.<br>$sys_admin";
    CleanUp();
  }
  if ($result->ntuples <= 0)
  {
    print "<h1>Error: No records returned!</h1>\n";
    print "Could not find the name that goes with the sub-account type number: $sub_acct_type.<br>$sys_admin";
    CleanUp();
  }
  my $sub_acct_type_name = $result->getvalue(0, 0);

  print "<table border=0 cellspacing=0 cellpadding=2 width=50% bgcolor=\"$menu_bgcolor\">\n";
  print "<tr><td><font color=\"$menu_fgcolor\">Sub Account Name</font></td>\n";
  my $baseURL = $portalObj->{methods}->createBaseURL(type => "Portal", linkType => "cgi", appConfigObj => $portalObj->{sessionObj}->{store}->{companyAppObj});
  if ($portalObj->didErrorOccur)
  {
    print "</tr></table></form>\n";
    print "<h1>Error: Obtaining base URL failed!</h1>\n";
    print $portalObj->errorMessage;
    CleanUp();
  }
  print "<td><input type=text name=\"sub_acct_name\" value=\"$sub_acct_name\">&nbsp;&nbsp;<input type=\"button\" name=\"lookupName\" value=\"Lookup Stock Info\" onClick=\"window.open('" . $baseURL . "?app=Accounting&state=StockPrices&command=lookupStockInfo&subAccountName=' + encodeURIComponent(document.sub_acc_form.sub_acct_name.value) + '&ticker=' + encodeURIComponent(document.sub_acc_form.ticker.value) + '&stockName=' + encodeURIComponent(document.sub_acc_form.stockName.value), window.name + 'lookupStockInfo', 'width=400,height=250,scrollbars,resizable');\"><input type=hidden name=\"old_sub_acct_name\" value=\"$sub_acct_name\"></td></tr>\n";
  print "<tr><td><font color=\"$menu_fgcolor\">Sub Account Number</font></td>\n";
  print "<td>$sub_acct_num<input type=hidden name=\"sub_acct_number\" value=\"$sub_acct_num\"></td></tr>\n";
  print "<tr><td><font color=\"$menu_fgcolor\">Sub Account Type</font></td>\n";
  print "<td>$sub_acct_type_name ($sub_acct_type)<input type=hidden name=\"sub_acct_type\" value=\"$sub_acct_type\"></td></tr>\n";
  if ($sub_acct_type == 70) # stock
  {
    # gather the stock info
    $result = $conn->exec("SELECT ticker, name, market, type FROM stocks_tb WHERE sub_acct = $sub_acct_num");
    if ($result->resultStatus != PGRES_TUPLES_OK)
    {
      print "<h1>Error: Accessing the database failed!</h1>\n";
      print $conn->errorMessage . "<br>Tried to get stock info for Sub Account = '$sub_acct_num'.<br>$sys_admin";
      CleanUp();
    }
    if ($result->ntuples == 0)
    {
      print "<h1>Error: No records returned!</h1>\n";
      print "Could not find the stock info that goes with the sub-account number: $sub_acct_num.<br>$sys_admin";
      CleanUp();
    }
    my $ticker = $result->getvalue(0, 0);
    my $name = $result->getvalue(0, 1);
    my $market = $result->getvalue(0, 2);
    my $type = $result->getvalue(0, 3);  # not used just yet.

    print "<tr><td colspan=2><hr noshade></td></tr>\n";
    print "<tr><td><font color=\"$menu_fgcolor\">Ticker</font></td>\n";
    print "<td><input type=text name=\"ticker\" value=\"$ticker\" size=\"10\" maxlength=\"10\"><input type=hidden name=\"oldTicker\" value=\"$ticker\"></td></tr>\n";
    print "<tr><td><font color=\"$menu_fgcolor\">Stock Name</font></td>\n";
    print "<td><input type=text name=\"stockName\" value=\"$name\"><input type=hidden name=\"oldStockName\" value=\"$name\"></td></tr>\n";
    print "<tr><td><font color=\"$menu_fgcolor\">Market</font></td>\n";
    print "<td><select name=\"market\">\n";
    foreach my $tmpMarket (@markets)
    {
      print "  <option value=\"$tmpMarket\"" . ($market eq "$tmpMarket" ? " selected" : "") . ">$tmpMarket\n";
    }
    print "</select><input type=hidden name=\"oldMarket\" value=\"$market\"></td></tr>\n";
  }
  print "</table>\n";
  print "<input type=submit name=\"update" . ($sub_acct_type == 70 ? "Stock" : "") . "\" value=\"Update Sub Account Info\">&nbsp;";
  print "<input type=reset value=\"Reset\">\n";
}
elsif ($mode == 2) # Create
{
  # retrieve all sub-account types from the sub_acct_type_tb table except for the Stock SubAccount Type = 70
  $result = $conn->exec("select name, num from sub_acct_type_tb WHERE num NOT IN (70, 1, 90, 15) order by name");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    print "<h1>Error: Accessing the database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to get the sub-account types from the database!<br>$sys_admin";
    CleanUp();
  }
  if ($result->ntuples <= 0)
  {
    print "<h1>Error: No records returned!</h1>\n";
    print "Tried to get the sub-account types from the database!<br>$sys_admin";
    CleanUp();
  }
  print "<h2>Creating</h2>\n";
  print "<table border=0 cellspacing=0 cellpadding=2 width=50% bgcolor=\"$menu_bgcolor\">\n";
  print "<tr><td><font color=\"$menu_fgcolor\">Sub Account Name</font></td>\n";
  print "<td><input type=text name=\"sub_acct_name\" value=\"\"></td></tr>\n";
  print "<tr><td><font color=\"$menu_fgcolor\">Sub Account Number</font></td>\n";
  print "<td><input type=text name=\"sub_acct_number\" value=\"\" size=\"$sub_acct_length\" maxlength=\"$sub_acct_length\"></td></tr>\n";
  print "<tr><td><font color=\"$menu_fgcolor\">Sub Account Type</font></td>\n";
  print "<td><select name=\"sub_acct_type\">\n";
  for ($count=0; $count < $result->ntuples; $count++)
  {
    print "<option value=\"" . ($result->getvalue($count, 1)) . "\">" . ($result->getvalue($count, 0)) . " (" . ($result->getvalue($count, 1)) . ")\n";
  }
  print "</select></td></tr>\n";
  print "</table>\n";
  print "<input type=submit name=\"create\" value=\"Create Sub Account\">&nbsp;";
  print "<input type=reset value=\"Reset\">\n";
}
elsif ($mode == 3) # Delete
{
  print "<h2>Deleting</h2>\n";
  my ($sub_num, $sub_name) = split /\|/, $subAcct;

  # verify that the sub account exists first
  $result = $conn->exec("select type from sub_acc_tb where sub_acc_name = '$sub_name' and sub_acc_number = $sub_num");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    print "</form>\n";
    print "<h1>Error:  Accessing Database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to verify that the sub account exists.<br>Sub Account = <font color=\"blue\">$sub_name - $sub_num</font><br>$sys_admin";
    CleanUp();
  }
  if ($result->ntuples == 0)
  {
    print "</form>\n";
    print "<h1>Error:  Sub Account does not exist!</h1>\n";
    print "Sub Account <font color=\"blue\">$sub_name - $sub_num</font> does not exist!<br>$sys_admin";
    CleanUp();
  }
  my $subAcctType = $result->getvalue(0, 0);

  # verify that it is not in use, make sure we don't mistake it for a check or deposit number.
  $result = $conn->exec("SELECT DISTINCT(ledger_code) FROM debitcredit_tb WHERE sub_account_num = $sub_num AND account_num NOT IN (SELECT number FROM account_tb WHERE type = 0)");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    print "</form>\n";
    print "<h1>Error:  Checking to see if the Sub Account is being used failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to see if Sub Account ($sub_name) = $sub_num is in use in any ledger.<br>$sys_admin";
    CleanUp();
  }
  if ($result->ntuples > 0)
  {
    print "<h2>Error:  Sub Account ($sub_name) = $sub_num is in Use in these ledgers:</h2>\n";
    my ($j);
    for ($j=0; $j < $result->ntuples; $j++)
    {
      my $ledger_code = $result->getvalue($j, 0);
      my $result2 = $conn->exec("SELECT name, color FROM ledger_tb WHERE code = '$ledger_code'");
      if ($result2->resultStatus != PGRES_TUPLES_OK)
      {
        print "</form>\n";
        print "<h1>Error:  Getting the Ledger Name failed!</h1>\n";
        print $conn->errorMessage . "<br>Tried to get the Ledger name for code = '$ledger_code'.<br>$sys_admin";
        CleanUp();
      }
      my $ledger_name = $result2->getvalue(0, 0);
      my $ledger_color = $result2->getvalue(0, 1);
      print "Ledger = <font color=\"$ledger_color\">$ledger_name</font><br>\n";
    }
  }
  else
  {
    # start transaction
    $result = $conn->exec("begin work");
    if ($result->resultStatus != PGRES_COMMAND_OK)
    {
      print "</form>\n";
      print "<h1>Error:  Starting Transaction failed!</h1>\n";
      print $conn->errorMessage . "<br>Before deletion of Sub Account <font color=\"blue\">$sub_name - $sub_num</font>.<br>$sys_admin";
      CleanUp();
    }

    $result = $conn->exec("delete from sub_acc_tb where sub_acc_name = '$sub_name' and sub_acc_number = $sub_num");
    if ($result->resultStatus != PGRES_COMMAND_OK)
    {
      print "</form>\n";
      print "<h1>Error:  Deleting Sub Account failed!</h1>\n";
      print $conn->errorMessage . "<br>Tried to delete Sub Account <font color=\"blue\">$sub_name - $sub_num</font>.<br>$sys_admin";
      CleanUp();
    }

    if ($subAcctType == 70)
    {
      # we also have to delete the Stock entry.
      $result = $conn->exec("DELETE FROM stocks_tb WHERE sub_acct = $sub_num");
      if ($result->resultStatus != PGRES_COMMAND_OK)
      {
        print "</form>\n";
        print "<h1>Error:  Deleting Stock failed!</h1>\n";
        print $conn->errorMessage . "<br>Tried to delete Stock with Sub Account <font color=\"blue\">$sub_name - $sub_num</font>.<br>$sys_admin";
        CleanUp();
      }

      # also delete any watched stock entries for this stock.
      $result = $conn->exec("DELETE FROM stock_watch_tb WHERE sub_acct = $sub_num");
      if ($result->resultStatus != PGRES_COMMAND_OK)
      {
        print "</form>\n";
        print "<h1>Error:  Deleting Watched Stock entries failed!</h1>\n";
        print $conn->errorMessage . "<br>Tried to delete Watched Stock entries with Sub Account <font color=\"blue\">$sub_name - $sub_num</font>.<br>$sys_admin";
        CleanUp();
      }

      # also delete any historical data gathered for this stock.
      $result = $conn->exec("DELETE FROM stock_price_tb WHERE sub_acct = $sub_num");
      if ($result->resultStatus != PGRES_COMMAND_OK)
      {
        print "</form>\n";
        print "<h1>Error:  Deleting Historical Stock data failed!</h1>\n";
        print $conn->errorMessage . "<br>Tried to delete Historical Stock data for Sub Account <font color=\"blue\">$sub_name - $sub_num</font>.<br>$sys_admin";
        CleanUp();
      }
    }

    $result = $conn->exec("commit work");
    if ($result->resultStatus != PGRES_COMMAND_OK)
    {
      print "</form>\n";
      print "<h1>Error:  Committing Transaction failed!</h1>\n";
      print $conn->errorMessage . "<br>After deletion of Sub Account <font color=\"blue\">$sub_name - $sub_num</font>.<br>$sys_admin";
      CleanUp();
    }

    print "Sub Account <font color=\"blue\">$sub_name - $sub_num</font> successfully deleted.<br>\n";
  }
  print "<form name=\"sub_acct_form\" method=post action=\"$site_dir/cgi-bin/sub_accounts_config2.cgi\">\n";
  print "<input type=hidden name=\"nothing\" value=\"\">\n";
  if (length $ledger_db > 0)
  {
    print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
  }
  print "<br><br><br>You will be returned to the Sub Account Administration page in <b>3</b> seconds.<br>\n";
  print "</form>\n";
}
elsif ($mode == 4)  # create Stock
{
  print "<h2>Creating Stock Sub Account</h2>\n";
  print "<table border=0 cellspacing=0 cellpadding=2 width=50% bgcolor=\"$menu_bgcolor\">\n";
  print "<tr><td><font color=\"$menu_fgcolor\">Sub Account Name</font></td>\n";
  my $baseURL = $portalObj->{methods}->createBaseURL(type => "Portal", linkType => "cgi", appConfigObj => $portalObj->{sessionObj}->{store}->{companyAppObj});
  if ($portalObj->didErrorOccur)
  {
    print "</tr></table></form>\n";
    print "<h1>Error: Obtaining base URL failed!</h1>\n";
    print $portalObj->errorMessage;
    CleanUp();
  }
  print "<td><input type=text name=\"sub_acct_name\" value=\"\">&nbsp;&nbsp;<input type=\"button\" name=\"lookupName\" value=\"Lookup Stock Info\" onClick=\"window.open('" . $baseURL . "?app=Accounting&state=StockPrices&command=lookupStockInfo&subAccountName=' + encodeURIComponent(document.sub_acc_form.sub_acct_name.value) + '&ticker=' + encodeURIComponent(document.sub_acc_form.ticker.value) + '&stockName=' + encodeURIComponent(document.sub_acc_form.stockName.value), window.name + 'lookupStockInfo', 'width=400,height=250,scrollbars,resizable');\"></td></tr>\n";
  print "<tr><td><font color=\"$menu_fgcolor\">Sub Account Number</font></td>\n";
  print "<td><input type=text name=\"sub_acct_number\" value=\"\" size=\"$sub_acct_length\" maxlength=\"$sub_acct_length\"></td></tr>\n";
  print "<tr><td><font color=\"$menu_fgcolor\">Sub Account Type</font></td>\n";
  print "<td><input type=hidden name=\"sub_acct_type\" value=\"70\">Stock (70)</td></tr>\n";
  print "<tr><td colspan=2><hr noshade></td></tr>\n";
  print "<tr><td><font color=\"$menu_fgcolor\">Ticker</font></td>\n";
  print "<td><input type=text name=\"ticker\" value=\"\" size=\"10\" maxlength=\"10\"></td></tr>\n";
  print "<tr><td><font color=\"$menu_fgcolor\">Stock Name</font></td>\n";
  print "<td><input type=text name=\"stockName\" value=\"\"></td></tr>\n";
  print "<tr><td><font color=\"$menu_fgcolor\">Market</font></td>\n";
  print "<td><select name=\"market\">\n";
  foreach my $tmpMarket (@markets)
  {
    print "  <option value=\"$tmpMarket\">$tmpMarket\n";
  }
  print "</select></td></tr>\n";
  print "</table>\n";
  print "<input type=submit name=\"createStock\" value=\"Create Sub Account\">&nbsp;";
  print "<input type=reset value=\"Reset\">\n";
}

if ($mode != 3)
{
  print <<"END_OF_FORM";
  </form>
END_OF_FORM
}

# print a horizontal rule to break the menu from the behind the scenes links
print "<br><hr>\n";

print "<form name=\"sub_acct_admin\" method=post action=\"$site_dir/cgi-bin/sub_accounts_config2.cgi\">\n";
print "<input type=hidden name=\"nothing\" value=\"\">\n";
if (length $ledger_db > 0)
{
  print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
}
print "<div align=left><a href=\"\" onClick=\"SubmitForm(); return false;\">Sub Account Administration</a></div>\n";
print "</form>\n";

if (length $ledger_db == 0)
{
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/account_config.cgi\">Account Configuration</a></div>\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/prg_config.cgi\">Program Configuration</a></div>\n";
}
else
{
  print "<form name=\"form1\" method=post action=\"\">\n";
  print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/account_config.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/account_config.cgi'); return false;\">Account Configuration</a></div>\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/prg_config.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/prg_config.cgi'); return false;\">Program Configuration</a></div>\n";
  print "<input type=hidden name=\"open\" value=\"Open\">\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/ledger.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/ledger.cgi'); return false;\">Return to Ledger Summary</a></div>\n";
  print "</form>\n";
}

print "</body>\n";
print "</html>\n";
exit 0;

sub getStockMarkets
{
  my $result = $conn->exec("SELECT DISTINCT market FROM stocks_tb WHERE market <> 'None' ORDER BY market");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    PrintErrorHeader();
    print "<h1>Error:  Accessing database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to get list of Stock Markets from database.<br>$sys_admin";
    CleanUp();
  }

  my @values = ();

  for (my $i=0; $i < $result->ntuples; $i++)
  {
    $values[$i] = $result->getvalue($i, 0);
  }

  push @values, "None";

  return @values;
}
