#!/usr/bin/perl
#$Id: sub_entities3.cgi,v 1.1 2001/09/06 10:21:20 moreejt Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
# sub_entities3.cgi  -  Created by James Pattie, 07/20/1999
# This script creates or updates sub entities.

use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features
use reports;

use vars '$date';
use vars '$ledger_db';
use vars '$ledger_name';
use vars '$working_mode';
use vars '$entry_code';
use vars '$entry_name';
use vars '$old_name';
my $error_string;  # used to hold the return value from IsValidString();
my $ledger_code;
my $ledger_color;
my $report_obj;

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

if (not (exists $input{'create'} || exists $input{'update'}))
{
  PrintErrorHeader();
  print "<h1>Error:  Access Denied!</h1>\n";
  print "You must specify a mode to work in.<br>$sys_admin";
  CleanUp();
}

if (exists $input{'create'})
{
  $working_mode = 1;
}
elsif (exists $input{'update'})
{
  $working_mode = 2;
}

$ledger_db = $input{'ledger'};
if (length $ledger_db == 0)
{
  PrintErrorHeader();
  print "<h1>Error: You must specify a ledger to work with!</h1>\n";
  print "$back_button";
  CleanUp();
}
$error_string = IsValidString($ledger_db);
if (length $error_string != 0)
{
  PrintErrorHeader();
  print "<h1>Error:  Invalid Ledger Database Name!</h1>\n";
  print "<b>$error_string</b> is invalid.<br>$back_button";
  CleanUp();
}

if (length $input{'name'} == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  You must specify a name!</h1>\n";
  print "$back_button";
  CleanUp();
}
if (length $input{'code'} == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  You must specify a code!</h1>\n";
  print "$back_button";
  CleanUp();
}
if ($working_mode == 2)
{
  # make sure we have the old_name to work with
  if (length $input{'old_name'} == 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  The old name must be specified!</h1>\n";
    print "$sys_admin";
    CleanUp();
  }
  $old_name = $input{'old_name'};
  $error_string = IsValidString($old_name);
  if (length $error_string != 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Old SubEntity Name!</h1>\n";
    print "<b>$error_string</b> is invalid.<br>$back_button";
    CleanUp();
  }
}

$entry_name = $input{'name'};
$error_string = IsValidString($entry_name);
if (length $error_string != 0)
{
  PrintErrorHeader();
  print "<h1>Error:  Invalid SubEntity Name!</h1>\n";
  print "<b>$error_string</b> is invalid.<br>$back_button";
  CleanUp();
}
$entry_code = $input{'code'};
$error_string = IsValidString($entry_code);
if (length $error_string != 0)
{
  PrintErrorHeader();
  print "<h1>Error:  Invalid SubEntity Code!</h1>\n";
  print "<b>$error_string</b> is invalid.<br>$back_button";
  CleanUp();
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

$result = $conn->exec("select name from entity_tb where database_name = '$ledger_db'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print "Trying to check entity table.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: Ledger <font color=\"blue\">$ledger_db</font> does not exist!</h1>\n";
  print "$sys_admin";
  CleanUp();
}
else
{
  $ledger_name = $result->getvalue(0, 0);
}

# create the reports object
$report_obj = reports->new($conn, $ledger_db);
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_code = $report_obj->getLedgerCode();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_color = $report_obj->getLedgerColor();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}

# start a transaction
$result = $conn->exec("begin work");
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Starting Transaction Failed!</h1>\n";
  print $conn->errorMessage . "<br>Ledger = '$ledger_db'<br>$sys_admin";
  CleanUp();
}

# lookup the name and code
if ($working_mode == 1)
{
  $result = $conn->exec("select name, code from sub_entity_tb where name = '$entry_name' and code = '$entry_code' AND ledger_code = '$ledger_code'");
}
elsif ($working_mode == 2)
{
  if (not $old_name eq $entry_name)
  {
    $result = $conn->exec("select name, code from sub_entity_tb where name = '$old_name' and code = '$entry_code' AND ledger_code = '$ledger_code'");
  }
  else
  {
    $result = $conn->exec("select name, code from sub_entity_tb where name = '$entry_name' and code = '$entry_code' AND ledger_code = '$ledger_code'");
  }
}

if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to see if Sub Entity already exists.<br>$sys_admin";
  CleanUp();
}

if ($working_mode == 1)
{ # create the sub entity
  if ($result->ntuples != 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Sub Entity already exists!</h1>\n";
    print "$back_button";
    CleanUp();
  }

  $result = $conn->exec("insert into sub_entity_tb (name, code, ledger_code) values ('$entry_name', '$entry_code', '$ledger_code')");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    PrintErrorHeader();
    print "<h1>Error:  Inserting record failed!</h1>\n";
    print $conn->errorMessage . "<br>Name = '$entry_name', Code = '$entry_code'.<br>$sys_admin";
    CleanUp();
  }
}
elsif ($working_mode == 2)
{ # update the sub entity
  if ($result->ntuples == 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Sub Entity does not exist!</h1>\n";
    print "The Sub Entity could have been deleted.<br>$sys_admin";
    CleanUp();
  }

  if (not $old_name eq $entry_name)
  {
    # make sure that the new name doesn't exist anywhere
    $result = $conn->exec("select code from sub_entity_tb where name = '$entry_name' AND ledger_code = '$ledger_code'");
    if ($result->resultStatus != PGRES_TUPLES_OK)
    {
      PrintErrorHeader();
      print "<h1>Error:  Accessing database failed!</h1>\n";
      print $conn->errorMessage . "<br>Tried to see if Name = '$entry_name' was in use already.<br>$sys_admin";
      CleanUp();
    }
    if ($result->ntuples != 0)
    {
      PrintErrorHeader();
      print "<h1>Error:  Sub Entity Name = '$entry_name' is already in use!</h1>\n";
      print "$back_button";
      CleanUp();
    }
  }

  $result = $conn->exec("update sub_entity_tb set name = '$entry_name' where code = '$entry_code' AND ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    PrintErrorHeader();
    print "<h1>Error:  Updating record failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to update Sub Entity with Code = '$entry_code' and Name = '$entry_name'.<br>$sys_admin";
    CleanUp();
  }
}

# commit work
$result = $conn->exec("commit work");
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Committing Transaction Failed!</h1>\n";
  print $conn->errorMessage . "<br>Ledger = '$ledger_db'<br>$sys_admin";
  CleanUp();
}

# end transaction
$result = $conn->exec("end work");
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Ending Transaction Failed!</h1>\n";
  print $conn->errorMessage . "<br>Ledger = '$ledger_db'<br>$sys_admin";
  CleanUp();
}

print "<head><title>$prog_name $prog_version Sub Entity Administration</title>\n";

print <<"END_OF_STYLE";
<script language="Javascript1.2">
<!--
  function SubmitForm()
  { // This function will submit the assign_account_form with the needed stuff set.
    document.SubEntity_Form.submit();
  }
//-->
</script>
END_OF_STYLE

print "</head>\n";
print "<body bgcolor=$body_bgcolor>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td width=47%>Working with Ledger <font color=\"$ledger_color\">$ledger_name</font>.&nbsp;&nbsp;";
if ($working_mode == 1)
{
  print "Creating Sub Entity";
}
elsif ($working_mode == 2)
{
  print "Editing Sub Entity <font color=\"red\">$entry_name</font>";
}
print "</td>\n";
print "<td align=left><a href=\"$site_dir/help.html#sub_entities\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardLedgerMainMenu($ledger_db);
print "<h2><font color=$prog_fgcolor>$prog_name $prog_version</font> Sub Entity Administration</h2>\n";

if ($working_mode == 1)
{ # create the entries for entering the name and code.
  print "<br><h2>Successfully created Sub Entity <font color=\"blue\">$entry_name</font> with Code = '$entry_code'</h2>\n";

  print "<form name=\"SubEntity_Form\" method=post action=\"$site_dir/cgi-bin/sub_entities2.cgi\">\n";
  print "<input type=hidden name=\"create\" value=\"\">\n";
  print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
  print "<input type=hidden name=\"sub_entity_command\" value=\"create\">\n";
  print "<br><br><a href=\"\" onClick=\"SubmitForm(); return false;\">To create another go <u>here</u>.</a>\n";
  print "</form>\n";
}
elsif ($working_mode == 2)
{
  print "<br><h2>Successfully updated Sub Entity <font color=\"blue\">$entry_name</font> with Code = '$entry_code'</h2>\n";

  print "<form name=\"SubEntity_Form\" method=post action=\"$site_dir/cgi-bin/sub_entities2.cgi\">\n";
  print "<input type=hidden name=\"edit\" value=\"\">\n";
  print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
  print "<input type=hidden name=\"sub_entity_command\" value=\"edit\">\n";
  print "<input type=hidden name=\"sub_entity\" value=\"$entry_code|$entry_name\">\n";
  print "<br><br><a href=\"\" onClick=\"SubmitForm(); return false;\">To edit again go <u>here</u>.</a>\n";
  print "</form>\n";
}

# print a horizontal rule to break the menu from the behind the scenes links
print "<br>\n";

print "<form method=post action=\"$site_dir/cgi-bin/sub_entities.cgi\" name=\"menu_bar2\">\n";
print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
print "<div align=left><a href=\"$site_dir/cgi-bin/sub_entities.cgi\" onClick=\"javascript: document.menu_bar2.submit(); return false;\">Return to Sub Entity Administration</a></div>\n";
print "</form>\n";

print "</body>\n";
print "</html>\n";
exit 0;
