/* common.js
	Created by James Pattie, 10/25/1999
	This file holds all common functions
	that are used in PC Xperience Inc. projects.
#$Id: common.js,v 1.1 2001/09/06 10:21:20 moreejt Exp $
#This code is released under the GNU General Public License.
#If you downloaded the package, you should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
*/

function IsValidDateSyntax(date)
{
  // Search the string and see what they are using for the seperator character - or /.
  var values;
  if (date.indexOf("-")  != -1)
  {  // They are using -
    values = date.split("-");
  }
  else
  {
    if (date.indexOf("/") != -1)
    {  // They are using /
      values = date.split("/");
    }
    else
    {
      return "No valid Seperator being used!  You must use '-' or '/'.";
    }
  }
  if (values.length != 3)
  {
    return "There is not the correct number of segments (Month, Day, Year)!";
  }
  if (values[2].length != 4)
  {
    return "The Year is not a 4 digit year!";
  }
  if (values[0].length == 0)
  {
    return "The Month is not specified!";
  }
  if (values[0].length > 2)
  {
    return "The Month can not be more than 2 digits in length!";
  }
  if (values[0] < 1)
  {
    return "The Month can not be less than 1!";
  }
  if (values[0] > 12)
  {
    return "The Month can not be greater than 12!";
  }
  if (values[1].length == 0)
  {
    return "The Day is not specified!";
  }
  if (values[1].length > 2)
  {
    return "The Day can not be more than 2 digits in length!";
  }
  if (values[1] < 1)
  {
    return "The Day can not be less than 1!";
  }
  if (values[1] > 31)
  {
    return "The Day can not be more than 31!";
  }
  return "";
}

function MakeValidUInt(value)
{
  // This function will take a string and return a valid unsigned integer.

  var pattern = /[^-\+\.\d,]+/;

  if (value.length == 0)
  {
    return "0";
  }

  if (pattern.test(value))
  {
    return "0";
  }

  value = value.replace(/-/g, "");
  value = value.replace(/\+/g, "");
  value = value.replace(/,/g, "");
  value = value.replace(/\./g, "");

  return value;
}

function MakeValidAmount(amount)
{
  // This function will take a string (hopefully of numbers) and convert it to a valid
  // monetary figure with trailing 00's if needed, etc.

  // First verify that we have all numbers and a hopefully a period.
  var pattern = /[^-\+\.\d,]+/;

  if (amount.length == 0)
  {  // set it to 0.00 if it is empty. (Shouldn't happen, but just to be safe!)
    return "0.00";
  }

  if (pattern.test(amount))
  { // set it to 0.00 if it is invalid!
    return "0.00";
  }

  // See if they have a minus in the number and remove all minuses.
  var index;
  amount = amount.replace(/-/g, "");
  amount = amount.replace(/\+/g, "");
  amount = amount.replace(/,/g, "");

  // now we need to see if we have a period
  index = amount.indexOf(".");
  if (index == -1)
  { // They don't have a period so append .00 to the end
    return (amount + ".00");
  }
  else
  {
    // see where the period is located and if not 2 digits from the end, adjust it.
    var values = amount.split(".");
    if (values.length == 2) // looking good!
    {
      // verify we have a valid dollar part.
      var dollar = values[0];
      if (values[0].length == 0)
      {
        dollar = "0";
      }
      // verify we have a valid cents part.
      var cents = values[1];
      if (values[1].length > 2)
      {
        cents = cents.substr(0, 2);
      }
      else
      {
        if (values[1].length == 1)
        {
          cents += "0";
        }
      }
      return (dollar + "." + cents);
    }
    else
    {
      if (values.length > 2) // UhOh!  Multiple periods!
      { // we'll take the first 2 parts and try to make it work.
        var dollar = values[0];
        if (values[0].length == 0)
        {
          dollar = "0";
        }
        var cents = values[1];
        if (values[1].length > 2)
        {
          cents = cents.substr(0, 2);
        }
        else
        {
          if (values[1].length == 1)
          {
            cents += "0";
          }
          else
          {
            if (values[1].length == 0)
            {
              cents = "00";
            }
          }
        }
        return (dollar + "." + cents);
      }
    }
  }
  return amount;  // This should never get here!  (hopefully)
}

function ConvertDateString(date)
{
  // This function will parse apart the given date string in month, day, year format
  // seperated by - or / and return it in year, month, day format, where all elements
  // of the month and day are 2 digits and the year is 4 digits.  The year must come in
  // as a 4 digit value for this to work.

  // Search the string and see what they are using for the seperator character - or /.
  if (date.indexOf("-")  != -1)
  {  // They are using -
    values = date.split("-");
  }
  else
  {
    if (date.indexOf("/") != -1)
    {  // They are using /
      values = date.split("/");
    }
    else
    {
      alert("The given date is in an invalid format!  Date = " + date);
      return "";
    }
  }

  // now we take the values and order them correctly and return the new date string.
  if (values[2].length != 4)
  {
    alert("The given year is not a 4 digit year!  Year = " + values[2]);
    return "";
  }

  date = values[2];
  if (values[0] < 10 && values[0].length == 1)
  {
    date += "0" + values[0];
  }
  else
  {
    date += values[0];
  }
  if (values[1] < 10 && values[1].length == 1)
  {
    date += "0" + values[1];
  }
  else
  {
    date += values[1];
  }

  return date;
}

function submit_my_form(x, url)
{
  // set the form's action to url
  x.action = url;
  //x.submit();
}

function go_here(x, url)
{
  x.action = url;
  x.submit();
  return false;
}

function go_here2(x, method, url)
{
  x.action = url;
  x.method = method;
  x.submit();
  return false;
}

function go_same(x, url)
{
  x.action = url;
  x.edit.value = "true";
  x.submit();
  return false;
}

function highlight(x, image_url)
{
  // set the location for the image contained in x to the image_url
  x.src = image_url;
}
