#!/usr/bin/perl
# report_customization.cgi - This program is the reporting system customization and data gathering phase.
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

# Program Modes:  Uses command
# command =
#	  = 'customize_report' - Takes the selected report and creates the customization screen. based upon report_type_tb entry.
#	  = 'memorize_report' - Takes the customized data and memorizes it in the database.  Generates code to close the window when successfully memorized.

use strict;
use cgi_lib;
use custom;
use Portal::Language;
use Portal::Accounting::Language;
use Portal::Methods;

my %errors;
%errors = ( 4000 => 'Eval of HTMLObject::FrameSet failed',
	       4001 => 'Invalid command specified',
	       4002 => 'Eval of HTMLObject::Normal failed',
	       4003 => 'command must be specified',
	       4004 => "Couldn't connect to Database",
	       4005 => "Parameter requires a value",
	       4006 => "Invalid Value for Parameter",
	       4007 => "DBI Preparation of SQL Statement Failed",
	       4008 => "DBI Execution of Statement Failed",
	       4009 => "DBI Error Occurred",
	       4013 => "Could not find Sub Account",
	       4014 => "Could not find Sub Account Type",
	       4017 => "Report ID is required",
	       4019 => "Error Committing Database changes",
	       4022 => "Ledger Code required",
	       4023 => "Could not find Ledger Database",
	       4024 => "Could not find Ledger Name",
	       4025 => "Could not find Ledger Code",
	       4026 => "Ledger DB Name required",
	       4027 => "Ledger Name required",
	       4028 => "Database Handle required",
	       4029 => "Memorized value required",
	       4030 => "Invalid ID format",
	       4034 => "Parameter Required",
	       4035 => "File Name required",
	       4037 => "Problem with Formula evaluation",
	     );

my %sql_statements;
%sql_statements = (  
			1005 => "SELECT name FROM report_type_tb WHERE id = ?",
			1006 => "SELECT name, report_class, report_type, column_variable, column_variable_data, date_range, display_dates, max_columns, min_columns, text_font, text_size, data_font, data_size, total_column, fg_color, bg_color, underline_length, editable FROM report_type_tb WHERE id = ?",
			1016 => "SELECT name FROM entity_tb WHERE database_name = ?",
			1017 => "SELECT code FROM ledger_tb WHERE name = ?",
			1018 => "SELECT color FROM ledger_tb WHERE code = ?",
			1019 => "SELECT name, num FROM acct_type_tb ORDER BY name, num",
			1020 => "SELECT name, number, type, active FROM account_tb ORDER BY name, number, type, active",
			1021 => "SELECT sub_acc_name, sub_acc_number, type FROM sub_acc_tb ORDER BY sub_acc_name, sub_acc_number, type",
			1023 => "SELECT name FROM report_type_tb WHERE id = ?",
			1024 => "SELECT index, cell_text, cell_type, item_name, fg_color, bg_color, display_level, initial_state, negative_format, other_attributes FROM report_data_tb WHERE id = ? ORDER BY index",
			1027 => "SELECT column_type, column_data, dates_entered, fg_color, bg_color, text_size, data_size, text_font, data_font, underline_length, account_format, data_precision, negative_format, total_column, sql_statement, headers, sorted_by, sorted_order, subtotal, subtotal_type FROM memorized_report_tb WHERE ledger_code = ? AND id = ? AND name = ?",
			1028 => "SELECT year, start_date, end_date, q1_start, q1_end, q2_start, q2_end, q3_start, q3_end, q4_start, q4_end FROM known_years_tb WHERE ledger_code = ? ORDER BY year",
			1029 => "INSERT INTO temp_file_tb (timeof, fname) VALUES (now(), ?)",
			1030 => "DELETE FROM memorized_report_tb WHERE id = ? AND ledger_code = ? AND name = ?",
			1031 => "INSERT INTO memorized_report_tb (id, name, column_type, column_data, dates_entered, fg_color, bg_color, text_size, text_font, data_size, data_font, underline_length, ledger_code, account_format, data_precision, negative_format, total_column, sql_statement, headers, sorted_by, sorted_order, subtotal, subtotal_type) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
			1032 => "SELECT id FROM memorized_report_tb WHERE name = ? AND ledger_code = ?",
			1033 => "SELECT name, code FROM sub_entity_tb WHERE ledger_code = ?",
			1034 => "SELECT sub_acc_name FROM sub_acc_tb WHERE sub_acc_number = ?",
			1035 => "SELECT name FROM sub_acct_type_tb WHERE num = ?",
			1042 => "SELECT name FROM acct_type_tb WHERE num = ?",
			1043 => "SELECT debit_increases FROM acct_type_tb WHERE num = ?",
			1044 => "SELECT number FROM account_tb WHERE type = ? ORDER BY number",
			1045 => "SELECT acct_num FROM local_accounts_tb WHERE ledger_code = ? AND taxable = true AND sub_entity_code IS NULL AND acct_num IN (##?##) ORDER BY acct_num",
			1046 => "SELECT acct_num FROM local_accounts_tb WHERE ledger_code = ? AND taxable = false AND sub_entity_code IS NULL AND acct_num IN (##?##) ORDER BY acct_num",
			1047 => "SELECT acct_num FROM local_accounts_tb WHERE ledger_code = ? AND sub_entity_code IS NULL AND acct_num IN (##?##) ORDER BY acct_num",
			1048 => "SELECT name FROM account_tb WHERE number = ?",
			1050 => "SELECT db.amount, db.dorc FROM debitcredit_tb AS db, journal_entry_tb AS je WHERE db.ledger_code = je.ledger_code AND db.ledger_code = ? AND db.parent_num = je.ref_num AND db.account_num = ? AND ##1?####2?####?##",
			1052 => "SELECT sum(db.amount) FROM debitcredit_tb AS db, journal_entry_tb AS je WHERE db.ledger_code = je.ledger_code AND db.ledger_code = ? AND db.parent_num = je.ref_num AND db.account_num = ? AND db.dorc = true AND ##?##",
			1053 => "SELECT sum(db.amount) FROM debitcredit_tb AS db, journal_entry_tb AS je WHERE db.ledger_code = je.ledger_code AND db.ledger_code = ? AND db.parent_num = je.ref_num AND db.account_num = ? AND db.dorc = false AND ##?##",
			1054 => "SELECT sub_acc_name, sub_acc_number FROM sub_acc_tb WHERE type IN (##1?##) ORDER BY ##2?##",
			1055 => "SELECT sub_acct_type FROM acct_type_tb WHERE num = ?",
			1056 => "SELECT account_tb.type, acct_type_tb.debit_increases FROM account_tb, acct_type_tb WHERE account_tb.number = ? AND acct_type_tb.num = account_tb.type",
			1057 => "SELECT sub_acc_name FROM sub_acc_tb WHERE sub_acc_number = ?",
			1058 => "SELECT name FROM sub_acct_type_tb WHERE num = ?",
			1059 => "SELECT type FROM sub_acc_tb WHERE sub_acc_number = ?",
			1060 => "SELECT num, sub_acct_type FROM acct_type_tb",
			1061 => "SELECT db.parent_num, db.sub_e_code, db.account_num, db.sub_account_num, db.amount, db.dorc, db.cleared, db.counter, je.trans_date, je.payee_payer, je.memo, je.journal_code, je.remember FROM debitcredit_tb AS db, journal_entry_tb AS je WHERE db.ledger_code = je.ledger_code AND db.ledger_code = ? AND db.parent_num = je.ref_num AND (##?##) ORDER BY je.trans_date, db.parent_num, db.account_num, db.sub_e_code, db.sub_account_num, db.counter",
			1062 => "SELECT balance, bal_date FROM acct_bal_tb WHERE ledger_code = ? AND acct_num = ? AND sub_acct_num = ? AND bal_date <= ? ORDER BY bal_date DESC",
			1063 => "SELECT db.amount, db.dorc FROM debitcredit_tb AS db WHERE db.ledger_code = ? AND (##?##db.trans_date <= ?) AND db.account_num = ?##1?##",
			1064 => "SELECT report_type FROM report_type_tb WHERE id = ?",
			1065 => "SELECT name FROM sub_entity_tb WHERE ledger_code = ? AND code = ?",
			1066 => "SELECT name FROM journal_tb WHERE code = ?",
			1067 => "SET datestyle TO 'POSTGRES,US'",
			1068 => "SELECT name FROM sub_entity_tb WHERE ledger_code = ? AND code = ?",
			1069 => "SELECT sub_acc_name FROM sub_acc_tb WHERE sub_acc_number = ?",
			1070 => "SELECT name FROM sub_acct_type_tb WHERE num = ?",
			1071 => "SELECT max(bal_date) FROM acct_bal_tb WHERE ledger_code = ? and bal_date <= ?",
			1072 => "SELECT working_year( ? )",
			# account_type row SQL queries
				# account type activity using the stored procedures in the database for account_type row
				1073 => "SELECT account_type_activity(?, ?, ?, ?, ?)",
				1074 => "SELECT account_type_activity(?, ?, ?, ?, ?::text, ?)",
				1075 => "SELECT account_type_activity(?, ?, ?, ?, ?, ?, ?)",
				# account activity using the stored procedures in the database for account_type row
				1076 => "SELECT account_num, account_name(account_num), account_activity(account_num, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and trans_date between ? and ? group by account_num",
				1077 => "SELECT account_num, account_name(account_num), account_activity(account_num, ?, ?, ?, ?::text, ?) from debitcredit_tb where ledger_code = ? and sub_e_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and trans_date between ? and ? group by account_num",
				1078 => "SELECT account_num, account_name(account_num), account_activity(account_num, ?, ?, ?, ?::bool, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date between ? and ? group by account_num",
				1079 => "SELECT account_num, account_name(account_num), account_activity(account_num, ?, ?, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and sub_e_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date between ? and ? group by account_num",
				1080 => "SELECT account_num, account_name(account_num), sub_account_activity(account_num, ?, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and trans_date between ? and ? and sub_account_num = ? group by account_num",
				1081 => "SELECT account_num, account_name(account_num), sub_account_activity(account_num, ?, ?, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date between ? and ? and sub_account_num = ? group by account_num",
				1082 => "SELECT account_num, account_name(account_num), sub_account_type_activity(account_num, ?, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and trans_date between ? and ? and sub_account_num = (select sub_acc_number from sub_acc_tb where sub_acc_number = sub_account_num and type = ?) group by account_num",
				1083 => "SELECT account_num, account_name(account_num), sub_account_type_activity(account_num, ?, ?, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date between ? and ? and sub_account_num = (select sub_acc_number from sub_acc_tb where sub_acc_number = sub_account_num and type = ?) group by account_num",
				# account balance/activity using the stored procedures in the database for account_type row
				1084 => "SELECT account_num, account_name(account_num), account_balance_plus_activity(account_num, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and trans_date <= ? group by account_num UNION SELECT acct_num, account_name(acct_num), account_balance_plus_activity(acct_num, ?, ?, ?) from acct_bal_tb where ledger_code = ? and acct_num = (select number from account_tb where number = acct_num and type = ?) and bal_date <= ? group by acct_num",
				1085 => "SELECT account_num, account_name(account_num), account_balance_plus_activity(account_num, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date <= ? group by account_num UNION SELECT acct_num, account_name(acct_num), account_balance_plus_activity(acct_num, ?, ?, ?) from acct_bal_tb where ledger_code = ? and acct_num = (select number from account_tb where number = acct_num and type = ?) and acct_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = acct_bal_tb.acct_num and taxable = ?) and bal_date <= ? group by acct_num",
				# sub-account activity using the stored procedures in the database for account_type row
				1086 => "SELECT account_num, account_name(account_num), sub_account_num, sub_account_name(sub_account_num), sub_account_activity(account_num, sub_account_num, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and trans_date between ? and ? group by account_num, sub_account_num",
				1087 => "SELECT account_num, account_name(account_num), sub_account_num, sub_account_name(sub_account_num), sub_account_activity(account_num, sub_account_num, ?, ?, ?, ?::text, ?) from debitcredit_tb where ledger_code = ? and sub_e_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and trans_date between ? and ? group by account_num, sub_account_num",
				1088 => "SELECT account_num, account_name(account_num), sub_account_num, sub_account_name(sub_account_num), sub_account_activity(account_num, sub_account_num, ?, ?, ?, ?::bool, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date between ? and ? group by account_num, sub_account_num",
				1089 => "SELECT account_num, account_name(account_num), sub_account_num, sub_account_name(sub_account_num), sub_account_activity(account_num, sub_account_num, ?, ?, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and sub_e_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date between ? and ? group by account_num, sub_account_num",
				# sub-account balance/activity using the stored procedures in the database for account_type row
				1090 => "SELECT account_num, account_name(account_num), sub_account_num, sub_account_name(sub_account_num), sub_account_balance_plus_activity(account_num, sub_account_num, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and trans_date <= ? group by account_num, sub_account_num UNION SELECT acct_num, account_name(acct_num), sub_acct_num, sub_account_name(sub_acct_num), sub_account_balance_plus_activity(acct_num, sub_acct_num, ?, ?, ?) from acct_bal_tb where ledger_code = ? and acct_num = (select number from account_tb where number = acct_num and type = ?) and bal_date <= ? and sub_acct_num <> -2 group by acct_num, sub_acct_num",
				1091 => "SELECT account_num, account_name(account_num), sub_account_num, sub_account_name(sub_account_num), sub_account_balance_plus_activity(account_num, sub_account_num, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date <= ? group by account_num, sub_account_num UNION SELECT acct_num, account_name(acct_num), sub_acct_num, sub_account_name(sub_acct_num), sub_account_balance_plus_activity(acct_num, sub_acct_num, ?, ?, ?) from acct_bal_tb where ledger_code = ? and acct_num = (select number from account_tb where number = acct_num and type = ?) and acct_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = acct_bal_tb.acct_num and taxable = ?) and bal_date <= ? and sub_acct_num <> -2 group by acct_num, sub_acct_num",
			# account row SQL queries
				# account row activity using the stored procedures in the database for account row
				1092 => "SELECT sub_account_num, sub_account_name(sub_account_num), sub_account_activity(?, sub_account_num, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = ? and trans_date between ? and ? group by sub_account_num",
				1093 => "SELECT sub_account_num, sub_account_name(sub_account_num), sub_account_activity(?, sub_account_num, ?, ?, ?, ?::text, ?) from debitcredit_tb where ledger_code = ? and sub_e_code = ? and account_num = ? and trans_date between ? and ? group by sub_account_num",
				# sub-account balance/activity using the stored procedures in the database for account row
				1094 => "SELECT sub_account_num, sub_account_name(sub_account_num), sub_account_balance_plus_activity(?, sub_account_num, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = ? and trans_date <= ? group by sub_account_num UNION SELECT sub_acct_num, sub_account_name(sub_acct_num), sub_account_balance_plus_activity(?, sub_acct_num, ?, ?, ?) from acct_bal_tb where ledger_code = ? and acct_num = ? and bal_date <= ? and sub_acct_num <> -2 group by sub_acct_num",
			# sub-account row SQL queries
				# sub-account row activity using the stored procedures in the database for sub-account row
				1095 => "SELECT account_num, account_name(account_num), sub_account_activity(account_num, ?, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and sub_account_num = ? and trans_date between ? and ? group by account_num",
				1096 => "SELECT account_num, account_name(account_num), sub_account_activity(account_num, ?, ?, ?, ?, ?::bool, ?) from debitcredit_tb where ledger_code = ? and sub_account_num = ? and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date between ? and ? group by account_num",
				1097 => "SELECT account_num, account_name(account_num), sub_account_activity(account_num, ?, ?, ?, ?, ?::text, ?) from debitcredit_tb where ledger_code = ? and sub_e_code = ? and sub_account_num = ? and trans_date between ? and ? group by account_num",
				1098 => "SELECT account_num, account_name(account_num), sub_account_activity(account_num, ?, ?, ?, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and sub_e_code = ? and sub_account_num = ? and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date between ? and ? group by account_num",
				# sub-account balance/activity using the stored procedures in the database for sub-account row
				1099 => "SELECT account_num, account_name(account_num), sub_account_balance_plus_activity(account_num, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and trans_date <= ? group by account_num UNION SELECT account_num, account_name(account_num), sub_account_balance_plus_activity(account_num, ?, ?, ?, ?) from acct_bal_tb where ledger_code = ? and bal_date <= ? group by acct_num",
				1100 => "SELECT account_num, account_name(account_num), sub_account_balance_plus_activity(account_num, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date <= ? group by account_num UNION SELECT account_num, account_name(account_num), sub_account_balance_plus_activity(account_num, ?, ?, ?, ?) from acct_bal_tb where ledger_code = ? and acct_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = acct_bal_tb.acct_num and taxable = ?) and bal_date <= ? group by acct_num",
			# sub-accounts of account_type row SQL queries
				# activity
				1101 => "SELECT sub_account_type_num(sub_account_num), sub_account_type_name(sub_account_type_num(sub_account_num)), sub_account_num, sub_account_name(sub_account_num), sub_account_account_type_activity(?, sub_account_num, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and trans_date between ? and ? group by sub_account_type_num, sub_account_num",
				1102 => "SELECT sub_account_type_num(sub_account_num), sub_account_type_name(sub_account_type_num(sub_account_num)), sub_account_num, sub_account_name(sub_account_num), sub_account_account_type_activity(?, sub_account_num, ?, ?, ?, ?::bool, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date between ? and ? group by sub_account_type_num, sub_account_num",
				1103 => "SELECT sub_account_type_num(sub_account_num), sub_account_type_name(sub_account_type_num(sub_account_num)), sub_account_num, sub_account_name(sub_account_num), sub_account_account_type_activity(?, sub_account_num, ?, ?, ?, ?::text, ?) from debitcredit_tb where ledger_code = ? and sub_e_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and trans_date between ? and ? group by sub_account_type_num, sub_account_num",
				1104 => "SELECT sub_account_type_num(sub_account_num), sub_account_type_name(sub_account_type_num(sub_account_num)), sub_account_num, sub_account_name(sub_account_num), sub_account_account_type_activity(?, sub_account_num, ?, ?, ?, ?::bool, ?::text, ?) from debitcredit_tb where ledger_code = ? and sub_e_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date between ? and ? group by sub_account_type_num, sub_account_num",
				# ----------
				1105 => "SELECT sub_account_type_num(sub_account_num), sub_account_type_name(sub_account_type_num(sub_account_num)), sub_account_num, sub_account_name(sub_account_num), account_num, account_name(account_num), sub_account_activity(account_num, sub_account_num, ?, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and trans_date between ? and ? group by sub_account_type_num, sub_account_num, account_num",
				1106 => "SELECT sub_account_type_num(sub_account_num), sub_account_type_name(sub_account_type_num(sub_account_num)), sub_account_num, sub_account_name(sub_account_num), account_num, account_name(account_num), sub_account_activity(account_num, sub_account_num, ?, ?, ?, ?::bool, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date between ? and ? group by sub_account_type_num, sub_account_num, account_num",
				1107 => "SELECT sub_account_type_num(sub_account_num), sub_account_type_name(sub_account_type_num(sub_account_num)), sub_account_num, sub_account_name(sub_account_num), account_num, account_name(account_num), sub_account_activity(account_num, sub_account_num, ?, ?, ?, ?::text, ?) from debitcredit_tb where ledger_code = ? and sub_e_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and trans_date between ? and ? group by sub_account_type_num, sub_account_num, account_num",
				1108 => "SELECT sub_account_type_num(sub_account_num), sub_account_type_name(sub_account_type_num(sub_account_num)), sub_account_num, sub_account_name(sub_account_num), account_num, account_name(account_num), sub_account_activity(account_num, sub_account_num, ?, ?, ?, ?::bool, ?::text, ?) from debitcredit_tb where ledger_code = ? and sub_e_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date between ? and ? group by sub_account_type_num, sub_account_num, account_num",
				# balance
				1109 => "SELECT sub_account_type_num(sub_account_num), sub_account_type_name(sub_account_type_num(sub_account_num)), sub_account_num, sub_account_name(sub_account_num), account_num, account_name(account_num), sub_account_balance_plus_activity(account_num, sub_account_num, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and trans_date <= ? group by sub_account_type_num, sub_account_num, account_num UNION SELECT sub_account_type_num(sub_acct_num), sub_account_type_name(sub_account_type_num(sub_acct_num)), sub_acct_num, sub_account_name(sub_acct_num), acct_num, account_name(acct_num), sub_account_balance_plus_activity(acct_num, sub_acct_num, ?, ?, ?) from acct_bal_tb where ledger_code = ? and acct_num = (select number from account_tb where number = acct_num and type = ?) and bal_date <= ? group by sub_account_type_num, sub_acct_num, acct_num",
				1110 => "SELECT sub_account_type_num(sub_account_num), sub_account_type_name(sub_account_type_num(sub_account_num)), sub_account_num, sub_account_name(sub_account_num), account_num, account_name(account_num), sub_account_balance_plus_activity(account_num, sub_account_num, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date <= ? group by sub_account_type_num, sub_account_num, account_num UNION SELECT sub_account_type_num(sub_acct_num), sub_account_type_name(sub_account_type_num(sub_acct_num)), sub_acct_num, sub_account_name(sub_acct_num), acct_num, account_name(acct_num), sub_account_balance_plus_activity(acct_num, sub_acct_num, ?, ?, ?) from acct_bal_tb where ledger_code = ? and acct_num = (select number from account_tb where number = acct_num and type = ?) and acct_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = acct_bal_tb.acct_num and taxable = ?) and bal_date <= ? group by sub_account_type_num, sub_acct_num, acct_num",
				# -----
				1111 => "SELECT sub_account_type_num(sub_account_num), sub_account_type_name(sub_account_type_num(sub_account_num)), sub_account_num, sub_account_name(sub_account_num), sub_account_act_type_balance_plus_activity(?, sub_account_num, ?, ?, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and trans_date <= ? group by sub_account_type_num, sub_account_num UNION SELECT sub_account_type_num(sub_acct_num), sub_account_type_name(sub_account_type_num(sub_acct_num)), sub_acct_num, sub_account_name(sub_acct_num), sub_account_act_type_balance_plus_activity(?, sub_acct_num, ?, ?, ?) from acct_bal_tb where ledger_code = ? and acct_num = (select number from account_tb where number = acct_num and type = ?) and bal_date <= ? group by sub_account_type_num, sub_acct_num",
				1112 => "SELECT sub_account_type_num(sub_account_num), sub_account_type_name(sub_account_type_num(sub_account_num)), sub_account_num, sub_account_name(sub_account_num), sub_account_act_type_balance_plus_activity(?, sub_account_num, ?, ?, ?::bool, ?) from debitcredit_tb where ledger_code = ? and account_num = (select number from account_tb where number = account_num and type = ?) and account_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = account_num and taxable = ?) and trans_date <= ? group by sub_account_type_num, sub_account_num UNION SELECT sub_account_type_num(sub_acct_num), sub_account_type_name(sub_account_type_num(sub_acct_num)), sub_acct_num, sub_account_name(sub_acct_num), sub_account_act_type_balance_plus_activity(?, sub_acct_num, ?, ?, ?::bool, ?) from acct_bal_tb where ledger_code = ? and acct_num = (select number from account_tb where number = acct_num and type = ?) and acct_num = (select acct_num from local_accounts_tb where ledger_code = ? and acct_num = acct_bal_tb.acct_num and taxable = ?) and bal_date <= ? group by sub_account_type_num, sub_acct_num",
			# account type balances
				1113 => "SELECT account_type_balance_plus_activity(?, ?, ?, ?)",
			# more account type activity
				1114 => "SELECT account_type_activity(?, ?, ?, ?, ?::bool, ?)",
				1115 => "SELECT acct_type_sub_acct_type_activity(?, ?, ?, ?, ?, ?)",
				1116 => "SELECT acct_type_sub_acct_type_activity(?, ?, ?, ?, ?, ?:bool, ?)",
			# more account type balances
				1117 => "SELECT account_type_balance_plus_activity(?, ?, ?, ?::bool, ?)",
				1118 => "SELECT acct_type_sub_acct_type_bp(?, ?, ?, ?, ?)",
				1119 => "SELECT acct_type_sub_acct_type_bp(?, ?, ?, ?, ?::bool, ?)",
			# sub-account activity for account type
				1120 => "SELECT sub_account_account_type_activity(?, ?, ?, ?, ?, ?)",
				1121 => "SELECT sub_account_account_type_activity(?, ?, ?, ?, ?, ?::bool, ?)",
			# sub-account balances/activity for account type
				1122 => "SELECT sub_account_act_type_balance_plus_activity(?, ?, ?, ?, ?)",
				1123 => "SELECT sub_account_act_type_balance_plus_activity(?, ?, ?, ?, ?::bool, ?)",
      # lookups needed for the detailed view customization screen.
			1124 => "SELECT name FROM account_tb WHERE number IN (SELECT acct_num FROM local_accounts_tb WHERE ledger_code = ? AND sub_entity_code IS NULL) ORDER BY name",
			1125 => "SELECT number FROM account_tb WHERE number IN (SELECT acct_num FROM local_accounts_tb WHERE ledger_code = ? AND sub_entity_code IS NULL) ORDER BY number",
      1126 => "SELECT sub_acc_name FROM sub_acc_tb ORDER BY sub_acc_name",
      1127 => "SELECT sub_acc_number FROM sub_acc_tb ORDER BY sub_acc_number",
      1128 => "SELECT DISTINCT(payee_payer) FROM journal_entry_tb WHERE ledger_code = ? ORDER BY payee_payer",
		     );

my %report_classes;
%report_classes = ( "custom" => "Custom", "cash" => "Cash Flow", "income" => "Income Statement", "balance" => "Balance Sheet", "acct_balance" => "Accounts Balance Sheet", "acct_activity" => "Accounts Activity Sheet" );
my @report_classes;
@report_classes = ( "custom", "cash", "income", "balance", "acct_balance", "acct_activity" );
my %report_types;
%report_types = ( "r" => "Regular Report", "d" => "Detailed View" );
my @report_types;
@report_types = ( "r", "d" );
my @report_levels;
@report_levels = ( "account_type", "account", "sub_account", "accounts_of", "sub_accounts_of_type", "sub_accounts_of_account", "begin_account_type_balance", "end_account_type_balance", "begin_account_balance", "end_account_balance", "begin_sub_account_balance", "end_sub_account_balance" );
my @account_type_levels;
@account_type_levels = ( "nothing", "accounts", "accounts, sub-accounts" );
my @account_levels;
@account_levels = ( "nothing", "sub-accounts" );
my @accounts_of_levels;
@accounts_of_levels = ( "nothing", "sub-accounts" );
my @sub_account_levels;
@sub_account_levels = ( "nothing", "accounts" );
my @sub_accounts_of_type_levels;
@sub_accounts_of_type_levels = ( "nothing", "accounts" );
my @sub_accounts_of_account_levels;
@sub_accounts_of_account_levels = ( "not available" );
my %report_levels;
%report_levels = ( "account_type" => \@account_type_levels, "account" => \@account_levels, "sub_account" => \@sub_account_levels, "accounts_of" => \@accounts_of_levels, "sub_accounts_of_type" => \@sub_accounts_of_type_levels, "sub_accounts_of_account" => \@sub_accounts_of_account_levels, "begin_account_type_balance" => \@account_type_levels, "end_account_type_balance" => \@account_type_levels, "begin_account_balance" => \@account_levels, "end_account_balance" => \@account_levels, "begin_sub_account_balance" => \@sub_account_levels, "end_sub_account_balance" => \@sub_account_levels );
my %regular_cell_types;
%regular_cell_types = ( "text" => "Text", "total" => "Total", "compare" => "Compare Rows", "account_type" => "Account Type", "account" => "Account", "sub_account" => "Sub Account", "accounts_of" => "Accounts of Account Type", "sub_accounts_of_type" => "Sub Accounts of Account Type", "sub_accounts_of_account" => "Sub Accounts of Account", "begin_account_type_balance" => "Beginning Account Type Balance", "end_account_type_balance" => "Ending Account Type Balance", "begin_account_balance" => "Beginning Account Balance", "end_account_balance" => "Ending Account Balance", "begin_sub_account_balance" => "Beginning Sub Account Balance", "end_sub_account_balance" => "Ending Sub Account Balance", "line" => "Line" );
my @regular_cell_types;
@regular_cell_types = ( "text", "total", "compare", "account_type", "account", "sub_account", "accounts_of", "sub_accounts_of_type", "sub_accounts_of_account", "begin_account_type_balance", "end_account_type_balance", "begin_account_balance", "end_account_balance", "line" ); # "begin_sub_account_balance", "end_sub_account_balance", (to be added later)
my %detailed_cell_types;
%detailed_cell_types = ( "text" => "Text", "line" => "Line", "detail" => "Detail" );
my @detailed_cell_types;
@detailed_cell_types = ( "text", "line", "detail" );
my @font_names;
@font_names = ( "helvetica" );
my %font_names;
%font_names = ( "helvetica" => "Helvetica" );
my @font_sizes;
@font_sizes = ( "+2", "+1", "+0", "-1", "-2" );
my %font_sizes;
%font_sizes = ( "+2" => "+2", "+1" => "+1", "+0" => "nothing", "-1" => "-1", "-2" => "-2" );
my @fg_colors;
@fg_colors = ( "black", "blue", "red" );
my %fg_colors;
%fg_colors = ( "black" => "Black", "blue" => "Blue", "red" => "Red" );
my @bg_colors;
@bg_colors = ( "white", "yellow", "pink" );
my %bg_colors;
%bg_colors = ( "white" => "White", "yellow" => "Yellow", "pink" => "Pink" );
my %column_names;
%column_names = ( 0 => "Zero", 1 => "One", 2 => "Two", 3 => "Three", 4 => "Four", 5 => "Five", 6 => "Six", 7 => "Seven", 8 => "Eight", 9 => "Nine", 10 => "Ten", 11 => "Eleven", 12 => "Twelve", 13 => "Thirteen", 14 => "Fourteen", 15 => "Fifteen", 16 => "Sixteen", 17 => "Seventeen", 18 => "Eighteen", 19 => "Nineteen", 20 => "Twenty", 21 => "Twenty One", 22 => "Twenty Two", 23 => "Twenty Three", 24 => "Twenty Four", 25 => "Twenty Five", 26 => "Twenty Six", 27 => "Twenty Seven", 28 => "Twenty Eight", 29 => "Twenty Nine", 30 => "Thirty", 31 => "Thirty One", 32 => "Thirty Two", 33 => "Thirty Three", 34 => "Thirty Four", 35 => "Thirty Five", 36 => "Thirty Six" );
my @column_headers;
@column_headers = ( "sub_entities", "sub_accounts", "sub_account_types" );
my %column_headers;
%column_headers = ( "sub_entities" => "Sub Entities", "sub_accounts" => "Sub Accounts", "sub_account_types" => "Sub Account Types" );
my @negative_formats;
@negative_formats = ( "0", "1", "2" );
my %negative_formats;
%negative_formats = ( "0" => "<font color=\"red\">(xxx.xx)</font>", "1" => "<font color=\"red\">-</font>xxx.xx", "2" => "xxx.xx<font color=\"red\">-</font>" );
my $max_dates;
$max_dates = 36;   # the maximum number of columns that we are currently supporting in the reporting system.
my $min_underline_length;
$min_underline_length =  5;  # the min and max lengths that underlines can range within.
my $max_underline_length;
$max_underline_length = 25;
my $default_underline_length;
$default_underline_length = 15;
my %t_or_f;
%t_or_f = ( 0 => "false", 1 => "true", "true" => "true", "false" => "false" );  # a mapping that will allow us to map 0 to false and 1 to true for the sql statements.
my @account_formats;
@account_formats = ( "0", "1", "2", "3" );
my %account_formats;
%account_formats = ( "0" => "Name - Number", "1" => "Number - Name", "2" => "Name only", "3" => "Number only" );
my @precision_formats;
@precision_formats = ( "1", "10", "100", "1000", "10000", "100000", "1000000", "1000000000" );
my %precision_formats;
%precision_formats = ( "1" => "1", "10" => "10", "100" => "100", "1000" => "1,000", "10000" => "10,000", "100000" => "100,000", "1000000" => "1,000,000", "1000000000" => "1,000,000,000" );
my %precision_names;
%precision_names = ( "10" => "tens", "100" => "hundreds", "1000" => "thousands", "10000" => "ten-thousands", "100000" => "hundred-thousands", "1000000" => "millions", "1000000000" => "billions" );
my %true_false;
%true_false = ( "true" => 1, "false" => 0, 1 => 1, 0 => 0 );
my %column_data_sql;
%column_data_sql = ( "sub_entities" => 1033, "sub_accounts" => 1034, "sub_account_types" => 1035 );
# detailed view customization settings values
my @field_names;
@field_names = ( "account_name", "db.account_num", "db.amount", "db.cleared", "credit_amount", "je.trans_date", "debit_amount", "je.journal_code", "je.memo", "je.payee_payer", "je.ref_num", "je.remember", "sub_account_name", "db.sub_account_num", "db.sub_e_code" );
my %field_names;
%field_names = ( "account_name" => "Account Name", "je.ref_num" => "Reference Number", "je.journal_code" => "Journal Code", "je.trans_date" => "Date", "je.payee_payer" => "Payee-Payer", "je.memo" => "Memo", "je.remember" => "Remember", "db.sub_e_code" => "Sub-Entity Code", "db.account_num" => "Account Number", "sub_account_name" => "Sub-Account Name", "db.sub_account_num" => "Sub-Account Number", "db.amount" => "Amount", "credit_amount" => "Credit", "debit_amount" => "Debit", "db.cleared" => "Cleared" );
my %sql_translations;
%sql_translations = ( "account_name"     => "db.account_num in (select number from account_tb where name <operator> <value>)",
                         "sub_account_name" => "db.sub_account_num in (select sub_acc_number from sub_acc_tb where sub_acc_name <operator> <value>)",
                         "credit_amount"    => "db.amount <operator> <value> AND db.dorc = false",
                         "debit_amount"     => "db.amount <operator> <value> AND db.dorc = true" );
my @operator_list;
@operator_list = ( "=", ">", "<", ">=", "<=", "<>", "like" );
my %operator_list;
%operator_list = ( ">" => "GREATER-THAN (>)", "<" => "LESS-THAN (<)", ">=" => "GREATER-THAN or EQUAL-TO (>=)", "<=" => "LESS-THAN or EQUAL-TO (<=)", "=" => "EQUAL-TO (=)", "<>" => "NOT EQUAL-TO (<>)", "like" => "LIKE" );
my @connector_list;
@connector_list = ( "AND", "OR" );
# detailed view display settings values
my @column_field_names;
@column_field_names = ( "Ref #", "Journal", "Date", "Payee-Payer", "Memo", "SbE", "Account", "Sub-account", "Cleared", "Debit", "Credit" );
my %header_map;
%header_map = ( "Ref #" => 0, "Journal" => 14, "Date" => 11, "Payee-Payer" => 12, "Memo" => 13, "SbE" => 1, "Account" => 3, "Sub-account" => 5, "Cleared" => 9, "Debit" => 7, "Credit" => 7 );

# commonly used compiled regular expressions
my $reportNumRE = qr/^(report_\d+)$/;
my $minusRE = qr/\-/;
my $accountRE = qr/\<ACCOUNT\>/;
my $subAccountRE = qr/\<SUBACCOUNT\>/;
my $subAccountTypeRE = qr/\<SUBACCOUNTTYPE\>/;
my $operatorRE = qr/^(\+|-|%|\*|\/)$/;
my $openParenRE = qr/^\($/;
my $closeParenRE = qr/^\)$/;
my $tokenRE = qr/^R\d+|-?\d+\.\d+|-?\d+$/;
my $rowIdentifierRE = qr/^R\d+$/;
my $rowIdentifier2RE = qr/^R(\d+)/;
my $substitutionRE = qr/\#\#\?\#\#/;
my $firstSubstitutionRE = qr/\#\#1\?\#\#/;
my $secondSubstitutionRE = qr/\#\#2\?\#\#/;

# regular view display settings values
my $font_underline_width;
$font_underline_width = 10;
my %hide_numbers;
%hide_numbers = {};
$hide_numbers{'account_type'} = {};
$hide_numbers{'account_type'}{'0'} = 1;

$hide_numbers{'accounts_of'} = {};
$hide_numbers{'accounts_of'}{'0'} = 1;

$hide_numbers{'begin_account_type_balance'} = {};
$hide_numbers{'begin_account_type_balance'}{'0'} = 1;

$hide_numbers{'end_account_type_balance'} = {};
$hide_numbers{'end_account_type_balance'}{'0'} = 1;

$hide_numbers{'sub_accounts_of_type'} = {};
$hide_numbers{'sub_accounts_of_type'}{'0'} = 1;
$hide_numbers{'sub_accounts_of_type'}{'1'} = 1;

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";

    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";

    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
($db_name = $dbObj->dbName) =~ s/^(")([^"]+)(")$/$2/;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $db_type;
$db_type = "Pg";
my $dsn;
$dsn = "dbi:$db_type:dbname=$db_name;host=$db_host";

my $email_address;
$email_address = "james\@pcxperience.com";  # JavaScript error messages should be sent here.

%input = ();
cgi_lib::ReadParse(*input);  # gather the data from the client

use HTMLObject::Base;
use HTMLObject::ReadCookie;	# the module used to read cookies sent to us from the client.
use Storable;		# the module that will allow me to dump data to disk and recover later.  The file names will be stored in the cookie.
use POSIX qw(tmpnam);	# the tmpnam function which will return a file name for a temporary file supposed to be unique.
use DBI;	# Using new Database engine for future compatibility.

# turn on debugging in Apache::DBI
#$Apache::DBI::DEBUG = 0;

my $cookies;
$cookies = HTMLObject::ReadCookie->new;
my %ch;
%ch = $cookies->getCookies(cookies => ['pcx_language']);

my $lang;
$lang = (exists $ch{pcx_language} ? $ch{pcx_language} : 'en');
my $langObj;
$langObj = Portal::Language->new(lang => $lang);
if ($langObj->didErrorOccur)
{
  my_die($langObj->errorMessage);
}
my $applangObj;
$applangObj = Portal::Accounting::Language->new(lang => $lang);
if ($applangObj->didErrorOccur)
{
  my_die($applangObj->errorMessage);
}

my $methods;  # The Portal::Methods object.
$methods = Portal::Methods->new(langObj => $langObj);
if ($methods->didErrorOccur)
{
  my_die($methods->errorMessage);
}

# merge the langObj and applangObj together.
mergeLanguageObjects(portalLangObj => $langObj, appLangObj => $applangObj);
$langObj = $applangObj;

if (!exists $input{'command'})
{
  my_die("Error: Report.cgi - command missing", 4003, "Access Denied!");
}
else
{
  my $dbh;  # the handle of the database.
  # connect to the database.  The AutoCommit option will force transactions to be used.
  $dbh = DBI->connect($dsn, $db_user, $db_passwd, { RaiseError => 0, AutoCommit => 0 }) || my_db_die("DBI Error", 4004, "Database: $db_name\@$db_host");

  my $currentDate = `/bin/date +%m-%d-%Y`;
  chomp $currentDate;

  # force the database session to use Postgres/US convention date styles
  my $sth = $dbh->prepare( $sql_statements{1067} ) || my_db_die("Error: Report.cgi", 4007, "SQL Statement = '$sql_statements{1067}'");
  my $rc = $sth->execute() || my_db_die("Error: Report.cgi", 4008, "SQL Statement = '$sql_statements{1067}'");

  my $command = $input{'command'};
  if ($command eq "customize_report")
  {
    # validate the input values
    my $windowName = $input{windowName};
    if ($windowName !~ /$reportNumRE/)
    {
      my_die("Error:  Report.cgi - customize_report", 4006, "windowName = '$windowName' is invalid at line " . __LINE__);
    }
    my $memorized = $input{'memorized'};
    if (length $memorized == 0)
    {
      my_die("Error: Report.cgi - customize_report", 4029, "Memorized value required at line " . __LINE__);
    }
    my $id = $input{'id'};
    if (length $id == 0)
    {
      my_die("Error: Report.cgi - customize_report", 4017, "Report ID required at line " . __LINE__);
    }

    my $report_name;
    if ($memorized eq "true")
    {
      # check to make sure the "id" value is pipe separated consisting of the id and the name of the report
      if (not exists $input{'report_name'} || length $input{'report_name'} == 0)
      {
        my_die("Error: Report.cgi - customize_report", 4030, "Report name must be specified with report ID at line " . __LINE__);
      }
      $report_name = $input{'report_name'};
    }
    else
    {
      # get the report name from the database
      my $sth = $dbh->prepare( $sql_statements{1023} ) || my_db_die("Error: Report.cgi - customize_report", 4007, "SQL Statement = '$sql_statements{1023}'");
      my $rc = $sth->execute( $id ) || my_db_die("Error: Report.cgi - customize_report", 4008, "SQL Statement = '$sql_statements{1023}'");
      my @temp_row = $sth->fetchrow_array;
      $report_name = $input{ledger} . " - " . $temp_row[0] . " - run on " . $currentDate;
      $sth->finish;
    }

    # check to make sure we were given the ledger_db name
    my $ledger = $input{'ledger'};
    if (length $ledger == 0)
    {
      my_die("Error: Report.cgi - customize_report", 4022, "The Ledger Code is required at line " . __LINE__);
    }

    my $ledger_name = get_ledger_name( ledger_db => $ledger, dbh => $dbh );
    my $ledger_code = get_ledger_code( ledger_name => $ledger_name, dbh => $dbh );
    my $ledger_color = get_ledger_color( ledger_code => $ledger_code, dbh => $dbh );

    # create generic structure to hold all values and their defaults in order to set them using memorized or pre-configured values
    my %report_attributes;
    $report_attributes{'dbh'} = $dbh;
    $report_attributes{'id'} = $id;
    $report_attributes{'report_name'} = $report_name;

    $report_attributes{'ledger'} = $ledger;
    $report_attributes{'ledger_name'} = $ledger_name;
    $report_attributes{'ledger_code'} = $ledger_code;
    $report_attributes{'ledger_color'} = $ledger_color;

    # get the report default values from the database
    my $sth = $dbh->prepare( $sql_statements{1006} ) || my_db_die("Error: Report.cgi - customize_report", 4007, "SQL Statement = '$sql_statements{1006}'");
    my $rc = $sth->execute( $id ) || my_db_die("Error: Report.cgi - customize_report", 4008, "SQL Statement = '$sql_statements{1006}'");
    my @temp_row = $sth->fetchrow_array;
    $sth->finish;
    $report_attributes{'report_class'} = $temp_row[1];
    $report_attributes{'report_type'} = $temp_row[2];
    $report_attributes{'column_variable'} = $temp_row[3];
    $report_attributes{'column_variable_data'} = $temp_row[4];
    # split the column_variable to add the -1 sub-account to the column_variable_data if sub_accounts exists
    my @temp_column_variable = split /,/, $report_attributes{'column_variable'};
    for (my $variable_count=0; $variable_count < scalar @temp_column_variable; $variable_count++)
    {
      if ($temp_column_variable[$variable_count] eq "sub_accounts")
      {
        # split the column_variable_data and insert "-1" at the beginning of the sub_accounts string
        my @temp_column_variable_data = split /\|/, $report_attributes{'column_variable_data'};
        $temp_column_variable_data[$variable_count] = "-1," . $temp_column_variable_data[$variable_count];
        # reconstruct the column_variable_data string
        $report_attributes{'column_variable_data'} = join "|", @temp_column_variable_data;
      }
    }
    $report_attributes{'date_range'} = $temp_row[5];
    $report_attributes{'display_dates'} = $temp_row[6];
    $report_attributes{'max_columns'} = $temp_row[7];
    $report_attributes{'min_columns'} = $temp_row[8];
    $report_attributes{'text_font'} = $temp_row[9];
    $report_attributes{'text_size'} = $temp_row[10];
    $report_attributes{'data_font'} = $temp_row[11];
    $report_attributes{'data_size'} = $temp_row[12];
    $report_attributes{'total_column'} = $temp_row[13];
    $report_attributes{'default_fg_color'} = $temp_row[14];
    $report_attributes{'default_bg_color'} = $temp_row[15];
    $report_attributes{'underline_length'} = ($temp_row[16] == -1 ? $default_underline_length : $temp_row[16]);

    # attributes that need default settings to start out with
    $report_attributes{'fg_color'} = "default";
    $report_attributes{'bg_color'} = "default";

    $report_attributes{reportWidth} = ($report_attributes{'report_type'} eq "r" ? "55" : "100");  # default to 55%

    # attributes not in report_type_tb but that are in memorized_report_tb and need to have defaults if this is not a memorized report
    $report_attributes{'account_format'} = "0";
    $report_attributes{'data_precision'} = "1";
    $report_attributes{'negative_format'} = "-1";
    $report_attributes{'dates_entered'} = ();
    $report_attributes{'column_data'} = ();
    $report_attributes{'sql_statement'} = "";
    $report_attributes{'headers'} = "";
    for (my $column_count=0; $column_count < scalar @column_field_names; $column_count++)
    {
      $report_attributes{'headers'} .= $column_field_names[$column_count];
      if ($column_count < scalar @column_field_names - 1)
      {
        $report_attributes{'headers'} .= "|";
      }
    }
    $report_attributes{'sorted_by'} = "Ref #";
    $report_attributes{'sorted_order'} = "a";
    $report_attributes{'subtotal'} = "false";
    $report_attributes{'subtotal_type'} = "d";
    $report_attributes{'border'} = "0";

    $report_attributes{windowName} = $windowName;

    $report_attributes{displayZeroRows} = 0;  # default to not showing zero rows.

    if ($memorized eq "true")
    {
      # overlay the memorized values onto the current default values for the report
      $sth = $dbh->prepare( $sql_statements{1027} ) || my_db_die("Error: Report.cgi - customize_report", 4007, "SQL Statement = '$sql_statements{1027}'");
      $rc = $sth->execute( $ledger_code, $id, $report_name ) || my_db_die("Error: Report.cgi - customize_report", 4008, "SQL Statement = '$sql_statements{1027}'");
      @temp_row = $sth->fetchrow_array;
      $sth->finish;
      $report_attributes{'column_type'} = $temp_row[0];
      $report_attributes{'column_data'} = $temp_row[1];
      $report_attributes{'dates_entered'} = [ split(/\|/, $temp_row[2]) ];
      $report_attributes{'fg_color'} = $temp_row[3];
      $report_attributes{'bg_color'} = $temp_row[4];
      $report_attributes{'text_size'} = $temp_row[5];
      $report_attributes{'data_size'} = $temp_row[6];
      $report_attributes{'text_font'} = $temp_row[7];
      $report_attributes{'data_font'} = $temp_row[8];
      $report_attributes{'underline_length'} = ($temp_row[9] == -1 ? $default_underline_length : $temp_row[9]);
      $report_attributes{'account_format'} = $temp_row[10];
      $report_attributes{'data_precision'} = $temp_row[11];
      $report_attributes{'negative_format'} = $temp_row[12];
      $report_attributes{'total_column'} = $t_or_f{$temp_row[13]};
      $report_attributes{'sql_statement'} = $temp_row[14];
      $report_attributes{'headers'} = $temp_row[15];
      $report_attributes{'sorted_by'} = $temp_row[16];
      $report_attributes{'sorted_order'} = $temp_row[17];
      $report_attributes{'subtotal'} = $t_or_f{$temp_row[18]};
      $report_attributes{'subtotal_type'} = $temp_row[19];

      my @tmp_names = ();
      # lookup the names for the column_data values of column_type
      if ($report_attributes{'column_type'} eq "sub_entities")
      {
        my @codes = split /,/, $report_attributes{'column_data'};
        foreach my $code (@codes)
        {
          if ($code eq "<non_sub_entity>")
          {
            @tmp_names[scalar @tmp_names] = "<i>None</i>";
          }
          else
          {
            my $tmp_sql_statement = $sql_statements{1068};
            $sth = $dbh->prepare( $tmp_sql_statement ) || my_db_die("Error: Report.cgi - customize_report", 4007, "SQL Statement = '$tmp_sql_statement'");
            $rc = $sth->execute( $ledger_code, $code ) || my_db_die("Error: Report.cgi - customize_report", 4008, "SQL Statement = '$tmp_sql_statement'");
            while (@temp_row = $sth->fetchrow_array)
            {
              @tmp_names[scalar @tmp_names] = $temp_row[0];
            }
          }
        }
      }
      elsif ($report_attributes{'column_type'} eq "sub_accounts")
      {
        my @codes = split /,/, $report_attributes{'column_data'};
        foreach my $code (@codes)
        {
          my $tmp_sql_statement = $sql_statements{1069};
          $sth = $dbh->prepare( $tmp_sql_statement ) || my_db_die("Error: Report.cgi - customize_report", 4007, "SQL Statement = '$tmp_sql_statement'");
          $rc = $sth->execute( $code ) || my_db_die("Error: Report.cgi - customize_report", 4008, "SQL Statement = '$tmp_sql_statement'");
          while (@temp_row = $sth->fetchrow_array)
          {
            @tmp_names[scalar @tmp_names] = $temp_row[0];
          }
        }
      }
      elsif ($report_attributes{'column_type'} eq "sub_account_types")
      {
        my @codes = split /,/, $report_attributes{'column_data'};
        foreach my $code (@codes)
        {
          my $tmp_sql_statement = $sql_statements{1070};
          $sth = $dbh->prepare( $tmp_sql_statement ) || my_db_die("Error: Report.cgi - customize_report", 4007, "SQL Statement = '$tmp_sql_statement'");
          $rc = $sth->execute( $code ) || my_db_die("Error: Report.cgi - customize_report", 4008, "SQL Statement = '$tmp_sql_statement'");
          while (@temp_row = $sth->fetchrow_array)
          {
            @tmp_names[scalar @tmp_names] = $temp_row[0];
          }
        }
      }
      if ($report_attributes{'column_type'} ne "nothing")
      {
        my @tmp_data = split /,/, $report_attributes{'column_data'};
        for (my $count=0; $count < scalar @tmp_data; $count++)
        {
          $report_attributes{'column_data_names'} .= $tmp_names[$count] . " - " . ($tmp_data[$count] ne "<non_sub_entity>" ? $tmp_data[$count] : "<i>None</i>");
          if ($count < scalar @tmp_data - 1)
          {
            $report_attributes{'column_data_names'} .= "|";
          }
        }
      }

      # gather the data
      if ($report_attributes{'report_type'} eq "r")
      {
        # regular report
        gather_regular_report_data( %report_attributes );
      }
      else
      {
        # detailed view report
        gather_detailed_view_report_data( %report_attributes );
      }
    }
    else
    {
      customize_report_display( %report_attributes );
    }
  }
  elsif ($command eq "memorize_report")
  {
    # The report needs to pass in the report_type.  A function memorized_report_display needs to be created that will regenerate the form that asks for the report name and has all the necessary
    # items in hidden items in the form for submital back to us to process after they change the report name.

    eval ("use HTMLObject::Normal;");
    if ($@)
    {
      my_die("Error: Report.cgi - Eval", 4002, "Error: $@ at line " . __LINE__);
    }

    my $doc = HTMLObject::Normal->new();

    $doc->setJavascriptErrorHandlerEmail(email => $email_address);

    my ($report_attributes, $error_string) = get_report_attributes(dbh => $dbh, memorize => 1);
    my %report_attributes = %{$report_attributes};
    if (length $error_string > 0)
    {
      foreach my $attr (("display_dates", "date_range", "total_column"))
      {
        $report_attributes{$attr} = $true_false{$report_attributes{$attr}};
      }
      memorized_report_display( %report_attributes, dbh => $dbh, error_string => $error_string );
    }
    else
    {
      my $report_type = $report_attributes{'report_type'};
      # check and make sure that a memorized report with the same name but different report id doesn't exist.
      my $sth = $dbh->prepare( $sql_statements{1032} ) || my_db_die("Error: Report.cgi - memorize_report", 4007, "SQL Statement = '$sql_statements{1032}'");
      my $rc = $sth->execute($report_attributes{'report_name'}, $report_attributes{'ledger_code'}) || my_db_die("Error: Report.cgi - memorize_report", 4008, "SQL Statement = '$sql_statements{1032}', Report Name = '$report_attributes{'report_name'}', Ledger Code = '$report_attributes{'ledger_code'}'");
      my $count = 0;
      my $count2 = 0;
      while (my @info = $sth->fetchrow_array)
      {
        if ($info[0] != $report_attributes{'id'})
        {
          $count++;
        }
        if ($info[0] == $report_attributes{'id'})
        {
          $count2++;
        }
      }
      $sth->finish;

      if ($count)
      {
        # error!  There is another memorized report with the same name but different id!
        memorized_report_display( %report_attributes, dbh => $dbh, error_string => "You are trying to overwrite a memorized report for a different report in the current ledger!" );
        goto LEAVE_PROGRAM_EARLY;
      }
      elsif ($count2)
      {
        if (!exists $input{'overwrite'})
        {
          memorized_report_display( %report_attributes, dbh => $dbh, question_string => "Do you want to overwrite this memorized report?" );
          goto LEAVE_PROGRAM_EARLY;
        }
        else
        {
          my $overwrite = $input{'overwrite'};
          if (length $overwrite == 0)
          {
            my_die("Error: Report.cgi - memorize_report", 4034, "overwrite is required!");
          }
          if ($overwrite eq "no")
          {
            memorized_report_display( %report_attributes, dbh => $dbh );
            goto LEAVE_PROGRAM_EARLY;
          }
        }
      }

      # then create the entry
      $sth = $dbh->prepare( $sql_statements{1030} ) || my_db_die("Error: Report.cgi - memorize_report", 4007, "SQL Statement = '$sql_statements{1030}'");
      $rc = $sth->execute($report_attributes{'id'}, $report_attributes{'ledger_code'}, $report_attributes{'report_name'}) || my_db_die("Error: Report.cgi - memorize_report", 4008, "SQL Statement = '$sql_statements{1030}', Report Name = '$report_attributes{'report_name'}', Ledger Code = '$report_attributes{'ledger_code'}', ID = '$report_attributes{'id'}'");

      my $dates_entered = "";
      if ($report_type eq "r")
      {
        $dates_entered = $report_attributes{'dates_entered'};  # comes from the memorization function as a string already | seperated.
      }
      else
      {
        $dates_entered = "";
      }

      $sth = $dbh->prepare( $sql_statements{1031} ) || my_db_die("Error: Report.cgi - memorize_report", 4007, "SQL Statement = '$sql_statements{1031}'");
      $rc = $sth->execute($report_attributes{'id'}, $report_attributes{'report_name'}, $report_attributes{'column_type'}, $report_attributes{'column_data'}, $dates_entered, $report_attributes{'fg_color'}, $report_attributes{'bg_color'}, $report_attributes{'text_size'}, $report_attributes{'text_font'}, $report_attributes{'data_size'}, $report_attributes{'data_font'}, $report_attributes{'underline_length'}, $report_attributes{'ledger_code'}, $report_attributes{'account_format'}, $report_attributes{'data_precision'}, $report_attributes{'negative_format'}, $report_attributes{'total_column'}, $report_attributes{'sql_statement'}, $report_attributes{'headers'}, $report_attributes{'sorted_by'}, $report_attributes{'sorted_order'}, $report_attributes{'subtotal'}, $report_attributes{'subtotal_type'})
      			|| my_db_die("Error: Report.cgi - memorize_report", 4008, "SQL Statement = '$sql_statements{1031}', ID = '$report_attributes{'id'}', Report Name = '$report_attributes{'report_name'}', Column Type = '$report_attributes{'column_type'}', Column Data = '$report_attributes{'column_data'}', Dates Entered = '$dates_entered', FG Color = '$report_attributes{'fg_color'}', BG Color = '$report_attributes{'bg_color'}', Text Size = '$report_attributes{'text_size'}', Text Font = '$report_attributes{'text_font'}', Data Size = '$report_attributes{'data_size'}', Data Font = '$report_attributes{'data_font'}', Underline Length = '$report_attributes{'underline_length'}', Ledger Code = '$report_attributes{'ledger_code'}', Account Format = '$report_attributes{'account_format'}', Data Precision = '$report_attributes{'data_precision'}', Negative Format = '$report_attributes{'negative_format'}', Total Column = '$report_attributes{'total_column'}', SQL Statement='$report_attributes{'sql_statement'}', Headers='$report_attributes{'headers'}', Sorted By='$report_attributes{'sorted_by'}', Sorted Order='$report_attributes{'sorted_order'}', SubTotal='$report_attributes{'subtotal'}', SubTotal Type='$report_attributes{'subtotal_type'}'");
      $dbh->commit || my_db_die("Error:  Report.cgi - memorize_report", 4019, "At line " . __LINE__);

      $doc->setFocus("body");
      $doc->setTitle("Memorize Report");
      $doc->print("<center><h3>Memorization of Report = '$report_attributes{'report_name'}' Succeeded!</h3></center>\n");
      $doc->setOnload(code => "window.setTimeout('self.close();',1000);\n");
      $doc->display();
    }
  }
  elsif ($command eq "run_report")
  {
    my %report_attributes;
    my $args_ref;
    if (exists $input{'file_name'} && length $input{'file_name'} > 0)
    {
      # retrieve the already customized settings from the file of an already running report to regather the data using the
      # customization settings already selected
      $args_ref = retrieve($input{'file_name'});
      %report_attributes = %{@{$args_ref}[0]};
      $report_attributes{'dbh'} = $dbh;
    }
    else
    {
      my ($report_attributes, $error_string) = get_report_attributes(dbh => $dbh);
      %report_attributes = %{$report_attributes};
      if (length $error_string > 0)
      {
        foreach my $attr (("display_dates", "date_range", "total_column"))
        {
          $report_attributes{$attr} = $true_false{$report_attributes{$attr}};
        }
        customize_report_display( %report_attributes, dbh => $dbh, error_string => $error_string );
      }
    }
    # gather the data
    if ($report_attributes{'report_type'} eq "r")
    {
      gather_regular_report_data( %report_attributes );
    }
    else
    {
      gather_detailed_view_report_data( %report_attributes );
    }
  }
  else
  {
    my_die("Error: Report.cgi - Invalid command", 4001, "'$command' is invalid!");
  }

LEAVE_PROGRAM_EARLY:
  # disconnect from the database (if under mod_perl this will actually never happen. :)  )
  $dbh->disconnect || my_db_die ("DBI Error on Disconnect", 4009, "Database: $db_name\@$db_host");
}
exit;  # leave the program.

#****************************** Helper functions *****************************

# gather_regular_report_data - does the work of gathering the data for a regular report and storing the data to disk in a
#			       usable data structure
sub gather_regular_report_data
{
  my %args = ( @_, );
  my $dbh = $args{'dbh'};
  my @arguments = ("dbh", "id", "ledger", "ledger_code", "ledger_color", "report_name", "report_class", "date_range", "fg_color", "bg_color", "underline_length", "data_precision", "negative_format", "text_size", "text_font", "data_size", "data_font", "account_format", "column_variable", "column_variable_data", "total_column", "column_data", "column_type", "dates_entered", "windowName", "reportWidth", "displayZeroRows");
  
  my $startTime = time();

  $args{'date_range'} = $true_false{$args{'date_range'}};

  eval ("use HTMLObject::Normal;");
  if ($@)
  {
    my_die("Error: Report.cgi - Eval", 4002, "Error: $@ at line " . __LINE__);
  }

  my $doc = HTMLObject::Normal->new();

  $doc->setJavascriptErrorHandlerEmail(email => $email_address);

  foreach my $name (@arguments)
  {
    if (!exists $args{$name})
    {
      $doc->doRequiredParameterError('gather_regular_report_data', $name);
    }
  }

  eval("use HTMLObject::FrameSet;");
  if ($@)
  {
    my_die("Error: Report.cgi - Eval", 4000, "Error: $@ at line " . __LINE__);
  }
  $doc = HTMLObject::FrameSet->new();

  # initiate a server push page
  #$|=1;  # force it to flush the buffer.
  #print "Content-type: multipart/x-mixed-replace;boundary=MyBoundaryMarker\n";
  #print "\n--MyBoundaryMarker\n";

  #my $doc2 = HTMLObject::Normal->new();
  #$doc2->setJavascriptErrorHandlerEmail(email => $email_address);
  #$doc2->setTitle("Data Gathering Stage");
  #$doc2->setFocus("body");
  #$doc2->print("<center><h1>Gathering Data...</h1><h2>Please Wait...</h2></center>\n");
  #$doc2->display();
  #print "\n--MyBoundaryMarker\n";

  # do the work of actually gathering the rows from report_data_tb, gathering each row's data, doing all calculations, etc. and then generate the JavaScript data and code to manipulate the data.
  my @report_data;

  my @dates_array = ();
  my $count;
  my $numEntries = scalar @{$args{dates_entered}};
  for ($count=0; $count < $numEntries; $count++)
  {
    my @date = split /,/, $args{'dates_entered'}[$count], 2;
    my @temp_dates = ();
    if ($args{'date_range'})
    {
      @temp_dates = ($date[0], $date[1]);
    }
    else
    {
      @temp_dates = ($date[0]);
    }
    $dates_array[scalar @dates_array] = [ @temp_dates ];
    #$doc->print("  date_header_array[$count] = \"" . join("<br>- ", @temp_dates) . "\";\n");
  }
  $doc->print("\n");

  my @row_array = GetReportRows( dbh => $dbh, id => $args{'id'} );
  my $max_display_level = -1;

  $numEntries = scalar @row_array;
  for ($count=0; $count < $numEntries; $count++)
  {
    my $temp_display_level = -1;
    my $index = $row_array[$count]{'index'};
    my $cell_text = $row_array[$count]{'cell_text'};
    my $cell_formula = $row_array[$count]{'cell_formula'};
    my $cell_type = $row_array[$count]{'cell_type'};
    my $item_name = $row_array[$count]{'item_name'};
    my $fg_color = $row_array[$count]{'fg_color'};
    my $bg_color = $row_array[$count]{'bg_color'};
    my $bold = $row_array[$count]{'bold'};
    my $sub_levels = $row_array[$count]{'sub_levels'};
    my $display_level = $row_array[$count]{'display_level'};
    my $initial_state = $row_array[$count]{'initial_state'};
    my $show = $row_array[$count]{'show'};
    my $drillable = $row_array[$count]{'drillable'};
    my $drill_report = $row_array[$count]{'drill_report'};
    my $negative_format = $row_array[$count]{'negative_format'};
    my $activity = $row_array[$count]{'activity'};
    my $query_type = $row_array[$count]{'query_type'};
    my $comparison_result_type = $row_array[$count]{'result_type'};
    my $show_row = $row_array[$count]{'show'};
    my $underline = $row_array[$count]{'underline'};

    my %temp_row = (index => $index, cell_text => $cell_text, cell_formula => $cell_formula, cell_type => $cell_type,
    		    item_name => $item_name, fg_color => $fg_color, bg_color => $bg_color, bold => $bold, sub_levels => $sub_levels,
    		    display_level => $display_level, initial_state => $initial_state, drillable => $drillable,
    		    drill_report => $drill_report, negative_format => $negative_format, activity => $activity,
    		    query_type => $query_type, result_type => $comparison_result_type);

    my %temp_hash = {};
    $temp_hash{'report_class'} = $args{'report_class'};
    $temp_hash{'cell_type'} = $cell_type;
    $temp_hash{'cell_text'} = $cell_text;
    $temp_hash{'item_name'} = $item_name;
    $temp_hash{'fg_color'} = $fg_color;
    $temp_hash{'bg_color'} = $bg_color;
    $temp_hash{'bold'} = $bold;
    $temp_hash{'display_level'} = $display_level;
    $temp_hash{'initial_state'} = $initial_state;
    $temp_hash{'show'} = $show;
    if ($show eq "1" || $show eq "")
    {
      $temp_hash{'visible'} = "true";
    }
    else
    {
      $temp_hash{'visible'} = "false";
    }
    $temp_hash{'date_range'} = $args{'date_range'};
    $temp_hash{'sub_levels'} = $sub_levels;
    $temp_hash{'negative_format'} = $negative_format;
    $temp_hash{'account_format'} = $args{'account_format'};
    $temp_hash{'text_font'} = $args{'text_font'};
    $temp_hash{'text_size'} = $args{'text_size'};
    $temp_hash{'data_font'} = $args{'data_font'};
    $temp_hash{'data_size'} = $args{'data_size'};
    $temp_hash{'drillable'} = $drillable;
    $temp_hash{'drill_report'} = $drill_report;
    $temp_hash{'underline'} = $underline;
    $temp_hash{'result_type'} = $comparison_result_type;
    $temp_hash{'dates'} = \@dates_array;

    if ($cell_type eq "text")
    {
      $temp_hash{'sub_levels'} = 0;

      $temp_display_level = $temp_hash{'display_level'};
    }
    elsif ($cell_type eq "line")
    {
      $temp_display_level = $temp_hash{'display_level'};
    }
    elsif ($cell_type eq "total")
    {
      $temp_hash{'sub_levels'} = 0;

      $temp_display_level = $temp_hash{'display_level'};
      get_total_data(args => \%args, temp_hash => \%temp_hash, temp_row => \%temp_row, report_data => \@report_data);
    }
    elsif ($cell_type eq "compare")
    {
      $temp_display_level = $temp_hash{'display_level'};
      get_comparison_data(args => \%args, temp_hash => \%temp_hash, temp_row => \%temp_row, report_data => \@report_data);
    }
    elsif ($cell_type eq "account_type" || $cell_type eq "begin_account_type_balance" || $cell_type eq "end_account_type_balance")
    {
      if ($cell_type eq "begin_account_type_balance" || $cell_type eq "end_account_type_balance")
      {
        $temp_row{'activity'} = 0;
      }

      $temp_display_level = get_account_type_data(args => \%args, temp_hash => \%temp_hash, temp_row => \%temp_row);
    }
    elsif ($cell_type eq "account" || $cell_type eq "begin_account_balance" || $cell_type eq "end_account_balance")
    {
      if ($cell_type eq "begin_account_balance" || $cell_type eq "end_account_balance")
      {
        $temp_row{'activity'} = 0;
      }

      $temp_display_level = get_account_data(args => \%args, temp_hash => \%temp_hash, temp_row => \%temp_row);
    }
    elsif ($cell_type eq "sub_account" || $cell_type eq "begin_sub_account_balance" || $cell_type eq "end_sub_account_balance")
    {
      if ($cell_type eq "begin_sub_account_balance" || $cell_type eq "end_sub_account_balance")
      {
        $temp_row{'activity'} = 0;
      }

      $temp_display_level = get_sub_account_data(args => \%args, temp_hash => \%temp_hash, temp_row => \%temp_row);
    }
    elsif ($cell_type eq "accounts_of")
    {
      $temp_row{'sub_levels'} += 1;

      $temp_display_level = get_account_type_data(args => \%args, temp_hash => \%temp_hash, temp_row => \%temp_row);
    }
    elsif ($cell_type eq "sub_accounts_of_type")
    {
      $temp_display_level = get_sub_account_type_data(args => \%args, temp_hash => \%temp_hash, temp_row => \%temp_row);
    }
    elsif ($cell_type eq "sub_accounts_of_account")
    {
      $temp_row{'sub_levels'} = 1;

      $temp_display_level = get_account_data(args => \%args, temp_hash => \%temp_hash, temp_row => \%temp_row);
    }

    $report_data[$count] = { %temp_hash };

    if ($temp_display_level > $max_display_level)
    {
      $max_display_level = $temp_display_level;
    }
  }

  $args{'max_display_level'} = $max_display_level;

  my $endTime = time();

  #### **************************** ####
  my $file_name = tmpnam(); # get the file name of the temp file to work with.
  # store the data object.
  my @data = ();
  $data[0] = \%args;
  $data[1] = \@report_data;
  $data[2] = ($endTime - $startTime);  # number of seconds it took to gather the data.

  $Storable::forgive_me = 1;  # try to get around the CODE error.
  store(\@data, $file_name);

  # create the entry in the temp_file_tb
  log_temp_file( dbh => $dbh, file_name => $file_name );

  # create the frameset and call the next step of the process.

  my $inner_content;
  $inner_content = $doc->createNoframe("You must have a frames capable browser!");
  $inner_content .= $doc->createFrame(src => "$site_dir/cgi-bin/report_display.cgi?command=run_report_top&id=$args{'id'}&file_name=$file_name&report_type=r", frameborder => 'yes', scrolling => 'auto', marginwidth => '0', marginheight => '0', border => '1', name => $args{windowName} . "_top", bordercolor => 'black');
  $inner_content .= $doc->createFrame(src => "$site_dir/cgi-bin/report_display.cgi?command=run_report_bottom&id=$args{'id'}&file_name=$file_name&report_type=r&report_command=display", frameborder => 'yes', scrolling => 'auto', name => $args{windowName} . "_bottom", marginwidth => '1', bordercolor => 'white');
  $doc->outerFrameset(rows => '40, *', content => $inner_content, border => '5', frameborder => 'yes');

  $doc->setTitle("Report - $args{'report_name'}");

  $doc->display();
  #print "\n--MyBoundaryMarker\n";
  #$|=0;  # turn off buffer flush.
}

# gather_detailed_view_report_data - does the work of gathering the data for a detailed view report and storing the data to
#				     disk in a usable data structure
sub gather_detailed_view_report_data
{
  my %args = ( @_, );
  my $dbh = $args{'dbh'};
  my @arguments = ("dbh", "id", "ledger", "ledger_code", "ledger_color", "report_name", "fg_color", "bg_color", "data_precision", "negative_format", "text_size", "text_font", "data_size", "data_font", "account_format", "sql_statement", "windowName", "reportWidth");

  eval ("use HTMLObject::Normal;");
  if ($@)
  {
    my_die("Error: Report.cgi - Eval", 4002, "Error: $@ at line " . __LINE__);
  }

  my $doc = HTMLObject::Normal->new();

  $doc->setJavascriptErrorHandlerEmail(email => $email_address);

  foreach my $name (@arguments)
  {
    if (!exists $args{$name})
    {
      $doc->doRequiredParameterError('gather_detailed_view_report_data', $name);
    }
  }

  eval("use HTMLObject::FrameSet;");
  if ($@)
  {
    my_die("Error: Report.cgi - Eval", 4000, "Error: $@ at line " . __LINE__);
  }
  $doc = HTMLObject::FrameSet->new();

  # initiate a server push page
  #$|=1;  # force it to flush the buffer.
  #print "Content-type: multipart/x-mixed-replace;boundary=MyBoundaryMarker\n";
  #print "\n--MyBoundaryMarker\n";

  #my $doc2 = HTMLObject::Normal->new();
  #$doc2->setJavascriptErrorHandlerEmail(email => $email_address);
  #$doc2->setTitle("Data Gathering Stage");
  #$doc2->setFocus("body");
  #$doc2->print("<center><h1>Gathering Data...</h1><h2>Please Wait...</h2></center>\n");
  #$doc2->display();
  #print "\n--MyBoundaryMarker\n";

  # do the work of actually gathering the rows from report_data_tb, gathering each row's data, doing all calculations, etc. and then generate the JavaScript data and code to manipulate the data.
  my @report_data;
  my $count;

  my @row_array = GetReportRows( dbh => $dbh, id => $args{'id'} );

  my $numEntries = scalar @row_array;
  for ($count=0; $count < $numEntries; $count++)
  {
    my $index = $row_array[$count]{'index'};
    my $cell_text = $row_array[$count]{'cell_text'};
    my $cell_formula = $row_array[$count]{'cell_formula'};
    my $cell_type = $row_array[$count]{'cell_type'};
    my $item_name = $row_array[$count]{'item_name'};
    my $fg_color = $row_array[$count]{'fg_color'};
    my $bg_color = $row_array[$count]{'bg_color'};
    my $bold = $row_array[$count]{'bold'};
    my $sub_levels = $row_array[$count]{'sub_levels'};
    my $display_level = $row_array[$count]{'display_level'};
    my $initial_state = $row_array[$count]{'initial_state'};
    my $drillable = $row_array[$count]{'drillable'};
    my $drill_report = $row_array[$count]{'drill_report'};
    my $negative_format = $row_array[$count]{'negative_format'};
    my $activity = $row_array[$count]{'activity'};
    my $query_type = $row_array[$count]{'query_type'};
    my $comparison_result_type = $row_array[$count]{'result_type'};
    my $show_row = $row_array[$count]{'show'};

    my %temp_row = (index => $index, cell_text => $cell_text, cell_formula => $cell_formula, cell_type => $cell_type,
    		    item_name => $item_name, fg_color => $fg_color, bg_color => $bg_color, bold => $bold, sub_levels => $sub_levels,
    		    display_level => $display_level, initial_state => $initial_state, drillable => $drillable,
    		    drill_report => $drill_report, negative_format => $negative_format, activity => $activity,
    		    query_type => $query_type);

    my %temp_hash = {};
    $temp_hash{'cell_type'} = $cell_type;
    $temp_hash{'item_name'} = $item_name;
    $temp_hash{'fg_color'} = $fg_color;
    $temp_hash{'bg_color'} = $bg_color;
    $temp_hash{'bold'} = $bold;
    $temp_hash{'display_level'} = $display_level;
    $temp_hash{'initial_state'} = $initial_state;
    $temp_hash{'date_range'} = $args{'date_range'};
    $temp_hash{'sub_levels'} = $sub_levels;
    $temp_hash{'negative_format'} = $negative_format;
    $temp_hash{'account_format'} = $args{'account_format'};
    $temp_hash{'text_font'} = $args{'text_font'};
    $temp_hash{'text_size'} = $args{'text_size'};
    $temp_hash{'data_font'} = $args{'data_font'};
    $temp_hash{'data_size'} = $args{'data_size'};
    $temp_hash{'drillable'} = $drillable;
    $temp_hash{'drill_report'} = $drill_report;

    if ($cell_type eq "text")
    {
      $temp_hash{'cell_text'} = $cell_text;
      $temp_hash{'sub_levels'} = 0;
    }
    elsif ($cell_type eq "detail")
    {
      $temp_hash{'sql_statement'} = $args{'sql_statement'};

      get_detail_data(args => \%args, temp_hash => \%temp_hash, temp_row => \%temp_row);
    }

    $report_data[$count] = { %temp_hash };
  }

  #### **************************** ####
  my $file_name = tmpnam(); # get the file name of the temp file to work with.
  # store the data object.
  my @data = ();
  $data[0] = \%args;
  $data[1] = \@report_data;

  $Storable::forgive_me = 1;  # try to get around the CODE error.
  store(\@data, $file_name);

  # create the entry in the temp_file_tb
  log_temp_file( dbh => $dbh, file_name => $file_name );

  # create the frameset and call the next step of the process.

  my $inner_content;
  $inner_content = $doc->createNoframe("You must have a frames capable browser!");
  $inner_content .= $doc->createFrame(src => "$site_dir/cgi-bin/report_display.cgi?command=run_report_top&id=$args{'id'}&file_name=$file_name&report_type=d", frameborder => 'yes', scrolling => 'auto', marginwidth => '0', marginheight => '0', border => '1', name => $args{windowName} . "_top", bordercolor => 'black');
  $inner_content .= $doc->createFrame(src => "$site_dir/cgi-bin/report_display.cgi?command=run_report_bottom&id=$args{'id'}&file_name=$file_name&report_type=d", frameborder => 'yes', scrolling => 'auto', name => $args{windowName} . "_bottom", marginwidth => '1', bordercolor => 'white');
  $doc->outerFrameset(rows => '40, *', content => $inner_content, border => '5', frameborder => 'yes');

  $doc->setTitle("Report - $args{'report_name'}");

  $doc->display();
  #print "\n--MyBoundaryMarker\n";
  #$|=0;  # turn off buffer flush.
}


# convert_data
# takes  : precision (defaults to 1), value
# returns: value (converted and massaged into the specified precision).
sub convert_data
{
  my %arguments = ( precision => 1, @_, );
  my $precision = $arguments{'precision'};
  my $value = $arguments{'value'};

  if (length $value == 0)
  {
    my_die("Error:  Report.cgi - convert_data", 4034, "value must be specified at line " . __LINE__);
  }

  if ($precision != 1)
  {
    $value /= $precision;
  }

  return $value;
}

sub get_account_type_data
{
  my %arguments = ( @_, );
  my %args = %{ $arguments{'args'} };
  my $ledger_code = $args{'ledger_code'};
  my $temp_hash = $arguments{'temp_hash'};
  my %temp_row = %{ $arguments{'temp_row'} };
  my @local_accounts = ();
  my @column_data = split /,/, $args{'column_data'};
  my $column_type = $args{'column_type'};
  my $data_precision = $args{'data_precision'};
  my $max_display_level = -1;
  my @sub_entities = ();
  my @sub_accounts = ();

  if ($column_type eq "nothing")
  {
    @column_data = ( "0" );
  }
  elsif ($column_type eq "sub_entities")
  {
    my $numEntries = scalar @column_data;
    for (my $count=0; $count < $numEntries; $count++)
    {
      # search for "<non_sub_entity>" and replace with ""
      if ($column_data[$count] eq "<non_sub_entity>")
      {
        $column_data[$count] = "";
      }
    }
  }

  # get the account type name
  my @temp_item_name = ();
  $temp_item_name[0] = $temp_row{'item_name'};
  my @temp_cell_text = GetAccountTypeNames(dbh => $args{'dbh'}, account_types => \@temp_item_name);
  $temp_hash->{'cell_text'} = $temp_cell_text[0];

  $temp_hash->{'values'} = ();
  $temp_hash->{'sql_clause'} = ();
  # run over the dates and gather the data for the dates
  my $numEntries = scalar @{$temp_hash->{dates}};
  for (my $count=0; $count < $numEntries; $count++)
  {
    my ($date_start, $date_end);
    if ($args{'date_range'} && $temp_row{'cell_type'} ne "begin_account_type_balance" && $temp_row{'cell_type'} ne "end_account_type_balance")
    {
      $date_start = $temp_hash->{'dates'}[$count][0];
      $date_end = $temp_hash->{'dates'}[$count][1];
    }
    else
    {
      if ($temp_row{'cell_type'} eq "begin_account_type_balance")
      {
        (my $temp_date = $temp_hash->{'dates'}[$count][0]) =~ s/$minusRE/\//g;
        # make date_start and date_end one less day
        $date_start = `/bin/date -d "$temp_date 1 day ago" +%m-%d-%Y`;
        $date_end = $date_start;
      }
      elsif ($temp_row{'cell_type'} eq "end_account_type_balance")
      {
        $date_end = $temp_hash->{'dates'}[$count][1];
        $date_start = $date_end;
      }
      else
      {
        $date_start = $temp_hash->{'dates'}[$count][0];
        $date_end = $date_start;
      }
    }

    if ($temp_row{'display_level'} > $max_display_level)
    {
      $max_display_level = $temp_row{'display_level'};
    }

    if ($temp_row{'sub_levels'} == 0)
    {
      my $dbh = $args{'dbh'};
      my $sth;
      my $rc;

      my $tmp_sql_statement = "";
      my @sql_arguments = ();
      my $sql_clause;
      my $error_string = 0;
      my $numEntries = scalar @column_data;
      for (my $column_count=0; $column_count < $numEntries; $column_count++)
      {
        if ($column_type eq "nothing")
        {
          if ($temp_row{'query_type'} == 0)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1073};
              @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $date_end, $temp_hash->{'report_class'});
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1113};
              @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_end, $temp_hash->{'report_class'});
            }
          }
          elsif ($temp_row{'query_type'} == 1)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1114};
              @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $date_end, "true", $temp_hash->{'report_class'});
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1117};
              @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_end, "true", $temp_hash->{'report_class'});
            }
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1114};
              @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $date_end, "false", $temp_hash->{'report_class'});
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1117};
              @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_end, "false", $temp_hash->{'report_class'});
            }
          }

          $sql_clause = "";
          if ($temp_row{'activity'})
          {
            $sql_clause = "je.trans_date >= '$date_start' AND ";
          }
          $sql_clause .= "je.trans_date <= '$date_end'";
          $sql_clause .= " AND db.account_num IN (SELECT number FROM account_tb WHERE type = $temp_row{'item_name'}";
          if ($temp_row{'query_type'} == 1)
          {
            $sql_clause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$ledger_code' AND acct_num = account_tb.number AND taxable = true)";
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            $sql_clause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$ledger_code' AND acct_num = account_tb.number AND taxable = false)";
          }
          $sql_clause .= ")";
        }
        elsif ($column_type eq "sub_entities")
        {
          if ($temp_row{'query_type'} == 0)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1074};
              @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $date_end, $column_data[$column_count], $temp_hash->{'report_class'});
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1113};
              @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_end, $temp_hash->{'report_class'});
              $error_string = 1;
            }
          }
          elsif ($temp_row{'query_type'} == 1)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1075};
              @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $date_end, "true", $column_data[$column_count], $temp_hash->{'report_class'});
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1117};
              @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_end, "true", $temp_hash->{'report_class'});
              $error_string = 1;
            }
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1075};
              @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $date_end, "false", $column_data[$column_count], $temp_hash->{'report_class'});
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1117};
              @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_end, "false", $temp_hash->{'report_class'});
              $error_string = 1;
            }
          }

          $sql_clause = "";
          if ($temp_row{'activity'})
          {
            $sql_clause = "je.trans_date >= '$date_start' AND ";
          }
          $sql_clause .= "je.trans_date <= '$date_end'";
          $sql_clause .= " AND db.account_num IN (SELECT number FROM account_tb WHERE type = $temp_row{'item_name'}";
          if ($temp_row{'query_type'} == 1)
          {
            $sql_clause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$ledger_code' AND acct_num = account_tb.number AND taxable = true)";
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            $sql_clause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$ledger_code' AND acct_num = account_tb.number AND taxable = false)";
          }
          $sql_clause .= ")";
          $sql_clause .= " AND db.sub_e_code = '$column_data[$column_count]'";
        }
        elsif ($column_type eq "sub_accounts")
        {
          if ($temp_row{'query_type'} == 0)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1120};
              @sql_arguments = ($temp_row{'item_name'}, $column_data[$column_count], $ledger_code, $date_start, $date_end, $temp_hash->{'report_class'});
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1122};
              @sql_arguments = ($temp_row{'item_name'}, $column_data[$column_count], $ledger_code, $date_end, $temp_hash->{'report_class'});
            }
          }
          elsif ($temp_row{'query_type'} == 1)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1121};
              @sql_arguments = ($temp_row{'item_name'}, $column_data[$column_count], $ledger_code, $date_start, $date_end, "true", $temp_hash->{'report_class'});
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1123};
              @sql_arguments = ($temp_row{'item_name'}, $column_data[$column_count], $ledger_code, $date_end, "true", $temp_hash->{'report_class'});
            }
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1121};
              @sql_arguments = ($temp_row{'item_name'}, $column_data[$column_count], $ledger_code, $date_start, $date_end, "false", $temp_hash->{'report_class'});
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1123};
              @sql_arguments = ($temp_row{'item_name'}, $column_data[$column_count], $ledger_code, $date_end, "false", $temp_hash->{'report_class'});
            }
          }

          if ($temp_hash->{'item_name'} == 0 || $temp_hash->{'item_name'} == 10)
          { # cash/bank account type or sales revenue account type
            $sql_clause = "invalid";
            $error_string = 1;
          }
          else
          {
            $sql_clause = "";
            if ($temp_row{'activity'})
            {
              $sql_clause = "je.trans_date >= '$date_start' AND ";
            }
            $sql_clause .= "je.trans_date <= '$date_end'";
            $sql_clause .= " AND db.account_num IN (SELECT number FROM account_tb WHERE type = $temp_row{'item_name'}";
            if ($temp_row{'query_type'} == 1)
            {
              $sql_clause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$ledger_code' AND acct_num = account_tb.number AND taxable = true)";
            }
            elsif ($temp_row{'query_type'} == 2)
            {
              $sql_clause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$ledger_code' AND acct_num = account_tb.number AND taxable = false)";
            }
            $sql_clause .= ")";
            $sql_clause .= " AND db.sub_account_num = $column_data[$column_count]";
          }
        }
        elsif ($column_type eq "sub_account_types")
        {
          if ($temp_row{'query_type'} == 0)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1115};
              @sql_arguments = ($temp_row{'item_name'}, $column_data[$column_count], $ledger_code, $date_start, $date_end, $temp_hash->{'report_class'});
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1118};
              @sql_arguments = ($temp_row{'item_name'}, $column_data[$column_count], $ledger_code, $date_end, $temp_hash->{'report_class'});
            }
          }
          elsif ($temp_row{'query_type'} == 1)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1116};
              @sql_arguments = ($temp_row{'item_name'}, $column_data[$column_count], $ledger_code, $date_start, $date_end, "true", $temp_hash->{'report_class'});
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1119};
              @sql_arguments = ($temp_row{'item_name'}, $column_data[$column_count], $ledger_code, $date_end, "true", $temp_hash->{'report_class'});
            }
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1116};
              @sql_arguments = ($temp_row{'item_name'}, $column_data[$column_count], $ledger_code, $date_start, $date_end, "false", $temp_hash->{'report_class'});
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1119};
              @sql_arguments = ($temp_row{'item_name'}, $column_data[$column_count], $ledger_code, $date_end, "false", $temp_hash->{'report_class'});
            }
          }

          if ($temp_hash->{'item_name'} == 0 || $temp_hash->{'item_name'} == 10)
          { # cash/bank account type or sales revenue account type
            $sql_clause = "invalid";
            #$error_string;
          }
          else
          {
            $sql_clause = "";
            if ($temp_row{'activity'})
            {
              $sql_clause = "je.trans_date >= '$date_start' AND ";
            }
            $sql_clause .= "je.trans_date <= '$date_end'";
            $sql_clause .= " AND db.account_num IN (SELECT number FROM account_tb WHERE type = $temp_row{'item_name'}";
            if ($temp_row{'query_type'} == 1)
            {
              $sql_clause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$ledger_code' AND acct_num = account_tb.number AND taxable = true)";
            }
            elsif ($temp_row{'query_type'} == 2)
            {
              $sql_clause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$ledger_code' AND acct_num = account_tb.number AND taxable = false)";
            }
            $sql_clause .= ")";
            $sql_clause .= " AND db.sub_account_num IN (SELECT sub_acc_number FROM sub_acc_tb WHERE type = $column_data[$column_count])";
          }
        }

        if ($error_string == 1 && ($temp_hash->{'item_name'} == 0 || $temp_hash->{'item_name'} == 10))
        { # cash/bank account type or sales revenue account type
          $temp_hash->{'values'}[$count][$column_count] = 0;
        }
        else
        {
          $sth = $dbh->prepare( $tmp_sql_statement ) || my_db_die("Error: Report.cgi - get_account_type_data", 4007, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);
          $rc = $sth->execute(@sql_arguments) || my_db_die("Error: Report.cgi - get_account_type_data", 4008, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);
          my $row_array = $sth->fetchrow_arrayref();
          my $value = $row_array->[0];
          $temp_hash->{'values'}[$count][$column_count] = $value;
        }

        if ($error_string == 1)
        {
          $temp_hash->{'error'} = $error_string;
        }
        $temp_hash->{'sql_clause'}[$count][$column_count] = $sql_clause;
        $temp_hash->{'total'} += $temp_hash->{'values'}[$count][$column_count];
        $temp_hash->{'total'} = $temp_hash->{'total'};
      }
    }
    elsif ($temp_row{'sub_levels'} > 0)
    {
      my @temp_account_total;
      my $dbh = $args{'dbh'};
      my $sth;
      my $rc;

      my $tmp_sql_statement = "";
      my @sql_arguments = ();
      my $sql_clause;
      my $level_sql_clause;
      my $error_string = 0;
      # calculate the value (activity/balance) for the account type for the current date range using each account given the column qualifiers
      if ($temp_row{'sub_levels'} == 1 || ($temp_row{'sub_levels'} == 2 && ($column_type eq "sub_accounts" || $column_type eq "sub_account_types")))
      {
        my $numEntries = scalar @column_data;
        for (my $column_count = 0; $column_count < $numEntries; $column_count++)
        {
          if ($column_type eq "nothing")
          {
            if ($temp_row{'query_type'} == 0)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1076};
                @sql_arguments = ($ledger_code, $date_start, $date_end, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1084};
                @sql_arguments = ($ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_end, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_end);
              }
            }
            elsif ($temp_row{'query_type'} == 1)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1078};
                @sql_arguments = ($ledger_code, $date_start, $date_end, "true", $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "true", $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1085};
                @sql_arguments = ($ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "true", $date_end, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "true", $date_end);
              }
            }
            elsif ($temp_row{'query_type'} == 2)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1078};
                @sql_arguments = ($ledger_code, $date_start, $date_end, "false", $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "false", $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1085};
                @sql_arguments = ($ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "false", $date_end, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "false", $date_end);
              }
            }

            $sql_clause = "";
            if ($temp_row{'activity'})
            {
              $sql_clause = "je.trans_date >= '$date_start' AND ";
            }
            $sql_clause .= "je.trans_date <= '$date_end'";
            $sql_clause .= " AND db.account_num = <ACCOUNT>";
          }
          elsif ($column_type eq "sub_entities")
          {
            if ($temp_row{'query_type'} == 0)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1077};
                @sql_arguments = ($ledger_code, $date_start, $date_end, $column_data[$column_count], $temp_hash->{'report_class'}, $ledger_code, $column_data[$column_count], $temp_row{'item_name'}, $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1084};
                @sql_arguments = ($ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_end, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_end);
                $error_string = 1;
              }
            }
            elsif ($temp_row{'query_type'} == 1)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1079};
                @sql_arguments = ($ledger_code, $date_start, $date_end, "true", $column_data[$column_count], $temp_hash->{'report_class'}, $ledger_code, $column_data[$column_count], $temp_row{'item_name'}, $ledger_code, "true", $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1085};
                @sql_arguments = ($ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "true", $date_end, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "true", $date_end);
                $error_string = 1;
              }
            }
            elsif ($temp_row{'query_type'} == 2)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1079};
                @sql_arguments = ($ledger_code, $date_start, $date_end, "false", $column_data[$column_count], $temp_hash->{'report_class'}, $ledger_code, $column_data[$column_count], $temp_row{'item_name'}, $ledger_code, "false", $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1085};
                @sql_arguments = ($ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "false", $date_end, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "false", $date_end);
                $error_string = 1;
              }
            }

            $sql_clause = "";
            if ($temp_row{'activity'})
            {
              $sql_clause = "je.trans_date >= '$date_start' AND ";
            }
            $sql_clause .= "je.trans_date <= '$date_end'";
            $sql_clause .= " AND db.account_num = <ACCOUNT>";
            $sql_clause .= " AND db.sub_e_code = '$column_data[$column_count]'";
          }
          elsif ($column_type eq "sub_accounts")
          {
            if ($temp_row{'query_type'} == 0)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1080};
                @sql_arguments = ($column_data[$column_count], $ledger_code, $date_start, $date_end, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_start, $date_end, $column_data[$column_count]);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1084};
                @sql_arguments = ($ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_end, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_end);
              }
            }
            elsif ($temp_row{'query_type'} == 1)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1081};
                @sql_arguments = ($column_data[$column_count], $ledger_code, $date_start, $date_end, "true", $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "true", $date_start, $date_end, $column_data[$column_count]);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1085};
                @sql_arguments = ($ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "true", $date_end, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "true", $date_end);
              }
            }
            elsif ($temp_row{'query_type'} == 2)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1081};
                @sql_arguments = ($column_data[$column_count], $ledger_code, $date_start, $date_end, "false", $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "false", $date_start, $date_end, $column_data[$column_count]);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1085};
                @sql_arguments = ($ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "false", $date_end, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "false", $date_end);
              }
            }

            $sql_clause = "";
            if ($temp_row{'activity'})
            {
              $sql_clause = "je.trans_date >= '$date_start' AND ";
            }
            $sql_clause .= "je.trans_date <= '$date_end'";
            $sql_clause .= " AND db.account_num = <ACCOUNT>";
            $sql_clause .= " AND db.sub_account_num = $column_data[$column_count]";
          }
          elsif ($column_type eq "sub_account_types")
          {
            if ($temp_row{'query_type'} == 0)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1082};
                @sql_arguments = ($column_data[$column_count], $ledger_code, $date_start, $date_end, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_start, $date_end, $column_data[$column_count]);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1084};
                @sql_arguments = ($ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_end, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_end);
              }
            }
            elsif ($temp_row{'query_type'} == 1)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1083};
                @sql_arguments = ($column_data[$column_count], $ledger_code, $date_start, $date_end, "true", $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "true", $date_start, $date_end, $column_data[$column_count]);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1085};
                @sql_arguments = ($ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "true", $date_end, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "true", $date_end);
              }
            }
            elsif ($temp_row{'query_type'} == 2)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1083};
                @sql_arguments = ($column_data[$column_count], $ledger_code, $date_start, $date_end, "false", $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "false", $date_start, $date_end, $column_data[$column_count]);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1085};
                @sql_arguments = ($ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "false", $date_end, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "false", $date_end);
              }
            }

            $sql_clause = "";
            if ($temp_row{'activity'})
            {
              $sql_clause = "je.trans_date >= '$date_start' AND ";
            }
            $sql_clause .= "je.trans_date <= '$date_end'";
            $sql_clause .= " AND db.account_num = <ACCOUNT>";
            $sql_clause .= " AND db.sub_account_num IN (SELECT sub_acc_number FROM sub_acc_tb WHERE type = $column_data[$column_count])";
          }
          $sth = $dbh->prepare( $tmp_sql_statement ) || my_db_die("Error: Report.cgi - get_account_type_data", 4007, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);
          $rc = $sth->execute(@sql_arguments) || my_db_die("Error: Report.cgi - get_account_type_data", 4008, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);
          while (my $row_array = $sth->fetchrow_arrayref())
          {
            my $account_num = $row_array->[0];
            my $account_name = $row_array->[1];
            my $value = $row_array->[2];

            if ($account_num ne "")
            {
              if (not defined $temp_hash->{'sub_rows'})
              {
                $temp_hash->{'sub_rows'} = {};
              }

              if (not defined $temp_hash->{'sub_rows'}{$account_num})
              {
                $temp_hash->{'sub_rows'}{$account_num} = {};
              }
              my $subRow = $temp_hash->{sub_rows}{$account_num};
              $subRow->{'cell_text'} = $account_name;
              $subRow->{'item_name'} = $account_num;
              $subRow->{'initial_state'} = $temp_row{'initial_state'};
              if ($temp_row{'initial_state'} eq "o")
              {
                $temp_hash->{'sub_rows_visible'} = "true";
                $subRow->{'visible'} = "true";
              }
              else
              {
                $temp_hash->{'sub_rows_visible'} = "false";
                $subRow->{'visible'} = "false";
              }
              $subRow->{'display_level'} = $temp_row{'display_level'} + 1;
              if ($temp_row{'display_level'} + 1 > $max_display_level)
              {
                $max_display_level = $temp_row{'display_level'} + 1;
              }
              $subRow->{'sub_levels'} = $temp_row{'sub_levels'} - 1;
              if (not defined $subRow->{'values'})
              {
                $subRow->{'values'} = ();
              }
              if (not defined $subRow->{'sql_clause'})
              {
                $subRow->{'sql_clause'} = ();
              }
              my $numEntries = scalar @{$temp_hash->{dates}};
              for (my $date_count=0; $date_count < $numEntries; $date_count++)
              {
                if (not defined $subRow->{'values'}[$date_count])
                {
                  $subRow->{'values'}[$date_count] = ();
                  my $subNumEntries = scalar @column_data;
                  for (my $fill_count=0; $fill_count < $subNumEntries; $fill_count++)
                  {
                    if (not defined $subRow->{'values'}[$date_count][$fill_count])
                    {
                      $subRow->{'values'}[$date_count][$fill_count] = "0.00";
                    }
                  }
                }
                if (not defined $subRow->{'sql_clause'}[$date_count])
                {
                  $subRow->{'sql_clause'}[$date_count] = ();
                  my $subNumEntries = scalar @column_data;
                  for (my $fill_count=0; $fill_count < $subNumEntries; $fill_count++)
                  {
                    if (not defined $subRow->{'sql_clause'}[$date_count][$fill_count])
                    {
                      $subRow->{'sql_clause'}[$date_count][$fill_count] = "invalid";
                    }
                  }
                }
              }

              if ($error_string == 1)
              {
                $temp_hash->{'error'} = $error_string;
              }
              $subRow->{'error'} = $error_string;
              my $tmp_sql_clause = $sql_clause;
              $tmp_sql_clause =~ s/$accountRE/$account_num/g;
              $subRow->{'sql_clause'}[$count][$column_count] = $tmp_sql_clause;
              $subRow->{'values'}[$count][$column_count] = $value;
              $subRow->{'total'} += $temp_hash->{'sub_rows'}{$account_num}{'values'}[$count][$column_count];
              $temp_account_total[$column_count] += $subRow->{'values'}[$count][$column_count];
            }
          }
        }
      }
      elsif ($temp_row{'sub_levels'} == 2)
      {
        my $numEntries = scalar @column_data;
        for (my $column_count=0; $column_count < $numEntries; $column_count++)
        {
          if ($column_type eq "nothing")
          {
            if ($temp_row{'query_type'} == 0)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1086};
                @sql_arguments = ($ledger_code, $date_start, $date_end, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1090};
                @sql_arguments = ($ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_end, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_end);
              }
            }
            elsif ($temp_row{'query_type'} == 1)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1088};
                @sql_arguments = ($ledger_code, $date_start, $date_end, "true", $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "true", $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1091};
                @sql_arguments = ($ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "true", $date_end, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "true", $date_end);
              }
            }
            elsif ($temp_row{'query_type'} == 2)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1088};
                @sql_arguments = ($ledger_code, $date_start, $date_end, "false", $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "false", $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1091};
                @sql_arguments = ($ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "false", $date_end, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "false", $date_end);
              }
            }

            $sql_clause = "";
            if ($temp_row{'activity'})
            {
              $sql_clause = "je.trans_date >= '$date_start' AND ";
            }
            $sql_clause .= "je.trans_date <= '$date_end'";
            $sql_clause .= " AND db.account_num = <ACCOUNT>";
            $sql_clause .= " AND db.sub_account_num = <SUBACCOUNT>";

            $level_sql_clause = "";
            if ($temp_row{'activity'})
            {
              $level_sql_clause = "je.trans_date >= '$date_start' AND ";
            }
            $level_sql_clause .= "je.trans_date <= '$date_end'";
            $level_sql_clause .= " AND db.account_num = <ACCOUNT>";
          }
          elsif ($column_type eq "sub_entities")
          {
            if ($temp_row{'query_type'} == 0)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1087};
                @sql_arguments = ($ledger_code, $date_start, $date_end, $column_data[$column_count], $temp_hash->{'report_class'}, $ledger_code, $column_data[$column_count], $temp_row{'item_name'}, $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1090};
                @sql_arguments = ($ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_end, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_end);
                $error_string = 1;
              }
            }
            elsif ($temp_row{'query_type'} == 1)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1089};
                @sql_arguments = ($ledger_code, $date_start, $date_end, "true", $column_data[$column_count], $temp_hash->{'report_class'}, $ledger_code, $column_data[$column_count], $temp_row{'item_name'}, $ledger_code, "true", $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1091};
                @sql_arguments = ($ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "true", $date_end, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "true", $date_end);
                $error_string = 1;
              }
            }
            elsif ($temp_row{'query_type'} == 2)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1089};
                @sql_arguments = ($ledger_code, $date_start, $date_end, "false", $column_data[$column_count], $temp_hash->{'report_class'}, $ledger_code, $column_data[$column_count], $temp_row{'item_name'}, $ledger_code, "false", $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1091};
                @sql_arguments = ($ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "false", $date_end, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "false", $date_end);
                $error_string = 1;
              }
            }

            $sql_clause = "";
            if ($temp_row{'activity'})
            {
              $sql_clause = "je.trans_date >= '$date_start' AND ";
            }
            $sql_clause .= "je.trans_date <= '$date_end'";
            $sql_clause .= " AND db.account_num = <ACCOUNT>";
            $sql_clause .= " AND db.sub_account_num = <SUBACCOUNT>";
            $sql_clause .= " AND db.sub_e_code = '$column_data[$column_count]'";

            $level_sql_clause = "";
            if ($temp_row{'activity'})
            {
              $level_sql_clause = "je.trans_date >= '$date_start' AND ";
            }
            $level_sql_clause .= "je.trans_date <= '$date_end'";
            $level_sql_clause .= " AND db.account_num = <ACCOUNT>";
            $level_sql_clause .= " AND db.sub_e_code = '$column_data[$column_count]'";
          }
          $sth = $dbh->prepare( $tmp_sql_statement ) || my_db_die("Error: Report.cgi - get_account_type_data", 4007, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);
          $rc = $sth->execute(@sql_arguments) || my_db_die("Error: Report.cgi - get_account_type_data", 4008, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);
          while (my $row_array = $sth->fetchrow_arrayref())
          {
            my $account_num = $row_array->[0];
            my $account_name = $row_array->[1];
            my $sub_account_num = $row_array->[2];
            my $sub_account_name = $row_array->[3];
            my $value = $row_array->[4];

            if ($account_num ne "" && $sub_account_num ne "")
            {
              if (not defined $temp_hash->{'sub_rows'})
              {
                $temp_hash->{'sub_rows'} = {};
              }

              if (not defined $temp_hash->{'sub_rows'}{$account_num})
              {
                $temp_hash->{'sub_rows'}{$account_num} = {};
              }
              my $subRow = $temp_hash->{sub_rows}{$account_num};
              if (not defined $subRow->{'sub_rows'})
              {
                $subRow->{'sub_rows'} = {};
              }
              $subRow->{'cell_text'} = $account_name;
              $subRow->{'item_name'} = $account_num;
              $subRow->{'initial_state'} = $temp_row{'initial_state'};
              if ($temp_row{'initial_state'} eq "o")
              {
                $temp_hash->{'sub_rows_visible'} = "true";
                $subRow->{'visible'} = "true";
              }
              else
              {
                $temp_hash->{'sub_rows_visible'} = "false";
                $subRow->{'visible'} = "false";
              }
              $subRow->{'display_level'} = $temp_row{'display_level'} + 1;
              if ($temp_row{'display_level'} + 1 > $max_display_level)
              {
                $max_display_level = $temp_row{'display_level'} + 1;
              }
              $subRow->{'sub_levels'} = $temp_row{'sub_levels'} - 1;
              if (not defined $subRow->{'values'})
              {
                $subRow->{'values'} = ();
              }
              if (not defined $subRow->{'sql_clause'})
              {
                $subRow->{'sql_clause'} = ();
              }
              my $subNumEntries = scalar @{$temp_hash->{dates}};
              for (my $date_count=0; $date_count < $subNumEntries; $date_count++)
              {
                my $subSubNumEntries = scalar @column_data;
                if (not defined $subRow->{'values'}[$date_count])
                {
                  $subRow->{'values'}[$date_count] = ();
                  for (my $fill_count=0; $fill_count < $subSubNumEntries; $fill_count++)
                  {
                    if (not defined $subRow->{'values'}[$date_count][$fill_count])
                    {
                      $subRow->{'values'}[$date_count][$fill_count] = "0.00";
                    }
                  }
                }
                if (not defined $subRow->{'sql_clause'}[$date_count])
                {
                  $subRow->{'sql_clause'}[$date_count] = ();
                  for (my $fill_count=0; $fill_count < $subSubNumEntries; $fill_count++)
                  {
                    if (not defined $subRow->{'sql_clause'}[$date_count][$fill_count])
                    {
                      $subRow->{'sql_clause'}[$date_count][$fill_count] = "invalid";
                    }
                  }
                }
              }

              if (not defined $subRow->{'sub_rows'}{$sub_account_num})
              {
                $subRow->{'sub_rows'}{$sub_account_num} = {};
              }
              my $subRow2 = $subRow->{sub_rows}{$sub_account_num};
              $subRow2->{'cell_text'} = $sub_account_name;
              $subRow2->{'item_name'} = $sub_account_num;
              $subRow2->{'initial_state'} = $temp_row{'initial_state'};
              if ($temp_row{'initial_state'} eq "o")
              {
                $subRow->{'sub_rows_visible'} = "true";
                $subRow2->{'visible'} = "true";
              }
              else
              {
                $subRow->{'sub_rows_visible'} = "false";
                $subRow2->{'visible'} = "false";
              }
              $subRow2->{'display_level'} = $temp_row{'display_level'} + 2;
              if ($temp_row{'display_level'} + 2 > $max_display_level)
              {
                $max_display_level = $temp_row{'display_level'} + 2;
              }
              $subRow2->{'sub_levels'} = $temp_row{'sub_levels'} - 2;
              if (not defined $subRow2->{'values'})
              {
                $subRow2->{'values'} = ();
              }
              if (not defined $subRow2->{'sql_clause'})
              {
                $subRow2->{'sql_clause'} = ();
              }
              $subNumEntries = scalar @{$temp_hash->{dates}};
              for (my $date_count=0; $date_count < $subNumEntries; $date_count++)
              {
                my $subSubNumEntries = scalar @column_data;
                if (not defined $subRow2->{'values'}[$date_count])
                {
                  $subRow2->{'values'}[$date_count] = ();
                  for (my $fill_count=0; $fill_count < $subSubNumEntries; $fill_count++)
                  {
                    if (not defined $subRow2->{'values'}[$date_count][$fill_count])
                    {
                      $subRow2->{'values'}[$date_count][$fill_count] = "0.00";
                    }
                  }
                }
                if (not defined $subRow2->{'sql_clause'}[$date_count])
                {
                  $subRow2->{'sql_clause'}[$date_count] = ();
                  for (my $fill_count=0; $fill_count < $subSubNumEntries; $fill_count++)
                  {
                    if (not defined $subRow2->{'sql_clause'}[$date_count][$fill_count])
                    {
                      $subRow2->{'sql_clause'}[$date_count][$fill_count] = "invalid";
                    }
                  }
                }
              }

              if ($error_string == 1)
              {
                $temp_hash->{'error'} = $error_string;
              }
              $subRow2->{'error'} = $error_string;
              my $tmp_sql_clause = $sql_clause;
              $tmp_sql_clause =~ s/$accountRE/$account_num/g;
              $tmp_sql_clause =~ s/$subAccountRE/$sub_account_num/g;
              $subRow2->{'sql_clause'}[$count][$column_count] = $tmp_sql_clause;
              $subRow2->{'values'}[$count][$column_count] = $value;
              $subRow2->{'total'} += $subRow2->{'values'}[$count][$column_count];

              $tmp_sql_clause = $level_sql_clause;
              $tmp_sql_clause =~ s/$accountRE/$account_num/g;
              $subRow->{'sql_clause'}[$count][$column_count] = $tmp_sql_clause;
              $subRow->{'values'}[$count][$column_count] += $subRow2->{'values'}[$count][$column_count];
              $subRow->{'total'} += $subRow2->{'values'}[$count][$column_count];
              $temp_account_total[$column_count] += $subRow2->{'values'}[$count][$column_count];
            }
          }
        }
      }

      $temp_hash->{'values'}[$count] = ();
      $temp_hash->{'sql_clause'}[$count] = ();
      if ($column_type eq "nothing")
      {
        $temp_hash->{'values'}[$count][0] = $temp_account_total[0];
        my $sqlClause = "";
        if ($temp_row{'activity'})
        {
          $sqlClause = "je.trans_date >= '$date_start' AND ";
        }
        $sqlClause .= "je.trans_date <= '$date_end'";
        $sqlClause .= " AND db.account_num IN (SELECT number FROM account_tb WHERE type = $temp_hash->{'item_name'}";
        if ($temp_row{'query_type'} == 1)
        {
          $sqlClause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = true)";
        }
        elsif ($temp_row{'query_type'} == 2)
        {
          $sqlClause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = false)";
        }
        $temp_hash->{'sql_clause'}[$count][0] = $sqlClause . ")";
        $temp_hash->{'total'} += $temp_hash->{'values'}[$count][0];
      }
      else
      {
        my $numEntries = scalar @column_data;
        for (my $column_count=0; $column_count < $numEntries; $column_count++)
        {
          if ($column_type eq "sub_entities")
          {
            $temp_hash->{'values'}[$count][$column_count] = $temp_account_total[$column_count];
            my $sqlClause = "";
            if ($temp_row{'activity'})
            {
              $sqlClause = "je.trans_date >= '$date_start' AND ";
            }
            $sqlClause .= "je.trans_date <= '$date_end'";
            $sqlClause .= " AND db.account_num IN (SELECT number FROM account_tb WHERE type = $temp_hash->{'item_name'}";
            if ($temp_row{'query_type'} == 1)
            {
              $sqlClause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = true)";
            }
            elsif ($temp_row{'query_type'} == 2)
            {
              $sqlClause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = false)";
            }
            $sqlClause .= ")";
            $sqlClause .= " AND db.sub_e_code = '$column_data[$column_count]'";
            $temp_hash->{'sql_clause'}[$count][$column_count] = $sqlClause;
          }
          elsif ($column_type eq "sub_accounts")
          {
            if ($temp_hash->{'item_name'} == 0 || $temp_hash->{'item_name'} == 10)
            { # cash/bank account type or sales revenue account type
              $temp_hash->{'values'}[$count][$column_count] = 0;
              $temp_hash->{'sql_clause'}[$count][$column_count] = "invalid";
            }
            else
            {
              $temp_hash->{'values'}[$count][$column_count] = $temp_account_total[$column_count];
              my $sqlClause = "";
              if ($temp_row{'activity'})
              {
                $sqlClause = "je.trans_date >= '$date_start' AND ";
              }
              $sqlClause .= "je.trans_date <= '$date_end'";
              $sqlClause .= " AND db.account_num IN (SELECT number FROM account_tb WHERE type = $temp_hash->{'item_name'}";
              if ($temp_row{'query_type'} == 1)
              {
                $sqlClause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = true)";
              }
              elsif ($temp_row{'query_type'} == 2)
              {
                $sqlClause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = false)";
              }
              $sqlClause .= ")";
              $sqlClause .= " AND db.sub_account_num = $column_data[$column_count]";
              $temp_hash->{'sql_clause'}[$count][$column_count] = $sqlClause;
            }
          }
          elsif ($column_type eq "sub_account_types")
          {
            if ($temp_hash->{'item_name'} == 0 || $temp_hash->{'item_name'} == 10)
            { # cash/bank account type or sales revenue account type
              $temp_hash->{'values'}[$count][$column_count] = 0;
              $temp_hash->{'sql_clause'}[$count][$column_count] = "invalid";
            }
            else
            {
              $temp_hash->{'values'}[$count][$column_count] = $temp_account_total[$column_count];
              my $sqlClause = "";
              if ($temp_row{'activity'})
              {
                $sqlClause = "je.trans_date >= '$date_start' AND ";
              }
              $sqlClause .= "je.trans_date <= '$date_end'";
              $sqlClause .= " AND db.account_num IN (SELECT number FROM account_tb WHERE type = $temp_hash->{'item_name'}";
              if ($temp_row{'query_type'} == 1)
              {
                $sqlClause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = true)";
              }
              elsif ($temp_row{'query_type'} == 2)
              {
                $sqlClause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = false)";
              }
              $sqlClause .= ")";
              $sqlClause .= " AND db.sub_account_num IN (SELECT sub_acc_number FROM sub_acc_tb WHERE type = $column_data[$column_count])";
              $temp_hash->{'sql_clause'}[$count][$column_count] = $sqlClause;
            }
          }
          $temp_hash->{'total'} += $temp_hash->{'values'}[$count][$column_count];
        }
      }
    }
  }

  return $max_display_level;
  # modifying the passed in reference to the hash should update the hash when the function returns
}

sub get_account_data
{
  my %arguments = ( @_, );
  my %args = %{ $arguments{'args'} };
  my $ledger_code = $args{'ledger_code'};
  my $temp_hash = $arguments{'temp_hash'};
  my %temp_row = %{ $arguments{'temp_row'} };
  my @sub_accounts = ();
  my @account_type = ();
  my @column_data = split /,/, $args{'column_data'};
  my $column_type = $args{'column_type'};
  my $max_display_level = -1;
  my $data_precision = $args{'data_precision'};

  if ($column_type eq "nothing")
  {
    @column_data = ( "0" );
  }
  elsif ($column_type eq "sub_entities")
  {
    my $numEntries = scalar @column_data;
    for (my $count=0; $count < $numEntries; $count++)
    {
      # search for "<non_sub_entity>" and replace with ""
      if ($column_data[$count] eq "<non_sub_entity>")
      {
        $column_data[$count] = "";
      }
    }
  }

  # get the account's name
  ($temp_hash->{'cell_text'}) = GetAccountName(dbh => $args{'dbh'}, acct_number => $temp_row{'item_name'});
  # get the account's account type and debit_increases flag
  @account_type = GetAccountType(dbh => $args{'dbh'}, acct_number => $temp_row{'item_name'});

  $temp_hash->{'values'} = ();
  $temp_hash->{'sql_clause'} = ();
  # run over the dates and gather the data for the dates
  my $numEntries = scalar @{$temp_hash->{dates}};
  for (my $count=0; $count < $numEntries; $count++)
  {
    my ($date_start, $date_end);
    if ($args{'date_range'} && $temp_row{'cell_type'} ne "begin_account_balance" && $temp_row{'cell_type'} ne "end_account_balance")
    {
      $date_start = $temp_hash->{'dates'}[$count][0];
      $date_end = $temp_hash->{'dates'}[$count][1];
    }
    else
    {
      if ($temp_row{'cell_type'} eq "begin_account_balance")
      {
        (my $temp_date = $temp_hash->{'dates'}[$count][0]) =~ s/$minusRE/\//g;
        # make date_start and date_end one less day
        $date_start = `/bin/date -d "$temp_date 1 day ago" +%m-%d-%Y`;
        $date_end = $date_start;
      }
      elsif ($temp_row{'cell_type'} eq "end_account_balance")
      {
        $date_end = $temp_hash->{'dates'}[$count][1];
        $date_start = $date_end;
      }
      else
      {
        $date_start = $temp_hash->{'dates'}[$count][0];
        $date_end = $date_start;
      }
    }

    if ($temp_row{'display_level'} > $max_display_level)
    {
      $max_display_level = $temp_row{'display_level'};
    }

    if ($temp_row{'sub_levels'} == 0 || ($column_type eq "sub_accounts" || $column_type eq "sub_account_types"))
    {
      $temp_hash->{'values'}[$count] = ();
      $temp_hash->{'sql_clause'}[$count] = ();
      if ($column_type eq "nothing")
      {
        my %options = ();
        $options{'begin_date'} = $date_start if ($temp_row{'activity'});
        $options{'end_date'} = $date_end;
        $options{'balance'} = 1 if (!$temp_row{'activity'});
        $options{'taxable'} = $temp_row{'query_type'};

        my @result;
        my @accounts = ();

        $accounts[0] = ();
        $accounts[0][0] = $temp_hash->{'item_name'};
        $accounts[0][1] = $account_type[1];
        $accounts[0][2] = $temp_hash->{'cell_text'};

        if ($options{'balance'})
        {
          @result = AccountBalanceActivity(dbh => $args{'dbh'}, ledger_code => $args{'ledger_code'}, accounts => \@accounts, end_date => $date_end, %options);
        }
        else
        {
          @result = AccountActivity(dbh => $args{'dbh'}, ledger_code => $args{'ledger_code'}, accounts => \@accounts, begin_date => $date_start, end_date => $date_end, %options);
        }
        $temp_hash->{'values'}[$count][0] = $result[0][0];
        my $sqlClause = "";
        if ($temp_row{'activity'})
        {
          $sqlClause = "je.trans_date >= '$date_start' AND ";
        }
        $sqlClause .= "je.trans_date <= '$date_end'";
        $sqlClause .= " AND db.account_num = $temp_hash->{'item_name'}";
        $temp_hash->{'sql_clause'}[$count][0] = $sqlClause;
        $temp_hash->{'total'} += $temp_hash->{'values'}[$count][0];
      }
      else
      {
        my $subNumEntries = scalar @column_data;
        for (my $column_count=0; $column_count < $subNumEntries; $column_count++)
        {
          if ($column_type eq "sub_entities")
          {
            my %options = ();
            $options{'begin_date'} = $date_start if ($temp_row{'activity'});
            $options{'end_date'} = $date_end;
            $options{'balance'} = 1 if (!$temp_row{'activity'});
            $options{'taxable'} = $temp_row{'query_type'};

            my @result;
            my @accounts = ();
            my @sub_entity = ();

            $accounts[0] = ();
            $accounts[0][0] = $temp_hash->{'item_name'};
            $accounts[0][1] = $account_type[1];
            $accounts[0][2] = $temp_hash->{'cell_text'};

            $sub_entity[0] = $column_data[$column_count];
            $options{'sub_entity'} = [ @sub_entity ];

            if ($options{'balance'})
            {
              @result = AccountBalanceActivity(dbh => $args{'dbh'}, ledger_code => $args{'ledger_code'}, accounts => \@accounts, end_date => $date_end, %options);
            }
            else
            {
              @result = AccountActivity(dbh => $args{'dbh'}, ledger_code => $args{'ledger_code'}, accounts => \@accounts, begin_date => $date_start, end_date => $date_end, %options);
            }
            $temp_hash->{'values'}[$count][$column_count] = $result[0][0];
            my $sqlClause = "";
            if ($temp_row{'activity'})
            {
              $sqlClause = "je.trans_date >= '$date_start' AND ";
            }
            $sqlClause .= "je.trans_date <= '$date_end'";
            $sqlClause .= " AND db.account_num = $temp_hash->{'item_name'}";
            $sqlClause .= " AND db.sub_e_code = '$column_data[$column_count]'";
            $temp_hash->{'sql_clause'}[$count][$column_count] = $sqlClause;
          }
          elsif ($column_type eq "sub_accounts")
          {
            if ($account_type[0] == 0 || $account_type[0] == 10)
            { # cash/bank account type or sales revenue account type
              $temp_hash->{'values'}[$count][$column_count] = 0;
              $temp_hash->{'sql_clause'}[$count][$column_count] = "invalid";
            }
            else
            {
              my %options = ();
              $options{'begin_date'} = $date_start if ($temp_row{'activity'});
              $options{'end_date'} = $date_end;
              $options{'balance'} = 1 if (!$temp_row{'activity'});
              $options{'taxable'} = $temp_row{'query_type'};

              my @result;
              my @accounts = ();
              my @sub_account = ();

              $accounts[0] = ();
              $accounts[0][0] = $temp_hash->{'item_name'};
              $accounts[0][1] = $account_type[1];
              $accounts[0][2] = $temp_hash->{'cell_text'};

              $sub_account[0] = $column_data[$column_count];
              $options{'sub_account'} = \@sub_account;

              if ($options{'balance'})
              {
                @result = AccountBalanceActivity(dbh => $args{'dbh'}, ledger_code => $args{'ledger_code'}, accounts => \@accounts, end_date => $date_end, %options);
              }
              else
              {
                @result = AccountActivity(dbh => $args{'dbh'}, ledger_code => $args{'ledger_code'}, accounts => \@accounts, begin_date => $date_start, end_date => $date_end, %options);
              }
              $temp_hash->{'values'}[$count][$column_count] = $result[0][0];
              my $sqlClause = "";
              if ($temp_row{'activity'})
              {
                $sqlClause = "je.trans_date >= '$date_start' AND ";
              }
              $sqlClause .= "je.trans_date <= '$date_end'";
              $sqlClause .= " AND db.account_num = $temp_hash->{'item_name'}";
              $sqlClause .= " AND db.sub_account_num = $column_data[$column_count]";
              $temp_hash->{'sql_clause'}[$count][$column_count] = $sqlClause;
            }
          }
          elsif ($column_type eq "sub_account_types")
          {
            if ($account_type[0] == 0 || $account_type[0] == 10)
            { # cash/bank account type or sales revenue account type
              $temp_hash->{'values'}[$count][$column_count] = 0;
              $temp_hash->{'sql_clause'}[$count][$column_count] = "invalid";
            }
            else
            {
              my @sub_accts_for_sub_acct_type = GetSubAccountsForSubAccountType(dbh => $args{'dbh'}, sub_account_type => $column_data[$column_count]);
              my $total = "0.00";
              my $subSubNumEntries = scalar @sub_accts_for_sub_acct_type;
              for (my $z=0; $z < $subSubNumEntries; $z++)
              {
                my %options = ();
                $options{'begin_date'} = $date_start if ($temp_row{'activity'});
                $options{'end_date'} = $date_end;
                $options{'balance'} = 1 if (!$temp_row{'activity'});
                $options{'taxable'} = $temp_row{'query_type'};
                $options{'sub_account'} = \@{$sub_accts_for_sub_acct_type[$z]};

                my @result;
                my @accounts = ();

                $accounts[0] = ();
                $accounts[0][0] = $temp_hash->{'item_name'};
                $accounts[0][1] = $account_type[1];
                $accounts[0][2] = $temp_hash->{'cell_text'};

                if ($options{'balance'})
                {
                  @result = AccountBalanceActivity(dbh => $args{'dbh'}, ledger_code => $args{'ledger_code'}, accounts => \@accounts, end_date => $date_end, %options);
                }
                else
                {
                  @result = AccountActivity(dbh => $args{'dbh'}, ledger_code => $args{'ledger_code'}, accounts => \@accounts, begin_date => $date_start, end_date => $date_end, %options);
                }
                $total += $result[0][0];
              }
              $temp_hash->{'values'}[$count][$column_count] = $total;
              my $sqlClause = "";
              if ($temp_row{'activity'})
              {
                $sqlClause = "je.trans_date >= '$date_start' AND ";
              }
              $sqlClause .= "je.trans_date <= '$date_end'";
              $sqlClause .= " AND db.account_num = $temp_hash->{'item_name'}";
              $sqlClause .= " AND db.sub_account_num IN (SELECT sub_acc_number FROM sub_acc_tb WHERE type = $column_data[$column_count])";
              $temp_hash->{'sql_clause'}[$count][$column_count] = $sqlClause;
            }
          }
          $temp_hash->{'total'} += $temp_hash->{'values'}[$count][$column_count];
        }
      }
    }
    elsif ($temp_row{'sub_levels'} > 0)
    {
      my @temp_sub_account_total;
      my $dbh = $args{'dbh'};
      my $sth;
      my $rc;

      my $tmp_sql_statement = "";
      my @sql_arguments = ();
      my $sql_clause;
      my $error_string = 0;
      # calculate the value (activity/balance) for the sub-account for each date range given the column qualifiers
      if (!($column_type eq "sub_accounts" || $column_type eq "sub_account_types"))
      {
        my $subNumEntries = scalar @column_data;
        for (my $column_count=0; $column_count < $subNumEntries; $column_count++)
        {
          if ($column_type eq "nothing")
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1092};
              @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $date_end, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_start, $date_end);
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1094};
              @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_end, $temp_row{'item_name'}, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_end);
            }

            $sql_clause = "";
            if ($temp_row{'activity'})
            {
              $sql_clause = "je.trans_date >= '$date_start' AND ";
            }
            $sql_clause .= "je.trans_date <= '$date_end'";
            $sql_clause .= " AND db.account_num = $temp_hash->{'item_name'}";
            $sql_clause .= " AND db.sub_account_num = <SUBACCOUNT>";
          }
          elsif ($column_type eq "sub_entities")
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1093};
              @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $date_end, $column_data[$column_count], $temp_hash->{'report_class'}, $ledger_code, $column_data[$column_count], $temp_row{'item_name'}, $date_start, $date_end);
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1094};
              @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_end, $temp_row{'item_name'}, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_end);
              $error_string = 1;
            }

            $sql_clause = "";
            if ($temp_row{'activity'})
            {
              $sql_clause = "je.trans_date >= '$date_start' AND ";
            }
            $sql_clause .= "je.trans_date <= '$date_end'";
            $sql_clause .= " AND db.account_num = $temp_hash->{'item_name'}";
            $sql_clause .= " AND db.sub_account_num = <SUBACCOUNT>";
            $sql_clause .= " AND db.sub_e_code = '$column_data[$column_count]'";
          }
          $sth = $dbh->prepare( $tmp_sql_statement ) || my_db_die("Error: Report.cgi - get_account_data", 4007, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);
          $rc = $sth->execute(@sql_arguments) || my_db_die("Error: Report.cgi - get_account_data", 4008, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);
          while (my $row_array = $sth->fetchrow_arrayref())
          {
            my $sub_account_num = $row_array->[0];
            my $sub_account_name = $row_array->[1];
            my $value = $row_array->[2];

            if ($sub_account_num ne "")
            {
              if (not defined $temp_hash->{'sub_rows'})
              {
                $temp_hash->{'sub_rows'} = {};
              }

              if (not defined $temp_hash->{'sub_rows'}{$sub_account_num})
              {
                $temp_hash->{'sub_rows'}{$sub_account_num} = {};
              }
              my $subRow = $temp_hash->{sub_rows}{$sub_account_num};
              $subRow->{'item_name'} = $sub_account_num;
              $subRow->{'cell_text'} = $sub_account_name;
              $subRow->{'initial_state'} = $temp_row{'initial_state'};
              if ($temp_row{'initial_state'} eq "o")
              {
                $temp_hash->{'sub_rows_visible'} = "true";
                $subRow->{'visible'} = "true";
              }
              else
              {
                $temp_hash->{'sub_rows_visible'} = "false";
                $subRow->{'visible'} = "false";
              }
              $subRow->{'display_level'} = $temp_row{'display_level'} + 1;
              if ($temp_row{'display_level'} + 1 > $max_display_level)
              {
                $max_display_level = $temp_row{'display_level'} + 1;
              }
              $subRow->{'sub_levels'} = $temp_row{'sub_levels'} - 1;
              if (not defined $subRow->{'values'})
              {
                $subRow->{'values'} = ();
              }
              if (not defined $subRow->{'sql_clause'})
              {
                $subRow->{'sql_clause'} = ();
              }
              my $subSubNumEntries = scalar @{$temp_hash->{dates}};
              for (my $date_count=0; $date_count < $subSubNumEntries; $date_count++)
              {
                my $numColumnData = scalar @column_data;
                if (not defined $subRow->{'values'}[$date_count])
                {
                  $subRow->{'values'}[$date_count] = ();
                  for (my $fill_count=0; $fill_count < $numColumnData; $fill_count++)
                  {
                    if (not defined $subRow->{'values'}[$date_count][$fill_count])
                    {
                      $subRow->{'values'}[$date_count][$fill_count] = "0.00";
                    }
                  }
                }
                if (not defined $subRow->{'sql_clause'}[$date_count])
                {
                  $subRow->{'sql_clause'}[$date_count] = ();
                  for (my $fill_count=0; $fill_count < $numColumnData; $fill_count++)
                  {
                    if (not defined $subRow->{'sql_clause'}[$date_count][$fill_count])
                    {
                      $subRow->{'sql_clause'}[$date_count][$fill_count] = "invalid";
                    }
                  }
                }
              }

              if ($error_string == 1)
              {
                $temp_hash->{'error'} = $error_string;
              }
              $subRow->{'error'} = $error_string;
              my $tmp_sql_clause = $sql_clause;
              $tmp_sql_clause =~ s/$subAccountRE/$sub_account_num/g;
              $subRow->{'sql_clause'}[$count][$column_count] = $tmp_sql_clause;
              $subRow->{'values'}[$count][$column_count] = $value;
              $subRow->{'total'} += $subRow->{'values'}[$count][$column_count];
              $temp_sub_account_total[$column_count] += $subRow->{'values'}[$count][$column_count];
            }
          }
        }
      }

      $temp_hash->{'values'}[$count] = ();
      if ($column_type eq "nothing")
      {
        if ($account_type[0] == 0 || $account_type[0] == 10)
        { # cash/bank account type or sales revenue account type
          my %options = ();
          $options{'begin_date'} = $date_start if ($temp_row{'activity'});
          $options{'end_date'} = $date_end;
          $options{'balance'} = 1 if (!$temp_row{'activity'});
          $options{'taxable'} = $temp_row{'query_type'};

          my @result;
          my @accounts = ();

          $accounts[0] = ();
          $accounts[0][0] = $temp_hash->{'item_name'};
          $accounts[0][1] = $account_type[1];
          $accounts[0][2] = $temp_hash->{'cell_text'};

          if ($options{'balance'})
          {
            @result = AccountBalanceActivity(dbh => $args{'dbh'}, ledger_code => $args{'ledger_code'}, accounts => \@accounts, end_date => $date_end, %options);
          }
          else
          {
            @result = AccountActivity(dbh => $args{'dbh'}, ledger_code => $args{'ledger_code'}, accounts => \@accounts, begin_date => $date_start, end_date => $date_end, %options);
          }
          $temp_hash->{'values'}[$count][0] = $result[0][0];
        }
        else
        {
          $temp_hash->{'values'}[$count][0] = $temp_sub_account_total[0];
        }
        my $sqlClause = "";
        if ($temp_row{'activity'})
        {
          $sqlClause = "je.trans_date >= '$date_start' AND ";
        }
        $sqlClause .= "je.trans_date <= '$date_end'";
        $sqlClause .= " AND db.account_num = $temp_hash->{'item_name'}";
        $temp_hash->{'sql_clause'}[$count][0] = $sqlClause;
        $temp_hash->{'total'} += $temp_hash->{'values'}[$count][0];
        $temp_hash->{'total'} = $temp_hash->{'total'};
      }
      else
      {
        my $subNumEntries = scalar @column_data;
        for (my $column_count=0; $column_count < $subNumEntries; $column_count++)
        {
          if ($column_type eq "sub_entities")
          {
            if ($account_type[0] == 0 || $account_type[0] == 10)
            { # cash/bank account type or sales revenue account type
              my %options = ();
              $options{'begin_date'} = $date_start if ($temp_row{'activity'});
              $options{'end_date'} = $date_end;
              $options{'balance'} = 1 if (!$temp_row{'activity'});
              $options{'taxable'} = $temp_row{'query_type'};

              my @result;
              my @accounts = ();
              my @sub_entity = ();

              $accounts[0] = ();
              $accounts[0][0] = $temp_hash->{'item_name'};
              $accounts[0][1] = $account_type[1];
              $accounts[0][2] = $temp_hash->{'cell_text'};

              $sub_entity[0] = $column_data[$column_count];
              $options{'sub_entity'} = \@sub_entity;

              if ($options{'balance'})
              {
                @result = AccountBalanceActivity(dbh => $args{'dbh'}, ledger_code => $args{'ledger_code'}, accounts => \@accounts, end_date => $date_end, %options);
              }
              else
              {
                @result = AccountActivity(dbh => $args{'dbh'}, ledger_code => $args{'ledger_code'}, accounts => \@accounts, begin_date => $date_start, end_date => $date_end, %options);
              }
              $temp_hash->{'values'}[$count][$column_count] = $result[0][0];
            }
            else
            {
              $temp_hash->{'values'}[$count][$column_count] = $temp_sub_account_total[$column_count];
            }
            my $sqlClause = "";
            if ($temp_row{'activity'})
            {
              $sqlClause = "je.trans_date >= '$date_start' AND ";
            }
            $sqlClause .= "je.trans_date <= '$date_end'";
            $sqlClause .= " AND db.account_num = $temp_hash->{'item_name'}";
            $sqlClause .= " AND db.sub_e_code = '$column_data[$column_count]'";
            $temp_hash->{'sql_clause'}[$count][$column_count] = $sqlClause;
          }
          elsif ($column_type eq "sub_accounts")
          {
            if ($account_type[0] == 0 || $account_type[0] == 10)
            { # cash/bank account type or sales revenue account type
              $temp_hash->{'values'}[$count][$column_count] = 0;
              $temp_hash->{'sql_clause'}[$count][$column_count] = "invalid";
            }
            else
            {
              my %options = ();
              $options{'begin_date'} = $date_start if ($temp_row{'activity'});
              $options{'end_date'} = $date_end;
              $options{'balance'} = 1 if (!$temp_row{'activity'});
              $options{'taxable'} = $temp_row{'query_type'};

              my @result;
              my @accounts = ();
              my @sub_account = ();

              $accounts[0] = ();
              $accounts[0][0] = $temp_hash->{'item_name'};
              $accounts[0][1] = $account_type[1];
              $accounts[0][2] = $temp_hash->{'cell_text'};

              $sub_account[0] = $column_data[$column_count];
              $options{'sub_account'} = \@sub_account;

              if ($options{'balance'})
              {
                @result = AccountBalanceActivity(dbh => $args{'dbh'}, ledger_code => $args{'ledger_code'}, accounts => \@accounts, end_date => $date_end, %options);
              }
              else
              {
                @result = AccountActivity(dbh => $args{'dbh'}, ledger_code => $args{'ledger_code'}, accounts => \@accounts, begin_date => $date_start, end_date => $date_end, %options);
              }
              $temp_hash->{'values'}[$count][$column_count] = $result[0][0];
              my $sqlClause = "";
              if ($temp_row{'activity'})
              {
                $sqlClause = "je.trans_date >= '$date_start' AND ";
              }
              $sqlClause .= "je.trans_date <= '$date_end'";
              $sqlClause .= " AND db.account_num = $temp_hash->{'item_name'}";
              $sqlClause .= " AND db.sub_account_num = $column_data[$column_count]";
              $temp_hash->{'sql_clause'}[$count][$column_count] = $sqlClause;
            }
          }
          elsif ($column_type eq "sub_account_types")
          {
            if ($account_type[0] == 0 || $account_type[0] == 10)
            { # cash/bank account type or sales revenue account type
              $temp_hash->{'values'}[$count][$column_count] = 0;
              $temp_hash->{'sql_clause'}[$count][$column_count] = "invalid";
            }
            else
            {
              my @sub_accts_for_sub_acct_type = GetSubAccountsForSubAccountType(dbh => $args{'dbh'}, sub_account_type => $column_data[$column_count]);
              my $total = "0.00";
              my $subSubNumEntries = scalar @sub_accts_for_sub_acct_type;
              for (my $z=0; $z < $subSubNumEntries; $z++)
              {
                my %options = ();
                $options{'begin_date'} = $date_start if ($temp_row{'activity'});
                $options{'end_date'} = $date_end;
                $options{'balance'} = 1 if (!$temp_row{'activity'});
                $options{'taxable'} = $temp_row{'query_type'};
                $options{'sub_account'} = \@{$sub_accts_for_sub_acct_type[$z]};

                my @result;
                my @accounts = ();

                $accounts[0] = ();
                $accounts[0][0] = $temp_hash->{'item_name'};
                $accounts[0][1] = $account_type[1];
                $accounts[0][2] = $temp_hash->{'cell_text'};

                if ($options{'balance'})
                {
                  @result = AccountBalanceActivity(dbh => $args{'dbh'}, ledger_code => $args{'ledger_code'}, accounts => \@accounts, end_date => $date_end, %options);
                }
                else
                {
                  @result = AccountActivity(dbh => $args{'dbh'}, ledger_code => $args{'ledger_code'}, accounts => \@accounts, begin_date => $date_start, end_date => $date_end, %options);
                }
                $total += $result[0][0];
              }
              $temp_hash->{'values'}[$count][$column_count] = $total;
              my $sqlClause = "";
              if ($temp_row{'activity'})
              {
                $sqlClause = "je.trans_date >= '$date_start' AND ";
              }
              $sqlClause .= "je.trans_date <= '$date_end'";
              $sqlClause .= " AND db.account_num = $temp_hash->{'item_name'}";
              $sqlClause .= " AND db.sub_account_num IN (SELECT sub_acc_number FROM sub_acc_tb WHERE type = $column_data[$column_count])";
              $temp_hash->{'sql_clause'}[$count][$column_count] = $sqlClause;
            }
          }
          $temp_hash->{'total'} += $temp_hash->{'values'}[$count][$column_count];
        }
      }
    }
  }

  return $max_display_level;
}

sub get_sub_account_type_data
{
  my %arguments = ( @_, );
  my %args = %{ $arguments{'args'} };
  my $ledger_code = $args{'ledger_code'};
  my $temp_hash = $arguments{'temp_hash'};
  my %temp_row = %{ $arguments{'temp_row'} };
  my @sub_accounts = ();
  my %sub_account_accounts = {};
  my @account_type = ();
  my @column_data = split /,/, $args{'column_data'};
  my $column_type = $args{'column_type'};
  my $max_display_level = -1;
  my $data_precision = $args{'data_precision'};

  if ($column_type eq "nothing")
  {
    @column_data = ( "0" );
  }
  elsif ($column_type eq "sub_entities")
  {
    my $numEntries = scalar @column_data;
    for (my $count=0; $count < $numEntries; $count++)
    {
      # search for "<non_sub_entity>" and replace with ""
      if ($column_data[$count] eq "<non_sub_entity>")
      {
        $column_data[$count] = "";
      }
    }
  }

  # get the name of the account type
  my @temp_item_name = ();
  $temp_item_name[0] = $temp_row{'item_name'};
  my @temp_cell_text = GetAccountTypeNames(dbh => $args{'dbh'}, account_types => \@temp_item_name);
  $temp_hash->{'cell_text'} = $temp_cell_text[0];

  $temp_hash->{'values'} = ();
  $temp_hash->{'sql_clause'} = ();
  # run over the dates and gather the data for the dates
  my $numEntries = scalar @{$temp_hash->{dates}};
  for (my $count=0; $count < $numEntries; $count++)
  {
    my ($date_start, $date_end);
    if ($args{'date_range'})
    {
      $date_start = $temp_hash->{'dates'}[$count][0];
      $date_end = $temp_hash->{'dates'}[$count][1];
    }
    else
    {
      $date_start = $temp_hash->{'dates'}[$count][0];
      $date_end = $date_start;
    }

    if ($temp_row{'display_level'} > $max_display_level)
    {
      $max_display_level = $temp_row{'display_level'};
    }

    my @temp_sub_account_type_total;
    my $dbh = $args{'dbh'};
    my $sth;
    my $rc;

    my $tmp_sql_statement = "";
    my @sql_arguments = ();
    my $sql_clause;
    my $level_sql_clause;
    my $sub_level_sql_clause;
    my $error_string = 0;
    # calculate the value (activity/balance) for the account type for the current date range using each account given the column qualifiers
    if ($temp_row{'sub_levels'} == 0 || ($column_type eq "sub_accounts" || $column_type eq "sub_account_types"))
    {
      my $subNumEntries = scalar @column_data;
      for (my $column_count = 0; $column_count < $subNumEntries; $column_count++)
      {
        if ($column_type eq "nothing")
        {
          if ($temp_row{'query_type'} == 0)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1101};
              @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $date_end, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_start, $date_end);
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1111};
              @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_end, $temp_row{'item_name'}, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_end);
            }
          }
          elsif ($temp_row{'query_type'} == 1)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1102};
              @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $date_end, "true", $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "true", $date_start, $date_end);
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1112};
              @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, "true", $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "true", $date_end, $temp_row{'item_name'}, $ledger_code, $date_start, "true", $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "true", $date_end);
            }
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1102};
              @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $date_end, "false", $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "false", $date_start, $date_end);
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1112};
              @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, "false", $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "false", $date_end, $temp_row{'item_name'}, $ledger_code, $date_start, "false", $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "false", $date_end);
            }
          }

          $sql_clause = "";
          if ($temp_row{'activity'})
          {
            $sql_clause = "je.trans_date >= '$date_start' AND ";
          }
          $sql_clause .= "je.trans_date <= '$date_end'";
          $sql_clause .= " AND db.account_num IN (SELECT number FROM account_tb WHERE type = $temp_hash->{'item_name'}";
          if ($temp_row{'query_type'} == 1)
          {
            $sql_clause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = true)";
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            $sql_clause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = false)";
          }
          $sql_clause .= ")";
          $sql_clause .= " AND db.sub_account_num = <SUBACCOUNT>";

          $level_sql_clause = "";
          if ($temp_row{'activity'})
          {
            $level_sql_clause = "je.trans_date >= '$date_start' AND ";
          }
          $level_sql_clause .= "je.trans_date <= '$date_end'";
          $level_sql_clause .= " AND db.account_num IN (SELECT number FROM account_tb WHERE type = $temp_hash->{'item_name'}";
          if ($temp_row{'query_type'} == 1)
          {
            $level_sql_clause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = true)";
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            $level_sql_clause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = false)";
          }
          $level_sql_clause .= ")";
          $level_sql_clause .= " AND db.sub_account_num IN (SELECT sub_acc_number FROM sub_acc_tb WHERE type = <SUBACCOUNTTYPE>)";
        }
        elsif ($column_type eq "sub_entities")
        {
          if ($temp_row{'query_type'} == 0)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1103};
              @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $date_end, $column_data[$column_count], $temp_hash->{'report_class'}, $ledger_code, $column_data[$column_count], $temp_row{'item_name'}, $date_start, $date_end);
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1111};
              @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_end, $temp_row{'item_name'}, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_end);
              $error_string = 1;
            }
          }
          elsif ($temp_row{'query_type'} == 1)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1104};
              @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $date_end, "true", $column_data[$column_count], $temp_hash->{'report_class'}, $ledger_code, $column_data[$column_count], $temp_row{'item_name'}, $ledger_code, "true", $date_start, $date_end);
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1112};
              @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, "true", $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "true", $date_end, $temp_row{'item_name'}, $ledger_code, $date_start, "true", $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "true", $date_end);
              $error_string = 1;
            }
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1104};
              @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $date_end, "false", $column_data[$column_count], $temp_hash->{'report_class'}, $ledger_code, $column_data[$column_count], $temp_row{'item_name'}, $ledger_code, "false", $date_start, $date_end);
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1112};
              @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, "false", $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "false", $date_end, $temp_row{'item_name'}, $ledger_code, $date_start, "false", $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "false", $date_end);
              $error_string = 1;
            }
          }

          $sql_clause = "";
          if ($temp_row{'activity'})
          {
            $sql_clause = "je.trans_date >= '$date_start' AND ";
          }
          $sql_clause .= "je.trans_date <= '$date_end'";
          $sql_clause .= " AND db.account_num IN (SELECT number FROM account_tb WHERE type = $temp_hash->{'item_name'}";
          if ($temp_row{'query_type'} == 1)
          {
            $sql_clause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = true)";
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            $sql_clause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = false)";
          }
          $sql_clause .= ")";
          $sql_clause .= " AND db.sub_account_num = <SUBACCOUNT>";
          $sql_clause .= " AND db.sub_e_code = '$column_data[$column_count]'";

          $level_sql_clause = "";
          if ($temp_row{'activity'})
          {
            $level_sql_clause = "je.trans_date >= '$date_start' AND ";
          }
          $level_sql_clause .= "je.trans_date <= '$date_end'";
          $level_sql_clause .= " AND db.account_num IN (SELECT number FROM account_tb WHERE type = $temp_hash->{'item_name'}";
          if ($temp_row{'query_type'} == 1)
          {
            $level_sql_clause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = true)";
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            $level_sql_clause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = false)";
          }
          $level_sql_clause .= ")";
          $level_sql_clause .= " AND db.sub_account_num IN (SELECT sub_acc_number FROM sub_acc_tb WHERE type = <SUBACCOUNTTYPE>)";
          $level_sql_clause .= " AND db.sub_e_code = '$column_data[$column_count]'";
        }
        $sth = $dbh->prepare( $tmp_sql_statement ) || my_db_die("Error: Report.cgi - get_sub_account_type_data", 4007, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);
        $rc = $sth->execute(@sql_arguments) || my_db_die("Error: Report.cgi - get_sub_account_type_data", 4008, "SQL Statement = '$tmp_sql_statement'<br>\n\@sql_arguments = (" . join(", ", @sql_arguments) . ")<br>\nAt Line " . __LINE__);
        while (my $row_array = $sth->fetchrow_arrayref())
        {
          my $sub_account_type_num = $row_array->[0];
          my $sub_account_type_name = $row_array->[1];
          my $sub_account_num = $row_array->[2];
          my $sub_account_name = $row_array->[3];
          my $value = $row_array->[4];

          if ($sub_account_type_num ne "" && $sub_account_num ne "")
          {
            if (not defined $temp_hash->{'sub_rows'})
            {
              $temp_hash->{'sub_rows'} = {};
            }

            if (not defined $temp_hash->{'sub_rows'}{$sub_account_type_num})
            {
              $temp_hash->{'sub_rows'}{$sub_account_type_num} = {};
            }
            my $subRow = $temp_hash->{sub_rows}{$sub_account_type_num};
            $subRow->{'cell_text'} = $sub_account_type_name;
            $subRow->{'item_name'} = $sub_account_type_num;
            $subRow->{'initial_state'} = $temp_row{'initial_state'};
            if ($temp_row{'initial_state'} eq "o")
            {
              $temp_hash->{'sub_rows_visible'} = "true";
              $subRow->{'visible'} = "true";
            }
            else
            {
              $temp_hash->{'sub_rows_visible'} = "false";
              $subRow->{'visible'} = "false";
            }
            $subRow->{'display_level'} = $temp_row{'display_level'} + 1;
            if ($temp_row{'display_level'} + 1 > $max_display_level)
            {
              $max_display_level = $temp_row{'display_level'} + 1;
            }
            $subRow->{'sub_levels'} = $temp_row{'sub_levels'};
            if (not defined $subRow->{'values'})
            {
              $subRow->{'values'} = ();
            }
            if (not defined $subRow->{'sql_clause'})
            {
              $subRow->{'sql_clause'} = ();
            }
            my $subSubNumEntries = scalar @{$temp_hash->{dates}};
            for (my $date_count=0; $date_count < $subSubNumEntries; $date_count++)
            {
              my $numColumnData = scalar @column_data;
              if (not defined $subRow->{'values'}[$date_count])
              {
                $subRow->{'values'}[$date_count] = ();
                for (my $fill_count=0; $fill_count < $numColumnData; $fill_count++)
                {
                  if (not defined $subRow->{'values'}[$date_count][$fill_count])
                  {
                    $subRow->{'values'}[$date_count][$fill_count] = "0.00";
                  }
                }
              }
              if (not defined $subRow->{'sql_clause'}[$date_count])
              {
                $subRow->{'sql_clause'}[$date_count] = ();
                for (my $fill_count=0; $fill_count < $numColumnData; $fill_count++)
                {
                  if (not defined $subRow->{'sql_clause'}[$date_count][$fill_count])
                  {
                    $subRow->{'sql_clause'}[$date_count][$fill_count] = "invalid";
                  }
                }
              }
            }

            if (not defined $subRow->{'sub_rows'})
            {
              $subRow->{'sub_rows'} = {};
            }
            if (not defined $subRow->{'sub_rows'}{$sub_account_num})
            {
              $subRow->{'sub_rows'}{$sub_account_num} = {};
            }
            my $subRow2 = $subRow->{sub_rows}{$sub_account_num};
            $subRow2->{'cell_text'} = $sub_account_name;
            $subRow2->{'item_name'} = $sub_account_num;
            $subRow2->{'initial_state'} = $temp_row{'initial_state'};
            if ($temp_row{'initial_state'} eq "o")
            {
              $subRow->{'sub_rows_visible'} = "true";
              $subRow2->{'visible'} = "true";
            }
            else
            {
              $subRow->{'sub_rows_visible'} = "false";
              $subRow2->{'visible'} = "false";
            }
            $subRow2->{'display_level'} = $temp_row{'display_level'} + 2;
            if ($temp_row{'display_level'} + 2 > $max_display_level)
            {
              $max_display_level = $temp_row{'display_level'} + 2;
            }
            $subRow2->{'sub_levels'} = $temp_row{'sub_levels'};
            if (not defined $subRow2->{'values'})
            {
              $subRow2->{'values'} = ();
            }
            if (not defined $subRow2->{'sql_clause'})
            {
              $subRow2->{'sql_clause'} = ();
            }
            $subSubNumEntries = scalar @{$temp_hash->{dates}};
            for (my $date_count=0; $date_count < $subSubNumEntries; $date_count++)
            {
              my $numColumnData = scalar @column_data;
              if (not defined $subRow2->{'values'}[$date_count])
              {
                $subRow2->{'values'}[$date_count] = ();
                for (my $fill_count=0; $fill_count < $numColumnData; $fill_count++)
                {
                  if (not defined $subRow2->{'values'}[$date_count][$fill_count])
                  {
                    $subRow2->{'values'}[$date_count][$fill_count] = "0.00";
                  }
                }
              }
              if (not defined $subRow2->{'sql_clause'}[$date_count])
              {
                $subRow2->{'sql_clause'}[$date_count] = ();
                for (my $fill_count=0; $fill_count < $numColumnData; $fill_count++)
                {
                  if (not defined $subRow2->{'sql_clause'}[$date_count][$fill_count])
                  {
                    $subRow2->{'sql_clause'}[$date_count][$fill_count] = "invalid";
                  }
                }
              }
            }

            if ($error_string == 1)
            {
              $temp_hash->{'error'} = $error_string;
            }
            $subRow2->{'error'} = $error_string;
            my $tmp_sql_clause = $sql_clause;
            $tmp_sql_clause =~ s/$subAccountRE/$sub_account_num/g;
            $subRow2->{'sql_clause'}[$count][$column_count] = $tmp_sql_clause;
            $subRow2->{'values'}[$count][$column_count] = $value;
            $subRow2->{'total'} += $subRow2->{'values'}[$count][$column_count];

            $subRow->{'error'} = $error_string;
            $tmp_sql_clause = $level_sql_clause;
            $tmp_sql_clause =~ s/$subAccountTypeRE/$sub_account_type_num/g;
            $subRow->{'sql_clause'}[$count][$column_count] = $tmp_sql_clause;
            $subRow->{'values'}[$count][$column_count] += $subRow2->{'values'}[$count][$column_count];
            $subRow->{'total'} += $subRow2->{'values'}[$count][$column_count];
            $temp_sub_account_type_total[$column_count] += $subRow2->{'values'}[$count][$column_count];
          }
        }
      }
    }
    elsif ($temp_row{'sub_levels'} > 0)
    {
      my $numColumnData = scalar @column_data;
      for (my $column_count = 0; $column_count < $numColumnData; $column_count++)
      {
        if ($column_type eq "nothing")
        {
          if ($temp_row{'query_type'} == 0)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1105};
              @sql_arguments = ($ledger_code, $date_start, $date_end, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_start, $date_end);
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1109};
              @sql_arguments = ($ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_end, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_end);
            }
          }
          elsif ($temp_row{'query_type'} == 1)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1106};
              @sql_arguments = ($ledger_code, $date_start, $date_end, "true", $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "true", $date_start, $date_end);
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1110};
              @sql_arguments = ($ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "true", $date_end, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "true", $date_end);
            }
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1106};
              @sql_arguments = ($ledger_code, $date_start, $date_end, "false", $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "false", $date_start, $date_end);
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1110};
              @sql_arguments = ($ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "false", $date_end, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "false", $date_end);
            }
          }

          $sub_level_sql_clause = "";
          if ($temp_row{'activity'})
          {
            $sub_level_sql_clause = "je.trans_date >= '$date_start' AND ";
          }
          $sub_level_sql_clause .= "je.trans_date <= '$date_end'";
          $sub_level_sql_clause .= " AND db.account_num = <ACCOUNT>";
          $sub_level_sql_clause .= " AND db.sub_account_num = <SUBACCOUNT>";

          $sql_clause = "";
          if ($temp_row{'activity'})
          {
            $sql_clause = "je.trans_date >= '$date_start' AND ";
          }
          $sql_clause .= "je.trans_date <= '$date_end'";
          $sql_clause .= " AND db.account_num IN (SELECT number FROM account_tb WHERE type = $temp_hash->{'item_name'}";
          if ($temp_row{'query_type'} == 1)
          {
            $sql_clause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = true)";
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            $sql_clause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = false)";
          }
          $sql_clause .= ")";
          $sql_clause .= " AND db.sub_account_num = <SUBACCOUNT>";

          $level_sql_clause = "";
          if ($temp_row{'activity'})
          {
            $level_sql_clause = "je.trans_date >= '$date_start' AND ";
          }
          $level_sql_clause .= "je.trans_date <= '$date_end'";
          $level_sql_clause .= " AND db.account_num IN (SELECT number FROM account_tb WHERE type = $temp_hash->{'item_name'}";
          if ($temp_row{'query_type'} == 1)
          {
            $level_sql_clause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = true)";
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            $level_sql_clause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = false)";
          }
          $level_sql_clause .= ")";
          $level_sql_clause .= " AND db.sub_account_num IN (SELECT sub_acc_number FROM sub_acc_tb WHERE type = <SUBACCOUNTTYPE>)";
        }
        elsif ($column_type eq "sub_entities")
        {
          if ($temp_row{'query_type'} == 0)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1107};
              @sql_arguments = ($ledger_code, $date_start, $date_end, $column_data[$column_count], $temp_hash->{'report_class'}, $ledger_code, $column_data[$column_count], $temp_row{'item_name'}, $date_start, $date_end);
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1109};
              @sql_arguments = ($ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_end, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_end);
              $error_string = 1;
            }
          }
          elsif ($temp_row{'query_type'} == 1)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1108};
              @sql_arguments = ($ledger_code, $date_start, $date_end, "true", $column_data[$column_count], $temp_hash->{'report_class'}, $ledger_code, $column_data[$column_count], $temp_row{'item_name'}, $ledger_code, "true", $date_start, $date_end);
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1110};
              @sql_arguments = ($ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "true", $date_end, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "true", $date_end);
              $error_string = 1;
            }
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            if ($temp_row{'activity'})
            {
              $tmp_sql_statement = $sql_statements{1108};
              @sql_arguments = ($ledger_code, $date_start, $date_end, "false", $column_data[$column_count], $temp_hash->{'report_class'}, $ledger_code, $column_data[$column_count], $temp_row{'item_name'}, $ledger_code, "false", $date_start, $date_end);
            }
            else
            {
              $tmp_sql_statement = $sql_statements{1110};
              @sql_arguments = ($ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "false", $date_end, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "false", $date_end);
              $error_string = 1;
            }
          }

          $sub_level_sql_clause = "";
          if ($temp_row{'activity'})
          {
            $sub_level_sql_clause = "je.trans_date >= '$date_start' AND ";
          }
          $sub_level_sql_clause .= "je.trans_date <= '$date_end'";
          $sub_level_sql_clause .= " AND db.account_num = <ACCOUNT>";
          $sub_level_sql_clause .= " AND db.sub_account_num = <SUBACCOUNT>";
          $sub_level_sql_clause .= " AND db.sub_e_code = '$column_data[$column_count]'";

          $sql_clause = "";
          if ($temp_row{'activity'})
          {
            $sql_clause = "je.trans_date >= '$date_start' AND ";
          }
          $sql_clause .= "je.trans_date <= '$date_end'";
          $sql_clause .= " AND db.account_num IN (SELECT number FROM account_tb WHERE type = $temp_hash->{'item_name'}";
          if ($temp_row{'query_type'} == 1)
          {
            $sql_clause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = true)";
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            $sql_clause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = false)";
          }
          $sql_clause .= ")";
          $sql_clause .= " AND db.sub_account_num = <SUBACCOUNT>";
          $sql_clause .= " AND db.sub_e_code = '$column_data[$column_count]'";

          $level_sql_clause = "";
          if ($temp_row{'activity'})
          {
            $level_sql_clause = "je.trans_date >= '$date_start' AND ";
          }
          $level_sql_clause .= "je.trans_date <= '$date_end'";
          $level_sql_clause .= " AND db.account_num IN (SELECT number FROM account_tb WHERE type = $temp_hash->{'item_name'}";
          if ($temp_row{'query_type'} == 1)
          {
            $level_sql_clause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = true)";
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            $level_sql_clause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = false)";
          }
          $level_sql_clause .= ")";
          $level_sql_clause .= " AND db.sub_account_num IN (SELECT sub_acc_number FROM sub_acc_tb WHERE type = <SUBACCOUNTTYPE>)";
          $level_sql_clause .= " AND db.sub_e_code = '$column_data[$column_count]'";
        }
        $sth = $dbh->prepare( $tmp_sql_statement ) || my_db_die("Error: Report.cgi - get_sub_account_type_data", 4007, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);
        $rc = $sth->execute(@sql_arguments) || my_db_die("Error: Report.cgi - get_sub_account_type_data", 4008, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);
        while (my $row_array = $sth->fetchrow_arrayref())
        {
          my $sub_account_type_num = $row_array->[0];
          my $sub_account_type_name = $row_array->[1];
          my $sub_account_num = $row_array->[2];
          my $sub_account_name = $row_array->[3];
          my $account_num = $row_array->[4];
          my $account_name = $row_array->[5];
          my $value = $row_array->[6];

          if ($sub_account_type_num ne "" && $sub_account_num ne "" && $account_num ne "")
          {
            if (not defined $temp_hash->{'sub_rows'})
            {
              $temp_hash->{'sub_rows'} = {};
            }

            if (not defined $temp_hash->{'sub_rows'}{$sub_account_type_num})
            {
              $temp_hash->{'sub_rows'}{$sub_account_type_num} = {};
            }
            my $subRow = $temp_hash->{sub_rows}{$sub_account_type_num};
            $subRow->{'cell_text'} = $sub_account_type_name;
            $subRow->{'item_name'} = $sub_account_type_num;
            $subRow->{'initial_state'} = $temp_row{'initial_state'};
            if ($temp_row{'initial_state'} eq "o")
            {
              $temp_hash->{'sub_rows_visible'} = "true";
              $subRow->{'visible'} = "true";
            }
            else
            {
              $temp_hash->{'sub_rows_visible'} = "false";
              $subRow->{'visible'} = "false";
            }
            $subRow->{'display_level'} = $temp_row{'display_level'} + 1;
            if ($temp_row{'display_level'} + 1 > $max_display_level)
            {
              $max_display_level = $temp_row{'display_level'} + 1;
            }
            $subRow->{'sub_levels'} = $temp_row{'sub_levels'};
            if (not defined $subRow->{'values'})
            {
              $subRow->{'values'} = ();
            }
            if (not defined $subRow->{'sql_clause'})
            {
              $subRow->{'sql_clause'} = ();
            }
            my $subNumEntries = scalar @{$temp_hash->{dates}};
            for (my $date_count=0; $date_count < $subNumEntries; $date_count++)
            {
              if (not defined $subRow->{'values'}[$date_count])
              {
                $subRow->{'values'}[$date_count] = ();
                for (my $fill_count=0; $fill_count < $numColumnData; $fill_count++)
                {
                  if (not defined $subRow->{'values'}[$date_count][$fill_count])
                  {
                    $subRow->{'values'}[$date_count][$fill_count] = "0.00";
                  }
                }
              }
              if (not defined $subRow->{'sql_clause'}[$date_count])
              {
                $subRow->{'sql_clause'}[$date_count] = ();
                for (my $fill_count=0; $fill_count < $numColumnData; $fill_count++)
                {
                  if (not defined $subRow->{'sql_clause'}[$date_count][$fill_count])
                  {
                    $subRow->{'sql_clause'}[$date_count][$fill_count] = "invalid";
                  }
                }
              }
            }

            if (not defined $subRow->{'sub_rows'})
            {
              $subRow->{'sub_rows'} = {};
            }
            if (not defined $subRow->{'sub_rows'}{$sub_account_num})
            {
              $subRow->{'sub_rows'}{$sub_account_num} = {};
            }
            my $subRow2 = $subRow->{sub_rows}{$sub_account_num};
            $subRow2->{'cell_text'} = $sub_account_name;
            $subRow2->{'item_name'} = $sub_account_num;
            $subRow2->{'initial_state'} = $temp_row{'initial_state'};
            if ($temp_row{'initial_state'} eq "o")
            {
              $subRow->{'sub_rows_visible'} = "true";
              $subRow2->{'visible'} = "true";
            }
            else
            {
              $subRow->{'sub_rows_visible'} = "false";
              $subRow2->{'visible'} = "false";
            }
            $subRow2->{'display_level'} = $temp_row{'display_level'} + 2;
            if ($temp_row{'display_level'} + 2 > $max_display_level)
            {
              $max_display_level = $temp_row{'display_level'} + 2;
            }
            $subRow2->{'sub_levels'} = $temp_row{'sub_levels'};
            if (not defined $subRow2->{'values'})
            {
              $subRow2->{'values'} = ();
            }
            if (not defined $subRow2->{'sql_clause'})
            {
              $subRow2->{'sql_clause'} = ();
            }
            $subNumEntries = scalar @{$temp_hash->{dates}};
            for (my $date_count=0; $date_count < $subNumEntries; $date_count++)
            {
              if (not defined $subRow2->{'values'}[$date_count])
              {
                $subRow2->{'values'}[$date_count] = ();
                for (my $fill_count=0; $fill_count < scalar @column_data; $fill_count++)
                {
                  if (not defined $subRow2->{'values'}[$date_count][$fill_count])
                  {
                    $subRow2->{'values'}[$date_count][$fill_count] = "0.00";
                  }
                }
              }
              if (not defined $subRow2->{'sql_clause'}[$date_count])
              {
                $subRow2->{'sql_clause'}[$date_count] = ();
                for (my $fill_count=0; $fill_count < scalar @column_data; $fill_count++)
                {
                  if (not defined $subRow2->{'sql_clause'}[$date_count][$fill_count])
                  {
                    $subRow2->{'sql_clause'}[$date_count][$fill_count] = "invalid";
                  }
                }
              }
            }

            if (not defined $subRow2->{'sub_rows'})
            {
              $subRow2->{'sub_rows'} = {};
            }
            if (not defined $subRow2->{'sub_rows'}{$account_num})
            {
              $subRow2->{'sub_rows'}{$account_num} = {};
            }
            my $subRow3 = $subRow2->{sub_rows}{$account_num};
            $subRow3->{'cell_text'} = $account_name;
            $subRow3->{'item_name'} = $account_num;
            $subRow3->{'initial_state'} = $temp_row{'initial_state'};
            if ($temp_row{'initial_state'} eq "o")
            {
              $subRow2->{'sub_rows_visible'} = "true";
              $subRow3->{'visible'} = "true";
            }
            else
            {
              $subRow2->{'sub_rows_visible'} = "false";
              $subRow3->{'visible'} = "false";
            }
            $subRow3->{'display_level'} = $temp_row{'display_level'} + 3;
            if ($temp_row{'display_level'} + 3 > $max_display_level)
            {
              $max_display_level = $temp_row{'display_level'} + 3;
            }
            $subRow3->{'sub_levels'} = $temp_row{'sub_levels'} - 1;
            if (not defined $subRow3->{'values'})
            {
              $subRow3->{'values'} = ();
            }
            if (not defined $subRow3->{'sql_clause'})
            {
              $subRow3->{'sql_clause'} = ();
            }
            for (my $date_count=0; $date_count < scalar @{$temp_hash->{'dates'}}; $date_count++)
            {
              if (not defined $subRow3->{'values'}[$date_count])
              {
                $subRow3->{'values'}[$date_count] = ();
                for (my $fill_count=0; $fill_count < scalar @column_data; $fill_count++)
                {
                  if (not defined $subRow3->{'values'}[$date_count][$fill_count])
                  {
                    $subRow3->{'values'}[$date_count][$fill_count] = "0.00";
                  }
                }
              }
              if (not defined $subRow3->{'sql_clause'}[$date_count])
              {
                $subRow3->{'sql_clause'}[$date_count] = ();
                for (my $fill_count=0; $fill_count < scalar @column_data; $fill_count++)
                {
                  if (not defined $subRow3->{'sql_clause'}[$date_count][$fill_count])
                  {
                    $subRow3->{'sql_clause'}[$date_count][$fill_count] = "invalid";
                  }
                }
              }
            }

            if ($error_string == 1)
            {
              $temp_hash->{'error'} = $error_string;
            }
            $subRow3->{'error'} = $error_string;
            my $tmp_sql_clause = $sub_level_sql_clause;
            $tmp_sql_clause =~ s/$accountRE/$account_num/g;
            $tmp_sql_clause =~ s/$subAccountRE/$sub_account_num/g;
            $subRow3->{'sql_clause'}[$count][$column_count] = $tmp_sql_clause;
            $subRow3->{'values'}[$count][$column_count] = $value;
            $subRow3->{'total'} += $subRow3->{'values'}[$count][$column_count];
            $subRow2->{'error'} = $error_string;
            $tmp_sql_clause = $sql_clause;
            $tmp_sql_clause =~ s/$subAccountRE/$sub_account_num/g;
            $subRow2->{'sql_clause'}[$count][$column_count] = $tmp_sql_clause;
            $subRow2->{'values'}[$count][$column_count] += $subRow3->{'values'}[$count][$column_count];
            $subRow2->{'total'} += $subRow3->{'values'}[$count][$column_count];
            $subRow->{'error'} = $error_string;
            $tmp_sql_clause = $level_sql_clause;
            $tmp_sql_clause =~ s/$subAccountTypeRE/$sub_account_type_num/g;
            $subRow->{'sql_clause'}[$count][$column_count] = $tmp_sql_clause;
            $subRow->{'values'}[$count][$column_count] += $subRow3->{'values'}[$count][$column_count];
            $subRow->{'total'} += $subRow3->{'values'}[$count][$column_count];
            $temp_sub_account_type_total[$column_count] += $subRow3->{'values'}[$count][$column_count];
          }
        }
      }
    }

    $temp_hash->{'values'}[$count] = ();
    $temp_hash->{'sql_clause'}[$count] = ();
    if ($column_type eq "nothing")
    {
      $temp_hash->{'values'}[$count][0] = $temp_sub_account_type_total[0];
      my $sqlClause = "";
      if ($temp_row{'activity'})
      {
        $sqlClause = "je.trans_date >= '$date_start' AND ";
      }
      $sqlClause .= "je.trans_date <= '$date_end'";
      $sqlClause .= " AND db.account_num IN (SELECT number FROM account_tb WHERE type = $temp_hash->{'item_name'}";
      if ($temp_row{'query_type'} == 1)
      {
        $sqlClause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = true)";
      }
      elsif ($temp_row{'query_type'} == 2)
      {
        $sqlClause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = false)";
      }
      $sqlClause .= ")";
      $temp_hash->{'sql_clause'}[$count][0] = $sqlClause;
      $temp_hash->{'total'} += $temp_hash->{'values'}[$count][0];
      $temp_hash->{'total'} = $temp_hash->{'total'};
    }
    else
    {
      for (my $column_count=0; $column_count < scalar @column_data; $column_count++)
      {
        if ($column_type eq "sub_entities")
        {
          $temp_hash->{'values'}[$count][$column_count] = $temp_sub_account_type_total[$column_count];
          my $sqlClause = "";
          if ($temp_row{'activity'})
          {
            $sqlClause = "je.trans_date >= '$date_start' AND ";
          }
          $sqlClause .= "je.trans_date <= '$date_end'";
          $sqlClause .= " AND db.account_num IN (SELECT number FROM account_tb WHERE type = $temp_hash->{'item_name'}";
          if ($temp_row{'query_type'} == 1)
          {
            $sqlClause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = true)";
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            $sqlClause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = false)";
          }
          $sqlClause .= ")";
          $sqlClause .= " AND db.sub_e_code = '$column_data[$column_count]'";
          $temp_hash->{'sql_clause'}[$count][$column_count] = $sqlClause;
        }
        elsif ($column_type eq "sub_accounts")
        {
          $temp_hash->{'values'}[$count][$column_count] = $temp_sub_account_type_total[$column_count];
          my $sqlClause = "";
          if ($temp_row{'activity'})
          {
            $sqlClause = "je.trans_date >= '$date_start' AND ";
          }
          $sqlClause .= "je.trans_date <= '$date_end'";
          $sqlClause .= " AND db.account_num IN (SELECT number FROM account_tb WHERE type = $temp_hash->{'item_name'}";
          if ($temp_row{'query_type'} == 1)
          {
            $sqlClause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = true)";
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            $sqlClause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = false)";
          }
          $sqlClause .= ")";
          $sqlClause .= " AND db.sub_account_num = $column_data[$column_count]";
          $temp_hash->{'sql_clause'}[$count][$column_count] = $sqlClause;
        }
        elsif ($column_type eq "sub_account_types")
        {
          $temp_hash->{'values'}[$count][$column_count] = $temp_sub_account_type_total[$column_count];
          my $sqlClause = "";
          if ($temp_row{'activity'})
          {
            $sqlClause = "je.trans_date >= '$date_start' AND ";
          }
          $sqlClause .= "je.trans_date <= '$date_end'";
          $sqlClause .= " AND db.account_num IN (SELECT number FROM account_tb WHERE type = $temp_hash->{'item_name'}";
          if ($temp_row{'query_type'} == 1)
          {
            $sqlClause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = true)";
          }
          elsif ($temp_row{'query_type'} == 2)
          {
            $sqlClause .= " AND number IN (SELECT acct_num FROM local_accounts_tb where ledger_code = '$args{'ledger_code'}' AND acct_num = account_tb.number AND taxable = false)";
          }
          $sqlClause .= ")";
          $sqlClause .= " AND db.sub_account_num IN (SELECT sub_acc_number FROM sub_acc_tb WHERE type = $column_data[$column_count])";
          $temp_hash->{'sql_clause'}[$count][$column_count] = $sqlClause;
        }
        $temp_hash->{'total'} += $temp_hash->{'values'}[$count][$column_count];
      }
    }
  }

  return $max_display_level;
}

sub get_sub_account_data
{
  my %arguments = ( @_, );
  my %args = %{ $arguments{'args'} };
  my $ledger_code = $args{'ledger_code'};
  my $temp_hash = $arguments{'temp_hash'};
  my %temp_row = %{ $arguments{'temp_row'} };
  my @local_accounts = ();
  my $ledger = $args{'ledger'};
  my @column_data = split /,/, $args{'column_data'};
  my $column_type = $args{'column_type'};
  my $max_display_level = -1;
  my $data_precision = $args{'data_precision'};

  if ($column_type eq "nothing")
  {
    @column_data = ( "0" );
  }
  elsif ($column_type eq "sub_entities")
  {
    for (my $count=0; $count < scalar @column_data; $count++)
    {
      # search for "<non_sub_entity>" and replace with ""
      if ($column_data[$count] eq "<non_sub_entity>")
      {
        $column_data[$count] = "";
      }
    }
  }

  # get the name of the sub-account
  $temp_hash->{'cell_text'} = GetSubAccountName(dbh => $args{'dbh'}, sub_acct_number => $temp_hash->{'item_name'});
  # get the accounts this sub-account was used with
  @local_accounts = GetAccountsForSubAccount(dbh => $args{'dbh'}, sub_account => $temp_row{'item_name'}, ledger_code => $ledger_code, taxable => $args{'query_type'});

  $temp_hash->{'sub_rows'} = ();
  $temp_hash->{'values'} = ();
  $temp_hash->{'sql_clause'} = ();
  # run over the dates and gather the data for the dates
  for (my $count=0; $count < scalar @{$temp_hash->{'dates'}}; $count++)
  {
    my ($date_start, $date_end);
    if ($args{'date_range'} && $temp_row{'cell_type'} ne "begin_sub_account_balance" && $temp_row{'cell_type'} ne "end_sub_account_balance")
    {
      $date_start = $temp_hash->{'dates'}[$count][0];
      $date_end = $temp_hash->{'dates'}[$count][1];
    }
    else
    {
      if ($temp_row{'cell_type'} eq "begin_sub_account_balance")
      {
        (my $temp_date = $temp_hash->{'dates'}[$count][0]) =~ s/$minusRE/\//g;
        # make date_start and date_end one less day
        $date_start = `/bin/date -d "$temp_date 1 day ago" +%m-%d-%Y`;
        $date_end = $date_start;
      }
      elsif ($temp_row{'cell_type'} eq "end_sub_account_balance")
      {
        $date_end = $temp_hash->{'dates'}[$count][1];
        $date_start = $date_end;
      }
      else
      {
        $date_start = $temp_hash->{'dates'}[$count][0];
        $date_end = $date_start;
      }
    }

    if ($temp_row{'display_level'} > $max_display_level)
    {
      $max_display_level = $temp_row{'display_level'};
    }

    if ($temp_row{'sub_levels'} == 0 || $column_type eq "sub_accounts" || $column_type eq "sub_account_types")
    {
      $temp_hash->{'values'}[$count] = ();
      $temp_hash->{'sql_clause'}[$count] = ();
      if ($column_type eq "nothing")
      {
        my %options = ();
        $options{'begin_date'} = $date_start if ($temp_row{'activity'});
        $options{'end_date'} = $date_end;
        $options{'balance'} = 1 if (!$temp_row{'activity'});
        $options{'taxable'} = $temp_row{'query_type'};

        my @result;
        my @accounts = ();
        my @sub_accounts = ();

        $accounts[0] = ();
        for (my $count2=0; $count2 < scalar @local_accounts; $count2++)
        {
          $accounts[0][$count2] = [ @{$local_accounts[$count2]} ];
        }

        $sub_accounts[0] = ();
        $sub_accounts[0][0] = $temp_hash->{'item_name'};
        $sub_accounts[0][1] = $temp_hash->{'cell_text'};

        @result = SubAccountActivity(dbh => $args{'dbh'}, ledger_code => $args{'ledger_code'}, sub_accounts => \@sub_accounts, accounts => \@accounts, begin_date => $date_start, end_date => $date_end, %options);
        $temp_hash->{'values'}[$count][0] = $result[0][0];
        my $sqlClause = "";
        if ($temp_row{'activity'})
        {
          $sqlClause = "je.trans_date >= '$date_start' AND ";
        }
        $sqlClause .= "je.trans_date <= '$date_end'";
        $sqlClause .= " AND db.sub_account_num = $temp_hash->{'item_name'}";
        $temp_hash->{'sql_clause'}[$count][0] = $sqlClause;
        $temp_hash->{'total'} += $temp_hash->{'values'}[$count][0];
      }
      else
      {
        for (my $column_count=0; $column_count < scalar @column_data; $column_count++)
        {
          if ($column_type eq "sub_entities")
          {
            my %options = ();
            $options{'begin_date'} = $date_start if ($temp_row{'activity'});
            $options{'end_date'} = $date_end;
            $options{'balance'} = 1 if (!$temp_row{'activity'});
            $options{'taxable'} = $temp_row{'query_type'};

            my @result;
            my @accounts = ();
            my @sub_accounts = ();
            my @sub_entity = ();

            $accounts[0] = ();
            for (my $count2=0; $count2 < scalar @local_accounts; $count2++)
            {
              $accounts[0][$count2] = [ @{$local_accounts[$count2]} ];
            }

            $sub_accounts[0] = ();
            $sub_accounts[0][0] = $temp_hash->{'item_name'};
            $sub_accounts[0][1] = $temp_hash->{'cell_text'};

            $sub_entity[0] = $column_data[$column_count];
            $options{'sub_entity'} = \@sub_entity;

            @result = SubAccountActivity(dbh => $args{'dbh'}, ledger_code => $args{'ledger_code'}, sub_accounts => \@sub_accounts, accounts => \@accounts, begin_date => $date_start, end_date => $date_end, %options);
            $temp_hash->{'values'}[$count][$column_count] = $result[0][0];
            my $sqlClause = "";
            if ($temp_row{'activity'})
            {
              $sqlClause = "je.trans_date >= '$date_start' AND ";
            }
            $sqlClause .= "je.trans_date <= '$date_end'";
            $sqlClause .= " AND db.sub_account_num = $temp_hash->{'item_name'}";
            $sqlClause .= " AND db.sub_e_code = '$column_data[$column_count]'";
            $temp_hash->{'sql_clause'}[$count][$column_count] = $sqlClause;
          }
          elsif ($column_type eq "sub_accounts")
          { # invalid row x column
            $temp_hash->{'values'}[$count][$column_count] = 0;
            $temp_hash->{'sql_clause'}[$count][$column_count] = "invalid";
          }
          elsif ($column_type eq "sub_account_types")
          { # invalid row x column
            $temp_hash->{'values'}[$count][$column_count] = 0;
            $temp_hash->{'sql_clause'}[$count][$column_count] = "invalid";
          }
          $temp_hash->{'total'} += $temp_hash->{'values'}[$count][$column_count];
        }
      }
    }
    elsif ($temp_row{'sub_levels'} > 0)
    {
      my @temp_account_total;
      my $dbh = $args{'dbh'};
      my $sth;
      my $rc;

      my $tmp_sql_statement = "";
      my @sql_arguments = ();
      my $sql_clause;
      my $error_string = 0;
      # calculate the value (activity/balance) for the sub-account for each date range given the column qualifiers
      if ($column_type ne "sub_accounts" && $column_type ne "sub_account_types")
      {
        for (my $column_count=0; $column_count < scalar @column_data; $column_count++)
        {
          if ($column_type eq "nothing")
          {
            if ($temp_row{'query_type'} == 0)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1095};
                @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $date_end, $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1099};
                @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $date_end, $temp_row{'item_name'}, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $date_end);
              }
            }
            elsif ($temp_row{'query_type'} == 1)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1096};
                @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $date_end, "true", $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "true", $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1100};
                @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $ledger_code, "true", $date_end, $temp_row{'item_name'}, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $ledger_code, "true", $date_end);
              }
            }
            elsif ($temp_row{'query_type'} == 2)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1096};
                @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $date_end, "false", $temp_hash->{'report_class'}, $ledger_code, $temp_row{'item_name'}, $ledger_code, "false", $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1100};
                @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $ledger_code, "false", $date_end, $temp_row{'item_name'}, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $ledger_code, "false", $date_end);
              }
            }

            $sql_clause = "";
            if ($temp_row{'activity'})
            {
              $sql_clause = "je.trans_date >= '$date_start' AND ";
            }
            $sql_clause .= "je.trans_date <= '$date_end'";
            $sql_clause .= " AND db.account_num = <ACCOUNT>";
            $sql_clause .= " AND db.sub_account_num = $temp_hash->{'item_name'}";
          }
          elsif ($column_type eq "sub_entities")
          {
            if ($temp_row{'query_type'} == 0)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1097};
                @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $date_end, $column_data[$column_count], $temp_hash->{'report_class'}, $ledger_code, $column_data[$column_count], $temp_row{'item_name'}, $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1099};
                @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $date_end, $temp_row{'item_name'}, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $date_end);
                $error_string = 1;
              }
            }
            elsif ($temp_row{'query_type'} == 1)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1098};
                @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $date_end, "true", $column_data[$column_count], $temp_hash->{'report_class'}, $ledger_code, $column_data[$column_count], $temp_row{'item_name'}, $ledger_code, "true", $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1100};
                @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $ledger_code, "true", $date_end, $temp_row{'item_name'}, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $ledger_code, "true", $date_end);
                $error_string = 1;
              }
            }
            elsif ($temp_row{'query_type'} == 2)
            {
              if ($temp_row{'activity'})
              {
                $tmp_sql_statement = $sql_statements{1098};
                @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $date_end, "false", $column_data[$column_count], $temp_hash->{'report_class'}, $ledger_code, $column_data[$column_count], $temp_row{'item_name'}, $ledger_code, "false", $date_start, $date_end);
              }
              else
              {
                $tmp_sql_statement = $sql_statements{1100};
                @sql_arguments = ($temp_row{'item_name'}, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $ledger_code, "false", $date_end, $temp_row{'item_name'}, $ledger_code, $date_start, $temp_hash->{'report_class'}, $ledger_code, $ledger_code, "false", $date_end);
                $error_string = 1;
              }
            }

            $sql_clause = "";
            if ($temp_row{'activity'})
            {
              $sql_clause = "je.trans_date >= '$date_start' AND ";
            }
            $sql_clause .= "je.trans_date <= '$date_end'";
            $sql_clause .= " AND db.account_num = <ACCOUNT>";
            $sql_clause .= " AND db.sub_account_num = $temp_hash->{'item_name'}";
            $sql_clause .= " AND db.sub_e_code = '$column_data[$column_count]'";
          }
          $sth = $dbh->prepare( $tmp_sql_statement ) || my_db_die("Error: Report.cgi - get_sub_account_data", 4007, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);
          $rc = $sth->execute(@sql_arguments) || my_db_die("Error: Report.cgi - get_sub_account_data", 4008, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);
          while (my $row_array = $sth->fetchrow_arrayref())
          {
            my $account_num = $row_array->[0];
            my $account_name = $row_array->[1];
            my $value = $row_array->[2];

            if ($account_num ne "")
            {
              if (not defined $temp_hash->{'sub_rows'})
              {
                $temp_hash->{'sub_rows'} = {};
              }

              if (not defined $temp_hash->{'sub_rows'}{$account_num})
              {
                $temp_hash->{'sub_rows'}{$account_num} = {};
              }
              my $subRow = $temp_hash->{sub_rows}{$account_num};
              $subRow->{'item_name'} = $account_num;
              $subRow->{'cell_text'} = $account_name;
              $subRow->{'initial_state'} = $temp_row{'initial_state'};
              if ($temp_row{'initial_state'} eq "o")
              {
                $temp_hash->{'sub_rows_visible'} = "true";
                $subRow->{'visible'} = "true";
              }
              else
              {
                $temp_hash->{'sub_rows_visible'} = "false";
                $subRow->{'visible'} = "false";
              }
              $subRow->{'display_level'} = $temp_row{'display_level'} + 1;
              if ($temp_row{'display_level'} + 1 > $max_display_level)
              {
                $max_display_level = $temp_row{'display_level'} + 1;
              }
              $subRow->{'sub_levels'} = $temp_row{'sub_levels'} - 1;
              if (not defined $subRow->{'values'})
              {
                $subRow->{'values'} = ();
              }
              if (not defined $subRow->{'sql_clause'})
              {
                $subRow->{'sql_clause'} = ();
              }
              for (my $date_count=0; $date_count < scalar @{$temp_hash->{'dates'}}; $date_count++)
              {
                if (not defined $subRow->{'values'}[$date_count])
                {
                  $subRow->{'values'}[$date_count] = ();
                  for (my $fill_count=0; $fill_count < scalar @column_data; $fill_count++)
                  {
                    if (not defined $subRow->{'values'}[$date_count][$fill_count])
                    {
                      $subRow->{'values'}[$date_count][$fill_count] = "0.00";
                    }
                  }
                }
                if (not defined $subRow->{'sql_clause'}[$date_count])
                {
                  $subRow->{'sql_clause'}[$date_count] = ();
                  for (my $fill_count=0; $fill_count < scalar @column_data; $fill_count++)
                  {
                    if (not defined $subRow->{'sql_clause'}[$date_count][$fill_count])
                    {
                      $subRow->{'sql_clause'}[$date_count][$fill_count] = "invalid";
                    }
                  }
                }
              }

              if ($error_string == 1)
              {
                $temp_hash->{'error'} = $error_string;
              }
              $subRow->{'error'} = $error_string;
              my $tmp_sql_clause = $sql_clause;
              $tmp_sql_clause =~ s/$accountRE/$account_num/g;
              $subRow->{'sql_clause'}[$count][$column_count] = $tmp_sql_clause;
              $subRow->{'values'}[$count][$column_count] = $value;
              $subRow->{'total'} += $subRow->{'values'}[$count][$column_count];
              $temp_account_total[$column_count] += $subRow->{'values'}[$count][$column_count];
            }
          }
        }
      }

      $temp_hash->{'values'}[$count] = ();
      $temp_hash->{'sql_clause'}[$count] = ();
      if ($column_type eq "nothing")
      {
        $temp_hash->{'values'}[$count][0] = $temp_account_total[0];
        my $sqlClause = "";
        if ($temp_row{'activity'})
        {
          $sqlClause = "je.trans_date >= '$date_start' AND ";
        }
        $sqlClause .= "je.trans_date <= '$date_end'";
        $sqlClause .= " AND db.sub_account_num = $temp_hash->{'item_name'}";
        $temp_hash->{'sql_clause'}[$count][0] = $sqlClause;
        $temp_hash->{'total'} += $temp_hash->{'values'}[$count][0];
      }
      else
      {
        for (my $column_count=0; $column_count < scalar @column_data; $column_count++)
        {
          if ($column_type eq "sub_entities")
          {
            $temp_hash->{'values'}[$count][$column_count] = $temp_account_total[$column_count];
            my $sqlClause = "";
            if ($temp_row{'activity'})
            {
              $sqlClause = "je.trans_date >= '$date_start' AND ";
            }
            $sqlClause .= "je.trans_date <= '$date_end'";
            $sqlClause .= " AND db.sub_account_num = $temp_hash->{'item_name'}";
            $sqlClause .= " AND db.sub_e_code = '$column_data[$column_count]'";
            $temp_hash->{'sql_clause'}[$count][$column_count] = $sqlClause;
          }
          elsif ($column_type eq "sub_accounts")
          { # invalid row x column
            $temp_hash->{'values'}[$count][$column_count] = $temp_account_total[$column_count];
            $temp_hash->{'sql_clause'}[$count][$column_count] = "invalid";
          }
          elsif ($column_type eq "sub_account_types")
          { # invalid row x column
            $temp_hash->{'values'}[$count][$column_count] = $temp_account_total[$column_count];
            $temp_hash->{'sql_clause'}[$count][$column_count] = "invalid";
          }
          $temp_hash->{'total'} += $temp_hash->{'values'}[$count][$column_count];
        }
      }
    }
  }

  return $max_display_level;
}

# parse_formula - validates the specified formula and returns the tokens that make up the formula.
# Takes  : formula string
# Returns: Error string and token array.
sub parse_formula
{
  my %args = ( @_, );
  my $formula = $args{'formula'};
  my $result = "";

  if (length $formula == 0)
  {
    my_die("Error:  Report.cgi - parse_formula", 4035, "formula must be specified!");
  }

  my @elements = ($formula =~ m/(\(|R\d+\.\d+|R\d+|-?\d+\.\d+|-?\d+|\+|-|%|\*|\/|\)|\s+|.*)/g);
  my $i;

  # get rid of any empty or whitespace related elements.
  my @nodesToDelete;
  for ($i=0; $i < @elements; $i++)
  {
    if ($elements[$i] =~ /^(\s+|)$/)
    {
      push @nodesToDelete, $i;
    }
  }
  for ($i=0; $i < @nodesToDelete; $i++)
  {
    splice(@elements, $nodesToDelete[$i]-$i, 1);
  }

  if (scalar @elements == 0)
  {
    $result = "Error:  The formula didn't split out any tokens!\n";
    return ($result, @elements);
  }

  my $open_parens = 0;
  for ($i=0; $i < scalar @elements; $i++)
  {
    if ($i == 0) # first element only.
    { # make sure we are not starting out with an operator
      if ($elements[$i] =~ /$operatorRE/)
      {
        $result = "Error:  Formula can not start with an operator!\n";
        return ($result, @elements);
      }
      elsif ($elements[$i] =~ /$openParenRE/)
      {
        $open_parens++;
      }
      elsif ($elements[$i] =~ /$closeParenRE/)
      {
        $result = "Error:  You can not start the Formula with an )!\n";
        return ($result, @elements);
      }
      elsif ($elements[$i] !~ /$tokenRE/)
      {
        $result = "Error:  Invalid token!  Token $i, '$elements[$i]'.\n";
        return ($result, @elements);
      }
    }
    else  # the rest of the elements.
    { # make sure that the last token was a row, number, or ) if we have an operator
      # else if we have a row make sure the last token was an (
      # and not another row.
      if ($elements[$i] =~ /$operatorRE/)
      {
        if ($elements[$i-1] !~ /$tokenRE/ && $elements[$i-1] !~ /$closeParenRE/)
        {
          $result = "Error:  Operator can not come after another Operator or (!  Token $i, '$elements[$i-1]' and '$elements[$i]'.\n";
          return ($result, @elements);
        }
      }
      elsif ($elements[$i] =~ /$openParenRE/)
      {
        $open_parens++;
        # make sure that the previous token was an ( or an operator.
        if ($elements[$i-1] !~ /$openParenRE/ && $elements[$i-1] !~ /$operatorRE/)
        {
          $result = "Error:  ( can not come after a Row, Number, or )!  Token $i, '$elements[$i-1]' and '$elements[$i]'.\n";
          return ($result, @elements);
        }
      }
      elsif ($elements[$i] =~ /$closeParenRE/)
      {
        if ($open_parens > 0)
        {
          $open_parens--;
        }
        else
        {
          $result = "Error:  Formula is invalid at token $i, '$elements[$i]'.  Close Paren when no Open Paren to match with!\n";
          return ($result, @elements);
        }

        # make sure that we are not following a ( or an operator.
        if ($elements[$i-1] =~ /$openParenRE/ || $elements[$i-1] =~ /$operatorRE/)
        {
          $result = "Error:  ) can not come after a ( or operator!  Token $i, '$elements[$i-1]' and '$elements[$i]'.\n";
          return ($result, @elements);
        }
      }
      elsif ($elements[$i] =~ /$rowIdentifierRE/ && $elements[$i-1] =~ /^-?\d+\.\d+|-?\d+$/)
      {
        $result = "Error:  You can not have a Number and a Row back to back without an intervening operator! Token $i, '$elements[$i-1]' and '$elements[$i]'.\n";
        return ($result, @elements);
      }
      elsif ($elements[$i] =~ /^-\d+\.\d+|-\d+$/ && $elements[$i-1] =~ /$tokenRE/)
      {
        $result = "Error:  You can not have Two Numbers or a Row and a Number back to back without an intervening operator!  Token $i, '$elements[$i-1]' and '$elements[$i]'.\n";
        return ($result, @elements);
      }
      elsif ($elements[$i] =~ /$tokenRE/ && $elements[$i-1] !~ /$rowIdentifierRE/)
      {
        if ($elements[$i] =~ /^R\d+\.\d+$/)
        {
          $result = "Error:  Invalid token!  Token $i, '$elements[$i]'.\n";
          return ($result, @elements);
        }
      }
      elsif ($elements[$i] =~ /$rowIdentifierRE/ && $elements[$i-1] =~ /$rowIdentifierRE/)
      {
        $result = "Error:  You can not have 2 Rows back to back without an intervening operator! Token $i, '$elements[$i-1]' and '$elements[$i]'.\n";
        return ($result, @elements);
      }
      else
      {
        $result = "Error:  Invalid token!  Token $i, '$elements[$i]'.\n";
        return ($result, @elements);
      }
    }
  }

  # make sure the last token wasn't an operator.
  if ($elements[$i-1] =~ /$operatorRE/)
  {
    $result = "Error:  The Formula can not end with an operator!  Token ${\($i-1)}, '$elements[${\($i-1)}]'.\n";
    return ($result, @elements);
  }

  # make sure we have all ()'s matched up.
  if ($open_parens > 0)
  {
    $result = "Error:  You have unmatched ('s!\n";
    return ($result, @elements);
  }

  return ($result, [ @elements ]);  # signal it passed.
}

sub get_total_data
{
  my %arguments = ( @_, );
  my %args = %{ $arguments{'args'} };
  my $dbh = $args{'dbh'};
  my $temp_hash = $arguments{'temp_hash'};
  my %temp_row = %{ $arguments{'temp_row'} };
  my @column_data = split /,/, $args{'column_data'};
  my $column_type = $args{'column_type'};
  my $data_precision = $args{'data_precision'};
  my @report_data = @{ $arguments{'report_data'} };
  my $formula = $temp_row{'cell_formula'};
  my @formula_tokens = parse_formula(formula => $formula);
  my @original_tokens = RetrieveSubArray(array => \@formula_tokens, index => 1);

  if ($column_type eq "sub_entities")
  {
    for (my $count=0; $count < scalar @column_data; $count++)
    {
      # search for "<non_sub_entity>" and replace with ""
      if ($column_data[$count] eq "<non_sub_entity>")
      {
        $column_data[$count] = "";
      }
    }
  }

  if (length $formula_tokens[0] > 0) # error happened parsing the formula
  {
    $temp_hash->{'cell_text'} .= "<br>(<b>Invalid row formula!</b>)";
    $temp_hash->{'values'} = ();
    for (my $count=0; $count < scalar @{$temp_hash->{'dates'}}; $count++)
    {
      $temp_hash->{'values'}[$count] = ();

      if ($column_type eq "nothing")
      {
        $temp_hash->{'values'}[$count][0] = 0;
        $temp_hash->{'total'} += $temp_hash->{'values'}[$count][0];
      }
      else
      {
        for (my $column_count=0; $column_count < scalar @column_data; $column_count++)
        {
          $temp_hash->{'values'}[$count][$column_count] = 0;
          $temp_hash->{'total'} += $temp_hash->{'values'}[$count][$column_count];
        }
      }
    }
  }
  else
  {
    $temp_hash->{'values'} = ();
    # run over the dates and calculate the formula of the data for each column
    for (my $count=0; $count < scalar @{$temp_hash->{'dates'}}; $count++)
    {
      $temp_hash->{'values'}[$count] = ();

      if ($column_type eq "nothing")
      {
        my $result;
        my $temp_formula = "\$result = ";
        for (my $token_count=0; $token_count < scalar @original_tokens; $token_count++)
        {
          if ($original_tokens[$token_count] =~ /$rowIdentifier2RE/)
          {
            # replace the row value with the value of that row
            # added parentheses around the value in case it is negative
            $temp_formula .= "(" . $report_data[$1]{'values'}[$count][0] . ")";
          }
          else
          {
            $temp_formula .= $original_tokens[$token_count];
          }
        }
        eval($temp_formula);
        if ($@)
        {
          my_die("Error evaluating formula - get_total_data", 4037, "Formula string: '$formula'<br>Row: $temp_row{'index'}<br>Error message: '$@'");
        }
        $temp_hash->{'values'}[$count][0] = $result;
        $temp_hash->{'total'} += $temp_hash->{'values'}[$count][0];
      }
      else
      {
        for (my $column_count=0; $column_count < scalar @column_data; $column_count++)
        {
          my $result;
          my $temp_formula = "\$result = ";
          for (my $token_count=0; $token_count < scalar @original_tokens; $token_count++)
          {
            if ($original_tokens[$token_count] =~ /$rowIdentifier2RE/)
            {
              # replace the row value with the value of that row
              # added parentheses around the value in case it is negative
              $temp_formula .= "(" . $report_data[$1]{'values'}[$count][$column_count] . ")";
            }
            else
            {
              $temp_formula .= $original_tokens[$token_count];
            }
          }
          eval($temp_formula);
          if ($@)
          {
            my_die("Error evaluating formula - get_total_data", 4037, "Formula string: '$formula'<br>Row: $temp_row{'index'}<br>Error message: '$@'");
          }
          $temp_hash->{'values'}[$count][$column_count] = $result;
          $temp_hash->{'total'} += $temp_hash->{'values'}[$count][$column_count];
        }
      }
    }
  }
}

sub outputDebug
{
  my $output = shift;
  open (FILE, ">>/tmp/report_customization.log") or die "Can't open log file!  $!\n";

  print FILE $output . "\n";
  close(FILE);
}

sub get_comparison_data
{
  my %arguments = ( @_, );
  my %args = %{ $arguments{'args'} };
  my $dbh = $args{'dbh'};
  my $temp_hash = $arguments{'temp_hash'};
  my %temp_row = %{ $arguments{'temp_row'} };
  my @column_data = split /,/, $args{'column_data'};
  my $column_type = $args{'column_type'};
  my $data_precision = $args{'data_precision'};
  my @report_data = @{ $arguments{'report_data'} };
  (my $formula = $temp_row{'cell_formula'}) =~ m/^\(\s*(.+)\s+(==|!=|<=|>=|<|>)\s+(.+)\s+\?\s*"(.*)"\s*:\s*"(.*)"\s*\)$/;
  my $formula1 = $1;
  my $formula2 = $3;
  my $operator = $2;
  my $result_true = $4;
  my $result_false = $5;
  my @formula1_tokens = parse_formula(formula => $formula1);
  my @formula2_tokens = parse_formula(formula => $formula2);
  my @original1_tokens = RetrieveSubArray(array => \@formula1_tokens, index => 1);
  my @original2_tokens = RetrieveSubArray(array => \@formula2_tokens, index => 1);
  my $result_formula = ($temp_row{'result_type'} eq "result_formula" ? 1 : 0);

  my @formula3_tokens = ();
  my @formula4_tokens = ();
  my @original3_tokens = ();
  my @original4_tokens = ();

  if ($result_formula)
  {
    @formula3_tokens = parse_formula(formula => $result_true);
    @formula4_tokens = parse_formula(formula => $result_false);
    @original3_tokens = RetrieveSubArray(array => \@formula3_tokens, index => 1);
    @original4_tokens = RetrieveSubArray(array => \@formula4_tokens, index => 1);
  }

  # support the Comparison Row as a Formula Result ----------------

  if ($column_type eq "sub_entities")
  {
    for (my $count=0; $count < scalar @column_data; $count++)
    {
      # search for "<non_sub_entity>" and replace with ""
      if ($column_data[$count] eq "<non_sub_entity>")
      {
        $column_data[$count] = "";
      }
    }
  }

  $temp_hash->{'values'} = ();
  if (length $formula1_tokens[0] > 0 || length $formula2_tokens[0] > 0 || ($result_formula && (length $formula3_tokens[0] > 0 || length $formula4_tokens[0] > 0))) # error happened parsing the formula(s)
  {
    $temp_hash->{'cell_text'} .= "<br>(<b>Invalid row formula!</b>)";
    for (my $count=0; $count < scalar @{$temp_hash->{'dates'}}; $count++)
    {
      $temp_hash->{'values'}[$count] = ();

      if ($column_type eq "nothing")
      {
        $temp_hash->{'values'}[$count][0] = 0;
        $temp_hash->{'total'} += $temp_hash->{'values'}[$count][0];
      }
      else
      {
        for (my $column_count=0; $column_count < scalar @column_data; $column_count++)
        {
          $temp_hash->{'values'}[$count][$column_count] = 0;
          $temp_hash->{'total'} += $temp_hash->{'values'}[$count][$column_count];
        }
      }
    }
  }
  else
  {
    # for each date calculate the formula of the data for each column
    for (my $count=0; $count < scalar @{$temp_hash->{'dates'}}; $count++)
    {
      my $result_true_string = $result_true;
      my $result_false_string = $result_false;

      if ($result_formula)
      {
        $result_true_string = $result_false_string = "(";
      }

      $temp_hash->{'values'}[$count] = ();

      if ($column_type eq "nothing")
      {
        my $result = "";
        my $temp_formula = "\$result = ((sprintf(\"\%f\", ";
        for (my $token_count=0; $token_count < scalar @original1_tokens; $token_count++)
        {
          if ($original1_tokens[$token_count] =~ /$rowIdentifier2RE/)
          {
            # replace the row value with the value of that row
            # added parentheses around the value in case it is negative
            $temp_formula .= "(" . $report_data[$1]{'values'}[$count][0] . ")";
          }
          else
          {
            $temp_formula .= $original1_tokens[$token_count];
          }
        }
        $temp_formula .= ") ) " . $operator . " (sprintf(\"\%f\", ";
        for (my $token_count=0; $token_count < scalar @original2_tokens; $token_count++)
        {
          if ($original2_tokens[$token_count] =~ /$rowIdentifier2RE/)
          {
            # replace the row value with the value of that row
            # added parentheses around the value in case it is negative
            $temp_formula .= "(" . $report_data[$1]{'values'}[$count][0] . ")";
          }
          else
          {
            $temp_formula .= $original2_tokens[$token_count];
          }
        }
        if ($result_formula)
        {
          for (my $token_count=0; $token_count < scalar @original3_tokens; $token_count++)
          {
            if ($original3_tokens[$token_count] =~ /$rowIdentifier2RE/)
            {
              # replace the row value with the value of that row
              # added parentheses around the value in case it is negative
              $result_true_string .= "(" . $report_data[$1]{'values'}[$count][0] . ")";
            }
            else
            {
              $result_true_string .= $original3_tokens[$token_count];
            }

          }
          for (my $token_count=0; $token_count < scalar @original4_tokens; $token_count++)
          {
            if ($original4_tokens[$token_count] =~ /$rowIdentifier2RE/)
            {
              # replace the row value with the value of that row
              # added parentheses around the value in case it is negative
              $result_false_string .= "(" . $report_data[$1]{'values'}[$count][0] . ")";
            }
            else
            {
              $result_false_string .= $original4_tokens[$token_count];
            }
          }
          $result_true_string .= ")";
          $result_false_string .= ")";
          $temp_formula .= ") ) ? $result_true_string : $result_false_string)";
        }
        else
        {
          $temp_formula .= ") ) ? \"" . $result_true_string . "\" : \"" . $result_false_string . "\")";
        }

        eval($temp_formula);
        if ($@)
        {
          my_die("Error evaluating formula - get_comparison_data", 4037, "Formula string: '$formula'<br>Temp Formula: \"$temp_formula\"<br>Result True String = '$result_true_string'<br>Result False String = '$result_false_string'<br>Row: $temp_row{'index'}<br>Error message: '$@'");
        }
        $temp_hash->{'values'}[$count][0] = $result;
        $temp_hash->{'debug_formula'}[$count][0] = $temp_formula;
        if ($result_formula)
        {
          $temp_hash->{total} += $result;
        }
        else
        {
          $temp_hash->{'total'} = "&nbsp;";
        }
      }
      else
      {
        for (my $column_count=0; $column_count < scalar @column_data; $column_count++)
        {
          my $result = "";
          my $temp_formula = "\$result = ((sprintf(\"\%f\", ";
          my $result_true_string = $result_true;
          my $result_false_string = $result_false;

          if ($result_formula)
          {
            $result_true_string = $result_false_string = "(";
          }

          for (my $token_count=0; $token_count < scalar @original1_tokens; $token_count++)
          {
            if ($original1_tokens[$token_count] =~ /$rowIdentifier2RE/)
            {
              # replace the row value with the value of that row
              # added parentheses around the value in case it is negative
              $temp_formula .= "(" . $report_data[$1]{'values'}[$count][$column_count] . ")";
            }
            else
            {
              $temp_formula .= $original1_tokens[$token_count];
            }
          }
          $temp_formula .= ") ) " . $operator . " (";
          if ($result_formula)
          {
            $temp_formula .= "sprintf(\"\%f\", ";
          }
          for (my $token_count=0; $token_count < scalar @original2_tokens; $token_count++)
          {
            if ($original2_tokens[$token_count] =~ /$rowIdentifier2RE/)
            {
              # replace the row value with the value of that row
              # added parentheses around the value in case it is negative
              $temp_formula .= "(" . $report_data[$1]{'values'}[$count][$column_count] . ")";
            }
            else
            {
              $temp_formula .= $original2_tokens[$token_count];
            }
          }
          if ($result_formula)
          {
            for (my $token_count=0; $token_count < scalar @original3_tokens; $token_count++)
            {
              if ($original3_tokens[$token_count] =~ /$rowIdentifier2RE/)
              {
                # replace the row value with the value of that row
                # added parentheses around the value in case it is negative
                $result_true_string .= "(" . $report_data[$1]{'values'}[$count][$column_count] . ")";
              }
              else
              {
                $result_true_string .= $original3_tokens[$token_count];
              }
            }
            for (my $token_count=0; $token_count < scalar @original4_tokens; $token_count++)
            {
              if ($original4_tokens[$token_count] =~ /$rowIdentifier2RE/)
              {
                # replace the row value with the value of that row
                # added parentheses around the value in case it is negative
                $result_false_string .= "(" . $report_data[$1]{'values'}[$count][$column_count] . ")";
              }
              else
              {
                $result_false_string .= $original4_tokens[$token_count];
              }
            }
            $result_true_string .= ")";
            $result_false_string .= ")";
            $temp_formula .= ") ) ? $result_true_string : $result_false_string)";
          }
          else
          {
            $temp_formula .= ") ) ? \"" . $result_true_string . "\" : \"" . $result_false_string . "\")";
          }

          eval($temp_formula);
          if ($@)
          {
            my_die("Error evaluating formula - get_comparison_data", 4037, "Formula string: '$formula'<br>Temp Formula: \"$temp_formula\"<br>Row: $temp_row{'index'}<br>Error message: '$@'");
          }
          $temp_hash->{'values'}[$count][$column_count] = $result;
          $temp_hash->{'debug_formula'}[$count][$column_count] = $temp_formula;
          if ($result_formula)
          {
            $temp_hash->{total} += $result;
          }
          else
          {
            $temp_hash->{'total'} = "&nbsp;";
          }
        }
      }
    }
  }
}

sub get_detail_data
{
  my %arguments = ( @_, );
  my %args = %{ $arguments{'args'} };
  my $dbh = $args{'dbh'};
  my $temp_hash = $arguments{'temp_hash'};
  my %temp_row = %{ $arguments{'temp_row'} };
  my $where_clause = $temp_hash->{'sql_statement'};

  $temp_hash->{'values'} = ();
  # get the data for the rows in the report
  $temp_hash->{'values'} = [ GetActivityData(dbh => $dbh, ledger_code => $args{'ledger_code'}, sql_statement => $where_clause) ];
}

# This function gathers the values from the client report customization screen and stores them in the hash after validation.
sub get_report_attributes
{
  my %args = ( memorize => 0, @_, );
  my $dbh = $args{'dbh'};
  my $memorize = $args{'memorize'};
  my %attributes = ();
  my @arguments = ("id", "ledger", "report_type", "report_name", "report_class", "date_range", "max_columns", "min_columns", "fg_color", "bg_color", "default_fg_color", "default_bg_color", "underline_length", "data_precision", "negative_format", "text_size", "text_font", "data_size", "data_font", "account_format", "column_variable", "column_variable_data", "total_column", "display_dates", "column_data", "column_data_names", "column_type", "sql_statement", "headers", "sorted_by", "sorted_order", "subtotal", "subtotal_type", "dates_entered", "border", "windowName", "reportWidth", "displayZeroRows");
  my $error_string = "";

  eval ("use HTMLObject::Normal;");
  if ($@)
  {
    my_die("Error: Report.cgi - Eval", 4002, "Error: $@ at line " . __LINE__);
  }

  my $doc = HTMLObject::Normal->new();

  $doc->setJavascriptErrorHandlerEmail(email => $email_address);

  if (!exists $args{'dbh'})
  {
    $doc->doRequiredParameterError('get_report_attributes', "dbh");
  }

  LOOP: foreach my $name (@arguments)
  {
    next LOOP if ($name eq "sql_statement" && $attributes{'report_type'} eq "r");
    next LOOP if ($name eq "headers" && $attributes{'report_type'} eq "r");
    next LOOP if ($name eq "sorted_by" && $attributes{'report_type'} eq "r");
    next LOOP if ($name eq "sorted_order" && $attributes{'report_type'} eq "r");
    next LOOP if ($name eq "subtotal" && $attributes{'report_type'} eq "r");
    next LOOP if ($name eq "subtotal_type" && $attributes{'report_type'} eq "r");
    if ($memorize)
    {
      next LOOP if ($name eq "default_fg_color");
      next LOOP if ($name eq "default_bg_color");
      next LOOP if ($name eq "total_column" && $attributes{'report_type'} eq "d");
      next LOOP if ($name eq "underline_length" && $attributes{'report_type'} eq "d");
      next LOOP if ($name eq "dates_entered" && $attributes{'report_type'} eq "d");
      next LOOP if ($name eq "max_columns");
      next LOOP if ($name eq "min_columns");
      next LOOP if ($name eq "column_data_names");
      #next LOOP if ($name eq "column_variable");
      #next LOOP if ($name eq "column_variable_data");
      next LOOP if ($name eq "display_dates");
      next LOOP if ($name eq "date_range");
      next LOOP if ($name eq "border");
      next LOOP if ($name eq "windowName");
    }
    else
    {
      next LOOP if ($name eq "dates_entered");
      next LOOP if ($name eq "border" && $attributes{'report_type'} eq "d");
      next LOOP if ($name eq "displayZeroRows");
    }
    next LOOP if ($name eq "report_class" && $attributes{'report_type'} eq "d");
    next LOOP if ($name eq "date_range" && $attributes{'report_type'} eq "d");
    next LOOP if ($name eq "min_columns" && $attributes{'report_type'} eq "d");
    next LOOP if ($name eq "column_variable" && $attributes{'report_type'} eq "d");
    next LOOP if ($name eq "column_data" && $attributes{'report_type'} eq "d");
    next LOOP if ($name eq "column_variable_data" && $attributes{'report_type'} eq "d");
    next LOOP if ($name eq "max_columns" && $attributes{'report_type'} eq "d");
    next LOOP if ($name eq "display_dates" && $attributes{'report_type'} eq "d");
    next LOOP if ($name eq "column_data_names" && $attributes{'report_type'} eq "d");
    next LOOP if ($name eq "column_type" && $attributes{'report_type'} eq "d");
    if (!exists $input{$name})
    {
      if ($name eq "column_data" || $name eq "column_data_names" || $name eq "column_type")
      {
      }
      else
      {
        $doc->doRequiredParameterError('get_report_attributes', $name);
      }
    }
    if (!validate_customized_report_attribute(name => $name, value => $input{$name}, dbh => $dbh))
    {
      $error_string .= "<br>\n" if (length $error_string > 0);
      $error_string .= "'$name' = '$input{$name}' is invalid!";
      #my_die("Error:  Report.cgi - get_report_attributes", 4006, "Name = '$name', Value = '$input{$name}'");
    }
    $attributes{$name} = $input{$name};
    if ($name eq "ledger")
    { # get the ledger_name, ledger_code, and ledger_color values
      my $ledger_name = get_ledger_name( ledger_db => $attributes{'ledger'}, dbh => $dbh );
      my $ledger_code = get_ledger_code( ledger_name => $ledger_name, dbh => $dbh );
      my $ledger_color = get_ledger_color( ledger_code => $ledger_code, dbh => $dbh );

      $attributes{'ledger_name'} = $ledger_name;
      $attributes{'ledger_code'} = $ledger_code;
      $attributes{'ledger_color'} = $ledger_color;
    }
    if ($name eq "column_data")
    {
      $attributes{$name} = join(",", split(/\0/, $input{$name}));
    }
  }

  $attributes{'dbh'} = $dbh;
  my $report_type = $attributes{'report_type'};

  if ($report_type eq "r")
  {
    # validate the column_data
    my $column_type = $attributes{'column_type'};
    my $column_data = $attributes{'column_data'};
    my @column_variable = split /,/, $attributes{'column_variable'};
    my @column_variable_data = split /\|/, $attributes{'column_variable_data'};
    if ($column_type ne "nothing")
    {
      # first validate that the column type is valid for this report
      my $found = 0;
      my $i;
      for ($i=0; $i < scalar @column_variable && !$found; $i++)
      {
        if ($column_variable[$i] eq $column_type)
        {
          $found = 1;
        }
      }
      $i--;  # point to the correct index.
      if (!$found)
      {
        $error_string .= "<br>\n" if (length $error_string > 0);
        $error_string .= "Column Type = '$column_type' is invalid for this report!";
      }
      else
      {
        # now validate the values entered
        if (length $column_data > 0)
        {
          foreach my $tmp_value (split(/,/, $column_data))
          {
            $found = 0;
            my $j;
            my @tmp_column_variable_data = split /,/, $column_variable_data[$i];
            for ($j=0; $j < scalar @tmp_column_variable_data && !$found; $j++)
            {
              if ($tmp_column_variable_data[$j] eq $tmp_value)
              {
                $found = 1;
              }
            }
            if (!$found)
            {
              $error_string .= "<br>\n" if (length $error_string > 0);
              $error_string .= "Column Data = '$tmp_value' is invalid for Column Type = '$column_type'!";
            }
          }
        }
        elsif (length $column_data == 0 && $column_type eq "sub_entities")
        {
        }
        else
        {
          $error_string .= "<br>\n" if (length $error_string > 0);
          $error_string .= "You must select at least one Column Data value to work with for Column Type = '$column_type'!";
        }
      }
    }

    my $date_counter = 0;  # keeps track of the number of valid dates we have encountered.
    if ($memorize)
    {
      my @dates = split(/\|/, $attributes{'dates_entered'});
      $date_counter = scalar @dates;
    }
    else
    {
      # now check for the dates.
      if ($attributes{'date_range'} eq "true")
      {
        @arguments = ();
        for (my $i=0; $i < $attributes{'max_columns'}; $i++)
        {
          push (@arguments, ("column$i"."_start_date", "column$i"."_end_date"));
        }
      }
      else # single dates
      {
        @arguments = ();
        for (my $i=0; $i < $attributes{'max_columns'}; $i++)
        {
          push (@arguments, ("column$i"."_start_date"));
        }
      }

      my @entered_dates = ();
      # now validate we got the minimum amount of required dates and that all dates input are valid.
      for (my $i=0; $i < scalar @arguments; $i++)
      {
        my $date = $arguments[$i];
        if (!exists $input{$date})
        {
          $doc->doRequiredParameterError('get_report_attributes', $date);
        }

        if (length $input{$date} > 0)
        {
          my @dateResult = $methods->fixupDate(date => $input{$date});
          if (!$dateResult[0])
          {
            $error_string .= "<br>\n" if (length $error_string > 0);
            $error_string .= "'$date' = '$input{$date}' is invalid!<br>$dateResult[2]\n";
          }
          else
          {
            # make sure the date is "fixed".
            $input{$date} = $dateResult[1];
            if ($attributes{'date_range'} eq "false")
            { # simple case
              $entered_dates[$date_counter++] = $input{$date};
            }
            else
            { # check the ending date
              my $old_date = $date;
              $date = $arguments[++$i];
              if (!exists $input{$date})
              {
                $doc->doRequiredParameterError('get_report_attributes', $date);
              }

              if (length $input{$date} > 0)
              {
                @dateResult = $methods->fixupDate(date => $input{$date});
                if (!$dateResult[0])
                {
                  $error_string .= "<br>\n" if (length $error_string > 0);
                  $error_string .= "'$date' = '$input{$date}' is invalid!<br>$dateResult[2]\n";
                }
                else
                {
                  # make sure the date is "fixed".
                  $input{$date} = $dateResult[1];
                  $entered_dates[$date_counter++] = "$input{$old_date},$input{$date}";
                }
              }
            }
          }
        }
      }

      #$error_string .= "<br>\n" if (length $error_string > 0);
      #$error_string .= "'$date_counter' Dates Entered = '" . join("<br>\n", @entered_dates) . "'";
      $attributes{'dates_entered'} = [ @entered_dates ];
    }

    if ($date_counter < $attributes{'min_columns'})
    {
      $error_string .= "<br>\n" if (length $error_string > 0);
      $error_string .= "Only <b>$date_counter</b> Date(s) were entered!&nbsp;&nbsp;At least <b>$attributes{'min_columns'}</b> Date(s) must be entered.";
    }

    $attributes{displayZeroRows} = exists $input{displayZeroRows};
  }
  else  # Detailed View Report
  {
    # Currently nothing to do.
  }

  return (\%attributes, $error_string);
}

# This function validates that the attribute and value given it from the get_report_attributes function which gets them from the customize report screen are valid.
sub validate_customized_report_attribute
{
  my %args = ( @_, );
  my $dbh = $args{'dbh'};
  my $name = $args{'name'};
  my $value = $args{'value'};
  my @arguments = ("dbh", "name", "value");
  my $result = 1;

  eval ("use HTMLObject::Normal;");
  if ($@)
  {
    my_die("Error: Report.cgi - Eval", 4002, "Error: $@ at line " . __LINE__);
  }

  my $doc = HTMLObject::Normal->new();

  $doc->setJavascriptErrorHandlerEmail(email => $email_address);

  foreach my $name (@arguments)
  {
    if (!exists $args{$name})
    {
      $doc->doRequiredParameterError('validate_customized_report', $name);
    }
  }

  # validate it is valid.
  if ($name eq "date_range" || $name eq "total_column" || $name eq "display_dates")
  {
    if (!exists $true_false{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "id")
  {
    if ($value < 0)
    {
      $result = 0;
    }
    # look the report up that has this id and make sure it exists.
    if (length DoesReportExist(dbh => $dbh, id => $value) == 0)
    {
      $result = 0;
    }
  }
  elsif ($name eq "column_data" || $name eq "column_type" || $name eq "column_variable" || $name eq "column_variable_data")
  { # these are checked later in the get_report_attributes function.
  }
  elsif ($name eq "column_data_names")
  { # ignore this value; it's just used to pass data through for the names of the column data numbers
  }
  elsif ($name eq "fg_color")
  {
    if ($value ne "default" && !exists $fg_colors{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "bg_color")
  {
    if ($value ne "default" && !exists $bg_colors{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "default_fg_color")
  {
    if ($value ne "default" && !exists $fg_colors{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "default_bg_color")
  {
    if ($value ne "default" && !exists $bg_colors{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "ledger")
  {
    if (length get_ledger_name(dbh => $dbh, ledger_db => $value) == 0)
    {
      $result = 0;
    }
  }
  elsif ($name eq "report_name")
  {
    if (length $value == 0)
    {
      $result = 0;
    }
  }
  elsif ($name eq "report_class")
  {
    if (length $value == 0)
    {
      $result = 0;
    }
  }
  elsif ($name eq "max_columns")
  {
    if ($value < 1 || $value > $max_dates)
    {
      $result = 0;
    }
  }
  elsif ($name eq "min_columns")
  {
    if ($value < 1 || $value > $max_dates)
    {
      $result = 0;
    }
  }
  elsif ($name eq "underline_length")
  {
    if ($value < $min_underline_length || $value > $max_underline_length)
    {
      $result = 0;
    }
  }
  elsif ($name eq "data_precision")
  {
    if (!exists $precision_formats{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "negative_format")
  {
    if ($value != -1 && !exists $negative_formats{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "text_size" || $name eq "data_size")
  {
    if (!exists $font_sizes{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "text_font" || $name eq "data_font")
  {
    if (!exists $font_names{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "account_format")
  {
    if (!exists $account_formats{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "report_type")
  {
    $result = 0 if (!exists $report_types{$value});
  }
  elsif ($name eq "sql_statement")
  {
    $result = 0 if (length $value == 0);
  }
  elsif ($name eq "headers")
  {
  }
  elsif ($name eq "sorted_by")
  {
  }
  elsif ($name eq "sorted_order")
  {
    $result = 0 if ($value != "a" && $value != "d");
  }
  elsif ($name eq "subtotal")
  {
    $result = 0 if (!exists $true_false{$value});
  }
  elsif ($name eq "subtotal_type")
  {
    $result = 0 if ($value != "d" && $value != "b");
  }
  elsif ($name eq "dates_entered")
  {
    $result = 0 if (length $value == 0);
  }
  elsif ($name eq "border")
  {
    $result = 0 if ($value != 0 && $value != 1);
  }
  elsif ($name eq "windowName")
  {
    $result = 0 if ($value !~ /$reportNumRE/);
  }
  elsif ($name eq "reportWidth")
  {
    $result = 0 if ($value !~ /^(\d+)$/ && ($value < 25 || $value > 100));
  }
  else
  {
    $result = 0;  # unknown parameter name.
  }

  return $result;
}

# This function returns the data from the storable object (if specified via cookie) or defaults for creating a report.
# currently takes nothing.
sub get_default_data
{
  my %values;
  my $value_ref; # reference to the hash.

  # check and see if the error cookie exists.
  my %ch = $cookies->getCookies(cookies => ['error_happened']);
  if (scalar keys %ch == 1)
  {
    my $file_name = $cookies->cookies->{'error_happened'};
    $value_ref = retrieve($file_name);
    %values = %{$value_ref};
    #unlink $file_name;  # delete the file.
  }
  else
  {
    $values{'name'} = "";
    $values{'id'} = 0;
    $values{'report_class'} = "custom";
    $values{'report_type'} = "r";
    $values{'column_variable'} = "";
    $values{'column_variable_data'} = "";
    $values{'display_dates'} = "true";
    $values{'date_range'} = "true";
    $values{'max_columns'} = 36;
    $values{'min_columns'} = 1;
    $values{'text_size'} = "-1";
    $values{'text_font'} = "helvetica";
    $values{'data_size'} = "-1";
    $values{'data_font'} = "helvetica";
    $values{'total_column'} = "false";
    $values{'fg_color'} = "black";
    $values{'bg_color'} = "white";
    $values{'underline_length'} = $default_underline_length;
    $values{displayZeroRows} = 0;
  }

  return %values;
}

sub validateParameter
{
  # this function takes the name of the parameter and it's value and does the work of making sure that the value is valid for that parameter.
  my ($name, $value) = ( @_ );
  my $result = 1;

  if (length $name == 0)
  {
    $result = 0;
  }
  if (length $value == 0)
  {
    $result = 0;
  }

  if ($name eq "mode")
  {
    if ($value ne "edit" && $value ne "create")
    {
      $result = 0;
    }
  }
  elsif ($name eq "name")
  {
  }
  elsif ($name eq "original_name")
  {
  }
  elsif ($name eq "id")
  {
    if ($value < -1)
    {
      $result = 0;
    }
  }
  elsif ($name eq "report_class")
  {
    if (!exists $report_classes{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "report_type")
  {
    if ($value ne "r" && $value ne "d")
    {
      $result = 0;
    }
  }
  elsif ($name eq "column_variable")
  {
  }
  elsif ($name eq "column_variable_data")
  {
  }
  elsif ($name eq "date_range")
  {
    if ($value ne "true" && $value ne "false")
    {
      $result = 0;
    }
  }
  elsif ($name eq "display_dates")
  {
    if ($value ne "true" && $value ne "false")
    {
      $result = 0;
    }
  }
  elsif ($name eq "max_columns")
  {
    if ($value <= 0 || $value > $max_dates)
    {
      $result = 0;
    }
  }
  elsif ($name eq "min_columns")
  {
    if ($value <= 0 || $value > $max_dates)
    {
      $result = 0;
    }
  }
  elsif ($name eq "text_size")
  {
    if (!exists $font_sizes{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "text_font")
  {
    if (!exists $font_names{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "data_size")
  {
    if (!exists $font_sizes{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "data_font")
  {
    if (!exists $font_names{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "total_column")
  {
    if ($value ne "true" && $value ne "false")
    {
      $result = 0;
    }
  }
  elsif ($name eq "fg_color")
  {
    if (!exists $fg_colors{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "bg_color")
  {
    if (!exists $bg_colors{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "underline_length")
  {
    if ($value < $min_underline_length || $value > $max_underline_length)
    {
      $result = 0;
    }
  }
  elsif ($name eq "dbh")
  {
  }
  elsif ($name eq "error_string")
  {
  }
  elsif ($name eq "rows")
  {
    my @rows = split /\n/, $value;
    if (scalar @rows == 0)
    {
      $result = 0;
    }
  }
  elsif ($name eq "file_name")
  {
  }
  elsif ($name eq "report_group")
  {
    if ($value ne "regular" && $value ne "detailed" && $value ne "memorized")
    {
      $result = 0;
    }
  }
  elsif ($name eq "new_report_name")
  {
  }
  elsif ($name eq "windowName")
  {
    $result = 0 if ($value !~ /^(edit_report_window)$/);
  }
  else
  {
    $result = 0;  # signal that an error occurred as we do not know anything about this variable.
  }

  return $result;
}

sub get_all_account_types
{
  my %args = ( @_, );
  my $dbh = $args{'dbh'};
  my @data = ();

  if (!exists $args{'dbh'} || length $dbh == 0)
  {
    my_die("Error:  Report.cgi - get_all_account_types", 4005, "The Database Handle is required!");
  }

  my $sth = $dbh->prepare( $sql_statements{1019} ) || my_db_die("Error: Report.cgi - get_all_account_types", 4007, "SQL Statement = '$sql_statements{1019}'");
  my $rc = $sth->execute || my_db_die("Error: Report.cgi - get_all_account_types", 4008, "SQL Statement = '$sql_statements{1019}'");

  my $ref = $sth->fetchall_arrayref;
  if ($sth->err)
  {
    my_db_die("Error:  Report.cgi - get_all_account_types", 4009, "Getting all Account Types failed!");
  }
  @data = @{$ref};

  return @data;
}

sub get_all_accounts
{
  my %args = ( @_, );
  my $dbh = $args{'dbh'};
  my @data = ();

  if (!exists $args{'dbh'} || length $dbh == 0)
  {
    my_die("Error:  Report.cgi - get_all_accounts", 4005, "The Database Handle is required!");
  }

  my $sth = $dbh->prepare( $sql_statements{1020} ) || my_db_die("Error: Report.cgi - get_all_accounts", 4007, "SQL Statement = '$sql_statements{1020}'");
  my $rc = $sth->execute || my_db_die("Error: Report.cgi - get_all_accounts", 4008, "SQL Statement = '$sql_statements{1020}'");

  my $ref = $sth->fetchall_arrayref;
  if ($sth->err)
  {
    my_db_die("Error:  Report.cgi - get_all_accounts", 4009, "Getting all Accounts failed!");
  }
  $sth->finish;
  @data = @{$ref};

  return @data;
}

sub get_all_sub_accounts
{
  my %args = ( @_, );
  my $dbh = $args{'dbh'};
  my @data = ();

  if (!exists $args{'dbh'} || length $dbh == 0)
  {
    my_die("Error:  Report.cgi - get_all_sub_accounts", 4005, "The Database Handle is required!");
  }

  my $sth = $dbh->prepare( $sql_statements{1021} ) || my_db_die("Error: Report.cgi - get_all_sub_accounts", 4007, "SQL Statement = '$sql_statements{1021}'");
  my $rc = $sth->execute || my_db_die("Error: Report.cgi - get_all_sub_accounts", 4008, "SQL Statement = '$sql_statements{1021}'");

  my $ref = $sth->fetchall_arrayref;
  if ($sth->err)
  {
    my_db_die("Error:  Report.cgi - get_all_sub_accounts", 4009, "Getting all Sub Accounts failed!");
  }
  $sth->finish;
  @data = @{$ref};

  return @data;
}

# takes  : account type array and number to find
# returns: 1 if found, 0 otherwise.
sub find_account_type
{
  my %args = ( @_, );
  my @array = @{$args{'array'}};
  my $num = $args{'num'};
  my $found = 0;

  if (!exists $args{'array'})
  {
    my_die("Error:  Report.cgi - find_account_type", 4005, "array is required");
  }

  if (scalar @array == 0)
  {
    return 0;
  }
  if (!exists $args{'num'} || length $num == 0)
  {
    my_die("Error:  Report.cgi - find_account_type", 4005, "num is required");
  }

  for (my $i=0; $i < scalar @array && !$found; $i++)
  {
    if ($array[$i][1] == $num)
    {
      $found = 1;
    }
  }

  return $found;
}

sub find_account
{
  my %args = ( @_, );
  my @array = @{$args{'array'}};
  my $num = $args{'num'};
  my $found = 0;

  if (!exists $args{'array'})
  {
    my_die("Error:  Report.cgi - find_account", 4005, "array is required");
  }

  if (scalar @array == 0)
  {
    return 0;
  }
  if (!exists $args{'num'} || length $num == 0)
  {
    my_die("Error:  Report.cgi - find_account", 4005, "num is required");
  }

  for (my $i=0; $i < scalar @array && !$found; $i++)
  {
    if ($array[$i][1] == $num)
    {
      $found = 1;
    }
  }

  return $found;
}

sub find_sub_account
{
  my %args = ( @_, );
  my @array = @{$args{'array'}};
  my $num = $args{'num'};
  my $found = 0;

  if (!exists $args{'array'})
  {
    my_die("Error:  Report.cgi - find_sub_account", 4005, "array is required");
  }

  if (scalar @array == 0)
  {
    return 0;
  }
  if (!exists $args{'num'} || length $num == 0)
  {
    my_die("Error:  Report.cgi - find_sub_account", 4005, "num is required");
  }

  for (my $i=0; $i < scalar @array && !$found; $i++)
  {
    if ($array[$i][1] == $num)
    {
      $found = 1;
    }
  }

  return $found;
}

sub find_level
{
  my %args = ( @_, );
  my @array = @{$args{'array'}};
  my $level = $args{'level'};
  my $found = -1;

  if (!exists $args{'array'})
  {
    my_die("Error:  Report.cgi - find_detailed_report", 4005, "array is required");
  }

  if (scalar @array == 0)
  {
    return 0;
  }
  if (!exists $args{'level'} || length $level == 0)
  {
    my_die("Error:  Report.cgi - find_detailed_report", 4005, "level is required");
  }

  for (my $i=0; $i < scalar @array && $found == -1; $i++)
  {
    if ($array[$i] eq $level)
    {
      $found = $i;
    }
  }

  return $found;
}

# FindSubAccount
# takes: number and reference to array to search in.
# returns the Name of the Sub Account.
sub FindSubAccount
{
  my ($number, @array);
  $number = $_[0];
  @array = @{$_[1]};

  if (length $number == 0)
  {
    my_die("Error: Report.cgi - FindSubAccount", 4005, "The Sub Account Number must be specified!");
  }
  if (scalar @array == 0)
  {
    my_die("Error: Report.cgi - FindSubAccount", 4005, "The Array to search must be specified!");
  }

  # now we walk through all SubAccountTypes and for those that have SubAccount arrays, we walk them till we find
  # the specified SubAccount number and then we can get it's name!
  my $name = "";
  my $found = 0;

  for (my $index=0; $index < scalar @array && !$found; $index++)
  {
    if (scalar @{$array[$index][2]} > 0)
    {
      for (my $index2=0; $index2 < scalar @{$array[$index][2]} && !$found; $index2++)
      {
        if ($array[$index][2][$index2][0] == $number)
        {
          $found = 1;
          $name = $array[$index][2][$index2][1];
        }
      }
    }
  }

  if (!$found)
  {
    my_die("Error: Report.cgi - FindSubAccount", 4013, "The specified Sub Account = '$number' was not found!");
  }

  return $name;
}

# FindSubAccountType
# takes: number and reference to array to search in.
# returns the Name of the Sub Account Type.
sub FindSubAccountType
{
  my ($number, @array);
  $number = $_[0];
  @array = @{$_[1]};

  if (length $number == 0)
  {
    my_die("Error: Report.cgi - FindSubAccount", 4005, "The Sub Account Type Number must be specified!");
  }
  if (scalar @array == 0)
  {
    my_die("Error: Report.cgi - FindSubAccount", 4005, "The Array to search must be specified!");
  }

  # now we walk through all SubAccountTypes till we find
  # the specified SubAccountType number and then we can get it's name!
  my $name = "";
  my $found = 0;

  for (my $index=0; $index < scalar @array && !$found; $index++)
  {
    if ($array[$index][0] == $number)
    {
      $found = 1;
      $name = $array[$index][1];
    }
  }

  if (!$found)
  {
    my_die("Error: Report.cgi - FindSubAccount", 4014, "The specified Sub Account Type = '$number' was not found!");
  }

  return $name;
}

sub customize_report_display
{
  my %args = ( error_string => '', @_, );
  my $error_string = $args{'error_string'};
  my $dbh = $args{'dbh'};
  my @known_dates;
  my $currentWorkingYear;

  eval ("use HTMLObject::Normal;");
  if ($@)
  {
    my_die("Error: Report.cgi - Eval", 4002, "Error: $@ at line " . __LINE__);
  }

  eval ("use HTMLObject::FrameSet;");
  if ($@)
  {
    my_die("Error: Report.cgi - Eval", 4002, "Error: $@ at line " . __LINE__);
  }

  # check to make sure we were given the ledger_db name
  my $ledger = $input{'ledger'};
  if (length $ledger == 0)
  {
    my_die("Error: Report.cgi - customize_report", 4022, "The Ledger Code is required at line " . __LINE__);
  }

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - customize_report", 4028, "The DB Handle is required at line " . __LINE__);
  }

  my $doc = HTMLObject::Normal->new();
  $doc->setJavascriptErrorHandlerEmail(email => $email_address);
  $doc->setTitle("Customize Report - $args{'report_name'}");
  $doc->setFocus('body');

  my $date = `/bin/date +$date_format`;
  chomp $date;

  my $ledger_name = $args{'ledger_name'};
  my $ledger_code = $args{'ledger_code'};
  my $ledger_color = $args{'ledger_color'};
  my $reportWidth = $args{reportWidth};

  # retrieve values from the known_years_tb table
  @known_dates = getKnownYearsInfo(dbh => $dbh, ledger_code => $ledger_code);
  $currentWorkingYear = GetWorkingYear(dbh => $dbh, ledger_code => $ledger_code);

  # create the header for the page.
  $doc->print(<<"END_OF_HEADER");
  <table border=0 cellpadding=0 cellspacing=0 width=100%>
    <tr>
      <td align=left width=47%>Working with Ledger <font color="$ledger_color">$ledger_name</font></td>
      <td align=left><a href="$site_dir/help.html#new_reporting_customize" target="docs"><img border=0 src="$site_dir/images/help.jpg" alt="Help"></a></td>
      <td align=right>$date</td>
    </tr>
  </table>
  <hr>
  <br>
  <table border=0 cellpadding=0 cellspacing=0 width=100%>
    <tr>
      <td align=left width=75%>
END_OF_HEADER

  if (length $error_string > 0)
  {
    $doc->print("<h2><font color=\"red\">Error</font>: &nbsp;$error_string</h2>\n");
  }

  $doc->print(<<"END_OF_HEADER");
      </td>
    </tr>
  </table>
END_OF_HEADER

  # turn off the link decorations
  $doc->setLinkDecorations();

  # set the overall font size to be smaller
  $doc->setStyleEntry(tag => "body", string => "font-size: smaller;");

  if ($args{'report_type'} eq "r")
  { # regular view report
    my $date_range = ($args{'date_range'}) ? "true" : "false";
    my $display_dates = ($args{'display_dates'}) ? "true" : "false";
    my $displayZeroRowsChecked = ($args{displayZeroRows} ? " checked=\"true\"" : "");

    $doc->print(<<"END_OF_BODY");

  <center>
  <h1>Customize <font color="red">$args{'report_name'}</font> Report</h1>
  <form method=post action="$site_dir/cgi-bin/report_customization.cgi" name="customize_form" target="_top">
    <input type=hidden name="command" value="run_report">
    <input type=hidden name="date_range" value="$date_range">
    <input type=hidden name="display_dates" value="$display_dates">
    <input type=hidden name="id" value="$args{'id'}">
    <input type=hidden name="ledger" value="$args{'ledger'}">
    <input type=hidden name="ledger_code" value="$args{'ledger_code'}">
    <input type=hidden name="report_class" value="$args{'report_class'}">
    <input type=hidden name="column_variable" value="$args{'column_variable'}">
    <input type=hidden name="column_variable_data" value="$args{'column_variable_data'}">
    <input type=hidden name="default_fg_color" value="$args{'default_fg_color'}">
    <input type=hidden name="default_bg_color" value="$args{'default_bg_color'}">
    <input type=hidden name="report_type" value="$args{'report_type'}">
    <input type=hidden name="fg_color" value="$args{'fg_color'}">
    <input type=hidden name="bg_color" value="$args{'bg_color'}">
    <input type=hidden name="text_font" value="$args{'text_font'}">
    <input type=hidden name="data_font" value="$args{'data_font'}">
    <input type=hidden name="text_size" value="$args{'text_size'}">
    <input type=hidden name="data_size" value="$args{'data_size'}">
    <input type=hidden name="underline_length" value="$args{'underline_length'}">
    <input type=hidden name="account_format" value="$args{'account_format'}">
    <input type=hidden name="negative_format" value="$args{'negative_format'}">
    <input type=hidden name="border" value="$args{'border'}">
    <input type="hidden" name="windowName" value="$args{windowName}">
    <table width=\"75%\" border=0 cellspacing=0 cellpadding=2 bgcolor="$menu_bgcolor">
      <tr>
        <th align=right><font color="$menu_fgcolor">Report Name:</font></th>
        <td align=left><input type=text name="report_name" value="$args{'report_name'}" size="30"></td>
      </tr>
      <tr>
        <td colspan=2><hr width="95%"></td>
      </tr>
      <tr>
      <td colspan=2>
      <table width="100%" cellspacing=0 cellpadding=2>
      <tr>
END_OF_BODY
    if ($args{'display_dates'})
    {
      $doc->print("        <td>\n");
    }
    else
    {
      $doc->print("        <td colspan=2>\n");
    }
    $doc->print(<<"END_OF_BODY");
          <input type=hidden name="min_columns" value="$args{'min_columns'}">
          <input type=hidden name="max_columns" value="$args{'max_columns'}">
          <table width="100%" cellspacing=0 cellpadding=2 bgcolor="$menu_bgcolor">
            <tr>
END_OF_BODY
    $doc->print("              <td align=center colspan=");
    if ($args{'date_range'})
    {
      $doc->print("3");
    }
    else
    {
      $doc->print("2");
    }
    $doc->print("><font color=\"$menu_fgcolor\">NOTE: A minimum of <b>$args{'min_columns'}</b> field(s) must be filled in.<br><br></font></td>\n");
    $doc->print(<<"END_OF_BODY");
            </tr>
END_OF_BODY
    my $count;
    for ($count=0; $count < $args{'max_columns'}; $count++)
    {
      $doc->print("            <tr>\n");
      $doc->print("              <td align=right nowrap>Column " . ($count + 1) . " Date");
      if ($args{'date_range'})
      {
        $doc->print("s");
      }
      $doc->print(":</td>\n");
      (my $temp_start_date, my $temp_end_date) = split(/,/, $args{'dates_entered'}[$count]);
      $doc->print("              <td align=left nowrap>");
      my $currYear = $methods->getCurrentDate(format => "%Y");
      my %datePicker = $methods->generatePickerCode(type => "date", doc => $doc, form => "customize_form",
                              itemName => "column" . $count . "_start_date", itemValue => $temp_start_date, year => $currYear,
                              phrase => $langObj->map("FromColon"), width => 300, height => 250);
      if ($methods->didErrorOccur)
      {
        my_die($methods->errorMessage);
      }
      $doc->print(%datePicker);
      $doc->print("</td>\n");
      if ($args{'date_range'})
      {
        %datePicker = $methods->generatePickerCode(type => "date", doc => $doc, form => "customize_form",
                                itemName => "column" . $count . "_end_date", itemValue => $temp_end_date, year => $currYear,
                                phrase => $langObj->map("ToColon"), width => 300, height => 250);
        if ($methods->didErrorOccur)
        {
          my_die($methods->errorMessage);
        }
        $doc->print("              <td align=\"left\" nowrap=\"1\">");
        $doc->print(%datePicker);
        $doc->print("</td>\n");
      }
      $doc->print("              <td align=left><input type=button value=\"Clear\" onClick=\"document.customize_form.column" . $count . "_start_date.value = '';");
      if ($args{'date_range'})
      {
        $doc->print(" document.customize_form.column" . $count . "_end_date.value = '';");
      }
      $doc->print(" tweakReportWidth(); document.customize_form.column" . $count . "_start_date.focus();\"></td>\n");
      $doc->print("            </tr>\n");
    }
    if ($args{'display_dates'})
    {
      $doc->print(<<"END_OF_BODY");
          </table>
        </td>
        <td>
          <table width="100%" cellspacing=0 cellpadding=2 bgcolor="$menu_bgcolor">
            <tr>
              <td>Date Selection:<br><select name=\"date_select\" size=10 width=100 onChange=\"date_select_change();\">
END_OF_BODY
      for ($count=scalar(@known_dates)-1; $count >= 0; $count--)
      {
        if ($args{'date_range'})
        {
          $doc->print("                <option value=\"$known_dates[$count][1],$known_dates[$count][2]\">$known_dates[$count][0]\n");
          if ($known_dates[$count][0] >= $currentWorkingYear - 1 && $known_dates[$count][0] <= $currentWorkingYear)
          {
            $doc->print("                <option value=\"$known_dates[$count][3],$known_dates[$count][4]\">&nbsp;&nbsp;&nbsp;&nbsp;Q1\n");
            $doc->print("                <option value=\"$known_dates[$count][5],$known_dates[$count][6]\">&nbsp;&nbsp;&nbsp;&nbsp;Q2\n");
            $doc->print("                <option value=\"$known_dates[$count][7],$known_dates[$count][8]\">&nbsp;&nbsp;&nbsp;&nbsp;Q3\n");
            $doc->print("                <option value=\"$known_dates[$count][9],$known_dates[$count][10]\">&nbsp;&nbsp;&nbsp;&nbsp;Q4\n");
          }
        }
        else
        {
          if ($known_dates[$count][0] >= $currentWorkingYear - 1 && $known_dates[$count][0] <= $currentWorkingYear)
          {
            $doc->print("                <option value=\"$known_dates[$count][1]\">$known_dates[$count][0] Start\n");
          }
          $doc->print("                <option value=\"$known_dates[$count][2]\">$known_dates[$count][0] End\n");
          if ($known_dates[$count][0] >= $currentWorkingYear - 1 && $known_dates[$count][0] <= $currentWorkingYear)
          {
            $doc->print("                <option value=\"$known_dates[$count][3]\">&nbsp;&nbsp;&nbsp;&nbsp;Q1 Start\n");
            $doc->print("                <option value=\"$known_dates[$count][4]\">&nbsp;&nbsp;&nbsp;&nbsp;Q1 End\n");
            $doc->print("                <option value=\"$known_dates[$count][5]\">&nbsp;&nbsp;&nbsp;&nbsp;Q2 Start\n");
            $doc->print("                <option value=\"$known_dates[$count][6]\">&nbsp;&nbsp;&nbsp;&nbsp;Q2 End\n");
            $doc->print("                <option value=\"$known_dates[$count][7]\">&nbsp;&nbsp;&nbsp;&nbsp;Q3 Start\n");
            $doc->print("                <option value=\"$known_dates[$count][8]\">&nbsp;&nbsp;&nbsp;&nbsp;Q3 End\n");
            $doc->print("                <option value=\"$known_dates[$count][9]\">&nbsp;&nbsp;&nbsp;&nbsp;Q4 Start\n");
            $doc->print("                <option value=\"$known_dates[$count][10]\">&nbsp;&nbsp;&nbsp;&nbsp;Q4 End\n");
          }
        }
      }
      $doc->print(<<"END_OF_BODY");
              </select></td>
            </tr>
END_OF_BODY
    }
    $doc->print(<<"END_OF_BODY");
          </table>
        </td>
      </tr>
      </table>
      </td>
      </tr>
END_OF_BODY
    if ($args{'column_variable'} ne "")
    {
      $doc->print(<<"END_OF_BODY");
      <tr>
        <td colspan=2><hr width="95%"></td>
      </tr>
      <tr>
        <td colspan=2>
          <table width="100%" cellspacing=0 cellpadding=2 bgcolor="$menu_bgcolor">
            <tr>
              <th colspan=3><font color="$menu_fgcolor">COLUMN VARIABLE</font></th>
            </tr>
            <tr>
              <th align=center><font color="$menu_fgcolor">Column Type</font></th>
              <th align=center><font color="$menu_fgcolor">Column Data</font></th>
              <td><br></td>
            </tr>
            <tr>
              <td align=center valign=top><select name="column_type" onChange="column_type_change();">
END_OF_BODY
      $doc->print("                <option value=\"nothing\">Nothing\n");
      my @temp_array = split /\,/, $args{'column_variable'};
      for ($count=0; $count < scalar @temp_array; $count++)
      {
        $doc->print("                <option value=\"$temp_array[$count]\"");
        if ($args{'column_type'} eq $temp_array[$count])
        {
          $doc->print(" selected");
        }
        $doc->print(">$column_headers{$temp_array[$count]}\n");
      }
      $doc->print(<<"END_OF_BODY");
              </select></td>
              <td align=center valign=top><select name="column_data" size=5 multiple width=250>
              </select><input type=hidden name="column_data_names" value=""></td>
              <td align=left valign=top><input type=button value="All" onClick="select_all_column_data();"><br>
              <input type=button value="None" onClick="unselect_column_data();"></td>
            </tr>
          </table>
        </td>
      </tr>
END_OF_BODY
    }
    else
    {
      $doc->print(<<"END_OF_BODY");
      <input type=hidden name="column_type" value="nothing">
END_OF_BODY
    }
    $doc->print(<<"END_OF_BODY");
      <tr>
        <td colspan=2><hr width="95%"></td>
      </tr>
      <tr>
        <th align=center><font color="$menu_fgcolor">Precision of Data</font></th>
        <th align=center><font color="$menu_fgcolor">Total Column</font></th>
      </tr>
      <tr>
        <td align=center><select name="data_precision">
END_OF_BODY
    for ($count=0; $count < scalar @precision_formats; $count++)
    {
      $doc->print("          <option value=\"$precision_formats[$count]\"");
      if ($args{'data_precision'} eq $precision_formats[$count])
      {
        $doc->print(" selected");
      }
      $doc->print(">$precision_formats{$precision_formats[$count]}\n");
    }
    $doc->print(<<"END_OF_BODY");
        </select></td>
        <td align=center>
          <select name="total_column">
END_OF_BODY
    $doc->print("            <option value=\"true\"" . ($args{total_column} ? " selected" : "") . ">Yes</option>\n");
    $doc->print("            <option value=\"false\"" . (!$args{total_column} ? " selected" : "") . ">No</option>\n");
    $doc->print(<<"END_OF_BODY");
          </select>
        </td>
      </tr>
      <tr>
        <th align="center"><font color="$menu_fgcolor">Printed Report Width %</font></th>
        <th align="center"><font color="$menu_fgcolor">Display Zero Rows?</font></th>
      </tr>
      <tr>
        <td align="center"><input type="text" name="reportWidth" value="$reportWidth" size="3" maxlength="3"></td>
        <td align="center"><input type="checkbox" name="displayZeroRows" value="1"$displayZeroRowsChecked></td>
      </tr>
    </table>
    <input type=submit name="custom_submit" value="Run Report" onClick="return validate_form();">&nbsp;<input type=button value="Reset" onClick="document.customize_form.reset(); reset_form();">
  </form>
  </center>
END_OF_BODY

    my $onload_code;
    $onload_code = "    // onload code\n";
    if ($args{'column_variable'} ne "")
    {
      $onload_code = <<"END_OF_ONLOAD_CODE";
    var f = document.customize_form;

    column_type_change();

    for (var count=0; count < f.column_data.options.length; count++)
    {
      if (typeof column_defaults[f.column_data.options[count].value] == "undefined" && column_defaults[f.column_data.options[count].value] != 1)
      {
        f.column_data.options[count].selected = false;
      }
      else
      {
        f.column_data.options[count].selected = true;
      }
    }
END_OF_ONLOAD_CODE
    }

    $doc->setFocus("javascript");
    $doc->print(<<"END_OF_SCRIPT");

  var column_defaults = new Array();

END_OF_SCRIPT
    my @reset_split_array = split(/,/, $args{'column_data'});
    for ($count=0; $count < scalar @reset_split_array; $count++)
    {
      $doc->print("  column_defaults[\"$reset_split_array[$count]\"] = 1;\n");
    }
    $doc->print(<<"END_OF_SCRIPT");

  function reset_form()
  {
    setTimeout("doOnLoad();", 10);
  }

  // data for column_type and column_data select objects
  var column_data = new Array();

  function create_column_object(value, text)
  {
    this.value = value;
    this.text = text;
  }

END_OF_SCRIPT
    my @temp_array = split /,/, $args{'column_variable'};
    my @temp_data_array = split /\|/, $args{'column_variable_data'};
    $doc->print("  column_data[\"nothing\"] = new Array();\n");
    for ($count=0; $count < scalar @temp_array; $count++)
    {
      my @temp_data_split_array = split /,/, $temp_data_array[$count];
      my @temp_data_name_array;
      # retrieve values from the database based on the type of data for the current column_type
      my $sth = $dbh->prepare( $sql_statements{$column_data_sql{$temp_array[$count]}} ) || my_db_die("Error: Report.cgi - customize_report", 4007, "SQL Statement = '$sql_statements{$column_data_sql{$temp_array[$count]}}'");
      if ($temp_array[$count] eq "sub_entities")
      {
        my $rc = $sth->execute( $ledger_code ) || my_db_die("Error: Report.cgi - customize_report", 4008, "SQL Statement = '$sql_statements{$column_data_sql{$temp_array[$count]}}'");
        my $count2 = 0;
        # add the default of "" (nothing) to the sub-entities arrays
        $temp_data_split_array[$count2] = "<non_sub_entity>";
        $temp_data_name_array[$count2] = "<None>";
        $count2++;
        while (my @temp_row = $sth->fetchrow_array)
        {
          $temp_data_split_array[$count2] = $temp_row[1];
          $temp_data_name_array[$count2] = $temp_row[0];
          $count2++;
        }
        $sth->finish;

        # add a comma-separated list of sub-entity codes to the appropriate array element and rebuild the column_variable_data
        # environment variable
        my $temp_data = "";
        for (my $count2=0; $count2 < scalar @temp_data_split_array; $count2++)
        {
          $temp_data .= $temp_data_split_array[$count2];
          if ($count2 < scalar @temp_data_split_array - 1)
          {
            $temp_data .= ",";
          }
        }
        $temp_data_array[$count] = $temp_data;
        my $temp_column_variable_data = "";
        for (my $count2=0; $count2 < scalar @temp_data_array; $count2++)
        {
          $temp_column_variable_data .= $temp_data_array[$count2];
          if ($count2 < scalar @temp_data_array - 1)
          {
            $temp_column_variable_data .= "|";
          }
        }
        $args{'column_variable_data'} = $temp_column_variable_data;
        $onload_code .= "\n    f.column_variable_data.value = \"$temp_column_variable_data\";\n";
      }
      else
      {
        for (my $count2=0; $count2 < scalar @temp_data_split_array; $count2++)
        {
          my $rc = $sth->execute( $temp_data_split_array[$count2] ) || my_db_die("Error: Report.cgi - customize_report", 4008, "SQL Statement = '$sql_statements{$column_data_sql{$temp_array[$count]}}'");
          my @temp_row = $sth->fetchrow_array;
          $sth->finish;
          $temp_data_name_array[$count2] = $temp_row[0];
        }
      }
      $doc->print("  column_data[\"$temp_array[$count]\"] = new Array();\n");
      for (my $count2=0; $count2 < scalar @temp_data_split_array; $count2++)
      {
        $doc->print("  column_data[\"$temp_array[$count]\"][$count2] = new create_column_object(\"$temp_data_split_array[$count2]\", \"$temp_data_name_array[$count2] - ");
        if ($temp_data_split_array[$count2] eq "-1" || $temp_data_split_array[$count2] eq "<non_sub_entity>")
        {
          $doc->print("(");
        }
        if ($temp_data_split_array[$count2] ne "<non_sub_entity>")
        {
          $doc->print("$temp_data_split_array[$count2]");
        }
        if ($temp_data_split_array[$count2] eq "-1" || $temp_data_split_array[$count2] eq "<non_sub_entity>")
        {
          $doc->print(")");
        }
        $doc->print("\");\n");
      }

      # clear out the name array for the next time through
      @temp_data_name_array = ();
    }
    # set the onload code for JavaScript
    $doc->setOnload( code => $onload_code );
    $doc->print(<<"END_OF_SCRIPT");

  function column_type_change()
  {
    var f = document.customize_form;

    // clear the column_data select
    f.column_data.options.length = 0;
    f.column_data_names.value = "";
    // add the data in the column_data array to the column_data select for the chosen column_type
    for (var count=0; count < column_data[f.column_type.options[f.column_type.selectedIndex].value].length; count++)
    {
      // set the default to selected
      f.column_data.options[count] = new Option(column_data[f.column_type.options[f.column_type.selectedIndex].value][count].text, column_data[f.column_type.options[f.column_type.selectedIndex].value][count].value, true, true);
      f.column_data_names.value += column_data[f.column_type.options[f.column_type.selectedIndex].value][count].text;
      if (count < column_data[f.column_type.options[f.column_type.selectedIndex].value].length - 1)
      {
        f.column_data_names.value += "|";
      }
    }
  }

  function select_all_column_data()
  {
    var f = document.customize_form;

    // run through the entire options array and select each option
    for (var count=0; count < document.customize_form.column_data.options.length; count++)
    {
      f.column_data.options[count].selected = true;
    }
  }

  function unselect_column_data()
  {
    // clear all selected options
    document.customize_form.column_data.selectedIndex = -1;
  }

  function date_select_change()
  {
    var f = document.customize_form;
    var free_start_date, free_end_date;

    free_start_date = null;
    free_end_date = null;

    // find a free date text field(s) to put the date values in
END_OF_SCRIPT
    for ($count=$args{'max_columns'}-1; $count >= 0; $count--)
    {
      $doc->print("    if (document.customize_form.column" . $count . "_start_date.value == \"\"");
      if ($args{'date_range'})
      {
        $doc->print(" && document.customize_form.column" . $count . "_end_date.value == \"\"");
      }
      $doc->print(")\n");
      $doc->print("    {\n");
      $doc->print("      free_start_date = document.customize_form.column" . $count . "_start_date;\n");
      if ($args{'date_range'})
      {
        $doc->print("      free_end_date = document.customize_form.column" . $count . "_end_date;\n");
      }
      $doc->print("    }\n");
    }
    $doc->print("\n");
    $doc->print("    if (free_start_date != null");
    if ($args{'date_range'})
    {
      $doc->print(" && free_end_date != null");
    }
    $doc->print(")\n");
    $doc->print(<<"END_OF_SCRIPT");
    {
      var date_array = f.date_select.options[f.date_select.selectedIndex].value.split(",", 2);
      free_start_date.value = date_array[0];
END_OF_SCRIPT
    if ($args{'date_range'})
    {
      $doc->print("      free_end_date.value = date_array[1];\n");
    }
    $doc->print(<<"END_OF_SCRIPT");
      tweakReportWidth();
    }
    else
    {
      alert("No available Date rows!  Clear one.");
    }

    f.date_select.selectedIndex = -1;
  }

  function tweakReportWidth()
  {
    var f = document.customize_form;
    var filledRows = 0;
END_OF_SCRIPT
    for ($count=0; $count < $args{'max_columns'}; $count++)
    {
      $doc->print("    if (document.customize_form.column" . $count . "_start_date.value != \"\"");
      if ($args{'date_range'})
      {
        $doc->print(" && document.customize_form.column" . $count . "_end_date.value != \"\"");
      }
      $doc->print(")\n");
      $doc->print("    {\n");
      $doc->print("      filledRows++;\n");
      $doc->print("    }\n");
    }
    $doc->print(<<"END_OF_SCRIPT");

    if (filledRows > 1)
    {
      var number = new Number (55 + (filledRows * (45 / $args{max_columns})));
      f.reportWidth.value = number.toFixed();
    }
    else
    {
      f.reportWidth.value = 55;
    }
  }

  function validate_form()
  {
    var errorString = "";
    var f = document.customize_form;

    // make sure that all the necessary values are filled in before allowing the submission to continue

    if (f.report_name.value == "")
    {
      errorString += "The report name must be filled in!\\n";
    }

    var tempFilledCount = 0;
END_OF_SCRIPT
    for ($count=0; $count < $args{'max_columns'}; $count++)
    {
      $doc->print("    if (f.column" . $count . "_start_date.value != \"\"");
      if ($args{'date_range'})
      {
        $doc->print(" && f.column" . $count . "_end_date.value != \"\"");
      }
      $doc->print(")\n");
      $doc->print("    {\n");
      $doc->print("      tempFilledCount++;\n");
      $doc->print("    }\n");
    }
    $doc->print(<<"END_OF_SCRIPT");
    if (tempFilledCount < f.min_columns.value)
    {
      errorString += "Not enough dates were filled in!\\n";
    }

END_OF_SCRIPT
    if ($args{'column_variable'} ne "")
    {
      $doc->print(<<"END_OF_SCRIPT");
    if (f.column_type.selectedIndex != 0)
    {
      var selected_value = false;
      // check to make sure the user has selected some values from column_data
      for (var count=0; count < f.column_data.options.length; count++)
      {
        if (f.column_data.options[count].selected)
        {
          selected_value = true;
        }
      }
      if (!selected_value)
      {
        errorString += "No column data values were selected for column type: " + f.column_type.options[f.column_type.selectedIndex].text + "!\\n";
      }
    }
END_OF_SCRIPT
    }
    $doc->print(<<"END_OF_SCRIPT");

    if (errorString != "")
    {
      alert(errorString);
      return false;
    }

    return true;
  }

END_OF_SCRIPT
  }
  else
  { # detailed view report
    # check to see if the FrameSet has already been generated
    if (not exists $input{'customized_frameset_generated'})
    {
      my $doc = HTMLObject::FrameSet->new();

      # create the frameset and call the next step of the process.
      my $inner_content;
      $inner_content = $doc->createNoframe("You must have a frames capable browser!");
      $inner_content .= $doc->createFrame(src => "$site_dir/cgi-bin/report_customization.cgi?command=customize_report&id=$args{'id'}&memorized=false&ledger=$args{'ledger'}&customized_frameset_generated=1&windowName=$args{windowName}", frameborder => 'yes', scrolling => 'auto', marginwidth => '0', marginheight => '0', border => '1', name => $args{windowName} . "_top", bordercolor => 'black');
      $inner_content .= $doc->createFrame(src => "$site_dir/cgi-bin/report.cgi?command=bottom", frameborder => 'yes', scrolling => 'auto', name => $args{windowName} . "_bottom", marginwidth => '1', bordercolor => 'white');
      $doc->outerFrameset(rows => '*, *', content => $inner_content, border => '5', frameborder => 'yes');

      $doc->setTitle("Report - $args{'report_name'}");

      $doc->display();
    }
    else
    {
      $doc->print(<<"END_OF_BODY");
  <center>
  <h1>Customize <font color="red">$args{'report_name'}</font> Report</h1>
  <form method=post name="customize_detailed_form" action="$site_dir/cgi-bin/report_customization.cgi" target="_top">
    <input type=hidden name="data_precision" value="1">
    <input type=hidden name="command" value="run_report">
    <input type=hidden name="id" value="$args{'id'}">
    <input type=hidden name="ledger" value="$args{'ledger'}">
    <input type=hidden name="default_fg_color" value="$args{'default_fg_color'}">
    <input type=hidden name="default_bg_color" value="$args{'default_bg_color'}">
    <input type=hidden name="report_type" value="$args{'report_type'}">
    <input type=hidden name="fg_color" value="$args{'fg_color'}">
    <input type=hidden name="bg_color" value="$args{'bg_color'}">
    <input type=hidden name="text_font" value="$args{'text_font'}">
    <input type=hidden name="data_font" value="$args{'data_font'}">
    <input type=hidden name="text_size" value="$args{'text_size'}">
    <input type=hidden name="data_size" value="$args{'data_size'}">
    <input type=hidden name="underline_length" value="$args{'underline_length'}">
    <input type=hidden name="account_format" value="$args{'account_format'}">
    <input type=hidden name="negative_format" value="$args{'negative_format'}">
    <input type=hidden name="total_column" value="$t_or_f{$args{'total_column'}}">
    <input type=hidden name="headers" value="$args{'headers'}">
    <input type=hidden name="sorted_by" value="$args{'sorted_by'}">
    <input type=hidden name="sorted_order" value="$args{'sorted_order'}">
    <input type=hidden name="subtotal" value="$args{'subtotal'}">
    <input type=hidden name="subtotal_type" value="$args{'subtotal_type'}">
    <input type="hidden" name="windowName" value="$args{windowName}">
    <input type="hidden" name="reportWidth" value="$args{reportWidth}">
    <table width="75%" border=0 cellspacing=0 cellpadding=2 bgcolor="$menu_bgcolor">
      <tr>
        <th align=right><font color="$menu_fgcolor">Report&nbsp;Name:</font></th>
        <td align=left><input type=text name="report_name" value="$args{'report_name'}" size="30"></td>
      </tr>
      <tr>
        <td colspan=2><hr width="95%"></td>
      </tr>
      <tr>
        <td align=center colspan=2>
          <table width="100%" border=0 cellspacing=0 cellpadding=2 bgcolor="$menu_bgcolor">
            <tr>
              <th><font color="$menu_fgcolor">Connector</font></th>
              <th><font color="$menu_fgcolor">Field</font></th>
              <th><font color="$menu_fgcolor">Operator</font></th>
              <th><font color="$menu_fgcolor">&nbsp;</font></th>
              <th><font color="$menu_fgcolor">Value</font></th>
              <td><br></td>
            </tr>
            <tr>
              <td align=center valign=top><font size="-1" color="$menu_fgcolor"><select name="connector">
                <option value="">-----
              </select></font></td>
              <td align=center valign=top><font size="-1" color="$menu_fgcolor"><select name="field_name" onChange="updateFillin();">
END_OF_BODY
      foreach my $field_name (@field_names)
      {
        $doc->print("                <option value=\"$field_name\">$field_names{$field_name}\n");
      }
      $doc->print(<<"END_OF_BODY");
              </select></font></td>
              <td align=center valign=top><font size="-1" color="$menu_fgcolor"><select name="operator" onChange="document.customize_detailed_form.value_text.focus();">
END_OF_BODY
      foreach my $operator (@operator_list)
      {
        $doc->print("                <option value=\"$operator\">$operator_list{$operator}\n");
      }
      $doc->print(<<"END_OF_BODY");
              </select></font></td>
              <td align="center" valign="top"><font size="-1" color="$menu_fgcolor">
                <select name="fillin" onChange="handleFillinSelection();">
                  <option value="----">----</option>
                </select>
              </td>
              <td align=center valign=top><font size="-1" color="$menu_fgcolor"><input type=text name="value_text" value="" size=25></td>
            </tr>
          </table>
        </td>
      </tr>
      <tr>
        <td align=left valign=bottom><font color="$menu_fgcolor"><input type=checkbox onClick="bDebug = !bDebug; write_frame();">&nbsp;Display SQL statement</font></td>
        <td align=right valign=top><font color="$menu_fgcolor"><input type=button name="add_button" value="Add" onClick="add_statement();">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<input type=button name="clear_button" value="Clear" onClick="clear_statement(); document.customize_detailed_form.field_name.focus();">&nbsp;&nbsp;&nbsp;&nbsp;</font></td>
      </tr>
    </table>
    <input type=hidden name="sql_statement" value="">
    <input type=submit name="custom_submit" value="Run Report" onClick="return validate_form();">&nbsp;<input type=button value="Reset" onClick="document.customize_detailed_form.reset(); reset_form();">
  </form>
  </center>
END_OF_BODY
      my $onload_code = <<"END_OF_ONLOAD";
  clear_statement();

  build_sql_statement();

  write_frame();
END_OF_ONLOAD
      $doc->setOnload(code => $onload_code);

      $doc->setFocus("javascript");
      $doc->print(<<"END_OF_SCRIPT");

  var subEntityArray = new Array();
  var accountNameArray = new Array();
  var accountNumberArray = new Array();
  var subAccountNameArray = new Array();
  var subAccountNumberArray = new Array();
  var payeeArray = new Array();

END_OF_SCRIPT

      # now lookup all the necessary info and assign to the arrays.
      
      # get the sub-entities
      my @subEntities = GetSubEntities(dbh => $dbh, ledger_code => $ledger_code);
      my $counter = 0;
      foreach my $subEntity (@subEntities)
      {
        $doc->print("  subEntityArray[" . $counter++ . "] = '$subEntity->[0]';\n");
      }

      # get the accounts
      # name = 1124
      my $sth = $dbh->prepare( $sql_statements{1124} ) || my_db_die("Error: Report.cgi - customize_report_display", 4007, "SQL Statement = '$sql_statements{1124}'");
      my $rc = $sth->execute( $ledger_code ) || my_db_die("Error: Report.cgi - customzie_report_display", 4008, "SQL Statement = '$sql_statements{1124}'");
      $counter = 0;
      while (my @temp_row = $sth->fetchrow_array)
      {
        $doc->print("  accountNameArray[" . $counter++ . "] = '$temp_row[0]';\n");
      }
      $sth->finish;

      # number = 1125
      $sth = $dbh->prepare( $sql_statements{1125} ) || my_db_die("Error: Report.cgi - customize_report_display", 4007, "SQL Statement = '$sql_statements{1125}'");
      $rc = $sth->execute( $ledger_code ) || my_db_die("Error: Report.cgi - customzie_report_display", 4008, "SQL Statement = '$sql_statements{1125}'");
      $counter = 0;
      while (my @temp_row = $sth->fetchrow_array)
      {
        $doc->print("  accountNumberArray[" . $counter++ . "] = '$temp_row[0]';\n");
      }
      $sth->finish;

      # get the sub accounts
      # name = 1126
      $sth = $dbh->prepare( $sql_statements{1126} ) || my_db_die("Error: Report.cgi - customize_report_display", 4007, "SQL Statement = '$sql_statements{1126}'");
      $rc = $sth->execute() || my_db_die("Error: Report.cgi - customzie_report_display", 4008, "SQL Statement = '$sql_statements{1126}'");
      $counter = 0;
      while (my @temp_row = $sth->fetchrow_array)
      {
        $doc->print("  subAccountNameArray[" . $counter++ . "] = '$temp_row[0]';\n");
      }
      $sth->finish;

      # number = 1127
      $sth = $dbh->prepare( $sql_statements{1127} ) || my_db_die("Error: Report.cgi - customize_report_display", 4007, "SQL Statement = '$sql_statements{1127}'");
      $rc = $sth->execute() || my_db_die("Error: Report.cgi - customzie_report_display", 4008, "SQL Statement = '$sql_statements{1127}'");
      $counter = 0;
      while (my @temp_row = $sth->fetchrow_array)
      {
        $doc->print("  subAccountNumberArray[" . $counter++ . "] = '$temp_row[0]';\n");
      }
      $sth->finish;

      # get the payee-payer info
      # 1128
      $sth = $dbh->prepare( $sql_statements{1128} ) || my_db_die("Error: Report.cgi - customize_report_display", 4007, "SQL Statement = '$sql_statements{1128}'");
      $rc = $sth->execute( $ledger_code ) || my_db_die("Error: Report.cgi - customzie_report_display", 4008, "SQL Statement = '$sql_statements{1128}'");
      $counter = 0;
      while (my @temp_row = $sth->fetchrow_array)
      {
        $doc->print("  payeeArray[" . $counter++ . "] = '$temp_row[0]';\n");
      }
      $sth->finish;

      $doc->print(<<"END_OF_SCRIPT");

  var row_array = new Array();
  var bEditing = false;
  var editRow = -1;

  var bDebug = false;

  function updateFillin()
  {
    var f = document.customize_detailed_form;
    var i;

    var field = f.field_name.options[f.field_name.selectedIndex].value;

    if (f.field_name.selectedIndex < 0 || f.field_name.selectedIndex >= f.field_name.options.length)
    {
      alert("Error:  Field has not been selected!");
      return;
    }

    f.fillin.options.length = 0;
    f.fillin.options[0] = new Option("----", "----");
    f.fillin.selectedIndex = 0;

    if (field == 'account_name')
    {
      for (i=0; i < accountNameArray.length; i++)
      {
        f.fillin.options[i+1] = new Option(accountNameArray[i], accountNameArray[i]);
      }
    }
    else if (field == 'db.account_num')
    {
      for (i=0; i < accountNumberArray.length; i++)
      {
        f.fillin.options[i+1] = new Option(accountNumberArray[i], accountNumberArray[i]);
      }
    }
    else if (field == 'je.payee_payer')
    {
      for (i=0; i < payeeArray.length; i++)
      {
        f.fillin.options[i+1] = new Option(payeeArray[i], payeeArray[i]);
      }
    }
    else if (field == 'sub_account_name')
    {
      for (i=0; i < subAccountNameArray.length; i++)
      {
        f.fillin.options[i+1] = new Option(subAccountNameArray[i], subAccountNameArray[i]);
      }
    }
    else if (field == 'db.sub_account_num')
    {
      for (i=0; i < subAccountNumberArray.length; i++)
      {
        f.fillin.options[i+1] = new Option(subAccountNumberArray[i], subAccountNumberArray[i]);
      }
    }
    else if (field == 'db.sub_e_code')
    {
      for (i=0; i < subEntityArray.length; i++)
      {
        f.fillin.options[i+1] = new Option(subEntityArray[i], subEntityArray[i]);
      }
    }
  }

  function handleFillinSelection()
  {
    var f = document.customize_detailed_form;

    var value = f.fillin.options[f.fillin.selectedIndex].value;

    if (f.fillin.selectedIndex < 0 || f.fillin.selectedIndex >= f.fillin.options.length)
    {
      alert("Error:  Fillin value has not been selected!");
      return;
    }
    if (f.fillin.selectedIndex == 0)
    {
      alert("You can not use the ---- value!  :)");
      return;
    }

    f.value_text.value = value;
  }

  function add_statement()
  {
    var f = document.customize_detailed_form;

    // assign the values to the next available location in the array or to the passed in value if editing
    if (bEditing)
    {
      var fieldText = f.field_name.options[f.field_name.selectedIndex].text;
      if (f.value_text.value.length == 0 && fieldText != "Payee-Payer" && fieldText != "Memo" && fieldText != "Sub-Entity Code")
      {
        alert("You need to specify a value for the statement to be valid!\\nbEditing = true");
        f.value_text.focus();
        return;
      }
      else
      {
        row_array[editRow]['connector']['text'] = f.connector.options[f.connector.selectedIndex].text;
        row_array[editRow]['connector']['value'] = f.connector.options[f.connector.selectedIndex].value;
        row_array[editRow]['field']['text'] = f.field_name.options[f.field_name.selectedIndex].text;
        row_array[editRow]['field']['value'] = f.field_name.options[f.field_name.selectedIndex].value;
        row_array[editRow]['operator']['text'] = f.operator.options[f.operator.selectedIndex].text;
        row_array[editRow]['operator']['value'] = f.operator.options[f.operator.selectedIndex].value;
        row_array[editRow]['value'] = f.value_text.value;

        bEditing = false;

        build_sql_statement();

        clear_statement();

        f.field_name.focus();
      }
    }
    else
    {
      var fieldText = f.field_name.options[f.field_name.selectedIndex].text;
      if (f.value_text.value.length == 0 && fieldText != "Payee-Payer" && fieldText != "Memo" && fieldText != "Sub-Entity Code")
      {
        alert("You need to specify a value for the statement to be valid!");
        f.value_text.focus();
        return;
      }
      else
      {
        var newElement = row_array.length;

        row_array[newElement] = new Array();
        row_array[newElement]['connector'] = new Array();
        row_array[newElement]['field'] = new Array();
        row_array[newElement]['operator'] = new Array();

        row_array[newElement]['connector']['text'] = f.connector.options[f.connector.selectedIndex].text;
        row_array[newElement]['connector']['value'] = f.connector.options[f.connector.selectedIndex].value;
        row_array[newElement]['field']['text'] = f.field_name.options[f.field_name.selectedIndex].text;
        row_array[newElement]['field']['value'] = f.field_name.options[f.field_name.selectedIndex].value;
        row_array[newElement]['operator']['text'] = f.operator.options[f.operator.selectedIndex].text;
        row_array[newElement]['operator']['value'] = f.operator.options[f.operator.selectedIndex].value;
        row_array[newElement]['value'] = f.value_text.value;
      }

      // rewrite the connector select to remove the blank entry and add the connector entries
      f.connector.options.length = 0;
END_OF_SCRIPT
      foreach my $connector (@connector_list)
      {
        $doc->print("      f.connector.options[f.connector.options.length] = new Option(\"$connector\", \"$connector\");\n");
      }
      $doc->print(<<"END_OF_SCRIPT");
      f.connector.selectedIndex = 0;

      clear_statement();

      build_sql_statement();

      write_frame();

      f.field_name.focus();
    }
  }

  function clear_statement()
  {
    var f = document.customize_detailed_form;

    // reset the row to defaults
    f.field_name.selectedIndex = 0;
    f.operator.selectedIndex = 0;
    f.value_text.value = "";
    f.connector.selectedIndex = 0;

    bEditing = false;

    if (row_array.length == 0)
    {
      // rewrite the connector select to add the blank entry and remove the connector entries
      f.connector.options.length = 0;
      f.connector.options[0] = new Option("-----", "");
      f.connector.selectedIndex = 0;
    }

    if (f.clear_button.value == "Cancel")
    {
      f.add_button.value = "Add";
      f.clear_button.value = "Clear";
      // rewrite the connector select to remove the blank entry and add the connector entries
      f.connector.options.length = 0;
END_OF_SCRIPT
      foreach my $connector (@connector_list)
      {
        $doc->print("      f.connector.options[f.connector.options.length] = new Option(\"$connector\", \"$connector\");\n");
      }
      $doc->print(<<"END_OF_SCRIPT");
      f.connector.selectedIndex = 0;

      write_frame();
    }
    
    updateFillin();  // update the fillin combo box.
  }

  function write_frame()
  {
    var f = document.customize_detailed_form;
    var bodyString = '';

    bodyString += '<html>\\n';
    bodyString += '<body bgcolor=\"$body_bgcolor\">\\n';
    bodyString += '<form onSubmit=\"return false;\">\\n';
    bodyString += '<table width=\"95%\" border=0 cellspacing=0 cellpadding=0>\\n';
    bodyString += '  <tr>\\n';
    if (row_array.length > 1)
    {
      bodyString += '    <th>Connector</th>\\n';
    }
    bodyString += '    <th>Field</th>\\n';
    bodyString += '    <th>Operator</th>\\n';
    bodyString += '    <th>Value</th>\\n';
    bodyString += '    <td><br></td>\\n';
    bodyString += '    <td><br></td>\\n';
    bodyString += '  </tr>\\n';
    for (var count=0; count < row_array.length; count++)
    {
      bodyString += '  <tr>\\n';
      if (row_array.length > 1 && count > 0)
      {
        bodyString += '    <td align=center>' + row_array[count]['connector']['text'] + '</td>\\n';
      }
      else
      {
        bodyString += '    <td><br></td>\\n';
      }
      bodyString += '    <td align=center>' + row_array[count]['field']['text'] + '</td>\\n';
      bodyString += '    <td align=center>' + row_array[count]['operator']['text'] + '</td>\\n';
      bodyString += '    <td align=center>' + row_array[count]['value'] + '</td>\\n';
      if (bEditing == true)
      {
        bodyString += '    <td><br></td>\\n';
        bodyString += '    <td><br></td>\\n';
      }
      else
      {
        bodyString += '    <td align=center><input type=button value=\"Edit\" onClick=\"window.opener.edit_statement(' + count + ');\"></td>\\n';
        bodyString += '    <td align=center><input type=button value=\"Delete\" onClick=\"window.opener.delete_statement(' + count + ');\"></td>\\n';
      }
      bodyString += '  </tr>\\n';
    }
    bodyString += '</table>\\n';
    bodyString += '</form>\\n';
    if (bDebug == true)
    {
      bodyString += '<br><br>\\n';
      bodyString += 'SQL statement: \\"' + f.sql_statement.value + '\\"\\n';
    }
    bodyString += '</body>\\n';
    bodyString += '</html>\\n';

    bottomFrame = window.open("", "$args{windowName}_bottom");
    bottomFrame.document.open();
    bottomFrame.document.writeln(bodyString);
    bottomFrame.document.close();
  }

  function edit_statement(n)
  {
    var f = document.customize_detailed_form;

    if (n == 0)
    {
      // rewrite the connector select to add the blank entry and remove the connector entries
      f.connector.options.length = 0;
      f.connector.options[0] = new Option("-----", "");
      f.connector.selectedIndex = 0;
    }

    for (var count=0; count < f.connector.options.length; count++)
    {
      if (f.connector.options[count].value == row_array[n]['connector']['value'])
      {
        f.connector.selectedIndex = count;
      }
    }
    for (var count=0; count < f.field_name.options.length; count++)
    {
      if (f.field_name.options[count].value == row_array[n]['field']['value'])
      {
        f.field_name.selectedIndex = count;
      }
    }
    for (var count=0; count < f.operator.options.length; count++)
    {
      if (f.operator.options[count].value == row_array[n]['operator']['value'])
      {
        f.operator.selectedIndex = count;
      }
    }
    f.value_text.value = row_array[n]['value'];

    bEditing = true;
    editRow = n;

    f.add_button.value = "Update";
    f.clear_button.value = "Cancel";

    write_frame();

    f.field_name.focus();
  }

  function delete_statement(n)
  {
    var f = document.customize_detailed_form;

    for (var count=n + 1; count < row_array.length; count++)
    {
      row_array[count - 1]['connector']['text'] = row_array[count]['connector']['text'];
      row_array[count - 1]['connector']['value'] = row_array[count]['connector']['value'];
      row_array[count - 1]['field']['text'] = row_array[count]['field']['text'];
      row_array[count - 1]['field']['value'] = row_array[count]['field']['value'];
      row_array[count - 1]['operator']['text'] = row_array[count]['operator']['text'];
      row_array[count - 1]['operator']['value'] = row_array[count]['operator']['value'];
      row_array[count - 1]['value'] = row_array[count]['value'];
    }
    row_array.length--;
    if (row_array.length > 0)
    {
      // delete the connector field for the first row
      row_array[0]['connector']['text'] = "-----";
      row_array[0]['connector']['value'] = "";
    }

    if (row_array.length <= 0)
    {
      // rewrite the connector select to add the blank entry and remove the connector entries
      f.connector.options.length = 0;
      f.connector.options[0] = new Option("-----", "");
      f.connector.selectedIndex = 0;
    }

    build_sql_statement();

    write_frame();

    f.field_name.focus();
  }

  function build_sql_statement()
  {
    var f = document.customize_detailed_form;

    f.sql_statement.value = "";

    for (var count=0; count < row_array.length; count++)
    {
      var beforeString = '';
      var afterString = '';

      f.sql_statement.value += ' ';
      f.sql_statement.value += row_array[count]['connector']['value'];
      f.sql_statement.value += ' ';

      // replace any \"?\"\'s and \"*\"\'s with \"_\"\'s and \"%\"\'s
      var rowValue = row_array[count]['value'];
      rowValue = rowValue.replace(/\\?/g, "_");
      rowValue = rowValue.replace(/\\*/g, "%");

      switch (row_array[count]['field']['value'])
      {
        case "account_name"    : var sqlTemp = "$sql_translations{'account_name'}";
                                 sqlTemp = sqlTemp.replace("<operator>", row_array[count]['operator']['value']);
                                 sqlTemp = sqlTemp.replace("<value>", "'" + rowValue + "'");
                                 f.sql_statement.value += sqlTemp;
                                 break;
        case "sub_account_name": var sqlTemp = "$sql_translations{'sub_account_name'}";
                                 sqlTemp = sqlTemp.replace("<operator>", row_array[count]['operator']['value']);
                                 sqlTemp = sqlTemp.replace("<value>", "'" + rowValue + "'");
                                 f.sql_statement.value += sqlTemp;
                                 break;
        case "credit_amount"   : var sqlTemp = "$sql_translations{'credit_amount'}";
                                 sqlTemp = sqlTemp.replace("<operator>", row_array[count]['operator']['value']);
                                 sqlTemp = sqlTemp.replace("<value>", "'" + rowValue + "'");
                                 f.sql_statement.value += sqlTemp;
                                 break;
        case "debit_amount"    : var sqlTemp = "$sql_translations{'debit_amount'}";
                                 sqlTemp = sqlTemp.replace("<operator>", row_array[count]['operator']['value']);
                                 sqlTemp = sqlTemp.replace("<value>", "'" + rowValue + "'");
                                 f.sql_statement.value += sqlTemp;
                                 break;
        case "je.journal_code" :
        case "je.trans_date"   :
        case "je.payee_payer"  :
        case "je.memo"         :
        case "db.amount"       :
        case "db.sub_e_code"   :
        case "db.cleared"      : beforeString = "'";
                                 afterString = "'";
                                 f.sql_statement.value += row_array[count]['field']['value'];
                                 f.sql_statement.value += ' ';
                                 f.sql_statement.value += row_array[count]['operator']['value'];
                                 f.sql_statement.value += ' ';
                                 f.sql_statement.value += beforeString + rowValue + afterString;
                                 break;
        default                : beforeString = "";
                                 afterString = "";
                                 f.sql_statement.value += row_array[count]['field']['value'];
                                 f.sql_statement.value += ' ';
                                 f.sql_statement.value += row_array[count]['operator']['value'];
                                 f.sql_statement.value += ' ';
                                 f.sql_statement.value += beforeString + rowValue + afterString;
                                 break;
      }
    }
  }

  function reset_form()
  {
    var f = document.customize_detailed_form;

    bDebug = false;

    // clear out the row_array
    row_array.length = 0;

    // rewrite the connector select to add the blank entry and remove the connector entries
    f.connector.options.length = 0;
    f.connector.options[0] = new Option("-----", "");
    f.connector.selectedIndex = 0;

    build_sql_statement();

    write_frame();

    f.field_name.focus();
  }

  function validate_form()
  {
    var f = document.customize_detailed_form;

    if (f.sql_statement.value == "")
    {
      alert("SQL statement is empty!\\nYou must enter at least one filter criteria.\\n");
      f.field_name.focus();
      return false;
    }

    return true;
  }

END_OF_SCRIPT
    }
  }

  $doc->display();
}

# memorized_report_display
# takes  : dbh, attributes to be resent for memorization, error_string (optional), question_string (optional)
# returns: nothing
sub memorized_report_display
{
  my %args = ( @_, );
  my $dbh = $args{'dbh'};
  my $report_type = $args{'report_type'};
  my $error_string = $args{'error_string'};
  my $question_string = $args{'question_string'};
  my @arguments = ();
  if ($report_type eq "r")
  {
    @arguments = ("id", "ledger", "ledger_code", "ledger_color", "report_class", "report_type", "report_name", "dates_entered", "fg_color", "bg_color", "underline_length", "data_precision", "negative_format", "text_size", "text_font", "data_size", "data_font", "account_format", "total_column", "column_data", "column_type", "column_variable", "column_variable_data", "reportWidth");
  }
  else
  {
    @arguments = ("id", "ledger", "ledger_code", "ledger_color", "report_class", "report_type", "report_name", "fg_color", "bg_color", "negative_format", "text_size", "text_font", "data_size", "data_font", "account_format", "sql_statement", "headers", "sorted_by", "sorted_order", "subtotal", "subtotal_type", "data_precision", "reportWidth");
  }

  eval ("use HTMLObject::Normal;");
  if ($@)
  {
    my_die("Error: Report.cgi - Eval", 4002, "Error: $@ at line " . __LINE__);
  }

  my $doc = HTMLObject::Normal->new();

  $doc->setJavascriptErrorHandlerEmail(email => $email_address);

  if (!exists $args{'dbh'})
  {
    $doc->doRequiredParameterError('memorized_report_display', "dbh");
  }

  # validate we have all the values we need, just in case.
  foreach my $name (@arguments)
  {
    if (!exists $args{$name})
    {
      $doc->doRequiredParameterError('memorized_report_display', $name);
    }
  }

  # generate the form that requests them to re-enter the memorized report name.
  $doc->setTitle("Memorize Report Error");
  $doc->setFocus("body");
  if (length $error_string > 0)
  {
  $doc->print(<<"END_OF_BODY");
  <h2><font color="red">Error</font>: &nbsp;$error_string</h2>
END_OF_BODY
  }
  $doc->print(<<"END_OF_BODY");
  <br>
  <center>
  <h1>Memorize Report</h1>
  <form name="memorize_report" method="post" action="$site_dir/cgi-bin/report_customization.cgi">
  <input type=hidden name="command" value="memorize_report">
END_OF_BODY
  foreach my $name (@arguments)
  {
    if ($name ne "report_name")
    {
      $doc->print("  <input type=hidden name=\"$name\" value=\"$args{$name}\">\n");
    }
  }

  if (length $question_string == 0)
  {
  $doc->print(<<"END_OF_BODY");

    <table border=0 cellpadding=2 cellspacing=0 bgcolor="$menu_bgcolor">
      <tr>
        <td align=right>Report Name:&nbsp;</td>
        <td align=left><input type=text name="report_name" value="$args{'report_name'}"></td>
      </tr>
    </table>
    <input type=submit name="memorize" value="Memorize">&nbsp;&nbsp;<input type=reset value="Reset">
    </center>
  </form>
END_OF_BODY
  }
  else
  {
    $doc->print(<<"END_OF_BODY");
    <input type=hidden name="report_name" value="$args{'report_name'}">
    <table border=0 cellpadding=2 cellspacing=0 bgcolor="$menu_bgcolor">
      <tr>
        <td colspan=2 align=center>Memorized Report <b>$args{'report_name'}</b> already exists for this report!</td>
      </tr>
      <tr>
        <td align=right>$question_string&nbsp;</td>
        <td align=left><input type=radio name="overwrite" value="yes" checked>&nbsp;Yes<br><input type=radio name="overwrite" value="no">&nbsp;No</td>
      </tr>
    </table>
    <input type=submit name="confirm" value="Confirm">&nbsp;&nbsp;<input type=reset value="Reset">
  </form>
END_OF_BODY
  }

  $doc->display();
}

# takes Title, Error Code, Custom Message
sub my_die
{
  my ($title, $code, $message) = @_;

  # close the database connection.
  #if (defined $dbh)
  #{
  #  $dbh->disconnect;
  #}

  my $doc = HTMLObject::Base->new();
  $doc->setErrorMessage(code => $code, message => $errors{$code});
  $doc->setError(code => $code);
  $doc->displayError(title => $title, message => "$message");
}

# takes Title, Error Code, Custom Message
sub my_db_die
{
  my ($title, $code, $message) = @_;
  my $error_str = $DBI::errstr;

  # close the database connection.
  #if (defined $dbh)
  #{
  #  $dbh->disconnect;
  #}

  my $doc = HTMLObject::Base->new();
  $doc->setErrorMessage(code => $code, message => $errors{$code});
  $doc->setError(code => $code);
  $doc->displayError(title => $title, message => "$message<br><br>\n$error_str");
}

###################### Data Gathering Functions converted from reports.pm to DBI structure #############################

# get_ledger_name
# Takes  : ledger_db name, handle to database connection
# Returns: ledger name
sub get_ledger_name
{
  my %args = ( @_, );
  my $ledger_db = $args{'ledger_db'};
  my $dbh = $args{'dbh'};
  my $ledger_name = "";

  eval ("use HTMLObject::Normal;");
  if ($@)
  {
    my_die("Error: Report.cgi - Eval", 4002, "Error: $@ at line " . __LINE__);
  }

  # check to make sure we were given the ledger_db name and database handle
  if (length $ledger_db == 0)
  {
    my_die("Error: Report.cgi - get_ledger_name", 4026, "The Ledger DB Name is required at line " . __LINE__);
  }

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - get_ledger_name", 4028, "The DB Handle is required at line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1016} ) || my_db_die("Error: Report.cgi - get_ledger_name", 4007, "SQL Statement = '$sql_statements{1016}'");
  my $rc = $sth->execute($ledger_db) || my_db_die("Error: Report.cgi - get_ledger_name", 4008, "SQL Statement = '$sql_statements{1016}', Ledger DB = '$ledger_db'");
  my @info = $sth->fetchrow_array;

  if (!defined $info[0])
  {
    my_die("Error: Report.cgi - get_ledger_name", 4023, "Ledger DB = '$ledger_db'");
  }
  else
  {
    $ledger_name = $info[0];
  }
  $sth->finish;

  return $ledger_name;
}

# get_ledger_code
# Takes  : ledger_db name, handle to database connection
# Returns: ledger code
sub get_ledger_code
{
  my %args = ( @_, );
  my $ledger_name = $args{'ledger_name'};
  my $dbh = $args{'dbh'};
  my $ledger_code = "";

  # check to make sure we were given the ledger name and database handle
  if (length $ledger_name == 0)
  {
    my_die("Error: Report.cgi - get_ledger_code", 4027, "The Ledger Name is required at line " . __LINE__);
  }

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - get_ledger_code", 4028, "The DB Handle is required at line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1017} ) || my_db_die("Error: Report.cgi - get_ledger_code", 4007, "SQL Statement = '$sql_statements{1017}'");
  my $rc = $sth->execute($ledger_name) || my_db_die("Error: Report.cgi - get_ledger_code", 4008, "SQL Statement = '$sql_statements{1017}', Ledger Name = '$ledger_name'");
  my @info = $sth->fetchrow_array;

  if (!defined $info[0])
  {
    my_die("Error: Report.cgi - get_ledger_code", 4024, "Ledger Name = '$ledger_name'");
  }
  else
  {
    $ledger_code = $info[0];
  }
  $sth->finish;

  return $ledger_code;
}

# get_ledger_color
# Takes  : ledger_db name, handle to database connection
# Returns: ledger color
sub get_ledger_color
{
  my %args = ( @_, );
  my $ledger_code = $args{'ledger_code'};
  my $dbh = $args{'dbh'};
  my $ledger_color = "";

  # check to make sure we were given the ledger code and database handle
  if (length $ledger_code == 0)
  {
    my_die("Error: Report.cgi - get_ledger_color", 4022, "The Ledger Code is required at line " . __LINE__);
  }

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - get_ledger_color", 4028, "The DB Handle is required at line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1018} ) || my_db_die("Error: Report.cgi - get_ledger_color", 4007, "SQL Statement = '$sql_statements{1018}'");
  my $rc = $sth->execute($ledger_code) || my_db_die("Error: Report.cgi - get_ledger_color", 4008, "SQL Statement = '$sql_statements{1018}', Ledger Code = '$ledger_code'");
  my @info = $sth->fetchrow_array;

  if (!defined $info[0])
  {
    my_die("Error: Report.cgi - get_ledger_color", 4025, "Ledger Name = '$ledger_code'");
  }
  else
  {
    $ledger_color = $info[0];
  }
  $sth->finish;

  return $ledger_color;
}

# Takes  : dbh, id of report
# Returns: "" if not found, report name if found
sub DoesReportExist
{
  my %args = ( @_ );
  my $dbh = $args{'dbh'};
  my $id = $args{'id'};
  my $result = "";  # assume it is not found.

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - DoesReportExist", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $id == 0)
  {
    my_die("Error: Report.cgi - DoesReportExist", 4034, "The Report ID is required at line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1005} ) || my_db_die("Error: Report.cgi - DoesReportExist", 4007, "SQL Statement = '$sql_statements{1005}'");
  my $rc = $sth->execute($id) || my_db_die("Error: Report.cgi - DoesReportExist", 4008, "SQL Statement = '$sql_statements{1005}', id = '$id'");
  my @info = $sth->fetchrow_array;

  if (defined $info[0])
  {
    $result = $info[0];
  }
  $sth->finish;

  return $result;
}

# Takes  : dbh, id of report
# Returns: array of arrays where each sub-array is the data for a row in the specified report.
sub GetReportRows
{
  my %args = ( @_ );
  my $dbh = $args{'dbh'};
  my $id = $args{'id'};

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetReportRows", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $id == 0)
  {
    my_die("Error: Report.cgi - GetReportRows", 4034, "The Report ID is required at line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1024} ) || my_db_die("Error: Report.cgi - GetReportRows", 4007, "SQL Statement = '$sql_statements{1024}'");
  my $rc = $sth->execute($id) || my_db_die("Error: Report.cgi - GetReportRows", 4008, "SQL Statement = '$sql_statements{1024}', id = '$id'");
  my $ref = $sth->fetchall_arrayref;
  if ($sth->err)
  {
    my_db_die("Error:  Report.cgi - GetReportRows", 4009, "Getting all Account Types failed!");
  }
  my @info = @{$ref};

  $sth->finish;

  # now walk the array and make hashes for each row that detail the items and their values.
  my @temp_array = ();
  for (my $i=0; $i < scalar @info; $i++)
  {
    $temp_array[$i] = ();  # make an empty array placeholder.
    my %temp_hash = {};
    $temp_hash{'index'} = $info[$i][0];
    $temp_hash{'cell_text'} = $info[$i][1];
    $temp_hash{'cell_type'} = $info[$i][2];
    $temp_hash{'item_name'} = $info[$i][3];
    $temp_hash{'fg_color'} = $info[$i][4];
    $temp_hash{'bg_color'} = $info[$i][5];
    $temp_hash{'display_level'} = $info[$i][6];
    $temp_hash{'initial_state'} = $info[$i][7];
    $temp_hash{'negative_format'} = $info[$i][8];
    my $other_attributes = $info[$i][9];
    $temp_hash{'other_attributes'} = $other_attributes;
    my @other_attributes = split /\|/, $other_attributes;
    foreach my $item (@other_attributes)
    {
      (my $name, my $value) = split /=/, $item, 2;
      $temp_hash{$name} = $value;
    }

    my $cell_type = $temp_hash{'cell_type'};
    if ($cell_type eq "text" || $cell_type eq "line" || $cell_type eq "detail")
    {
      $temp_hash{'cell_formula'} = "";
      $temp_hash{'sub_levels'} = "-1";
      $temp_hash{'drillable'} = 0;
      $temp_hash{'drill_report'} = -1;
      if ($cell_type ne "detail")
      {
        $temp_hash{'activity'} = 1;  # activity
      }
      else
      {
        $temp_hash{'activity'} = 0;  # balances
      }
      $temp_hash{'query_type'} = 0;  # everything
      $temp_hash{'show'} = 1;  # show this row.
      $temp_hash{'underline'} = 0; # don't need to underline the previous row.
    }
    elsif ($cell_type eq "total" || $cell_type eq "compare")
    {
      $temp_hash{'sub_levels'} = "-1";
      $temp_hash{'drillable'} = 0;
      $temp_hash{'drill_report'} = -1;
      $temp_hash{'activity'} = 1;  # activity
      $temp_hash{'query_type'} = 0;  # everything
      if (!exists $temp_hash{'underline'})
      {
        if ($cell_type eq "total")
        {
          $temp_hash{'underline'} = 1;  # Default to underline the previous row.
        }
        elsif ($cell_type eq "compare")
        {
          $temp_hash{'underline'} = 0;  # Default to not underline the previous row.
        }
      }
    }
    else
    {
      $temp_hash{'underline'} = 0;
    }
    $temp_array[$i] = { %temp_hash };
  }

  return @temp_array;
}

# GetAccountTypeNames
# Takes  : Array of Account Types (account_types), dbh
# Returns: Array of Account Type Names in order given.
sub GetAccountTypeNames
{
  my %args = ( @_ );
  my $dbh = $args{'dbh'};
  my @account_types = @{$args{'account_types'}};
  my @account_type_names = ();

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetAccountTypeNames", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (scalar @account_types == 0)
  {
    my_die("Error: Report.cgi - GetAccountTypeNames", 4034, "The Array of Account Types is required at line " . __LINE__);
  }

  foreach my $account_type (@account_types)
  {
    my $sth = $dbh->prepare( $sql_statements{1042} ) || my_db_die("Error: Report.cgi - GetAccountTypeNames", 4007, "SQL Statement = '$sql_statements{1042}'<br>\nAt Line " . __LINE__);
    my $rc = $sth->execute($account_type) || my_db_die("Error: Report.cgi - GetAccountTypeNames", 4008, "SQL Statement = '$sql_statements{1042}', Account Type = '$account_type'<br>\nAt Line " . __LINE__);
    my @info = $sth->fetchrow_array;

    if (!defined $info[0])
    {
      my_db_die("Error:  Report.cgi - GetAccountTypeNames", 4009, "No Account Type Name returned for Account Type = '$account_type'!<br>\nAt Line " . __LINE__);
    }
    $sth->finish;
    push(@account_type_names, $info[0]);
  }

  return @account_type_names;
}

# GetAccountType
# Takes  : Account Number (acct_number), dbh
# Returns: Account Type Number associated with the Account Number, debit_increases flag
sub GetAccountType
{
  my %args = ( @_ );
  my $dbh = $args{'dbh'};
  my $acct_number = $args{'acct_number'};
  my @acct_type_info = ();

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetAccountType", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $acct_number == 0)
  {
    my_die("Error: Report.cgi - GetAccountType", 4034, "The Account Number is required at line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1056} ) || my_db_die("Error: Report.cgi - GetAccountType", 4007, "SQL Statement = '$sql_statements{1056}'<br>\nAt Line " . __LINE__);
  my $rc = $sth->execute($acct_number) || my_db_die("Error: Report.cgi - GetAccountType", 4008, "SQL Statement = '$sql_statements{1056}', Account = '$acct_number'<br>\nAt Line " . __LINE__);
  my @info = $sth->fetchrow_array;
  $sth->finish;

  if (!defined $info[0])
  {
    my_db_die("Error:  Report.cgi - GetAccountType", 4009, "No Account Type returned for Account = '$acct_number'!<br>\nAt Line " . __LINE__);
  }
  $acct_type_info[0] = $info[0];
  $acct_type_info[1] = $info[1];

  return @acct_type_info;
}

# GetAccountName
# Takes  : Account Number (acct_number), dbh
# Returns: Account Name
sub GetAccountName
{
  my %args = ( @_ );
  my $dbh = $args{'dbh'};
  my $acct_number = $args{'acct_number'};
  my $acct_name = "";

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetAccountName", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $acct_number == 0)
  {
    my_die("Error: Report.cgi - GetAccountName", 4034, "The Account Number is required at line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1048} ) || my_db_die("Error: Report.cgi - GetAccountName", 4007, "SQL Statement = '$sql_statements{1048}'<br>\nAt Line " . __LINE__);
  my $rc = $sth->execute($acct_number) || my_db_die("Error: Report.cgi - GetAccountName", 4008, "SQL Statement = '$sql_statements{1048}', Account = '$acct_number'<br>\nAt Line " . __LINE__);
  my @info = $sth->fetchrow_array;
  $sth->finish;

  if (!defined $info[0])
  {
    my_db_die("Error:  Report.cgi - GetAccountName", 4009, "No Account Name returned for Account = '$acct_number'!<br>\nAt Line " . __LINE__);
  }
  $acct_name = $info[0];

  return $acct_name;
}

# GetSubAccountName
# Takes  : Sub Account Number (sub_acct_number), dbh
# Returns: Sub Account Name
sub GetSubAccountName
{
  my %args = ( @_ );
  my $dbh = $args{'dbh'};
  my $sub_acct_number = $args{'sub_acct_number'};
  my $sub_acct_name = "";

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetSubAccountName", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $sub_acct_number == 0)
  {
    my_die("Error: Report.cgi - GetSubAccountName", 4034, "The Sub Account Number is required at line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1057} ) || my_db_die("Error: Report.cgi - GetSubAccountName", 4007, "SQL Statement = '$sql_statements{1057}'<br>\nAt Line " . __LINE__);
  my $rc = $sth->execute($sub_acct_number) || my_db_die("Error: Report.cgi - GetSubAccountName", 4008, "SQL Statement = '$sql_statements{1057}', Sub Account = '$sub_acct_number'<br>\nAt Line " . __LINE__);
  my @info = $sth->fetchrow_array;
  $sth->finish;

  if (!defined $info[0])
  {
    my_db_die("Error:  Report.cgi - GetSubAccountName", 4009, "No Account Name returned for Account = '$sub_acct_number'!<br>\nAt Line " . __LINE__);
  }
  $sub_acct_name = $info[0];

  return $sub_acct_name;
}

# GetLocalAccounts
# Takes  : Array of Account Types (account_types), dbh, ledger_code
# Takes  : Taxable Flag (Optional, if not specified assumed to be 0) (0=don't care, 1=taxable only, 2=non-taxable only)
# Returns: Array of Local Accounts for each Account Type with the Debit Increases Flag for each Account Type and the Account Name
#          Array [AccountType]
#          [AccountType] = [LocalAccounts]
#          [LocalAccounts] = Account Number, Debit Increases Flag, Account Name
sub GetLocalAccounts
{
  my %args = ( taxable => '0', @_ );
  my $dbh = $args{'dbh'};
  my @account_types = @{$args{'account_types'}};
  my $taxable = $args{'taxable'};
  my $ledger_code = $args{'ledger_code'};
  my @local_accounts = ();

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetLocalAccounts", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (scalar @account_types == 0)
  {
    my_die("Error: Report.cgi - GetLocalAccounts", 4034, "The Array of Account Types is required at line " . __LINE__);
  }

  if ($taxable < 0 || $taxable > 2)
  {
    my_die("Error:  Report.cgi - GetLocalAccounts", 4034, "The Taxable flag can only be 0, 1, or 2!&nbsp;&nbsp;'$taxable' is invalid!<br>\nAt Line " . __LINE__);
  }

  if (length $ledger_code == 0)
  {
    my_die("Error:  Report.cgi - GetLocalAccounts", 4034, "The ledger_code must be specified!<br>\nAt Line " . __LINE__);
  }

  for(my $i=0; $i < scalar @account_types; $i++)
  {
    my $account_type = $account_types[$i];
    my @Info = ();

    my $sth = $dbh->prepare( $sql_statements{1043} ) || my_db_die("Error: Report.cgi - GetLocalAccounts", 4007, "SQL Statement = '$sql_statements{1043}'<br>\nAt Line " . __LINE__);
    my $rc = $sth->execute($account_type) || my_db_die("Error: Report.cgi - GetLocalAccounts", 4008, "SQL Statement = '$sql_statements{1043}', Account Type = '$account_type'<br>\nAt Line " . __LINE__);
    my @info = $sth->fetchrow_array;
    $sth->finish;

    if (!defined $info[0])
    {
      my_db_die("Error:  Report.cgi - GetLocalAccounts", 4009, "No Debit Increases Flag returned for Account Type = '$account_type'!<br>\nAt Line " . __LINE__);
    }
    my $debit_increases = $info[0];

    $sth = $dbh->prepare( $sql_statements{1044} ) || my_db_die("Error: Report.cgi - GetLocalAccounts", 4007, "SQL Statement = '$sql_statements{1044}'<br>\nAt Line " . __LINE__);
    $rc = $sth->execute($account_type) || my_db_die("Error: Report.cgi - GetLocalAccounts", 4008, "SQL Statement = '$sql_statements{1044}', Account Type = '$account_type'<br>\nAt Line " . __LINE__);
    @info = @{$sth->fetchall_arrayref};
    $sth->finish;

    if (scalar @info > 0)
    {
      my ($acct_num_list, $j, $acct_number, $acct_name);
      for ($j=0; $j < scalar @info; $j++)
      {
        $acct_number = $info[$j][0];

        $acct_num_list .= ", " if ($j > 0);
        $acct_num_list .= $acct_number;
      }

      # now get the accounts that exist in the local_accounts_tb that match up
      if ($taxable == 1 && $account_type != 15)
      {
        my $tmp_sql_statement;
        ($tmp_sql_statement = $sql_statements{1045}) =~ s/$substitutionRE/$acct_num_list/;
        $sth = $dbh->prepare( $tmp_sql_statement ) || my_db_die("Error: Report.cgi - GetLocalAccounts", 4007, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);
        $rc = $sth->execute($ledger_code) || my_db_die("Error: Report.cgi - GetLocalAccounts", 4008, "SQL Statement = '$tmp_sql_statement', Ledger Code = '$ledger_code', Account Num In '$acct_num_list'<br>\nAt Line " . __LINE__);
      }
      elsif ($taxable == 2)
      {
        my $tmp_sql_statement;
        ($tmp_sql_statement = $sql_statements{1046}) =~ s/$substitutionRE/$acct_num_list/;
        $sth = $dbh->prepare( $tmp_sql_statement ) || my_db_die("Error: Report.cgi - GetLocalAccounts", 4007, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);
        $rc = $sth->execute($ledger_code) || my_db_die("Error: Report.cgi - GetLocalAccounts", 4008, "SQL Statement = '$tmp_sql_statement', Ledger Code = '$ledger_code', Account Num In '$acct_num_list'<br>\nAt Line " . __LINE__);
      }
      else
      {
        my $tmp_sql_statement;
        ($tmp_sql_statement = $sql_statements{1047}) =~ s/$substitutionRE/$acct_num_list/;
        $sth = $dbh->prepare( $tmp_sql_statement ) || my_db_die("Error: Report.cgi - GetLocalAccounts", 4007, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);
        $rc = $sth->execute($ledger_code) || my_db_die("Error: Report.cgi - GetLocalAccounts", 4008, "SQL Statement = '$tmp_sql_statement', Ledger Code = '$ledger_code', Account Num In '$acct_num_list'<br>\nAt Line " . __LINE__);
      }
      my @accounts = @{$sth->fetchall_arrayref};
      $sth->finish;

      # now we walk the list and create the array that will make up this Account Type's entry
      for ($j=0; $j < scalar @accounts; $j++)
      {
        my @element = ();
        $acct_number = $accounts[$j][0];

        # get this accounts name
        $acct_name = GetAccountName(dbh => $dbh, acct_number => $acct_number);

        $element[0] = $acct_number;
        $element[1] = $debit_increases;
        $element[2] = $acct_name;

        $Info[$j] = [ @element ];
      }
    }

    $local_accounts[$i] = [ @Info ];
  }

  return @local_accounts;
}

# AccountActivity
# Takes  : Array of Accounts with Debit Increases Flag and Account Name (accounts), dbh, ledger_code
# Takes  : begin_date, end_date, search_type (optional, defaults to 1) (0=exclude both dates, 1=include both dates, 2=exclude begin but include end date)
# Takes  : sub_account (optional, the Sub Account number and name), sub_entity (optional, the Code and Name)
# Returns: Array of Activity and the # of entries for each account
sub AccountActivity
{
  my %args = ( search_type => '1', @_ );
  my $dbh = $args{'dbh'};
  my @accounts = @{$args{'accounts'}};
  my $search_type = $args{'search_type'};
  my $ledger_code = $args{'ledger_code'};
  my $begin_date = $args{'begin_date'};
  my $end_date = $args{'end_date'};
  my @sub_account = ();
  my @sub_entity = ();
  my @activity = ();

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - AccountActivity", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (scalar @accounts == 0)
  {
    my_die("Error: Report.cgi - AccountActivity", 4034, "The Array of Accounts is required at line " . __LINE__);
  }

  if ($search_type < 0 || $search_type > 2)
  {
    my_die("Error:  Report.cgi - AccountActivity", 4034, "The Date Search flag can only be 0, 1, or 2!&nbsp;&nbsp;'$search_type' is invalid!<br>\nAt Line " . __LINE__);
  }

  if (length $ledger_code == 0)
  {
    my_die("Error:  Report.cgi - AccountActivity", 4034, "The ledger_code must be specified!<br>\nAt Line " . __LINE__);
  }

  if (length $begin_date == 0)
  {
    my_die("Error:  Report.cgi - AccountActivity", 4034, "The begin_date must be specified!<br>\nAt Line " . __LINE__);
  }

  if (length $end_date == 0)
  {
    my_die("Error:  Report.cgi - AccountActivity", 4034, "The end_date must be specified!<br>\nAt Line " . __LINE__);
  }

  if (exists $args{'sub_account'})
  {
    @sub_account = @{$args{'sub_account'}};
    if (scalar @sub_account == 0)
    {
      my_die("Error:  Report.cgi - AccountActivity", 4034, "The Sub Account info can not be empty!<br>\nAt Line " . __LINE__);
    }
  }

  if (exists $args{'sub_entity'})
  {
    @sub_entity = @{$args{'sub_entity'}};
    if (scalar @sub_entity == 0)
    {
      my_die("Error:  Report.cgi - AccountActivity", 4034, "The Sub Entity info can not be empty!<br>\nAt Line " . __LINE__);
    }
  }

  my ($i, $acct_num, $debit_increases, $balance, $acct_name);
  my $date_search_string = "";
  if ($search_type == 1)
  {
    $date_search_string = "(je.trans_date BETWEEN '$begin_date' AND '$end_date')";
  }
  elsif ($search_type == 2)
  {
    $date_search_string = "(je.trans_date > '$begin_date' AND je.trans_date <= '$end_date')";
  }
  elsif ($search_type == 0)
  {
    $date_search_string = "(je.trans_date > '$begin_date' AND je.trans_date < '$end_date')";
  }

  my $tmp_sql_statement = "";
  $tmp_sql_statement = $sql_statements{1050};
  if (scalar @sub_account > 0)
  {
    my $temp_string = "db.sub_account_num = $sub_account[0] AND ";
    $tmp_sql_statement =~ s/$firstSubstitutionRE/$temp_string/;
  }
  else
  {
    $tmp_sql_statement =~ s/$firstSubstitutionRE//;
  }
  if (scalar @sub_entity > 0)
  {
    my $temp_string = "db.sub_e_code = '$sub_entity[0]' AND ";
    $tmp_sql_statement =~ s/$secondSubstitutionRE/$temp_string/;
  }
  else
  {
    $tmp_sql_statement =~ s/$secondSubstitutionRE//;
  }
  $tmp_sql_statement =~ s/$substitutionRE/$date_search_string/;
  my $sth = $dbh->prepare( $tmp_sql_statement ) || my_db_die("Error: Report.cgi - AccountActivity", 4007, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);

  for ($i=0; $i < scalar @accounts; $i++)
  {
    $acct_num = $accounts[$i][0];
    my @account_type = GetAccountType(dbh => $dbh, acct_number => $acct_num);
    #$debit_increases = $accounts[$i][1];
    if ($account_type[0] == 0)
    {
      $debit_increases = 1;
    }
    else
    {
      $debit_increases = 0;
    }
    $acct_name = $accounts[$i][2];
    $balance = "0.00";

    # the power of DBI!
    my $error_string = "";
    if (scalar @sub_account > 0)
    {
      $error_string .= ",&nbsp;Sub Account = '$sub_account[0]'";
    }
    if (scalar @sub_entity > 0)
    {
      $error_string .= ",&nbsp;Sub Entity = '$sub_entity[0]'";
    }
    my $rc = $sth->execute($ledger_code, $acct_num) || my_db_die("Error: Report.cgi - AccountActivity", 4008, "SQL Statement = '$tmp_sql_statement', Ledger Code = '$ledger_code', Account Num = '$acct_num'$error_string<br>\nAt Line " . __LINE__);
    my @info = @{$sth->fetchall_arrayref};
    #$sth->finish;

    for (my $j=0; $j < scalar @info; $j++)
    {
      my $amount = $info[$j][0];
      my $dorc = $info[$j][1];
      $balance = AddMoneyByDebitCredit(balance => $balance, debit_increases => $debit_increases, dorc => $dorc, amount => $amount);
    }

    $balance = $balance;

    my @entry = ();
    $entry[0] = $balance;
    $entry[1] = $sth->rows;

    $activity[$i] = [ @entry ];
  }

  return @activity;
}

# AccountBalanceActivity
# Takes  : Array of Accounts with Debit Increases Flag and Account Name (accounts), dbh, ledger_code
# Takes  : end_date, sub_account (optional, the Sub Account number and name), sub_entity (optional, the Code and Name)
# Returns: Array of Balances and the # of entries for each account
sub AccountBalanceActivity
{
  my %args = ( search_type => '1', @_ );
  my $dbh = $args{'dbh'};
  my @accounts = @{$args{'accounts'}};
  my $ledger_code = $args{'ledger_code'};
  my $end_date = $args{'end_date'};
  my @sub_account = ();
  my @sub_entity = ();
  my @activity = ();

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - AccountBalanceActivity", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (scalar @accounts == 0)
  {
    my_die("Error: Report.cgi - AccountBalanceActivity", 4034, "The Array of Accounts is required at line " . __LINE__);
  }

  if (length $ledger_code == 0)
  {
    my_die("Error:  Report.cgi - AccountBalanceActivity", 4034, "The ledger_code must be specified!<br>\nAt Line " . __LINE__);
  }

  if (length $end_date == 0)
  {
    my_die("Error:  Report.cgi - AccountBalanceActivity", 4034, "The end_date must be specified!<br>\nAt Line " . __LINE__);
  }

  if (exists $args{'sub_account'})
  {
    @sub_account = @{$args{'sub_account'}};
    if (scalar @sub_account == 0)
    {
      my_die("Error:  Report.cgi - AccountBalanceActivity", 4034, "The Sub Account info can not be empty!<br>\nAt Line " . __LINE__);
    }
  }

  if (exists $args{'sub_entity'})
  {
    @sub_entity = @{$args{'sub_entity'}};
    if (scalar @sub_entity == 0)
    {
      my_die("Error:  Report.cgi - AccountBalanceActivity", 4034, "The Sub Entity info can not be empty!<br>\nAt Line " . __LINE__);
    }
  }

  my ($i);
  for ($i=0; $i < scalar @accounts; $i++)
  {
    my %options = ();
    if (scalar @sub_entity > 0)
    {
      $options{'sub_entity'} = \@sub_entity;
    }
    if (scalar @sub_account > 0)
    {
      $options{'sub_account'} = \@sub_account;
    }
    my @result = GetAccountBalance(dbh => $dbh, account => \@{$accounts[$i]}, end_date => $end_date, ledger_code => $ledger_code, %options);

    $activity[$i] = [ @result ];
  }

  return @activity;
}

# AddMoneyByDebitCredit
# Takes  : Current Balance (balance), Debit Increases Flag (debit_increases) (1/0), DORC Flag (dorc) (1/0), Amount (amount)
# Returns: New Balance
sub AddMoneyByDebitCredit
{
  my %args = ( @_ );
  my $balance = $args{'balance'};
  my $debit_increases = $args{'debit_increases'};
  my $dorc = $args{'dorc'};
  my $amount = $args{'amount'};

  if (length $balance == 0)
  {
    my_die("Error: Report.cgi - AddMoneyByDebitCredit", 4034, "The Beginning Balance is required at line " . __LINE__);
  }

  if (length $debit_increases == 0)
  {
    my_die("Error: Report.cgi - AddMoneyByDebitCredit", 4034, "The Debit Increases Flag is required at line " . __LINE__);
  }

  if (length $dorc == 0)
  {
    my_die("Error:  Report.cgi - AddMoneyByDebitCredit", 4034, "The Dorc Flag must be specified!<br>\nAt Line " . __LINE__);
  }

  if (length $amount == 0)
  {
    my_die("Error:  Report.cgi - AddMoneyByDebitCredit", 4034, "The Amount to Increment/Decrement must be specified!<br>\nAt Line " . __LINE__);
  }

  if ($debit_increases && $dorc)
  {
    $balance += $amount;
  }
  elsif ($debit_increases && !$dorc)
  {
    $balance -= $amount;
  }
  elsif (!$debit_increases && $dorc)
  {
    $balance -= $amount;
  }
  elsif (!$debit_increases && !$dorc)
  {
    $balance += $amount;
  }

  $balance = $balance;

  return $balance;
}

# AccountActivitySplit
# Takes  : Array of Accounts with Debit Increases Flag and Account Name (accounts), dbh, ledger_code
# Takes  : begin_date, end_date, search_type (optional, defaults to 0) (0=exclude both dates, 1=include both dates, 2=exclude begin but include end date)
# Returns: Array of debits and credits for each Account specified
sub AccountActivitySplit
{
  my %args = ( search_type => '0', @_ );
  my $dbh = $args{'dbh'};
  my @accounts = @{$args{'accounts'}};
  my $search_type = $args{'search_type'};
  my $ledger_code = $args{'ledger_code'};
  my $begin_date = $args{'begin_date'};
  my $end_date = $args{'end_date'};
  my @activity = ();

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - AccountActivitySplit", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (scalar @accounts == 0)
  {
    my_die("Error: Report.cgi - AccountActivitySplit", 4034, "The Array of Accounts is required at line " . __LINE__);
  }

  if ($search_type < 0 || $search_type > 2)
  {
    my_die("Error:  Report.cgi - AccountActivitySplit", 4034, "The Date Search flag can only be 0, 1, or 2!&nbsp;&nbsp;'$search_type' is invalid!<br>\nAt Line " . __LINE__);
  }

  if (length $ledger_code == 0)
  {
    my_die("Error:  Report.cgi - AccountActivitySplit", 4034, "The ledger_code must be specified!<br>\nAt Line " . __LINE__);
  }

  if (length $begin_date == 0)
  {
    my_die("Error:  Report.cgi - AccountActivitySplit", 4034, "The begin_date must be specified!<br>\nAt Line " . __LINE__);
  }

  if (length $end_date == 0)
  {
    my_die("Error:  Report.cgi - AccountActivitySplit", 4034, "The end_date must be specified!<br>\nAt Line " . __LINE__);
  }

  my ($i, $acct_num, $debit_increases, $acct_name);
  my $date_search_string = "";
  if ($search_type == 1)
  {
    $date_search_string = "je.trans_date BETWEEN '$begin_date' AND '$end_date'";
  }
  elsif ($search_type == 2)
  {
    $date_search_string = "je.trans_date > '$begin_date' AND je.trans_date <= '$end_date'";
  }
  elsif ($search_type == 0)
  {
    $date_search_string = "je.trans_date > '$begin_date' AND je.trans_date < '$end_date'";
  }

  my $tmp_sql_statement  = "";
  my $tmp_sql_statement2 = "";
  ($tmp_sql_statement = $sql_statements{1052}) =~ s/$substitutionRE/$date_search_string/;
  my $sth = $dbh->prepare( $tmp_sql_statement ) || my_db_die("Error: Report.cgi - AccountActivitySplit", 4007, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);
  ($tmp_sql_statement2 = $sql_statements{1053}) =~ s/$substitutionRE/$date_search_string/;
  my $sth2 = $dbh->prepare( $tmp_sql_statement2 ) || my_db_die("Error: Report.cgi - AccountActivitySplit", 4007, "SQL Statement = '$tmp_sql_statement2'<br>\nAt Line " . __LINE__);

  for ($i=0; $i < scalar @accounts; $i++)
  {
    $acct_num = $accounts[$i][0];
    $debit_increases = $accounts[$i][1];
    $acct_name = $accounts[$i][2];
    my $debits = "0.00";
    my $credits = "0.00";

    # the power of DBI!
    my $rc = $sth->execute($ledger_code, $acct_num) || my_db_die("Error: Report.cgi - AccountActivitySplit", 4008, "SQL Statement = '$tmp_sql_statement', Ledger Code = '$ledger_code', Account Num = '$acct_num'<br>\nAt Line " . __LINE__);
    my @info = $sth->fetchrow_array;
    $sth->finish;

    if (!defined $info[0])
    {
      $debits = "0.00";
    }
    else
    {
      $debits = $info[0];
    }
    $debits = $debits;

    $rc = $sth2->execute($ledger_code, $acct_num) || my_db_die("Error: Report.cgi - AccountActivitySplit", 4008, "SQL Statement = '$tmp_sql_statement2', Ledger Code = '$ledger_code', Account Num = '$acct_num'<br>\nAt Line " . __LINE__);
    @info = $sth2->fetchrow_array;
    $sth2->finish;

    if (!defined $info[0])
    {
      $credits = "0.00";
    }
    else
    {
      $credits = $info[0];
    }
    $credits = $credits;

    my @entry = ();
    $entry[0] = $debits;
    $entry[1] = $credits;

    $activity[$i] = [ @entry ];
  }

  return @activity;
}

# GetSubAccountTypeName
# Takes  : sub_account_type, dbh
# Returns: Sub Account Type Name
sub GetSubAccountTypeName
{
  my %args = ( order => '0', @_ );
  my $dbh = $args{'dbh'};
  my $sub_account_type = $args{'sub_account_type'};
  my $sub_account_name = "";

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetSubAccountTypeName", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $sub_account_type == 0)
  {
    my_die("Error: Report.cgi - GetSubAccountTypeName", 4034, "The Sub Account Type is required at line " . __LINE__);
  }

  if ($sub_account_type == -1)
  {
    $sub_account_name = "<i>None</i>";
  }
  else
  {
    my $sth = $dbh->prepare( $sql_statements{1058} ) || my_db_die("Error: Report.cgi - GetSubAccountTypeName", 4007, "SQL Statement = '$sql_statements{1058}'<br>\nAt Line " . __LINE__);
    my $rc = $sth->execute($sub_account_type) || my_db_die("Error: Report.cgi - GetSubAccountTypeName", 4008, "SQL Statement = '$sql_statements{1058}', Sub Account Type = '$sub_account_type'<br>\nAt Line " . __LINE__);
    my @info = $sth->fetchrow_array;
    $sth->finish;

    if (!defined $info[0])
    {
      my_db_die("Error:  Report.cgi - GetSubAccountTypeName", 4009, "No Account Name returned for Account = '$sub_account_type'!<br>\nAt Line " . __LINE__);
    }
    $sub_account_name = $info[0];
  }

  return $sub_account_name;
}

# GetSubAccountTypes
# Takes  : account_type, dbh
# Returns: Array of sub-account types assigned to the account type ordered number first
#          The first element will contain the number, the second element will contain the name
sub GetSubAccountTypes
{
  my %args = ( order => '0', @_ );
  my $dbh = $args{'dbh'};
  my $account_type = $args{'account_type'};

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetSubAccountTypes", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $account_type == 0)
  {
    my_die("Error: Report.cgi - GetSubAccountTypes", 4034, "The Account Type is required at line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1055} ) || my_db_die("Error: Report.cgi - GetSubAccountTypes", 4007, "SQL Statement = '$sql_statements{1055}'<br>\nAt Line " . __LINE__);
  my $rc = $sth->execute($account_type) || my_db_die("Error: Report.cgi - GetSubAccountTypes", 4008, "SQL Statement = '$sql_statements{1055}', Account Type = '$account_type'<br>\nAt Line " . __LINE__);
  my @temp_array = $sth->fetchrow_array;
  $sth->finish;

  my @info = sort {$a <=> $b} split(/,/, $temp_array[0]);
  my @name_num_array = ();
  # get the names for each sub-account type
  for (my $count=0; $count < scalar @info; $count++)
  {
    $name_num_array[$count] = ();
    $name_num_array[$count][0] = $info[$count];
    $name_num_array[$count][1] = GetSubAccountTypeName(dbh => $dbh, sub_account_type => $info[$count]);
  }

  return @name_num_array;
}

# GetSubAccounts
# Takes   : account_type, dbh
# Optional: order (0=number, 1=name)
# Returns : Array of sub-accounts ordered number first unless overridden by the order argument
#	    The first element of the array will contain the number of the sub-account, the second element will contain the
#           name.
sub GetSubAccounts
{
  my %args = ( order => '0', @_ );
  my $dbh = $args{'dbh'};
  my $account_type = $args{'account_type'};
  my $order = $args{'order'};

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetSubAccounts", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if ($order < 0 || $order > 1)
  {
    my_die("Error:  Report.cgi - GetSubAccounts", 4034, "The Order flag can only be 0 or 1!&nbsp;&nbsp;'$order' is invalid!<br>\nAt Line " . __LINE__);
  }

  if (length $account_type == 0)
  {
    my_die("Error: Report.cgi - GetSubAccounts", 4034, "The Account Type is required at line " . __LINE__);
  }

  # get the sub-accounts for the given account type
  my $sth = $dbh->prepare( $sql_statements{1055} ) || my_db_die("Error: Report.cgi - GetSubAccounts", 4007, "SQL Statement = '$sql_statements{1055}'<br>\nAt Line " . __LINE__);
  my $rc = $sth->execute($account_type) || my_db_die("Error: Report.cgi - GetSubAccounts", 4008, "SQL Statement = '$sql_statements{1055}', Account Type = '$account_type'<br>\nAt Line " . __LINE__);
  my @temp_array = $sth->fetchrow_array;
  $sth->finish;

  my $tmp_sql_statement  = "";
  ($tmp_sql_statement = $sql_statements{1054}) =~ s/$firstSubstitutionRE/$temp_array[0]/;
  my $temp_order_by = ($order) ? "sub_acc_name" : "sub_acc_number";
  $tmp_sql_statement =~ s/$secondSubstitutionRE/$temp_order_by/;
  $sth = $dbh->prepare( $tmp_sql_statement ) || my_db_die("Error: Report.cgi - GetSubAccounts", 4007, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);
  $rc = $sth->execute || my_db_die("Error: Report.cgi - GetSubAccounts", 4008, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);

  my @sub_accounts = ();
  while (@temp_array = $sth->fetchrow_array)
  {
    my @temp1 = ($temp_array[1], $temp_array[0]);
    $sub_accounts[scalar @sub_accounts] = [ @temp1 ];
  }
  $sth->finish;

  return @sub_accounts;
}

# GetSubAccountsForSubAccountType
# Takes   : sub_account_type, dbh
# Optional: order (0=number, 1=name)
# Returns : Array of sub-accounts ordered number first unless overridden by the order argument
#           The first element of the array will contain the number of the sub-account, the second element will contain the name.
sub GetSubAccountsForSubAccountType
{
  my %args = ( order => '0', @_ );
  my $dbh = $args{'dbh'};
  my $sub_account_type = $args{'sub_account_type'};
  my $order = $args{'order'};

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetSubAccountsForSubAccountType", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if ($order < 0 || $order > 1)
  {
    my_die("Error:  Report.cgi - GetSubAccountsForSubAccountType", 4034, "The Order flag can only be 0 or 1!&nbsp;&nbsp;'$order' is invalid!<br>\nAt Line " . __LINE__);
  }

  if (length $sub_account_type == 0)
  {
    my_die("Error: Report.cgi - GetSubAccountsForSubAccountType", 4034, "The Sub Account Type is required at line " . __LINE__);
  }

  my @sub_accounts = ();
  if ($sub_account_type != -1)
  {
    # get the sub-accounts for the given sub-account type
    my $tmp_sql_statement  = "";
    ($tmp_sql_statement = $sql_statements{1054}) =~ s/$firstSubstitutionRE/$sub_account_type/;
    my $temp_order_by = ($order) ? "sub_acc_name" : "sub_acc_number";
    $tmp_sql_statement =~ s/$secondSubstitutionRE/$temp_order_by/;
    my $sth = $dbh->prepare( $tmp_sql_statement ) || my_db_die("Error: Report.cgi - GetSubAccountsForSubAccountType", 4007, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);
    my $rc = $sth->execute || my_db_die("Error: Report.cgi - GetSubAccountsForSubAccountType", 4008, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);

    while (my @temp_array = $sth->fetchrow_array)
    {
      my @temp1 = ($temp_array[1], $temp_array[0]);
      $sub_accounts[scalar @sub_accounts] = [ @temp1 ];
    }
    $sth->finish;
  }

  return @sub_accounts;
}

# GetTypeForSubAccount
# Takes  : sub_account, dbh
# Returns: Sub Account Type
sub GetTypeForSubAccount
{
  my %args = ( order => '0', @_ );
  my $dbh = $args{'dbh'};
  my $sub_account = $args{'sub_account'};

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetTypeForSubAccount", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $sub_account == 0)
  {
    my_die("Error: Report.cgi - GetTypeForSubAccount", 4034, "The Sub Account is required at line " . __LINE__);
  }

  if ($sub_account == -1)
  {
    return -2;  # This should never be a Sub Account Type!
  }

  # get the sub-account type for the given sub-account
  my $sth = $dbh->prepare( $sql_statements{1059} ) || my_db_die("Error: Report.cgi - GetTypeForSubAccount", 4007, "SQL Statement = '$sql_statements{1059}'<br>\nAt Line " . __LINE__);
  my $rc = $sth->execute($sub_account) || my_db_die("Error: Report.cgi - GetTypeForSubAccount", 4008, "SQL Statement = '$sql_statements{1059}', Sub Account = '$sub_account'<br>\nAt Line " . __LINE__);
  my @temp_array = $sth->fetchrow_array;
  $sth->finish;

  my $sub_account_type = $temp_array[0];

  return $sub_account_type;
}

# GetAccountsForSubAccount
# Takes   : sub_account, dbh, ledger_code
# Optional: taxable, account_type
# Returns : Array of Local Accounts that are used with the given sub-account
sub GetAccountsForSubAccount
{
  my %args = ( taxable => '0', @_ );
  my $dbh = $args{'dbh'};
  my $sub_account = $args{'sub_account'};
  my $ledger_code = $args{'ledger_code'};
  my $account_type = $args{'account_type'};
  my @account_type_list = ();
  my $taxable = $args{'taxable'};

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetAccountsForSubAccount", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $sub_account == 0)
  {
    my_die("Error: Report.cgi - GetAccountsForSubAccount", 4034, "The Sub Account is required at line " . __LINE__);
  }

  if (length $ledger_code == 0)
  {
    my_die("Error: Report.cgi - GetAccountsForSubAccount", 4034, "The Ledger Code is required at line " . __LINE__);
  }

  if (defined $account_type && length $account_type == 0)
  {
    my_die("Error: Report.cgi - GetAccountsForSubAccount", 4034, "The Account Type value must be defined at line " . __LINE__);
  }

  # get the sub-account type for the given sub-account
  if (!defined $account_type)
  {
    my $sub_account_type = GetTypeForSubAccount(dbh => $dbh, sub_account => $sub_account);
    # get a list of account types this sub-account type is assigned to
    my $sth = $dbh->prepare( $sql_statements{1060} ) || my_db_die("Error: Report.cgi - GetAccountsForSubAccount", 4007, "SQL Statement = '$sql_statements{1060}'<br>\nAt Line " . __LINE__);
    my $rc = $sth->execute() || my_db_die("Error: Report.cgi - GetAccountsForSubAccount", 4008, "SQL Statement = '$sql_statements{1060}'<br>\nAt Line " . __LINE__);
    while (my @temp_array = $sth->fetchrow_array)
    {
      my @temp_type = split /,/, $temp_array[1];
      for (my $count=0; $count < scalar @temp_type; $count++)
      {
        if ($sub_account_type == $temp_type[$count] || $sub_account == -1)
        {
          $account_type_list[scalar @account_type_list] = $temp_array[0];
        }
      }
    }
    $sth->finish;
  }
  else
  {
    $account_type_list[0] = $account_type;
  }

  my @local_accounts = ();
  # get the local list of accounts for the account_type_list of account types
  if (scalar @account_type_list > 0)
  {
    my @local_accounts_list = GetLocalAccounts(dbh => $dbh, account_types => \@account_type_list, taxable => $taxable, ledger_code => $ledger_code);
    my $count3 = 0;
    for (my $count=0; $count < scalar @local_accounts_list; $count++)
    {
      for (my $count2=0; $count2 < scalar @{$local_accounts_list[$count]}; $count2++)
      {
        $local_accounts[$count3] = ();
        $local_accounts[$count3][0] = $local_accounts_list[$count][$count2][0];
        $local_accounts[$count3][1] = $local_accounts_list[$count][$count2][1];
        $local_accounts[$count3][2] = $local_accounts_list[$count][$count2][2];
        $count3++;
      }
    }
  }

  return @local_accounts;
}

# AccountTypeActivity
# Takes  : Array of Account Types (account_types), dbh, ledger_code
# Takes  : begin_date (optional if doing a balance), end_date, search_type (optional, defaults to 1) (0=exclude both dates, 1=include both dates, 2=exclude begin but include end date)
# Takes  : taxable (optional, assumed to be 0) (0=don't care, 1 = taxable only, 2 = non-taxable only), balance (optional, assumed to be 0) (0=activity, 1=balance)
# Takes  : sub_entity Array (Code, Name) (optional), sub_account Array (Number, Name) (optional)
# Returns: Array of Activity and the # of entries for each account type
sub AccountTypeActivity
{
  my %args = ( search_type => '1', taxable => '0', balance => '0', @_ );
  my $dbh = $args{'dbh'};
  my @account_types = @{$args{'account_types'}};
  my $search_type = $args{'search_type'};
  my $ledger_code = $args{'ledger_code'};
  my $begin_date = $args{'begin_date'};
  my $end_date = $args{'end_date'};
  my $taxable = $args{'taxable'};
  my $balance = $args{'balance'};
  my @sub_entity = ();
  my @sub_account = ();
  my @activity = ();

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - AccountTypeActivity", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (scalar @account_types == 0)
  {
    my_die("Error: Report.cgi - AccountTypeActivity", 4034, "The Array of Account Types is required at line " . __LINE__);
  }

  if ($search_type < 0 || $search_type > 2)
  {
    my_die("Error:  Report.cgi - AccountTypeActivity", 4034, "The Date Search flag can only be 0, 1, or 2!&nbsp;&nbsp;'$search_type' is invalid!<br>\nAt Line " . __LINE__);
  }

  if ($taxable < 0 || $taxable > 2)
  {
    my_die("Error:  Report.cgi - AccountTypeActivity", 4034, "The Taxable flag can only be 0, 1, or 2!&nbsp;&nbsp;'$taxable' is invalid!<br>\nAt Line " . __LINE__);
  }

  if ($balance < 0 || $balance > 1)
  {
    my_die("Error:  Report.cgi - AccountTypeActivity", 4034, "The Balance flag can only be 0 or 1!&nbsp;&nbsp;'$balance' is invalid!<br>\nAt Line " . __LINE__);
  }

  if (length $ledger_code == 0)
  {
    my_die("Error:  Report.cgi - AccountTypeActivity", 4034, "The ledger_code must be specified!<br>\nAt Line " . __LINE__);
  }

  if (length $begin_date == 0 && $balance == 0)
  {
    my_die("Error:  Report.cgi - AccountTypeActivity", 4034, "The begin_date must be specified!<br>\nAt Line " . __LINE__);
  }

  if (length $end_date == 0)
  {
    my_die("Error:  Report.cgi - AccountTypeActivity", 4034, "The end_date must be specified!<br>\nAt Line " . __LINE__);
  }

  if (exists $args{'sub_entity'})
  {
    @sub_entity = @{$args{'sub_entity'}};
    if (scalar @sub_entity == 0)
    {
      my_die("Error: Report.cgi - AccountTypeActivity", 4034, "The SubEntity can not be empty at line " . __LINE__);
    }
  }

  if (exists $args{'sub_account'})
  {
    @sub_account = @{$args{'sub_account'}};
    if (scalar @sub_account == 0)
    {
      my_die("Error:  Report.cgi - AccountTypeActivity", 4034, "The Sub Account is required at line " . __LINE__);
    }
  }

  my ($i, $tmp_balance, $num_entries);

  for ($i=0; $i < scalar @account_types; $i++)
  {
    $tmp_balance = "0.00";
    $num_entries = 0;
    my @temp_array = ();

    $temp_array[0] = $account_types[$i];

    # Use GetLocalAccounts to get all Accounts for the specified Account Type
    my @accounts = GetLocalAccounts(dbh => $dbh, ledger_code => $ledger_code, taxable => $taxable, account_types => \@temp_array);

    # now we massage the 2 dimensional array of accounts to 1 dimension.
    @accounts = RetrieveSubArray(array => \@accounts, index => 0);

    if (scalar @accounts > 0)
    {
      my @balances = ();
      my %options = ();
      if (scalar @sub_account > 0)
      {
        $options{'sub_account'} = \@sub_account;
      }
      if (scalar @sub_entity > 0)
      {
        $options{'sub_entity'} = \@sub_entity;
      }

      if ($balance)
      {
        @balances = AccountBalanceActivity(dbh => $dbh, ledger_code => $ledger_code, accounts => \@accounts, end_date => $end_date, %options);
      }
      else
      {
        @balances = AccountActivity(dbh => $dbh, ledger_code => $ledger_code, accounts => \@accounts, begin_date => $begin_date, end_date => $end_date, search_type => $search_type, %options);
      }

      for (my $j=0; $j < scalar @balances; $j++)
      {
        $tmp_balance += $balances[$j][0];
        $num_entries += $balances[$j][1];
      }
    }

    $tmp_balance = $tmp_balance;

    my @entry = ();
    $entry[0] = $tmp_balance;
    $entry[1] = $num_entries;

    $activity[$i] = [ @entry ];
  }

  return @activity;
}

# RetrieveSubArray - Extracts the specified sub array from a multi-dimensional array and returns a flat array.
# Takes  : array, index
# Returns: sub array from array at index.
sub RetrieveSubArray
{
  my %args = ( @_ );
  my @array = @{$args{'array'}};
  my $index = $args{'index'};

  my @sub_array = @{$array[$index]};

  return @sub_array;
}

# SubAccountActivity
# Takes  : Array of SubAccounts (number, name)
# Takes  : Array of Accounts (number, debit increases, name).  For each Sub Account is an array of the accounts.
# Takes  : Begin and End Dates, Search Type (optional, defaults to 1) (0=exclude both dates, 1=include both dates, 2=exclude begin but include end date)
# Returns: Array of Activity with a subarray for each sub account.
#	   Array [Sub Accounts]
#	   [Sub Accounts] = total activity, [Accounts Activity], total # entries
#	   [Accounts Activity] = activity, # entries
sub SubAccountActivity
{
  my %args = ( search_type => '1', @_ );
  my $dbh = $args{'dbh'};
  my @sub_accounts = @{$args{'sub_accounts'}};
  my @accounts = @{$args{'accounts'}};
  my $search_type = $args{'search_type'};
  my $ledger_code = $args{'ledger_code'};
  my $begin_date = $args{'begin_date'};
  my $end_date = $args{'end_date'};
  my $balance = $args{'balance'};
  my @activity = ();
  my @sub_entity = ();

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - SubAccountActivity", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (scalar @sub_accounts == 0)
  {
    my_die("Error: Report.cgi - SubAccountActivity", 4034, "The Array of SubAccounts is required at line " . __LINE__);
  }

  if (scalar @accounts == 0)
  {
    my_die("Error: Report.cgi - SubAccountActivity", 4034, "The Array of Accounts is required at line " . __LINE__);
  }

  if ($search_type < 0 || $search_type > 2)
  {
    my_die("Error:  Report.cgi - SubAccountActivity", 4034, "The Date Search flag can only be 0, 1, or 2!&nbsp;&nbsp;'$search_type' is invalid!<br>\nAt Line " . __LINE__);
  }

  if (length $ledger_code == 0)
  {
    my_die("Error:  Report.cgi - SubAccountActivity", 4034, "The ledger_code must be specified!<br>\nAt Line " . __LINE__);
  }

  if (length $begin_date == 0)
  {
    my_die("Error:  Report.cgi - SubAccountActivity", 4034, "The begin_date must be specified!<br>\nAt Line " . __LINE__);
  }

  if (length $end_date == 0)
  {
    my_die("Error:  Report.cgi - SubAccountActivity", 4034, "The end_date must be specified!<br>\nAt Line " . __LINE__);
  }

  if (exists $args{'sub_entity'})
  {
    @sub_entity = @{$args{'sub_entity'}};
    if (scalar @sub_entity == 0)
    {
      my_die("Error:  Report.cgi - SubAccountActivity", 4034, "The Sub Entity info can not be empty!<br>\nAt Line " . __LINE__);
    }
  }

  my ($i, $tmp_balance, $num_entries);
  # now we need to walk through the SubAccounts array and for each entry, get all Account Activity totals and sum them up.
  for ($i=0; $i < scalar @sub_accounts; $i++)
  {
    $tmp_balance = "0.00";
    $num_entries = 0;
    my @entry = ();

    my @tmp_accounts = RetrieveSubArray(array => \@accounts, index => $i);

    if (scalar @tmp_accounts > 0)
    {
      my %options = ();
      if (scalar @sub_entity > 0)
      {
        $options{'sub_entity'} = \@sub_entity;
      }
      my @balances = ();
      my @tmp_sub_account = RetrieveSubArray(array => \@sub_accounts, index => $i);
      if ($balance)
      {
        @balances = AccountBalanceActivity(dbh => $dbh, ledger_code => $ledger_code, accounts => \@tmp_accounts, search_type => $search_type, begin_date => $begin_date, end_date => $end_date, sub_account => \@tmp_sub_account, %options);
      }
      else
      {
        @balances = AccountActivity(dbh => $dbh, ledger_code => $ledger_code, accounts => \@tmp_accounts, search_type => $search_type, begin_date => $begin_date, end_date => $end_date, sub_account => \@tmp_sub_account, %options);
      }
      @entry = @balances;
      for (my $j=0; $j < scalar @balances; $j++)
      {
        $tmp_balance += $balances[$j][0];
        $num_entries += $balances[$j][1];
      }
    }
    else
    {
      @entry = [ ];
    }

    $tmp_balance = $tmp_balance;

    my @tmp_entry = ();
    $tmp_entry[0] = $tmp_balance;
    $tmp_entry[1] = [ @entry ];
    $tmp_entry[2] = $num_entries;

    $activity[$i] = [ @tmp_entry ];
  }
  #my_die("Test",4006,"There are " . int(scalar@activity) . " entries for subaccount ($sub_accounts[$i][1]) = $sub_accounts[$i][0] in SubAccountActivity.");

  return @activity;
}

# GetSubEntities
# Takes  : dbh, ledger_code
# Returns: Array of SubEntities in the current ledger
#          [SubEntities] = Code, Name
sub GetSubEntities
{
  my %args = ( @_ );
  my $dbh = $args{'dbh'};
  my $ledger_code = $args{'ledger_code'};
  my @sub_entities = ();

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetSubEntities", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $ledger_code == 0)
  {
    my_die("Error:  Report.cgi - GetSubEntities", 4034, "The ledger_code must be specified!<br>\nAt Line " . __LINE__);
  }

  my @tmp_array = ();
  $tmp_array[0] = '';
  $tmp_array[1] = '<i>None</i>';
  $sub_entities[0] = [ @tmp_array ];

  my $counter=1;

  my $sth = $dbh->prepare( $sql_statements{1033} ) || my_db_die("Error: Report.cgi - GetSubEntities", 4007, "SQL Statement = '$sql_statements{1033}'<br>\nAt Line " . __LINE__);
  my $rc = $sth->execute($ledger_code) || my_db_die("Error: Report.cgi - GetSubEntities", 4008, "SQL Statement = '$sql_statements{1033}', ledger_code = '$ledger_code'<br>\nAt Line " . __LINE__);

  while (my @info = $sth->fetchrow_array)
  {
    @tmp_array = ();
    $tmp_array[0] = $info[1];
    $tmp_array[1] = $info[0];

    $sub_entities[$counter++] = [ @tmp_array ];
  }

  return @sub_entities;
}

# SubEntityAccountActivity
# Takes  : dbh, ledger_code, Array of SubEntities (Code, Name), Array of Accounts (Number, Debit Increases, Name), begin_date, end_date
# Takes  : sub_account Array (Optional) (Number, Name), search_type (optional, defaults to 1) (0=exclude both dates, 1=include both dates, 2=exclude begin but include end date)
# Returns: Array of SubEntity Activity
#          [SubEntities] = activity_total, [Accounts Activity], num_entries
#          [Accounts Activity] = Activity of Account, # of entries in same order as in Account Array.
sub SubEntityAccountActivity
{
  my %args = ( search_type => '1', @_ );
  my $dbh = $args{'dbh'};
  my @sub_entities = @{$args{'sub_entities'}};
  my @accounts = @{$args{'accounts'}};
  my $search_type = $args{'search_type'};
  my $ledger_code = $args{'ledger_code'};
  my $begin_date = $args{'begin_date'};
  my $end_date = $args{'end_date'};
  my @activity = ();
  my @sub_account = ();

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - SubEntityAccountActivity", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (scalar @sub_entities == 0)
  {
    my_die("Error: Report.cgi - SubEntityAccountActivity", 4034, "The Array of SubEntities is required at line " . __LINE__);
  }

  if (scalar @accounts == 0)
  {
    my_die("Error: Report.cgi - SubEntityAccountActivity", 4034, "The Array of Accounts is required at line " . __LINE__);
  }

  if ($search_type < 0 || $search_type > 2)
  {
    my_die("Error:  Report.cgi - SubEntityAccountActivity", 4034, "The Date Search flag can only be 0, 1, or 2!&nbsp;&nbsp;'$search_type' is invalid!<br>\nAt Line " . __LINE__);
  }

  if (length $ledger_code == 0)
  {
    my_die("Error:  Report.cgi - SubEntityAccountActivity", 4034, "The ledger_code must be specified!<br>\nAt Line " . __LINE__);
  }

  if (length $begin_date == 0)
  {
    my_die("Error:  Report.cgi - SubEntityAccountActivity", 4034, "The begin_date must be specified!<br>\nAt Line " . __LINE__);
  }

  if (length $end_date == 0)
  {
    my_die("Error:  Report.cgi - SubEntityAccountActivity", 4034, "The end_date must be specified!<br>\nAt Line " . __LINE__);
  }

  if (exists $args{'sub_account'})
  {
    @sub_account = @{$args{'sub_account'}};
    if (scalar @sub_account == 0)
    {
      my_die("Error:  Report.cgi - SubEntityAccountActivity", 4034, "The Sub Account info can not be empty!<br>\nAt Line " . __LINE__);
    }
  }

  my ($i, $balance, $sub_balance, $num_entries);

  # for each sub_entity, get all Account balances and sum them up.
  for ($i=0; $i < scalar @sub_entities; $i++)
  {
    $balance = "0.00";
    $num_entries = 0;
    my %other_args = {};
    if (scalar @sub_account > 0)
    {
      $other_args{'sub_account'} = \@sub_account;
    }
    my @sub_activity = AccountActivity(dbh => $dbh, accounts => \@accounts, ledger_code => $ledger_code, begin_date => $begin_date, end_date => $end_date, search_type => $search_type, sub_entity => \@{$sub_entities[$i]}, %other_args);
    for (my $j=0; $j < scalar @sub_activity; $j++)
    {
      $balance += $sub_activity[$j][0];
      $num_entries += $sub_activity[$j][1];
    }

    $balance = $balance;

    my @entry = ();
    $entry[0] = $balance;
    $entry[1] = [ @sub_activity ];
    $entry[2] = $num_entries;

    $activity[$i] = [ @entry ];
  }

  return @activity;
}

# SubEntitySubAccountActivity
# Takes  : dbh, ledger_code, Array of SubEntities (Code, Name), Array of Accounts (Number, Debit Increases, Name), begin_date, end_date
# Takes  : array of sub_accounts (Number, Name), search_type (optional, defaults to 1) (0=exclude both dates, 1=include both dates, 2=exclude begin but include end date)
# Returns: Array of SubEntity Activity
#	   Array [SubEntities]
#          [SubEntities] = activity_total = sum of Accounts activity_total, [Accounts], num_entries = Sum of Accounts num_entries
#	   [Accounts] = activity_total = sum of SubAccounts activity_total, [SubAccounts], num_entries = Sum of SubAccounts num_entries
#	   [SubAccounts] = activity_total, num_entries

# Assumption:  All accounts are of same account type, thus we only need a single array of subaccounts as they would be the same for every account.
sub SubEntitySubAccountActivity
{
  my %args = ( search_type => '1', @_ );
  my $dbh = $args{'dbh'};
  my @sub_entities = @{$args{'sub_entities'}};
  my @accounts = @{$args{'accounts'}};
  my $search_type = $args{'search_type'};
  my $ledger_code = $args{'ledger_code'};
  my $begin_date = $args{'begin_date'};
  my $end_date = $args{'end_date'};
  my @activity = ();
  my @sub_accounts = @{$args{'sub_accounts'}};

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - SubEntitySubAccountActivity", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (scalar @sub_entities == 0)
  {
    my_die("Error: Report.cgi - SubEntitySubAccountActivity", 4034, "The Array of SubEntities is required at line " . __LINE__);
  }

  if (scalar @accounts == 0)
  {
    my_die("Error: Report.cgi - SubEntitySubAccountActivity", 4034, "The Array of Accounts is required at line " . __LINE__);
  }

  if ($search_type < 0 || $search_type > 2)
  {
    my_die("Error:  Report.cgi - SubEntitySubAccountActivity", 4034, "The Date Search flag can only be 0, 1, or 2!&nbsp;&nbsp;'$search_type' is invalid!<br>\nAt Line " . __LINE__);
  }

  if (length $ledger_code == 0)
  {
    my_die("Error:  Report.cgi - SubEntitySubAccountActivity", 4034, "The ledger_code must be specified!<br>\nAt Line " . __LINE__);
  }

  if (length $begin_date == 0)
  {
    my_die("Error:  Report.cgi - SubEntitySubAccountActivity", 4034, "The begin_date must be specified!<br>\nAt Line " . __LINE__);
  }

  if (length $end_date == 0)
  {
    my_die("Error:  Report.cgi - SubEntitySubAccountActivity", 4034, "The end_date must be specified!<br>\nAt Line " . __LINE__);
  }

  if (scalar @sub_accounts == 0)
  {
    my_die("Error:  Report.cgi - SubEntitySubAccountActivity", 4034, "The Array of Sub Accounts is required at line " . __LINE__);
  }

  my ($i, $balance, $sub_balance, $num_entries);

  for (my $z=0; $z < scalar @sub_entities; $z++) # loop over all sub_entities for this account and subaccount pair.
  {
    my @tmp_sub_entity = ();
    @tmp_sub_entity = RetrieveSubArray(array => \@sub_entities, index => $z);

    # for each account, get all SubAccount/SubEntity balances and sum them up.
    my @account_activity = ();
    for ($i=0; $i < scalar @accounts; $i++)
    {
      my @tmp_account = ();
      $tmp_account[0] = [ RetrieveSubArray(array => \@accounts, index => $i) ];

      my @sub_account_activity = ();
      for (my $k=0; $k < scalar @sub_accounts; $k++)  # loop over all subaccounts for this account.
      {
        my @tmp_sub_account = ();
        @tmp_sub_account = RetrieveSubArray(array => \@sub_accounts, index => $k);

        my @sub_activity = AccountActivity(dbh => $dbh, accounts => \@tmp_account, ledger_code => $ledger_code, begin_date => $begin_date, end_date => $end_date, search_type => $search_type, sub_entity => \@tmp_sub_entity, sub_account => \@tmp_sub_account);
        $balance = $sub_activity[0][0];
        $num_entries = $sub_activity[0][1];

        my @tmp_entry = ();
        $tmp_entry[0] = $balance;
        $tmp_entry[1] = $num_entries;
        $sub_account_activity[$k] = [ @tmp_entry ];
      }
      my @tmp_entry = ();
      $balance = "0.00";
      $num_entries = 0;
      for (my $k=0; $k < scalar @sub_account_activity; $k++)
      {
        $balance += $sub_account_activity[$k][0];
        $num_entries += $sub_account_activity[$k][1];
      }
      $tmp_entry[0] = $balance;
      $tmp_entry[1] = [ @sub_account_activity ];
      $tmp_entry[2] = $num_entries;
      $account_activity[$i] = [ @tmp_entry ];
    }

    $balance = "0.00";
    $num_entries = 0;

    for ($i=0; $i < scalar @account_activity; $i++)
    {
      $balance += $account_activity[$i][0];
      $num_entries += $account_activity[$i][2];
    }

    my @entry = ();
    $entry[0] = $balance;
    $entry[1] = [ @account_activity ];
    $entry[2] = $num_entries;

    $activity[$z] = [ @entry ];
  }

  return @activity;
}

# GetActivityData
# Takes  : dbh, ledger_code, sql_statement (The Where clause in our SQL statement)
# Returns: Array of Entries from the debitcredit_tb that match the given data.
#          Array [Entries]
#          [Entries] = parent_num, sub_e_code, sub_e_name, account_num, account_name, sub_account_num, sub_account_name, amount, dorc, cleared, counter, trans_date, payee_payer, memo, journal_code, journal_name, remember
sub GetActivityData
{
  my %args = ( @_ );
  my $dbh = $args{'dbh'};
  my $ledger_code = $args{'ledger_code'};
  my @activity = ();
  my $sql_statement = $args{'sql_statement'};

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetActivityData", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $ledger_code == 0)
  {
    my_die("Error:  Report.cgi - GetActivityData", 4034, "The ledger_code must be specified!<br>\nAt Line " . __LINE__);
  }

  if (exists $args{'sql_statement'} && length $sql_statement == 0)
  {
    my_die("Error:  Report.cgi - GetActivityData", 4034, "The SQL Statement is required at line " . __LINE__);
  }

  my $tmp_sql_statement = $sql_statements{1061};
  $tmp_sql_statement =~ s/$substitutionRE/$sql_statement/;

  my $old_account = -1;
  my $account_type = -1;
  my $account_name = "";

  my $sth = $dbh->prepare( $tmp_sql_statement ) || my_db_die("Error: Report.cgi - GetActivityData", 4007, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);
  my $rc = $sth->execute($ledger_code) || my_db_die("Error: Report.cgi - GetActivityData", 4008, "SQL Statement = '$tmp_sql_statement', ledger_code = '$ledger_code'<br>WHERE clause: '$sql_statement'<br>\nAt Line " . __LINE__);
  while (my @info = $sth->fetchrow_array)
  {
    my $parent_num = $info[0];
    my $sub_e_code = $info[1];
    my $sub_e_name = "";
    my $account_num = $info[2];
    my $sub_account_num = $info[3];
    my $amount = $info[4];
    my $dorc = $info[5];
    my $cleared = $info[6];
    my $counter = $info[7];
    my $trans_date = $info[8];
    my $payee_payer = $info[9];
    my $memo = $info[10];
    my $journal_code = $info[11];
    my $journal_name = "";
    my $remember = $info[12];
    my $sub_account_name = "";

    if ($account_num != $old_account)
    {
      # get the Account Type of this Account
      my $sth2 = $dbh->prepare( $sql_statements{1056} ) || my_db_die("Error: Report.cgi - GetActivityData", 4007, "SQL Statement = '$sql_statements{1056}'<br>\nAt Line " . __LINE__);
      my $rc2 = $sth2->execute($account_num) || my_db_die("Error: Report.cgi - GetActivityData", 4008, "SQL Statement = '$sql_statements{1056}', account = '$account_num'<br>\nAt Line " . __LINE__);
      my @info2 = $sth2->fetchrow_array;
      $sth2->finish;
      $account_type = $info2[0];
      $old_account = $account_num;

      # get the Account Name of this Account
      my $sth3 = $dbh->prepare( $sql_statements{1048} ) || my_db_die("Error: Report.cgi - GetActivityData", 4007, "SQL Statement = '$sql_statements{1048}'<br>\nAt Line " . __LINE__);
      my $rc3 = $sth3->execute($account_num) || my_db_die("Error: Report.cgi - GetActivityData", 4008, "SQL Statement = '$sql_statements{1048}', account = '$account_num'<br>\nAt Line " . __LINE__);
      my @info3 = $sth3->fetchrow_array;
      $sth3->finish;
      $account_name = $info3[0];
    }

    if ($sub_account_num != -1 && $account_type != 0) # lookup the sub account name for this non-cash account sub-account.
    {
      my $sth2 = $dbh->prepare( $sql_statements{1057} ) || my_db_die("Error: Report.cgi - GetActivityData", 4007, "SQL Statement = '$sql_statements{1057}'<br>\nAt Line " . __LINE__);
      my $rc2 = $sth2->execute($sub_account_num) || my_db_die("Error: Report.cgi - GetActivityData", 4008, "SQL Statement = '$sql_statements{1057}', sub_account = '$sub_account_num'<br>\nAt Line " . __LINE__);
      my @info2 = $sth2->fetchrow_array;
      $sth2->finish;
      $sub_account_name = $info2[0];
    }

    my $sth2 = $dbh->prepare( $sql_statements{1065} ) || my_db_die("Error: Report.cgi - GetActivityData", 4007, "SQL Statement = '$sql_statements{1065}'<br>\nAt Line " . __LINE__);
    my $rc2 = $sth2->execute($ledger_code, $sub_e_code) || my_db_die("Error: Report.cgi - GetActivityData", 4008, "SQL Statement = '$sql_statements{1065}', sub_e_code = '$sub_e_code'<br>\nAt Line " . __LINE__);
    my @info2 = $sth2->fetchrow_array;
    $sth2->finish;
    $sub_e_name = $info2[0];

    $sth2 = $dbh->prepare( $sql_statements{1066} ) || my_db_die("Error: Report.cgi - GetActivityData", 4007, "SQL Statement = '$sql_statements{1066}'<br>\nAt Line " . __LINE__);
    $rc2 = $sth2->execute($journal_code) || my_db_die("Error: Report.cgi - GetActivityData", 4008, "SQL Statement = '$sql_statements{1066}', journal_code = '$journal_code'<br>\nAt Line " . __LINE__);
    @info2 = $sth2->fetchrow_array;
    $sth2->finish;
    $journal_name = $info2[0];

    # code values must be ordered in the element one above the name value that is associated with them; if this is changed
    # much code will break!
    my @entry = ();
    $entry[0] = $parent_num;
    $entry[1] = $sub_e_code;
    $entry[2] = $sub_e_name;
    $entry[3] = $account_num;
    $entry[4] = $account_name;
    $entry[5] = $sub_account_num;
    $entry[6] = $sub_account_name;
    $entry[7] = $amount;
    $entry[8] = $dorc;
    $entry[9] = $cleared;
    $entry[10] = $counter;
    $entry[11] = $trans_date;
    $entry[12] = $payee_payer;
    $entry[13] = $memo;
    $entry[14] = $journal_code;
    $entry[15] = $journal_name;
    $entry[16] = $remember;

    $activity[scalar @activity] = [ @entry ];
  }
  $sth->finish;

  return @activity;
}

# getKnownYearsInfo
# Takes  : dbh, ledger_code
# Returns: KnownYears Array
sub getKnownYearsInfo
{
  my %args = ( @_ );
  my $dbh = $args{'dbh'};
  my $ledger_code = $args{'ledger_code'};

  my @result = ();

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - getKnownYearsInfo", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $ledger_code == 0)
  {
    my_die("Error:  Report.cgi - getKnownYearsInfo", 4034, "The ledger_code must be specified!<br>\nAt Line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1028} ) || my_db_die("Error: Report.cgi - getKnownYearsInfo", 4007, "SQL Statement = '$sql_statements{1028}'");
  my $rc = $sth->execute( $ledger_code ) || my_db_die("Error: Report.cgi - getKnownYearsInfo", 4008, "SQL Statement = '$sql_statements{1028}'");
  my $count = 0;
  while (my @temp_row = $sth->fetchrow_array)
  {
    $result[$count] = [ @temp_row ];

    $count++;
  }
  $sth->finish;

  return @result;
}

# GetWorkingYear
# Takes  : dbh, ledger_code
# Returns: working_year
sub GetWorkingYear
{
  my %args = ( @_ );
  my $dbh = $args{'dbh'};
  my $ledger_code = $args{'ledger_code'};

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetAccountBalance", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $ledger_code == 0)
  {
    my_die("Error:  Report.cgi - GetAccountBalance", 4034, "The ledger_code must be specified!<br>\nAt Line " . __LINE__);
  }

  my ($sth, $rc, @info);
  $sth = $dbh->prepare( $sql_statements{1072} ) || my_db_die("Error: Report.cgi - GetAccountBalance", 4007, "SQL Statement = '$sql_statements{1072}'<br>\nAt Line " . __LINE__);
  $rc = $sth->execute($ledger_code) || my_db_die("Error: Report.cgi - GetAccountBalance", 4008, "SQL Statement = '$sql_statements{1072}', ledger_code = '$ledger_code'<br>\nAt Line " . __LINE__);
  @info = $sth->fetchrow_array;
  my $result = -1;  # signals an error.

  if ($sth->rows > 0)
  {
    $result = $info[0];
  }

  return $result;
}

# GetAccountBalance
# Takes  : dbh, ledger_code, SubEntity Array (Code, Name) (optional), Account Array (Number, Debit Increases, Name), end_date
# Takes  : sub_account Array (Number, Name) (optional)
# Returns: Array of the Balance and # of entries for the account specified.

# Note:  As SubEntities/Accounts do not have their balances stored when the system is initialized or closed out each year, the
# SubEntity value is currently being ignored, but allowed to be entered for future implementation.

# Note:  Since Revenues and Expenses can never have balances stored in the acct_bal_tb, when one of these accounts is passed in,
# the last close out date is gathered and used as the beginning date to gather activity for that account and use that as the
# balance value.
sub GetAccountBalance
{
  my %args = ( @_ );
  my $dbh = $args{'dbh'};
  my @sub_entity = ();
  my @account = @{$args{'account'}};
  my $ledger_code = $args{'ledger_code'};
  my $end_date = $args{'end_date'};
  my @balance = ();
  my @sub_account = ();

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetAccountBalance", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (exists $args{'sub_entity'})
  {
    @sub_entity = @{$args{'sub_entity'}};
    if (scalar @sub_entity == 0)
    {
      my_die("Error: Report.cgi - GetAccountBalance", 4034, "The SubEntity can not be empty at line " . __LINE__);
    }
  }

  if (scalar @account == 0)
  {
    my_die("Error: Report.cgi - GetAccountBalance", 4034, "The Account is required at line " . __LINE__);
  }

  if (length $ledger_code == 0)
  {
    my_die("Error:  Report.cgi - GetAccountBalance", 4034, "The ledger_code must be specified!<br>\nAt Line " . __LINE__);
  }

  if (length $end_date == 0)
  {
    my_die("Error:  Report.cgi - GetAccountBalance", 4034, "The end_date must be specified!<br>\nAt Line " . __LINE__);
  }

  if (exists $args{'sub_account'})
  {
    @sub_account = @{$args{'sub_account'}};
    if (scalar @sub_account == 0)
    {
      my_die("Error:  Report.cgi - GetAccountBalance", 4034, "The Sub Account is required at line " . __LINE__);
    }
  }

  my $temp_bal = "0.00";
  my $temp_date = "Never";
  my $red_account = 0;
  my $acct_number = $account[0];
  my $acct_name = $account[2];
  my @account_type = GetAccountType(dbh => $dbh, acct_number => $acct_number);
  my $debit_increases = $account[1];
  my $tmp_sql_statement = $sql_statements{1063};
  my $other_date_info = "";
  my $sub_acct_num = -2;  # the default sub_account that indicates the total balance for this account and all it's sub-accounts.
  my $sub_entity = '';
  my ($sth, $rc, @info);

  if (scalar @sub_account > 0)
  {
    $sub_acct_num = $sub_account[0];
  }
  if (scalar @sub_entity > 0)
  {
    $sub_entity = $sub_entity[0];
  }

  if ($acct_number =~ /^(4|5)/ || $account_type[0] == 17) # this is a Revenue or Expense or Dividends Paid account
  {
    $red_account = 1;
    # gather the last close out date from the acct_bal_tb or the first date of the first fiscal year if never been closed out.
    $sth = $dbh->prepare( $sql_statements{1071} ) || my_db_die("Error: Report.cgi - GetAccountBalance", 4007, "SQL Statement = '$sql_statements{1071}'<br>\nAt Line " . __LINE__);
    $rc = $sth->execute($ledger_code, $end_date) || my_db_die("Error: Report.cgi - GetAccountBalance", 4008, "SQL Statement = '$sql_statements{1071}', ledger_code = '$ledger_code'<br>\nAt Line " . __LINE__);
    @info = $sth->fetchrow_array;
    my $operator = ">";
    my $tmp_string = "";

    if (length $info[0] > 0)
    {
      $temp_date = $info[0];
      $tmp_string = "db.trans_date $operator '$temp_date' AND ";
    }

    $tmp_sql_statement =~ s/$substitutionRE/$tmp_string/;
    $other_date_info = " start_date = '$temp_date',";
  }
  else
  {
    $sth = $dbh->prepare( $sql_statements{1062} ) || my_db_die("Error: Report.cgi - GetAccountBalance", 4007, "SQL Statement = '$sql_statements{1062}'<br>\nAt Line " . __LINE__);
    $rc = $sth->execute($ledger_code, $acct_number, $sub_acct_num, $end_date) || my_db_die("Error: Report.cgi - GetAccountBalance", 4008, "SQL Statement = '$sql_statements{1062}', ledger_code = '$ledger_code', account = '$acct_number', sub_account = '$sub_acct_num', end_date = '$end_date'<br>\nAt Line " . __LINE__);
    @info = $sth->fetchrow_array;

    if ($sth->rows > 0)
    {
      $temp_bal = $info[0];
      $temp_date = $info[1];

      if ($temp_date ne $end_date)  # there's more data to gather
      {
        my $tmp_string = "db.trans_date > '$temp_date' AND ";
        $tmp_sql_statement =~ s/$substitutionRE/$tmp_string/;
        $other_date_info = " start_date = '$temp_date',";
      }
    }
    else
    {
      $tmp_sql_statement =~ s/$substitutionRE//;
    }
    $sth->finish;
  }

  # replace the ##1?## with the correct sub-account/sub-entity information
  if (scalar @sub_account > 0 && scalar @sub_entity > 0)
  {
    $tmp_sql_statement =~ s/$firstSubstitutionRE/AND db.sub_account_num = $sub_acct_num AND db.sub_e_code = '$sub_entity'/;
  }
  elsif (scalar @sub_account > 0)
  {
    $tmp_sql_statement =~ s/$firstSubstitutionRE/AND db.sub_account_num = $sub_acct_num/;
  }
  elsif (scalar @sub_entity > 0)
  {
    $tmp_sql_statement =~ s/$firstSubstitutionRE/AND db.sub_e_code = '$sub_entity'/;
  }
  else
  {
    $tmp_sql_statement =~ s/$firstSubstitutionRE//;
  }
  if ($tmp_sql_statement !~ /$substitutionRE/ && !($red_account && ($temp_date eq $end_date)))
  {
    $sth = $dbh->prepare( $tmp_sql_statement ) || my_db_die("Error: Report.cgi - GetAccountBalance", 4007, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);
    $rc = $sth->execute($ledger_code, $end_date, $acct_number) || my_db_die("Error: Report.cgi - GetAccountBalance", 4008, "SQL Statement = '$tmp_sql_statement', ledger_code = '$ledger_code',$other_date_info end_date = '$end_date', account = '$acct_number', sub_account = '$sub_acct_num'<br>\nAt Line " . __LINE__);
  }

  while (@info = $sth->fetchrow_array)
  {
    my $amount = $info[0];
    my $dorc = $info[1];

    $temp_bal = AddMoneyByDebitCredit(balance => $temp_bal, amount => $amount, debit_increases => $debit_increases, dorc => $dorc);
  }

  $balance[0] = $temp_bal;
  $balance[1] = $sth->rows;

  return @balance;
}

# log_temp_file - makes the entry in the database that denotes a temp file that needs to be cleaned up later via cron.
# takes  : dbh, filename
# returns: nothing
sub log_temp_file
{
  my %args = ( @_, );
  my $dbh = $args{'dbh'};
  my $file_name = $args{'file_name'};

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - log_temp_file", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $file_name == 0)
  {
    my_die("Error: Report.cgi - log_temp_file", 4035, "The filename is required at line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1029} ) || my_db_die("Error: Report.cgi - log_temp_file", 4007, "SQL Statement = '$sql_statements{1029}'");
  my $rc = $sth->execute($file_name) || my_db_die("Error: Report.cgi - log_temp_file", 4008, "SQL Statement = '$sql_statements{1029}', File Name = '$file_name'");
  $dbh->commit || my_db_die("Error:  Report.cgi - log_temp_file", 4019, "At line " . __LINE__);
}

=item void mergeLanguageObjects(portalLangObj, appLangObj)

  takes: portalLangObj, appLangObj
  returns: nothing

  portalLangObj is an instance of the Portals langObj.
  appLangObj is the langObj just instantiated for the
    App in question.

  The phrases from the portalLangObj are then added to the
  appLangObj's phrases hash (not overriding).
  This allows a single langObj to be passed around and
  to have both the Portals phrases and the apps phrases
  without the user having to duplicate Portal phrases
  in their language mappings.  Any phrases that exist in
  the Portal hash and in the apps hash, are not overwritten.

=cut
sub mergeLanguageObjects
{
  my %args = ( portalLangObj => undef, appLangObj => undef, @_ );
  my $portalLangObj = $args{portalLangObj};
  my $appLangObj = $args{appLangObj};

  foreach my $phrase (keys %{$portalLangObj->{phraseObj}->{phrases}})
  {
    if (!exists $appLangObj->{phraseObj}->{phrases}->{$phrase})
    {
      $appLangObj->{phraseObj}->{phrases}->{$phrase} = $portalLangObj->{phraseObj}->{phrases}->{$phrase};
    }
  }
}
