#!/usr/bin/perl
# report_editor.cgi - This program is the editor for the reporting system.
# It will either generate the create/edit report screens.
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

# Program Modes:  Uses command
# command = 'display_for_edit' - List all created reports to allow the user to select one to edit/delete
#	  = 'create_report' - Start a new edit session that will create a report - generates the frameset.
#	  = 'create_report1' - displays the first screen in the frameset.
#   = 'create_report2' - Takes the default settings from 'create_report' and allows the user to specify the contents of the report in the top frame.  Creates the entry in report_type_tb.
#	  = 'create_report2_top' - Creates the form and code in the top frame to actually work with the report contents.
#	  = 'copy_report' - Takes the id and new_name for the specified report and copies the report_type_tb and report_data_tb entries for the old report, naming it as the new report name.
#	  = 'update_report' - Takes the new report or edited data and updates the entries in report_data_tb in the database.
#	  = 'edit_report' - Takes the selected report and allows the user to edit the date in report_type_tb.  Generates the frameset.
#	  = 'edit_report1' - displays the first screen in the frameset.
#	  = 'delete_report' - Takes the selected report and deletes it from the database.
#	  = 'edit_report2' - Updates the edited default data in report_type_tb and then allows the user to edit the contents of the report in the top frame.
#	  = 'edit_report2_top' - Creates the form and code in the top frame to actually work with the report contents.

use strict;
use cgi_lib;
use custom;

my %errors;
%errors = ( 4000 => 'Eval of HTMLObject::FrameSet failed',
	       4001 => 'Invalid command specified',
	       4002 => 'Eval of HTMLObject::Normal failed',
	       4003 => 'command must be specified',
	       4004 => "Couldn't connect to Database",
	       4005 => "Parameter requires a value",
	       4006 => "Invalid Value for Parameter",
	       4007 => "DBI Preparation of SQL Statement Failed",
	       4008 => "DBI Execution of Statement Failed",
	       4009 => "DBI Error Occurred",
	       4010 => "Column Header Already Used",
	       4011 => "Invalid Column Header",
	       4012 => "Invalid use of Column Header",
	       4013 => "Could not find Sub Account",
	       4014 => "Could not find Sub Account Type",
	       4015 => "Report Name already used",
	       4016 => "Report ID already used",
	       4017 => "Report ID is required",
	       4018 => "Error opening file",
	       4019 => "Error Committing Database changes",
	       4020 => "Could not find Report ID",
	       4021 => "Count not returned from Database",
	       4022 => "Ledger Code required",
	       4023 => "Could not find Ledger Database",
	       4024 => "Could not find Ledger Name",
	       4025 => "Could not find Ledger Code",
	       4026 => "Ledger DB Name required",
	       4027 => "Ledger Name required",
	       4028 => "Database Handle required",
	       4029 => "Memorized value required",
	       4030 => "Invalid ID format",
	       4031 => "Invalid Item in Row",
	       4032 => "Item missing from Row",
	       4033 => "Invalid Value for Row Item",
	       4034 => "Parameter Required",
	       4035 => "File Name required",
	     );

my %sql_statements;
%sql_statements = (  1000 => "SELECT num, name FROM sub_acct_type_tb ORDER BY num",
			1001 => "SELECT sub_acc_number, sub_acc_name FROM sub_acc_tb WHERE type = ? ORDER BY sub_acc_number",
			1002 => "INSERT INTO report_type_tb (name, report_class, report_type, column_variable, column_variable_data, date_range, display_dates, max_columns, min_columns, text_font, text_size, data_font, data_size, total_column, fg_color, bg_color, underline_length, editable) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, 'true')",
			1003 => "SELECT id FROM report_type_tb WHERE name = ?",
			1004 => "UPDATE report_type_tb SET name = ?, report_class = ?, report_type = ?, column_variable = ?, column_variable_data = ?, date_range = ?, display_dates = ?, max_columns = ?, min_columns = ?, text_font = ?, text_size = ?, data_font = ?, data_size = ?, total_column = ?, fg_color = ?, bg_color = ?, underline_length = ? WHERE id = ?",
			1005 => "SELECT name FROM report_type_tb WHERE id = ?",
			1006 => "SELECT name, report_class, report_type, column_variable, column_variable_data, date_range, display_dates, max_columns, min_columns, text_font, text_size, data_font, data_size, total_column, fg_color, bg_color, underline_length, editable FROM report_type_tb WHERE id = ?",
			1008 => "DELETE FROM report_data_tb WHERE id = ?",
			1009 => "DELETE FROM report_type_tb WHERE id = ?",
			1014 => "SELECT count(name) FROM memorized_report_tb WHERE id = ?",
			1016 => "SELECT name FROM entity_tb WHERE database_name = ?",
			1017 => "SELECT code FROM ledger_tb WHERE name = ?",
			1018 => "SELECT color FROM ledger_tb WHERE code = ?",
			1019 => "SELECT name, num FROM acct_type_tb ORDER BY name, num",
			1020 => "SELECT name, number, type, active FROM account_tb ORDER BY name, number, type, active",
			1021 => "SELECT sub_acc_name, sub_acc_number, type FROM sub_acc_tb ORDER BY sub_acc_name, sub_acc_number, type",
			1022 => "SELECT DISTINCT(report_type_tb.id), report_type_tb.name, report_type_tb.editable FROM report_type_tb, report_data_tb WHERE report_type_tb.report_type = 'd' AND report_type_tb.id = report_data_tb.id ORDER BY report_type_tb.name, report_type_tb.id",
			1024 => "SELECT index, cell_text, cell_type, item_name, fg_color, bg_color, display_level, initial_state, negative_format, other_attributes FROM report_data_tb WHERE id = ? ORDER BY index",
			1025 => "DELETE FROM report_data_tb WHERE id = ?",
			1026 => "INSERT INTO report_data_tb (id, index, cell_text, cell_type, item_name, fg_color, bg_color, display_level, initial_state, negative_format, other_attributes) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
			1028 => "SELECT year, start_date, end_date, q1_start, q1_end, q2_start, q2_end, q3_start, q3_end, q4_start, q4_end FROM known_years_tb WHERE ledger_code = ? ORDER BY year",
			1029 => "INSERT INTO temp_file_tb (timeof, fname) VALUES (now(), ?)",
			1030 => "DELETE FROM memorized_report_tb WHERE id = ? AND ledger_code = ? AND name = ?",
			1033 => "SELECT name, code FROM sub_entity_tb WHERE ledger_code = ?",
			1034 => "SELECT sub_acc_name FROM sub_acc_tb WHERE sub_acc_number = ?",
			1036 => "SELECT name, id, ledger_code FROM memorized_report_tb ORDER BY ledger_code, name, id",
			1037 => "SELECT name FROM memorized_report_tb WHERE id = ? AND name = ? AND ledger_code = ?",
			1038 => "SELECT editable FROM report_type_tb WHERE id = ?",
			1040 => "SELECT name, id, editable FROM report_type_tb WHERE report_type = 'r' ORDER BY name, id",
			1041 => "SELECT name, id, editable FROM report_type_tb WHERE report_type = 'd' ORDER BY name, id",
			1042 => "SELECT name FROM acct_type_tb WHERE num = ?",
			1043 => "SELECT debit_increases FROM acct_type_tb WHERE num = ?",
			1044 => "SELECT number FROM account_tb WHERE type = ? ORDER BY number",
			1045 => "SELECT acct_num FROM local_accounts_tb WHERE ledger_code = ? AND taxable = true AND sub_entity_code IS NULL AND acct_num IN (##?##) ORDER BY acct_num",
			1046 => "SELECT acct_num FROM local_accounts_tb WHERE ledger_code = ? AND taxable = false AND sub_entity_code IS NULL AND acct_num IN (##?##) ORDER BY acct_num",
			1047 => "SELECT acct_num FROM local_accounts_tb WHERE ledger_code = ? AND sub_entity_code IS NULL AND acct_num IN (##?##) ORDER BY acct_num",
			1048 => "SELECT name FROM account_tb WHERE number = ?",
			1049 => "SELECT num FROM acct_type_tb ##?## ORDER BY num",
			1054 => "SELECT sub_acc_name, sub_acc_number FROM sub_acc_tb WHERE type IN (##1?##) ORDER BY ##2?##",
			1055 => "SELECT sub_acct_type FROM acct_type_tb WHERE num = ?",
			1056 => "SELECT account_tb.type, acct_type_tb.debit_increases FROM account_tb, acct_type_tb WHERE account_tb.number = ? AND acct_type_tb.num = account_tb.type",
			1057 => "SELECT sub_acc_name FROM sub_acc_tb WHERE sub_acc_number = ?",
			1058 => "SELECT name FROM sub_acct_type_tb WHERE num = ?",
			1059 => "SELECT type FROM sub_acc_tb WHERE sub_acc_number = ?",
			1060 => "SELECT num, sub_acct_type FROM acct_type_tb",
			1064 => "SELECT report_type FROM report_type_tb WHERE id = ?",
			1067 => "SET datestyle TO 'POSTGRES,US'",
			1072 => "SELECT working_year(?)",
		     );

my %report_classes;
%report_classes = ( "custom" => "Custom", "cash" => "Cash Flow", "income" => "Income Statement", "balance" => "Balance Sheet", "acct_balance" => "Accounts Balance Sheet", "acct_activity" => "Accounts Activity Sheet" );
my @report_classes;
@report_classes = ( "custom", "cash", "income", "balance", "acct_balance", "acct_activity" );
my %report_types;
%report_types = ( "r" => "Regular Report", "d" => "Detailed View" );
my @report_types;
@report_types = ( "r", "d" );
my @report_levels;
@report_levels = ( "account_type", "account", "sub_account", "accounts_of", "sub_accounts_of_type", "sub_accounts_of_account", "begin_account_type_balance", "end_account_type_balance", "begin_account_balance", "end_account_balance", "begin_sub_account_balance", "end_sub_account_balance" );
my @account_type_levels;
@account_type_levels = ( "nothing", "accounts", "accounts, sub-accounts" );
my @account_levels;
@account_levels = ( "nothing", "sub-accounts" );
my @accounts_of_levels;
@accounts_of_levels = ( "nothing", "sub-accounts" );
my @sub_account_levels;
@sub_account_levels = ( "nothing", "accounts" );
my @sub_accounts_of_type_levels;
@sub_accounts_of_type_levels = ( "nothing", "accounts" );
my @sub_accounts_of_account_levels;
@sub_accounts_of_account_levels = ( "not available" );
my %report_levels;
%report_levels = ( "account_type" => \@account_type_levels, "account" => \@account_levels, "sub_account" => \@sub_account_levels, "accounts_of" => \@accounts_of_levels, "sub_accounts_of_type" => \@sub_accounts_of_type_levels, "sub_accounts_of_account" => \@sub_accounts_of_account_levels, "begin_account_type_balance" => \@account_type_levels, "end_account_type_balance" => \@account_type_levels, "begin_account_balance" => \@account_levels, "end_account_balance" => \@account_levels, "begin_sub_account_balance" => \@sub_account_levels, "end_sub_account_balance" => \@sub_account_levels );
my %regular_cell_types;
%regular_cell_types = ( "text" => "Text", "total" => "Total", "compare" => "Compare Rows", "account_type" => "Account Type", "account" => "Account", "sub_account" => "Sub Account", "accounts_of" => "Accounts of Account Type", "sub_accounts_of_type" => "Sub Accounts of Account Type", "sub_accounts_of_account" => "Sub Accounts of Account", "begin_account_type_balance" => "Beginning Account Type Balance", "end_account_type_balance" => "Ending Account Type Balance", "begin_account_balance" => "Beginning Account Balance", "end_account_balance" => "Ending Account Balance", "begin_sub_account_balance" => "Beginning Sub Account Balance", "end_sub_account_balance" => "Ending Sub Account Balance", "line" => "Line" );
my @regular_cell_types;
@regular_cell_types = ( "text", "total", "compare", "account_type", "account", "sub_account", "accounts_of", "sub_accounts_of_type", "sub_accounts_of_account", "begin_account_type_balance", "end_account_type_balance", "begin_account_balance", "end_account_balance", "line" ); # "begin_sub_account_balance", "end_sub_account_balance", (to be added later)
my %detailed_cell_types;
%detailed_cell_types = ( "text" => "Text", "line" => "Line", "detail" => "Detail" );
my @detailed_cell_types;
@detailed_cell_types = ( "text", "line", "detail" );
my @font_names;
@font_names = ( "helvetica" );
my %font_names;
%font_names = ( "helvetica" => "Helvetica" );
my @font_sizes;
@font_sizes = ( "+2", "+1", "+0", "-1", "-2" );
my %font_sizes;
%font_sizes = ( "+2" => "+2", "+1" => "+1", "+0" => "nothing", "-1" => "-1", "-2" => "-2" );
my @fg_colors;
@fg_colors = ( "black", "blue", "red" );
my %fg_colors;
%fg_colors = ( "black" => "Black", "blue" => "Blue", "red" => "Red" );
my @bg_colors;
@bg_colors = ( "white", "yellow", "pink" );
my %bg_colors;
%bg_colors = ( "white" => "White", "yellow" => "Yellow", "pink" => "Pink" );
my %column_names;
%column_names = ( 0 => "Zero", 1 => "One", 2 => "Two", 3 => "Three", 4 => "Four", 5 => "Five", 6 => "Six", 7 => "Seven", 8 => "Eight", 9 => "Nine", 10 => "Ten", 11 => "Eleven", 12 => "Twelve", 13 => "Thirteen", 14 => "Fourteen", 15 => "Fifteen", 16 => "Sixteen", 17 => "Seventeen", 18 => "Eighteen", 19 => "Nineteen", 20 => "Twenty", 21 => "Twenty One", 22 => "Twenty Two", 23 => "Twenty Three", 24 => "Twenty Four", 25 => "Twenty Five", 26 => "Twenty Six", 27 => "Twenty Seven", 28 => "Twenty Eight", 29 => "Twenty Nine", 30 => "Thirty", 31 => "Thirty One", 32 => "Thirty Two", 33 => "Thirty Three", 34 => "Thirty Four", 35 => "Thirty Five", 36 => "Thirty Six" );
my @column_headers;
@column_headers = ( "sub_entities", "sub_accounts", "sub_account_types" );
my %column_headers;
%column_headers = ( "sub_entities" => "Sub Entities", "sub_accounts" => "Sub Accounts", "sub_account_types" => "Sub Account Types" );
my @negative_formats;
@negative_formats = ( "0", "1", "2" );
my %negative_formats;
%negative_formats = ( "0" => "<font color=\"red\">(xxx.xx)</font>", "1" => "<font color=\"red\">-</font>xxx.xx", "2" => "xxx.xx<font color=\"red\">-</font>" );
my $max_dates;
$max_dates = 36;   # the maximum number of columns that we are currently supporting in the reporting system.
my $min_underline_length;
$min_underline_length =  5;  # the min and max lengths that underlines can range within.
my $max_underline_length;
$max_underline_length = 25;
my $default_underline_length;
$default_underline_length = -1;
my %t_or_f;
%t_or_f = ( 0 => "false", 1 => "true", "true" => "true", "false" => "false" );  # a mapping that will allow us to map 0 to false and 1 to true for the sql statements.
my %true_false;
%true_false = ( "true" => 1, "false" => 0, 1 => 1, 0 => 0 );

# commonly used compiled regular expressions
my $editReportWindowRE = qr/^(edit_report_window)$/;
my $operatorsRE = qr/^(\+|-|\*|\/)$/;
my $openParenRE = qr/^\($/;
my $closeParenRE = qr/^\)$/;
my $tokenRE = qr/^R\d+|-?\d+\.\d+|-?\d+$/;
my $rowIdentifierRE = qr/^R\d+$/;
my $substitutionRE = qr/\#\#\?\#\#/;
my $firstSubstitutionRE = qr/\#\#1\?\#\#/;
my $secondSubstitutionRE = qr/\#\#2\?\#\#/;

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";

    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";

    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
($db_name = $dbObj->dbName) =~ s/^(")([^"]+)(")$/$2/;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $db_type;
$db_type = "Pg";
my $dsn;
$dsn = "dbi:$db_type:dbname=$db_name;host=$db_host";

my $email_address;
$email_address = "james\@pcxperience.com";  # JavaScript error messages should be sent here.

%input = ();
cgi_lib::ReadParse(*input);  # gather the data from the client

use HTMLObject::Base;
use HTMLObject::ReadCookie;	# the module used to read cookies sent to us from the client.
use Storable;		# the module that will allow me to dump data to disk and recover later.  The file names will be stored in the cookie.
use POSIX qw(tmpnam);	# the tmpnam function which will return a file name for a temporary file supposed to be unique.
use DBI;	# Using new Database engine for future compatibility.

# turn on debugging in Apache::DBI
#$Apache::DBI::DEBUG = 0;

my $cookies;
$cookies = HTMLObject::ReadCookie->new;

if (!exists $input{'command'})
{
  my_die("Error: Report.cgi - command missing", 4003, "Access Denied!");
}
else
{
  my $dbh;  # the handle of the database.
  # connect to the database.  The AutoCommit option will force transactions to be used.
  $dbh = DBI->connect($dsn, $db_user, $db_passwd, { RaiseError => 0, AutoCommit => 0 }) || my_db_die("DBI Error", 4004, "Database: $db_name\@$db_host");

  # force the database session to use Postgres/US convention date styles
  my $sth = $dbh->prepare( $sql_statements{1067} ) || my_db_die("Error: Report.cgi", 4007, "SQL Statement = '$sql_statements{1067}'");
  my $rc = $sth->execute() || my_db_die("Error: Report.cgi", 4008, "SQL Statement = '$sql_statements{1067}'");

  my $command = $input{'command'};
  if ($command eq "display_for_edit")
  {
    display_for_edit( dbh => $dbh );
  }
  elsif ($command eq "create_report")
  {
    my $report_type = "";
    if ((!exists $input{'report_group'}) || (length $input{'report_group'} == 0))
    {
      my_die("Error: Report.cgi - create_report", 4017, "Report Group required at line " . __LINE__);
    }
    if ($input{'report_group'} eq "regular")
    {
      $report_type = "r";
    }
    else
    {
      $report_type = "d";
    }
    create_report_frames( dbh => $dbh, report_type => $report_type, windowName => $input{windowName} );
  }
  elsif ($command eq "create_report1")
  {
    my %args = ();
    # check and see if we are regenerating the create report screen with the user specified values and an error message or just defaults.
    %args = get_default_data();
    $args{'dbh'} = $dbh;
    $args{'mode'} = "create";
    $args{'report_type'} = $input{'report_type'};
    # call the first screen edit/create routine with the default values we want displayed.
    report_first_screen_edit_create( %args );
  }
  elsif ($command eq "create_report2")
  {
    eval("use HTMLObject::FrameSet;");
    if ($@)
    {
      my_die("Error: Report.cgi - Eval", 4000, "Error: $@ at line " . __LINE__);
    }
    my $doc = HTMLObject::FrameSet->new();
    $doc->setTitle("Create Report");

    # check for the input from the first stage of creating a report and create the entry in report_type_tb before we proceed to the next stage.
    use vars qw( $name $id $report_class $report_type $date_range $max_columns $min_columns $display_dates $text_font $data_font $text_size $data_size $fg_color $bg_color $total_column $underline_length $column_variable $column_variable_data $windowName );

    my @arguments = ( "name", "id", "report_class", "report_type", "column_variable", "column_variable_data", "date_range", "display_dates", "max_columns",
    		    "min_columns", "text_size", "text_font", "data_size", "data_font", "total_column", "fg_color", "bg_color", "underline_length", "windowName" );

    $id = 0;  # inititalize to 0.

    eval ("use HTMLObject::Normal;");
    if ($@)
    {
      my_die("Error: Report.cgi - Eval", 4002, "Error: $@ at line " . __LINE__);
    }

    my $doc2 = HTMLObject::Normal->new();

    $doc2->setJavascriptErrorHandlerEmail(email => $email_address);

    no strict 'refs';  # turn off the use of strict for symbolic references.

    foreach my $value (@arguments)
    {
      if (!exists $input{$value})
      { # error!
        $doc2->doRequiredParameterError('create_report2', $value);
      }
      if (length $input{$value} == 0)
      {
        if ($value ne "column_variable" && $value ne "column_variable_data")
        { # error!
          my_die("Error: Report.cgi - create_report2", 4005, "Parameter = '$value'");
        }
      }
      elsif (!validateParameter($value, $input{$value}))
      {
        my_die("Error: Report.cgi - create_report2", 4006, "$value = '$input{$value}'");
      }
      $$value = $input{$value};  # assign the value to this variable via a symbolic reference :).
    }

    use strict 'refs';  # turn strict checking back on.

    # first verify that this report name is not already being used.
    my $sth = $dbh->prepare( $sql_statements{1003} ) || my_db_die("Error: Report.cgi - create_report2", 4007, "SQL Statement = '$sql_statements{1003}'");
    my $rc = $sth->execute($name) || my_db_die("Error: Report.cgi - create_report2", 4008, "SQL Statement = '$sql_statements{1003}', Name = '$name'");
    my @info = $sth->fetchrow_array;
    $sth->finish;

    if (defined($info[0]))
    { # error!
      my $found = 1;  # assume we are the same report until we know for sure.
      # first check and make sure that our report_id cookie is not set and that it is not the same id as we have.  this will keep us from blowing errors on a reload page event.
      my %ch = $cookies->getCookies(cookies => ['report_id']);
      if (scalar keys %ch == 0)
      {
        $found = 0;
      }
      else
      {
        if ($cookies->cookies->{'report_id'} != $info[0])
        {
          $found = 0;
        }
      }
      if (!$found)
      {
        # call the first screen edit/create routine with the values of the selected report displayed.
        my %data = ( name => "$name", id => "$id", report_class => "$report_class", report_type => "$report_type", column_variable => "$column_variable", column_variable_data => "$column_variable_data", date_range => "$date_range",
                     display_dates => "$display_dates", max_columns => "$max_columns", min_columns => "$min_columns", text_size => "$text_size", text_font => "$text_font", data_size => "$data_size", data_font => "$data_font",
        	     total_column => "$total_column", fg_color => "$fg_color", bg_color => "$bg_color", underline_length => "$underline_length", error_string => "'$name' is already assigned to Report ID = '$info[0]'." );
        my $file_name = tmpnam(); # get the file name of the temp file to work with.
        # store the data object.
        store(\%data, $file_name);
        # my_die("Error: Report.cgi - create_report2", 4015, "Report Name = '$name' is assigned to Report ID = '$info[0]' at Line " . __LINE__);

        # now we need to generate the frameset, set the error_happened cookie with the file_name and then exit.
        create_report_frames( cookie_name => "error_happened", cookie_value => "$file_name", dbh => $dbh, report_type => $report_type, windowName => $windowName );
        goto LEAVE_PROGRAM_EARLY;
      }
      $id = $info[0];
    }
    else
    {
      # now create this entry in the database.
      $sth = $dbh->prepare( $sql_statements{1002} ) || my_db_die("Error: Report.cgi - create_report2", 4007, "SQL Statement = '$sql_statements{1002}'");
      $rc = $sth->execute($name, $report_class, $report_type, $column_variable, $column_variable_data, $date_range, $display_dates, $max_columns, $min_columns, $text_font, $text_size, $data_font, $data_size, $total_column, $fg_color, $bg_color, $underline_length);
      if ($sth->err)
      {
      	my_db_die("Error: Report.cgi - create_report2", 4008, "SQL Statement = '$sql_statements{1002}'<br>\nName = '$name'<br>\nReport Class = '$report_class'<br>\nReport Type = '$report_type'<br>\n\
      		       		     Column Variable = '$column_variable'<br>\nColumn Variable Data = '$column_variable_data'<br>\nDate Range = '$date_range'<br>\nDisplay Dates = '$display_dates'<br>\n\
      		       		     Max Columns = '$max_columns'<br>\nMin Columns = '$min_columns'<br>\nText Font = '$text_font'<br>\nText Size = '$text_size'<br>\nData Font = '$data_font'<br>\n\
      		       		     Data Size = '$data_size'<br>\nTotal Column = '$total_column'<br>\nForeground Color = '$fg_color'<br>\nBackground Color = '$bg_color'<br>\nUnder Line Length = '$underline_length'<br>\n");
      }

      # commit the transaction to the database
      $dbh->commit || my_db_die("Error: Report.cgi - create_report2", 4019, "Tried to commit work!");

      # get the id of the report we just created.
      $sth = $dbh->prepare( $sql_statements{1003} ) || my_db_die("Error: Report.cgi - create_report2", 4007, "SQL Statement = '$sql_statements{1003}'");
      $rc = $sth->execute($name) || my_db_die("Error: Report.cgi - create_report2", 4008, "SQL Statement = '$sql_statements{1003}', Name = '$name'");
      @info = $sth->fetchrow_array;
      $sth->finish;

      if (defined(@info))
      {
        $id = $info[0];
      }

      # set a cookie that says what the id of the report is that we just created.
      $doc->setCookie(name => 'report_id', value => "$id");
      my %ch = $cookies->getCookies(cookies => ['error_happened']);
      if (scalar keys %ch == 0)
      {
        # reset the cookie so that it is forgotten.
        my $date = `/bin/date --date "1 day ago" "+%a, %d-%b-%Y %T GMT"`;
        chomp $date;

        $doc->setCookie(name => 'error_happened', value => '', expires => "$date");
      }
    }

    # create the frameset and call the next step of the process.

    my $inner_content;
    $inner_content = $doc->createNoframe("You must have a frames capable browser!");
    $inner_content .= $doc->createFrame(src => "$site_dir/cgi-bin/report_editor.cgi?command=create_report2_top&id=$id", frameborder => 'yes', scrolling => 'auto', marginwidth => '0', marginheight => '0', border => '1', name => $windowName . '_top', bordercolor => 'black');
    $inner_content .= $doc->createFrame(src => "$site_dir/cgi-bin/report.cgi?command=bottom", frameborder => 'yes', scrolling => 'auto', name => $windowName . '_middle', marginwidth => '1', bordercolor => 'white');
    $inner_content .= $doc->createFrame(src => "$site_dir/cgi-bin/report.cgi?command=bottom", frameborder => 'yes', scrolling => 'auto', name => $windowName . '_bottom', marginwidth => '1', bordercolor => 'white');
    $doc->outerFrameset(rows => '*, *, 2*', content => $inner_content, border => '5', frameborder => 'yes');

    $doc->display();
  }
  elsif ($command eq "create_report2_top")
  {
    # check to make sure we were given the id of the report we are creating.
    my $id = $input{'id'};
    if (length $id == 0)
    {
      my_die("Error: Report.cgi - create_report2_top", 4017, "Report ID required at line " . __LINE__);
    }

    # call the report second screen create/edit function with the specified ID to work with.
    report_second_screen_edit_create( mode => "create", id => "$id", dbh => $dbh );
  }
  elsif ($command eq "update_report")
  {
    eval ("use HTMLObject::Normal;");
    if ($@)
    {
      my_die("Error: Report.cgi - Eval", 4002, "Error: $@ at line " . __LINE__);
    }

    my $doc = HTMLObject::Normal->new();

    $doc->setJavascriptErrorHandlerEmail(email => $email_address);

    # validate I was given a report to delete.
    use vars qw( $id $rows $mode $report_type $windowName );
    my @arguments = ( "id", "rows", "mode", "report_type", "windowName" );
    no strict 'refs';  # turn off the use of strict for symbolic references.

    foreach my $value (@arguments)
    {
      if (!exists $input{$value})
      { # error!
        $doc->doRequiredParameterError('update_report', $value);
      }
      if (length $input{$value} == 0)
      {
        my_die("Error: Report.cgi - update_report", 4005, "Parameter = '$value'");
      }
      elsif (!validateParameter($value, $input{$value}))
      {
        my_die("Error: Report.cgi - update_report", 4006, "$value = '$input{$value}'");
      }
      $$value = $input{$value};  # assign the value to this variable via a symbolic reference :).
    }

    use strict 'refs';  # turn strict checking back on.

    # next check and make sure it still exists.
    my $sth = $dbh->prepare( $sql_statements{1005} ) || my_db_die("Error: Report.cgi - update_report", 4007, "SQL Statement = '$sql_statements{1005}'");
    my $rc = $sth->execute($id) || my_db_die("Error: Report.cgi - update_report", 4008, "SQL Statement = '$sql_statements{1005}', ID = '$id'");
    my @info = $sth->fetchrow_array;
    $sth->finish;

    if (!defined @info)
    {
      #my_die("Error: Report.cgi - update_report", 4020, "ID = '$id'");
      display_for_edit( error_string => "Report ID = '$id' no longer exists! &nbsp;It may have been deleted.", dbh => $dbh );
      goto LEAVE_PROGRAM_EARLY;
    }
    my $name = $info[0];

    # extract the rows from the input and parse apart
    my @rows = parse_rows( rows => $rows, dbh => $dbh, report_type => $report_type );

    # if it does, first delete all row entries from the database.
    $sth = $dbh->prepare( $sql_statements{1025} ) || my_db_die("Error: Report.cgi - update_report", 4007, "SQL Statement = '$sql_statements{1025}'");
    $rc = $sth->execute($id);
    if ($sth->err)
    {
      my_db_die("Error: Report.cgi - update_report", 4008, "SQL Statement = '$sql_statements{1025}', ID = '$id'");
    }

    # now we insert all the rows
    $sth = $dbh->prepare( $sql_statements{1026} ) || my_db_die("Error: Report.cgi - update_report", 4007, "SQL Statement = '$sql_statements{1026}'");
    for (my $i=0; $i < scalar @rows; $i++)
    {
      my $other_attributes = "";
      my $cell_type = $rows[$i][3];
      if ($cell_type eq "text")
      {
        $other_attributes = "bold=" . $true_false{$rows[$i][7]};
      }
      elsif ($cell_type eq "total")
      {
        $other_attributes = "bold=" . $true_false{$rows[$i][7]} . "|cell_formula=" . $rows[$i][2] . "|show=" . $rows[$i][16] . "|underline=" . $rows[$i][18];
      }
      elsif ($cell_type eq "line")
      {
        $other_attributes = "bold=" . $true_false{$rows[$i][7]};
      }
      elsif ($cell_type eq "compare")
      {
        $other_attributes = "bold=" . $true_false{$rows[$i][7]} . "|cell_formula=" . $rows[$i][2] . "|result_type=" . $rows[$i][17] . "|show=" . $rows[$i][16] . "|underline=" . $rows[$i][18];
      }
      elsif ($cell_type eq "account_type" || $cell_type eq "accounts_of" || $cell_type eq "sub_accounts_of_type" || $cell_type eq "begin_account_type_balance" || $cell_type eq "end_account_type_balance" ||
      $cell_type eq "account" || $cell_type eq "sub_accounts_of_account" || $cell_type eq "begin_account_balance" || $cell_type eq "end_account_balance" ||
      $cell_type eq "sub_account")
      {
        $other_attributes = "bold=" . $true_false{$rows[$i][7]} . "|sub_levels=" . $rows[$i][8] . "|drillable=" . $true_false{$rows[$i][11]} . "|drill_report=" . $rows[$i][12] . "|activity=" . $true_false{$rows[$i][14]} . "|query_type=" . $rows[$i][15] . "|show=" . $rows[$i][16];
      }
      elsif ($cell_type eq "detail")
      {
        $other_attributes = "bold=" . $true_false{$rows[$i][7]};
      }
      $rc = $sth->execute($id, $rows[$i][0], $rows[$i][1], $rows[$i][3], $rows[$i][4], $rows[$i][5], $rows[$i][6], $rows[$i][9], $rows[$i][10], $rows[$i][13], $other_attributes)
          || my_db_die("Error: Report.cgi - update_report", 4008, "SQL Statement = '$sql_statements{1026}', ID = '$id', index='$rows[$i][0]', cell_text='$rows[$i][1]', cell_type='$rows[$i][3]', item_name='$rows[$i][4]', fg_color='$rows[$i][5]', bg_color='$rows[$i][6]', display_level='$rows[$i][9]', initial_state='$rows[$i][10]', negative_format='$rows[$i][13]', other_attributes='$other_attributes'");
    }

    # commit the transaction to the database
    $dbh->commit || my_db_die("Error: Report.cgi - update_report", 4019, "Tried to commit work!");

    # regenerate the display_for_edit screen
    if ((!exists $input{'redisplay'}) || $input{'redisplay'} eq "false")
    {
      display_for_edit( status_string => "Report '$name' = $id, Successfully " . ($mode eq "create" ? "Created" : "Edited"), dbh => $dbh );
    }
    else
    {
      eval("use HTMLObject::FrameSet;");
      if ($@)
      {
        my_die("Error: Report.cgi - Eval", 4000, "Error: $@ at line " . __LINE__);
      }
      my $doc = HTMLObject::FrameSet->new();

      $doc->setTitle("Edit Report - $name");

      my $inner_content;
      $inner_content = $doc->createNoframe("You must have a frames capable browser!");
      $inner_content .= $doc->createFrame(src => "$site_dir/cgi-bin/report_editor.cgi?command=edit_report2_top&id=$id", frameborder => 'yes', scrolling => 'auto', marginwidth => '0', marginheight => '0', border => '1', name => $windowName . '_top', bordercolor => 'black');
      $inner_content .= $doc->createFrame(src => "$site_dir/cgi-bin/report.cgi?command=bottom", frameborder => 'yes', scrolling => 'auto', name => $windowName . '_middle', marginwidth => '1', bordercolor => 'white');
      $inner_content .= $doc->createFrame(src => "$site_dir/cgi-bin/report.cgi?command=bottom", frameborder => 'yes', scrolling => 'auto', name => $windowName . '_bottom', marginwidth => '1', bordercolor => 'white');
      $doc->outerFrameset(rows => '*, *, 2*', content => $inner_content, border => '5', frameborder => 'yes');

      $doc->display();
    }
  }
  elsif ($command eq "copy_report")
  {
    eval ("use HTMLObject::Normal;");
    if ($@)
    {
      my_die("Error: Report.cgi - Eval", 4002, "Error: $@ at line " . __LINE__);
    }

    my $doc = HTMLObject::Normal->new();

    $doc->setJavascriptErrorHandlerEmail(email => $email_address);

    # validate I was given a report to delete.
    use vars qw( $id $report_group $new_report_name );
    my @arguments = ( "id", "report_group", "new_report_name" );
    no strict 'refs';  # turn off the use of strict for symbolic references.

    foreach my $value (@arguments)
    {
      if (!exists $input{$value})
      { # error!
        $doc->doRequiredParameterError('copy_report', $value);
      }
      if (length $input{$value} == 0)
      {
        my_die("Error: Report.cgi - copy_report", 4005, "Parameter = '$value'");
      }
      elsif (!validateParameter($value, $input{$value}))
      {
        my_die("Error: Report.cgi - copy_report", 4006, "$value = '$input{$value}'");
      }
      $$value = $input{$value};  # assign the value to this variable via a symbolic reference :).
    }

    use strict 'refs';  # turn strict checking back on.

    if ($report_group eq "memorized")
    {
      display_for_edit( error_string => "Copy Failed! &nbsp;You can not copy a Memorized Report!", dbh => $dbh );
    }
    else
    {
      my $report_name = DoesReportExist(dbh => $dbh, id => $id);
      if (length $report_name == 0)
      {
        display_for_edit( error_string => "Copy Failed! &nbsp;Report with ID = '$id' does not exist!", dbh => $dbh );
      }
      elsif ($report_name eq $new_report_name)
      {
        display_for_edit( error_string => "Copy Failed! &nbsp;You must specify a different name for Report '$report_name' = ($id)!", dbh => $dbh );
      }
      else
      {
        my $new_id = FindReportByName(dbh => $dbh, name => $new_report_name);
        if ($new_id != -1)
        {
          display_for_edit( error_string => "Copy Failed! &nbsp;Report '$new_report_name' = ($new_id) already exists!", dbh => $dbh );
        }
        else
        {
          my @report_type_data = GetReportData(dbh => $dbh, id => $id);
          my $report_class = $report_type_data[1];
          my $report_type = $report_type_data[2];
          my $column_variable = $report_type_data[3];
          my $column_variable_data = $report_type_data[4];
          my $date_range = $report_type_data[5];
          my $display_dates = $report_type_data[6];
          my $max_columns = $report_type_data[7];
          my $min_columns = $report_type_data[8];
          my $text_font = $report_type_data[9];
          my $text_size = $report_type_data[10];
          my $data_font = $report_type_data[11];
          my $data_size = $report_type_data[12];
          my $total_column = $report_type_data[13];
          my $fg_color = $report_type_data[14];
          my $bg_color = $report_type_data[15];
          my $underline_length = $report_type_data[16];

          # create the report_type_tb entry and gather the id of the report
          my $sth = $dbh->prepare( $sql_statements{1002} ) || my_db_die("Error: Report.cgi - copy_report", 4007, "SQL Statement = '$sql_statements{1002}'");
          my $rc = $sth->execute($new_report_name, $report_class, $report_type, $column_variable, $column_variable_data, $date_range, $display_dates, $max_columns, $min_columns, $text_font, $text_size, $data_font, $data_size, $total_column, $fg_color, $bg_color, $underline_length);
          if ($sth->err)
          {
          	my_db_die("Error: Report.cgi - copy_report", 4008, "SQL Statement = '$sql_statements{1002}'<br>\nName = '$new_report_name'<br>\nReport Class = '$report_class'<br>\nReport Type = '$report_type'<br>\n\
          		       		     Column Variable = '$column_variable'<br>\nColumn Variable Data = '$column_variable_data'<br>\nDate Range = '$date_range'<br>\nDisplay Dates = '$display_dates'<br>\n\
          		       		     Max Columns = '$max_columns'<br>\nMin Columns = '$min_columns'<br>\nText Font = '$text_font'<br>\nText Size = '$text_size'<br>\nData Font = '$data_font'<br>\n\
          		       		     Data Size = '$data_size'<br>\nTotal Column = '$total_column'<br>\nForeground Color = '$fg_color'<br>\nBackground Color = '$bg_color'<br>\nUnder Line Length = '$underline_length'<br>\n");
          }
          		       		
          # commit the transaction to the database
          $dbh->commit || my_db_die("Error: Report.cgi - copy_report", 4019, "Tried to commit work!");

          # get the id of the report we just created.
          $sth = $dbh->prepare( $sql_statements{1003} ) || my_db_die("Error: Report.cgi - copy_report", 4007, "SQL Statement = '$sql_statements{1003}'");
          $rc = $sth->execute($new_report_name) || my_db_die("Error: Report.cgi - copy_report", 4008, "SQL Statement = '$sql_statements{1003}', Name = '$new_report_name'");
          my @info = $sth->fetchrow_array;
          $sth->finish;

          if (defined(@info))
          {
            $new_id = $info[0];
            my @rows = GetReportRows(dbh => $dbh, id => $id);

            # create the report_data_tb entries using the new id of the report.
            $sth = $dbh->prepare( $sql_statements{1026} ) || my_db_die("Error: Report.cgi - copy_report", 4007, "SQL Statement = '$sql_statements{1026}'");
            for (my $i=0; $i < scalar @rows; $i++)
            {
              $rc = $sth->execute($new_id, $rows[$i]{'index'}, $rows[$i]{'cell_text'}, $rows[$i]{'cell_type'}, $rows[$i]{'item_name'}, $rows[$i]{'fg_color'}, $rows[$i]{'bg_color'}, $rows[$i]{'display_level'}, $rows[$i]{'initial_state'}, $rows[$i]{'negative_format'}, $rows[$i]{'other_attributes'})
                  || my_db_die("Error: Report.cgi - copy_report", 4008, "SQL Statement = '$sql_statements{1026}', ID = '$id', index='$rows[$i]{'index'}', cell_text='$rows[$i]{'cell_text'}', cell_type='$rows[$i]{'cell_type'}', item_name='$rows[$i]{'item_name'}', fg_color='$rows[$i]{'fg_color'}', bg_color='$rows[$i]{'bg_color'}', display_level='$rows[$i]{'display_level'}', initial_state='$rows[$i]{'initial_state'}', negative_format='$rows[$i]{'negative_format'}', other_attributes='$rows[$i]{'other_attributes'}'");
            }

            # commit the transaction to the database
            $dbh->commit || my_db_die("Error: Report.cgi - copy_report", 4019, "Tried to commit work!");

            display_for_edit( status_string => "Copy Succeeded! &nbsp;Report '$new_report_name' = ($new_id) created!", dbh => $dbh );
          }
          else
          {
            display_for_edit( error_string => "Copy Failed! &nbsp;Creation of Report '$new_report_name' failed!&nbsp; New Report ID not defined!", dbh => $dbh );
          }
        }
      }
    }
  }
  elsif ($command eq "edit_report")
  {
    edit_report_frames( dbh => $dbh, windowName => $input{windowName} );
  }
  elsif ($command eq "edit_report1")
  {
    my %ch = $cookies->getCookies(cookies => ['error_happened']);
    if (scalar keys %ch == 1)
    {
      my %args = ();
      # check and see if we are regenerating the create report screen with the user specified values and an error message or just defaults.
      %args = get_default_data();
      $args{'dbh'} = $dbh;
      $args{'mode'} = "edit";
      $args{'report_type'} = $input{'report_type'};
      report_first_screen_edit_create( %args );
    }
    else
    {
      # check to make sure we were given the id of the report we are editing.
      my $id = $input{'id'};
      if (length $id == 0)
      {
        my_die("Error: Report.cgi - edit_report", 4017, "Report ID required at line " . __LINE__);
      }

      # Gather the data from the database for the specified report.
      my $sth = $dbh->prepare( $sql_statements{1006} ) || my_db_die("Error: Report.cgi - edit_report1", 4007, "SQL Statement = '$sql_statements{1006}'");
      my $rc = $sth->execute($id) || my_db_die("Error: Report.cgi - edit_report1", 4008, "SQL Statement = '$sql_statements{1006}', ID = '$id'");
      my @info = $sth->fetchrow_array;
      $sth->finish;

      if (!defined @info)
      {
        my_die("Error: Report.cgi - edit_report1", 4020, "ID = '$id'");
      }
      # call the first screen edit/create routine with the values of the selected report displayed.
      report_first_screen_edit_create( mode => "edit", name => $info[0], id => $id, report_class => $info[1], report_type => $info[2], column_variable => $info[3], column_variable_data => $info[4], date_range => "$t_or_f{$info[5]}",
      				    display_dates => "$t_or_f{$info[6]}", max_columns => $info[7], min_columns => $info[8], text_size => "$info[10]", text_font => $info[9], data_size => "$info[12]", data_font => $info[11],
      				    total_column => "$t_or_f{$info[13]}", fg_color => $info[14], bg_color => $info[15], underline_length => $info[16], dbh => $dbh );
    }
  }
  elsif ($command eq "delete_report")
  {
    eval ("use HTMLObject::Normal;");
    if ($@)
    {
      my_die("Error: Report.cgi - Eval", 4002, "Error: $@ at line " . __LINE__);
    }

    my $doc = HTMLObject::Normal->new();

    $doc->setJavascriptErrorHandlerEmail(email => $email_address);

    # validate I was given a report to delete.
    use vars '$id';
    my @arguments = ( "id" );
    no strict 'refs';  # turn off the use of strict for symbolic references.

    foreach my $value (@arguments)
    {
      if (!exists $input{$value})
      { # error!
        $doc->doRequiredParameterError('delete_report', $value);
      }
      if (length $input{$value} == 0)
      {
        my_die("Error: Report.cgi - delete_report", 4005, "Parameter = '$value'");
      }
      elsif (!validateParameter($value, $input{$value}))
      {
        my_die("Error: Report.cgi - delete_report", 4006, "$value = '$input{$value}'");
      }
      $$value = $input{$value};  # assign the value to this variable via a symbolic reference :).
    }

    use strict 'refs';  # turn strict checking back on.

    # see if we are working with a normal report or a memorized report.  Memorized reports have 2 |'s in them.
    my @id_parts = split /\|/, $id;
    my $memorized;
    if (scalar @id_parts == 3)
    {
      $memorized = 1;
    }
    else
    {
      $memorized = 0;
    }

    # next check and make sure it still exists.
    if (!$memorized)
    {
      my $name = "";
      if (length ($name = DoesReportExist(dbh => $dbh, id => $id)) == 0)
      {
        #my_die("Error: Report.cgi - delete_report", 4020, "ID = '$id'");
        display_for_edit( error_string => "Report ID = '$id' no longer exists! &nbsp;It appears to have been deleted already.", dbh => $dbh );
        goto LEAVE_PROGRAM_EARLY;
      }

      if (!IsReportEditable(dbh => $dbh, id => $id))
      {
        display_for_edit( error_string => "Report ID = '$id' is a System Report! &nbsp;You can not delete it.", dbh => $dbh );
        goto LEAVE_PROGRAM_EARLY;
      }

      # if it does, first make sure there are no memorized reports associated with this report.  If so, blow an error.
      my $sth = $dbh->prepare( $sql_statements{1014} ) || my_db_die("Error: Report.cgi - delete_report", 4007, "SQL Statement = '$sql_statements{1014}'");
      my $rc = $sth->execute($id) || my_db_die("Error: Report.cgi - delete_report", 4008, "SQL Statement = '$sql_statements{1014}', ID = '$id'");
      my @info = $sth->fetchrow_array;
      $sth->finish;

      if (!defined @info)
      {
        my_die("Error: Report.cgi - delete_report", 4021, "ID = '$id'");
      }
      if ($info[0] > 0)
      {
        display_for_edit( error_string => "You have '$info[0]' memorized reports defined for Report '$name' = $id.", dbh => $dbh );
        goto LEAVE_PROGRAM_EARLY;
      }

      # else we can go ahead and delete the data for the report and then the report itself.
      $sth = $dbh->prepare( $sql_statements{1008} ) || my_db_die("Error: Report.cgi - delete_report", 4007, "SQL Statement = '$sql_statements{1008}'");
      $rc = $sth->execute($id) || my_db_die("Error: Report.cgi - delete_report", 4008, "SQL Statement = '$sql_statements{1008}', ID = '$id'");

      $sth = $dbh->prepare( $sql_statements{1009} ) || my_db_die("Error: Report.cgi - delete_report", 4007, "SQL Statement = '$sql_statements{1009}'");
      $rc = $sth->execute($id) || my_db_die("Error: Report.cgi - delete_report", 4008, "SQL Statement = '$sql_statements{1009}', ID = '$id'");

      # commit the transaction to the database
      $dbh->commit || my_db_die("Error: Report.cgi - delete_report", 4019, "Tried to commit work!");

      # regenerate the display_for_edit screen
      display_for_edit( status_string => "Report '$name' = $id Successfully Deleted.", dbh => $dbh );
    }
    else
    {
      if (!DoesMemorizedReportExist(dbh => $dbh, id => $id_parts[0], report_name => $id_parts[1], ledger_code => $id_parts[2]))
      {
        display_for_edit( error_string => "Memorized Report ID = '$id_parts[0]', Name = '$id_parts[1]', Ledger Code = '$id_parts[2]' no longer exists! &nbsp;It appears to have been deleted already.", dbh => $dbh );
        goto LEAVE_PROGRAM_EARLY;
      }

      # now delete this memorized report.
      my $sth = $dbh->prepare( $sql_statements{1030} ) || my_db_die("Error: Report.cgi - delete_report", 4007, "SQL Statement = '$sql_statements{1030}'");
      my $rc = $sth->execute($id_parts[0], $id_parts[2], $id_parts[1]) || my_db_die("Error: Report.cgi - delete_report", 4008, "SQL Statement = '$sql_statements{1030}', ID = '$id_parts[0]', Name = '$id_parts[1]', Ledger Code = '$id_parts[2]'");

      # commit the transaction to the database
      $dbh->commit || my_db_die("Error: Report.cgi - delete_report", 4019, "Tried to commit work!");

      # regenerate the display_for_edit screen
      display_for_edit( status_string => "Memorized Report '$id_parts[1]' = $id_parts[0] ($id_parts[2]) Successfully Deleted.", dbh => $dbh );
    }
  }
  elsif ($command eq "edit_report2")
  {
    eval("use HTMLObject::FrameSet;");
    if ($@)
    {
      my_die("Error: Report.cgi - Eval", 4000, "Error: $@ at line " . __LINE__);
    }
    my $doc = HTMLObject::FrameSet->new();

    # check for the input from the first stage of editing the report and update the entry in report_type_tb before we proceed to the next stage.
    use vars qw( $name $id $original_name $report_class $report_type $date_range $max_columns $min_columns $display_dates $text_font $data_font $text_size $data_size $fg_color $bg_color $total_column $underline_length $column_variable $column_variable_data );

    my @arguments = ( "name", "original_name", "id", "report_class", "report_type", "column_variable", "column_variable_data", "date_range", "display_dates", "max_columns",
    		    "min_columns", "text_size", "text_font", "data_size", "data_font", "total_column", "fg_color", "bg_color", "underline_length" );

    eval ("use HTMLObject::Normal;");
    if ($@)
    {
      my_die("Error: Report.cgi - Eval", 4002, "Error: $@ at line " . __LINE__);
    }

    my $doc2 = HTMLObject::Normal->new();

    $doc2->setJavascriptErrorHandlerEmail(email => $email_address);

    no strict 'refs';  # turn off the use of strict for symbolic references.

    foreach my $value (@arguments)
    {
      if (!exists $input{$value})
      { # error!
        $doc2->doRequiredParameterError('edit_report2', $value);
      }
      if (length $input{$value} == 0)
      {
        if ($value ne "column_variable" && $value ne "column_variable_data")
        { # error!
          my_die("Error: Report.cgi - edit_report2", 4005, "Parameter = '$value'");
        }
      }
      elsif (!validateParameter($value, $input{$value}))
      {
        my_die("Error: Report.cgi - edit_report2", 4006, "$value = '$input{$value}'");
      }
      $$value = $input{$value};  # assign the value to this variable via a symbolic reference :).
    }

    use strict 'refs';  # turn strict checking back on.

    $doc->setTitle("Edit Report - $name");

    # first verify that the report in question exists.
    my $sth = $dbh->prepare( $sql_statements{1005} ) || my_db_die("Error: Report.cgi - edit_report2", 4007, "SQL Statement = '$sql_statements{1005}'");
    my $rc = $sth->execute($id) || my_db_die("Error: Report.cgi - edit_report2", 4008, "SQL Statement = '$sql_statements{1005}', ID = '$id'");
    my @info = $sth->fetchrow_array;
    $sth->finish;

    if (!defined @info)
    {
      my_die("Error: Report.cgi - edit_report2", 4020, "ID = '$id'");
    }

    # see if they want to change the report name and if so verify that the new name is not already being used.
    if ($name ne $original_name)
    {
      $sth = $dbh->prepare( $sql_statements{1003} ) || my_db_die("Error: Report.cgi - edit_report2", 4007, "SQL Statement = '$sql_statements{1003}'");
      $rc = $sth->execute($name) || my_db_die("Error: Report.cgi - edit_report2", 4008, "SQL Statement = '$sql_statements{1003}', Name = '$name', Original Name = '$original_name'");
      @info = $sth->fetchrow_array;
      $sth->finish;

      if (defined($info[0]) && $info[0] != $id)
      { # error!
        my $found = 1;  # assume we are the same report until we know for sure.
        # first check and make sure that our report_id cookie is not set and that it is not the same id as we have.  this will keep us from blowing errors on a reload page event.
        my %ch = $cookies->getCookies(cookies => ['report_id']);
        if (scalar keys %ch == 0)
        {
          $found = 0;
        }
        else
        {
          if ($cookies->cookies->{'report_id'} != $info[0])
          {
            $found = 0;
          }
        }
        if (!$found)
        {
          # call the first screen edit/create routine with the values of the selected report displayed.
          my %data = ( name => "$name", id => "$id", report_class => "$report_class", report_type => "$report_type", column_variable => "$column_variable", column_variable_data => "$column_variable_data", date_range => "$date_range",
                       display_dates => "$display_dates", max_columns => "$max_columns", min_columns => "$min_columns", text_size => "$text_size", text_font => "$text_font", data_size => "$data_size", data_font => "$data_font",
          	     total_column => "$total_column", fg_color => "$fg_color", bg_color => "$bg_color", underline_length => "$underline_length", error_string => "'$name' is already assigned to Report ID = '$info[0]'.", original_name => "$original_name" );
          my $file_name = tmpnam(); # get the file name of the temp file to work with.
          # store the data object.
          store(\%data, $file_name);
          # my_die("Error: Report.cgi - create_report2", 4015, "Report Name = '$name' is assigned to Report ID = '$info[0]' at Line " . __LINE__);

          # now we need to generate the frameset, set the error_happened cookie with the file_name and then exit.
          edit_report_frames( cookie_name => "error_happened", cookie_value => "$file_name", dbh => $dbh, windowName => $input{windowName} );
          goto LEAVE_PROGRAM_EARLY;
          # my_die("Error: Report.cgi - edit_report2", 4015, "Report Name = '$name' is assigned to Report ID = '$info[0]'");
        }
      }
    }
    # now update this entry in the database.
    $sth = $dbh->prepare( $sql_statements{1004} ) || my_db_die("Error: Report.cgi - edit_report2", 4007, "SQL Statement = '$sql_statements{1004}'");
    $rc = $sth->execute($name, $report_class, $report_type, $column_variable, $column_variable_data, $date_range, $display_dates, $max_columns, $min_columns, $text_font, $text_size, $data_font, $data_size, $total_column, $fg_color, $bg_color, $underline_length, $id);
    if ($sth->err)
    {
      my_db_die("Error: Report.cgi - edit_report2", 4008, "SQL Statement = '$sql_statements{1004}'<br>\nName = '$name'<br>\nReport Class = '$report_class'<br>\nReport Type = '$report_type'<br>\n\
    		       		     Column Variable = '$column_variable'<br>\nColumn Variable Data = '$column_variable_data'<br>\nDate Range = '$date_range'<br>\nDisplay Dates = '$display_dates'<br>\n\
    		       		     Max Columns = '$max_columns'<br>\nMin Columns = '$min_columns'<br>\nText Font = '$text_font'<br>\nText Size = '$text_size'<br>\nData Font = '$data_font'<br>\n\
    		       		     Data Size = '$data_size'<br>\nTotal Column = '$total_column'<br>\nForeground Color = '$fg_color'<br>\nBackground Color = '$bg_color'<br>\nUnder Line Length = '$underline_length'<br>\n");
    }

    # commit the transaction to the database
    $dbh->commit || my_db_die("Error: Report.cgi - edit_report2", 4019, "Tried to commit work!");

    # set a cookie that says what the id of the report is that we just updated.
    $doc->setCookie(name => 'report_id', value => "$id");
    my %ch = $cookies->getCookies(cookies => ['error_happened']);
    if (scalar keys %ch == 1)
    {
      # reset the cookie so that it is forgotten.
      my $date = `/bin/date --date "1 day ago" "+%a, %d-%b-%Y %T GMT"`;
      chomp $date;

      $doc->setCookie(name => 'error_happened', value => '', expires => "$date");
    }

    # create the frameset and call the next step of the process.

    my $windowName = $input{windowName};
    my $inner_content;
    $inner_content = $doc->createNoframe("You must have a frames capable browser!");
    $inner_content .= $doc->createFrame(src => "$site_dir/cgi-bin/report_editor.cgi?command=edit_report2_top&id=$id", frameborder => 'yes', scrolling => 'auto', marginwidth => '0', marginheight => '0', border => '1', name => $windowName . '_top', bordercolor => 'black');
    $inner_content .= $doc->createFrame(src => "$site_dir/cgi-bin/report.cgi?command=bottom", frameborder => 'yes', scrolling => 'auto', name => $windowName . '_middle', marginwidth => '1', bordercolor => 'white');
    $inner_content .= $doc->createFrame(src => "$site_dir/cgi-bin/report.cgi?command=bottom", frameborder => 'yes', scrolling => 'auto', name => $windowName . '_bottom', marginwidth => '1', bordercolor => 'white');
    $doc->outerFrameset(rows => '*, *, 2*', content => $inner_content, border => '5', frameborder => 'yes');

    $doc->display();
  }
  elsif ($command eq "edit_report2_top")
  {
    # check to make sure we were given the id of the report we are creating.
    my $id = $input{'id'};
    if (length $id == 0)
    {
      my_die("Error: Report.cgi - edit_report2_top", 4017, "Report ID required at line " . __LINE__);
    }

    # call the report second screen create/edit function with the specified ID to work with.
    report_second_screen_edit_create( mode => "edit", id => "$id", dbh => $dbh );
  }
  else
  {
    my_die("Error: Report.cgi - Invalid command", 4001, "'$command' is invalid!");
  }

LEAVE_PROGRAM_EARLY:
  # disconnect from the database (if under mod_perl this will actually never happen. :)  )
  $dbh->disconnect || my_db_die ("DBI Error on Disconnect", 4009, "Database: $db_name\@$db_host");
}
exit;  # leave the program.

#****************************** Helper functions *****************************

# This function generates the display_for_edit screen.
# Takes: error_string and/or status_string to display (optional)
sub display_for_edit
{
  my %args = ( error_string => '', status_string => '', @_, );
  my $error_string = $args{'error_string'};
  my $status_string = $args{'status_string'};
  my $dbh = $args{'dbh'};

  # gather all reports from the report_type_tb and display them in a select box so the user can edit them.  Also present a create report button.
  eval ("use HTMLObject::Normal;");
  if ($@)
  {
    my_die("Error: Report.cgi - Eval", 4002, "Error: $@ at line " . __LINE__);
  }

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - display_for_edit", 4028, "The DB Handle is required at line " . __LINE__);
  }

  my $doc = HTMLObject::Normal->new();
  $doc->setJavascriptErrorHandlerEmail(email => $email_address);
  $doc->setTitle("Select Report to Edit");
  $doc->setFocus("body");
  $doc->setLinkDecorations();
  $doc->print(<<"END_OF_BODY");
  <table border=0 width=100%>
    <tr>
      <td align=left width=50% valign=top>
END_OF_BODY

  if (length $error_string > 0)
  {
    $doc->print("<h2><font color=\"red\">Error</font>: &nbsp;$error_string</h2>");
  }
  if (length $status_string > 0)
  {
    $doc->print("<h2><font color=\"red\">Status</font>: &nbsp;$status_string</h2>");
  }

  $doc->print(<<"END_OF_BODY");
      </td>
      <td align=right width=50% nowrap valign=top>
        <form method=post action="$site_dir/cgi-bin/report_editor.cgi" name="refresh_form" target="_top">
          <input type=hidden name="command" value="display_for_edit">
          <a href="" onMouseOver="window.status='Refresh'" onClick="document.refresh_form.action='$site_dir/cgi-bin/report_editor.cgi';document.refresh_form.command.value='display_for_edit';document.refresh_form.target='_top';document.refresh_form.submit();return false;">Refresh</a> | <a href=\"\" onMouseOver=\"window.status='Close Window';\" onClick=\"window.close(); return false;\">Close</a> this Window
        </form>
      </td>
    </tr>
  </table>
  <center>
  <h1>Select a Report to Edit</h1>
  <form method=post name="report_form" action="$site_dir/cgi-bin/report_editor.cgi" target="_top">
    <input type=hidden name="command" value="">
    <input type=hidden name="new_report_name" value="">
    <input type="hidden" name="windowName" value="">
    <table border=0 cellpadding=0 cellspacing=0 bgcolor="$menu_bgcolor">
      <tr>
        <td align=right>
          <select name="report_group" onChange="report_group_change();">
            <option value="regular" selected>Standard
            <option value="detailed">Detail View
            <option value="memorized">Memorized
          </select>
        </td>
        <td align=left>
          <select name="id" size=6 width=250>
          </select>
        </td>
      </tr>
    </table>
    <br>
    <i>Reports marked with a <b>*</b> are not editable or deletable.&nbsp;They are System Reports.</i>
    <br>
    <br>
    <input type=button name="edit" value="Edit" onClick="do_edit();">&nbsp;&nbsp;<input type=button name="create" value="Create" onClick="do_create();">&nbsp;&nbsp;<input type=button name="delete" value="Delete" onClick="do_delete();">&nbsp;&nbsp;<input type=button name="copy" value="Copy" onClick="do_copy()">
  </form>
END_OF_BODY
  $doc->setFocus("javascript");
  $doc->print(<<"END_OF_CODE");

  function initialize()
  {
    var d=document.report_form;

    var topWindowName = window.parent.name;  // get the window name
    d.windowName.value = topWindowName;
  }

  function report_entry(value, text, name, editable)
  {
    this.value = value;
    this.text = text;
    this.name = name;
    this.editable = editable;
  }

  var reports = new Array();

END_OF_CODE
  # only gather the reports that are of type Regular to select from for editing
  my $sth = $dbh->prepare( $sql_statements{1040} ) || my_db_die("Error: Report.cgi - display_for_edit", 4007, "SQL Statement = '$sql_statements{1040}'");
  my $rc = $sth->execute || my_db_die("Error: Report.cgi - display_for_edit", 4008, "SQL Statement = '$sql_statements{1040}'");
  my $counter = 0;
  $doc->print("  reports[\"regular\"] = new Array();\n");
  while (my @info = $sth->fetchrow_array)
  {
    my $editable = ($info[2] ? "" : " *");
    $doc->print("  reports[\"regular\"][" . $counter++ . "] = new report_entry(\"$info[1]\", \"$info[0] ($info[1])$editable\", \"$info[0]\", $t_or_f{$info[2]});\n");
  }
  $sth->finish;

  # select detailed view reports
  $sth = $dbh->prepare( $sql_statements{1041} ) || my_db_die("Error: Report.cgi - display_for_edit", 4007, "SQL Statement = '$sql_statements{1041}'");
  $rc = $sth->execute || my_db_die("Error: Report.cgi - display_for_edit", 4008, "SQL Statement = '$sql_statements{1041}'");
  $counter = 0;
  $doc->print("  reports[\"detailed\"] = new Array();\n");
  while (my @info = $sth->fetchrow_array)
  {
    my $editable = ($info[2] ? "" : " *");
    $doc->print("  reports[\"detailed\"][" . $counter++ . "] = new report_entry(\"$info[1]\", \"$info[0] ($info[1])$editable\", \"$info[0]\", $t_or_f{$info[2]});\n");
  }
  $sth->finish;

  # select memorized reports
  $sth = $dbh->prepare( $sql_statements{1036} ) || my_db_die("Error: Report.cgi - display_for_edit", 4007, "SQL Statement = '$sql_statements{1036}'");
  $rc = $sth->execute || my_db_die("Error: Report.cgi - display_for_edit", 4008, "SQL Statement = '$sql_statements{1036}'");
  $counter = 0;
  $doc->print("  reports[\"memorized\"] = new Array();\n");
  while (my @info = $sth->fetchrow_array)
  {
    $doc->print("  reports[\"memorized\"][" . $counter++ . "] = new report_entry(\"$info[1]|$info[0]|$info[2]\", \"$info[0] ($info[1] - $info[2])\", \"$info[0]\", true);\n");
  }
  $sth->finish;

  $doc->print(<<"END_OF_CODE");

  // Helper functions

  function report_group_change()
  {
    var d = document.report_form;
    var index = d.report_group.selectedIndex;

    if (index == -1)
    {
      alert("You must select a Report Group to work with!");
      return;
    }

    var reportgroup = d.report_group.options[index].value;

    // Clear the id select box.
    d.id.options.length = 0;

    // fill in the id select box with the selected groups reports
    for (var i=0; i < reports[reportgroup].length; i++)
    {
      d.id.options[d.id.length] = new Option(reports[reportgroup][i].text, reports[reportgroup][i].value);
    }
  }

  function validate_edit()
  {
    // make sure that a report has been selected to edit.
    if (document.report_form.id.selectedIndex > -1 && document.report_form.report_group.selectedIndex > -1)
    {
      if (document.report_form.report_group[document.report_form.report_group.selectedIndex].value == "memorized")
      {
        alert("You can not edit a memorized report!\\n\\nJust re-run the memorized report and Re-Memorize it after making your changes.");
        return false;
      }
      else
      {
        var group_name = document.report_form.report_group[document.report_form.report_group.selectedIndex].value;
        var editable = reports[group_name][document.report_form.id.selectedIndex].editable;
        if (!editable)
        {
          alert("You can not Edit a System Report!");
          return false;
        }
        return true;
      }
    }
    else
    {
      alert("You must select a report to edit!");
      return false;
    }
  }

  function do_edit()
  {
    var d = document.report_form;

    if (validate_edit())
    {
      d.command.value="edit_report";
      d.submit();
    }
  }

  function validate_delete()
  {
    var d = document.report_form;
    // make sure they selected a report to delete.
    if (d.report_group.selectedIndex == -1)
    {
      alert("You must select a report group!");
      return false;
    }
    if (d.id.selectedIndex == -1)
    {
      alert("You must select a report to delete!");
      return false;
    }

    // now make sure they want to delete the selected report.
    if (d.report_group.options[d.report_group.selectedIndex].value == "memorized")
    {
      if (window.confirm("Are you sure you want to DELETE Memorized Report '"+d.id.options[d.id.selectedIndex].text+"'?\\nClick OK to Delete, Cancel to Abort the Deletion process."))
      {
        return true;
      }
    }
    else
    {
      var group_name = document.report_form.report_group[document.report_form.report_group.selectedIndex].value;
      var editable = reports[group_name][document.report_form.id.selectedIndex].editable;
      if (!editable)
      {
        alert("You can not Delete a System Report!");
        return false;
      }
      if (window.confirm("Are you sure you want to DELETE Report '"+d.id.options[d.id.selectedIndex].text+"'?\\nClick OK to Delete, Cancel to Abort the Deletion process."))
      {
        return true;
      }
    }
    return false;
  }

  function do_delete()
  {
    var d = document.report_form;

    if (validate_delete())
    {
      d.command.value = "delete_report";
      d.submit();
    }
  }

  function validate_create()
  {
    var d = document.report_form;

    if (d.report_group.options[d.report_group.selectedIndex].value == "memorized")
    {
      alert("You can not Create a Memorized Report!");
      return false;
    }
    return true;
  }

  function do_create()
  {
    var d = document.report_form;

    if (validate_create())
    {
      d.command.value = "create_report";
      d.submit();
    }
  }

  function validate_copy()
  {
    // make sure that a report has been selected to copy.
    if (document.report_form.id.selectedIndex > -1 && document.report_form.report_group.selectedIndex > -1)
    {
      if (document.report_form.report_group[document.report_form.report_group.selectedIndex].value == "memorized")
      {
        alert("You can not copy a memorized report!\\n\\nJust re-run the memorized report and Memorize it under a new name.");
        return false;
      }
      else
      {
        return true;
      }
    }
    else
    {
      alert("You must select a report to copy!");
      return false;
    }
  }

  function do_copy()
  {
    var d = document.report_form;

    if (validate_copy())
    {
      d.command.value = "copy_report";
      // get the new name of the report
      var input = "";
      var done = false;
      var group_name = document.report_form.report_group[document.report_form.report_group.selectedIndex].value;
      var report_name = reports[group_name][document.report_form.id.selectedIndex].name;
      while (!done)
      {
        input = window.prompt("Enter the new Name for this Report:", report_name);
        if (input == null)
        {
          done = true;
        }
        else
        {
          if (input.length > 0 && input != report_name)
          {
            done = true;
          }
          else
          {
            if (input == report_name)
            {
              // only way to make it modal.
              window.confirm("You must enter a name different than the current name!");
            }
          }
        }
      }
      if (input == null)
      {
        return;
      }

      d.new_report_name.value = input;

      d.submit();
    }
  }
END_OF_CODE

  my $onload_code = "";
  $onload_code = "document.report_form.report_group.selectedIndex = 0;\n";
  $onload_code .= "report_group_change();\n";
  $onload_code .= "initialize();\n";
  $doc->setOnload( code => $onload_code );

  # reset the cookie so that it is forgotten.
  my $date = `/bin/date --date "1 day ago" "+%a, %d-%b-%Y %T GMT"`;
  chomp $date;

  my %ch = $cookies->getCookies(cookies => ['error_happened']);
  if (scalar keys %ch == 1)
  {
    $doc->setCookie(name => 'error_happened', value => '', expires => "$date");
  }
  # make sure that the report_id cookie is not set.
  $doc->setCookie(name => 'report_id', value => '', expires => "$date");

  $doc->display();
}

# This function generates the frameset which causes a report to be created (screen 1).
# Takes: cookie_name, cookie_value, dbh, report_type, windowName
# Result: Sets cookie, if specified, and generates the frameset.
sub create_report_frames
{
  my %args = ( @_, );
  my $cookie_name = $args{'cookie_name'};
  my $cookie_value = $args{'cookie_value'};
  my $windowName = $args{windowName};
  my $dbh = $args{'dbh'};
  my $report_type = $args{'report_type'};

  # generate the frameset for the create screen 1 to go into.
  eval("use HTMLObject::FrameSet;");
  if ($@)
  {
    my_die("Error: Report.cgi - Eval", 4000, "Error: $@ at line " . __LINE__);
  }
  my $doc = HTMLObject::FrameSet->new();
  $doc->setTitle("Create Report");

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - create_report_frames", 4028, "The DB Handle is required at line " . __LINE__);
  }
  if ($windowName !~ /$editReportWindowRE/)
  {
    my_die("Error: Report.cgi - create_report_frames", 4006, "windowName = '$windowName' is invalid at line " . __LINE__);
  }

  my $inner_content;
  $inner_content = $doc->createNoframe("You must have a frames capable browser!");
  $inner_content .= $doc->createFrame(src => "$site_dir/cgi-bin/report_editor.cgi?command=create_report1&report_type=$report_type", frameborder => 'yes', scrolling => 'auto', marginwidth => '0', marginheight => '0', border => '1', name => $windowName . '_top', bordercolor => 'black');
  $inner_content .= $doc->createFrame(src => "$site_dir/cgi-bin/report.cgi?command=bottom", frameborder => 'yes', scrolling => 'auto', name => $windowName . '_bottom', marginwidth => '1', bordercolor => 'white');
  $doc->outerFrameset(rows => '3*, *', content => $inner_content, border => '5', frameborder => 'yes');

  if (length $cookie_name > 0 && length $cookie_value > 0)
  {
    $doc->setCookie( name => $cookie_name, value => $cookie_value );
    if ($cookie_name eq "error_happened")
    {
      # log the time and file name
      my $sth = $dbh->prepare( $sql_statements{1029} ) || my_db_die("Error: Report.cgi - create_report_frames", 4007, "SQL Statement = '$sql_statements{1029}'");
      my $rc = $sth->execute($cookie_value) || my_db_die("Error: Report.cgi - create_report_frames", 4008, "SQL Statement = '$sql_statements{1029}', File Name = '$cookie_value'");
      $dbh->commit || my_db_die("Error:  Report.cgi - create_report_frames", 4019, "At line " . __LINE__);
    }
  }

  $doc->display();
}

# This function generates the frameset which causes a report to be edited (screen 1).
# Takes: cookie_name, cookie_value, windowName
# Result: Sets cookie, if specified, and generates the frameset.
sub edit_report_frames
{
  my %args = ( @_, );
  my $cookie_name = $args{'cookie_name'};
  my $cookie_value = $args{'cookie_value'};
  my $windowName = $args{windowName};
  my $dbh = $args{'dbh'};
  my $report_type = "";

  # check to make sure we were given the id of the report we are editing.
  my $id = $input{'id'};
  if (length $id == 0)
  {
    my_die("Error: Report.cgi - edit_report_frames", 4017, "Report ID required at line " . __LINE__);
  }

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - edit_report_frames", 4028, "The DB Handle is required at line " . __LINE__);
  }
  if ($windowName !~ /$editReportWindowRE/)
  {
    my_die("Error: Report.cgi - edit_report_frames", 4006, "windowName = '$windowName' is invalid at line " . __LINE__);
  }

  # first verify that the report in question exists.
  my $sth = $dbh->prepare( $sql_statements{1005} ) || my_db_die("Error: Report.cgi - edit_report", 4007, "SQL Statement = '$sql_statements{1005}'");
  my $rc = $sth->execute($id) || my_db_die("Error: Report.cgi - edit_report", 4008, "SQL Statement = '$sql_statements{1005}', ID = '$id'");
  my @info = $sth->fetchrow_array;
  $sth->finish;

  if (!defined @info)
  {
    my_die("Error: Report.cgi - edit_report", 4020, "ID = '$id'");
  }

  if ($input{'report_group'} eq "regular")
  {
    $report_type = "r";
  }
  else
  {
    $report_type = "d";
  }

  # generate the frameset to hold the first screen in.
  eval("use HTMLObject::FrameSet;");
  if ($@)
  {
    my_die("Error: Report.cgi - Eval", 4000, "Error: $@ at line " . __LINE__);
  }
  my $doc = HTMLObject::FrameSet->new();
  $doc->setTitle("Edit Report");

  my $inner_content;
  $inner_content = $doc->createNoframe("You must have a frames capable browser!");
  $inner_content .= $doc->createFrame(src => "$site_dir/cgi-bin/report_editor.cgi?command=edit_report1&id=$id&report_type=$report_type", frameborder => 'yes', scrolling => 'auto', marginwidth => '0', marginheight => '0', border => '1', name => $windowName . '_top', bordercolor => 'black');
  $inner_content .= $doc->createFrame(src => "$site_dir/cgi-bin/report.cgi?command=bottom", frameborder => 'yes', scrolling => 'auto', name => $windowName . '_bottom', marginwidth => '1', bordercolor => 'white');
  $doc->outerFrameset(rows => '3*, *', content => $inner_content, border => '5', frameborder => 'yes');

  if (length $cookie_name > 0 && length $cookie_value > 0)
  {
    $doc->setCookie( name => $cookie_name, value => $cookie_value );
    if ($cookie_name eq "error_happened")
    {
      # log the time and file name
      my $sth = $dbh->prepare( $sql_statements{1029} ) || my_db_die("Error: Report.cgi - edit_report_frames", 4007, "SQL Statement = '$sql_statements{1029}'");
      my $rc = $sth->execute($cookie_value) || my_db_die("Error: Report.cgi - edit_report_frames", 4008, "SQL Statement = '$sql_statements{1029}', File Name = '$cookie_value'");
      $dbh->commit || my_db_die("Error:  Report.cgi - edit_report_frames", 4019, "At line " . __LINE__);
    }
  }

  $doc->display();
}

# This function returns the data from the storable object (if specified via cookie) or defaults for creating a report.
# currently takes nothing.
sub get_default_data
{
  my %values;
  my $value_ref; # reference to the hash.

  # check and see if the error cookie exists.
  my %ch = $cookies->getCookies(cookies => ['error_happened']);
  if (scalar keys %ch == 1)
  {
    my $file_name = $cookies->cookies->{'error_happened'};
    $value_ref = retrieve($file_name);
    %values = %{$value_ref};
    #unlink $file_name;  # delete the file.
  }
  else
  {
    $values{'name'} = "";
    $values{'id'} = 0;
    $values{'report_class'} = "custom";
    $values{'report_type'} = "r";
    $values{'column_variable'} = "";
    $values{'column_variable_data'} = "";
    $values{'display_dates'} = "true";
    $values{'date_range'} = "true";
    $values{'max_columns'} = 36;
    $values{'min_columns'} = 1;
    $values{'text_size'} = "-1";
    $values{'text_font'} = "helvetica";
    $values{'data_size'} = "-1";
    $values{'data_font'} = "helvetica";
    $values{'total_column'} = "false";
    $values{'fg_color'} = "black";
    $values{'bg_color'} = "white";
    $values{'underline_length'} = $default_underline_length;
  }

  return %values;
}

sub validateParameter
{
  # this function takes the name of the parameter and it's value and does the work of making sure that the value is valid for that parameter.
  my ($name, $value) = ( @_ );
  my $result = 1;

  if (length $name == 0)
  {
    $result = 0;
  }
  if (length $value == 0)
  {
    $result = 0;
  }

  if ($name eq "mode")
  {
    if ($value ne "edit" && $value ne "create")
    {
      $result = 0;
    }
  }
  elsif ($name eq "name")
  {
  }
  elsif ($name eq "original_name")
  {
  }
  elsif ($name eq "id")
  {
    if ($value < -1)
    {
      $result = 0;
    }
  }
  elsif ($name eq "report_class")
  {
    if (!exists $report_classes{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "report_type")
  {
    if ($value ne "r" && $value ne "d")
    {
      $result = 0;
    }
  }
  elsif ($name eq "column_variable")
  {
  }
  elsif ($name eq "column_variable_data")
  {
  }
  elsif ($name eq "date_range")
  {
    if ($value ne "true" && $value ne "false")
    {
      $result = 0;
    }
  }
  elsif ($name eq "display_dates")
  {
    if ($value ne "true" && $value ne "false")
    {
      $result = 0;
    }
  }
  elsif ($name eq "max_columns")
  {
    if ($value <= 0 || $value > $max_dates)
    {
      $result = 0;
    }
  }
  elsif ($name eq "min_columns")
  {
    if ($value <= 0 || $value > $max_dates)
    {
      $result = 0;
    }
  }
  elsif ($name eq "text_size")
  {
    if (!exists $font_sizes{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "text_font")
  {
    if (!exists $font_names{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "data_size")
  {
    if (!exists $font_sizes{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "data_font")
  {
    if (!exists $font_names{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "total_column")
  {
    if ($value ne "true" && $value ne "false")
    {
      $result = 0;
    }
  }
  elsif ($name eq "fg_color")
  {
    if (!exists $fg_colors{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "bg_color")
  {
    if (!exists $bg_colors{$value})
    {
      $result = 0;
    }
  }
  elsif ($name eq "underline_length")
  {
    if ($value < $min_underline_length || $value > $max_underline_length)
    {
      $result = 0 if ($value != -1);
    }
  }
  elsif ($name eq "dbh")
  {
  }
  elsif ($name eq "error_string")
  {
  }
  elsif ($name eq "rows")
  {
    my @rows = split /\n/, $value;
    if (scalar @rows == 0)
    {
      $result = 0;
    }
  }
  elsif ($name eq "file_name")
  {
  }
  elsif ($name eq "report_group")
  {
    if ($value ne "regular" && $value ne "detailed" && $value ne "memorized")
    {
      $result = 0;
    }
  }
  elsif ($name eq "new_report_name")
  {
  }
  elsif ($name eq "windowName")
  {
    $result = 0 if ($value !~ /$editReportWindowRE/);
  }
  else
  {
    $result = 0;  # signal that an error occurred as we do not know anything about this variable.
  }

  return $result;
}

sub get_all_account_types
{
  my %args = ( @_, );
  my $dbh = $args{'dbh'};
  my @data = ();

  if (!exists $args{'dbh'} || length $dbh == 0)
  {
    my_die("Error:  Report.cgi - get_all_account_types", 4005, "The Database Handle is required!");
  }

  my $sth = $dbh->prepare( $sql_statements{1019} ) || my_db_die("Error: Report.cgi - get_all_account_types", 4007, "SQL Statement = '$sql_statements{1019}'");
  my $rc = $sth->execute || my_db_die("Error: Report.cgi - get_all_account_types", 4008, "SQL Statement = '$sql_statements{1019}'");

  my $ref = $sth->fetchall_arrayref;
  if ($sth->err)
  {
    my_db_die("Error:  Report.cgi - get_all_account_types", 4009, "Getting all Account Types failed!");
  }
  @data = @{$ref};

  return @data;
}

sub get_all_accounts
{
  my %args = ( @_, );
  my $dbh = $args{'dbh'};
  my @data = ();

  if (!exists $args{'dbh'} || length $dbh == 0)
  {
    my_die("Error:  Report.cgi - get_all_accounts", 4005, "The Database Handle is required!");
  }

  my $sth = $dbh->prepare( $sql_statements{1020} ) || my_db_die("Error: Report.cgi - get_all_accounts", 4007, "SQL Statement = '$sql_statements{1020}'");
  my $rc = $sth->execute || my_db_die("Error: Report.cgi - get_all_accounts", 4008, "SQL Statement = '$sql_statements{1020}'");

  my $ref = $sth->fetchall_arrayref;
  if ($sth->err)
  {
    my_db_die("Error:  Report.cgi - get_all_accounts", 4009, "Getting all Accounts failed!");
  }
  $sth->finish;
  @data = @{$ref};

  return @data;
}

sub get_all_sub_accounts
{
  my %args = ( @_, );
  my $dbh = $args{'dbh'};
  my @data = ();

  if (!exists $args{'dbh'} || length $dbh == 0)
  {
    my_die("Error:  Report.cgi - get_all_sub_accounts", 4005, "The Database Handle is required!");
  }

  my $sth = $dbh->prepare( $sql_statements{1021} ) || my_db_die("Error: Report.cgi - get_all_sub_accounts", 4007, "SQL Statement = '$sql_statements{1021}'");
  my $rc = $sth->execute || my_db_die("Error: Report.cgi - get_all_sub_accounts", 4008, "SQL Statement = '$sql_statements{1021}'");

  my $ref = $sth->fetchall_arrayref;
  if ($sth->err)
  {
    my_db_die("Error:  Report.cgi - get_all_sub_accounts", 4009, "Getting all Sub Accounts failed!");
  }
  $sth->finish;
  @data = @{$ref};

  return @data;
}

sub get_all_detailed_reports
{
  my %args = ( @_, );
  my $dbh = $args{'dbh'};
  my @data = ();

  if (!exists $args{'dbh'} || length $dbh == 0)
  {
    my_die("Error:  Report.cgi - get_all_detailed_reports", 4005, "The Database Handle is required!");
  }

  my $sth = $dbh->prepare( $sql_statements{1022} ) || my_db_die("Error: Report.cgi - get_all_detailed_reports", 4007, "SQL Statement = '$sql_statements{1022}'");
  my $rc = $sth->execute || my_db_die("Error: Report.cgi - get_all_detailed_reports", 4008, "SQL Statement = '$sql_statements{1022}'");

  my $ref = $sth->fetchall_arrayref;
  if ($sth->err)
  {
    my_db_die("Error:  Report.cgi - get_all_detailed_reports", 4009, "Getting all Detailed Reports failed!");
  }
  $sth->finish;
  @data = @{$ref};

  return @data;
}

# takes  : account type array and number to find
# returns: 1 if found, 0 otherwise.
sub find_account_type
{
  my %args = ( @_, );
  my @array = @{$args{'array'}};
  my $num = $args{'num'};
  my $found = 0;

  if (!exists $args{'array'})
  {
    my_die("Error:  Report.cgi - find_account_type", 4005, "array is required");
  }

  if (scalar @array == 0)
  {
    return 0;
  }
  if (!exists $args{'num'} || length $num == 0)
  {
    my_die("Error:  Report.cgi - find_account_type", 4005, "num is required");
  }

  for (my $i=0; $i < scalar @array && !$found; $i++)
  {
    if ($array[$i][1] == $num)
    {
      $found = 1;
    }
  }

  return $found;
}

sub find_account
{
  my %args = ( @_, );
  my @array = @{$args{'array'}};
  my $num = $args{'num'};
  my $found = 0;

  if (!exists $args{'array'})
  {
    my_die("Error:  Report.cgi - find_account", 4005, "array is required");
  }

  if (scalar @array == 0)
  {
    return 0;
  }
  if (!exists $args{'num'} || length $num == 0)
  {
    my_die("Error:  Report.cgi - find_account", 4005, "num is required");
  }

  for (my $i=0; $i < scalar @array && !$found; $i++)
  {
    if ($array[$i][1] == $num)
    {
      $found = 1;
    }
  }

  return $found;
}

sub find_sub_account
{
  my %args = ( @_, );
  my @array = @{$args{'array'}};
  my $num = $args{'num'};
  my $found = 0;

  if (!exists $args{'array'})
  {
    my_die("Error:  Report.cgi - find_sub_account", 4005, "array is required");
  }

  if (scalar @array == 0)
  {
    return 0;
  }
  if (!exists $args{'num'} || length $num == 0)
  {
    my_die("Error:  Report.cgi - find_sub_account", 4005, "num is required");
  }

  for (my $i=0; $i < scalar @array && !$found; $i++)
  {
    if ($array[$i][1] == $num)
    {
      $found = 1;
    }
  }

  return $found;
}

sub find_detailed_report
{
  my %args = ( @_, );
  my @array = @{$args{'array'}};
  my $num = $args{'num'};
  my $found = 0;

  if (!exists $args{'array'})
  {
    my_die("Error:  Report.cgi - find_detailed_report", 4005, "array is required");
  }

  if (scalar @array == 0)
  {
    return 0;
  }
  if (!exists $args{'num'} || length $num == 0)
  {
    my_die("Error:  Report.cgi - find_detailed_report", 4005, "num is required");
  }

  for (my $i=0; $i < scalar @array && !$found; $i++)
  {
    if ($array[$i][0] == $num)
    {
      $found = 1;
    }
  }

  return $found;
}

sub find_level
{
  my %args = ( @_, );
  my @array = @{$args{'array'}};
  my $level = $args{'level'};
  my $found = -1;

  if (!exists $args{'array'})
  {
    my_die("Error:  Report.cgi - find_detailed_report", 4005, "array is required");
  }

  if (scalar @array == 0)
  {
    return 0;
  }
  if (!exists $args{'level'} || length $level == 0)
  {
    my_die("Error:  Report.cgi - find_detailed_report", 4005, "level is required");
  }

  for (my $i=0; $i < scalar @array && $found == -1; $i++)
  {
    if ($array[$i] eq $level)
    {
      $found = $i;
    }
  }

  return $found;
}

# parse_rows
# takes the string that represents the rows for the report from create/edit screen 2
# returns the array that represents the rows in their native format
sub parse_rows
{
  my %args = ( @_, );
  my $rows = $args{'rows'};
  my $dbh = $args{'dbh'};
  my $report_type = $args{'report_type'};
  my @rows = ();
  my %row_to_index = ( "index" => "0", "cell_text" => "1", "cell_formula" => "2", "cell_type" => "3", "item_name" => "4", "fg_color" => "5", "bg_color" => "6", "bold" => "7", "sub_levels" => "8", "display_level" => "9", "initial_state" => "10", "drillable" => "11", "drill_report" => "12", "negative_format" => "13", "activity" => "14", "query_type" => "15", "show" => "16", "result_type" => "17", "underline" => "18" );

  if (length $rows == 0)
  {
    my_die("Error: Report.cgi - parse_rows", 4005, "The Rows String to parse must be specified!");
  }

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - parse_rows", 4005, "The Database Handle must be specified!");
  }

  my (@account_types, @accounts, @sub_accounts, @detailed_reports);
  if ($report_type eq "r")
  {
    @account_types = get_all_account_types(dbh => $dbh);
    @accounts = get_all_accounts(dbh => $dbh);
    @sub_accounts = get_all_sub_accounts(dbh => $dbh);
    @detailed_reports = get_all_detailed_reports(dbh => $dbh);
  }

  my $rows_counter = 0;
  my @temp = split /\n/, $rows;   # split apart on the \n
  foreach my $row (@temp)
  {
    my @values = split /\|/, $row;  # split apart on the |
    foreach my $item (@values)
    {
      my ($name, $value) = split /=/, $item, 2;  # only split on the first =
      if (!exists $row_to_index{$name})
      {
        my_die("Error:  Report.cgi - parse_rows", 4031, "Item = '$name'.");
      }
      if (!validate_row_item($name, $value, $report_type))
      {
        my_die("Error:  Report.cgi - parse_rows", 4033, "Item = '$name', Value = '$value'.");
      }

      # store in array
      $rows[$rows_counter][$row_to_index{$name}] = $value;
    }
    $rows_counter++;
  }

  # now walk the array and for the special items, validate that they are what they claim to be.
  for ($rows_counter=0; $rows_counter < scalar @rows; $rows_counter++)
  {
    my ($index, $cell_type, $item_name, $sub_levels, $drillable, $drill_report);
    $index        = $rows[$rows_counter][0];
    $cell_type    = $rows[$rows_counter][3];
    $item_name    = $rows[$rows_counter][4];
    $sub_levels   = $rows[$rows_counter][8];
    $drillable    = $rows[$rows_counter][11];
    $drill_report = $rows[$rows_counter][12];

    if ($report_type eq "r")
    {
      if ($cell_type eq "text" || $cell_type eq "total" || $cell_type eq "line" || $cell_type eq "compare")
      {
        if ($item_name ne "---------------")
        {
          my_die("Error:  Report.cgi - parse_rows", 4033, "'$item_name' is invalid for Cell Type = '$cell_type'!<br>Row = '$index'.");
        }
        if ($sub_levels ne "---------------")
        {
          my_die("Error:  Report.cgi - parse_rows", 4033, "'$sub_levels' is invalid for Cell Type = '$cell_type'!<br>Row = '$index'.");
        }
        $rows[$rows_counter][8] = -1;
        if ($drill_report ne "---------------")
        {
          my_die("Error:  Report.cgi - parse_rows", 4033, "'$drill_report' is invalid for Cell Type = '$cell_type'!<br>Row = '$index'.");
        }
        $rows[$rows_counter][12] = -1;
        if ($cell_type ne "compare")
        {
          $rows[$rows_counter][17] = ""; # The Result Type doesn't matter.
        }
      }
      else
      {
        if ($cell_type eq "account_type" || $cell_type eq "accounts_of" || $cell_type eq "begin_account_type_balance" || $cell_type eq "end_account_type_balance" || $cell_type eq "sub_accounts_of_type")
        {
          if (!find_account_type( array => \@account_types, num => $item_name ))
          {
            my_die("Error:  Report.cgi - parse_rows", 4033, "Item Name = '$item_name' is invalid for Cell Type = '$cell_type'!<br>Row = '$index'.");
          }
        }
        elsif ($cell_type eq "account" || $cell_type eq "sub_accounts_of_account" || $cell_type eq "begin_account_balance" || $cell_type eq "end_account_balance")
        {
          if (!find_account( array => \@accounts, num => $item_name ))
          {
            my_die("Error:  Report.cgi - parse_rows", 4033, "Item Name = '$item_name' is invalid for Cell Type = '$cell_type'!<br>Row = '$index'.");
          }
        }
        elsif ($cell_type eq "sub_account")
        {
          if (!find_sub_account( array => \@sub_accounts, num => $item_name ))
          {
            my_die("Error:  Report.cgi - parse_rows", 4033, "Item Name = '$item_name' is invalid for Cell Type = '$cell_type'!<br>Row = '$index'.");
          }
        }

        my ($level);
        if (($level = find_level( array => \@{$report_levels{$cell_type}}, level => "$sub_levels" )) == -1)
        {
          my_die("Error:  Report.cgi - parse_rows", 4033, "Display Sub Level = '$sub_levels' is invalid for Cell Type = '$cell_type'!<br>Row = '$index'.");
        }
        $rows[$rows_counter][8] = $level;  # set the index
        if ($drillable eq "true")
        {
          if (!find_detailed_report( array => \@detailed_reports, num => $drill_report ))
          {
            my_die("Error:  Report.cgi - parse_rows", 4033, "Detailed Report = '$drill_report' is invalid for Cell Type = '$cell_type'!<br>Row = '$index'.<br>Drillable = True.");
          }
        }
        else
        {
          if ($drill_report ne "---------------")
          {
            my_die("Error:  Report.cgi - parse_rows", 4033, "Detailed Report = '$drill_report' is invalid for Cell Type = '$cell_type'!<br>Row = '$index'.<br>Drillable = False.");
          }
          $rows[$rows_counter][12] = -1;
        }
        $rows[$rows_counter][17] = ""; # The Result Type doesn't matter.
        $rows[$rows_counter][18] = "0"; # The Underline Flag should always be false.
      } # end of else
    }
    else  # Detailed View Report
    {
      if ($cell_type eq "text" || $cell_type eq "line" || $cell_type eq "detail")
      {
        $rows[$rows_counter][4] = "---------------";
        $rows[$rows_counter][8] = -1;
        $rows[$rows_counter][11] = "false";
        $rows[$rows_counter][12] = -1;
        $rows[$rows_counter][2] = "";
        $rows[$rows_counter][9] = "1";
        $rows[$rows_counter][10] = "c";
        $rows[$rows_counter][13] = 0;
        $rows[$rows_counter][14] = "true";
        $rows[$rows_counter][15] = "0";
        $rows[$rows_counter][16] = "1";  # always show this row.
        $rows[$rows_counter][17] = ""; # The Result Type doesn't matter.
        $rows[$rows_counter][18] = "0"; # The Underline Flag should always be false.
      }
    }
  }

  return @rows;
}

# validate_row_item
# takes  : name, value, report_type
# returns: 1 if ok, 0 if not
sub validate_row_item
{
  my ($name, $value, $report_type);
  $name  = $_[0];
  $value = $_[1];
  $report_type = $_[2];

  my $result = 1;

  if (length $name == 0)
  {
    my_die("Error:  Report.cgi - validate_row_item", 4005, "The name parameter is required!");
  }

  if ($name eq "index")
  {
    $result = 0 if ($value < 0);
  }
  elsif ($name eq "cell_text")
  {
  }
  elsif ($name eq "cell_formula")
  {
    # this is where I need to parse apart the formula and validate it.
  }
  elsif ($name eq "cell_type")
  {
    if ($report_type eq "r")
    {
      $result = 0 if (!exists $regular_cell_types{$value});
    }
    elsif ($report_type eq "d")
    {
      $result = 0 if (!exists $detailed_cell_types{$value});
    }
  }
  elsif ($name eq "item_name")
  {
    $result = 0 if (length $value == 0);
  }
  elsif ($name eq "fg_color")
  {
    $result = 0 if (!exists $fg_colors{$value});
  }
  elsif ($name eq "bg_color")
  {
    $result = 0 if (!exists $bg_colors{$value});
  }
  elsif ($name eq "bold")
  {
    $result = 0 if (!exists $true_false{$value});
  }
  elsif ($name eq "sub_levels")
  {
    $result = 0 if (length $value == 0);
  }
  elsif ($name eq "display_level")
  {
    $result = 0 if ($value < 0 || $value > 2);
  }
  elsif ($name eq "initial_state")
  {
    $result = 0 if ($value ne "o" && $value ne "c");
  }
  elsif ($name eq "drillable")
  {
    $result = 0 if (!exists $true_false{$value});
  }
  elsif ($name eq "drill_report")
  {
    $result = 0 if (length $value == 0);
  }
  elsif ($name eq "negative_format")
  {
    $result = 0 if (!exists $negative_formats{$value});
  }
  elsif ($name eq "activity")
  {
    $result = 0 if (!exists $true_false{$value});
  }
  elsif ($name eq "query_type")
  {
    $result = 0 if (!($value == 0 || $value == 1 || $value == 2));
  }
  elsif ($name eq "show")
  {
    $result = 0 if (!($value == 0 || $value == 1));
  }
  elsif ($name eq "underline")
  {
    $result = 0 if (!($value == 0 || $value == 1));
  }
  elsif ($name eq "result_type")
  {
  }
  else
  {
    $result = 0;  # signal an error occurred.
  }

  return $result;
}

# FindSubAccount
# takes: number and reference to array to search in.
# returns the Name of the Sub Account.
sub FindSubAccount
{
  my ($number, @array);
  $number = $_[0];
  @array = @{$_[1]};

  if (length $number == 0)
  {
    my_die("Error: Report.cgi - FindSubAccount", 4005, "The Sub Account Number must be specified!");
  }
  if (scalar @array == 0)
  {
    my_die("Error: Report.cgi - FindSubAccount", 4005, "The Array to search must be specified!");
  }

  # now we walk through all SubAccountTypes and for those that have SubAccount arrays, we walk them till we find
  # the specified SubAccount number and then we can get it's name!
  my $name = "";
  my $found = 0;

  for (my $index=0; $index < scalar @array && !$found; $index++)
  {
    if (scalar @{$array[$index][2]} > 0)
    {
      for (my $index2=0; $index2 < scalar @{$array[$index][2]} && !$found; $index2++)
      {
        if ($array[$index][2][$index2][0] == $number)
        {
          $found = 1;
          $name = $array[$index][2][$index2][1];
        }
      }
    }
  }

  if (!$found)
  {
    my_die("Error: Report.cgi - FindSubAccount", 4013, "The specified Sub Account = '$number' was not found!");
  }

  return $name;
}

# FindSubAccountType
# takes: number and reference to array to search in.
# returns the Name of the Sub Account Type.
sub FindSubAccountType
{
  my ($number, @array);
  $number = $_[0];
  @array = @{$_[1]};

  if (length $number == 0)
  {
    my_die("Error: Report.cgi - FindSubAccount", 4005, "The Sub Account Type Number must be specified!");
  }
  if (scalar @array == 0)
  {
    my_die("Error: Report.cgi - FindSubAccount", 4005, "The Array to search must be specified!");
  }

  # now we walk through all SubAccountTypes till we find
  # the specified SubAccountType number and then we can get it's name!
  my $name = "";
  my $found = 0;

  for (my $index=0; $index < scalar @array && !$found; $index++)
  {
    if ($array[$index][0] == $number)
    {
      $found = 1;
      $name = $array[$index][1];
    }
  }

  if (!$found)
  {
    my_die("Error: Report.cgi - FindSubAccount", 4014, "The specified Sub Account Type = '$number' was not found!");
  }

  return $name;
}

sub report_first_screen_edit_create
{
  # this function will take a list of parameters that defines the settings for the first screen of editing or creating a report.
  # mode = (edit|create), name, id=(-1 if creating), report_class, report_type, column_variable=(empty if creating), column_variable_data=(empty if creating)
  # date_range, display_dates, max_columns, min_columns, text_size, data_size, text_font, data_font, total_column, fg_color, bg_color, underline_length

  my %args = ( @_, );  # gather all arguments from the command line.

  # validate we have all parameters we need.
  my $mode = $args{'mode'};
  my $name = $args{'name'};
  my $id = $args{'id'};
  my $report_class = $args{'report_class'};
  my $report_type = $args{'report_type'};
  my $column_variable = $args{'column_variable'};
  my $column_variable_data = $args{'column_variable_data'};
  my $date_range = $args{'date_range'};
  my $display_dates = $args{'display_dates'};
  my $max_columns = $args{'max_columns'};
  my $min_columns = $args{'min_columns'};
  my $text_size = $args{'text_size'};
  my $text_font = $args{'text_font'};
  my $data_size = $args{'data_size'};
  my $data_font = $args{'data_font'};
  my $total_column = $args{'total_column'};
  my $fg_color = $args{'fg_color'};
  my $bg_color = $args{'bg_color'};
  my $underline_length = $args{'underline_length'};
  my $dbh = $args{'dbh'};
  my $error_string = $args{'error_string'};

  my @arguments = ( "mode", "name", "id", "report_class", "report_type", "column_variable", "column_variable_data", "date_range", "display_dates", "max_columns",
  		    "min_columns", "text_size", "text_font", "data_size", "data_font", "total_column", "fg_color", "bg_color", "underline_length", "dbh" );

  eval ("use HTMLObject::Normal;");
  if ($@)
  {
    my_die("Error: Report.cgi - Eval", 4002, "Error: $@ at line " . __LINE__);
  }

  my $doc = HTMLObject::Normal->new();

  $doc->setJavascriptErrorHandlerEmail(email => $email_address);

  foreach my $value (@arguments)
  {
    if (!exists $args{$value})
    { # error!
      $doc->doRequiredParameterError('report_first_screen_edit_create', $value);
    }
    if (length $args{$value} == 0)
    {
      if (($value eq "name" && $mode eq "create"))
      { # do nothing, this is the default value.
      }
      elsif ($value ne "column_variable" && $value ne "column_variable_data")
      { # error!
        my_die("Error: Report.cgi - report_first_screen_edit_create", 4005, "Parameter = '$value'");
      }
    }
    elsif (!validateParameter($value, $args{$value}))
    {
      my_die("Error: Report.cgi - report_first_screen_edit_create", 4006, "$value = '$args{$value}'");
    }
  }

  my $sth;
  my $rc;
  my $sth2;
  my @info = ();
  my $title = "";
  my $target_window = "";
  my $command = "";
  my $header = "";
  my %already_selected_column_headers = ();
  if ($mode eq "create")
  {
    $title = "Create Report";
    $header = "Create Report";
    $target_window = "create_report_window";
    $command = "create_report2";
  }
  elsif ($mode eq "edit")
  {
    $title = "Edit Report";
    $header = "Edit Report - ID = $id";
    $target_window = "edit_report_window";
    $command = "edit_report2";
  }

  if ($report_type eq "r")
  {
    $sth = $dbh->prepare( $sql_statements{1000} ) || my_db_die("Error: Report.cgi - report_first_screen_edit_create", 4007, "SQL Statement = '$sql_statements{1000}'");
    $rc = $sth->execute || my_db_die("Error: Report.cgi - report_first_screen_edit_create", 4008, "SQL Statement = '$sql_statements{1000}'");

    $sth2 = $dbh->prepare( $sql_statements{1001} ) || my_db_die("Error: Report.cgi - report_first_screen_edit_create", 4007, "SQL Statement = '$sql_statements{1001}'");

    $doc->setFocus("javascript");  # point to the JavaScript section of this document.

    # define some helper functions we'll need.
    $doc->print(<<"END_OF_CODE");
    function SubAccountType(name, num)
    {
      this.name = name;
      this.num = num;
    }

    function SubAccount(name, num)
    {
      this.name = name;
      this.num = num;
    }

END_OF_CODE

    # create the Sub Account Types array, where each index holds an object that will initially just have the sub account type name and later an array of all sub accounts for that type.
    $doc->print("  var SubAccountTypes = new Array();\n\n");

    # walk through all sub-account-types returned and store them in the array.
    my $array_index = 0;  # start out at 0.
    my @SubAccountTypes = ();  # the perl array to store the SubAccountTypes and SubAccounts in.
    while (my @db_items = $sth->fetchrow_array)
    {
      $doc->print("  SubAccountTypes[$array_index] = new SubAccountType(\"$db_items[1]\", $db_items[0]);\n");
      my @temp = ();
      $temp[0] = $db_items[0];
      $temp[1] = $db_items[1];

      # Now we need to gather all SubAccounts for this SubAccountType.
      my $rc2 = $sth2->execute( $db_items[0] ) || my_db_die("Error: Report.cgi - report_first_screen_edit_create", 4008, "SQL Statement = '$sql_statements{1001}'\n<br>\nValue = '$db_items[0]'");
      my $sub_array_index = 0;  # start out at 0.
      my @temp2 = ();
      # create the array for the sub accounts - even if we don't have any sub accounts.
      $doc->print("  SubAccountTypes[$array_index].sub_accounts = new Array();\n");
      while (my @db_items2 = $sth2->fetchrow_array)
      {
        $doc->print("  SubAccountTypes[$array_index].sub_accounts[$sub_array_index] = new SubAccount(\"$db_items2[1]\", $db_items2[0]);\n");
        my @temp3 = ();
        $temp3[0] = $db_items2[0];
        $temp3[1] = $db_items2[1];
        $temp2[$sub_array_index++] = [ @temp3 ];
      }
      $sth2->finish || my_db_die ("DBI Error", 4009, "Handle = 'sth2'\n<br>\nAction = 'finish'\n<br>\nSub Account Type = '$db_items[0]'");

      $temp[2] = [ @temp2 ];  # Assign the SubAccounts Array to the SubAccountType

      $SubAccountTypes[$array_index++] = [ @temp ];  # Store this SubAccountType in the overall Array.
    }
    $sth->finish || my_db_die ("DBI Error", 4009, "Handle = 'sth'\n<br>\nAction = 'finish'");

    # initialize the Selected Column Headers Array and all necessary data storage
    $doc->print(<<"END_OF_CODE");

    function ColumnInfo(text, value)
    {
      this.text = text;
      this.value = value;
    }

    function ColumnItemInfo(text, value)
    {
      this.text = text;
      this.value = value;
    }

    var SelectedColumns;  // this will hold the data for the Selected Column Headers.

    function InitializeSelectedColumnsArray()
    {
      SelectedColumns = new Array();
END_OF_CODE

    # split the column_variable string apart and put the found items in the already_selected_column_headers hash.
    $array_index = 0;  # start over at 0 again.
    my @column_items = split /\|/, $column_variable_data;

    foreach my $column ( split /,/, $column_variable )
    {
      if (!exists $column_headers{$column})
      {
        my_die("Error:  Report.cgi - report_first_screen_edit_create", 4011, "Column Header = '$column' used in $mode of Report = '$id'");
      }
      if (!exists $already_selected_column_headers{$column})
      {
        $already_selected_column_headers{$column} = 1;
        $doc->print("    SelectedColumns[$array_index] = new ColumnInfo(\"$column_headers{$column}\", \"$column\");\n");
        $doc->print("    SelectedColumns[$array_index].items = new Array();\n");

        # split the data from the column_variable_data string and assign it to the items array.
        my @items = split /,/, $column_items[$array_index];
        my $index2 = 0;  # start at 0.
        foreach my $item (@items)
        {
          my $value = $item;
          my $text = "";

          if ($column eq "sub_accounts")
          {
            $text = FindSubAccount($value, \@SubAccountTypes);  # get the name of the SubAccount
          }
          elsif ($column eq "sub_account_types")
          {
            $text = FindSubAccountType($value, \@SubAccountTypes);  # get the name of the SubAccountType
          }
          else
          {
            my_die("Error:  Report.cgi - report_first_screen_edit_create", 4012, "Column Header = '$column' has items assigned to it when it should not in $mode of Report = '$id'");
          }
          $doc->print("    SelectedColumns[$array_index].items[" . $index2++ . "] = new ColumnItemInfo(\"$text\", \"$value\");\n");
        }

        $array_index++;
      }
      else
      {
        my_die("Error:  Report.cgi - report_first_screen_edit_create", 4010, "Column Header = '$column' already used in Edit of Report = '$id'");
      }
    }
    $doc->print("  }\n\n");  # put space between the SelectedColumns variables and the rest of the code.
  }

  $target_window = "_top";  # temporarily send it to the same window unless they want it elsewhere later.
  $doc->setTitle($title);
  $doc->setFocus("body");
  if (length $error_string > 0)
  {
    $doc->print("<h2><font color=\"red\">Error</font>: &nbsp;$error_string</h2>\n");
  }
  my $original_name = $name;
  if (exists $args{'original_name'})
  {
    if (length $args{'original_name'} > 0)
    {
      $original_name = $args{'original_name'};
    }
  }
  $doc->print(<<"END_OF_BODY");
  <center><h1>$header</h1></center>
  <form name="report_form" method=post action="$site_dir/cgi-bin/report_editor.cgi" target="$target_window" onSubmit="return validate_form()">
    <input type=hidden name="command" value="$command">
    <input type=hidden name="id" value="$id">
    <input type=hidden name="report_type" value="$report_type">
    <input type="hidden" name="windowName" value="">
END_OF_BODY
  if ($report_type eq "r")
  {
    $doc->print(<<"END_OF_BODY");
    <input type=hidden name="column_variable" value="">
    <input type=hidden name="column_variable_data" value="">
    <input type=hidden name="original_name" value="$original_name">
    <table border=1 cellpadding=2 cellspacing=0 align=center>
      <tr bgcolor="#cccccc">
        <th>Report Name</th><th>Report Class</th><th colspan=2> Report Type</th>
      </tr>
      <tr bgcolor="#eeeeee">
        <td align=center><input type=text name="name" value="$name"></td>
        <td align=center><select name="report_class">
END_OF_BODY
        for (my $i=0; $i < scalar @report_classes; $i++)
        {
          my $string = "          <option value=\"$report_classes[$i]\"";
          if ($report_classes[$i] eq $report_class)
          {
            $string .= " selected";
          }
          $string .= ">$report_classes{$report_classes[$i]}\n";
          $doc->print($string);
        }
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
        <td align=center colspan=2>$report_types{$report_type}</td>
      </tr>
      <tr bgcolor="#cccccc">
        <th>Date Type</th><th>Max Columns to Show</th><th>Min Columns to Fill</th><th>Display Date Selector</th>
      </tr>
      <tr bgcolor="#eeeeee">
        <td align=center><select name="date_range">
END_OF_BODY
          my $string = "          <option value=\"true\"";
          if ($date_range eq "true")
          {
            $string .= " selected";
          }
          $string .= ">Date Range\n";
          $doc->print($string);

          $string = "          <option value=\"false\"";
          if ($date_range eq "false")
          {
            $string .= " selected";
          }
          $string .= ">Single Date\n";
          $doc->print($string);
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
        <td align=center><select name="max_columns">
END_OF_BODY
        for (my $i=1; $i <= $max_dates; $i++)
        {
          my $string = "          <option value=\"$i\"";
          if ($i eq $max_columns)
          {
            $string .= " selected";
          }
          $string .= ">$column_names{$i}\n";
          $doc->print($string);
        }
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
        <td align=center><select name="min_columns">
END_OF_BODY
        for (my $i=1; $i <= $max_dates; $i++)
        {
          my $string = "          <option value=\"$i\"";
          if ($i eq $min_columns)
          {
            $string .= " selected";
          }
          $string .= ">$column_names{$i}\n";
          $doc->print($string);
        }
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
        <td align=center><select name="display_dates">
END_OF_BODY
          my $string = "          <option value=\"true\"";
          if ($display_dates eq "true")
          {
            $string .= " selected";
          }
          $string .= ">Yes\n";
          $doc->print($string);

          $string = "          <option value=\"false\"";
          if ($display_dates eq "false")
          {
            $string .= " selected";
          }
          $string .= ">No\n";
          $doc->print($string);
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
      </tr>
      <tr bgcolor="#cccccc">
        <th>Text Font</th><th>Increase Text Font Size By</th><th>Data Font</th><th>Increase Data Font Size By</th>
      </tr>
      <tr bgcolor="#eeeeee">
        <td align=center><select name="text_font">
END_OF_BODY
        for (my $i=0; $i < scalar @font_names; $i++)
        {
          my $string = "          <option value=\"$font_names[$i]\"";
          if ($font_names[$i] eq $text_font)
          {
            $string .= " selected";
          }
          $string .= ">$font_names{$font_names[$i]}\n";
          $doc->print($string);
        }
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
        <td align=center><select name="text_size">
END_OF_BODY
        for (my $i=0; $i < scalar @font_sizes; $i++)
        {
          my $string = "          <option value=\"$font_sizes[$i]\"";
          if ($font_sizes[$i] eq $text_size)
          {
            $string .= " selected";
          }
          $string .= ">$font_sizes{$font_sizes[$i]}\n";
          $doc->print($string);
        }
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
        <td align=center><select name="data_font">
END_OF_BODY
        for (my $i=0; $i < scalar @font_names; $i++)
        {
          my $string = "          <option value=\"$font_names[$i]\"";
          if ($font_names[$i] eq $data_font)
          {
            $string .= " selected";
          }
          $string .= ">$font_names{$font_names[$i]}\n";
          $doc->print($string);
        }
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
        <td align=center><select name="data_size">
END_OF_BODY
        for (my $i=0; $i < scalar @font_sizes; $i++)
        {
          my $string = "          <option value=\"$font_sizes[$i]\"";
          if ($font_sizes[$i] eq $data_size)
          {
            $string .= " selected";
          }
          $string .= ">$font_sizes{$font_sizes[$i]}\n";
          $doc->print($string);
        }
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
      </tr>
      <tr bgcolor="#cccccc">
        <th>ForeGround Color</th><th>BackGround Color</th><th>Display Total Column?</th><th>Length of Underlines</th>
      </tr>
      <tr bgcolor="#eeeeee">
        <td align=center><select name="fg_color">
END_OF_BODY
        for (my $i=0; $i < scalar @fg_colors; $i++)
        {
          my $string = "          <option value=\"$fg_colors[$i]\"";
          if ($fg_colors[$i] eq $fg_color)
          {
            $string .= " selected";
          }
          $string .= ">$fg_colors{$fg_colors[$i]}\n";
          $doc->print($string);
        }
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
        <td align=center><select name="bg_color">
END_OF_BODY
        for (my $i=0; $i < scalar @bg_colors; $i++)
        {
          my $string = "          <option value=\"$bg_colors[$i]\"";
          if ($bg_colors[$i] eq $bg_color)
          {
            $string .= " selected";
          }
          $string .= ">$bg_colors{$bg_colors[$i]}\n";
          $doc->print($string);
        }
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
        <td align=center><select name="total_column">
END_OF_BODY
          my $string = "          <option value=\"true\"";
          if ($total_column eq "true")
          {
            $string .= " selected";
          }
          $string .= ">Yes\n";
          $doc->print($string);

          $string = "          <option value=\"false\"";
          if ($total_column eq "false")
          {
            $string .= " selected";
          }
          $string .= ">No\n";
          $doc->print($string);
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
        <td align=center><select name="underline_length">
END_OF_BODY
        $doc->print("          <option value=\"-1\"" . ($underline_length == -1 ? " selected" : "") . ">User Default</option>\n");
        for (my $i=$min_underline_length; $i <= $max_underline_length; $i++)
        {
          my $string = "          <option value=\"$i\"";
          if ($i eq $underline_length)
          {
            $string .= " selected";
          }
          $string .= ">$column_names{$i}</option>\n";
          $doc->print($string);
        }
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
      </tr>
      <tr bgcolor="#cccccc">
        <th>Available Column Headers</th><th>Selected Column Headers</th><th colspan=2>&nbsp;</th>
      </tr>
      <tr bgcolor="#eeeeee">
        <td align=center valign=center>
          <table border=0 align=center cellpadding=0 cellspacing=0>
            <tr>
              <td align=center width=85%><select name="available_columns" size="5" width="150">
                <option value="---------------">---------------
                </select>
              </td>
              <td align=center valign=center width=15%><input type=button name="insert" value=">" onClick="InsertColumnHeader()"><br><input type=button name="remove" value="<" onClick="RemoveColumnHeader()"></td>
            </tr>
          </table>
        </td>
        <td align=center><select name="selected_columns" size="5" width="150">
          <option value="---------------">---------------
          </select>
        </td>
        <td colspan=2 align=left><input type=button name="edit" value="Customize Column Header" onClick="DisplaySelectedColumnHeader()"></td>
      </tr>
    </table>
END_OF_BODY
  }
  else  # This is a detailed view report
  {
    $doc->print(<<"END_OF_BODY");
    <input type=hidden name="original_name" value="$original_name">
    <input type=hidden name="report_class" value="custom">
    <input type=hidden name="column_variable" value="">
    <input type=hidden name="column_variable_data" value="">
    <input type=hidden name="date_range" value="false">
    <input type=hidden name="display_dates" value="false">
    <input type=hidden name="max_columns" value="1">
    <input type=hidden name="min_columns" value="1">
    <input type=hidden name="total_column" value="false">
    <input type=hidden name="underline_length" value="$default_underline_length">
    <table border=1 cellpadding=2 cellspacing=0 align=center>
      <tr bgcolor="#cccccc">
        <th>Report Name</th><th> Report Type</th>
      </tr>
      <tr bgcolor="#eeeeee">
        <td align=center><input type=text name="name" value="$name"></td>
        <td align=center>$report_types{$report_type}</td>
      </tr>
      <tr bgcolor="#cccccc">
        <th>Text Font</th><th>Increase Text Font Size By</th>
      </tr>
      <tr bgcolor="#eeeeee">
        <td align=center><select name="text_font">
END_OF_BODY
        for (my $i=0; $i < scalar @font_names; $i++)
        {
          my $string = "          <option value=\"$font_names[$i]\"";
          if ($font_names[$i] eq $text_font)
          {
            $string .= " selected";
          }
          $string .= ">$font_names{$font_names[$i]}\n";
          $doc->print($string);
        }
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
        <td align=center><select name="text_size">
END_OF_BODY
        for (my $i=0; $i < scalar @font_sizes; $i++)
        {
          my $string = "          <option value=\"$font_sizes[$i]\"";
          if ($font_sizes[$i] eq $text_size)
          {
            $string .= " selected";
          }
          $string .= ">$font_sizes{$font_sizes[$i]}\n";
          $doc->print($string);
        }
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
      </tr>
      <tr bgcolor="#cccccc">
        <th>Data Font</th><th>Increase Data Font Size By</th>
      </tr>
      <tr bgcolor="#eeeeee">
        <td align=center><select name="data_font">
END_OF_BODY
        for (my $i=0; $i < scalar @font_names; $i++)
        {
          my $string = "          <option value=\"$font_names[$i]\"";
          if ($font_names[$i] eq $data_font)
          {
            $string .= " selected";
          }
          $string .= ">$font_names{$font_names[$i]}\n";
          $doc->print($string);
        }
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
        <td align=center><select name="data_size">
END_OF_BODY
        for (my $i=0; $i < scalar @font_sizes; $i++)
        {
          my $string = "          <option value=\"$font_sizes[$i]\"";
          if ($font_sizes[$i] eq $data_size)
          {
            $string .= " selected";
          }
          $string .= ">$font_sizes{$font_sizes[$i]}\n";
          $doc->print($string);
        }
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
      </tr>
      <tr bgcolor="#cccccc">
        <th>ForeGround Color</th><th>BackGround Color</th>
      </tr>
      <tr bgcolor="#eeeeee">
        <td align=center><select name="fg_color">
END_OF_BODY
        for (my $i=0; $i < scalar @fg_colors; $i++)
        {
          my $string = "          <option value=\"$fg_colors[$i]\"";
          if ($fg_colors[$i] eq $fg_color)
          {
            $string .= " selected";
          }
          $string .= ">$fg_colors{$fg_colors[$i]}\n";
          $doc->print($string);
        }
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
        <td align=center><select name="bg_color">
END_OF_BODY
        for (my $i=0; $i < scalar @bg_colors; $i++)
        {
          my $string = "          <option value=\"$bg_colors[$i]\"";
          if ($bg_colors[$i] eq $bg_color)
          {
            $string .= " selected";
          }
          $string .= ">$bg_colors{$bg_colors[$i]}\n";
          $doc->print($string);
        }
  $doc->print(<<"END_OF_BODY");
          </select>
        </td>
      </tr>
    </table>
END_OF_BODY
  }

  $doc->print(<<"END_OF_BODY");
    <center><input type=submit name="next" value="Next Step">&nbsp;&nbsp;<input type=button name="reset_button" value="Reset Form" onClick="reset_form()"></center>
  </form>

  <form name="edit_form" method=post action="$site_dir/cgi-bin/report_editor.cgi" target="_top">
    <input type=hidden name="command" value="display_for_edit">
    <div align=right><a href="" onClick="document.edit_form.submit();return false;" onMouseOver="window.status='Return to Edit/Create Report - Main Screen';">Edit/Create Report - Main Screen</a></div>
  </form>
END_OF_BODY

  # turn off the link decorations
  $doc->setLinkDecorations();

  if ($report_type eq "r")
  {
    # set the onLoad code in place.
    $doc->setOnload(code => "InitializeAvailableColumns();\nInitializeSelectedColumns();\ninitialize();");
  }
  else
  {
    $doc->setOnload(code => "initialize();");
  }

  $doc->setFocus("javascript");
  $doc->print(<<"END_OF_CODE");

  function initialize()
  {
    var d = document.report_form;

    d.windowName.value = window.parent.name;
  }

END_OF_CODE
  if ($report_type eq "r")
  {
  $doc->print(<<"END_OF_CODE");
  function InitializeAvailableColumns()
  {
    var s = document.report_form.available_columns;
    // Clear the Select box
    s.options.length = 0;

    // fill in the options that are currently available.
END_OF_CODE
    for (my $i=0; $i < scalar @column_headers; $i++)
    {
      my $string = "    s.options[s.length] = new Option(\"$column_headers{$column_headers[$i]}\", \"$column_headers[$i]\");\n";
      if (!exists $already_selected_column_headers{$column_headers[$i]})
      {
        $doc->print($string);
      }
    }
  $doc->print(<<"END_OF_CODE");
  }

  function InitializeSelectedColumns()
  {
    var s = document.report_form.selected_columns;
    // Clear the Select box
    s.options.length = 0;

    // fill in the options that are already selected.
END_OF_CODE
    foreach my $column (split /,/, $column_variable)
    {
      my $string = "    s.options[s.length] = new Option(\"$column_headers{$column}\", \"$column\");\n";
      $doc->print($string);
    }
  $doc->print(<<"END_OF_CODE");

    // call the data array initialization function
    InitializeSelectedColumnsArray();

    // make sure the bottom window is empty.
    var bottomWindowName = window.parent.name + '_bottom';
    var w = window.open("", bottomWindowName);
    var d = w.document;
    d.open();
    d.close();
  }

  function validate_form()
  {
    // Make sure that the user has entered everything that is required.
    var d = document.report_form;
    if (d.name.value.length == 0)
    {
      alert("You must enter the Name for the report!");
      d.name.focus();
      return false;
    }

    // make sure that the max_columns and min_columns are not impossible.
    if (d.max_columns.options[d.max_columns.selectedIndex].value < d.min_columns.options[d.min_columns.selectedIndex].value)
    {
      alert("You can not specify a Minimum Column value > than the Maximum Columns to display!");
      return false;
    }

    // set the column_variable and column_variable_data hidden elements
    d.column_variable.value = CreateColumnHeaderString();
    d.column_variable_data.value = CreateColumnHeaderDataString();

    // alert("Column Variable = '" + d.column_variable.value + "'\\nColumn Variable Data = '" + d.column_variable_data.value + "'");
    return true;  // change to true when done debugging form.
  }

  function reset_form()
  {
    document.report_form.reset();

    window.setTimeout("document.report_form.name.value = document.report_form.original_name.value;initialize();", 10);

    // make both the available and selected column header list boxes have the 15 dashes back.
    document.report_form.available_columns.options[document.report_form.available_columns.length] = new Option("---------------", "---------------");
    document.report_form.selected_columns.options[document.report_form.selected_columns.length] = new Option("---------------", "---------------");

    // resets the dynamic content for the Column Headers back to the defaults.
    InitializeAvailableColumns();
    InitializeSelectedColumns();
  }

  function InsertColumnHeader()
  {
    var s = document.report_form.available_columns;
    var selected_index = s.selectedIndex;

    if (selected_index > -1)
    {
      var Column_Name = s.options[selected_index].text;
      var Column_Value = s.options[selected_index].value;

      // remove this item from the Available list and then insert it into the Selected list
      var i;
      for (i=selected_index+1; i < s.length; i++)
      {
        s.options[i-1] = s.options[i];
      }
      s.options.length--;  // remove the item from the array
      // select the newly inserted item in the Available Column list
      var next_index = selected_index-1;
      if (next_index == -1 && s.length > 0)
      {
        next_index = 0;
      }
      s.selectedIndex = next_index;

      s = document.report_form.selected_columns;
      s.options[s.length] = new Option(Column_Name, Column_Value);
      // select the newly inserted item in the Selected Column list
      s.selectedIndex = s.length-1;

      // make the entry in the SelectedColumns Array.
      var index = SelectedColumns.length;

      SelectedColumns[index] = new ColumnInfo(Column_Name, Column_Value);
      SelectedColumns[index].items = new Array();
    }
    else
    {
      alert("You must select a Column Header to Insert!");
    }
  }

  function RemoveColumnHeader()
  {
    var s = document.report_form.selected_columns;
    var selected_index = s.selectedIndex;

    if (selected_index > -1)
    {
      var Column_Name = s.options[selected_index].text;
      var Column_Value = s.options[selected_index].value;

      // remove this item from the Selected list and Data Array and then insert it into the Available list
      var i;
      for (i=selected_index+1; i < s.length; i++)
      {
        s.options[i-1] = s.options[i];
      }
      s.options.length--;  // remove the item from the array
      // select the newly inserted item in the Selected Column list
      var next_index = selected_index-1;
      if (next_index == -1 && s.length > 0)
      {
        next_index = 0;
      }
      s.selectedIndex = next_index;

      s = document.report_form.available_columns;
      s.options[s.length] = new Option(Column_Name, Column_Value);
      // select the newly inserted item in the Available Column list
      s.selectedIndex = s.length-1;

      // remove the entry in the SelectedColumns Array.
      for (i=selected_index+1; i < SelectedColumns.length; i++)
      {
        SelectedColumns[i-1] = SelectedColumns[i];
      }
      SelectedColumns.length--;  // remove the item (and all it's sub-items) from the array.

      // Clear the bottom window.
      var bottomWindowName = window.parent.name + '_bottom';
      var w = window.open("", bottomWindowName);
      var d = w.document;
      d.open();
      d.close();
    }
    else
    {
      alert("You must select a Column Header to Remove!");
    }
  }

  function InsertItem(col_type, bname)
  {
    var bottomWindowName = window.parent.name + '_bottom';
    var w = window.open("", bottomWindowName);
    var d = w.document;

    if (col_type == "sub_entities")
    {
      // error!
      alert("InsertItem should never be called from a Sub Entity Screen!");
    }
    else
    {
      if (col_type == "sub_account_types")
      {
        if (bname == "insert")
        {
          var available = d.header_form.available_items;
          var selected = d.header_form.selected_items;

          var index = available.selectedIndex;

          if (index > -1)
          {
            // Move the selected Sub Account Type over.
            // remove this item from the Available list and then insert it into the Selected list
            var i;
            var Column_Name = available.options[index].text;
            var Column_Value = available.options[index].value;
            for (i=index+1; i < available.length; i++)
            {
              available.options[i-1] = available.options[i];
            }
            available.options.length--;  // remove the item from the array
            // select the next item in the Available Column list
            var next_index = index-1;
            if (next_index == -1 && available.length > 0)
            {
              next_index = 0;
            }
            available.selectedIndex = next_index;

            selected.options[selected.length] = new Option(Column_Name, Column_Value);
            // select the newly inserted item in the Selected Column list
            selected.selectedIndex = selected.length-1;

            // make the item entry in the SelectedColumns Array.
            var found=false;
            for (i=0; i < SelectedColumns.length && !found; i++)
            {
              if (SelectedColumns[i].value == "sub_account_types")
              {
                found = true;
              }
            }
            if (found)
            {
              i--;  // Get the actual entry.
              // Add the Selected Sub Account Type to the items array.
              SelectedColumns[i].items[SelectedColumns[i].items.length] = new ColumnItemInfo(Column_Name, Column_Value);
            }
            else
            {
              alert("Error:  Could not find Sub Account Type entry in SelectedColumns Array!\\nThis means there is a major problem!");
            }
          }
          else
          {
            if (available.length > 0)
            {
              alert("You must select a Sub Account Type to work with!");
            }
            else
            {
              alert("There are no available Sub Account Types to select from!");
            }
          }
        }
        else
        {
          if (bname == "insert2")  // Sub Account Types for the Sub Account Customization screen.
          {
            var available = d.header_form.available_items;
            var selected = d.header_form.selected_items;

            var index = available.selectedIndex;

            if (index > -1)
            {
              // Move the selected Sub Account Type over.
              // remove this item from the Available list and then insert it into the Selected list
              var i;
              var Column_Name = available.options[index].text;
              var Column_Value = available.options[index].value;
              for (i=index+1; i < available.length; i++)
              {
                available.options[i-1] = available.options[i];
              }
              available.options.length--;  // remove the item from the array
              // select the next item in the Available Column list
              var next_index = index-1;
              if (next_index == -1 && available.length > 0)
              {
                next_index = 0;
              }
              available.selectedIndex = next_index;

              selected.options[selected.length] = new Option(Column_Name, Column_Value);
              // select the newly inserted item in the Selected Column list
              selected.selectedIndex = selected.length-1;
            }
            else
            {
              if (available.length > 0)
              {
                alert("You must select a Sub Account Type to work with!");
              }
              else
              {
                alert("There are no available Sub Account Types to select from!");
              }
            }
          }
          else
          {
            alert("Error: Invalid button name!\\nName = '"+bname+"'");
          }
        }
      }
      else
      {
        if (col_type == "sub_accounts")
        {
          if (bname == "insert_sub")  // Sub Accounts
          {
            var selected_sub = d.header_form.selected_sub_items;
            var available_sub = d.header_form.available_sub_items;
            var index = available_sub.selectedIndex;

            if (index > -1)
            {
              // Move the selected Sub Account over.
              // remove this item from the Available list and then insert it into the Selected list
              var i;
              var Column_Name = available_sub.options[index].text;
              var Column_Value = available_sub.options[index].value;
              for (i=index+1; i < available_sub.length; i++)
              {
                available_sub.options[i-1] = available_sub.options[i];
              }
              available_sub.options.length--;  // remove the item from the array
              // select the next item in the Available Sub Accounts Column list
              var next_index = index-1;
              if (next_index == -1 && available_sub.length > 0)
              {
                next_index = 0;
              }
              available_sub.selectedIndex = next_index;

              selected_sub.options[selected_sub.length] = new Option(Column_Name, Column_Value);
              // select the newly inserted item in the Selected Sub Accounts Column list
              selected_sub.selectedIndex = selected_sub.length-1;

              // make the item entry in the SelectedColumns Array.
              var found=false;
              for (i=0; i < SelectedColumns.length && !found; i++)
              {
                if (SelectedColumns[i].value == "sub_accounts")
                {
                  found = true;
                }
              }
              if (found)
              {
                i--;  // Get the actual entry.
                // Add the Selected Sub Account to the items array.
                SelectedColumns[i].items[SelectedColumns[i].items.length] = new ColumnItemInfo(Column_Name, Column_Value);
              }
              else
              {
                alert("Error:  Could not find Sub Account entry in SelectedColumns Array!\\nThis means there is a major problem!");
              }
            }
            else
            {
              if (available_sub.length > 0)
              {
                alert("You must select a Sub Account to assign!");
              }
              else
              {
                alert("You must have Sub Accounts to work with!");
              }
            }
          }
          else
          {
            alert("Error: Invalid button name!\\nName = '"+bname+"'");
          }
        }
        else
        {
          alert("Error!  Unknown Column Header specified!\\nType given = '" + col_type + "'");
        }
      }
    }
  }

  function InsertAllItems(col_type, bname)
  {
    var bottomWindowName = window.parent.name + '_bottom';
    var w = window.open("", bottomWindowName);
    var d = w.document;

    if (col_type == "sub_entities")
    {
      // error!
      alert("InsertItem should never be called from a Sub Entity Screen!");
    }
    else
    {
      if (col_type == "sub_account_types")
      {
        if (bname == "insert_all")
        {
          var available = d.header_form.available_items;
          var selected = d.header_form.selected_items;

          if (available.length > 0)
          {
            available.options.length = 0;  // clear both select boxes.
            selected.options.length = 0;

            // walk the Sub Account Types array and populate the Selected List with all Sub Account Types
            var i;
            for (i=0; i < SubAccountTypes.length; i++)
            {
              selected.options[selected.length] = new Option(SubAccountTypes[i].name, SubAccountTypes[i].num);
            }

            // Add all Sub Account Types to the items array for the SubAccountType entry in SelectedColumns.
            var found=false;
            for (i=0; i < SelectedColumns.length && !found; i++)
            {
              if (SelectedColumns[i].value == "sub_account_types")
              {
                found = true;
              }
            }
            if (found)
            {
              i--;  // Get the actual entry.
              var j;

              // Clear the SelectedColumns items array.
              SelectedColumns[i].items.length = 0;

              for (j=0; j < SubAccountTypes.length; j++)
              {
                // Add the Sub Account Type to the items array.
                SelectedColumns[i].items[SelectedColumns[i].items.length] = new ColumnItemInfo(SubAccountTypes[j].name, SubAccountTypes[j].num);
              }
            }
            else
            {
              alert("Error:  Could not find Sub Account Type entry in SelectedColumns Array!\\nThis means there is a major problem!");
              return;
            }
            // Select the first entry.
            selected.selectedIndex = 0;
          }
          else
          {
            alert("All Sub Account Types are already Selected!");
          }
        }
        else
        {
          if (bname == "insert_all2")
          {
            var available = d.header_form.available_items;
            var selected = d.header_form.selected_items;

            if (available.length > 0)
            {
              available.options.length = 0;  // clear both select boxes.
              selected.options.length = 0;

              // walk the Sub Account Types array and populate the Selected List with all Sub Account Types
              var i;
              for (i=0; i < SubAccountTypes.length; i++)
              {
                selected.options[selected.length] = new Option(SubAccountTypes[i].name, SubAccountTypes[i].num);
              }

              // Select the first entry.
              selected.selectedIndex = 0;
            }
            else
            {
              alert("All Sub Account Types are already Selected!");
            }
          }
          else
          {
            alert("Error: Invalid button name!\\nName = '"+bname+"'");
          }
        }
      }
      else
      {
        if (col_type == "sub_accounts")
        {
          if (bname == "insert_all_sub")  // Sub Accounts
          {
            var selected_sub = d.header_form.selected_sub_items;
            var available_sub = d.header_form.available_sub_items;

            if (available_sub.length > 0)
            {
              // Move the all available Sub Accounts over.
              var i, j;
              var found=false;
              for (i=0; i < SelectedColumns.length && !found; i++)
              {
                if (SelectedColumns[i].value == "sub_accounts")
                {
                  found = true;
                }
              }
              if (found)
              {
                i--;  // Get the actual entry.
              }
              else
              {
                alert("Error:  Could not find Sub Account entry in SelectedColumns Array!\\nThis means there is a major problem!");
                return;
              }

              for (j=0; j < available_sub.length; j++)
              {
                var Column_Name = available_sub.options[j].text;
                var Column_Value = available_sub.options[j].value;

                selected_sub.options[selected_sub.length] = new Option(Column_Name, Column_Value);

                // make the item entry in the SelectedColumns Array.
                // Add the Selected Sub Account to the items array.
                SelectedColumns[i].items[SelectedColumns[i].items.length] = new ColumnItemInfo(Column_Name, Column_Value);
              }
              // select the last item in the Selected Sub Accounts Column list
              selected_sub.selectedIndex = selected_sub.length-1;

              // clear the Available Sub Accounts Column List
              available_sub.options.length = 0;
            }
            else
            {
              alert("You must have Sub Accounts to work with!");
            }
          }
          else
          {
            alert("Error: Invalid button name!\\nName = '"+bname+"'");
          }
        }
        else
        {
          alert("Error!  Unknown Column Header specified!\\nType given = '" + col_type + "'");
        }
      }
    }
  }

  function RemoveItem(col_type, bname)
  {
    var bottomWindowName = window.parent.name + '_bottom';
    var w = window.open("", bottomWindowName);
    var d = w.document;

    if (col_type == "sub_entities")
    {
      // error!
      alert("RemoveItem should never be called from a Sub Entity Screen!");
    }
    else
    {
      if (col_type == "sub_account_types")
      {
        if (bname == "remove")
        {
          var available = d.header_form.available_items;
          var selected = d.header_form.selected_items;

          var index = selected.selectedIndex;

          if (index > -1)
          {
            // Move the selected Sub Account Type over.
            // remove this item from the Selected list and then insert it into the Available list
            var i;
            var Column_Name = selected.options[index].text;
            var Column_Value = selected.options[index].value;
            for (i=index+1; i < selected.length; i++)
            {
              selected.options[i-1] = selected.options[i];
            }
            selected.options.length--;  // remove the item from the array
            // select the next item in the Selected Column list
            var next_index = index-1;
            if (next_index == -1 && selected.length > 0)
            {
              next_index = 0;
            }
            selected.selectedIndex = next_index;

            available.options[available.length] = new Option(Column_Name, Column_Value);
            // select the newly inserted item in the Available Column list
            available.selectedIndex = available.length-1;

            // make the item entry in the SelectedColumns Array.
            var found=false;
            for (i=0; i < SelectedColumns.length && !found; i++)
            {
              if (SelectedColumns[i].value == "sub_account_types")
              {
                found = true;
              }
            }
            if (found)
            {
              i--;  // Get the actual entry.
              // Remove the Selected Sub Account Type from the items array.
              var j;
              for (j=index+1; j < SelectedColumns[i].items.length; j++)
              {
                SelectedColumns[i].items[j-1] = SelectedColumns[i].items[j];
              }
              SelectedColumns[i].items.length--;  // decrement the length of the array.
            }
            else
            {
              alert("Error:  Could not find Sub Account Type entry in SelectedColumns Array!\\nThis means there is a major problem!");
            }
          }
          else
          {
            if (selected.length > 0)
            {
              alert("You must select a Sub Account Type to work with!");
            }
            else
            {
              alert("There are no selected Sub Account Types to select from!");
            }
          }
        }
        else
        {
          if (bname == "remove2")
          {
            var available = d.header_form.available_items;
            var selected = d.header_form.selected_items;
            var available_sub = d.header_form.available_sub_items;

            var index = selected.selectedIndex;

            if (index > -1)
            {
              // Move the selected Sub Account Type over.
              // remove this item from the Selected list and then insert it into the Available list
              var i;
              var Column_Name = selected.options[index].text;
              var Column_Value = selected.options[index].value;
              for (i=index+1; i < selected.length; i++)
              {
                selected.options[i-1] = selected.options[i];
              }
              selected.options.length--;  // remove the item from the array
              // select the next item in the Selected Column list
              var next_index = index-1;
              if (next_index == -1 && selected.length > 0)
              {
                next_index = 0;
              }
              selected.selectedIndex = next_index;

              available.options[available.length] = new Option(Column_Name, Column_Value);
              // select the newly inserted item in the Available Column list
              available.selectedIndex = available.length-1;

              if (selected.length == 0)
              {
                // clear the available sub items list box.
                available_sub.options.length = 0;
              }
            }
            else
            {
              if (selected.length > 0)
              {
                alert("You must select a Sub Account Type to work with!");
              }
              else
              {
                alert("There are no selected Sub Account Types to select from!");
              }
            }
          }
          else
          {
            alert("Error: Invalid button name!\\nName = '"+bname+"'");
          }
        }
      }
      else
      {
        if (col_type == "sub_accounts")
        {
          if (bname == "remove_sub")  // Sub Accounts
          {
            var selected_sub = d.header_form.selected_sub_items;
            var available_sub = d.header_form.available_sub_items;
            var index = selected_sub.selectedIndex;

            if (index > -1)
            {
              // Move the selected Sub Account over.
              // remove this item from the Selected list and then insert it into the Available list
              var i;
              var Column_Name = selected_sub.options[index].text;
              var Column_Value = selected_sub.options[index].value;
              for (i=index+1; i < selected_sub.length; i++)
              {
                selected_sub.options[i-1] = selected_sub.options[i];
              }
              selected_sub.options.length--;  // remove the item from the array
              // select the next item in the Selected Sub Accounts Column list
              var next_index = index-1;
              if (next_index == -1 && selected_sub.length > 0)
              {
                next_index = 0;
              }
              selected_sub.selectedIndex = next_index;

              available_sub.options[available_sub.length] = new Option(Column_Name, Column_Value);
              // select the newly inserted item in the Available Sub Accounts Column list
              available_sub.selectedIndex = available_sub.length-1;

              // remove the item entry from the SelectedColumns Array.
              var found=false;
              for (i=0; i < SelectedColumns.length && !found; i++)
              {
                if (SelectedColumns[i].value == "sub_accounts")
                {
                  found = true;
                }
              }
              if (found)
              {
                i--;  // Get the actual entry.
                // Remove the Selected Sub Account from the items array.
                var j;
                for (j=index+1; j < SelectedColumns[i].items.length; j++)
                {
                  SelectedColumns[i].items[j-1] = SelectedColumns[i].items[j];
                }
                SelectedColumns[i].items.length--;
              }
              else
              {
                alert("Error:  Could not find Sub Account entry in SelectedColumns Array!\\nThis means there is a major problem!");
              }
            }
            else
            {
              if (selected_sub.length > 0)
              {
                alert("You must select a Sub Account to un-assign!");
              }
              else
              {
                alert("You must have Sub Accounts assigned first!");
              }
            }
          }
          else
          {
            alert("Error: Invalid button name!\\nName = '"+bname+"'");
          }
        }
        else
        {
          alert("Error!  Unknown Column Header specified!\\nType given = '" + col_type + "'");
        }
      }
    }
  }

  function RemoveAllItems(col_type, bname)
  {
    var bottomWindowName = window.parent.name + '_bottom';
    var w = window.open("", bottomWindowName);
    var d = w.document;

    if (col_type == "sub_entities")
    {
      // error!
      alert("RemoveItem should never be called from a Sub Entity Screen!");
    }
    else
    {
      if (col_type == "sub_account_types")
      {
        if (bname == "remove_all")
        {
          var available = d.header_form.available_items;
          var selected = d.header_form.selected_items;

          if (selected.length > 0)
          {
            available.options.length = 0;  // clear both select boxes.
            selected.options.length = 0;

            // walk the Sub Account Types array and populate the Available List with all Sub Account Types
            var i;
            for (i=0; i < SubAccountTypes.length; i++)
            {
              available.options[available.length] = new Option(SubAccountTypes[i].name, SubAccountTypes[i].num);
            }

            // Remove all Sub Account Types from the items array for the SubAccountType entry in SelectedColumns.
            var found=false;
            for (i=0; i < SelectedColumns.length && !found; i++)
            {
              if (SelectedColumns[i].value == "sub_account_types")
              {
                found = true;
              }
            }
            if (found)
            {
              i--;  // Get the actual entry.
              SelectedColumns[i].items.length = 0;  // empty the array.
            }
            else
            {
              alert("Error:  Could not find Sub Account Type entry in SelectedColumns Array!\\nThis means there is a major problem!");
              return;
            }
            // Select the first entry.
            available.selectedIndex = 0;
          }
          else
          {
            alert("All Sub Account Types are already unselected!");
          }
        }
        else
        {
          if (bname == "remove_all2")
          {
            var available = d.header_form.available_items;
            var selected = d.header_form.selected_items;
            var available_sub = d.header_form.available_sub_items;

            if (selected.length > 0)
            {
              available.options.length = 0;  // clear both select boxes.
              selected.options.length = 0;

              // walk the Sub Account Types array and populate the Available List with all Sub Account Types
              var i;
              for (i=0; i < SubAccountTypes.length; i++)
              {
                available.options[available.length] = new Option(SubAccountTypes[i].name, SubAccountTypes[i].num);
              }

              // Select the first entry.
              available.selectedIndex = 0;

              // clear the Available Sub Items list box.
              available_sub.options.length = 0;
            }
            else
            {
              alert("All Sub Account Types are already unselected!");
            }
          }
          else
          {
            alert("Error: Invalid button name!\\nName = '"+bname+"'");
          }
        }
      }
      else
      {
        if (col_type == "sub_accounts")
        {
          if (bname == "remove_all_sub")  // Sub Accounts
          {
            var selected_sub = d.header_form.selected_sub_items;
            var available_sub = d.header_form.available_sub_items;

            if (selected_sub.length > 0)
            {
              // REMove all selected Sub Accounts over.
              var i, j;
              var found=false;
              for (i=0; i < SelectedColumns.length && !found; i++)
              {
                if (SelectedColumns[i].value == "sub_accounts")
                {
                  found = true;
                }
              }
              if (found)
              {
                i--;  // Get the actual entry.
              }
              else
              {
                alert("Error:  Could not find Sub Account entry in SelectedColumns Array!\\nThis means there is a major problem!");
                return;
              }

              for (j=0; j < selected_sub.length; j++)
              {
                var Column_Name = selected_sub.options[j].text;
                var Column_Value = selected_sub.options[j].value;

                available_sub.options[available_sub.length] = new Option(Column_Name, Column_Value);
              }
              // Remove all entries from the SelectedColumns array
              SelectedColumns[i].items.length = 0;

              // select the first item in the Available Sub Accounts Column list
              available_sub.selectedIndex = 0;

              // clear the Selected Sub Accounts Column List
              selected_sub.options.length = 0;
            }
            else
            {
              alert("You must have Sub Accounts to work with!");
            }
          }
          else
          {
            alert("Error: Invalid button name!\\nName = '"+bname+"'");
          }
        }
        else
        {
          alert("Error!  Unknown Column Header specified!\\nType given = '" + col_type + "'");
        }
      }
    }
  }

  function DisplayItem(col_type, bname)
  {
    var bottomWindowName = window.parent.name + '_bottom';
    var w = window.open("", bottomWindowName);
    var d = w.document;

    if (col_type == "sub_entities")
    {
      // error!
      alert("DisplayItem should never be called from a Sub Entity Screen!");
    }
    else
    {
      if (col_type == "sub_account_types")
      {
        // error!
        alert("DisplayItem should never be called from a Sub Account Type Screen!");
      }
      else
      {
        if (col_type == "sub_accounts")
        {
          if (bname == "populate")
          {
            var selected = d.header_form.selected_items;
            var selected_sub = d.header_form.selected_sub_items;
            var available_sub = d.header_form.available_sub_items;
            var index = selected.selectedIndex;

            if (index > -1)
            {
              available_sub.options.length = 0;  // clear the Available Sub Accounts list box.

              // first we need to get the index of the Sub Accounts in the SelectedColumn array.
              var k, found = false;
              for (k=0; k < SelectedColumns.length && !found; k++)
              {
                if (SelectedColumns[k].value == "sub_accounts")
                {
                  found = true;
                }
              }
              if (!found)
              {
                alert("Error:  Could not find Sub Account entry in SelectedColumns Array!\\nThis means there is a major problem!");
                return;
              }
              k--;  // point to the correct entry.

              // now we find the entry in SubAccountTypes for the SubAccountType selected.
              var j;
              for (j=0, found=false; j < SubAccountTypes.length && !found; j++)
              {
                if (SubAccountTypes[j].num == selected.options[index].value)
                {
                  found = true;
                }
              }
              if (!found)
              {
                alert("Error:  Could not find Sub Account Type entry in SubAccountTypes Array!\\nThis means there is a major problem!");
                return;
              }
              j--;

              // now we walk through the list of sub accounts for this sub account type and display all those not currently selected in the available_sub_items list box.
              var i;
              for (i=0; i < SubAccountTypes[j].sub_accounts.length; i++)
              {
                var m, found=false;
                for (m=0; m < SelectedColumns[k].items.length && !found; m++)
                {
                  if (SelectedColumns[k].items[m].value == SubAccountTypes[j].sub_accounts[i].num)
                  {
                    found = true;
                  }
                }
                if (!found)
                {
                  // add this sub account to the available_sub_items list box.
                  available_sub.options[available_sub.length] = new Option(SubAccountTypes[j].sub_accounts[i].name, SubAccountTypes[j].sub_accounts[i].num);
                }
              }

              if (available_sub.length == 0)
              {
                if (SubAccountTypes[j].sub_accounts.length == 0)
                {
                  alert("This Sub Account Type does not have any Sub Accounts to work with!");
                }
                else
                {
                  alert("There are no un-selected Sub Accounts to choose from!");
                }
              }

              // set the first item to be selected
              available_sub.selectedIndex = 0;
            }
            else
            {
              if (selected.length > 0)
              {
                alert("Error:  You must select a Sub Account Type to work with!");
              }
              else
              {
                alert("Error:  You must select Sub Account Types to work with first!");
              }
            }
          }
          else
          {
            if (bname == "populate_all")
            {
              var selected = d.header_form.selected_items;
              var selected_sub = d.header_form.selected_sub_items;
              var available_sub = d.header_form.available_sub_items;

              if (selected.length > 0)
              {
                available_sub.options.length = 0;  // clear the Available Sub Accounts list box.

                // first we need to get the index of the Sub Accounts in the SelectedColumn array.
                var k, found = false;
                for (k=0; k < SelectedColumns.length && !found; k++)
                {
                  if (SelectedColumns[k].value == "sub_accounts")
                  {
                    found = true;
                  }
                }
                if (!found)
                {
                  alert("Error:  Could not find Sub Account entry in SelectedColumns Array!\\nThis means there is a major problem!");
                  return;
                }
                k--;  // point to the correct entry.

                var z;
                for (z=0; z < selected.length; z++)  // walk through all Selected Sub Account Types and display all non-selected sub accounts.
                {
                  // now we find the entry in SubAccountTypes for the SubAccountType selected.
                  var j;
                  for (j=0, found=false; j < SubAccountTypes.length && !found; j++)
                  {
                    if (SubAccountTypes[j].num == selected.options[z].value)
                    {
                      found = true;
                    }
                  }
                  if (!found)
                  {
                    alert("Error:  Could not find Sub Account Type '"+selected.options[z].value+"' in SubAccountTypes Array!\\nThis means there is a major problem!");
                    return;
                  }
                  j--;

                  // now we walk through the list of sub accounts for this sub account type and display all those not currently selected in the available_sub_items list box.
                  var i;
                  for (i=0; i < SubAccountTypes[j].sub_accounts.length; i++)
                  {
                    var m, found=false;
                    for (m=0; m < SelectedColumns[k].items.length && !found; m++)
                    {
                      if (SelectedColumns[k].items[m].value == SubAccountTypes[j].sub_accounts[i].num)
                      {
                        found = true;
                      }
                    }
                    if (!found)
                    {
                      // add this sub account to the available_sub_items list box.
                      available_sub.options[available_sub.length] = new Option(SubAccountTypes[j].sub_accounts[i].name, SubAccountTypes[j].sub_accounts[i].num);
                    }
                  }
                }

                if (available_sub.length == 0)
                {
                  alert("No Sub Accounts were found to display!");
                }

                // set the first item to be selected
                available_sub.selectedIndex = 0;
              }
              else
              {
                alert("Error:  You must select Sub Account Types to work with first!");
              }
            }
            else
            {
              alert("Error: Invalid button name!\\nName = '"+bname+"'");
            }
          }
        }
        else
        {
          alert("Error!  Unknown Column Header specified!\\nType given = '" + col_type + "'");
        }
      }
    }
  }

  function ClearItem(col_type, bname)
  {
    var bottomWindowName = window.parent.name + '_bottom';
    var w = window.open("", bottomWindowName);
    var d = w.document;

    if (col_type == "sub_entities")
    {
      // error!
      alert("ClearItem should never be called from a Sub Entity Screen!");
    }
    else
    {
      if (col_type == "sub_account_types")
      {
        // error!
        alert("ClearItem should never be called from a Sub Account Type Screen!");
      }
      else
      {
        if (col_type == "sub_accounts")
        {
          if (bname == "clear")
          {
            var available_sub = d.header_form.available_sub_items;

            if (available_sub.length > 0)
            {
              available_sub.options.length = 0;  // clear the Available Sub Accounts list box.
            }
            else
            {
              alert("Error:  You must have Sub Accounts available to work with first!");
            }
          }
          else
          {
            alert("Error: Invalid button name!\\nName = '"+bname+"'");
          }
        }
        else
        {
          alert("Error!  Unknown Column Header specified!\\nType given = '" + col_type + "'");
        }
      }
    }
  }

  function InitializeChildData(col_type)
  {
    var bottomWindowName = window.parent.name + '_bottom';
    var w = window.open("", bottomWindowName);
    var d = w.document;

    if (col_type == "sub_entities")
    {
      // ignore, do nothing.
    }
    else
    {
      if (col_type == "sub_account_types")
      {
        var available = d.header_form.available_items;
        var selected = d.header_form.selected_items;

        available.options.length = 0;  // clear both select boxes.
        selected.options.length = 0;

        // find our entry in the SelectedColumns Array
        var k, found=false;
        for (k=0; k < SelectedColumns.length && !found; k++)
        {
          if (SelectedColumns[k].value == "sub_account_types")
          {
            found = true;
          }
        }
        if (!found)
        {
          alert("Error:  Could not find Sub Account Type entry in SelectedColumns Array!\\nThis means there is a major problem!");
        }
        k--;  // point at the correct entry in the SelectedColumns array.

        // walk the Sub Account Types array and populate the Available list and the Selected List based upon the status of the SelectedColumns object.
        var i;
        for (i=0; i < SubAccountTypes.length; i++)
        {
          var j, found=false;
          for (j=0; j < SelectedColumns[k].items.length && !found; j++)
          {
            if (SelectedColumns[k].items[j].value == SubAccountTypes[i].num)
            {
              found = true;
            }
          }
          if (!found)
          { // Put this Sub Account Type in the Available List box.
            available.options[available.length] = new Option(SubAccountTypes[i].name, SubAccountTypes[i].num);
          }
        }
        // now walk the SelectedColumns array and put the items in the Selected box in the order we encounter them.
        for (i=0; i < SelectedColumns[k].items.length; i++)
        {
          selected.options[selected.length] = new Option(SelectedColumns[k].items[i].text, SelectedColumns[k].items[i].value);
        }
        available.selectedIndex = 0;  // Select the first item in both boxes.
        selected.selectedIndex = 0;
      }
      else
      {
        if (col_type == "sub_accounts")
        {
          var available = d.header_form.available_items;
          var selected = d.header_form.selected_items;
          var available_sub = d.header_form.available_sub_items;
          var selected_sub = d.header_form.selected_sub_items;

          available.options.length = 0;  // clear all select boxes.
          selected.options.length = 0;
          available_sub.options.length = 0;
          selected_sub.options.length = 0;

          // find our entry in the SelectedColumns Array
          var k, found=false;
          for (k=0; k < SelectedColumns.length && !found; k++)
          {
            if (SelectedColumns[k].value == "sub_accounts")
            {
              found = true;
            }
          }
          if (!found)
          {
            alert("Error:  Could not find Sub Account entry in SelectedColumns Array!\\nThis means there is a major problem!");
          }
          k--;  // point at the correct entry in the SelectedColumns array.

          // walk the Sub Account Types array and populate the Available list.
          var i;
          for (i=0; i < SubAccountTypes.length; i++)
          {
            available.options[available.length] = new Option(SubAccountTypes[i].name, SubAccountTypes[i].num);
          }
          // now walk the SelectedColumns array and put the items in the Selected Sub Account box in the order we encounter them.
          for (i=0; i < SelectedColumns[k].items.length; i++)
          {
            selected_sub.options[selected_sub.length] = new Option(SelectedColumns[k].items[i].text, SelectedColumns[k].items[i].value);
          }
          available.selectedIndex = 0;  // Select the first item in all boxes.
          selected.selectedIndex = 0;
          available_sub.selectedIndex = 0;
          selected_sub.selectedIndex = 0;
        }
        else
        {
          alert("Error!  Unknown Column Header specified!\\nType given = '" + col_type + "'");
        }
      }
    }
  }

  function DisplaySelectedColumnHeader()
  {  // this function will display the selected column header and it's current state in the bottom frame.
    var s = document.report_form.selected_columns;
    var selected_index = s.selectedIndex;

    if (selected_index > -1)
    {
      var bottomWindowName = window.parent.name + '_bottom';
      var w = window.open("", bottomWindowName);
      var d = w.document;
      var header = s.options[selected_index];

      d.open();

      // output the html form that will work with the selected Column Header Type
      d.writeln("<html>");
      d.writeln("<head>\\n</head>");
      d.writeln("<body bgcolor=\\\"white\\\" onLoad=\\\"opener.InitializeChildData('" + header.value + "')\\\">");

      // start the form
      d.writeln("  <form name=\\\"header_form\\\" onSubmit=\\\"return false;\\\">");

      // What kind of Column Header are we?
      if (header.value == "sub_entities")
      {
        d.writeln("  <center><h1>No Customization Available!</h1></center>");
      }
      else
      {
        if (header.value == "sub_account_types")
        {
          d.writeln("  <center><h2>Sub Account Types - Customization</h2></center>");
          d.writeln("  <table border=1 cellpadding=2 cellspacing=0 align=center>");
          d.writeln("  <tr bgcolor=\\\"#cccccc\\\">");
          d.writeln("    <th>Available</th><th>Selected</th>");
          d.writeln("  </tr>");
          d.writeln("  <tr bgcolor=\\\"#eeeeee\\\">");
          d.writeln("    <td align=center valign=center>");
          // create the table with the Available Select box and the input and remove buttons.
          d.writeln("      <table border=0 align=center cellpadding=0 cellspacing=0>");
          d.writeln("        <tr>");
          d.writeln("          <td align=center width=85%><select name=\\\"available_items\\\" size=\\\"5\\\" width=\\\"150\\\">");
          d.writeln("            <option value=\\\"---------------\\\">---------------");
          // The Available Sub Account Types will be filled in when the page is initialized.
          d.writeln("            </select>");
          d.writeln("          </td>");
          d.writeln("          <td align=center valign=center width=15%>");
          d.writeln("            <input type=button name=\\\"insert\\\" value=\\\">\\\" onClick=\\\"opener.InsertItem('"+header.value+"', 'insert')\\\"><br><input type=button name=\\\"remove\\\" value=\\\"<\\\" onClick=\\\"opener.RemoveItem('"+header.value+"', 'remove')\\\">");
          d.writeln("            <br><input type=button name=\\\"insert_all\\\" value=\\\">>\\\" onClick=\\\"opener.InsertAllItems('"+header.value+"', 'insert_all')\\\"><br><input type=button name=\\\"remove_all\\\" value=\\\"<<\\\" onClick=\\\"opener.RemoveAllItems('"+header.value+"', 'remove_all')\\\">");
          d.writeln("          </td>");
          d.writeln("        </tr>");
          d.writeln("      </table>");
          d.writeln("    </td>");
          d.writeln("    <td align=center valign=center>");
          // create the Selected Select box
          d.writeln("      <select name=\\\"selected_items\\\" size=\\\"5\\\" width=\\\"150\\\">");
          d.writeln("        <option value=\\\"---------------\\\">---------------");
          d.writeln("      </select>");
          d.writeln("    </td>");
          d.writeln("  </tr>");
          d.writeln("  </table>");
        }
        else
        {
          if (header.value == "sub_accounts")
          {
            d.writeln("  <center><h2>Sub Accounts - Customization</h2></center>");
            d.writeln("  <table border=1 cellpadding=2 cellspacing=0 align=center>");
            d.writeln("  <tr bgcolor=\\\"#cccccc\\\">");
            d.writeln("    <th>Available Sub Account Types</th><th>Selected Sub Account Types</th><th>Available Sub Accounts</th><th>Selected Sub Accounts</th>");
            d.writeln("  </tr>");
            d.writeln("  <tr bgcolor=\\\"#eeeeee\\\">");
            d.writeln("    <td align=center valign=center>");
            // create the table with the Available Select box and the input and remove buttons.
            d.writeln("      <table border=0 align=center cellpadding=0 cellspacing=0>");
            d.writeln("        <tr>");
            d.writeln("          <td align=center width=85%><select name=\\\"available_items\\\" size=\\\"5\\\" width=\\\"150\\\">");
            d.writeln("            <option value=\\\"---------------\\\">---------------");
            // The Available Sub Account Types will be filled in when the page is initialized.
            d.writeln("            </select>");
            d.writeln("          </td>");
            d.writeln("          <td align=center valign=center width=15%>");
            d.writeln("            <input type=button name=\\\"insert2\\\" value=\\\">\\\" onClick=\\\"opener.InsertItem('sub_account_types', 'insert2')\\\"><br><input type=button name=\\\"remove2\\\" value=\\\"<\\\" onClick=\\\"opener.RemoveItem('sub_account_types', 'remove2')\\\">");
            d.writeln("            <br><input type=button name=\\\"insert_all2\\\" value=\\\">>\\\" onClick=\\\"opener.InsertAllItems('sub_account_types', 'insert_all2')\\\"><br><input type=button name=\\\"remove_all2\\\" value=\\\"<<\\\" onClick=\\\"opener.RemoveAllItems('sub_account_types', 'remove_all2')\\\">");
            d.writeln("          </td>");
            d.writeln("        </tr>");
            d.writeln("      </table>");
            d.writeln("    </td>");
            d.writeln("    <td align=center valign=center>");
            // create the table with the Available Sub Accounts box and the input and remove buttons.
            d.writeln("      <table border=0 align=center cellpadding=0 cellspacing=0>");
            d.writeln("        <tr>");
            // create the Selected Sub Account Type Select box
            d.writeln("          <td align=center width=80%><select name=\\\"selected_items\\\" size=\\\"5\\\" width=\\\"150\\\">");
            d.writeln("            <option value=\\\"---------------\\\">---------------");
            d.writeln("            </select>");
            d.writeln("          </td>");
            d.writeln("          <td align=center valign=center width=20%>");
            d.writeln("            <input type=button name=\\\"populate\\\" value=\\\"show\\\" onClick=\\\"opener.DisplayItem('"+header.value+"', 'populate')\\\"><br><input type=button name=\\\"clear\\\" value=\\\"clear\\\" onClick=\\\"opener.ClearItem('"+header.value+"', 'clear')\\\">");
            d.writeln("            <br><input type=button name=\\\"populate_all\\\" value=\\\"show all\\\" onClick=\\\"opener.DisplayItem('"+header.value+"', 'populate_all')\\\">");
            d.writeln("          </td>");
            d.writeln("        </tr>");
            d.writeln("      </table>");
            d.writeln("    </td>");
            d.writeln("    <td align=center valign=center>");
            // create the table with the Available Sub Accounts box and the input and remove buttons.
            d.writeln("      <table border=0 align=center cellpadding=0 cellspacing=0>");
            d.writeln("        <tr>");
            d.writeln("          <td align=center width=85%><select name=\\\"available_sub_items\\\" size=\\\"5\\\" width=\\\"150\\\">");
            d.writeln("            <option value=\\\"---------------\\\">---------------");
            // The Available Sub Accounts will be filled in when the page is initialized.
            d.writeln("            </select>");
            d.writeln("          </td>");
            d.writeln("          <td align=center valign=center width=15%>");
            d.writeln("            <input type=button name=\\\"insert_sub\\\" value=\\\">\\\" onClick=\\\"opener.InsertItem('"+header.value+"', 'insert_sub')\\\"><br><input type=button name=\\\"remove_sub\\\" value=\\\"<\\\" onClick=\\\"opener.RemoveItem('"+header.value+"', 'remove_sub')\\\">");
            d.writeln("            <br><input type=button name=\\\"insert_all_sub\\\" value=\\\">>\\\" onClick=\\\"opener.InsertAllItems('"+header.value+"', 'insert_all_sub')\\\"><br><input type=button name=\\\"remove_all_sub\\\" value=\\\"<<\\\" onClick=\\\"opener.RemoveAllItems('"+header.value+"', 'remove_all_sub')\\\">");
            d.writeln("          </td>");
            d.writeln("        </tr>");
            d.writeln("      </table>");
            d.writeln("    </td>");
            d.writeln("    <td align=center valign=center>");
            // create the Selected Sub Account Select box
            d.writeln("      <select name=\\\"selected_sub_items\\\" size=\\\"5\\\" width=\\\"150\\\">");
            d.writeln("        <option value=\\\"---------------\\\">---------------");
            d.writeln("      </select>");
            d.writeln("    </td>");
            d.writeln("  </tr>");
            d.writeln("  </table>");
          }
          else
          {
            alert("Invalid Column Header Type!\\nType given = '"+header.value+"'\\nType name = '"+header.text+"'");
          }
        }
      }

      d.writeln("  </form>");
      d.writeln("</body>");
      d.writeln("</html>");

      d.close();  // make sure we close the document.
    }
    else
    {
      if (s.length == 0)
      {
        alert("You must have entries in the Selected Column Headers box before you can Customize the Column Header!");
      }
      else
      {
        alert("You must select a Column Header to work with!");
      }
    }
  }

  function CreateColumnHeaderString()
  {
    var data_string = "";
    var i;

    for (i=0; i < SelectedColumns.length; i++)
    {
      data_string += SelectedColumns[i].value;

      // seperate by a , if more Column Headers
      if (i < SelectedColumns.length-1)
      {
        data_string += ",";
      }
    }

    return data_string;
  }

  function CreateColumnHeaderDataString()
  {
    var data_string = "";
    var i;

    for (i=0; i < SelectedColumns.length; i++)
    {
      if (SelectedColumns[i].items.length > 0)
      {
        var j;
        for (j=0; j < SelectedColumns[i].items.length; j++)
        {
          data_string += SelectedColumns[i].items[j].value;

          // Seperate by a , if more items for this Column Header.
          if (j < SelectedColumns[i].items.length-1)
          {
            data_string += ",";
          }
        }
      }

      // seperate by a | if more Column Headers
      if (i < SelectedColumns.length-1)
      {
        data_string += "|";
      }
    }

    return data_string;
  }
END_OF_CODE
  }
  else  # Detailed View Report
  {
    $doc->print(<<"END_OF_CODE");
    function validate_form()
    {
      // Make sure that the user has entered everything that is required.
      var d = document.report_form;
      if (d.name.value.length == 0)
      {
        alert("You must enter the Name for the report!");
        d.name.focus();
        return false;
      }

      return true;  // change to true when done debugging form.
    }

    function reset_form()
    {
      document.report_form.reset();

      window.setTimeout("document.report_form.name.value = document.report_form.original_name.value;initialize();", 10);
    }
END_OF_CODE
  }

  $doc->display();
}

# report_second_screen_edit_create
# takes: mode, id
# creates or edits the specified report's rows.
sub report_second_screen_edit_create
{
  my %args = ( @_, );
  my $mode = $args{'mode'};
  my $id = $args{'id'};
  my $dbh = $args{'dbh'};
  my $name = "";

  my @arguments = ( "mode", "id", "dbh" );

  eval ("use HTMLObject::Normal;");
  if ($@)
  {
    my_die("Error: Report.cgi - Eval", 4002, "Error: $@ at line " .__LINE__);
  }

  my $doc = HTMLObject::Normal->new();

  $doc->setJavascriptErrorHandlerEmail(email => $email_address);

  foreach my $value (@arguments)
  {
    if (!exists $args{$value})
    { # error!
      $doc->doRequiredParameterError('report_second_screen_edit_create', $value);
    }
    if (length $args{$value} == 0)
    {
      my_die("Error: Report.cgi - report_second_screen_edit_create", 4005, "Parameter = '$value'");
    }
    elsif (!validateParameter($value, $args{$value}))
    {
      my_die("Error: Report.cgi - report_second_screen_edit_create", 4006, "$value = '$args{$value}'");
    }
  }

  my $title = "";
  my $target_window = "_top";
  my $command = "update_report";
  my $header = "";
  my ($sth, $rc, @info);

  # validate that the report exists in the report_type_tb.
  $name = DoesReportExist(dbh => $dbh, id => $id);
  if (length $name == 0)
  {
    my_die("Error: Report.cgi - $mode". "_report2_top", 4020, "ID = '$id' at line " . __LINE__);
  }

  # get the data for the Report we Are editing.
  my @report_info = GetReportData(dbh => $dbh, id => $id);

  my $report_type = $report_info[2];
  my $fg_color = $report_info[14];
  my $bg_color = $report_info[15];
  my $date_range = $report_info[5];

  if ($mode eq "create")
  {
    $title = "Create Report Rows";
    $header = "Create Report Rows - '$name', ID = $id";
  }
  elsif ($mode eq "edit")
  {
    $title = "Edit Report Rows";
    $header = "Edit Report Rows - '$name', ID = $id";
  }

  my $doc = HTMLObject::Normal->new();
  $doc->setJavascriptErrorHandlerEmail(email => $email_address);
  $doc->setTitle($title);
  $doc->setFocus("body");
  $doc->print(<<"END_OF_BODY");
  <center><h1>$header</h1></center>
  <form name="work_form" method=post action="$site_dir/cgi-bin/report_editor.cgi" onSubmit="return false;">
END_OF_BODY
  $doc->print(<<"END_OF_BODY");
  <table border=1 cellpadding=2 cellspacing=1 align=center>
    <tr bgcolor="#cccccc">
      <th>Row #</th><th>Row Type</th><th>Visible</th>
    </tr>
    <tr bgcolor="#eeeeee">
      <td align=center>
        <input type=hidden name="real_index" value="0">
        <input type=text name="index" value="0" size="2" maxlength="2">
      </td>
      <td align=center>
        <select name="cell_type" onChange="display_middle_window(EditMode);">
END_OF_BODY
  if ($report_type eq "r")
  {
    foreach my $item (@regular_cell_types)
    {
      $doc->print("          <option value=\"$item\"" . ($item eq "text" ? " selected" : "") . ">$regular_cell_types{$item}\n");
    }
  }
  else
  {
    foreach my $item (@detailed_cell_types)
    {
      $doc->print("          <option value=\"$item\"" . ($item eq "text" ? " selected" : "") . ">$detailed_cell_types{$item}\n");
    }
  }
  $doc->print(<<"END_OF_BODY");
        </select>
      </td>
      <td align=center>
        <input type=checkbox name="show" value="true" checked onClick="Checked=true;validate_show_row();">
      </td>
    </tr>
END_OF_BODY
  $doc->print(<<"END_OF_BODY");
    </table>
  </form>

  <table border=0 width=100%>
    <tr>
      <td align=left width=50%>
        <form name="report_form" method=post action="$site_dir/cgi-bin/report_editor.cgi" target="$target_window" onSubmit="return validate_form()">
          <input type=hidden name="command" value="$command">
          <input type=hidden name="id" value="$id">
          <input type=hidden name="mode" value="$mode">
          <input type=hidden name="report_type" value="$report_type">
          <input type=hidden name="rows" value="">
          <input type=hidden name="redisplay" value="false">
          <input type="hidden" name="windowName" value="">
          <input type=submit name="finish" value="Finish">&nbsp;<input type=button name="reset_report_button" value="Reset" onClick="reset_report();">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<input type=button name="quick_save" value="Quick Save" onClick="document.report_form.redisplay.value='true';if (validate_form()) document.report_form.submit();">
        </form>
      </td>
      <td align=right width=50%>
        <form name="edit_form" method=post action="$site_dir/cgi-bin/report_editor.cgi" target="_top">
          <input type=hidden name="command" value="display_for_edit">
          <div align=right><a href="" onClick="document.edit_form.submit();return false;" onMouseOver="window.status='Return to Edit/Create Report - Main Screen';">Edit/Create Report - Main Screen</a></div>
        </form>
      </td>
    </tr>
  </table>
END_OF_BODY

  # turn off the link decorations
  $doc->setLinkDecorations();

  $doc->setFocus("javascript");
  if ($report_type eq "r")
  {
  $doc->print(<<"END_OF_CODE");
  var RowBeingEdited = -1;
  var EditMode = "create";

  var Rows = new Array();

  function rowEntry(index, cell_type, item_name, cell_text, cell_formula, activity, negative_format, bold, fg_color, bg_color, sub_levels, initial_state, display_level, drillable, drill_report, query_type, show, result_type, underline)
  {
    this.index = index;
    this.cell_type = cell_type;
    this.item_name = item_name;
    this.cell_text = cell_text;
    this.cell_formula = cell_formula;
    this.activity = activity;
    this.negative_format = negative_format;
    this.bold = bold;
    this.fg_color = fg_color;
    this.bg_color = bg_color;
    this.sub_levels = sub_levels;
    this.initial_state = initial_state;
    this.display_level = display_level;
    this.drillable = drillable;
    this.drill_report = drill_report;
    this.query_type = query_type;
    this.show = show;
    this.result_type = result_type;
    this.underline = underline;
  }

  function AccountTypeEntry(name, num)
  {
    this.name = name;
    this.num = num;
  }

  function AccountEntry(name, num, type, active)
  {
    this.name = name;
    this.num = num;
    this.type = type;
    this.active = active;
  }

  function SubAccountEntry(name, num, type)
  {
    this.name = name;
    this.num = num;
    this.type = type;
  }

  function ReportEntry(name, id)
  {
    this.name = name;
    this.id = id;
  }

END_OF_CODE

  # gather the Account Types, Accounts, Sub Accounts and Detailed Reports

  # Account Types
  $doc->print("  var AccountTypesArray = new Array();\n");

  $sth = $dbh->prepare( $sql_statements{1019} ) || my_db_die("Error: Report.cgi - $mode" . "_report2_top", 4007, "SQL Statement = '$sql_statements{1019}'");
  $rc = $sth->execute || my_db_die("Error: Report.cgi - $mode" . "_report2_top", 4008, "SQL Statement = '$sql_statements{1019}'");

  my $counter = 0;
  while (@info = $sth->fetchrow_array)
  {
    $doc->print("  AccountTypesArray[$counter] = new AccountTypeEntry(\"$info[0]\", \"$info[1]\");\n");
    $counter++;
  }
  $sth->finish;

  # Accounts
  $doc->print(<<"END_OF_CODE");

  var AccountsArray = new Array();
END_OF_CODE

  $sth = $dbh->prepare( $sql_statements{1020} ) || my_db_die("Error: Report.cgi - $mode" . "_report2_top", 4007, "SQL Statement = '$sql_statements{1020}'");
  $rc = $sth->execute || my_db_die("Error: Report.cgi - $mode" . "_report2_top", 4008, "SQL Statement = '$sql_statements{1020}'");

  $counter = 0;
  while (@info = $sth->fetchrow_array)
  {
    $doc->print("  AccountsArray[$counter] = new AccountEntry(\"$info[0]\", \"$info[1]\", \"$info[2]\", \"$info[3]\");\n");
    $counter++;
  }
  $sth->finish;

  # Sub Accounts
  $doc->print(<<"END_OF_CODE");

  var SubAccountsArray = new Array();
END_OF_CODE

  $sth = $dbh->prepare( $sql_statements{1021} ) || my_db_die("Error: Report.cgi - $mode" . "_report2_top", 4007, "SQL Statement = '$sql_statements{1021}'");
  $rc = $sth->execute || my_db_die("Error: Report.cgi - $mode" . "_report2_top", 4008, "SQL Statement = '$sql_statements{1021}'");

  $counter = 0;
  while (@info = $sth->fetchrow_array)
  {
    $doc->print("  SubAccountsArray[$counter] = new SubAccountEntry(\"$info[0]\", \"$info[1]\", \"$info[2]\");\n");
    $counter++;
  }
  $sth->finish;

  # Detailed Reports
  $doc->print(<<"END_OF_CODE");

  var DetailedReportsArray = new Array();
END_OF_CODE

  my @detailed_reports = get_all_detailed_reports( dbh => $dbh );

  for ($counter=0; $counter < @detailed_reports; $counter++)
  {
    $doc->print("  DetailedReportsArray[$counter] = new ReportEntry(\"$detailed_reports[$counter][1]\", \"$detailed_reports[$counter][0]\");\n");
  }

  $doc->print(<<"END_OF_CODE");

  var ReportDisplayLevels = new Array();
END_OF_CODE

  foreach my $item (@report_levels)
  {
    $doc->print("  ReportDisplayLevels[\"$item\"] = new Array();\n");
    my $counter = 0;
    foreach my $level (@{$report_levels{$item}})
    {
      $doc->print("  ReportDisplayLevels[\"$item\"][" . $counter++ . "] = \"$level\";\n");
    }
  }

  $doc->print(<<"END_OF_CODE");

  var Fonts = new Array();
END_OF_CODE

  foreach my $item (@font_names)
  {
    $doc->print("  Fonts[\"$item\"] = \"$font_names{$item}\";\n");
  }

  $doc->print(<<"END_OF_CODE");

  var fg_colors = new Array();
END_OF_CODE

  foreach my $item (@fg_colors)
  {
    $doc->print("  fg_colors[\"$item\"] = \"$fg_colors{$item}\";\n");
  }

  $doc->print(<<"END_OF_CODE");

  var bg_colors = new Array();
END_OF_CODE

  foreach my $item (@bg_colors)
  {
    $doc->print("  bg_colors[\"$item\"] = \"$bg_colors{$item}\";\n");
  }

  $doc->print(<<"END_OF_CODE");

  var negative_formats = new Array();
END_OF_CODE

  foreach my $item (@negative_formats)
  {
    $doc->print("  negative_formats[\"$item\"] = '$negative_formats{$item}';\n");
  }

  $doc->print(<<"END_OF_CODE");

  var cell_types = new Array();
END_OF_CODE

  foreach my $item (@regular_cell_types)
  {
    $doc->print("  cell_types[\"$item\"] = '$regular_cell_types{$item}';\n");
  }

  $doc->print(<<"END_OF_CODE");

  var YesNo = new Array();
  YesNo["true"]  = "Yes";
  YesNo["false"] = "No";
  YesNo["0"] = "No";
  YesNo["1"] = "Yes";

  var TrueFalse = new Array();
  TrueFalse["1"] = "true";
  TrueFalse["0"] = "false";

  var Drillable = new Array();
  Drillable["0"] = "Yes";
  Drillable["1"] = "No";

  var RadioTF = new Array();
  RadioTF["0"] = "true";
  RadioTF["1"] = "false";

  var RowState = new Array();
  RowState["o"] = "Opened";
  RowState["c"] = "Closed";

  var DisplayLevels = new Array();
  DisplayLevels["0"] = "Level 1";
  DisplayLevels["1"] = "&nbsp;&nbsp;Level 2";
  DisplayLevels["2"] = "&nbsp;&nbsp;&nbsp;&nbsp;Level 3";

  var CalcType = new Array();
  CalcType["true"] = "Activity";
  CalcType["false"] = "Balance";

  var QueryType = new Array();
  QueryType["0"] = "Taxable And Non-Taxable";
  QueryType["1"] = "Taxable Only";
  QueryType["2"] = "Non-Taxable Only";

  var Clicked = false;  // Used to know when the Drillable checkbox is clicked.
  var Checked = false;  // Used to know when the Show checkbox is clicked.

  var FormulaWindow;  // Reference to the FormulaWindow.
  var CompareFormulaWindow;  // Reference to the CompareFormulaWindow.
  var MiddleWindow;	// Reference to the Middle Frame.
  var TopWindow;	// Reference to the Top Frame.

  var Date_Range = $date_range;  // Indicates if we are doing an activity or balance report.

  // Helper functions

  function DisplayForeGroundColors(fg_color)
  {
    var string = '';

    string += '      <select name="fg_color">\\n';
    for (var color in fg_colors)
    {
      string += '        <option value="'+color+'"'+(fg_color == color ? " selected" : "")+'>' + fg_colors[color] + '\\n';
    }
    string += '      </select>\\n';

    return string;
  }

  function DisplayBackGroundColors(bg_color)
  {
    var string = '';

    string += '      <select name="bg_color">\\n';
    for (var color in bg_colors)
    {
      string += '        <option value="'+color+'"'+(bg_color == color ? " selected" : "")+'>' + bg_colors[color] + '\\n';
    }
    string += '      </select>\\n';

    return string;
  }

  function DisplayNegativeFormats(format)
  {
    var string = '';

    for (var negative_format in negative_formats)
    {
      string += '        <input type=radio name="negative_format" value="'+negative_format+'"'+(negative_format == format ? " checked" : "")+'>&nbsp;'+negative_formats[negative_format]+'<br>\\n';
    }

    return string;
  }

  function DisplayBold(bold)
  {
    var string = '';

    string += '      <select name="bold">\\n';
    string += '        <option value="true"'+(bold == "true" ? " selected" : "")+'>Yes\\n';
    string += '        <option value="false"'+(bold == "false" ? " selected" : "")+'>No\\n';
    string += '      </select>\\n';

    return string;
  }

  function DisplayInitialState(initial_state)
  {
    var string = '';

    string += '      <select name="initial_state">\\n';
    string += '        <option value="o"'+(initial_state == "o" ? " selected" : "")+'>Opened\\n';
    string += '        <option value="c"'+(initial_state == "c" ? " selected" : "")+'>Closed\\n';
    string += '      </select>\\n';

    return string;
  }

  function DisplayDisplayLevel(display_level)
  {
    var string = '';

    string += '      <select name="display_level">\\n';
    string += '        <option value="0"'+(display_level == "0" ? " selected" : "")+'>Level 1\\n';
    string += '        <option value="1"'+(display_level == "1" ? " selected" : "")+'>&nbsp;&nbsp;Level 2\\n';
    string += '        <option value="2"'+(display_level == "2" ? " selected" : "")+'>&nbsp;&nbsp;&nbsp;&nbsp;Level 3\\n';
    string += '      </select>\\n';

    return string;
  }

  function DisplayQueryType(query_type)
  {
    var string = '';

    string += '      <select name="query_type">\\n';
    string += '        <option value="0"'+(query_type == "0" ? " selected" : "")+'>Taxable And Non-Taxable\\n';
    string += '        <option value="1"'+(query_type == "1" ? " selected" : "")+'>Taxable Only\\n';
    string += '        <option value="2"'+(query_type == "2" ? " selected" : "")+'>Non-Taxable Only\\n';
    string += '      </select>\\n';

    return string;
  }

  function DisplayAccountTypes(account_type, field_name)
  {
    var string = '';
    if (account_type == '')
    {
      account_type = AccountTypesArray[0].num;  // point to the first item.
    }

    string += '      <select name="'+field_name+'">\\n';
    for (var i=0; i < AccountTypesArray.length; i++)
    {
      string += '        <option value="'+AccountTypesArray[i].num+'"'+(account_type == AccountTypesArray[i].num ? " selected" : "")+'>'+AccountTypesArray[i].name+'\\n';
    }
    string += '      </select>\\n';

    return string;
  }

  function DisplayAccounts(account, field_name)
  {
    var string = '';
    if (account == '')
    {
      account = AccountsArray[0].num;  // point to the first item.
    }

    string += '      <select name="'+field_name+'">\\n';
    for (var i=0; i < AccountsArray.length; i++)
    {
      string += '        <option value="'+AccountsArray[i].num+'"'+(account == AccountsArray[i].num ? " selected" : "")+'>'+AccountsArray[i].name+'\\n';
    }
    string += '      </select>\\n';

    return string;
  }

  function DisplaySubAccounts(sub_account, field_name)
  {
    var string = '';
    if (sub_account == '')
    {
      sub_account = SubAccountsArray[0].num;  // point to the first item.
    }

    string += '      <select name="'+field_name+'">\\n';
    for (var i=0; i < SubAccountsArray.length; i++)
    {
      string += '        <option value="'+SubAccountsArray[i].num+'"'+(sub_account == SubAccountsArray[i].num ? " selected" : "")+'>'+SubAccountsArray[i].name+'\\n';
    }
    string += '      </select>\\n';

    return string;
  }

  function DisplaySubLevels(cell_type, sub_level)
  {
    var string = '';

    string += '      <select name="sub_levels">\\n';
    for (var i=0; i < ReportDisplayLevels[cell_type].length; i++)
    {
      string += '        <option value="'+ReportDisplayLevels[cell_type][i]+'"'+(sub_level == ReportDisplayLevels[cell_type][i] ? " selected" : "")+'>'+ReportDisplayLevels[cell_type][i]+'\\n';
    }
    string += '      </select>\\n';

    return string;
  }

  function DisplayDetailedReports()
  {
    var string = '';

    string += '      <select name="drill_report">\\n';
    string += '        <option value="---------------" selected>---------------\\n';
    string += '      </select>\\n';

    return string;
  }

  function DisplayDrillable(drillable)
  {
    var string = '';

    string += '      <input type=radio name="drillable" value="true"'+(!drillable ? " checked" : "")+' onClick="window.opener.Clicked=true;window.opener.process_drillable(true);window.opener.Clicked=false;">Yes<br>\\n';
    string += '      <input type=radio name="drillable" value="false"'+(drillable ? " checked" : "")+' onClick="window.opener.Clicked=true;window.opener.process_drillable(false);window.opener.Clicked=false;">No\\n';

    return string;
  }

  function DisplayResultType(result_type)
  {
    var string = '';

    string += '        <input type=checkbox name="result_type" value="on"'+(result_type == "result_formula" ? " checked" : "")+'>&nbsp;Results Use Formulas\\n';

    return string;
  }

  function DisplayUnderlineRow(underline)
  {
    var string = '';

    string += '        <input type=checkbox name="underline" value="on"'+(underline ? " checked" : "")+'>\\n';

    return string;
  }

  function validate_show_row()
  {
    var d = document.work_form;

    if (d.cell_type.selectedIndex == -1)
    {
      alert("You must select a Row Type first!");
      return;
    }
    var cell_type = d.cell_type.options[d.cell_type.selectedIndex].value;
    var Type_text = d.cell_type.options[d.cell_type.selectedIndex].text;

    if (!d.show.checked)
    {
      if (cell_type == "text" || cell_type == "line" || cell_type == "detail")
      {
        alert("You can not hide a " + Type_text + " Row!");
        d.show.checked = true; // force it to be selected.
      }
    }

    if (Checked)
    {
      Checked = false;
      display_middle_window(EditMode);  // update the displayall of this row.
    }
  }

  function serialize_rows()
  {
    var d = document.report_form;

    if (Rows.length > 0)
    {
      var data = "";
      for (var i=0; i < Rows.length; i++)
      {
        if (i > 0)
        {
          data += "\\n";  // seperate rows by a new line.
        }
        data += "index="+Rows[i].index+"|cell_text="+Rows[i].cell_text+"|cell_formula="+Rows[i].cell_formula+"|cell_type="+Rows[i].cell_type+"|item_name="+Rows[i].item_name+"|fg_color="+Rows[i].fg_color+"|bg_color="+Rows[i].bg_color+"|bold="+Rows[i].bold;
        data += "|sub_levels="+Rows[i].sub_levels+"|display_level="+Rows[i].display_level+"|initial_state="+Rows[i].initial_state+"|drillable="+RadioTF[Rows[i].drillable]+"|drill_report="+Rows[i].drill_report+"|negative_format="+Rows[i].negative_format+"|activity="+Rows[i].activity;
        data += "|query_type="+Rows[i].query_type+"|show="+Rows[i].show+"|result_type="+Rows[i].result_type+"|underline="+Rows[i].underline;
      }
      d.rows.value = data;
    }
    else
    {
      d.rows.value = "";
    }
    return true;
  }

  function validate_form()
  {
    // Make sure that the user has entered everything that is required.
    var d = document.report_form;
    //alert("Entering Validate Form");

    for (var i=0; i < Rows.length; i++)
    {
      //alert("i='"+i+"'");
      if (Rows[i].cell_type == "total")
      {  // validate the formula
        //alert("Validate Form - Total Row = "+i);
        var formula = Rows[i].cell_formula;
        if (validate_formula(formula, i) == false)
        {
          return false;
        }
      }
      else if (Rows[i].cell_type == "compare")
      {  // validate the comparison formula (if possible).
        if (validate_comparison_formula(Rows[i].cell_formula, i) == false)
        {
          return false;
        }
      }
    }

    //alert("About to Serialize Rows!");
    if (!serialize_rows())
    {
      alert("Serialization of Rows failed!");
      return false;
    }

    if (d.rows.value.length == 0)
    {
      alert("You must define at least 1 row for this report!");
      return false;
    }

    //alert("Rows = '"+d.rows.value+"'");

    // make sure the windowName is set.
    d.windowName.value = window.parent.name;

    return true;  // change to true when done debugging form.
  }

  function clear_select_box(box)
  {
    box.options.length = 0;
  }

  function create_dummy_select_entry(box)
  {
    box.options[box.length] = new Option("---------------", "---------------");
    box.selectedIndex = box.length - 1;  // Select the latest entry.
  }

  function display_middle_window(mode)
  {
    var w = MiddleWindow;
    var body = '';
    var d = document.work_form;
    var cell_type = d.cell_type.options[d.cell_type.selectedIndex].value;
    var visible = d.show.checked;
    var real_index = d.real_index.value;
    var index = d.index.value;

    validate_show_row();  // Check to make sure the visibility setting is valid.

    // do validity checks first.
    if (cell_type == "text" || cell_type == "total" || cell_type == "compare")
    {
      if (cell_type == "total" && Rows.length == 0)
      {
        alert("You can not define a Total Row as the first Row!");
        return;
      }
      else if (cell_type == "total" && Rows.length > 0)
      {
        // check and make sure that we have at least 1 non-text / non-line row before us.
        var found = false;
        for (j=0; j < real_index; j++)
        {
          if (Rows[j].cell_type != "text" && Rows[j].cell_type != "line" && Rows[j].cell_type != "compare")
          {
            found = true;
          }
        }
        if (!found)
        {
          alert("You can not define a Total Row until you have defined a Row that contains Data!");
          return;
        }
      }
      if (cell_type == "compare" && Rows.length == 0)
      {
        alert("You can not define a Comparison Row as the first Row!");
        return;
      }
      else if (cell_type == "compare" && Rows.length > 0)
      {
        // check and make sure that we have at least 1 non-text / non-line row before us.
        var found = false;
        for (j=0; j < real_index; j++)
        {
          if (Rows[j].cell_type != "text" && Rows[j].cell_type != "line" && Rows[j].cell_type != "compare")
          {
            found = true;
          }
        }
        if (!found)
        {
          alert("You can not define a Comparison Row until you have defined a Row that contains Data!");
          return;
        }
      }
    }

    // Defaults
    var cell_text, fg_color, bg_color, initial_state, display_level, bold, drillable, drill_report, sub_levels, cell_formula, item_name, negative_format, query_type, result_type, underline;
    if (mode == "edit")
    {
      cell_text = Rows[real_index].cell_text;
      fg_color = Rows[real_index].fg_color;
      bg_color = Rows[real_index].bg_color;
      initial_state = Rows[real_index].initial_state;
      display_level = Rows[real_index].display_level;
      drill_report = Rows[real_index].drill_report;
      drillable = Rows[real_index].drillable;
      sub_levels = Rows[real_index].sub_levels;
      cell_formula = Rows[real_index].cell_formula;
      item_name = Rows[real_index].item_name;
      bold = Rows[real_index].bold;
      negative_format = Rows[real_index].negative_format;
      query_type = Rows[real_index].query_type;
      result_type = Rows[real_index].result_type;
      underline = Rows[real_index].underline;
    }
    else
    {
      cell_text = "";
      fg_color = "$fg_color";
      bg_color = "$bg_color";
      initial_state = "o";
      display_level = "0";
      bold = "false";
      negative_format = 0;
      if (cell_type == "text" || cell_type == "line" || cell_type == "total" || cell_type == "compare" || cell_type == "detail")
      {
        drillable = 1;  // the index of No
        drill_report = -1;
        sub_levels = "---------------";
        if (cell_type == "total")
        {
          underline = 1; // Default to underline the previous row.
        }
        else
        {
          underline = 0; // Don't underline the previous row.
        }
      }
      else
      {
        drillable = 0;  // the index of Yes
        drill_report = 0;  // the index of the Report to drill in on.
        sub_levels = "nothing";
        underline = 0;
      }
      item_name = "";
      cell_formula = "";
      query_type = 0;
      result_type = "";
      if (cell_type == "compare")
      {
        cell_formula = "(1 == 1 ? \\\"\\\" : \\\"\\\")";
      }
    }

    body += '<html>\\n';
    body += '<head><title>Report Middle Window</title></head>\\n';
    body += '<body bgcolor="white">\\n';
    body += '<form name="work_form">\\n';
    if (cell_type != "begin_account_type_balance" && cell_type != "end_account_type_balance" && cell_type != "begin_account_balance" && cell_type != "end_account_balance")
    {
      body += '<input type=hidden name="activity" value="'+(Date_Range ? "true" : "false")+'">\\n';
    }
    else
    {
      body += '<input type=hidden name="activity" value="false">\\n';
    }
    body += '<center>\\n';
    body += '<table border=1 cellpadding=1 cellspacing=1 align=center>\\n';
    if (cell_type == "line")
    {
      body += '  <tr bgcolor="#cccccc">\\n';
      body += '    <th align=center>There is nothing to Configure.</th>\\n';
      body += '  </tr>\\n';
    }
    else if (cell_type == "text")
    {
      body += '  <tr bgcolor="#cccccc">\\n';
      body += '    <th>Cell Text</th><th>ForeGround Color</th><th>BackGround Color</th>\\n';
      body += '  </tr>\\n';
      body += '  <tr bgcolor="#eeeeee">\\n';
      body += '    <td align=center><input type=text name="cell_text" value="'+cell_text+'" onBlur="window.opener.validate_cell_text()"></td>\\n';
      body += '    <td align=center>\\n';
      body += DisplayForeGroundColors(fg_color);
      body += '    </td>\\n';
      body += '    <td align=center>\\n';
      body += DisplayBackGroundColors(bg_color);
      body += '    </td>\\n';
      body += '  </tr>\\n';
      body += '  <tr bgcolor="#cccccc">\\n';
      body += '    <th>Bold</th><th>Display Level</th><th>Initial State</th>\\n';
      body += '  </tr>\\n';
      body += '  <tr bgcolor="#eeeeee">\\n';
      body += '    <td align=center>\\n';
      body += DisplayBold(bold);
      body += '    </td>\\n';
      body += '    <td align=center>\\n';
      body += DisplayDisplayLevel(display_level);
      body += '    </td>\\n';
      body += '    <td align=center>\\n';
      body += DisplayInitialState(initial_state);
      body += '    </td>\\n';
      body += '  </tr>\\n';
    }
    else if (cell_type == "total")
    {
      var cols=5;
      if (!visible)
      {
        cols=0;
      }
      body += '  <tr bgcolor="#cccccc">\\n';
      if (visible)
      {
        body += '    <th>Cell Text</th><th colspan='+cols+'>Cell Formula</th><th>Negative Format</th>\\n';
      }
      else
      {
        body += '    <th>Cell Formula</th>\\n';
      }
      body += '  </tr>\\n';
      body += '  <tr bgcolor="#eeeeee">\\n';
      if (visible)
      {
        body += '    <td align=center><input type=text name="cell_text" value="'+cell_text+'" onBlur="window.opener.validate_cell_text()"></td>\\n';
      }
      body += '    <td align=center colspan='+cols+'><input type=text name="cell_formula" value="'+cell_formula+'" size=40>&nbsp;<input type=button name="generate_formula_button" value="Edit" onClick="window.opener.generate_formula()">\\n';
      body += '    </td>\\n';
      if (visible)
      {
        body += '    <td align=center rowspan=3>\\n';
        body += DisplayNegativeFormats(negative_format);
        body += '    </td>\\n';
      }
      body += '  </tr>\\n';
      if (visible)
      {
        body += '  <tr bgcolor="#cccccc">\\n';
        body += '    <th>ForeGround Color</th><th>BackGround Color</th><th>Bold</th><th>Underline Prev. Row</th><th>Display Level</th><th>Initial State</th>\\n';
        body += '  </tr>\\n';
        body += '  <tr bgcolor="#eeeeee">\\n';
        body += '    <td align=center>\\n';
        body += DisplayForeGroundColors(fg_color);
        body += '    </td>\\n';
        body += '    <td align=center>\\n';
        body += DisplayBackGroundColors(bg_color);
        body += '    </td>\\n';
        body += '    <td align=center>\\n';
        body += DisplayBold(bold);
        body += '    </td>\\n';
        body += '    <td align=center>\\n';
        body += DisplayUnderlineRow(underline);
        body += '    </td>\\n';
        body += '    <td align=center>\\n';
        body += DisplayDisplayLevel(display_level);
        body += '    </td>\\n';
        body += '    <td align=center>\\n';
        body += DisplayInitialState(initial_state);
        body += '    </td>\\n';
        body += '  </tr>\\n';
      }
    }
    else if (cell_type == "compare")
    {
      var elements = null;
      if (mode == "edit")
      {
        elements = cell_formula.match(/^\\(\\s*(.+)\\s+(==|<=|>=|<|>|!=)\\s+(.+)\\s+\\?\\s*\"(.*)\"\\s*:\\s*\"(.*)\"\\s*\\)\$/);
      }
      var formula1, formula2, operator, result_true, result_false;
      if (elements == null)
      {
        if (mode == "edit")
        {
          alert("Error:  Tried to parse apart the Comparison Row formula and nothing was parsed!");
        }
        formula1 = "";
        operator = "==";
        formula2 = "";
        result_true = "";
        result_false = "";
      }
      else
      {
        formula1 = elements[1];
        operator = elements[2];
        formula2 = elements[3];
        result_true = elements[4];
        result_false = elements[5];
      }
      var cols=4;
      if (!visible)
      {
        cols=0;
      }
      body += '  <tr bgcolor="#cccccc">\\n';
      if (visible)
      {
        body += '    <th>Cell Text</th><th colspan='+cols+'>Cell Formula</th><th>Negative Format</th><th>Result Type</th>\\n';
      }
      else
      {
        body += '    <th>Cell Formula</th><th>Result Type</th>\\n';
      }
      body += '  </tr>\\n';
      body += '  <tr bgcolor="#eeeeee">\\n';
      if (visible)
      {
        body += '    <td align=center><input type=text name="cell_text" value="'+cell_text+'" onBlur="window.opener.validate_cell_text()"></td>\\n';
      }
      body += '    <td align=center colspan='+cols+'><input type=text name="cell_formula" value=\\''+cell_formula+'\\' size=40>&nbsp;<input type=button name="generate_formula_button" value="Edit" onClick="window.opener.generate_formula()">\\n';
      body += '      <input type=hidden name="compare_formula1" value="'+formula1+'">\\n';
      body += '      <input type=hidden name="compare_formula2" value="'+formula2+'">\\n';
      body += '      <input type=hidden name="compare_operator" value="'+operator+'">\\n';
      body += '      <input type=hidden name="compare_result_true" value="'+result_true+'">\\n';
      body += '      <input type=hidden name="compare_result_false" value="'+result_false+'">\\n';
      body += '    </td>\\n';
      if (visible)
      {
        body += '    <td align=center rowspan=3>\\n';
        body += DisplayNegativeFormats(negative_format);
        body += '    </td>\\n';
      }
      body += '     <td align=center>\\n';
      body += DisplayResultType(result_type);
      body += '     </td>\\n';
      body += '  </tr>\\n';
      if (visible)
      {
        body += '  <tr bgcolor="#cccccc">\\n';
        body += '    <th>ForeGround Color</th><th>BackGround Color</th><th>Bold</th><th>Underline Prev. Row</th><th>Display Level</th><th>Initial State</th>\\n';
        body += '  </tr>\\n';
        body += '  <tr bgcolor="#eeeeee">\\n';
        body += '    <td align=center>\\n';
        body += DisplayForeGroundColors(fg_color);
        body += '    </td>\\n';
        body += '    <td align=center>\\n';
        body += DisplayBackGroundColors(bg_color);
        body += '    </td>\\n';
        body += '    <td align=center>\\n';
        body += DisplayBold(bold);
        body += '    </td>\\n';
        body += '    <td align=center>\\n';
        body += DisplayUnderlineRow(underline);
        body += '    </td>\\n';
        body += '    <td align=center>\\n';
        body += DisplayDisplayLevel(display_level);
        body += '    </td>\\n';
        body += '    <td align=center>\\n';
        body += DisplayInitialState(initial_state);
        body += '    </td>\\n';
        body += '  </tr>\\n';
      }
    }
    else if (cell_type == "account_type" || cell_type == "accounts_of" || cell_type == "sub_accounts_of_type" || cell_type == "begin_account_type_balance" || cell_type == "end_account_type_balance")
    {
      body += '  <tr bgcolor="#cccccc">\\n';
      if (visible)
      {
        body += '    <th>Account Type</th><th>ForeGround Color</th><th>BackGround Color</th><th>Negative Format</th>\\n';
      }
      else
      {
        body += '    <th>Account Type</th>';
        if (cell_type != "begin_account_type_balance" && cell_type != "end_account_type_balance")
        {
          body += '<th>Query Type</th>\\n';
        }
        else
        {
          body += '\\n';
        }
      }
      body += '  </tr>\\n';
      body += '  <tr bgcolor="#eeeeee">\\n';
      body += '    <td align=center>\\n';
      body += DisplayAccountTypes(item_name, 'item_name');
      body += '    </td>\\n';
      if (visible)
      {
        body += '    <td align=center>\\n';
        body += DisplayForeGroundColors(fg_color);
        body += '    </td>\\n';
        body += '    <td align=center>\\n';
        body += DisplayBackGroundColors(bg_color);
        body += '    </td>\\n';
        body += '    <td align=center'+(cell_type == "begin_account_type_balance" || cell_type == "end_account_type_balance" ? " rowspan=3" : "")+'>\\n';
        body += DisplayNegativeFormats(negative_format);
        body += '    </td>\\n';
      }
      else
      {
        if (cell_type != "begin_account_type_balance" && cell_type != "end_account_type_balance")
        {
          body += '    <td align=center>\\n';
          body += DisplayQueryType(query_type);
          body += '    </td>\\n';
        }
      }
      body += '  </tr>\\n';
      if (visible)
      {
        body += '  <tr bgcolor="#cccccc">\\n';
        if (cell_type != "begin_account_type_balance" && cell_type != "end_account_type_balance")
        {
          body += '    <th>Bold</th><th>Display Level</th><th>Initial State</th><th>Query Type</th>\\n';
        }
        else
        {
          body += '    <th>Bold</th><th>Display Level</th><th>Initial State</th>\\n';
        }
        body += '  </tr>\\n';
        body += '  <tr bgcolor="#eeeeee">\\n';
        body += '    <td align=center>\\n';
        body += DisplayBold(bold);
        body += '    </td>\\n';
        body += '    <td align=center>\\n';
        body += DisplayDisplayLevel(display_level);
        body += '    </td>\\n';
        body += '    <td align=center>\\n';
        body += DisplayInitialState(initial_state);
        body += '    </td>\\n';
        if (cell_type != "begin_account_type_balance" && cell_type != "end_account_type_balance")
        {
          body += '    <td align=center>\\n';
          body += DisplayQueryType(query_type);
          body += '    </td>\\n';
        }
        body += '  </tr>\\n';
        body += '  <tr bgcolor="#cccccc">\\n';
        body += '    <th>Sub Levels</th><th>Drillable</th><th colspan=2>Drill Report</th>\\n';
        body += '  </tr>\\n';
        body += '  <tr bgcolor="#eeeeee">\\n';
        body += '    <td align=center>\\n';
        body += DisplaySubLevels(cell_type, sub_levels);
        body += '    </td>\\n';
        body += '    <td align=center>\\n';
        body += DisplayDrillable(drillable);
        body += '    </td>\\n';
        body += '    <td align=center colspan=2>\\n';
        body += DisplayDetailedReports();
        body += '    </td>\\n';
        body += '  </tr>\\n';
      }
    }
    else if (cell_type == "account" || cell_type == "sub_accounts_of_account" || cell_type == "begin_account_balance" || cell_type == "end_account_balance")
    {
      body += '  <tr bgcolor="#cccccc">\\n';
      if (visible)
      {
        body += '    <th>Account</th><th>ForeGround Color</th><th>BackGround Color</th><th>Negative Format</th>\\n';
      }
      else
      {
        body += '    <th>Account</th>';
        if (cell_type != "begin_account_balance" && cell_type != "end_account_balance")
        {
          body += '<th>Query Type</th>\\n';
        }
        else
        {
          body += '\\n';
        }
      }
      body += '  </tr>\\n';
      body += '  <tr bgcolor="#eeeeee">\\n';
      body += '    <td align=center>\\n';
      body += DisplayAccounts(item_name, 'item_name');
      body += '    </td>\\n';
      if (visible)
      {
        body += '    <td align=center>\\n';
        body += DisplayForeGroundColors(fg_color);
        body += '    </td>\\n';
        body += '    <td align=center>\\n';
        body += DisplayBackGroundColors(bg_color);
        body += '    </td>\\n';
        body += '    <td align=center'+(cell_type == "begin_account_balance" || cell_type == "end_account_balance" ? " rowspan=3" : "")+'>\\n';
        body += DisplayNegativeFormats(negative_format);
        body += '    </td>\\n';
      }
      else
      {
        if (cell_type != "begin_account_balance" && cell_type != "end_account_balance")
        {
          body += '    <td align=center>\\n';
          body += DisplayQueryType(query_type);
          body += '    </td>\\n';
        }
      }
      body += '  </tr>\\n';
      if (visible)
      {
        body += '  <tr bgcolor="#cccccc">\\n';
        if (cell_type != "begin_account_balance" && cell_type != "end_account_balance")
        {
          body += '    <th>Bold</th><th>Display Level</th><th>Initial State</th><th>Query Type</th>\\n';
        }
        else
        {
          body += '    <th>Bold</th><th>Display Level</th><th>Initial State</th>\\n';
        }
        body += '  </tr>\\n';
        body += '  <tr bgcolor="#eeeeee">\\n';
        body += '    <td align=center>\\n';
        body += DisplayBold(bold);
        body += '    </td>\\n';
        body += '    <td align=center>\\n';
        body += DisplayDisplayLevel(display_level);
        body += '    </td>\\n';
        body += '    <td align=center>\\n';
        body += DisplayInitialState(initial_state);
        body += '    </td>\\n';
        if (cell_type != "begin_account_balance" && cell_type != "end_account_balance")
        {
          body += '    <td align=center>\\n';
          body += DisplayQueryType(query_type);
          body += '    </td>\\n';
        }
        body += '  </tr>\\n';
        body += '  <tr bgcolor="#cccccc">\\n';
        body += '    <th>Sub Levels</th><th>Drillable</th><th colspan=2>Drill Report</th>\\n';
        body += '  </tr>\\n';
        body += '  <tr bgcolor="#eeeeee">\\n';
        body += '    <td align=center>\\n';
        body += DisplaySubLevels(cell_type, sub_levels);
        body += '    </td>\\n';
        body += '    <td align=center>\\n';
        body += DisplayDrillable(drillable);
        body += '    </td>\\n';
        body += '    <td align=center colspan=2>\\n';
        body += DisplayDetailedReports();
        body += '    </td>\\n';
        body += '  </tr>\\n';
      }
    }
    else if (cell_type == "sub_account" || cell_type == "begin_sub_account_balance" || cell_type == "sub_end_account_balance")
    {
      body += '  <tr bgcolor="#cccccc">\\n';
      if (visible)
      {
        body += '    <th>Sub Account</th><th>ForeGround Color</th><th>BackGround Color</th><th>Negative Format</th>\\n';
      }
      else
      {
        body += '    <th>Sub Account</th>';
        if (cell_type != "begin_sub_account_balance" && cell_type != "end_sub_account_balance")
        {
          body += '<th>Query Type</th>\\n';
        }
        else
        {
          body += '\\n';
        }
      }
      body += '  </tr>\\n';
      body += '  <tr bgcolor="#eeeeee">\\n';
      body += '    <td align=center>\\n';
      body += DisplaySubAccounts(item_name, 'item_name');
      body += '    </td>\\n';
      if (visible)
      {
        body += '    <td align=center>\\n';
        body += DisplayForeGroundColors(fg_color);
        body += '    </td>\\n';
        body += '    <td align=center>\\n';
        body += DisplayBackGroundColors(bg_color);
        body += '    </td>\\n';
        body += '    <td align=center'+(cell_type == "begin_sub_account_balance" || cell_type == "end_sub_account_balance" ? " rowspan=3" : "")+'>\\n';
        body += DisplayNegativeFormats(negative_format);
        body += '    </td>\\n';
      }
      else
      {
        if (cell_type != "begin_sub_account_balance" && cell_type != "end_sub_account_balance")
        {
          body += '    <td align=center>\\n';
          body += DisplayQueryType(query_type);
          body += '    </td>\\n';
        }
      }
      body += '  </tr>\\n';
      if (visible)
      {
        body += '  <tr bgcolor="#cccccc">\\n';
        if (cell_type != "begin_sub_account_balance" && cell_type != "end_sub_account_balance")
        {
          body += '    <th>Bold</th><th>Display Level</th><th>Initial State</th><th>Query Type</th>\\n';
        }
        else
        {
          body += '    <th>Bold</th><th>Display Level</th><th>Initial State</th>\\n';
        }
        body += '  </tr>\\n';
        body += '  <tr bgcolor="#eeeeee">\\n';
        body += '    <td align=center>\\n';
        body += DisplayBold(bold);
        body += '    </td>\\n';
        body += '    <td align=center>\\n';
        body += DisplayDisplayLevel(display_level);
        body += '    </td>\\n';
        body += '    <td align=center>\\n';
        body += DisplayInitialState(initial_state);
        body += '    </td>\\n';
        if (cell_type != "begin_sub_account_balance" && cell_type != "end_sub_account_balance")
        {
          body += '    <td align=center>\\n';
          body += DisplayQueryType(query_type);
          body += '    </td>\\n';
        }
        body += '  </tr>\\n';
        body += '  <tr bgcolor="#cccccc">\\n';
        body += '    <th>Sub Levels</th><th>Drillable</th><th colspan=2>Drill Report</th>\\n';
        body += '  </tr>\\n';
        body += '  <tr bgcolor="#eeeeee">\\n';
        body += '    <td align=center>\\n';
        body += DisplaySubLevels(cell_type, sub_levels);
        body += '    </td>\\n';
        body += '    <td align=center>\\n';
        body += DisplayDrillable(drillable);
        body += '    </td>\\n';
        body += '    <td align=center colspan=2>\\n';
        body += DisplayDetailedReports();
        body += '    </td>\\n';
        body += '  </tr>\\n';
      }
    }
    body += '</table>\\n';
    body += '<input type=submit name="add" value="'+(mode == "edit" ? "Update Row" : "Add Row")+'" onClick="window.opener.add_row_to_report(); return false;">&nbsp;<input type=button name="reset_row_button" value="Reset Row" onClick="window.opener.reset_row()">\\n';
    body += '</form>\\n';
    body += '</center>\\n';
    body += '</body>\\n';
    body += '</html>';

    w.document.open();
    w.document.writeln(body);
    w.document.close();

    if (cell_type == "text" || cell_type == "total")
    {
      if (visible)
      {
        w.document.work_form.cell_text.focus();
      }
      else if (cell_type == "total")
      {
        w.document.work_form.cell_formula.focus();
      }
    }
    if (cell_type == "account_type" || cell_type == "accounts_of" || cell_type == "sub_accounts_of_type" || cell_type == "begin_account_type_balance" || cell_type == "end_account_type_balance" ||
        cell_type == "account" || cell_type == "sub_accounts_of_account" || cell_type == "begin_account_balance" || cell_type == "end_account_balance" ||
        cell_type == "sub_account" || cell_type == "begin_sub_account_balance" || cell_type == "end_sub_account_balance")
    {
      if (visible)
      {
        process_drillable((drillable ? "false" : "true"));  // update the Drillable and Detailed Reports entries.
      }
    }
  }

  function validate_cell_text()
  {
    var d = document.work_form;
    var w = MiddleWindow;
    var d2 = w.document.work_form;

    var index = d.cell_type.selectedIndex;
    if (index == -1)
    {
      alert("You can not enter a value for Row Text without having a Row Type selected!");
      return;
    }
    var value = d.cell_type.options[index].value;
    if (d2.cell_text.value.length == 0)
    {
      return;
    }
  }

  function process_drillable(status)
  {
    var w = TopWindow;
    var d = w.document.work_form;
    var index = d.cell_type.selectedIndex;
    w = MiddleWindow;
    var d2 = w.document.work_form;

    if (index == -1)
    {
      alert("You must have a Row Type selected before you can specify if this row is drillable!");
      return;
    }

    var value = d.cell_type.options[index].value;

    clear_select_box(d2.drill_report);
    if (status)
    {
      if (EditMode == "edit" && Clicked == false)
      {
        if (Rows[d.real_index.value].drillable == 0) // It is drillable
        {
          var drill_index = 0;
          for (var i=0; i < DetailedReportsArray.length; i++)
          {
            d2.drill_report.options[d2.drill_report.length] = new Option(DetailedReportsArray[i].name, DetailedReportsArray[i].id);
            if (DetailedReportsArray[i].id == Rows[d.real_index.value].drill_report)
            {
              drill_index = i;
            }
          }
          d2.drill_report.selectedIndex = drill_index;
          d2.drillable[0].checked = true;
        }
        else
        {
          create_dummy_select_entry(d2.drill_report);
          d2.drillable[1].checked = true;
        }
      }
      else
      {
        for (i=0; i < DetailedReportsArray.length; i++)
        {
          d2.drill_report.options[d2.drill_report.length] = new Option(DetailedReportsArray[i].name, DetailedReportsArray[i].id);
        }
        d2.drill_report.selectedIndex = 0;
        d2.drillable[0].checked = true;
      }
    }
    else
    {
      create_dummy_select_entry(d2.drill_report);
      d2.drillable[1].checked = true;
    }
  }

  function generate_formula()
  {
    var w = MiddleWindow;
    var d = w.document.work_form;
    var d2 = document.work_form;
    var index = d2.cell_type.selectedIndex;

    if (index == -1)
    {
      alert("You must have a Row Type selected before you can specify a Formula!");
      return;
    }

    var value = d2.cell_type.options[index].value;

    if (value != "total" && value != "compare")
    {
      alert("You can not specify a Formula for Row Type = '"+d2.cell_type.options[index].text+"'!");
      return;
    }

    if (value == "total")
    {
      if (d.cell_formula.value.length > 0 && validate_formula(d.cell_formula.value, d2.index.value));

      // create the formula editor window.
      var w2 = window.open("", "formula_editor", "height=300,width=625,resizable,scrollbars", true);
      FormulaWindow = w2;

      var body = '<html>\\n';
      body += '  <head>\\n';
      body += '    <title>Formula Editor</title>\\n';
      body += '  </head>\\n';
      body += '  <body bgcolor="white">\\n';
      body += '    <form name="formula_form" method=post onSubmit="return false;">\\n';
      body += '    <table border=0 cellpadding=2 cellspacing=0 width=100% align=center>\\n';
      body += '      <tr>\\n';
      body += '        <th colspan=2 align=center>Edit Formula</th>\\n';
      body += '      </tr>\\n';
      body += '      <tr>\\n';
      body += '        <td align=left valign=top>\\n';
      body += '          <select name="operators">\\n';
      body += '            <option>+\\n';
      body += '            <option>-\\n';
      body += '            <option>*\\n';
      body += '            <option>/\\n';
      body += '            <option>%\\n';
      body += '            <option>(\\n';
      body += '            <option>)\\n';
      body += '          </select>&nbsp;<input type=button name="add_operator" value="Add Operator" onClick="window.opener.formula_add_operator();">\\n';
      body += '        </td>\\n';
      body += '        <td align=right valign=top>\\n';
      body += '          <select name="rows">\\n';
      for (var i=0; i < d2.index.value; i++)
      {
        if (Rows[i].cell_type != "text" && Rows[i].cell_type != "line")  // only indicate this row available if not a text or line row.
        {
          if (Rows[i].cell_type != "compare" || (Rows[i].cell_type == "compare" && Rows[i].result_type == "result_formula"))
          {
            body += '            <option>R'+i+'\\n';
          }
        }
      }
      body += '          </select>&nbsp;<input type=button name="add_row" value="Add Row" onClick="window.opener.formula_add_row();">\\n';
      body += '        </td>\\n';
      body += '      </tr>\\n';
      body += '      <tr>\\n';
      body += '        <td align=left><input type=button name="validate" value="Validate" onClick="window.opener.formula_validate_formula(document.formula_form.formula.value, '+d2.index.value+');"></td>\\n';
      body += '        <td align=center><b>Formula</b><br><textarea name=formula rows=5 cols=40 wrap=virtual>'+d.cell_formula.value+'</textarea></td>\\n';
      body += '      </tr>\\n';
      body += '      <tr>\\n';
      body += '        <td align=center><input type=button name="done" value="Done" onClick="window.opener.formula_done('+d2.index.value+');"></td>\\n';
      body += '        <td align=center><input type=button name="clear" value="Clear" onClick="window.opener.formula_clear();"></td>\\n';
      body += '      </tr>\\n';
      body += '    </table>\\n';
      body += '    </form>\\n';
      body += '  </body>\\n';
      body += '</html>';

      w2.document.open();
      w2.document.writeln(body);
      w2.document.close();
    }
    else if (value == "compare")
    {
      // build up the formula string from the hidden fields.
      //var formula = "(" + d.compare_formula1.value + " " + d.compare_operator.value + " " + d.compare_formula2.value + " ? \\\"" + d.compare_result_true.value + "\\\" : \\\"" + d.compare_result_false.value + "\\\")";
      var formula = d.cell_formula.value;

      // parse it apart.
      var elements = parse_comparison_formula(formula);

      if (elements == null)
      {
        alert("Error:  Tried to parse apart the Comparison Row formula and nothing was parsed!");
        return false;
      }
      else
      {
        if (elements.length != 6)
        {
          alert("Error:  The Comparison Row Formula did not have the correct number of items!");
          return false;
        }
        d.compare_formula1.value = elements[1];
        d.compare_operator.value = elements[2];
        d.compare_formula2.value = elements[3];
        d.compare_result_true.value = elements[4];
        d.compare_result_false.value = elements[5];
      }

      if (d.compare_formula1.value.length > 0 && d.compare_formula2.value.length > 0 && d.compare_operator.value.length > 0 && (d.compare_result_true.value.length > 0 || d.compare_result_false.value.length > 0))
      {
        validate_comparison_formula(formula, d2.index.value);
      }
      d.cell_formula.value = formula;

      // alert("The Comparison Formula is currently:\\n"+formula);

      // create the formula editor window.
      var windowName = window.parent.name + '_compare_formula_editor';
      var w2 = window.open("", windowName, "height=200,width=1024,resizable,scrollbars", true);
      CompareFormulaWindow = w2;

      var body = '<html>\\n';
      body += '  <head>\\n';
      body += '    <title>Comparison Formula Editor</title>\\n';
      body += '  </head>\\n';
      body += '  <body bgcolor="white">\\n';
      body += '    <form name="formula_form" method=post onSubmit="return false;">\\n';
      body += '    <table border=0 cellpadding=2 cellspacing=0 width=100% align=center>\\n';
      body += '      <tr>\\n';
      body += '        <th colspan=2 align=center>Edit Comparison Formula</th>\\n';
      body += '      </tr>\\n';
      body += '      <tr>\\n';
      body += '        <th>&nbsp;</th><th align=center>Left Formula</th><th align=center>Operator</th><th align=center>Right Formula</th><th>&nbsp;</th><th align=center>True Result</th><th>&nbsp;</th><th align=center>False Result</th><th>&nbsp;</th>\\n';
      body += '      </tr>\\n';
      body += '      <tr>\\n';
      body += '        <td><b>(</b></td>\\n';
      body += '        <td align=center><input type=text name="formula1" value="'+d.compare_formula1.value+'"><input type=button name="gen_form1" value="Edit" onClick="window.opener.edit_formula(document.formula_form.formula1.value, '+d2.index.value+', \\'formula1\\');"></td>\\n';
      body += '        <td align=center><select name="operators">\\n';
      body += '            <option value="=="'+(d.compare_operator.value == "==" ? " selected" : "")+'>==\\n';
      body += '            <option value="<="'+(d.compare_operator.value == "<=" ? " selected" : "")+'>&lt;=\\n';
      body += '            <option value=">="'+(d.compare_operator.value == ">=" ? " selected" : "")+'>&gt;=\\n';
      body += '            <option value="<"'+(d.compare_operator.value == "<" ? " selected" : "")+'>&lt;\\n';
      body += '            <option value=">"'+(d.compare_operator.value == ">" ? " selected" : "")+'>&gt;\\n';
      body += '            <option value="!="'+(d.compare_operator.value == "!=" ? " selected" : "")+'>!=\\n';
      body += '          </select></td>\\n';
      body += '        <td align=center><input type=text name="formula2" value="'+d.compare_formula2.value+'"><input type=button name="gen_form2" value="Edit" onClick="window.opener.edit_formula(document.formula_form.formula2.value, '+d2.index.value+', \\'formula2\\');"></td>\\n';
      body += '        <td align=center><b>?</b></td>\\n';
      body += '        <td align=center><input type=text name="result_true" value="'+d.compare_result_true.value+'">';
      if (d.result_type.checked)
      {
        body += '<input type=button name="gen_form3" value="Edit" onClick="window.opener.edit_formula(document.formula_form.result_true.value, '+d2.index.value+', \\'result_true\\');">';
      }
      body += '</td>\\n';
      body += '        <td align=center><b>:</b></td>\\n';
      body += '        <td align=center><input type=text name="result_false" value="'+d.compare_result_false.value+'">';
      if (d.result_type.checked)
      {
        body += '<input type=button name="gen_form4" value="Edit" onClick="window.opener.edit_formula(document.formula_form.result_false.value, '+d2.index.value+', \\'result_false\\');">';
      }
      body += '</td>\\n';
      body += '        <td align=center><b>)</b></td>\\n';
      body += '      </tr>\\n';
      body += '      <tr>\\n';
      body += '        <td colspan=9 align=left><input type=button name="validate" value="Validate" onClick="window.opener.comparison_validate_formula('+d2.index.value+');">&nbsp;&nbsp;&nbsp;<input type=button name="done" value="Done" onClick="window.opener.comparison_done('+d2.index.value+');">&nbsp;<input type=button name="clear" value="Clear" onClick="window.opener.comparison_clear();"></td>\\n';
      body += '      </tr>\\n';
      body += '    </table>\\n';
      body += '    </form>\\n';
      body += '  </body>\\n';
      body += '</html>';

      w2.document.open();
      w2.document.writeln(body);
      w2.document.close();
    }
  }

  function edit_formula(formula, row, item)
  {
    if (formula.length > 0 && validate_formula(formula, row));

    // create the formula editor window.
    var windowName = window.parent.name + '_formula_editor';
    var w = window.open("", windowName, "height=300,width=625,resizable,scrollbars", true);
    FormulaWindow = w;

    var body = '<html>\\n';
    body += '  <head>\\n';
    body += '    <title>Formula Editor</title>\\n';
    body += '  </head>\\n';
    body += '  <body bgcolor="white">\\n';
    body += '    <form name="formula_form" method=post onSubmit="return false;">\\n';
    body += '    <table border=0 cellpadding=2 cellspacing=0 width=100% align=center>\\n';
    body += '      <tr>\\n';
    body += '        <th colspan=2 align=center>Edit Formula</th>\\n';
    body += '      </tr>\\n';
    body += '      <tr>\\n';
    body += '        <td align=left valign=top>\\n';
    body += '          <select name="operators">\\n';
    body += '            <option>+\\n';
    body += '            <option>-\\n';
    body += '            <option>*\\n';
    body += '            <option>/\\n';
    body += '            <option>%\\n';
    body += '            <option>(\\n';
    body += '            <option>)\\n';
    body += '          </select>&nbsp;<input type=button name="add_operator" value="Add Operator" onClick="window.opener.formula_add_operator();">\\n';
    body += '        </td>\\n';
    body += '        <td align=right valign=top>\\n';
    body += '          <select name="rows">\\n';
    for (var i=0; i < row; i++)
    {
      if (Rows[i].cell_type != "text" && Rows[i].cell_type != "line")  // only indicate this row available if not a text or line row.
      {
        if (Rows[i].cell_type != "compare" || (Rows[i].cell_type == "compare" && Rows[i].result_type == "result_formula"))
        {
          body += '            <option>R'+i+'\\n';
        }
      }
    }
    body += '          </select>&nbsp;<input type=button name="add_row" value="Add Row" onClick="window.opener.formula_add_row();">\\n';
    body += '        </td>\\n';
    body += '      </tr>\\n';
    body += '      <tr>\\n';
    body += '        <td align=left><input type=button name="validate" value="Validate" onClick="window.opener.formula_validate_formula(document.formula_form.formula.value, '+row+');"></td>\\n';
    body += '        <td align=center><b>Formula</b><br><textarea name=formula rows=5 cols=40 wrap=virtual>'+formula+'</textarea></td>\\n';
    body += '      </tr>\\n';
    body += '      <tr>\\n';
    body += '        <td align=center><input type=button name="done" value="Done" onClick="window.opener.sub_formula_done('+row+', \\''+item+'\\');"></td>\\n';
    body += '        <td align=center><input type=button name="clear" value="Clear" onClick="window.opener.formula_clear();"></td>\\n';
    body += '      </tr>\\n';
    body += '    </table>\\n';
    body += '    </form>\\n';
    body += '  </body>\\n';
    body += '</html>';

    w.document.open();
    w.document.writeln(body);
    w.document.close();
  }

  function formula_add_operator()
  {
    var w = FormulaWindow;
    var d = w.document.formula_form;

    if (d.operators.selectedIndex > -1)
    {
      if (d.formula.value.length > 0)
      {
        d.formula.value += " ";
      }
      d.formula.value += d.operators.options[d.operators.selectedIndex].text;
    }
    else
    {
      alert("You must select an operator first!");
    }
  }

  function formula_add_row()
  {
    var w = FormulaWindow;
    var d = w.document.formula_form;

    if (d.rows.selectedIndex > -1)
    {
      if (d.formula.value.length > 0)
      {
        d.formula.value += " ";
      }
      d.formula.value += d.rows.options[d.rows.selectedIndex].text;
    }
    else
    {
      alert("You must select a row first!");
    }
  }

  function formula_validate_formula(row)
  {
    var w = FormulaWindow;
    var d = w.document.formula_form;

    if (validate_formula(d.formula.value, row))
    {
      alert("Formula is OK!");
    }
  }

  function formula_done(row)
  {
    var w = FormulaWindow;
    var d = w.document.formula_form;
    var w2 = MiddleWindow;
    var d2 = w2.document.work_form;

    if (validate_formula(d.formula.value, row))
    {
      d2.cell_formula.value = d.formula.value;
      w.close();
    }
  }

  function sub_formula_done(row, formula)
  {
    var w = FormulaWindow;
    var d = w.document.formula_form;
    var w2 = CompareFormulaWindow;

    if (validate_formula(d.formula.value, row))
    {
      var text = eval("w2.document.formula_form."+formula+".value = d.formula.value;");
      w.close();
    }
  }

  function formula_clear()
  {
    var w = FormulaWindow;
    var d = w.document.formula_form;

    d.formula.value = "";
  }

  function comparison_validate_formula(row)
  {
    var w = CompareFormulaWindow;
    var d = w.document.formula_form;
    var w2 = MiddleWindow;
    var d2 = w2.document.work_form;
    var error = false;
    var message = "";

    if (d.formula1.value.length == 0)
    {
      message = "You must specify the Left Formula!\\n";
      error = true;
    }
    if (d.operators.selectedIndex == -1)
    {
      message += "You must select an Operator!\\n";
      error = true;
    }
    if (d.formula2.value.length == 0)
    {
      message += "You must specify the Right Formula!\\n";
      error = true;
    }
    if (d.result_true.value.length == 0 && d.result_false.value.length == 0 && !d2.result_type.checked)
    {
      message += "You must specify one of the True or False Results!\\n";
      error = true;
    }
    else if (d2.result_type.checked && (d.result_true.value.length == 0 || d.result_false.value.length == 0))
    {
      message += "You must specify both the True and False Result Formulas!\\n";
      error = true;
    }
    if (error)
    {
      alert(message);
      return;
    }
    var formula = "( " + d.formula1.value + " " + d.operators.options[d.operators.selectedIndex].value + " " + d.formula2.value + " ? \\\"" + d.result_true.value + "\\\" : \\\"" + d.result_false.value + "\\\" )";
    if (validate_comparison_formula(formula, row, (d2.result_type.checked ? "result_formula" : "")))
    {
      alert("Comparison Formula OK!");
    }
  }

  function comparison_done(row)
  {
    var w = CompareFormulaWindow;
    var d = w.document.formula_form;
    var w2 = MiddleWindow;
    var d2 = w2.document.work_form;
    var error = false;
    var message = "";

    if (d.formula1.value.length == 0)
    {
      message = "You must specify the Left Formula!\\n";
      error = true;
    }
    if (d.operators.selectedIndex == -1)
    {
      message += "You must select an Operator!\\n";
      error = true;
    }
    if (d.formula2.value.length == 0)
    {
      message += "You must specify the Right Formula!\\n";
      error = true;
    }
    if (d.result_true.value.length == 0 && d.result_false.value.length == 0 && !d2.result_type.checked)
    {
      message += "You must specify one of the True or False Results!\\n";
      error = true;
    }
    else if (d2.result_type.checked && (d.result_true.value.length == 0 || d.result_false.value.length == 0))
    {
      message += "You must specify both the True and False Result Formulas!\\n";
      error = true;
    }
    if (error)
    {
      alert(message);
      return;
    }
    var formula = "( " + d.formula1.value + " " + d.operators.options[d.operators.selectedIndex].value + " " + d.formula2.value + " ? \\\"" + d.result_true.value + "\\\" : \\\"" + d.result_false.value + "\\\" )";
    if (validate_comparison_formula(formula, row, (d2.result_type.checked ? "result_formula" : "")))
    {
      d2.compare_formula1.value = d.formula1.value;
      d2.compare_operator.value = d.operators.options[d.operators.selectedIndex].value;
      d2.compare_formula2.value = d.formula2.value;
      d2.compare_result_true.value = d.result_true.value;
      d2.compare_result_false.value = d.result_false.value;
      d2.cell_formula.value = formula;

      w.close();  // close the window.
    }
  }

  function comparison_clear()
  {
    var w = CompareFormulaWindow;
    var d = w.document.formula_form;

    d.formula1.value = "";
    d.formula2.value = "";
    d.result_true.value = "";
    d.result_false.value = "";
    d.operators.selectedIndex = 0; // select the == operator.
  }

  function parse_comparison_formula(formula)
  {
    var elements = formula.match(/^\\(\\s*(.+)\\s+(==|<=|>=|<|>|!=)\\s+(.+)\\s+\\?\\s*\"(.*)\"\\s*:\\s*\"(.*)\"\\s*\\)\$/);

    return elements;
  }

  function validate_comparison_formula(formula, row, result_type)
  {
    var formula1, formula2, operator, result_true, result_false, check_results;

    check_results = (result_type == "result_formula" ? true : false);

    if (formula.length == 0)
    {
      alert("The Comparison Formula can not be empty!");
      return false;
    }
    var elements = parse_comparison_formula(formula);
    if (elements == null)
    {
      alert("Error:  Tried to parse apart the Comparison Row formula and nothing was parsed!");
      return false;
    }
    else
    {
      if (elements.length != 6)
      {
        alert("Error:  The Comparison Row Formula did not have the correct number of items!");
        return false;
      }
      formula1 = elements[1];
      operator = elements[2];
      formula2 = elements[3];
      result_true = elements[4];
      result_false = elements[5];

      // now validate the items returned.
      if (!validate_formula(formula1, row))
      {
        alert("The Left Formula is invalid!");
        return false;
      }
      if (operator.length == 0)
      {
        alert("An operator was not specified!");
        return false;
      }
      if (!validate_formula(formula2, row))
      {
        alert("The Right Formula is invalid!");
        return false;
      }
      if (result_true.length == 0 && result_false.length == 0)
      {
        alert("You can not specify both the True and False results to be empty!");
        return false;
      }
      if (check_results)
      {
        if (result_true.length == 0 && result_false.length == 0)
        {
          alert("You have to specify both the True and False Result Formulas!");
          return false;
        }
        if (!validate_formula(result_true, row))
        {
          alert("The True Result Formula is invalid!");
          return false;
        }
        if (!validate_formula(result_false, row))
        {
          alert("The False Result Formula is invalid!");
          return false;
        }
      }
    }
    return true;
  }

  function validate_formula(formula, row)
  {
    //alert("Entering Validate_Formula(formula='"+formula+"',row='"+row+"')");
    var result = parse_formula(formula);
    if (result[0].length == 0)
    {
      // now check and make sure that the formula does not reference the current row or any future rows.
      for (var i=0; i < result[1].length; i++)
      {
        var temp = result[1][i].match(/^R(\\d+)\$/);
        if (temp != null && temp[0] == result[1][i])
        {
          // temp[1] holds the number!
          if (parseInt(temp[1], 10) >= row)
          {
            //alert("Leaving Validate_Formula(formula='"+formula+"',row='"+row+"')");
            alert("You can not reference the current Row or a Future Row!\\nToken = '"+temp[0]+"'");
            return false;
          }
          if (Rows[temp[1]].cell_type == "text" || Rows[temp[1]].cell_type == "line" || (Rows[temp[1]].cell_type == "compare" && Rows[temp[1]].result_type != "result_formula"))
          {
            //alert("Leaving Validate_Formula(formula='"+formula+"',row='"+row+"')");
            alert("You can not reference a Text, Comparison, or Line Row!\\nInvalid Row = 'R"+temp[1]+"'");
            return false;
          }
        }
      }
    }
    else
    {
      //alert("Leaving Validate_Formula(formula='"+formula+"',row='"+row+"')");
      alert("Formula = '"+formula+"' is not OK!\\nError is '"+result[0]+"'\\nTokens are:\\n"+result[1]);
      return false;
    }
    //alert("Leaving Validate_Formula(formula='"+formula+"',row='"+row+"')");
    return true;
  }

  function parse_formula(formula)
  {
    var result = new Array();

    if (formula.length == 0)
    {
      result[0] = "Error:  formula must be specified!";
      result[1] = new Array();
      return result;
    }

    var elements = formula.match(/(\\(|R\\d+\\.\d+|R\\d+|-?\\d+\\.\\d+|-?\\d+|\\+|-|%|\\*|\\/|\\)|\\s+|.*)/g);
    var i;

    // get rid of any empty or whitespace related elements.
    var nodesToDelete = new Array();
    for (i=0; i < elements.length; i++)
    {
      var temp;
      temp = elements[i].match(/^(\\s+|)\$/);
      if (temp != null && temp[0] == elements[i])
      {
        nodesToDelete.push(i);
      }
    }
    for (i=0; i < nodesToDelete.length; i++)
    {
      elements.splice(nodesToDelete[i]-i, 1);
    }

    if (elements.length == 0)
    {
      result[0] = "Error:  The formula didn't split out any tokens!\\n";
      result[1] = elements;
      return result;
    }

    var open_parens = 0;
    for (i=0; i < elements.length; i++)
    {
      if (i == 0) // first element only.
      { // make sure we are not starting out with an operator
        var temp = elements[i].match(/^(\\+|-|%|\\*|\\/)\$/);
        //if (temp == null)
        //{
        //  alert("check 1\\ni="+i+"\\ntemp = null");
        //}
        //else
        //{
        //  alert("check 1\\ni="+i+"\\ntemp[0]='"+temp[0]+"'\\nelements[i]='"+elements[i]+"'");
        //}
        if (temp != null && temp[0] == elements[i])
        {
          result[0] = "Error:  Formula can not start with an operator!\\n";
          result[1] = elements;
          return result;
        }
        else
        {
          var temp = elements[i].match(/^\\(\$/);
          //if (temp == null)
          //{
          //  alert("check 2\\ni="+i+"\\ntemp = null");
          //}
          //else
          //{
          //  alert("check 2\\ni="+i+"\\ntemp[0]='"+temp[0]+"'\\nelements[i]='"+elements[i]+"'");
          //}
          if (temp != null && temp[0] == elements[i])
          {
            open_parens++;
          }
          else
          {
            var temp = elements[i].match(/^\\)\$/);
            //if (temp == null)
            //{
            //  alert("check 3\\ni="+i+"\\ntemp = null");
            //}
            //else
            //{
            //  alert("check 3\\ni="+i+"\\ntemp[0]='"+temp[0]+"'\\nelements[i]='"+elements[i]+"'");
            //}
            if (temp != null && temp[0] == elements[i])
            {
              result[0] = "Error:  You can not start the Formula with an )!\\n";
              result[1] = elements;
              return result;
            }
            else
            {
              var temp = elements[i].match(/^R\\d+|-?\\d+\\.\\d+|-?\\d+\$/);
              //if (temp == null)
              //{
              //  alert("check 4\\ni="+i+"\\ntemp = null");
              //}
              //else
              //{
              //  alert("check 4\\ni="+i+"\\ntemp[0]='"+temp[0]+"'\\nelements[i]='"+elements[i]+"'");
              //}
              if ((temp == null) || (temp != null && temp[0] != elements[i]))
              {
                result[0] = "Error:  Invalid token!  Token " + i + ", '" + elements[i] + "'.\\n";
                result[1] = elements;
                return result;
              }
            }
          }
        }
      }
      else  // the rest of the elements.
      { // make sure that the last token was a row, number, or ) if we have an operator
        // else if we have a row make sure the last token was an (
        // and not another row.
        var temp = elements[i].match(/^(\\+|-|%|\\*|\\/)\$/);
        //if (temp == null)
        //{
        //  alert("check 1\\ni="+i+"\\ntemp = null");
        //}
        //else
        //{
        //  alert("check 1\\ni="+i+"\\ntemp[0]='"+temp[0]+"'\\nelements[i]='"+elements[i]+"'");
        //}
        if (temp != null && temp[0] == elements[i])
        {
          var temp1 = elements[i-1].match(/^R\\d+|-?\\d+\\.\\d+|-?\\d+\$/);
          var temp2 = elements[i-1].match(/^\\)\$/);
          //if (temp1 == null)
          //{
          //  alert("check 2\\ni="+i+"\\ntemp1 = null");
          //}
          //else
          //{
          //  alert("check 2\\ni="+i+"\\ntemp1[0]='"+temp1[0]+"'\\nelements[i-1]='"+elements[i-1]+"'");
          //}
          //if (temp2 == null)
          //{
          //  alert("check 2\\ni="+i+"\\ntemp2 = null");
          //}
          //else
          //{
          //  alert("check 2\\ni="+i+"\\ntemp2[0]='"+temp2[0]+"'\\nelements[i-1]='"+elements[i-1]+"'");
          //}
          if (((temp1 != null && temp1[0] != elements[i-1]) || temp1 == null) && ((temp2 != null && temp2[0] != elements[i-1]) || temp2 == null))
          {
            result[0] = "Error:  Operator can not come after another Operator or (!  Token " + i + ", '" + elements[i-1] + "' and '" + elements[i] + "'.\\n";
            result[1] = elements;
            return result;
          }
        }
        else
        {
          var temp = elements[i].match(/^\\(\$/);
          //if (temp == null)
          //{
          //  alert("check 3\\ni="+i+"\\ntemp = null");
          //}
          //else
          //{
          //  alert("check 3\\ni="+i+"\\ntemp[0]='"+temp[0]+"'\\nelements[i]='"+elements[i]+"'");
          //}
          if (temp != null && temp[0] == elements[i])
          {
            open_parens++;
            // make sure that the previous token was an ( or an operator.
            var temp1 = elements[i-1].match(/^\\(\$/);
            var temp2 = elements[i-1].match(/^(\\+|-|\\*|\\/)\$/);
            //if (temp1 == null)
            //{
            //  alert("check 4\\ni="+i+"\\ntemp1 = null");
            //}
            //else
            //{
            //  alert("check 4\\ni="+i+"\\ntemp1[0]='"+temp1[0]+"'\\nelements[i-1]='"+elements[i-1]+"'");
            //}
            //if (temp2 == null)
            //{
            //  alert("check 4\\ni="+i+"\\ntemp2 = null");
            //}
            //else
            //{
            //  alert("check 4\\ni="+i+"\\ntemp2[0]='"+temp2[0]+"'\\nelements[i-1]='"+elements[i-1]+"'");
            //}
            if (((temp1 != null && temp1[0] != elements[i-1]) || temp1 == null) && ((temp2 != null && temp2[0] != elements[i-1]) || temp2 == null))
            {
              result[0] = "Error:  ( can not come after a Row, Number, or )!  Token " + i + ", '" + elements[i-1] + "' and '" + elements[i] + "'.\\n";
              result[1] = elements;
              return result;
            }
          }
          else
          {
            var temp = elements[i].match(/^\\)\$/);
            //if (temp == null)
            //{
            //  alert("check 5\\ni="+i+"\\ntemp = null");
            //}
            //else
            //{
            //  alert("check 5\\ni="+i+"\\ntemp[0]='"+temp[0]+"'\\nelements[i]='"+elements[i]+"'");
            //}
            if (temp != null && temp[0] == elements[i])
            {
              if (open_parens > 0)
              {
                open_parens--;
              }
              else
              {
                result[0] = "Error:  Formula is invalid at token " + i + ", '" + elements[i] + "'.  Close Paren when no Open Paren to match with!\\n";
                result[1] = elements;
                return result;
              }

              // make sure that we are not following a ( or an operator.
              var temp1 = elements[i-1].match(/^\\(\$/);
              var temp2 = elements[i-1].match(/^(\\+|-|%|\\*|\\/)\$/);
              //if (temp1 == null)
              //{
              //  alert("check 6\\ni="+i+"\\ntemp1 = null");
              //}
              //else
              //{
              //  alert("check 6\\ni="+i+"\\ntemp1[0]='"+temp1[0]+"'\\nelements[i-1]='"+elements[i-1]+"'");
              //}
              //if (temp2 == null)
              //{
              //  alert("check 6\\ni="+i+"\\ntemp2 = null");
              //}
              //else
              //{
              //  alert("check 6\\ni="+i+"\\ntemp2[0]='"+temp2[0]+"'\\nelements[i-1]='"+elements[i-1]+"'");
              //}
              if ((temp1 != null && temp1[0] == elements[i-1]) || (temp2 != null && temp2[0] == elements[i-1]))
              {
                result[0] = "Error:  ) can not come after a ( or operator!  Token " + i + ", '" + elements[i-1] + "' and '" + elements[i] + "'.\\n";
                result[1] = elements;
                return result;
              }
            }
            else
            {
              var temp1 = elements[i].match(/^R\\d+\$/);
              var temp2 = elements[i-1].match(/^-?\\d+\\.\\d+|-?\\d+\$/);
              //if (temp1 == null)
              //{
              //  alert("check 7\\ni="+i+"\\ntemp1 = null");
              //}
              //else
              //{
              //  alert("check 7\\ni="+i+"\\ntemp1[0]='"+temp1[0]+"'\\nelements[i]='"+elements[i]+"'");
              //}
              //if (temp2 == null)
              //{
              //  alert("check 7\\ni="+i+"\\ntemp2 = null");
              //}
              //else
              //{
              //  alert("check 7\\ni="+i+"\\ntemp2[0]='"+temp2[0]+"'\\nelements[i-1]='"+elements[i-1]+"'");
              //}
              if ((temp1 != null && temp1[0] == elements[i]) && (temp2 != null && temp2[0] == elements[i-1]))
              {
                result[0] = "Error:  You can not have a Number and a Row back to back without an intervening operator! Token " + i + ", '" + elements[i-1] + "' and '" + elements[i] + "'.\\n";
                result[1] = elements;
                return result;
              }
              else
              {
                var temp1 = elements[i].match(/^R\\d+|-?\\d+\\.\\d+|-?\\d+\$/);
                var temp2 = elements[i-1].match(/^R\\d+\$/);
                //if (temp1 == null)
                //{
                //  alert("check 8\\ni="+i+"\\ntemp1 = null");
                //}
                //else
                //{
                //  alert("check 8\\ni="+i+"\\ntemp1[0]='"+temp1[0]+"'\\nelements[i]='"+elements[i]+"'");
                //}
                //if (temp2 == null)
                //{
                //  alert("check 8\\ni="+i+"\\ntemp2 = null");
                //}
                //else
                //{
                //  alert("check 8\\ni="+i+"\\ntemp2[0]='"+temp2[0]+"'\\nelements[i-1]='"+elements[i-1]+"'");
                //}
                if ((temp1 != null && temp1[0] == elements[i]) && ((temp2 != null && temp2[0] != elements[i-1]) || temp2 == null))
                {
                  var temp = elements[i].match(/^R\\d+\\.\\d+\$/);
                  if (temp != null && temp[0] == elements[i])
                  {
                    result[0] = "Error:  Invalid token!  Token " + i + ", '" + elements[i] + "'.\\n";
                    result[1] = elements;
                    return result;
                  }
                }
                else
                {
                  var temp1 = elements[i].match(/^R\\d+\$/);
                  var temp2 = elements[i-1].match(/^R\\d+\$/);
                  //if (temp1 == null)
                  //{
                  //  alert("check 9\\ni="+i+"\\ntemp1 = null");
                  //}
                  //else
                  //{
                  //  alert("check 9\\ni="+i+"\\ntemp1[0]='"+temp1[0]+"'\\nelements[i]='"+elements[i]+"'");
                  //}
                  //if (temp2 == null)
                  //{
                  //  alert("check 9\\ni="+i+"\\ntemp2 = null");
                  //}
                  //else
                  //{
                  //  alert("check 9\\ni="+i+"\\ntemp2[0]='"+temp2[0]+"'\\nelements[i-1]='"+elements[i-1]+"'");
                  //}
                  if ((temp1 != null && temp1[0] == elements[i]) && (temp2 != null && temp2[0] == elements[i-1]))
                  {
                    result[0] = "Error:  You can not have 2 Rows back to back without an intervening operator! Token " + i + ", '" + elements[i-1] + "' and '" + elements[i] + "'.\\n";
                    result[1] = elements;
                    return result;
                  }
                  else
                  {
                    var temp1 = elements[i].match(/^-\\d+\\.\\d+|-\\d+\$/);
                    var temp2 = elements[i-1].match(/^R\\d+|-?\\d+\\.\\d+|-?\\d+\$/);
                    //if (temp1 == null)
                    //{
                    //  alert("check 9\\ni="+i+"\\ntemp1 = null");
                    //}
                    //else
                    //{
                    //  alert("check 9\\ni="+i+"\\ntemp1[0]='"+temp1[0]+"'\\nelements[i]='"+elements[i]+"'");
                    //}
                    //if (temp2 == null)
                    //{
                    //  alert("check 9\\ni="+i+"\\ntemp2 = null");
                    //}
                    //else
                    //{
                    //  alert("check 9\\ni="+i+"\\ntemp2[0]='"+temp2[0]+"'\\nelements[i-1]='"+elements[i-1]+"'");
                    //}
                    if ((temp1 != null && temp1[0] == elements[i]) && (temp2 != null && temp2[0] == elements[i-1]))
                    {
                      result[0] = "Error:  You can not have 2 Numbers or a Row and a Number back to back without an intervening operator! Token " + i + ", '" + elements[i-1] + "' and '" + elements[i] + "'.\\n";
                      result[1] = elements;
                      return result;
                    }
                    else
                    {
                      result[0] = "Error:  Invalid token!  Token " + i + ", '" + elements[i] + "'.\\n";
                      result[1] = elements;
                      return result;
                    }
                  }
                }
              }
            }
          }
        }
      }
    }

    // make sure the last token wasn't an operator.
    var temp = elements[i-1].match(/^(\\+|-|%|\\*|\\/)\$/);
    if (temp != null && temp[0] == elements[i-1])
    {
      result[0] = "Error:  The Formula can not end with an operator!  Token " + i-1 + ", '" + elements[i-1] + "'.\\n";
      result[1] = elements;
      return result;
    }

    // make sure we have all ()'s matched up.
    if (open_parens > 0)
    {
      result[0] = "Error:  You have unmatched ('s!\\n";
      result[1] = elements;
      return result;
    }

    result[0] = "";
    result[1] = elements;

    return result;  // signal it passed.
  }

  function validate_row()
  {
    var d = document.work_form;
    var w = MiddleWindow;
    var d2 = w.document.work_form;
    var visible = d.show.checked;

    var index = d.cell_type.selectedIndex;
    if (index == -1)
    {
      alert("You must select a Row Type!");
      return false;
    }
    var type = d.cell_type.options[index].value;
    if (type == "text")
    {
      if (d2.cell_text.value.length == 0)
      {
        alert("You must specify a value for Row Text!");
        d2.cell_text.focus();
        return false;
      }
      return true;
    }
    else if (type == "total")
    {
      if (visible)
      {
        if (d2.cell_text.value.length == 0)
        {
          alert("You must specify a value for Row Text!");
          d2.cell_text.focus();
          return false;
        }
      }
      if (d2.cell_formula.value.length == 0)
      {
        alert("You must specify a Cell Formula!");
        d2.cell_formula.focus();
        return false;
      }
      if (!validate_formula(d2.cell_formula.value, d.real_index.value))
      {
        return false;
      }
    }
    else if (type == "compare")
    {
      if (d2.cell_formula.value.length == 0)
      {
        alert("You must specify a Comparison Formula!");
        d2.cell_formula.focus();
        return false;
      }
      // validate the Comparison formula or attempt to.
      //var formula = "(" + d2.compare_formula1.value + " " + d2.compare_operator.value + " " + d2.compare_formula2.value + " ? \\\"" + d2.compare_result_true.value + "\\\" : \\\"" + d2.compare_result_false.value + "\\\")";

      //if (d2.compare_formula1.value.length > 0 && d2.compare_formula2.value.length > 0 && d2.compare_operator.value.length > 0 && (d2.compare_result_true.value.length > 0 || d2.compare_result_false.value.length > 0))
      //{
      //  if (!validate_comparison_formula(formula, d.real_index.value))
        if (!validate_comparison_formula(d2.cell_formula.value, d.real_index.value))
        {
          return false;
        }
      //}
      //else
      //{
      //  alert("Error:  You have not specified a complete Comparison Formula!");
      //  d2.cell_formula.focus();
      //  return false;
      //}
    }
    else if (type == "line")
    {
      return true;
    }
    else
    {
      index = d2.item_name.selectedIndex;
      if (index == -1)
      {
        alert("You must select a Row Item!");
        return false;
      }
      if (visible)
      {
        index = d2.sub_levels.selectedIndex;
        if (index == -1)
        {
          alert("You must select a Sub Level Display value!");
          return false;
        }
        if (d2.drillable[0].checked)
        {
          index = d2.drill_report.selectedIndex;
          if (index == -1)
          {
            alert("You must select a Drill down Report!");
            return false;
          }
        }
      }
    }

    return true;
  }

  function CopyRow(a, b)
  {
    // copy the values from Rows[b] to Rows[a]
    // make sure both a and b are inside the Rows array.
    if (a < 0 || a >= Rows.length)
    {
      alert("Error:  a='"+a+"' is invalid!\\nTrying to CopyRow("+a+","+b+").");
      return;
    }
    if (b < 0 || b >= Rows.length)
    {
      alert("Error:  b='"+b+"' is invalid!\\nTrying to CopyRow("+a+","+b+").");
      return;
    }
    for (var i in Rows[parseInt(b,10)])
    {
      eval("Rows[parseInt(a,10)]."+i+" = Rows[parseInt(b,10)]."+i+";");
    }
  }

  function fixup_formula_insert(formula, index, row)
  {
    var rebuild_formula = false;
    var result = parse_formula(formula);
    if (result[0].length == 0)
    {
      // now fix any reference problems that may have been created.
      for (var j=0; j < result[1].length; j++)
      {
        var temp = result[1][j].match(/^R(\\d+)\$/);
        if (temp != null && temp[0] == result[1][j])
        {
          // temp[1] holds the number!
          if (parseInt(temp[1], 10) >= index)
          {
            result[1][j] = "R"+(parseInt(temp[1], 10)+1);  // Increment by one.
            rebuild_formula = true;
          }
        }
      }
      if (rebuild_formula)
      {
        formula = "";
        for (var j=0; j < result[1].length; j++)
        {
          if (formula.length > 0)
          {
            formula += " ";
          }
          formula += result[1][j];
        }
      }
    }
    else
    {
      alert("Formula = '"+formula+"' in Row = '"+row+"' is not OK!\\nError is '"+result[0]+"'\\nTokens are:\\n"+result[1]);

      return "";
    }

    return formula;
  }

  function fixup_comparison_formula_insert(formula, index, result_type, row)
  {
    var result = parse_comparison_formula(formula);
    var rebuild_formula = false;
    var formula1, formula2, operator, result_true, result_false, check_results;

    check_results = (result_type == "result_formula" ? true : false);

    if (result != null)
    {
      if (result.length != 6)
      {
        alert("Error:  The Comparison Row Formula in Row = '"+row+"' did not have the correct number of items!");

        return "";
      }
      formula1 = result[1];
      operator = result[2];
      formula2 = result[3];
      result_true = result[4];
      result_false = result[5];

      // now process the items returned.
      result = fixup_formula_insert(formula1, index, row);
      if (result.length == 0)
      {
        return "";
      }
      else
      {
        if (result != formula1) // it has changed.
        {
          formula1 = result;
          rebuild_formula = true;
        }
      }

      result = fixup_formula_insert(formula2, index, row);
      if (result.length == 0)
      {
        return "";
      }
      else
      {
        if (result != formula2) // it has changed.
        {
          formula2 = result;
          rebuild_formula = true;
        }
      }

      if (check_results)
      {
        result = fixup_formula_insert(result_true, index, row);
        if (result.length == 0)
        {
          return "";
        }
        else
        {
          if (result != result_true) // it has changed.
          {
            result_true = result;
            rebuild_formula = true;
          }
        }

        result = fixup_formula_insert(result_false, index, row);
        if (result.length == 0)
        {
          return "";
        }
        else
        {
          if (result != result_false) // it has changed.
          {
            result_false = result;
            rebuild_formula = true;
          }
        }
      }

      if (rebuild_formula)
      {
        formula = "( " + formula1 + " " + operator + " " + formula2 + " ? \\\"" + result_true + "\\\" : \\\"" + result_false + "\\\" )";
      }
    }
    else
    {
      alert("Could not parse apart Comparison Formula = '"+formula+"' in Row = '"+row+"'!");

      return "";
    }

    return formula;
  }

  function fixup_formula_delete(formula, index, row)
  {
    var rebuild_formula = false;
    var result = parse_formula(formula);
    //alert("formula = '"+formula+"', parsed formula = '"+result[1]+"'");
    if (result[0].length == 0)
    {
      // now fix any reference problems that may have been created.
      for (var j=0; j < result[1].length; j++)
      {
        var temp = result[1][j].match(/^R(\\d+)\$/);
        if (temp != null && temp[0] == result[1][j])
        {
          var number = parseInt(temp[1], 10);
          // temp[1] holds the number!
          if (number > index)
          {
            result[1][j] = "R"+(number-1);  // Decrement by one.
            rebuild_formula = true;
          }
          else if (number == index)
          {
            // we have to remove it from the formula.
            if (j == 0)
            {
              // we are a number at the beginning.
              result[1][j] = "";
              rebuild_formula = true;
              // see if we have to remove an operator
              if (j < result[1].length - 1)
              {
                result[1][j+1] = "";
              }
            }
            else
            {
              // find out what is to the left of us.
              if (result[1][j-1] == "(")
              {
                // find out what is to the right of us.
                if (j+1 < result[1].length && result[1][j+1] == ")")
                {
                  // just remove ourself and the () we are in.
                  result[1][j] = "";
                  result[1][j-1] = "";
                  result[1][j+1] = "";
                  rebuild_formula = true;
                  // find out if there is an operator after the ).
                  if (j+2 < result[1].length && result[1][j+2] != ")")
                  {
                    // remove the operator at j+2.
                    result[1][j+2] = "";
                  }
                  if (j+2 == result[1].length)
                  {
                    // we have to remove whatever was to the left of us, if it exists.
                    if (j-2 >= 0)
                    {
                      alert("Removing operator = '" + result[1][j-2] + "'");
                      result[1][j-2] = "";  // remove the operator that was in front of us.
                    }
                  }
                }
                else
                {
                  // we can remove the guy to the right of us (it has to be an operator)
                  result[1][j] = "";
                  result[1][j+1] = "";
                  rebuild_formula = true;
                }
              }
              else
              {
                // we can remove ourselves and the guy to the left (it has to be an operator)
                result[1][j] = "";
                result[1][j-1] = "";
                rebuild_formula = true;
              }
            }
          }
        }
      }
      if (rebuild_formula)
      {
        formula = "";
        for (var j=0; j < result[1].length; j++)
        {
          if (result[1][j].length > 0)
          {
            if (formula.length > 0)
            {
              formula += " ";
            }
            formula += result[1][j];
          }
        }
      }
    }
    else
    {
      alert("Formula = '"+formula+"' in Row = '"+row+"' is not OK!\\nError is '"+result[0]+"'\\nTokens are:\\n"+result[1]);

      return "";
    }

    return formula;
  }

  function fixup_comparison_formula_delete(formula, index, result_type, row)
  {
    var result = parse_comparison_formula(formula);
    var rebuild_formula = false;
    var formula1, formula2, operator, result_true, result_false, check_results;

    check_results = (result_type == "result_formula" ? true : false);

    if (result != null)
    {
      if (result.length != 6)
      {
        alert("Error:  The Comparison Row Formula in Row = '"+row+"' did not have the correct number of items!");

        return "";
      }
      formula1 = result[1];
      operator = result[2];
      formula2 = result[3];
      result_true = result[4];
      result_false = result[5];

      // now process the items returned.
      result = fixup_formula_delete(formula1, index, row);
      if (result.length == 0)
      {
        return "";
      }
      else
      {
        if (result != formula1) // it has changed.
        {
          formula1 = result;
          rebuild_formula = true;
        }
      }

      result = fixup_formula_delete(formula2, index, row);
      if (result.length == 0)
      {
        return "";
      }
      else
      {
        if (result != formula2) // it has changed.
        {
          formula2 = result;
          rebuild_formula = true;
        }
      }

      if (check_results)
      {
        result = fixup_formula_delete(result_true, index, row);
        if (result.length == 0)
        {
          return "";
        }
        else
        {
          if (result != result_true) // it has changed.
          {
            result_true = result;
            rebuild_formula = true;
          }
        }

        result = fixup_formula_delete(result_false, index, row);
        if (result.length == 0)
        {
          return "";
        }
        else
        {
          if (result != result_false) // it has changed.
          {
            result_false = result;
            rebuild_formula = true;
          }
        }
      }

      if (rebuild_formula)
      {
        formula = "( " + formula1 + " " + operator + " " + formula2 + " ? \\\"" + result_true + "\\\" : \\\"" + result_false + "\\\" )";
      }
    }
    else
    {
      alert("Could not parse apart Comparison Formula = '"+formula+"' in Row = '"+row+"'!");

      return "";
    }

    return formula;
  }

  function fixup_formula_move(formula, index1, index2, row)
  {
    var rebuild_formula = false;
    var increment = 0;

    if (parseInt(index1, 10) > parseInt(index2, 10))
    {
      increment = -1;
      var temp = index2;  // swap the two rows around.
      index2 = index1;
      index1 = temp;
    }
    else
    {
      increment = 1;
    }
    var result = parse_formula(formula);
    if (result[0].length == 0)
    {
      // now fix any reference problems that may have been created.
      for (var j=0; j < result[1].length; j++)
      {
        var temp = result[1][j].match(/^R(\\d+)\$/);
        if (temp != null && temp[0] == result[1][j])
        {
          var number = parseInt(temp[1], 10);
          // temp[1] holds the number!
          if (number > parseInt(index1, 10) && number < parseInt(index2, 10))
          {
            result[1][j] = "R"+(number+increment);
            rebuild_formula = true;
          }
          else if (number == index1)
          {
            // check to make sure we are not doing a forward reference!
            if (increment == -1)
            {
              if (index2 >= row)
              {
                alert("Formula = '"+formula+"' in Row = '"+row+"' is not OK!\\nError is 'You are trying to forward reference the new row!'");

                return "";
              }
              result[1][j] = "R"+index2;
              rebuild_formula = true;
            }
            else  // treat as normal (just increment the row that was there, but don't insert the new row into the formula.
            {
              result[1][j] = "R"+(number+increment);
              rebuild_formula = true;
            }
          }
          else if (number == index2)
          {
            if (increment == 1)
            {
              result[1][j] = "R"+index1;
              rebuild_formula = true;
            }
            else
            {
              result[1][j] = "R"+(number+increment);
              rebuild_formula = true;
            }
          }
        }
      }
      if (rebuild_formula)
      {
        formula = "";
        for (var j=0; j < result[1].length; j++)
        {
          if (formula.length > 0)
          {
            formula += " ";  // Add whitespace seperators.
          }
          formula += result[1][j];
        }
      }
    }
    else
    {
      alert("Formula = '"+formula+"' in Row = '"+row+"' is not OK!\\nError is '"+result[0]+"'\\nTokens are:\\n"+result[1]);

      return "";
    }

    return formula;
  }

  function fixup_comparison_formula_move(formula, index1, index2, result_type, row)
  {
    var result = parse_comparison_formula(formula);
    var rebuild_formula = false;
    var formula1, formula2, operator, result_true, result_false, check_results;

    check_results = (result_type == "result_formula" ? true : false);

    if (result != null)
    {
      if (result.length != 6)
      {
        alert("Error:  The Comparison Row Formula in Row = '"+row+"' did not have the correct number of items!");

        return "";
      }
      formula1 = result[1];
      operator = result[2];
      formula2 = result[3];
      result_true = result[4];
      result_false = result[5];

      // now process the items returned.
      result = fixup_formula_move(formula1, index1, index2, row);
      if (result.length == 0)
      {
        return "";
      }
      else
      {
        if (result != formula1) // it has changed.
        {
          formula1 = result;
          rebuild_formula = true;
        }
      }

      result = fixup_formula_move(formula2, index1, index2, row);
      if (result.length == 0)
      {
        return "";
      }
      else
      {
        if (result != formula2) // it has changed.
        {
          formula2 = result;
          rebuild_formula = true;
        }
      }

      if (check_results)
      {
        result = fixup_formula_move(result_true, index1, index2, row);
        if (result.length == 0)
        {
          return "";
        }
        else
        {
          if (result != result_true) // it has changed.
          {
            result_true = result;
            rebuild_formula = true;
          }
        }

        result = fixup_formula_move(result_false, index1, index2, row);
        if (result.length == 0)
        {
          return "";
        }
        else
        {
          if (result != result_false) // it has changed.
          {
            result_false = result;
            rebuild_formula = true;
          }
        }
      }

      if (rebuild_formula)
      {
        formula = "(" + formula1 + " " + operator + " " + formula2 + " ? \\\"" + result_true + "\\\" : \\\"" + result_false + "\\\")";
      }
    }
    else
    {
      alert("Could not parse apart Comparison Formula = '"+formula+"' in Row = '"+row+"'!");

      return "";
    }

    return formula;
  }

  function add_row_to_report()
  {
    var w1 = TopWindow;
    var d = w1.document.work_form;
    var w = MiddleWindow;
    var d2 = w.document.work_form;
    var cell_type = d.cell_type.options[d.cell_type.selectedIndex].value;

    if (!validate_row())
    {
      return;
    }

    // backup the Rows array first so we can undo any moves, etc.
    var backupRows = new Array();
    for (var i=0; i < Rows.length; i++)
    {
      backupRows[i] = new Object();
      for (var j in Rows[i])
      {
        eval("backupRows[i]."+j+" = Rows[i]."+j+";");
      }
    }

    if (EditMode == "create")
    {
      var negative_index = 0;
      var drillable_index = 1;
      var item_name = "---------------";
      var drill_report = "---------------";
      var sub_levels = "---------------";
      var initial_state = "o";
      var display_level = 0;
      var bold = "false";
      var fg_color = "$fg_color";
      var bg_color = "$bg_color";
      var cell_text = "";
      var activity = "true";
      var show = "1";
      var result_type = "";
      var cell_formula = "";
      var query_type = "0";
      var underline = 0;

      show = (d.show.checked ? 1 : 0);

      if (cell_type != "text" && cell_type != "line" && cell_type != "detail")
      {
        if (show)
        {
          for (var i=0; i < d2.negative_format.length; i++)
          {
            if (d2.negative_format[i].checked)
            {
              negative_index = i;
            }
          }
        }
        if (cell_type != "total" && cell_type != "compare")
        {
          if (show)
          {
            for (var i=0; i < d2.drillable.length; i++)
            {
              if (d2.drillable[i].checked)
              {
                drillable_index = i;
              }
            }
            drill_report = d2.drill_report.options[d2.drill_report.selectedIndex].value;
            sub_levels = d2.sub_levels.options[d2.sub_levels.selectedIndex].value;
          }
          else
          {
            sub_levels = ReportDisplayLevels[cell_type][0];
          }
          if (cell_type != "begin_account_type_balance" && cell_type != "begin_account_balance" && cell_type != "end_account_type_balance" && cell_type != "end_account_balance" && cell_type != "begin_sub_account_balance" && cell_type != "end_sub_account_balance")
          {
            query_type = d2.query_type.options[d2.query_type.selectedIndex].value;
          }
          item_name = d2.item_name.options[d2.item_name.selectedIndex].value;
        }
        else if (cell_type == "total")
        {
          if (show)
          {
            cell_text = d2.cell_text.value;
            underline = (d2.underline.checked ? 1 : 0);
          }
          cell_formula = d2.cell_formula.value;
        }
        else if (cell_type == "compare")
        {
          cell_formula = d2.cell_formula.value;
          result_type = (d2.result_type.checked ? "result_formula" : "");
          if (show && d2.cell_text.value.length > 0)
          {
            cell_text = d2.cell_text.value;
          }
          if (show)
          {
            underline = (d2.underline.checked ? 1 : 0);
          }
        }
        if (show)
        {
          bold = d2.bold.options[d2.bold.selectedIndex].value;
          display_level = d2.display_level.options[d2.display_level.selectedIndex].value;
          fg_color = d2.fg_color.options[d2.fg_color.selectedIndex].value;
          bg_color = d2.bg_color.options[d2.bg_color.selectedIndex].value;
          initial_state = d2.initial_state.options[d2.initial_state.selectedIndex].value;
        }
      }
      else if (cell_type == "text")
      {
        cell_text = d2.cell_text.value;
        bold = d2.bold.options[d2.bold.selectedIndex].value;
        display_level = d2.display_level.options[d2.display_level.selectedIndex].value;
        fg_color = d2.fg_color.options[d2.fg_color.selectedIndex].value;
        bg_color = d2.bg_color.options[d2.bg_color.selectedIndex].value;
        initial_state = d2.initial_state.options[d2.initial_state.selectedIndex].value;
      }
      activity = d2.activity.value;

      if (cell_type == "compare")
      {
        // build the correct cell_formula from the hidden fields
        //cell_formula = "( " + d2.compare_formula1.value + " " + d2.compare_operator.value + " " + d2.compare_formula2.value + " ? \\\"" + d2.compare_result_true.value + "\\\" : \\\"" + d2.compare_result_false.value + "\\\" )";
        cell_formula = d2.cell_formula.value;
      }

      if (parseInt(d.index.value,10) != Rows.length)
      {
        // They want to insert this row into the array.
        // Make sure it is inside the Array boundary first.
        if (parseInt(d.index.value,10) < 0 || parseInt(d.index.value,10) > Rows.length)
        {
          alert("Error:  You tried to insert a Row that is outside the Boundaries!\\nAppending to End.\\n");
          d.index.value = Rows.length;
        }
        else
        {
          // Now we move the rows around, validate total and compare rows and then insert the final row.
          // alert("You are inserting a row!\\nJames, validate the Total and Compare Rows before doing this!\\n");
          for (var i=0; i < Rows.length; i++)
          {
            if (Rows[i].cell_type == "total")
            {
              var formula = Rows[i].cell_formula;
              var result = fixup_formula_insert(formula, parseInt(d.index.value,10), i);
              if (result.length == 0)
              {
                // undo any formula fixups.
                Rows.length = 0;
                for (var j=0; j < backupRows.length; j++)
                {
                  Rows[j] = new Object();
                  for (var k in backupRows[j])
                  {
                    eval("Rows[j]."+k+" = backupRows[j]."+k+";");
                  }
                }
                //reset_row();
                return;
              }
              else
              {
                if (result != formula) // it has changed.
                {
                  Rows[i].cell_formula = result;
                }
              }
            }
            else if (Rows[i].cell_type == "compare")
            {
              var formula = Rows[i].cell_formula;
              var result = fixup_comparison_formula_insert(formula, parseInt(d.index.value,10), Rows[i].result_type, i);
              if (result.length == 0)
              {
                // undo any formula fixups
                Rows.length = 0;
                for (var j=0; j < backupRows.length; j++)
                {
                  Rows[j] = new Object();
                  for (var k in backupRows[j])
                  {
                    eval("Rows[j]."+k+" = backupRows[j]."+k+";");
                  }
                }
                //reset_row();
                return;
              }
              else
              {
                if (result != formula)
                {
                  Rows[i].cell_formula = result;
                }
              }
            }
          }
          Rows.length++;  // Increase the array by 1.
          Rows[Rows.length-1] = new rowEntry();  // Give it a reference first.
          for (var i=Rows.length-1; i > parseInt(d.index.value,10); i--)
          {
            CopyRow(i, i-1);
            Rows[i].index = i;
          }

          // fixup the row being inserted if it is a Total or Comparison row.  (This is for forward references only!)

          if (cell_type == "total")
          {
            var result = fixup_formula_insert(cell_formula, parseInt(d.index.value,10), parseInt(d.index.value,10));
            if (result.length == 0)
            {
              // undo the insert
              Rows.length = 0;
              for (var j=0; j < backupRows.length; j++)
              {
                Rows[j] = new Object();
                for (var k in backupRows[j])
                {
                  eval("Rows[j]."+k+" = backupRows[j]."+k+";");
                }
              }
              Rows.length--;
              reset_row();
              return;
            }
            else
            {
              if (result != cell_formula) // it has changed.
              {
                cell_formula = result;
              }
            }
          }
          else if (cell_type == "compare")
          {
            var result = fixup_comparison_formula_insert(cell_formula, parseInt(d.index.value,10), result_type, parseInt(d.index.value,10));
            if (result.length == 0)
            {
              // undo the insert
              Rows.length = 0;
              for (var j=0; j < backupRows.length; j++)
              {
                Rows[j] = new Object();
                for (var k in backupRows[j])
                {
                  eval("Rows[j]."+k+" = backupRows[j]."+k+";");
                }
              }
              Rows.length--;
              reset_row();
              return;
            }
            else
            {
              if (result != cell_formula) // it has changed.
              {
                cell_formula = result;
              }
            }
          }
        }
      }

      Rows[parseInt(d.index.value,10)] = new rowEntry(parseInt(d.index.value,10), cell_type, item_name, cell_text, cell_formula, activity, negative_index, bold, fg_color, bg_color, sub_levels, initial_state, display_level, drillable_index, drill_report, query_type, show, result_type, underline);

      reset_row();
    }
    else  // mode = "edit"
    {
      var negative_index = 0;
      var drillable_index = 1;
      var item_name = "---------------";
      var drill_report = "---------------";
      var sub_levels = "---------------";
      var initial_state = "o";
      var display_level = 0;
      var bold = "false";
      var fg_color = "$fg_color";
      var bg_color = "$bg_color";
      var cell_text = "";
      var activity = "true";
      var show = "1";
      var result_type = "";
      var cell_formula = "";
      var query_type = "0";
      var underline = 0;

      show = (d.show.checked ? 1 : 0);

      if (cell_type != "text" && cell_type != "line" && cell_type != "detail")
      {
        if (show)
        {
          for (var i=0; i < d2.negative_format.length; i++)
          {
            if (d2.negative_format[i].checked)
            {
              negative_index = i;
            }
          }
        }
        if (cell_type != "total" && cell_type != "compare")
        {
          if (show)
          {
            for (var i=0; i < d2.drillable.length; i++)
            {
              if (d2.drillable[i].checked)
              {
                drillable_index = i;
              }
            }
            drill_report = d2.drill_report.options[d2.drill_report.selectedIndex].value;
            sub_levels = d2.sub_levels.options[d2.sub_levels.selectedIndex].value;
          }
          else
          {
            sub_levels = ReportDisplayLevels[cell_type][0];
          }
          if (cell_type != "begin_account_type_balance" && cell_type != "begin_account_balance" && cell_type != "end_account_type_balance" && cell_type != "end_account_balance" && cell_type != "begin_sub_account_balance" && cell_type != "end_sub_account_balance")
          {
            query_type = d2.query_type.options[d2.query_type.selectedIndex].value;
          }
          item_name = d2.item_name.options[d2.item_name.selectedIndex].value;
        }
        else if (cell_type == "total")
        {
          if (show)
          {
            cell_text = d2.cell_text.value;
            underline = (d2.underline.checked ? 1 : 0);
          }
          cell_formula = d2.cell_formula.value;
        }
        else if (cell_type == "compare")
        {
          cell_formula = d2.cell_formula.value;
          result_type = (d2.result_type.checked ? "result_formula" : "");
          if (show && d2.cell_text.value.length > 0)
          {
            cell_text = d2.cell_text.value;
          }
          if (show)
          {
            underline = (d2.underline.checked ? 1 : 0);
          }
        }
        if (show)
        {
          bold = d2.bold.options[d2.bold.selectedIndex].value;
          display_level = d2.display_level.options[d2.display_level.selectedIndex].value;
          fg_color = d2.fg_color.options[d2.fg_color.selectedIndex].value;
          bg_color = d2.bg_color.options[d2.bg_color.selectedIndex].value;
          initial_state = d2.initial_state.options[d2.initial_state.selectedIndex].value;
        }
      }
      else if (cell_type == "text")
      {
        cell_text = d2.cell_text.value;
        bold = d2.bold.options[d2.bold.selectedIndex].value;
        display_level = d2.display_level.options[d2.display_level.selectedIndex].value;
        fg_color = d2.fg_color.options[d2.fg_color.selectedIndex].value;
        bg_color = d2.bg_color.options[d2.bg_color.selectedIndex].value;
        initial_state = d2.initial_state.options[d2.initial_state.selectedIndex].value;
      }
      activity = d2.activity.value;

      if (cell_type == "compare")
      {
        // build the correct cell_formula from the hidden fields
        //cell_formula = "(" + d2.compare_formula1.value + " " + d2.compare_operator.value + " " + d2.compare_formula2.value + " ? \\\"" + d2.compare_result_true.value + "\\\" : \\\"" + d2.compare_result_false.value + "\\\")";
        cell_formula = d2.cell_formula.value;
      }

      if (parseInt(d.index.value,10) != parseInt(d.real_index.value,10))
      {
        // They want to move this row around the array.
        // Make sure it is inside the Array boundary first.
        if (parseInt(d.index.value,10) < 0 || parseInt(d.index.value,10) >= Rows.length)
        {
          alert("Error:  You tried to move a Row outside the Boundaries!\\nPlacing in original location.\\n");
          d.index.value = d.real_index.value;
        }
        else
        {
          // Fixup the Total and Comparison Rows formulas.
          for (var i=0; i < Rows.length; i++)
          {
            if (Rows[i].cell_type == "total")
            {
              var formula = Rows[i].cell_formula;
              var result = fixup_formula_move(formula, parseInt(d.index.value,10), parseInt(d.real_index.value,10), i);
              if (result.length == 0)
              {
                // undo the formula fixups
                Rows.length = 0;
                for (var j=0; j < backupRows.length; j++)
                {
                  Rows[j] = new Object();
                  for (var k in backupRows[j])
                  {
                    eval("Rows[j]."+k+" = backupRows[j]."+k+";");
                  }
                }
                //reset_row();
                return;
              }
              else
              {
                if (result != formula) // it has changed.
                {
                  Rows[i].cell_formula = result;
                }
              }
            }
            else if (Rows[i].cell_type == "compare")
            {
              var formula = Rows[i].cell_formula;
              var result = fixup_comparison_formula_move(formula, parseInt(d.index.value,10), parseInt(d.real_index.value,10), parseInt(Rows[i].result_type,10), i);
              if (result.length == 0)
              {
                // undo the formula fixups
                Rows.length = 0;
                for (var j=0; j < backupRows.length; j++)
                {
                  Rows[j] = new Object();
                  for (var k in backupRows[j])
                  {
                    eval("Rows[j]."+k+" = backupRows[j]."+k+";");
                  }
                }
                //reset_row();
                return;
              }
              else
              {
                if (result != formula)
                {
                  Rows[i].cell_formula = result;
                }
              }
            }
          }
          // Now we move the rows around, validate total and compare rows and then insert the final row.
          // alert("You are moving a row!\\nJames, validate the Total and Compare Rows before doing this!\\n");
          if (parseInt(d.index.value,10) > parseInt(d.real_index.value,10)) // Case 1 - New row > Old row.
          {
            for (var i=parseInt(d.real_index.value,10); i < parseInt(d.index.value,10); i++)
            {
              CopyRow(i, i+1);
              Rows[i].index = i;
            }
          }
          else  // Case 2 - Old row > New row.
          {
            for (var i=parseInt(d.real_index.value,10); i > parseInt(d.index.value,10); i--)
            {
              CopyRow(i, i-1);
              Rows[i].index = i;
            }
          }
          // fixup the row being moved if it is a Total or Comparison row.  (This is for forward references only!)

          if (cell_type == "total")
          {
            var result = fixup_formula_move(cell_formula, parseInt(d.index.value,10), parseInt(d.real_index.value,10), parseInt(d.index.value,10));
            if (result.length == 0)
            {
              // need to undo the copy!
              Rows.length = 0;
              for (var j=0; j < backupRows.length; j++)
              {
                Rows[j] = new Object();
                for (var k in backupRows[j])
                {
                  eval("Rows[j]."+k+" = backupRows[j]."+k+";");
                }
              }
              //reset_row();
              return;
            }
            else
            {
              if (result != cell_formula) // it has changed.
              {
                cell_formula = result;
              }
            }
          }
          else if (cell_type == "compare")
          {
            var result = fixup_comparison_formula_move(cell_formula, parseInt(d.index.value,10), parseInt(d.real_index.value,10), result_type, parseInt(d.index.value,10));
            if (result.length == 0)
            {
              // need to undo the copy!
              Rows.length = 0;
              for (var j=0; j < backupRows.length; j++)
              {
                Rows[j] = new Object();
                for (var k in backupRows[j])
                {
                  eval("Rows[j]."+k+" = backupRows[j]."+k+";");
                }
              }
              //reset_row();
              return;
            }
            else
            {
              if (result != cell_formula) // it has changed.
              {
                cell_formula = result;
              }
            }
          }
        }
      }

      Rows[parseInt(d.index.value,10)] = new rowEntry(parseInt(d.index.value,10), cell_type, item_name, cell_text, cell_formula, activity, negative_index, bold, fg_color, bg_color, sub_levels, initial_state, display_level, drillable_index, drill_report, query_type, show, result_type, underline);

      reset_row();
    }
  }

  function signal_edit(i)
  {
    var code = "edit_row("+i+");";
    window.setTimeout(code, 10);
  }

  function signal_delete(i)
  {
    var code = "delete_row("+i+");";
    window.setTimeout(code, 10);
  }

  function edit_row(i)
  {
    // fill in the work_form with the values for the selected Row.
    var d = document.work_form;

    EditMode = "edit";

    d.real_index.value = Rows[i].index;
    d.index.value = Rows[i].index;
    found = false;
    for (j=0; j < d.cell_type.length && !found; j++)
    {
      if (d.cell_type.options[j].value == Rows[i].cell_type)
      {
        d.cell_type.selectedIndex = j;
        found = true;
      }
    }
    d.show.checked = Rows[i].show;

    display_middle_window(EditMode);

    update_bottom_window(false);
  }

  function delete_row(i)
  {
    var backupRows = new Array();
    for (var k=0; k < Rows.length; k++)
    {
      backupRows[k] = new Object();
      for (var j in Rows[k])
      {
        eval("backupRows[k]."+j+" = Rows[k]."+j+";");
      }
    }
    // fixup the Total and Comparison Rows that are affected by this Deletion.
    for (var j=0; j < Rows.length; j++)
    {
      if (Rows[j].cell_type == "total")
      {
        var formula = Rows[j].cell_formula;
        var result = fixup_formula_delete(formula, i, j);
        if (result.length == 0)
        {
          Rows.length = 0;
          for (var j=0; j < backupRows.length; j++)
          {
            Rows[j] = new Object();
            for (var k in backupRows[j])
            {
              eval("Rows[j]."+k+" = backupRows[j]."+k+";");
            }
          }
          reset_row();
          return;
        }
        else
        {
          if (result != formula)
          {
            Rows[j].cell_formula = result;
          }
        }
      }
      else if (Rows[j].cell_type == "compare")
      {
        var formula = Rows[j].cell_formula;
        var result = fixup_comparison_formula_delete(formula, i, Rows[j].result_type, j);
        if (result.length == 0)
        {
          Rows.length = 0;
          for (var j=0; j < backupRows.length; j++)
          {
            Rows[j] = new Object();
            for (var k in backupRows[j])
            {
              eval("Rows[j]."+k+" = backupRows[j]."+k+";");
            }
          }
          reset_row();
          return;
        }
        else
        {
          if (result != formula)
          {
            Rows[j].cell_formula = result;
          }
        }
      }
    }

    // remove this row from the Rows array and then refresh the bottom window
    for (j=i+1; j < Rows.length; j++)
    {
      CopyRow(parseInt(j, 10)-1, j);
      Rows[j-1].index = j-1;
    }
    Rows.length--;

    // make sure all formula rows are still valid.
    for (j=0; j < Rows.length; j++)
    {
      if (Rows[j].cell_type == "total")
      {
        if (!validate_formula(Rows[j].cell_formula, j))
        {
          // undo the delete
          Rows.length = 0;
          for (var j=0; j < backupRows.length; j++)
          {
            Rows[j] = new Object();
            for (var k in backupRows[j])
            {
              eval("Rows[j]."+k+" = backupRows[j]."+k+";");
            }
          }
        }
      }
      else if (Rows[j].cell_type == "compare")
      {
        if (!validate_comparison_formula(Rows[j].cell_formula, j, Rows[j].result_type))
        {
          // undo the delete
          Rows.length = 0;
          for (var j=0; j < backupRows.length; j++)
          {
            Rows[j] = new Object();
            for (var k in backupRows[j])
            {
              eval("Rows[j]."+k+" = backupRows[j]."+k+";");
            }
          }
        }
      }
    }

    update_bottom_window(true);

    // set the Row # to the new next value
    document.work_form.real_index.value = Rows.length;
    document.work_form.index.value = Rows.length;
  }

  function display_first_row(d, i, status)
  {
    d.writeln('  <tr bgcolor="#eeeeee"' + (Rows[i].cell_type == "line" ? ' rowspan="2"' : '') + '>');
    d.write("    <td align=center>"+Rows[i].index+"</td><td align=center>"+cell_types[Rows[i].cell_type]+"</td>");
    if (Rows[i].cell_type == "line")
    {
      d.writeln('<td colspan="6" align="center"><hr width="75%" /></td>');
    }
    else
    {
      d.write("<td align=center>");
      if (Rows[i].cell_type == "account_type" || Rows[i].cell_type == "accounts_of" || Rows[i].cell_type == "sub_accounts_of_type" || Rows[i].cell_type == "begin_account_type_balance" || Rows[i].cell_type == "end_account_type_balance")
      {
        found = false;
        for (j=0; j < AccountTypesArray.length && !found; j++)
        {
          if (Rows[i].item_name == AccountTypesArray[j].num)
          {
            d.write(AccountTypesArray[j].name);
            found = true;
          }
        }
        if (!found)
        {
          d.write("<font color=\\\"red\\\">Error!</font>");
          alert("Could not find Account Type = '"+Rows[i].item_name+"' in the AccountTypesArray!");
        }
      }
      else
      {
        if (Rows[i].cell_type == "account" || Rows[i].cell_type == "sub_accounts_of_account" || Rows[i].cell_type == "begin_account_balance" || Rows[i].cell_type == "end_account_balance")
        {
          found = false;
          for (j=0; j < AccountsArray.length && !found; j++)
          {
            if (Rows[i].item_name == AccountsArray[j].num)
            {
              d.write(AccountsArray[j].name);
              found = true;
            }
          }
          if (!found)
          {
            d.write("<font color=\\\"red\\\">Error!</font>");
            alert("Could not find Account = '"+Rows[i].item_name+"' in the AccountsArray!");
          }
        }
        else
        {
          if (Rows[i].cell_type == "sub_account" || Rows[i].cell_type == "begin_sub_account_balance" || Rows[i].cell_type == "end_sub_account_balance")
          {
            found = false;
            for (j=0; j < SubAccountsArray.length && !found; j++)
            {
              if (Rows[i].item_name == SubAccountsArray[j].num)
              {
                d.write(SubAccountsArray[j].name);
                found = true;
              }
            }
            if (!found)
            {
              d.write("<font color=\\\"red\\\">Error!</font>");
              alert("Could not find Sub-Account = '"+Rows[i].item_name+"' in the SubAccountsArray!");
            }
          }
          else
          {
            d.write("&nbsp;");
          }
        }
      }
      d.write("</td><td align=center>");
      if (Rows[i].cell_text.length > 0)
      {
        d.write(Rows[i].cell_text);
      }
      else
      {
        d.write("&nbsp;");
      }
      d.write("</td><td align=center>");
      if (Rows[i].cell_formula.length > 0)
      {
        d.write(Rows[i].cell_formula);
      }
      else
      {
        d.write("&nbsp;");
      }
      d.write("</td><td align=center>"+CalcType[Rows[i].activity]+"</td><td align=center>"+QueryType[Rows[i].query_type]+"</td><td align=center>"+negative_formats[Rows[i].negative_format]+"</td>");
    }
    if (status)
    {
      d.writeln("<td align=center><input type=button name=\\\"edit_"+i+"\\\" value=\\\"E\\\" onClick=\\\"opener.signal_edit("+i+")\\\"> <input type=button name=\\\"delete_"+i+"\\\" value=\\\"D\\\" onClick=\\\"opener.signal_delete("+i+")\\\"></td>");
    }
    else
    {
      d.writeln("<td align=center>&nbsp;</td>");
    }
    d.writeln("  </tr>");
  }

  function display_second_row(d, i, status)
  {
    if (Rows[i].cell_type != "line")
    {
      d.writeln("  <tr bgcolor=\\\"#eeeeee\\\">");
      d.write("    <td align=center>"+YesNo[Rows[i].bold]+"</td><td align=center>"+fg_colors[Rows[i].fg_color]+"</td><td align=center>"+bg_colors[Rows[i].bg_color]+"</td><td align=center>");
      var type = Rows[i].cell_type;
      if (type == "text" || type == "total" || type == "line" || type == "compare")
      {
        d.write("&nbsp;");
      }
      else
      {
        if (Rows[i].show)
        {
          d.write(Rows[i].sub_levels);
        }
        else
        {
          d.write("&nbsp;");
        }
      }
      d.write("</td><td align=center>"+RowState[Rows[i].initial_state]+"</td><td align=center>"+DisplayLevels[Rows[i].display_level]+"</td><td align=center>"+Drillable[Rows[i].drillable]+"</td><td align=center>");

      if ((type == "text" || type == "total" || type == "line" || type == "compare") || Rows[i].drillable == 1)
      {
        d.write("&nbsp;");
      }
      else
      {
        var found = false;
        for (j=0; j < DetailedReportsArray.length && !found; j++)
        {
          if (DetailedReportsArray[j].id == Rows[i].drill_report)
          {
            d.write(DetailedReportsArray[j].name);
            found = true;
          }
        }
        if (!found)
        {
          d.write("<font color=\\\"red\\\">Error!</font>");
          alert("Could not find Drill Down Report ID = '"+Rows[i].drill_report+"'!");
        }
      }
      d.write("</td>");
      d.writeln("<td align=center>"+(Rows[i].show ? "True" : "False")+"</td>");
      d.writeln("  </tr>");
    }
  }

  function update_bottom_window(status)
  {
    var bottomWindowName = window.parent.name + '_bottom';
    //alert("update_bottom_window status = '"+status+"'");
    var w = window.open("", bottomWindowName);
    var d = w.document;

    d.open();
    d.writeln("<html>");
    d.writeln("<body bgcolor=\\\"white\\\">");

    if (Rows.length > 0)
    {
      d.writeln("<form name=\\\"row_form\\\">");
      d.writeln("<table border=1 cellpadding=0 cellspacing=1 align=center>");
      d.writeln("  <tr bgcolor=\\\"#cccccc\\\">");
      d.write("    <th>#</th><th>Type</th><th>Item</th><th>Text</th><th>Formula</th><th>Calc Type</th><th>Query Type</th><th>Negative format</th><th>Edit / Delete</th>");
      d.writeln("  </tr>");
      // display the first rows values
      display_first_row(d, 0, status);

      d.writeln("  <tr bgcolor=\\\"#cccccc\\\">");
      d.write("    <th>Bold</th><th>ForeGround</th><th>BackGround</th><th>Sub Level</th><th>State</th><th>Display Level</th><th>Drillable</th><th>Drill Report</th>");
      d.writeln("<th>Visible</th>");
      d.writeln("  </tr>");
      // display the rest of the first rows values
      display_second_row(d, 0, status);

      // now display the rest of the rows.
      for (i=1; i < Rows.length; i++)
      {
        var cols = 9;
        d.writeln("  <tr>");
        d.writeln("    <td colspan="+cols+">&nbsp;</td>");
        d.writeln("  </tr>");

        display_first_row(d, i, status);
        display_second_row(d, i, status);
      }

      d.writeln("</table>");
      d.writeln("</form>");
    }
    d.writeln("</body>");
    d.writeln("</html>");

    d.close();
  }

  function reset_row()
  {
    var d = document.work_form;
    d.reset();

    if (typeof(FormulaWindow) != "undefined")
    {
      // see if the window is closed.
      if (FormulaWindow.closed == false)
      {
        FormulaWindow.close();  // close the window.
      }
    }

    if (typeof(CompareFormulaWindow) != "undefined")
    {
      // see if the window is closed.
      if (CompareFormulaWindow.closed == false)
      {
        CompareFormulaWindow.close();  // close the window.
      }
    }

    if (EditMode == "edit")
    {
      EditMode = "create";
    }
    update_bottom_window(true);

    window.setTimeout("document.work_form.index.value='"+Rows.length+"';document.work_form.real_index.value='"+Rows.length+"';document.work_form.show.checked=1;display_middle_window(EditMode);", 10);
  }

  function reset_report()
  {
    if (window.confirm("Are you sure you want to reset this report?\\nOk=Yes, Cancel=No"))
    {
      if (typeof(FormulaWindow) != "undefined")
      {
        // see if the window is closed.
        if (FormulaWindow.closed == false)
        {
          FormulaWindow.close();  // close the window.
        }
      }

      if (typeof(CompareFormulaWindow) != "undefined")
      {
        // see if the window is closed.
        if (CompareFormulaWindow.closed == false)
        {
          CompareFormulaWindow.close();  // close the window.
        }
      }

      EditMode = "create";

      initialize_work_form();
    }
  }

  function initialize_work_form()
  {
    Rows.length = 0;

END_OF_CODE

  # gather all rows from the database and populate the Rows array with them.
  my @rows = GetReportRows(dbh => $dbh, id => $id);
  my $activityFix = 0;

  for ($counter = 0; $counter < scalar @rows; $counter++)
  {
    my %info = %{$rows[$counter]};
    my $cell_type = $info{'cell_type'};
    $info{'cell_formula'} =~ s/"/\\\"/g;
    if ($cell_type !~ /^(text|line|total|compare)$/ && $info{activity} != $date_range)
    {
      $activityFix = 1;
      $info{activity} = $date_range;
    }
    $doc->print("    Rows[$counter] = new rowEntry(\"$info{'index'}\", \"$info{'cell_type'}\", \"$info{'item_name'}\", \"$info{'cell_text'}\", \"$info{'cell_formula'}\", \"$t_or_f{$info{'activity'}}\", \"$info{'negative_format'}\", \"$t_or_f{$info{'bold'}}\", \"$info{'fg_color'}\", \"$info{'bg_color'}\", \"" . ($info{'sub_levels'} == -1 ? "---------------" : "$report_levels{$cell_type}[$info{'sub_levels'}]") . "\", \"$info{'initial_state'}\", \"$info{'display_level'}\", " . ($info{'drillable'} ? 0 : 1) . ", \"" . ($info{'drill_report'} == -1 ? "---------------" : $info{'drill_report'}) . "\", \"$info{'query_type'}\", $info{'show'}, \"$info{'result_type'}\", $info{'underline'});\n");
  }

  if ($activityFix)
  {
    my $onLoadCode = <<"END_OF_CODE";
  alert("You must save the report as the activity flag is now different.");
END_OF_CODE
    $doc->setOnload(code => $onLoadCode);
  }

  $doc->print(<<"END_OF_CODE");
    var middleWindowName = window.parent.name + '_middle';
    var w = window.open("", middleWindowName);
    MiddleWindow = w;

    var topWindowName = window.parent.name + '_top';
    var x = window.open("", topWindowName);
    TopWindow = x;

    window.setTimeout('reset_row();', 45);

    //window.setTimeout('update_bottom_window(true);', 30);

    document.report_form.windowName.value = window.parent.name;
  }
END_OF_CODE
  my $on_unload_code = <<"END_OF_CODE";
  if (typeof(FormulaWindow) != "undefined")
  {
    // see if the window is closed.
    if (FormulaWindow.closed == false)
    {
      FormulaWindow.close();  // close the window.
    }
  }

  if (typeof(CompareFormulaWindow) != "undefined")
  {
    // see if the window is closed.
    if (CompareFormulaWindow.closed == false)
    {
      CompareFormulaWindow.close();  // close the window.
    }
  }
END_OF_CODE
  $doc->setOnunload(code => $on_unload_code);  # set the onUnload Event handler to make sure that the windows are cleaned up.
  }
  else  # Detailed View Report   **********************************************************----------------------------------*****************************
  {
  $doc->print(<<"END_OF_CODE");
  var RowBeingEdited = -1;
  var EditMode = "create";

  var Rows = new Array();

  function rowEntry(index, cell_type, cell_text, bold, fg_color, bg_color, show)
  {
    this.index = index;
    this.cell_type = cell_type;
    this.cell_text = cell_text;
    this.bold = bold;
    this.fg_color = fg_color;
    this.bg_color = bg_color;
    this.show = show;
  }

  var Fonts = new Array();
END_OF_CODE

  foreach my $item (@font_names)
  {
    $doc->print("  Fonts[\"$item\"] = \"$font_names{$item}\";\n");
  }

  $doc->print(<<"END_OF_CODE");

  var fg_colors = new Array();
END_OF_CODE

  foreach my $item (@fg_colors)
  {
    $doc->print("  fg_colors[\"$item\"] = \"$fg_colors{$item}\";\n");
  }

  $doc->print(<<"END_OF_CODE");

  var bg_colors = new Array();
END_OF_CODE

  foreach my $item (@bg_colors)
  {
    $doc->print("  bg_colors[\"$item\"] = \"$bg_colors{$item}\";\n");
  }

  $doc->print(<<"END_OF_CODE");

  var negative_formats = new Array();
END_OF_CODE

  foreach my $item (@negative_formats)
  {
    $doc->print("  negative_formats[\"$item\"] = '$negative_formats{$item}';\n");
  }

  $doc->print(<<"END_OF_CODE");

  var cell_types = new Array();
END_OF_CODE

  foreach my $item (@detailed_cell_types)
  {
    $doc->print("  cell_types[\"$item\"] = '$detailed_cell_types{$item}';\n");
  }

  $doc->print(<<"END_OF_CODE");

  var YesNo = new Array();
  YesNo["true"]  = "Yes";
  YesNo["false"] = "No";
  YesNo["0"] = "No";
  YesNo["1"] = "Yes";

  var TrueFalse = new Array();
  TrueFalse["1"] = "true";
  TrueFalse["0"] = "false";

  var RadioTF = new Array();
  RadioTF["0"] = "true";
  RadioTF["1"] = "false";

  var Checked = false;  // Used to know when the Show checkbox is clicked.

  var MiddleWindow;	// Reference to the Middle Frame.
  var TopWindow;	// Reference to the Top Frame.

  // Helper functions

  function DisplayForeGroundColors(fg_color)
  {
    var string = '';

    string += '      <select name="fg_color">\\n';
    for (var color in fg_colors)
    {
      string += '        <option value="'+color+'"'+(fg_color == color ? " selected" : "")+'>' + fg_colors[color] + '\\n';
    }
    string += '      </select>\\n';

    return string;
  }

  function DisplayBackGroundColors(bg_color)
  {
    var string = '';

    string += '      <select name="bg_color">\\n';
    for (var color in bg_colors)
    {
      string += '        <option value="'+color+'"'+(bg_color == color ? " selected" : "")+'>' + bg_colors[color] + '\\n';
    }
    string += '      </select>\\n';

    return string;
  }

  function DisplayBold(bold)
  {
    var string = '';

    string += '      <select name="bold">\\n';
    string += '        <option value="true"'+(bold == "true" ? " selected" : "")+'>Yes\\n';
    string += '        <option value="false"'+(bold == "false" ? " selected" : "")+'>No\\n';
    string += '      </select>\\n';

    return string;
  }

  function serialize_rows()
  {
    var d = document.report_form;

    if (Rows.length > 0)
    {
      var data = "";
      for (i=0; i < Rows.length; i++)
      {
        if (i > 0)
        {
          data += "\\n";  // seperate rows by a new line.
        }
        data += "index="+Rows[i].index+"|cell_type="+Rows[i].cell_type+"|cell_text="+Rows[i].cell_text+"|fg_color="+Rows[i].fg_color+"|bg_color="+Rows[i].bg_color+"|bold="+Rows[i].bold+"|show="+Rows[i].show;
      }
      d.rows.value = data;
    }
    else
    {
      d.rows.value = "";
    }
  }

  function validate_form()
  {
    // Make sure that the user has entered everything that is required.
    var d = document.report_form;

    serialize_rows();

    if (d.rows.value.length == 0)
    {
      alert("You must define at least 1 row for this report!");
      return false;
    }

    // make sure that they have entered a Detail Row
    var i;
    var found=false;
    for (i=0; i < Rows.length; i++)
    {
      if (Rows[i].cell_type == "detail")
      {
        found = true;
      }
    }
    if (!found)
    {
      alert("You must define a 'Detail' Row!");
      return false;
    }

    // alert("Rows = '"+d.rows.value+"'");

    // make sure the windowName is set.
    d.windowName.value = window.parent.name;

    return true;  // change to true when done debugging form.
  }

  function clear_select_box(box)
  {
    box.options.length = 0;
  }

  function create_dummy_select_entry(box)
  {
    box.options[box.length] = new Option("---------------", "---------------");
    box.selectedIndex = box.length - 1;  // Select the latest entry.
  }

  function validate_show_row()
  {
    var d = document.work_form;

    if (d.cell_type.selectedIndex == -1)
    {
      alert("You must select a Row Type first!");
      return;
    }
    var cell_type = d.cell_type.options[d.cell_type.selectedIndex].value;
    var Type_text = d.cell_type.options[d.cell_type.selectedIndex].text;

    if (!d.show.checked)
    {
      if (cell_type == "text" || cell_type == "line" || cell_type == "detail")
      {
        alert("You can not hide a " + Type_text + " Row!");
        d.show.checked = true; // force it to be selected.
      }
    }

    if (Checked)
    {
      Checked = false;
      display_middle_window(EditMode);  // update the displayall of this row.
    }
  }

  function display_middle_window(mode)
  {
    var w = MiddleWindow;
    var body = '';
    var d = document.work_form;
    var cell_type = d.cell_type.options[d.cell_type.selectedIndex].value;
    var visible = d.show.checked;
    var real_index = d.real_index.value;
    var index = d.index.value;

    validate_show_row();  // Check to make sure the visibility setting is valid.

    // do validity checks first.
    // Defaults
    var cell_text, fg_color, bg_color, initial_state, display_level, bold, negative_format;
    if (mode == "edit")
    {
      cell_text = Rows[real_index].cell_text;
      fg_color = Rows[real_index].fg_color;
      bg_color = Rows[real_index].bg_color;
      initial_state = Rows[real_index].initial_state;
      display_level = Rows[real_index].display_level;
      bold = Rows[real_index].bold;
      negative_format = Rows[real_index].negative_format;
    }
    else
    {
      cell_text = "";
      fg_color = "$fg_color";
      bg_color = "$bg_color";
      initial_state = "o";
      display_level = "0";
      bold = "false";
      negative_format = 0;
    }

    body += '<html>\\n';
    body += '<head><title>Report Middle Window</title></head>\\n';
    body += '<body bgcolor="white">\\n';
    body += '<form name="work_form">\\n';
    body += '<center>\\n';
    body += '<table border=1 cellpadding=1 cellspacing=1 align=center>\\n';
    if (cell_type == "line")
    {
      body += '  <tr bgcolor="#cccccc">\\n';
      body += '    <th align=center>There is nothing to Configure.</th>\\n';
      body += '  </tr>\\n';
    }
    else if (cell_type == "text")
    {
      body += '  <tr bgcolor="#cccccc">\\n';
      body += '    <th>Cell Text</th><th>ForeGround Color</th><th>BackGround Color</th><th>Bold</th>\\n';
      body += '  </tr>\\n';
      body += '  <tr bgcolor="#eeeeee">\\n';
      body += '    <td align=center><input type=text name="cell_text" value="'+cell_text+'" onBlur="window.opener.validate_cell_text()"></td>\\n';
      body += '    <td align=center>\\n';
      body += DisplayForeGroundColors(fg_color);
      body += '    </td>\\n';
      body += '    <td align=center>\\n';
      body += DisplayBackGroundColors(bg_color);
      body += '    </td>\\n';
      body += '    <td align=center>\\n';
      body += DisplayBold(bold);
      body += '    </td>\\n';
      body += '  </tr>\\n';
    }
    else if (cell_type == "detail")
    {
      body += '  <tr bgcolor="#cccccc">\\n';
      body += '    <th>ForeGround Color</th><th>BackGround Color</th><th>Bold</th>\\n';
      body += '  </tr>\\n';
      body += '  <tr bgcolor="#eeeeee">\\n';
      body += '    <td align=center>\\n';
      body += DisplayForeGroundColors(fg_color);
      body += '    </td>\\n';
      body += '    <td align=center>\\n';
      body += DisplayBackGroundColors(bg_color);
      body += '    </td>\\n';
      body += '    <td align=center>\\n';
      body += DisplayBold(bold);
      body += '    </td>\\n';
      body += '  </tr>\\n';
    }
    body += '</table>\\n';
    body += '<input type=submit name="add" value="'+(mode == "edit" ? "Update Row" : "Add Row")+'" onClick="window.opener.add_row_to_report(); return false;">&nbsp;<input type=button name="reset_row_button" value="Reset Row" onClick="window.opener.reset_row()">\\n';
    body += '</form>\\n';
    body += '</center>\\n';
    body += '</body>\\n';
    body += '</html>';

    w.document.open();
    w.document.writeln(body);
    w.document.close();

    if (cell_type == "text")
    {
      if (visible)
      {
        w.document.work_form.cell_text.focus();
      }
    }
  }

  function validate_cell_text()
  {
    var d = document.work_form;
    var w = MiddleWindow;
    var d2 = w.document.work_form;

    var index = d.cell_type.selectedIndex;
    if (index == -1)
    {
      alert("You can not enter a value for Row Text without having a Row Type selected!");
      return;
    }
    var value = d.cell_type.options[index].value;
    if (d2.cell_text.value.length == 0)
    {
      return;
    }
  }

  function validate_row()
  {
    var d = document.work_form;
    var w = MiddleWindow;
    var d2 = w.document.work_form;

    var index = d.cell_type.selectedIndex;
    if (index == -1)
    {
      alert("You must select a Row Type!");
      return false;
    }
    var type = d.cell_type.options[index].value;
    if (type == "text")
    {
      if (d2.cell_text.value.length == 0)
      {
        alert("You must specify a value for Row Text!");
        d2.cell_text.focus();
        return false;
      }
    }

    return true;
  }

  function CopyRow(a, b)
  {
    // copy the values from Rows[b] to Rows[a]
    // make sure both a and b are inside the Rows array.
    if (a < 0 || a >= Rows.length)
    {
      alert("Error:  a='"+a+"' is invalid!\\nTrying to CopyRow("+a+","+b+").");
      return;
    }
    if (b < 0 || b >= Rows.length)
    {
      alert("Error:  b='"+b+"' is invalid!\\nTrying to CopyRow("+a+","+b+").");
      return;
    }
    for (var i in Rows[parseInt(b,10)])
    {
      eval("Rows[parseInt(a,10)]."+i+" = Rows[parseInt(b,10)]."+i+";");
    }
  }

  function add_row_to_report()
  {
    var w1 = TopWindow;
    var d = w1.document.work_form;
    var w = MiddleWindow;
    var d2 = w.document.work_form;
    var cell_type = d.cell_type.options[d.cell_type.selectedIndex].value;
    var visible = (d.show.checked ? 1 : 0);

    if (!validate_row())
    {
      return;
    }
    if (EditMode == "create")
    {
      var cell_text = "";
      var fg_color = "$fg_color";
      var bg_color = "$bg_color";
      var bold = "false";

      // make sure that we are not entering multiple Detail rows if this is a Detail row.
      if (cell_type == "detail")
      {
        for (i=0; i < Rows.length; i++)
        {
          if (Rows[i].cell_type == "detail")
          {
            alert("You can not create multiple 'Detail' rows!");

            reset_row();
            update_bottom_window(true);
            return;
          }
        }
        fg_color = d2.fg_color.options[d2.fg_color.selectedIndex].value;
        bg_color = d2.bg_color.options[d2.bg_color.selectedIndex].value;
        bold = d2.bold.options[d2.bold.selectedIndex].value;
      }
      else if (cell_type == "text")
      {
        cell_text = d2.cell_text.value;
        fg_color = d2.fg_color.options[d2.fg_color.selectedIndex].value;
        bg_color = d2.bg_color.options[d2.bg_color.selectedIndex].value;
        bold = d2.bold.options[d2.bold.selectedIndex].value;
      }

      if (d.index.value != Rows.length)
      {
        // They want to insert this row into the array.
        // Make sure it is inside the Array boundary first.
        if (d.index.value < 0 || d.index.value > Rows.length)
        {
          alert("Error:  You tried to insert a Row that is outside the Boundaries!\\nAppending to End.\\n");
          d.index.value = Rows.length;
        }
        else
        {
          // Now we move the rows around and then insert the final row.
          Rows.length++;  // Increase the array by 1.
          Rows[Rows.length-1] = new rowEntry();  // Give it a reference first.
          for (var i=Rows.length-1; i > d.index.value; i--)
          {
            CopyRow(i, parseInt(i, 10)-1);
            Rows[i].index = i;
          }
        }
      }

      Rows[d.index.value] = new rowEntry(d.index.value, cell_type, cell_text, bold, fg_color, bg_color, visible);

      reset_row();
    }
    else  // mode = "edit"
    {
      var cell_text = "";
      var fg_color = "$fg_color";
      var bg_color = "$bg_color";
      var bold = "false";

      // make sure that we are not entering multiple Detail rows if this is a Detail row.
      if (cell_type == "detail")
      {
        for (i=0; i < Rows.length; i++)
        {
          if (Rows[i].cell_type == "detail" && i != d.index.value)
          {
            alert("You can not create multiple 'Detail' rows!");

            reset_row();
            update_bottom_window(true);
            return;
          }
        }
        fg_color = d2.fg_color.options[d2.fg_color.selectedIndex].value;
        bg_color = d2.bg_color.options[d2.bg_color.selectedIndex].value;
        bold = d2.bold.options[d2.bold.selectedIndex].value;
      }
      else if (cell_type == "text")
      {
        cell_text = d2.cell_text.value;
        fg_color = d2.fg_color.options[d2.fg_color.selectedIndex].value;
        bg_color = d2.bg_color.options[d2.bg_color.selectedIndex].value;
        bold = d2.bold.options[d2.bold.selectedIndex].value;
      }

      if (d.index.value != d.real_index.value)
      {
        // They want to move this row around the array.
        // Make sure it is inside the Array boundary first.
        if (d.index.value < 0 || d.index.value >= Rows.length)
        {
          alert("Error:  You tried to move a Row outside the Boundaries!\\nPlacing in original location.\\n");
          d.index.value = d.real_index.value;
        }
        else
        {
          // Now we move the rows around and then insert the final row.
          if (d.index.value > d.real_index.value) // Case 1 - New row > Old row.
          {
            for (var i=d.real_index.value; i < d.index.value; i++)
            {
              CopyRow(i, parseInt(i, 10)+1);
              Rows[i].index = i;
            }
          }
          else  // Case 2 - Old row > New row.
          {
            for (var i=d.real_index.value; i > d.index.value; i--)
            {
              CopyRow(i, parseInt(i, 10)-1);
              Rows[i].index = i;
            }
          }
        }
      }

      Rows[d.index.value] = new rowEntry(d.index.value, cell_type, cell_text, bold, fg_color, bg_color, visible);

      reset_row();
    }
  }

  function signal_edit(i)
  {
    var code = "edit_row("+i+");";
    window.setTimeout(code, 10);
  }

  function signal_delete(i)
  {
    var code = "delete_row("+i+");";
    window.setTimeout(code, 10);
  }

  function edit_row(i)
  {
    // fill in the work_form with the values for the selected Row.
    var d = document.work_form;

    EditMode = "edit";

    d.real_index.value = Rows[i].index;
    d.index.value = Rows[i].index;
    found = false;
    for (j=0; j < d.cell_type.length && !found; j++)
    {
      if (d.cell_type.options[j].value == Rows[i].cell_type)
      {
        d.cell_type.selectedIndex = j;
        found = true;
      }
    }

    display_middle_window(EditMode);  // this will properly propagate the correct items.

    update_bottom_window(false);
  }

  function delete_row(i)
  {
    // remove this row from the Rows array and then refresh the bottom window
    for (j=i+1; j < Rows.length; j++)
    {
      Rows[j-1] = Rows[j];
      Rows[j-1].index = j-1;
    }
    Rows.length--;

    update_bottom_window(true);
  }

  function display_row(d, i, status)
  {
    d.writeln("  <tr bgcolor=\\\"#eeeeee\\\">");
    d.write("    <td align=center>"+Rows[i].index+"</td><td align=center>"+cell_types[Rows[i].cell_type]+"</td><td align=center>");
    if (Rows[i].cell_text.length > 0)
    {
      d.write(Rows[i].cell_text);
    }
    else
    {
      d.write("&nbsp;");
    }
    d.write("</td><td align=center>"+YesNo[Rows[i].bold]+"</td><td align=center>"+fg_colors[Rows[i].fg_color]+"</td><td align=center>"+bg_colors[Rows[i].bg_color]+"</td><td align=center>"+(Rows[i].show ? "True" : "False")+"</td>");
    if (status)
    {
      d.writeln("<td align=center><input type=button name=\\\"edit_"+i+"\\\" value=\\\"edit\\\" onClick=\\\"opener.signal_edit("+i+")\\\"><input type=button name=\\\"delete_"+i+"\\\" value=\\\"delete\\\" onClick=\\\"opener.signal_delete("+i+")\\\"></td>");
    }
    else
    {
      d.writeln("");
    }
    d.writeln("  </tr>");
  }

  function update_bottom_window(status)
  {
    var bottomWindowName = window.parent.name + '_bottom';
    //alert("update_bottom_window status='"+status+"'");
    var w = window.open("", bottomWindowName);
    var d = w.document;

    d.open();
    d.writeln("<html>");
    d.writeln("<body bgcolor=\\\"white\\\">");

    if (Rows.length > 0)
    {
      d.writeln("<form name=\\\"row_form\\\">");
      d.writeln("<table border=1 cellpadding=0 cellspacing=1 align=center>");
      d.writeln("  <tr bgcolor=\\\"#cccccc\\\">");
      d.write("    <th>#</th><th>Type</th><th>Text</th><th>Bold</th><th>ForeGround</th><th>BackGround</th><th>Visible</th>");
      if (status)
      {
        d.writeln("<th>&nbsp;</th>");
      }
      else
      {
        d.writeln("");
      }
      d.writeln("  </tr>");
      // display the first rows values
      display_row(d, 0, status);

      // now display the rest of the rows.
      for (i=1; i < Rows.length; i++)
      {
        var cols = 7;
        if (!status)
        {
          cols = 6;
        }
        d.writeln("  <tr>");
        d.writeln("    <td colspan="+cols+">&nbsp;</td>");
        d.writeln("  </tr>");

        display_row(d, i, status);
      }

      d.writeln("</table>");
      d.writeln("</form>");
    }
    d.writeln("</body>");
    d.writeln("</html>");

    d.close();
  }

  function reset_row()
  {
    var d = document.work_form;
    d.reset();

    if (EditMode == "edit")
    {
      EditMode = "create";
    }
    update_bottom_window(true);

    window.setTimeout("document.work_form.index.value='"+Rows.length+"';document.work_form.real_index.value='"+Rows.length+"';document.work_form.show.checked=1;display_middle_window(EditMode);", 10);
  }

  function reset_report()
  {
    if (window.confirm("Are you sure you want to reset this report?\\nOk=Yes, Cancel=No"))
    {
      initialize_work_form();
    }
  }

  function initialize_work_form()
  {
    Rows.length = 0;

END_OF_CODE

  # gather all rows from the database and populate the Rows array with them.
  my @rows = GetReportRows(dbh => $dbh, id => $id);

  for (my $counter = 0; $counter < scalar @rows; $counter++)
  {
    my %info = %{$rows[$counter]};
    my $cell_type = $info{'cell_type'};
    $doc->print("    Rows[$counter] = new rowEntry(\"$info{'index'}\", \"$info{'cell_type'}\", \"$info{'cell_text'}\", \"$t_or_f{$info{'bold'}}\", \"$info{'fg_color'}\", \"$info{'bg_color'}\", $t_or_f{$info{'show'}});\n");
  }

  $doc->print(<<"END_OF_CODE");
    var middleWindowName = window.parent.name + '_middle';
    var w = window.open("", middleWindowName);
    MiddleWindow = w;

    var topWindowName = window.parent.name + '_top';
    var x = window.open("", topWindowName);
    TopWindow = x;

    window.setTimeout('reset_row();', 45);

    //window.setTimeout('update_bottom_window(true);', 30);

    document.report_form.windowName.value = window.parent.name;
  }
END_OF_CODE
  }

  # generate the javascript code to be run when the page has been loaded.
  my $on_load_code = $doc->{javascriptOnload};  # make sure to preserve anything already specified.

  $on_load_code .= <<"END_OF_CODE";
  initialize_work_form();
END_OF_CODE

  $doc->setOnload(code => $on_load_code);
  $doc->display();
}

# parse_formula - validates the specified formula and returns the tokens that make up the formula.
# Takes  : formula string
# Returns: Error string and token array.
sub parse_formula
{
  my %args = ( @_, );
  my $formula = $args{'formula'};
  my $result = "";

  if (length $formula == 0)
  {
    my_die("Error:  Report.cgi - parse_formula", 4035, "formula must be specified!");
  }

  my @elements = ($formula =~ m/(\(|R\d+\.\d+|R\d+|-?\d+\.\d+|-?\d+|\+|-|\*|\/|\)|\s+|.*)/g);
  my $i;

  # get rid of any empty or whitespace related elements.
  for ($i=0; $i < scalar @elements; $i++)
  {
    if (length $elements[$i] == 0 || $elements[$i] =~ /^\s+$/)
    {
      for (my $j=$i+1; $j < scalar @elements; $j++)
      {
        $elements[$j-1] = $elements[$j];
      }
      $#elements--;  # shrink the array by 1.
    }
  }

  if (scalar @elements == 0)
  {
    $result = "Error:  The formula didn't split out any tokens!\n";
    return ($result, @elements);
  }

  my $open_parens = 0;
  for ($i=0; $i < scalar @elements; $i++)
  {
    if ($i == 0) # first element only.
    { # make sure we are not starting out with an operator
      if ($elements[$i] =~ /$operatorsRE/)
      {
        $result = "Error:  Formula can not start with an operator!\n";
        return ($result, @elements);
      }
      elsif ($elements[$i] =~ /$openParenRE/)
      {
        $open_parens++;
      }
      elsif ($elements[$i] =~ /$closeParenRE/)
      {
        $result = "Error:  You can not start the Formula with an )!\n";
        return ($result, @elements);
      }
      elsif ($elements[$i] !~ /$tokenRE/)
      {
        $result = "Error:  Invalid token!  Token $i, '$elements[$i]'.\n";
        return ($result, @elements);
      }
    }
    else  # the rest of the elements.
    { # make sure that the last token was a row, number, or ) if we have an operator
      # else if we have a row make sure the last token was an (
      # and not another row.
      if ($elements[$i] =~ /$operatorsRE/)
      {
        if ($elements[$i-1] !~ /$tokenRE/ && $elements[$i-1] !~ /$closeParenRE/)
        {
          $result = "Error:  Operator can not come after another Operator or (!  Token $i, '$elements[$i-1]' and '$elements[$i]'.\n";
          return ($result, @elements);
        }
      }
      elsif ($elements[$i] =~ /$openParenRE/)
      {
        $open_parens++;
        # make sure that the previous token was an ( or an operator.
        if ($elements[$i-1] !~ /$openParenRE/ && $elements[$i-1] !~ /$operatorsRE/)
        {
          $result = "Error:  ( can not come after a Row, Number, or )!  Token $i, '$elements[$i-1]' and '$elements[$i]'.\n";
          return ($result, @elements);
        }
      }
      elsif ($elements[$i] =~ /$closeParenRE/)
      {
        if ($open_parens > 0)
        {
          $open_parens--;
        }
        else
        {
          $result = "Error:  Formula is invalid at token $i, '$elements[$i]'.  Close Paren when no Open Paren to match with!\n";
          return ($result, @elements);
        }

        # make sure that we are not following a ( or an operator.
        if ($elements[$i-1] =~ /$openParenRE/ || $elements[$i-1] =~ /$operatorsRE/)
        {
          $result = "Error:  ) can not come after a ( or operator!  Token $i, '$elements[$i-1]' and '$elements[$i]'.\n";
          return ($result, @elements);
        }
      }
      elsif ($elements[$i] =~ /$rowIdentifierRE/ && $elements[$i-1] =~ /^-?\d+\.\d+|-?\d+$/)
      {
        $result = "Error:  You can not have a Number and a Row back to back without an intervening operator! Token $i, '$elements[$i-1]' and '$elements[$i]'.\n";
        return ($result, @elements);
      }
      elsif ($elements[$i] =~ /^-\d+\.\d+|-\d+$/ && $elements[$i-1] =~ /$tokenRE/)
      {
        $result = "Error:  You can not have Two Numbers or a Row and a Number back to back without an intervening operator!  Token $i, '$elements[$i-1]' and '$elements[$i]'.\n";
        return ($result, @elements);
      }
      elsif ($elements[$i] =~ /$tokenRE/ && $elements[$i-1] !~ /$rowIdentifierRE/)
      {
        if ($elements[$i] =~ /^R\d+\.\d+$/)
        {
          $result = "Error:  Invalid token!  Token $i, '$elements[$i]'.\n";
          return ($result, @elements);
        }
      }
      elsif ($elements[$i] =~ /$rowIdentifierRE/ && $elements[$i-1] =~ /$rowIdentifierRE/)
      {
        $result = "Error:  You can not have 2 Rows back to back without an intervening operator! Token $i, '$elements[$i-1]' and '$elements[$i]'.\n";
        return ($result, @elements);
      }
      else
      {
        $result = "Error:  Invalid token!  Token $i, '$elements[$i]'.\n";
        return ($result, @elements);
      }
    }
  }

  # make sure the last token wasn't an operator.
  if ($elements[$i-1] =~ /$operatorsRE/)
  {
    $result = "Error:  The Formula can not end with an operator!  Token ${\($i-1)}, '$elements[${\($i-1)}]'.\n";
    return ($result, @elements);
  }

  # make sure we have all ()'s matched up.
  if ($open_parens > 0)
  {
    $result = "Error:  You have unmatched ('s!\n";
    return ($result, @elements);
  }

  return ($result, [ @elements ]);  # signal it passed.
}

# takes Title, Error Code, Custom Message
sub my_die
{
  my ($title, $code, $message) = @_;

  # close the database connection.
  #if (defined $dbh)
  #{
  #  $dbh->disconnect;
  #}

  my $doc = HTMLObject::Base->new();
  $doc->setErrorMessage(code => $code, message => $errors{$code});
  $doc->setError(code => $code);
  $doc->displayError(title => $title, message => "$message");
}

# takes Title, Error Code, Custom Message
sub my_db_die
{
  my ($title, $code, $message) = @_;
  my $error_str = $DBI::errstr;

  # close the database connection.
  #if (defined $dbh)
  #{
  #  $dbh->disconnect;
  #}

  my $doc = HTMLObject::Base->new();
  $doc->setErrorMessage(code => $code, message => $errors{$code});
  $doc->setError(code => $code);
  $doc->displayError(title => $title, message => "$message<br><br>\n$error_str");
}

###################### Data Gathering Functions converted from reports.pm to DBI structure #############################

# get_ledger_name
# Takes  : ledger_db name, handle to database connection
# Returns: ledger name
sub get_ledger_name
{
  my %args = ( @_, );
  my $ledger_db = $args{'ledger_db'};
  my $dbh = $args{'dbh'};
  my $ledger_name = "";

  eval ("use HTMLObject::Normal;");
  if ($@)
  {
    my_die("Error: Report.cgi - Eval", 4002, "Error: $@ at line " . __LINE__);
  }

  # check to make sure we were given the ledger_db name and database handle
  if (length $ledger_db == 0)
  {
    my_die("Error: Report.cgi - get_ledger_name", 4026, "The Ledger DB Name is required at line " . __LINE__);
  }

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - get_ledger_name", 4028, "The DB Handle is required at line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1016} ) || my_db_die("Error: Report.cgi - get_ledger_name", 4007, "SQL Statement = '$sql_statements{1016}'");
  my $rc = $sth->execute($ledger_db) || my_db_die("Error: Report.cgi - get_ledger_name", 4008, "SQL Statement = '$sql_statements{1016}', Ledger DB = '$ledger_db'");
  my @info = $sth->fetchrow_array;

  if (!defined $info[0])
  {
    my_die("Error: Report.cgi - get_ledger_name", 4023, "Ledger DB = '$ledger_db'");
  }
  else
  {
    $ledger_name = $info[0];
  }
  $sth->finish;

  return $ledger_name;
}

# get_ledger_code
# Takes  : ledger_db name, handle to database connection
# Returns: ledger code
sub get_ledger_code
{
  my %args = ( @_, );
  my $ledger_name = $args{'ledger_name'};
  my $dbh = $args{'dbh'};
  my $ledger_code = "";

  # check to make sure we were given the ledger name and database handle
  if (length $ledger_name == 0)
  {
    my_die("Error: Report.cgi - get_ledger_code", 4027, "The Ledger Name is required at line " . __LINE__);
  }

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - get_ledger_code", 4028, "The DB Handle is required at line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1017} ) || my_db_die("Error: Report.cgi - get_ledger_code", 4007, "SQL Statement = '$sql_statements{1017}'");
  my $rc = $sth->execute($ledger_name) || my_db_die("Error: Report.cgi - get_ledger_code", 4008, "SQL Statement = '$sql_statements{1017}', Ledger Name = '$ledger_name'");
  my @info = $sth->fetchrow_array;

  if (!defined $info[0])
  {
    my_die("Error: Report.cgi - get_ledger_code", 4024, "Ledger Name = '$ledger_name'");
  }
  else
  {
    $ledger_code = $info[0];
  }
  $sth->finish;

  return $ledger_code;
}

# get_ledger_color
# Takes  : ledger_db name, handle to database connection
# Returns: ledger color
sub get_ledger_color
{
  my %args = ( @_, );
  my $ledger_code = $args{'ledger_code'};
  my $dbh = $args{'dbh'};
  my $ledger_color = "";

  # check to make sure we were given the ledger code and database handle
  if (length $ledger_code == 0)
  {
    my_die("Error: Report.cgi - get_ledger_color", 4022, "The Ledger Code is required at line " . __LINE__);
  }

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - get_ledger_color", 4028, "The DB Handle is required at line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1018} ) || my_db_die("Error: Report.cgi - get_ledger_color", 4007, "SQL Statement = '$sql_statements{1018}'");
  my $rc = $sth->execute($ledger_code) || my_db_die("Error: Report.cgi - get_ledger_color", 4008, "SQL Statement = '$sql_statements{1018}', Ledger Code = '$ledger_code'");
  my @info = $sth->fetchrow_array;

  if (!defined $info[0])
  {
    my_die("Error: Report.cgi - get_ledger_color", 4025, "Ledger Name = '$ledger_code'");
  }
  else
  {
    $ledger_color = $info[0];
  }
  $sth->finish;

  return $ledger_color;
}

# Takes  : dbh, id of report
# Returns: "" if not found, report name if found
sub DoesReportExist
{
  my %args = ( @_ );
  my $dbh = $args{'dbh'};
  my $id = $args{'id'};
  my $result = "";  # assume it is not found.

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - DoesReportExist", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $id == 0)
  {
    my_die("Error: Report.cgi - DoesReportExist", 4034, "The Report ID is required at line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1005} ) || my_db_die("Error: Report.cgi - DoesReportExist", 4007, "SQL Statement = '$sql_statements{1005}'");
  my $rc = $sth->execute($id) || my_db_die("Error: Report.cgi - DoesReportExist", 4008, "SQL Statement = '$sql_statements{1005}', id = '$id'");
  my @info = $sth->fetchrow_array;

  if (defined $info[0])
  {
    $result = $info[0];
  }
  $sth->finish;

  return $result;
}

# Takes  : dbh, name of report to find
# Returns: -1 if not found, id otherwise
sub FindReportByName
{
  my %args = ( @_ );
  my $dbh = $args{'dbh'};
  my $name = $args{'name'};
  my $result = -1;  # assume it is not found.

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - FindReportByName", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $name == 0)
  {
    my_die("Error: Report.cgi - FindReportByName", 4034, "The Report Name is required at line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1003} ) || my_db_die("Error: Report.cgi - FindReportByName", 4007, "SQL Statement = '$sql_statements{1003}'");
  my $rc = $sth->execute($name) || my_db_die("Error: Report.cgi - FindReportByName", 4008, "SQL Statement = '$sql_statements{1003}', name = '$name'");
  my @info = $sth->fetchrow_array;

  if (defined $info[0])
  {
    $result = $info[0];
  }
  $sth->finish;

  return $result;
}

# Takes  : dbh, id of report
# Returns: array with the report_type_tb entry values.
sub GetReportData
{
  my %args = ( @_ );
  my $dbh = $args{'dbh'};
  my $id = $args{'id'};

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetReportData", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $id == 0)
  {
    my_die("Error: Report.cgi - GetReportData", 4034, "The Report ID is required at line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1006} ) || my_db_die("Error: Report.cgi - GetReportData", 4007, "SQL Statement = '$sql_statements{1006}'");
  my $rc = $sth->execute($id) || my_db_die("Error: Report.cgi - GetReportData", 4008, "SQL Statement = '$sql_statements{1006}', id = '$id'");
  my @info = $sth->fetchrow_array;

  $sth->finish;

  return @info;
}

# Takes  : dbh, id of report
# Returns: array of arrays where each sub-array is the data for a row in the specified report.
sub GetReportRows
{
  my %args = ( @_ );
  my $dbh = $args{'dbh'};
  my $id = $args{'id'};

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetReportRows", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $id == 0)
  {
    my_die("Error: Report.cgi - GetReportRows", 4034, "The Report ID is required at line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1024} ) || my_db_die("Error: Report.cgi - GetReportRows", 4007, "SQL Statement = '$sql_statements{1024}'");
  my $rc = $sth->execute($id) || my_db_die("Error: Report.cgi - GetReportRows", 4008, "SQL Statement = '$sql_statements{1024}', id = '$id'");
  my $ref = $sth->fetchall_arrayref;
  if ($sth->err)
  {
    my_db_die("Error:  Report.cgi - GetReportRows", 4009, "Getting all Account Types failed!");
  }
  my @info = @{$ref};

  $sth->finish;

  # now walk the array and make hashes for each row that detail the items and their values.
  my @temp_array = ();
  for (my $i=0; $i < scalar @info; $i++)
  {
    $temp_array[$i] = ();  # make an empty array placeholder.
    my %temp_hash = {};
    $temp_hash{'index'} = $info[$i][0];
    $temp_hash{'cell_text'} = $info[$i][1];
    $temp_hash{'cell_type'} = $info[$i][2];
    $temp_hash{'item_name'} = $info[$i][3];
    $temp_hash{'fg_color'} = $info[$i][4];
    $temp_hash{'bg_color'} = $info[$i][5];
    $temp_hash{'display_level'} = $info[$i][6];
    $temp_hash{'initial_state'} = $info[$i][7];
    $temp_hash{'negative_format'} = $info[$i][8];
    my $other_attributes = $info[$i][9];
    $temp_hash{'other_attributes'} = $other_attributes;
    my @other_attributes = split /\|/, $other_attributes;
    foreach my $item (@other_attributes)
    {
      (my $name, my $value) = split /=/, $item, 2;
      $temp_hash{$name} = $value;
    }

    my $cell_type = $temp_hash{'cell_type'};
    if ($cell_type eq "text" || $cell_type eq "line" || $cell_type eq "detail")
    {
      $temp_hash{'cell_formula'} = "";
      $temp_hash{'sub_levels'} = "-1";
      $temp_hash{'drillable'} = 0;
      $temp_hash{'drill_report'} = -1;
      if ($cell_type ne "detail")
      {
        $temp_hash{'activity'} = 1;  # activity
      }
      else
      {
        $temp_hash{'activity'} = 0;  # balances
      }
      $temp_hash{'query_type'} = 0;  # everything
      $temp_hash{'show'} = 1;  # show this row.
      $temp_hash{'underline'} = 0; # don't need to underline the previous row.
    }
    elsif ($cell_type eq "total" || $cell_type eq "compare")
    {
      $temp_hash{'sub_levels'} = "-1";
      $temp_hash{'drillable'} = 0;
      $temp_hash{'drill_report'} = -1;
      $temp_hash{'activity'} = 1;  # activity
      $temp_hash{'query_type'} = 0;  # everything
      if (!exists $temp_hash{'underline'})
      {
        if ($cell_type eq "total")
        {
          $temp_hash{'underline'} = 1;  # Default to underline the previous row.
        }
        elsif ($cell_type eq "compare")
        {
          $temp_hash{'underline'} = 0;  # Default to not underline the previous row.
        }
      }
    }
    else
    {
      $temp_hash{'underline'} = 0;
    }
    $temp_array[$i] = { %temp_hash };
  }

  return @temp_array;
}

# Takes  : dbh, id of report (I'm assuming the report exists!)
# Returns: 1 if editable, 0 if not
sub IsReportEditable
{
  my %args = ( @_ );
  my $dbh = $args{'dbh'};
  my $id = $args{'id'};
  my $result = 0;  # assume it is not editable.

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - IsReportEditable", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $id == 0)
  {
    my_die("Error: Report.cgi - IsReportEditable", 4034, "The Report ID is required at line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1038} ) || my_db_die("Error: Report.cgi - IsReportEditable", 4007, "SQL Statement = '$sql_statements{1038}'");
  my $rc = $sth->execute($id) || my_db_die("Error: Report.cgi - IsReportEditable", 4008, "SQL Statement = '$sql_statements{1038}', id = '$id'");
  my @info = $sth->fetchrow_array;

  $result = $info[0];
  $sth->finish;

  return $result;
}

# Takes  : dbh, id of report
# Returns: report_type
sub TypeOfReport
{
  my %args = ( @_ );
  my $dbh = $args{'dbh'};
  my $id = $args{'id'};
  my $result;

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - TypeOfReport", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $id == 0)
  {
    my_die("Error: Report.cgi - TypeOfReport", 4034, "The Report ID is required at line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1064} ) || my_db_die("Error: Report.cgi - TypeOfReport", 4007, "SQL Statement = '$sql_statements{1064}'");
  my $rc = $sth->execute($id) || my_db_die("Error: Report.cgi - TypeOfReport", 4008, "SQL Statement = '$sql_statements{1064}', id = '$id'");
  my @info = $sth->fetchrow_array;

  $result = $info[0];
  $sth->finish;

  return $result;
}

# Takes  : dbh, id of report, report_name, ledger_code
# Returns: 0 if not found, 1 if found
sub DoesMemorizedReportExist
{
  my %args = ( @_ );
  my $dbh = $args{'dbh'};
  my $id = $args{'id'};
  my $report_name = $args{'report_name'};
  my $ledger_code = $args{'ledger_code'};
  my $result = 0;  # assume it is not found.

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - DoesMemorizedReportExist", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $id == 0)
  {
    my_die("Error: Report.cgi - DoesMemorizedReportExist", 4034, "The Report ID is required at line " . __LINE__);
  }

  if (length $report_name == 0)
  {
    my_die("Error: Report.cgi - DoesMemorizedReportExist", 4034, "The Report Name is required at line " . __LINE__);
  }

  if (length $ledger_code == 0)
  {
    my_die("Error: Report.cgi - DoesMemorizedReportExist", 4022, "The Ledger Code is required at line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1037} ) || my_db_die("Error: Report.cgi - DoesMemorizedReportExist", 4007, "SQL Statement = '$sql_statements{1037}'");
  my $rc = $sth->execute($id, $report_name, $ledger_code) || my_db_die("Error: Report.cgi - DoesMemorizedReportExist", 4008, "SQL Statement = '$sql_statements{1037}', id = '$id', name = '$report_name', ledger_code = '$ledger_code'");
  my @info = $sth->fetchrow_array;

  if (defined $info[0])
  {
    $result = 1;
  }
  $sth->finish;

  return $result;
}

# GetAccountTypes
# Takes  : dbh, first_digit (optional)
# Returns: Array of Account Types numerically sorted.
sub GetAccountTypes
{
  my %args = ( @_ );
  my $dbh = $args{'dbh'};
  my @account_types = ();
  my $first_digit = $args{'first_digit'};

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetAccountTypes", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (exists $args{'first_digit'} && ($first_digit < 1 || $first_digit > 5))
  {
    my_die("Error:  Report.cgi - GetAccountTypes", 4034, "first_digit = '$first_digit' is invalid!<br>At Line " . __LINE__);
  }

  my $tmp_sql_statement = $sql_statements{1049};
  if (exists $args{'first_digit'})
  {
    $tmp_sql_statement =~ s/$substitutionRE/WHERE first_digit = '$first_digit'/;
  }
  else
  {
    $tmp_sql_statement =~ s/$substitutionRE//;
  }
  my $sth = $dbh->prepare( $tmp_sql_statement ) || my_db_die("Error: Report.cgi - GetAccountTypes", 4007, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);
  my $rc = $sth->execute || my_db_die("Error: Report.cgi - GetAccountTypes", 4008, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);
  my $info_ref = $sth->fetchall_arrayref;
  @account_types = @{$info_ref};
  $sth->finish;

  if (scalar @account_types == 0)
  {
    my_db_die("Error:  Report.cgi - GetAccountTypes", 4009, "No Account Types returned!<br>\nAt Line " . __LINE__);
  }

  for (my $i=0; $i < scalar @account_types; $i++)
  {
    $account_types[$i] = $account_types[$i][0];  # get rid of the sub-array stuff.
  }

  return @account_types;
}

# GetAccountTypeNames
# Takes  : Array of Account Types (account_types), dbh
# Returns: Array of Account Type Names in order given.
sub GetAccountTypeNames
{
  my %args = ( @_ );
  my $dbh = $args{'dbh'};
  my @account_types = @{$args{'account_types'}};
  my @account_type_names = ();

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetAccountTypeNames", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (scalar @account_types == 0)
  {
    my_die("Error: Report.cgi - GetAccountTypeNames", 4034, "The Array of Account Types is required at line " . __LINE__);
  }

  foreach my $account_type (@account_types)
  {
    my $sth = $dbh->prepare( $sql_statements{1042} ) || my_db_die("Error: Report.cgi - GetAccountTypeNames", 4007, "SQL Statement = '$sql_statements{1042}'<br>\nAt Line " . __LINE__);
    my $rc = $sth->execute($account_type) || my_db_die("Error: Report.cgi - GetAccountTypeNames", 4008, "SQL Statement = '$sql_statements{1042}', Account Type = '$account_type'<br>\nAt Line " . __LINE__);
    my @info = $sth->fetchrow_array;

    if (!defined $info[0])
    {
      my_db_die("Error:  Report.cgi - GetAccountTypeNames", 4009, "No Account Type Name returned for Account Type = '$account_type'!<br>\nAt Line " . __LINE__);
    }
    $sth->finish;
    push(@account_type_names, $info[0]);
  }

  return @account_type_names;
}

# GetAccountType
# Takes  : Account Number (acct_number), dbh
# Returns: Account Type Number associated with the Account Number, debit_increases flag
sub GetAccountType
{
  my %args = ( @_ );
  my $dbh = $args{'dbh'};
  my $acct_number = $args{'acct_number'};
  my @acct_type_info = ();

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetAccountType", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $acct_number == 0)
  {
    my_die("Error: Report.cgi - GetAccountType", 4034, "The Account Number is required at line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1056} ) || my_db_die("Error: Report.cgi - GetAccountType", 4007, "SQL Statement = '$sql_statements{1056}'<br>\nAt Line " . __LINE__);
  my $rc = $sth->execute($acct_number) || my_db_die("Error: Report.cgi - GetAccountType", 4008, "SQL Statement = '$sql_statements{1056}', Account = '$acct_number'<br>\nAt Line " . __LINE__);
  my @info = $sth->fetchrow_array;
  $sth->finish;

  if (!defined $info[0])
  {
    my_db_die("Error:  Report.cgi - GetAccountType", 4009, "No Account Type returned for Account = '$acct_number'!<br>\nAt Line " . __LINE__);
  }
  $acct_type_info[0] = $info[0];
  $acct_type_info[1] = $info[1];

  return @acct_type_info;
}

# GetAccountName
# Takes  : Account Number (acct_number), dbh
# Returns: Account Name
sub GetAccountName
{
  my %args = ( @_ );
  my $dbh = $args{'dbh'};
  my $acct_number = $args{'acct_number'};
  my $acct_name = "";

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetAccountName", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $acct_number == 0)
  {
    my_die("Error: Report.cgi - GetAccountName", 4034, "The Account Number is required at line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1048} ) || my_db_die("Error: Report.cgi - GetAccountName", 4007, "SQL Statement = '$sql_statements{1048}'<br>\nAt Line " . __LINE__);
  my $rc = $sth->execute($acct_number) || my_db_die("Error: Report.cgi - GetAccountName", 4008, "SQL Statement = '$sql_statements{1048}', Account = '$acct_number'<br>\nAt Line " . __LINE__);
  my @info = $sth->fetchrow_array;
  $sth->finish;

  if (!defined $info[0])
  {
    my_db_die("Error:  Report.cgi - GetAccountName", 4009, "No Account Name returned for Account = '$acct_number'!<br>\nAt Line " . __LINE__);
  }
  $acct_name = $info[0];

  return $acct_name;
}

# GetSubAccountName
# Takes  : Sub Account Number (sub_acct_number), dbh
# Returns: Sub Account Name
sub GetSubAccountName
{
  my %args = ( @_ );
  my $dbh = $args{'dbh'};
  my $sub_acct_number = $args{'sub_acct_number'};
  my $sub_acct_name = "";

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetSubAccountName", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $sub_acct_number == 0)
  {
    my_die("Error: Report.cgi - GetSubAccountName", 4034, "The Sub Account Number is required at line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1057} ) || my_db_die("Error: Report.cgi - GetSubAccountName", 4007, "SQL Statement = '$sql_statements{1057}'<br>\nAt Line " . __LINE__);
  my $rc = $sth->execute($sub_acct_number) || my_db_die("Error: Report.cgi - GetSubAccountName", 4008, "SQL Statement = '$sql_statements{1057}', Sub Account = '$sub_acct_number'<br>\nAt Line " . __LINE__);
  my @info = $sth->fetchrow_array;
  $sth->finish;

  if (!defined $info[0])
  {
    my_db_die("Error:  Report.cgi - GetSubAccountName", 4009, "No Account Name returned for Account = '$sub_acct_number'!<br>\nAt Line " . __LINE__);
  }
  $sub_acct_name = $info[0];

  return $sub_acct_name;
}

# GetLocalAccounts
# Takes  : Array of Account Types (account_types), dbh, ledger_code
# Takes  : Taxable Flag (Optional, if not specified assumed to be 0) (0=don't care, 1=taxable only, 2=non-taxable only)
# Returns: Array of Local Accounts for each Account Type with the Debit Increases Flag for each Account Type and the Account Name
#          Array [AccountType]
#          [AccountType] = [LocalAccounts]
#          [LocalAccounts] = Account Number, Debit Increases Flag, Account Name
sub GetLocalAccounts
{
  my %args = ( taxable => '0', @_ );
  my $dbh = $args{'dbh'};
  my @account_types = @{$args{'account_types'}};
  my $taxable = $args{'taxable'};
  my $ledger_code = $args{'ledger_code'};
  my @local_accounts = ();

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetLocalAccounts", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (scalar @account_types == 0)
  {
    my_die("Error: Report.cgi - GetLocalAccounts", 4034, "The Array of Account Types is required at line " . __LINE__);
  }

  if ($taxable < 0 || $taxable > 2)
  {
    my_die("Error:  Report.cgi - GetLocalAccounts", 4034, "The Taxable flag can only be 0, 1, or 2!&nbsp;&nbsp;'$taxable' is invalid!<br>\nAt Line " . __LINE__);
  }

  if (length $ledger_code == 0)
  {
    my_die("Error:  Report.cgi - GetLocalAccounts", 4034, "The ledger_code must be specified!<br>\nAt Line " . __LINE__);
  }

  for(my $i=0; $i < scalar @account_types; $i++)
  {
    my $account_type = $account_types[$i];
    my @Info = ();

    my $sth = $dbh->prepare( $sql_statements{1043} ) || my_db_die("Error: Report.cgi - GetLocalAccounts", 4007, "SQL Statement = '$sql_statements{1043}'<br>\nAt Line " . __LINE__);
    my $rc = $sth->execute($account_type) || my_db_die("Error: Report.cgi - GetLocalAccounts", 4008, "SQL Statement = '$sql_statements{1043}', Account Type = '$account_type'<br>\nAt Line " . __LINE__);
    my @info = $sth->fetchrow_array;
    $sth->finish;

    if (!defined $info[0])
    {
      my_db_die("Error:  Report.cgi - GetLocalAccounts", 4009, "No Debit Increases Flag returned for Account Type = '$account_type'!<br>\nAt Line " . __LINE__);
    }
    my $debit_increases = $info[0];

    $sth = $dbh->prepare( $sql_statements{1044} ) || my_db_die("Error: Report.cgi - GetLocalAccounts", 4007, "SQL Statement = '$sql_statements{1044}'<br>\nAt Line " . __LINE__);
    $rc = $sth->execute($account_type) || my_db_die("Error: Report.cgi - GetLocalAccounts", 4008, "SQL Statement = '$sql_statements{1044}', Account Type = '$account_type'<br>\nAt Line " . __LINE__);
    @info = @{$sth->fetchall_arrayref};
    $sth->finish;

    if (scalar @info > 0)
    {
      my ($acct_num_list, $j, $acct_number, $acct_name);
      for ($j=0; $j < scalar @info; $j++)
      {
        $acct_number = $info[$j][0];

        $acct_num_list .= ", " if ($j > 0);
        $acct_num_list .= $acct_number;
      }

      # now get the accounts that exist in the local_accounts_tb that match up
      if ($taxable == 1 && $account_type != 15)
      {
        my $tmp_sql_statement;
        ($tmp_sql_statement = $sql_statements{1045}) =~ s/$substitutionRE/$acct_num_list/;
        $sth = $dbh->prepare( $tmp_sql_statement ) || my_db_die("Error: Report.cgi - GetLocalAccounts", 4007, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);
        $rc = $sth->execute($ledger_code) || my_db_die("Error: Report.cgi - GetLocalAccounts", 4008, "SQL Statement = '$tmp_sql_statement', Ledger Code = '$ledger_code', Account Num In '$acct_num_list'<br>\nAt Line " . __LINE__);
      }
      elsif ($taxable == 2)
      {
        my $tmp_sql_statement;
        ($tmp_sql_statement = $sql_statements{1046}) =~ s/$substitutionRE/$acct_num_list/;
        $sth = $dbh->prepare( $tmp_sql_statement ) || my_db_die("Error: Report.cgi - GetLocalAccounts", 4007, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);
        $rc = $sth->execute($ledger_code) || my_db_die("Error: Report.cgi - GetLocalAccounts", 4008, "SQL Statement = '$tmp_sql_statement', Ledger Code = '$ledger_code', Account Num In '$acct_num_list'<br>\nAt Line " . __LINE__);
      }
      else
      {
        my $tmp_sql_statement;
        ($tmp_sql_statement = $sql_statements{1047}) =~ s/$substitutionRE/$acct_num_list/;
        $sth = $dbh->prepare( $tmp_sql_statement ) || my_db_die("Error: Report.cgi - GetLocalAccounts", 4007, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);
        $rc = $sth->execute($ledger_code) || my_db_die("Error: Report.cgi - GetLocalAccounts", 4008, "SQL Statement = '$tmp_sql_statement', Ledger Code = '$ledger_code', Account Num In '$acct_num_list'<br>\nAt Line " . __LINE__);
      }
      my @accounts = @{$sth->fetchall_arrayref};
      $sth->finish;

      # now we walk the list and create the array that will make up this Account Type's entry
      for ($j=0; $j < scalar @accounts; $j++)
      {
        my @element = ();
        $acct_number = $accounts[$j][0];

        # get this accounts name
        $acct_name = GetAccountName(dbh => $dbh, acct_number => $acct_number);

        $element[0] = $acct_number;
        $element[1] = $debit_increases;
        $element[2] = $acct_name;

        $Info[$j] = [ @element ];
      }
    }

    $local_accounts[$i] = [ @Info ];
  }

  return @local_accounts;
}

# GetSubAccountTypeName
# Takes  : sub_account_type, dbh
# Returns: Sub Account Type Name
sub GetSubAccountTypeName
{
  my %args = ( order => '0', @_ );
  my $dbh = $args{'dbh'};
  my $sub_account_type = $args{'sub_account_type'};
  my $sub_account_name = "";

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetSubAccountTypeName", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $sub_account_type == 0)
  {
    my_die("Error: Report.cgi - GetSubAccountTypeName", 4034, "The Sub Account Type is required at line " . __LINE__);
  }

  if ($sub_account_type == -1)
  {
    $sub_account_name = "<i>None</i>";
  }
  else
  {
    my $sth = $dbh->prepare( $sql_statements{1058} ) || my_db_die("Error: Report.cgi - GetSubAccountTypeName", 4007, "SQL Statement = '$sql_statements{1058}'<br>\nAt Line " . __LINE__);
    my $rc = $sth->execute($sub_account_type) || my_db_die("Error: Report.cgi - GetSubAccountTypeName", 4008, "SQL Statement = '$sql_statements{1058}', Sub Account Type = '$sub_account_type'<br>\nAt Line " . __LINE__);
    my @info = $sth->fetchrow_array;
    $sth->finish;

    if (!defined $info[0])
    {
      my_db_die("Error:  Report.cgi - GetSubAccountTypeName", 4009, "No Account Name returned for Account = '$sub_account_type'!<br>\nAt Line " . __LINE__);
    }
    $sub_account_name = $info[0];
  }

  return $sub_account_name;
}

# GetSubAccountTypes
# Takes  : account_type, dbh
# Returns: Array of sub-account types assigned to the account type ordered number first
#          The first element will contain the number, the second element will contain the name
sub GetSubAccountTypes
{
  my %args = ( order => '0', @_ );
  my $dbh = $args{'dbh'};
  my $account_type = $args{'account_type'};

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetSubAccountTypes", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $account_type == 0)
  {
    my_die("Error: Report.cgi - GetSubAccountTypes", 4034, "The Account Type is required at line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1055} ) || my_db_die("Error: Report.cgi - GetSubAccountTypes", 4007, "SQL Statement = '$sql_statements{1055}'<br>\nAt Line " . __LINE__);
  my $rc = $sth->execute($account_type) || my_db_die("Error: Report.cgi - GetSubAccountTypes", 4008, "SQL Statement = '$sql_statements{1055}', Account Type = '$account_type'<br>\nAt Line " . __LINE__);
  my @temp_array = $sth->fetchrow_array;
  $sth->finish;

  my @info = sort {$a <=> $b} split(/,/, $temp_array[0]);
  my @name_num_array = ();
  # get the names for each sub-account type
  for (my $count=0; $count < scalar @info; $count++)
  {
    $name_num_array[$count] = ();
    $name_num_array[$count][0] = $info[$count];
    $name_num_array[$count][1] = GetSubAccountTypeName(dbh => $dbh, sub_account_type => $info[$count]);
  }

  return @name_num_array;
}

# GetSubAccounts
# Takes   : account_type, dbh
# Optional: order (0=number, 1=name)
# Returns : Array of sub-accounts ordered number first unless overridden by the order argument
#	    The first element of the array will contain the number of the sub-account, the second element will contain the
#           name.
sub GetSubAccounts
{
  my %args = ( order => '0', @_ );
  my $dbh = $args{'dbh'};
  my $account_type = $args{'account_type'};
  my $order = $args{'order'};

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetSubAccounts", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if ($order < 0 || $order > 1)
  {
    my_die("Error:  Report.cgi - GetSubAccounts", 4034, "The Order flag can only be 0 or 1!&nbsp;&nbsp;'$order' is invalid!<br>\nAt Line " . __LINE__);
  }

  if (length $account_type == 0)
  {
    my_die("Error: Report.cgi - GetSubAccounts", 4034, "The Account Type is required at line " . __LINE__);
  }

  # get the sub-accounts for the given account type
  my $sth = $dbh->prepare( $sql_statements{1055} ) || my_db_die("Error: Report.cgi - GetSubAccounts", 4007, "SQL Statement = '$sql_statements{1055}'<br>\nAt Line " . __LINE__);
  my $rc = $sth->execute($account_type) || my_db_die("Error: Report.cgi - GetSubAccounts", 4008, "SQL Statement = '$sql_statements{1055}', Account Type = '$account_type'<br>\nAt Line " . __LINE__);
  my @temp_array = $sth->fetchrow_array;
  $sth->finish;

  my $tmp_sql_statement  = "";
  ($tmp_sql_statement = $sql_statements{1054}) =~ s/$firstSubstitutionRE/$temp_array[0]/;
  my $temp_order_by = ($order) ? "sub_acc_name" : "sub_acc_number";
  $tmp_sql_statement =~ s/$secondSubstitutionRE/$temp_order_by/;
  $sth = $dbh->prepare( $tmp_sql_statement ) || my_db_die("Error: Report.cgi - GetSubAccounts", 4007, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);
  my $rc = $sth->execute || my_db_die("Error: Report.cgi - GetSubAccounts", 4008, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);

  my @sub_accounts = ();
  while (@temp_array = $sth->fetchrow_array)
  {
    my @temp1 = ($temp_array[1], $temp_array[0]);
    $sub_accounts[scalar @sub_accounts] = [ @temp1 ];
  }
  $sth->finish;

  return @sub_accounts;
}

# GetSubAccountsForSubAccountType
# Takes   : sub_account_type, dbh
# Optional: order (0=number, 1=name)
# Returns : Array of sub-accounts ordered number first unless overridden by the order argument
#           The first element of the array will contain the number of the sub-account, the second element will contain the name.
sub GetSubAccountsForSubAccountType
{
  my %args = ( order => '0', @_ );
  my $dbh = $args{'dbh'};
  my $sub_account_type = $args{'sub_account_type'};
  my $order = $args{'order'};

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetSubAccountsForSubAccountType", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if ($order < 0 || $order > 1)
  {
    my_die("Error:  Report.cgi - GetSubAccountsForSubAccountType", 4034, "The Order flag can only be 0 or 1!&nbsp;&nbsp;'$order' is invalid!<br>\nAt Line " . __LINE__);
  }

  if (length $sub_account_type == 0)
  {
    my_die("Error: Report.cgi - GetSubAccountsForSubAccountType", 4034, "The Sub Account Type is required at line " . __LINE__);
  }

  my @sub_accounts = ();
  if ($sub_account_type != -1)
  {
    # get the sub-accounts for the given sub-account type
    my $tmp_sql_statement  = "";
    ($tmp_sql_statement = $sql_statements{1054}) =~ s/$firstSubstitutionRE/$sub_account_type/;
    my $temp_order_by = ($order) ? "sub_acc_name" : "sub_acc_number";
    $tmp_sql_statement =~ s/$secondSubstitutionRE/$temp_order_by/;
    my $sth = $dbh->prepare( $tmp_sql_statement ) || my_db_die("Error: Report.cgi - GetSubAccountsForSubAccountType", 4007, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);
    my $rc = $sth->execute || my_db_die("Error: Report.cgi - GetSubAccountsForSubAccountType", 4008, "SQL Statement = '$tmp_sql_statement'<br>\nAt Line " . __LINE__);

    while (my @temp_array = $sth->fetchrow_array)
    {
      my @temp1 = ($temp_array[1], $temp_array[0]);
      $sub_accounts[scalar @sub_accounts] = [ @temp1 ];
    }
    $sth->finish;
  }

  return @sub_accounts;
}

# GetTypeForSubAccount
# Takes  : sub_account, dbh
# Returns: Sub Account Type
sub GetTypeForSubAccount
{
  my %args = ( order => '0', @_ );
  my $dbh = $args{'dbh'};
  my $sub_account = $args{'sub_account'};

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetTypeForSubAccount", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $sub_account == 0)
  {
    my_die("Error: Report.cgi - GetTypeForSubAccount", 4034, "The Sub Account is required at line " . __LINE__);
  }

  if ($sub_account == -1)
  {
    return -2;  # This should never be a Sub Account Type!
  }

  # get the sub-account type for the given sub-account
  my $sth = $dbh->prepare( $sql_statements{1059} ) || my_db_die("Error: Report.cgi - GetTypeForSubAccount", 4007, "SQL Statement = '$sql_statements{1059}'<br>\nAt Line " . __LINE__);
  my $rc = $sth->execute($sub_account) || my_db_die("Error: Report.cgi - GetTypeForSubAccount", 4008, "SQL Statement = '$sql_statements{1059}', Sub Account = '$sub_account'<br>\nAt Line " . __LINE__);
  my @temp_array = $sth->fetchrow_array;
  $sth->finish;

  my $sub_account_type = $temp_array[0];

  return $sub_account_type;
}

# GetAccountsForSubAccount
# Takes   : sub_account, dbh, ledger_code
# Optional: taxable, account_type
# Returns : Array of Local Accounts that are used with the given sub-account
sub GetAccountsForSubAccount
{
  my %args = ( taxable => '0', @_ );
  my $dbh = $args{'dbh'};
  my $sub_account = $args{'sub_account'};
  my $ledger_code = $args{'ledger_code'};
  my $account_type = $args{'account_type'};
  my @account_type_list = ();
  my $taxable = $args{'taxable'};

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetAccountsForSubAccount", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $sub_account == 0)
  {
    my_die("Error: Report.cgi - GetAccountsForSubAccount", 4034, "The Sub Account is required at line " . __LINE__);
  }

  if (length $ledger_code == 0)
  {
    my_die("Error: Report.cgi - GetAccountsForSubAccount", 4034, "The Ledger Code is required at line " . __LINE__);
  }

  if (defined $account_type && length $account_type == 0)
  {
    my_die("Error: Report.cgi - GetAccountsForSubAccount", 4034, "The Account Type value must be defined at line " . __LINE__);
  }

  # get the sub-account type for the given sub-account
  if (!defined $account_type)
  {
    my $sub_account_type = GetTypeForSubAccount(dbh => $dbh, sub_account => $sub_account);
    # get a list of account types this sub-account type is assigned to
    my $sth = $dbh->prepare( $sql_statements{1060} ) || my_db_die("Error: Report.cgi - GetAccountsForSubAccount", 4007, "SQL Statement = '$sql_statements{1060}'<br>\nAt Line " . __LINE__);
    my $rc = $sth->execute() || my_db_die("Error: Report.cgi - GetAccountsForSubAccount", 4008, "SQL Statement = '$sql_statements{1060}'<br>\nAt Line " . __LINE__);
    while (my @temp_array = $sth->fetchrow_array)
    {
      my @temp_type = split /,/, $temp_array[1];
      for (my $count=0; $count < scalar @temp_type; $count++)
      {
        if ($sub_account_type == $temp_type[$count] || $sub_account == -1)
        {
          $account_type_list[scalar @account_type_list] = $temp_array[0];
        }
      }
    }
    $sth->finish;
  }
  else
  {
    $account_type_list[0] = $account_type;
  }

  my @local_accounts = ();
  # get the local list of accounts for the account_type_list of account types
  if (scalar @account_type_list > 0)
  {
    my @local_accounts_list = GetLocalAccounts(dbh => $dbh, account_types => \@account_type_list, taxable => $taxable, ledger_code => $ledger_code);
    my $count3 = 0;
    for (my $count=0; $count < scalar @local_accounts_list; $count++)
    {
      for (my $count2=0; $count2 < scalar @{$local_accounts_list[$count]}; $count2++)
      {
        $local_accounts[$count3] = ();
        $local_accounts[$count3][0] = $local_accounts_list[$count][$count2][0];
        $local_accounts[$count3][1] = $local_accounts_list[$count][$count2][1];
        $local_accounts[$count3][2] = $local_accounts_list[$count][$count2][2];
        $count3++;
      }
    }
  }

  return @local_accounts;
}

# RetrieveSubArray - Extracts the specified sub array from a multi-dimensional array and returns a flat array.
# Takes  : array, index
# Returns: sub array from array at index.
sub RetrieveSubArray
{
  my %args = ( @_ );
  my @array = @{$args{'array'}};
  my $index = $args{'index'};

  my @sub_array = @{$array[$index]};

  return @sub_array;
}

# GetSubEntities
# Takes  : dbh, ledger_code
# Returns: Array of SubEntities in the current ledger
#          [SubEntities] = Code, Name
sub GetSubEntities
{
  my %args = ( @_ );
  my $dbh = $args{'dbh'};
  my $ledger_code = $args{'ledger_code'};
  my @sub_entities = ();

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetSubEntities", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $ledger_code == 0)
  {
    my_die("Error:  Report.cgi - GetSubEntities", 4034, "The ledger_code must be specified!<br>\nAt Line " . __LINE__);
  }

  my @tmp_array = ();
  $tmp_array[0] = '';
  $tmp_array[1] = '<i>None</i>';
  $sub_entities[0] = [ @tmp_array ];

  my $counter=1;

  my $sth = $dbh->prepare( $sql_statements{1033} ) || my_db_die("Error: Report.cgi - GetSubEntities", 4007, "SQL Statement = '$sql_statements{1033}'<br>\nAt Line " . __LINE__);
  my $rc = $sth->execute($ledger_code) || my_db_die("Error: Report.cgi - GetSubEntities", 4008, "SQL Statement = '$sql_statements{1033}', ledger_code = '$ledger_code'<br>\nAt Line " . __LINE__);

  while (my @info = $sth->fetchrow_array)
  {
    @tmp_array = ();
    $tmp_array[0] = $info[1];
    $tmp_array[1] = $info[0];

    $sub_entities[$counter++] = [ @tmp_array ];
  }

  return @sub_entities;
}

# getKnownYearsInfo
# Takes  : dbh, ledger_code
# Returns: KnownYears Array
sub getKnownYearsInfo
{
  my %args = ( @_ );
  my $dbh = $args{'dbh'};
  my $ledger_code = $args{'ledger_code'};

  my @result = ();

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - getKnownYearsInfo", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $ledger_code == 0)
  {
    my_die("Error:  Report.cgi - getKnownYearsInfo", 4034, "The ledger_code must be specified!<br>\nAt Line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1028} ) || my_db_die("Error: Report.cgi - getKnownYearsInfo", 4007, "SQL Statement = '$sql_statements{1028}'");
  my $rc = $sth->execute( $ledger_code ) || my_db_die("Error: Report.cgi - getKnownYearsInfo", 4008, "SQL Statement = '$sql_statements{1028}'");
  my $count = 0;
  while (my @temp_row = $sth->fetchrow_array)
  {
    $result[$count] = [ @temp_row ];

    $count++;
  }
  $sth->finish;

  return @result;
}

# GetWorkingYear
# Takes  : dbh, ledger_code
# Returns: working_year
sub GetWorkingYear
{
  my %args = ( @_ );
  my $dbh = $args{'dbh'};
  my $ledger_code = $args{'ledger_code'};

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - GetAccountBalance", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $ledger_code == 0)
  {
    my_die("Error:  Report.cgi - GetAccountBalance", 4034, "The ledger_code must be specified!<br>\nAt Line " . __LINE__);
  }

  my ($sth, $rc, @info);
  $sth = $dbh->prepare( $sql_statements{1072} ) || my_db_die("Error: Report.cgi - GetAccountBalance", 4007, "SQL Statement = '$sql_statements{1072}'<br>\nAt Line " . __LINE__);
  $rc = $sth->execute($ledger_code) || my_db_die("Error: Report.cgi - GetAccountBalance", 4008, "SQL Statement = '$sql_statements{1072}', ledger_code = '$ledger_code'<br>\nAt Line " . __LINE__);
  @info = $sth->fetchrow_array;
  my $result = -1;  # signals an error.

  if ($sth->rows > 0)
  {
    $result = $info[0];
  }

  return $result;
}

# log_temp_file - makes the entry in the database that denotes a temp file that needs to be cleaned up later via cron.
# takes  : dbh, filename
# returns: nothing
sub log_temp_file
{
  my %args = ( @_, );
  my $dbh = $args{'dbh'};
  my $file_name = $args{'file_name'};

  if (length $dbh == 0)
  {
    my_die("Error: Report.cgi - log_temp_file", 4028, "The DB Handle is required at line " . __LINE__);
  }

  if (length $file_name == 0)
  {
    my_die("Error: Report.cgi - log_temp_file", 4035, "The filename is required at line " . __LINE__);
  }

  my $sth = $dbh->prepare( $sql_statements{1029} ) || my_db_die("Error: Report.cgi - log_temp_file", 4007, "SQL Statement = '$sql_statements{1029}'");
  my $rc = $sth->execute($file_name) || my_db_die("Error: Report.cgi - log_temp_file", 4008, "SQL Statement = '$sql_statements{1029}', File Name = '$file_name'");
  $dbh->commit || my_db_die("Error:  Report.cgi - log_temp_file", 4019, "At line " . __LINE__);
}
